Package: statuser
Type: Package
Title: Statistical Tools Designed for End Users
Version: 0.3.0
Authors@R: c(
    person(given = "Uri",
           family = "Simonsohn",
           role = c("aut", "cre"),
           email = "urisohn@gmail.com"))
Author: Uri Simonsohn [aut, cre]
Maintainer: Uri Simonsohn <urisohn@gmail.com>
Description: The statistical tools in this package do one of four things:
    1) Enhance basic statistical functions with more 
    flexible inputs, smarter defaults, and richer, clearer, and ready-to-use 
    output (e.g., t.test2())
    2) Produce publication-ready commonly needed figures with one line of code (e.g., plot_cdf())
    3) Implement novel analytical tools developed by the authors (e.g., twolines())
    4) Deliver niche functions of high value to the authors that are not easily
    available elsewhere (e.g., clear(), convert_to_sql(), resize_images()).
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: mgcv, marginaleffects, rsvg, magick, sandwich, lmtest,
        lmerTest, digest, beeswarm, utils
Suggests: testthat (>= 3.0.0), crayon, quantreg, estimatr, broom,
        modelsummary
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-20 20:24:19 UTC; uri_s
Depends: R (>= 3.5.0)
Repository: CRAN
Date/Publication: 2026-05-20 20:50:02 UTC
Built: R 4.5.2; ; 2026-05-21 23:33:15 UTC; unix

Package: statuser
Type: Package
Title: Statistical Tools Designed for End Users
Version: 0.3.0
Authors@R: c(
    person(given = "Uri",
           family = "Simonsohn",
           role = c("aut", "cre"),
           email = "urisohn@gmail.com"))
Author: Uri Simonsohn [aut, cre]
Maintainer: Uri Simonsohn <urisohn@gmail.com>
Description: The statistical tools in this package do one of four things:
    1) Enhance basic statistical functions with more 
    flexible inputs, smarter defaults, and richer, clearer, and ready-to-use 
    output (e.g., t.test2())
    2) Produce publication-ready commonly needed figures with one line of code (e.g., plot_cdf())
    3) Implement novel analytical tools developed by the authors (e.g., twolines())
    4) Deliver niche functions of high value to the authors that are not easily
    available elsewhere (e.g., clear(), convert_to_sql(), resize_images()).
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: mgcv, marginaleffects, rsvg, magick, sandwich, lmtest,
        lmerTest, digest, beeswarm, utils
Suggests: testthat (>= 3.0.0), crayon, quantreg, estimatr, broom,
        modelsummary
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-20 20:24:19 UTC; uri_s
Depends: R (>= 3.5.0)
Repository: CRAN
Date/Publication: 2026-05-20 20:50:02 UTC
Built: R 4.6.0; ; 2026-05-22 00:22:00 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(predict,lm2)
S3method(print,desc_var)
S3method(print,lm2)
S3method(print,t.test2)
S3method(print,table2)
S3method(summary,lm2)
S3method(t,test2)

export("var_labels<-")
export(clear)
export(clear_stimulus_cache)
export(convert_to_sql)
export(desc_var)
export(format_pvalue)
export(interprobe)
export(list2)
export(lm2)
export(lm2_notes)
export(message2)
export(plot_cdf)
export(plot_density)
export(plot_freq)
export(plot_gam)
export(plot_means)
export(resize_images)
export(scatter.gam)
export(stimulus.beeswarm)
export(stimulus.plot)
export(t.test2)
export(table2)
export(text2)
export(twolines)
export(var_labels)
import(lmerTest)
importFrom(beeswarm,beeswarm)
importFrom(digest,digest)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(grDevices,recordPlot)
importFrom(grDevices,replayPlot)
importFrom(grDevices,svg)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(lmtest,coeftest)
importFrom(mgcv,gam)
importFrom(sandwich,vcovHC)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,ave)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,drop.terms)
importFrom(stats,ecdf)
importFrom(stats,formula)
importFrom(stats,glm)
importFrom(stats,ks.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,predict)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,t.test)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)

# statuser

Miscellaneous R functions for papers, blogposts, & teaching by Uri Simonsohn.

## Installation

```r
# Install with `groundhog` (for version control)
groundhog::groundhog.library("statuser", date)   #date used for version control 

# Or with base R
install.packages("statuser")
```

## Overview

Functions I often use and are not (sufficiently?) available in existing packages.

## Functions

### 📄 Uri's Papers

<details>
<summary><code>twolines()</code>: Two-Lines Test of U-Shapes</summary>

Implements the two-lines test for U-shaped (or inverted U-shaped) relationships introduced by Simonsohn (2018).

**Reference:** Simonsohn, Uri (2018) "Two lines: A valid alternative to the invalid testing of U-shaped relationships with quadratic regressions." AMPPS, 538-555. https://doi.org/10.1177/2515245918805755

```r
# Simple example with simulated data
set.seed(123)
x <- rnorm(100)
y <- -x^2 + rnorm(100)
data <- data.frame(x = x, y = y)
result <- twolines(y ~ x, data = data)

# With covariates
z <- rnorm(100)
y <- -x^2 + 0.5*z + rnorm(100)
data <- data.frame(x = x, y = y, z = z)
result <- twolines(y ~ x + z, data = data)

# Without data argument (variables evaluated from environment)
x <- rnorm(100)
y <- -x^2 + rnorm(100)
result <- twolines(y ~ x)

# Suppress Robin Hood details
result <- twolines(y ~ x, data = data, quiet = TRUE)

# Save plot to PNG
result <- twolines(y ~ x, data = data, pngfile = "twolines_plot.png")
```
</details>

### 📊 Graphing

<details>
<summary><code>plot_cdf()</code>: CDF for multiple groups in one plot</summary>

```r
y <- rnorm(100)
x <- rep(c("A", "B"), 50)
plot_cdf(y ~ x)  # Uses default colors
plot_cdf(y ~ x, col = c("red", "blue"))  # Custom colors

# Control group order
plot_cdf(y ~ x, order = -1)  # Reverse default order
plot_cdf(y ~ x, order = c("B", "A"))  # Custom order
```
</details>

<details>
<summary><code>plot_density()</code>: Density for multiple groups in one plot</summary>

```r
y <- rnorm(100)
x <- rep(c("A", "B"), 50)
plot_density(y ~ x)  # Uses default colors
plot_density(y ~ x, col = c("red", "blue"))  # Custom colors
plot_density(y ~ x, show.means = FALSE)  # Hide mean segments

# Control group order
plot_density(y ~ x, order = -1)  # Reverse default order
plot_density(y ~ x, order = c("B", "A"))  # Custom order
```
</details>

<details>
<summary><code>plot_freq()</code>: Frequency distribution without binning, with value labels</summary>

```r
x <- c(1, 1, 2, 2, 2, 5, 5)
plot_freq(x)

# Grouped frequency plot
df <- data.frame(value = c(1, 1, 2, 2, 2, 5, 5), group = c("A", "A", "A", "B", "B", "A", "B"))
plot_freq(value ~ group, data = df)

# Control group order in legend and plot
plot_freq(value ~ group, data = df, order = c("B", "A"))  # B first, then A
plot_freq(value ~ group, data = df, order = -1)  # Reverse default order

# Factor levels are respected automatically
df$group <- factor(df$group, levels = c("B", "A"))
plot_freq(value ~ group, data = df)  # B will appear first

# Show percentages instead of frequencies
plot_freq(value ~ group, data = df, freq = FALSE)

# Customize legend and labels
plot_freq(value ~ group, data = df, legend.title = "Treatment", col.text = "black")
```
</details>

<details>
<summary><code>plot_gam()</code>: Plot GAM model predictions with optional distribution plot</summary>

```r
library(mgcv)
# Fit a GAM model
data(mtcars)
model <- gam(mpg ~ s(hp) + s(wt) + factor(cyl), data = mtcars)

# Basic plot
plot_gam(model, "hp")

# With distribution plot below (auto-selects plot_freq or plot_density)
plot_gam(model, "hp", plot2 = "auto")

# Always use frequency plot
plot_gam(model, "hp", plot2 = "freq")

# Always use density plot
plot_gam(model, "hp", plot2 = "density")

# Customize colors for main plot and bottom plot
plot_gam(model, "hp", plot2 = "auto", 
         col = "blue4", bg = adjustcolor('dodgerblue', .2),  # Main plot colors
         col2 = "steelblue", bg2 = "gray95")  # Bottom plot colors

# Hold other variables at different quantile
plot_gam(model, "hp", quantile.others = 25)
```
</details>

<details>
<summary><code>scatter.gam()</code>: Scatter plots with GAM smooth lines</summary>

```r
x <- rnorm(100)
y <- 2*x + rnorm(100)
scatter.gam(x, y)
```
</details>

<details>
<summary><code>text2()</code>: Adds to text(): align='center' , bg='yellow'</summary>

```r
plot(1:10, 1:10, type = "n")
   text2(2, 8, "Left", align = "left", bg = "lightblue")
   text2(5, 8, "Center", align = "center", bg = "lightgreen")
   text2(8, 8, "Right",    align = "right", bg = "lightyellow")
   text2(5, 5, "Red Text", col = "red", bg = "white")
```
</details>

<details>
<summary><code>resize_images()</code>: Saves any image (or all in folder) as PNG with set width.</summary>

```r
# Resize a single image file
   resize_images("path/to/image.svg", width = 800)

# Resize all images in a folder to 800px width
   resize_images("path/to/images", width = 800)

# Resize images to different widths
   resize_images("path/to/images", width = c(800, 1200, 600))
```
</details>

### 📈 Statistical Analyses

<details>
<summary><code>table2()</code>: Enhances base table(): (1) variable names are shown, (2) proportions are an option</summary>

```r
df <- data.frame(
  group = c("A", "A", "B", "B", "A"),
  status = c("X", "Y", "X", "Y", "X")
)

# table() does not show var names, table2() does
table (df$group, df$status)
table2(df$group, df$status)

# can report proportions (built into prop.table())
table2(df$group, df$status, prop = "all")    # Overall proportions
table2(df$group, df$status, prop = "row")    # Row proportions
table2(df$group, df$status, prop = "column") # Column proportions
```
</details>

<details>
<summary><code>desc_var()</code>: Comprehensive variable summary stats, optionally by grouping variable(s)</summary>

```r
# Why use desc_var() instead of psych::describeBy()?
# - Returns a single dataframe (not a list) - easier to export, filter, merge
# - Shows mode statistics (most frequent values) - useful for discrete data
# - Counts missing values automatically
# - Columns are labeled for easy interpretation
# - Supports multiple grouping variables with formula syntax
# - Results are sorted by grouping variables

# With grouping - compare groups side-by-side
df <- data.frame(score = rnorm(100), condition = rep(c("Control", "Treatment"), 50))
desc_var(score, condition, data = df)

# Formula syntax (single grouping variable)
desc_var(score ~ condition, data = df)

# Multiple grouping variables - results sorted by all grouping variables
df2 <- data.frame(
  score = rnorm(200),
  x1 = rep(c("A", "B"), 100),
  x2 = rep(c("men", "women"), each = 100),
  x3 = sample(1:3, replace = TRUE, size = 200)
)
desc_var(score ~ x1 + x2 + x3, data = df2)

# Without grouping - get stats for full dataset
desc_var(score, data = df)

# Direct vectors (no data frame needed)
scores <- rnorm(100)
groups <- rep(c("A", "B"), 50)
desc_var(scores, groups)

# Custom decimal places for cleaner output
desc_var(score, condition, data = df, decimals = 2)
```
</details>

<details>
<summary><code>lm2()</code>: Enhanced linear regression with robust SEs, standardized coefficients, and diagnostics</summary>

```r
# Basic usage - uses HC3 robust standard errors by default
lm2(mpg ~ wt + hp, data = mtcars)

# Works without data argument if variables exist in environment
y <- rnorm(100)
x1 <- rnorm(100)
x2 <- rnorm(100)
lm2(y ~ x1 + x2)

# Output includes:
# - Robust and classical standard errors side-by-side
# - Standardized coefficients (effect.size)
# - Missing value counts per variable
# - Red flags when robust and classical SEs differ substantially

# Get estimatr's native output instead
lm2(mpg ~ wt + hp, data = mtcars, output = "estimatr")

# Use different robust SE type (HC0, HC1, HC2, HC3)
lm2(mpg ~ wt + hp, data = mtcars, se_type = "HC2")
```
</details>

<details>
<summary><code>t.test2()</code>: Enhances base t.test: (1) console shows mean diff & var names, (2) output is dataframe, not list</summary>

```r
# Data for example
	men <- rnorm(100, mean = 5, sd = 1)
	women <- rnorm(100, mean = 4.8, sd = 1)

# t.test() is harder to interpret, does not show difference of means(!) or indicate which mean is subtracted from which

	t.test(men, women) 
	t.test2(men, women)

# Formula syntax
	data <- data.frame(y = rnorm(100), group = rep(c("A", "B"), 50))
	t2 = t.test2(y ~ group, data = data)  # Columns: A, B, A-B, SE_A-B, conf.intL, conf.intH, level, t, df, p.value, method, se_A, se_B
	t2

# Formula syntax without data argument
	x1=rnorm(100)
	x2=rnorm(100)
	y <- c(x1, x2)
	condition <- rep(c('A', 'B'), c(length(x1), length(x2)))
	t.test2(y ~ condition)  
```
</details>

### ✨ Formatting

<details>
<summary><code>format_pvalue()</code>: Format p-values for clean display in figures and tables (e.g., p<.0001)</summary>

```r
format_pvalue(0.05)
format_pvalue(0.0001, include_p = TRUE)
```
</details>

<details>
<summary><code>message2()</code>: Print colored messages to console</summary>

```r
message2("This is a red message", col = "red", font = 2)
message2("This is a cyan message", col = "cyan")
```
</details>

### 🗂️ Miscellaneous

<details>
<summary><code>list2()</code>: Create lists with objects without having to name them</summary>

```r
	x <- 1:5
	y <- letters[1:3]
	z <- matrix(1:4, nrow = 2)
	list2(x, y, z)
```
</details>

<details>
<summary><code>convert_to_sql()</code>: Convert CSV to SQL</summary>

```r
	convert_to_sql("data.csv", "data.sql")
```
</details>

<details>
<summary><code>clear()</code>: Clear plot, global environment, and console</summary>

Clears plot, global environment, and console. On first use you are prompted to authorize (one-time; your choice is saved).

```r
# Create some objects
	x <- 1:10
	y <- rnorm(10)
	plot(x, y)

# Clear everything (first run may prompt for one-time permission)
	clear()
```
</details> 

## Dependencies

- `mgcv` (for `scatter.gam()`, `plot_gam()`, and `twolines()`)
- `rsvg` (for `resize_images()`)
- `magick` (for `resize_images()`)
- `sandwich` (for `twolines()`)
- `lmtest` (for `twolines()`)
- `estimatr` (for `lm2()`)

## Author

**Uri Simonsohn**  
Email: urisohn@gmail.com

## License

GPL-3

## Version

- **Release:** 0.3.0

#' Stat Tools for End Users 
#'
#' @description
#' Basic and custom statistical tools designed with end users in mind. 
#' Functions have optimized defaults, produce decluttered and informative output 
#' that is self-explanatory, and generate publication-ready results in 1 line of code. 
#' 
#' @section Basic Stats (improved):
#' \itemize{
#'   \item \code{\link{lm2}}: like lm(), with robust SE and much more informative output
#'   \item \code{\link{t.test2}}: like t.test(), decluttered, and more informative output 
#'   \item \code{\link{table2}}: like table(), showing variable names, and with proportions & chi2 built in
#'   \item \code{\link{desc_var}}: Descriptive statistics for variables (optional, by group(s))
#' }
#' @section Custom tools from papers by Uri Simonsohn:
#' \itemize{
#'   \item \code{\link{twolines}}: Two-lines test for U-shapes (Simonsohn 2018)
#'   \item \code{\link{interprobe}}: Probe and visualize nonlinear interactions (Simonsohn 2024; Montealegre & Simonsohn 2026)
#'   \item \code{\link{stimulus.plot}}: Stimulus plots for matched/treated-stimulus designs (Simonsohn, Montealegre, & Evangelidis 2024)
#'   \item \code{\link{stimulus.beeswarm}}: Beeswarm plots for compared-stimulus designs (Simonsohn, Montealegre, & Evangelidis 2024)
#' }
#'
#' @section Graphing:
#' \itemize{
#'   \item \code{\link{scatter.gam}}: Makes scatter plot for x & y, with fitted GAM line y=f(x)
#'   \item \code{\link{plot_cdf}}: Plot empirical cumulative distribution functions (optional, by group)
#'   \item \code{\link{plot_density}}: Plot density functions (optional, by group)
#'   \item \code{\link{plot_freq}}: Plot frequency of observed values (optional, by group)
#'   \item \code{\link{plot_means}}: Barplot of means with confidence intervals and (optionally) tests. See also \code{\link{stimulus.plot}} for per-stimulus plots.
#'   \item \code{\link{plot_gam}}: Plot fitted GAM values for a focal predictor
#'   \item \code{\link{text2}}: like text() adding horizontal alignment and background color
#' }
#'
#' @section Formatting:
#' \itemize{
#'   \item \code{\link{format_pvalue}}: Format p-values for display
#'   \item \code{\link{var_labels}}: Get/set variable labels (as base-R \code{"label"} attributes)
#'   \item \code{\link{message2}}: Print colored messages to console
#'   \item \code{\link{resize_images}}: Resize images (SVG, PDF, EPS, JPG, PNG, etc.) to PNG with specified width
#' }
#'
#' @section Miscellaneous:
#' \itemize{
#'   \item \code{\link{clear}}: Clear environment, console, and all graphics devices
#'   \item \code{\link{list2}}: Like list(), but unnamed objects are automatically named
#'   \item \code{\link{convert_to_sql}}: Convert CSV files to SQL INSERT statements

#' }
#'
#' @author
#' Uri Simonsohn \email{urisohn@gmail.com}
#'
#' @references
#' Simonsohn, U. (2018). Two lines: A valid alternative to the invalid testing of 
#' U-shaped relationships with quadratic regressions. \emph{Advances in Methods and 
#' Practices in Psychological Science}, 1(4), 538-555. \doi{10.1177/2515245918805755}
#'
#' Simonsohn, U. (2024). Interacting with curves: How to validly test and probe interactions in the real (nonlinear) world. \emph{Advances in Methods and Practices in Psychological Science}, 7(1), Article 25152459231207787. \doi{10.1177/25152459231207787}
#'
#' Montealegre, D., & Simonsohn, U. (2026). \emph{Johnson Neyman 2.0} (working paper).
#'
#' Simonsohn, U., Montealegre, A., & Evangelidis, I. (2024). Stimulus sampling reimagined.
#' \emph{Advances in Methods and Practices in Psychological Science}.
#' 
#' @seealso
#' Useful links:
#' \itemize{
#'   \item \url{https://github.com/urisohn/statuser}
#' }
#'
#' @importFrom grDevices adjustcolor dev.list dev.off png
#' @importFrom graphics abline arrows axis box hist layout legend lines mtext par points polygon rect segments strheight strwidth text
#' @importFrom stats aggregate as.formula complete.cases density drop.terms ecdf formula glm ks.test lm median model.frame na.omit predict quantile runif sd setNames terms update
#' @importFrom utils packageVersion read.csv
#' @keywords internal
"_PACKAGE"

#' Convert CSV file to SQL INSERT statements
#' 
#' Reads a CSV file and generates SQL statements to insert all rows. Optionally
#' can also generate a CREATE TABLE statement. The function automatically infers
#' column types (REAL for numeric, DATE for date strings matching YYYY-MM-DD
#' format, TEXT otherwise).
#' 
#' @param input Character string. Path to the input CSV file.
#' @param output Character string. Path to the output SQL file where the
#'   statements will be written.
#' @param create_table Logical. If \code{TRUE}, includes a CREATE TABLE
#'   statement before the INSERT statements. Default is \code{FALSE}.
#' 
#' @return Invisibly returns NULL. The function writes SQL statements to the
#'   specified output file.
#' 
#' @details
#' The function performs the following steps:
#' \enumerate{
#'   \item Reads the CSV file using \code{read.csv()} with
#'     \code{stringsAsFactors = FALSE}
#'   \item Infers SQL column types:
#'     \itemize{
#'       \item Numeric columns become \code{REAL}
#'       \item Date columns (matching YYYY-MM-DD format) become \code{DATE}
#'       \item All other columns become \code{TEXT}
#'     }
#'   \item If \code{create_table = TRUE}, generates a \code{CREATE TABLE}
#'     statement using the base filename (without extension) as the table name
#'   \item Generates \code{INSERT INTO} statements for each row
#'   \item Writes all SQL statements to the output file
#' }
#' 
#' Single quotes in text values are escaped by doubling them (SQL standard).
#' Numeric values are inserted without quotes, while text and date values are
#' wrapped in single quotes.
#' 
#' @examples
#' # Convert a CSV file to SQL (INSERT statements only)
#' tmp_csv <- tempfile(fileext = ".csv")
#' tmp_sql <- tempfile(fileext = ".sql")
#' write.csv(
#'   data.frame(id = 1:2, value = c("a", "b"), date = c("2024-01-01", "2024-02-02")),
#'   tmp_csv,
#'   row.names = FALSE
#' )
#' convert_to_sql(tmp_csv, tmp_sql)
#' 
#' # Convert a CSV file to SQL with CREATE TABLE statement
#' convert_to_sql(tmp_csv, tmp_sql, create_table = TRUE)
#' 
#' @export
convert_to_sql <- function(input, output, create_table = FALSE) {
  
  # --- load CSV ---
  df <- read.csv(input, stringsAsFactors = FALSE)
  
  # table name from file
  table_name <- tools::file_path_sans_ext(basename(input))
  
  # --- infer SQL types ---
  infer_type <- function(x) {
    if (is.numeric(x)) return("REAL")
    if (all(grepl("^\\d{4}-\\d{2}-\\d{2}$", x))) return("DATE")
    return("TEXT")
  }
  
  types <- sapply(df, infer_type)
  
  # --- CREATE TABLE (optional) ---
  create_stmt <- NULL
  if (create_table) {
    fields <- paste0("  `", names(types), "` ", types, collapse = ",\n")
    create_stmt <- paste0("CREATE TABLE `", table_name, "` (\n", fields, "\n);\n\n")
  }
  
  # --- INSERT rows ---
  escape <- function(x) gsub("'", "''", x)
  
  rows <- apply(df, 1, function(r) {
    vals <- sapply(seq_along(r), function(i) {
      v <- r[i]
      # detect numeric based on inferred type
      if (types[i] == "REAL") v else paste0("'", escape(v), "'")
    })
    paste0("INSERT INTO `", table_name, "` VALUES (", paste(vals, collapse=", "), ");")
  })
  
  # --- write to output file ---
  if (create_table) {
    writeLines(c(create_stmt, rows), output)
  } else {
    writeLines(rows, output)
  }
}

#convert_to_sql_001
test_that("convert_to_sql creates SQL file", {
  # Create temporary CSV file
  temp_csv <- tempfile(fileext = ".csv")
  temp_sql <- tempfile(fileext = ".sql")
  
  # Create test data
  test_df <- data.frame(
    name = c("Alice", "Bob", "Charlie"),
    age = c(25, 30, 35),
    score = c(85.5, 92.0, 78.5)
  )
  
  write.csv(test_df, temp_csv, row.names = FALSE)
  
  # Should run without errors
  expect_error(convert_to_sql(temp_csv, temp_sql), NA)
  
  # Check that output file was created
  expect_true(file.exists(temp_sql))
  
  # Check that file has content
  sql_content <- readLines(temp_sql)
  expect_true(length(sql_content) > 0)
  
  # Cleanup
  unlink(temp_csv)
  unlink(temp_sql)
})

#convert_to_sql_002
test_that("convert_to_sql creates INSERT statements", {
  temp_csv <- tempfile(fileext = ".csv")
  temp_sql <- tempfile(fileext = ".sql")
  
  test_df <- data.frame(
    name = c("Alice", "Bob"),
    value = c(10, 20)
  )
  
  write.csv(test_df, temp_csv, row.names = FALSE)
  
  convert_to_sql(temp_csv, temp_sql)
  
  sql_content <- readLines(temp_sql)
  
  # Should have INSERT statements
  expect_true(any(grepl("INSERT INTO", sql_content)))
  
  # Cleanup
  unlink(temp_csv)
  unlink(temp_sql)
})

#convert_to_sql_003
test_that("convert_to_sql handles create_table parameter", {
  temp_csv <- tempfile(fileext = ".csv")
  temp_sql <- tempfile(fileext = ".sql")
  
  test_df <- data.frame(
    name = c("Alice"),
    age = c(25)
  )
  
  write.csv(test_df, temp_csv, row.names = FALSE)
  
  # With create_table = TRUE
  convert_to_sql(temp_csv, temp_sql, create_table = TRUE)
  
  sql_content <- readLines(temp_sql)
  
  # Should have CREATE TABLE statement
  expect_true(any(grepl("CREATE TABLE", sql_content)))
  
  # Cleanup
  unlink(temp_csv)
  unlink(temp_sql)
})

#convert_to_sql_004
test_that("convert_to_sql handles date columns", {
  temp_csv <- tempfile(fileext = ".csv")
  temp_sql <- tempfile(fileext = ".sql")
  
  test_df <- data.frame(
    name = c("Alice"),
    date = c("2024-01-01")
  )
  
  write.csv(test_df, temp_csv, row.names = FALSE)
  
  convert_to_sql(temp_csv, temp_sql, create_table = TRUE)
  
  sql_content <- paste(readLines(temp_sql), collapse = "\n")
  
  # Should detect DATE type
  expect_true(grepl("DATE", sql_content))
  
  # Cleanup
  unlink(temp_csv)
  unlink(temp_sql)
})

#convert_to_sql_005
test_that("convert_to_sql handles numeric columns", {
  temp_csv <- tempfile(fileext = ".csv")
  temp_sql <- tempfile(fileext = ".sql")
  
  test_df <- data.frame(
    name = c("Alice"),
    value = c(123.45)
  )
  
  write.csv(test_df, temp_csv, row.names = FALSE)
  
  convert_to_sql(temp_csv, temp_sql, create_table = TRUE)
  
  sql_content <- paste(readLines(temp_sql), collapse = "\n")
  
  # Should detect REAL type for numeric columns
  expect_true(grepl("REAL", sql_content))
  
  # Cleanup
  unlink(temp_csv)
  unlink(temp_sql)
})

#convert_to_sql_006
test_that("convert_to_sql escapes single quotes", {
  temp_csv <- tempfile(fileext = ".csv")
  temp_sql <- tempfile(fileext = ".sql")
  
  test_df <- data.frame(
    name = c("Alice's name"),
    value = c(10)
  )
  
  write.csv(test_df, temp_csv, row.names = FALSE)
  
  convert_to_sql(temp_csv, temp_sql)
  
  sql_content <- readLines(temp_sql)
  
  # Should escape single quotes (double them)
  expect_true(any(grepl("''", sql_content)))
  
  # Cleanup
  unlink(temp_csv)
  unlink(temp_sql)
})

#convert_to_sql_007
test_that("convert_to_sql returns invisibly", {
  temp_csv <- tempfile(fileext = ".csv")
  temp_sql <- tempfile(fileext = ".sql")
  
  test_df <- data.frame(name = c("Alice"), value = c(10))
  write.csv(test_df, temp_csv, row.names = FALSE)
  
  result <- convert_to_sql(temp_csv, temp_sql)
  expect_null(result)
  
  # Cleanup
  unlink(temp_csv)
  unlink(temp_sql)
})

#' Plot Empirical Cumulative Distribution Functions by Group
#'
#' Plots empirical cumulative distribution functions (ECDFs) separately for
#' each unique value of a grouping variable, with support for vectorized
#' plotting parameters. If no grouping variable is provided, plots a single ECDF.
#'
#' @param formula Two possible uses (similar to \code{t.test()}):
#'   \itemize{
#'     \item {Single Variable (possibly by subgroup)}: \code{plot_cdf(y)} or \code{plot_cdf(y~x)}
#'     \item {Contrast Two Variables}: \code{plot_cdf(y1, y2)}
#'   }
#' @param y2 optional second variable when contrasting two variables \code{plot_cdf(y1,y2)}
#' @param data An optional data frame containing the variables in the formula.
#'   If \code{data} is not provided, variables are evaluated from the calling environment.
#' @param order Controls the order in which groups appear in the plot and legend. 
#'   Use \code{-1} to reverse the default order. Alternatively, provide a vector specifying
#'   the exact order (e.g., \code{c("B", "A", "C")}). If \code{NULL} (default), groups are 
#'   ordered by their factor levels (if the grouping variable is a factor) or sorted 
#'   alphabetically/numerically. Only applies when using grouped plots.
#' @param show.ks Logical. If TRUE (default), shows Kolmogorov-Smirnov test results
#'   when there are exactly 2 groups. If FALSE, KS test results are not displayed.
#' @param show.quantiles Logical. If TRUE (default), shows horizontal lines and results
#'   at 25th, 50th, and 75th percentiles when there are exactly 2 groups. If FALSE,
#'   quantile lines and results are not displayed.
#' @param ... Additional arguments passed to plotting functions. Can be single values
#'   (applied to all groups) or vectors (applied element-wise to each group).
#'   Common parameters include \code{col}, \code{lwd}, \code{lty}, \code{pch},
#'   \code{type}, etc.
#'
#' @return Invisibly returns a list containing:
#'   \itemize{
#'     \item \code{ecdfs}: A list of ECDF function objects, one per group. Each can be
#'       called as a function to compute cumulative probabilities (e.g., \code{result$ecdfs[[1]](5)}
#'       returns P(X <= 5) for group 1).
#'     \item \code{ks_test}: (Only when exactly 2 groups) The Kolmogorov-Smirnov test result
#'       comparing the two distributions. Access p-value with \code{result$ks_test$p.value}.
#'     \item \code{quantile_regression_25}: (Only when exactly 2 groups) Quantile regression
#'       model for the 25th percentile.
#'     \item \code{quantile_regression_50}: (Only when exactly 2 groups) Quantile regression
#'       model for the 50th percentile (median).
#'     \item \code{quantile_regression_75}: (Only when exactly 2 groups) Quantile regression
#'       model for the 75th percentile.
#'     \item \code{warnings}: Any warnings captured during execution (if any).
#'   }
#'
#' @examples
#' # Basic usage with single variable (no grouping)
#' y <- rnorm(100)
#' plot_cdf(y)
#' 
#' # Basic usage with formula syntax and grouping
#' group <- rep(c("A", "B", "C"), c(30, 40, 30))
#' plot_cdf(y ~ group)
#'
#' # With custom colors (scalar - same for all)
#' plot_cdf(y ~ group, col = "blue")
#'
#' # With custom colors (vector - different for each group)
#' plot_cdf(y ~ group, col = c("red", "green", "blue"))
#'
#' # Multiple parameters
#' plot_cdf(y ~ group, col = c("red", "green", "blue"), lwd = c(1, 2, 3))
#'
#' # With line type and point character
#' plot_cdf(y ~ group, col = c("red", "green", "blue"), lty = c(1, 2, 3), lwd = 2)
#'
#' # Using data frame
#' df <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
#' plot_cdf(value ~ group, data = df)
#' plot_cdf(value ~ group, data = df, col = c("red", "blue"))
#'
#' # Compare two vectors
#' y1 <- rnorm(50)
#' y2 <- rnorm(50, mean = 1)
#' plot_cdf(y1, y2)
#' 
#' # Formula syntax without data (variables evaluated from environment)
#' widgetness <- rnorm(100)
#' gender <- rep(c("M", "F"), 50)
#' plot_cdf(widgetness ~ gender)
#'
#' # Using the returned object
#' df <- data.frame(value = c(rnorm(50, 0), rnorm(50, 1)), group = rep(c("A", "B"), each = 50))
#' result <- plot_cdf(value ~ group, data = df)
#' 
#' # Use ECDF to find P(X <= 0.5) for group A
#' result$ecdfs[[1]](0.5)
#' 
#' # Access KS test p-value
#' result$ks_test$p.value
#' 
#' # Summarize median quantile regression
#' summary(result$quantile_regression_50)

#' @export
plot_cdf <- function(formula, y2 = NULL, data = NULL, order = NULL, show.ks = TRUE, show.quantiles = TRUE, ...) {
  #0. CAPTURE UNEVALUATED ARGUMENTS FIRST (before ANY evaluation!)
  mc <- match.call()
  
  # Resolve first argument (formula or first vector)
  formula_resolved <- evaluate_variable_arguments(
    arg_expr = mc$formula,
    arg_name = "formula",
    data = data,
    calling_env = parent.frame(),
    func_name = "plot_cdf",
    allow_null = FALSE
  )
  
  # Resolve second argument if present (for two-vector mode)
  y2_resolved <- if (!is.null(mc$y2)) {
    evaluate_variable_arguments(
      arg_expr = mc$y2,
      arg_name = "y2",
      data = data,
      calling_env = parent.frame(),
      func_name = "plot_cdf",
      allow_null = TRUE
    )
  } else {
    list(value = NULL, name = NULL, name_raw = NULL, was_symbol = FALSE)
  }
  
  # Now overwrite the arguments with resolved values
  formula <- formula_resolved$value
  y2 <- y2_resolved$value
  
  # Extract plotting parameters from ...
    dots <- list(...)
    
    # Remove show.ks and show.quantiles from dots if passed (they're formal parameters)
    dots$show.ks <- NULL
    dots$show.quantiles <- NULL
    
  # Check if we're in two-vector comparison mode (formula is vector, y is vector)
  if (!is.null(y2) && !inherits(formula, "formula")) {
    # Two-vector comparison mode: plot_cdf(y1, y2)
    # Use resolved names
    y1_name <- formula_resolved$name
    y2_name <- y2_resolved$name
    
    # Validate inputs
    if (!is.numeric(formula) || !is.vector(formula)) {
      stop(sprintf("plot_cdf(): First argument '%s' must be a numeric vector", y1_name), call. = FALSE)
    }
    if (!is.numeric(y2) || !is.vector(y2)) {
      stop(sprintf("plot_cdf(): Second argument '%s' must be a numeric vector", y2_name), call. = FALSE)
    }
    
    # Create a data frame and recursively call with grouped syntax
    df <- data.frame(
      value = c(formula, y2),
      group = c(rep(y1_name, length(formula)), rep(y2_name, length(y2))),
      stringsAsFactors = FALSE
    )
    
    # Forward all arguments to the grouped version
    return(plot_cdf(value ~ group, data = df, order = order, 
                    show.ks = show.ks, show.quantiles = show.quantiles, ...))
  }
  
  # Validate formula early if it is one
  validate_formula(formula, data, func_name = "plot_cdf", calling_env = parent.frame())
  
  # Check if input is a formula or a variable
  is_formula_input <- tryCatch(inherits(formula, "formula"), error = function(e) FALSE)
  
  # Capture data name for error messages
  mc <- match.call()
  data_expr <- mc$data
  data_name <- if (!is.null(data_expr)) {
    data_name_val <- deparse(data_expr)
    gsub('^"|"$', '', data_name_val)
  } else {
    NULL
  }
  
  # Capture original group before validation (to preserve factor levels)
  group_original_before_validation <- NULL
  if (is_formula_input && inherits(formula, "formula")) {
    f <- formula
    if (length(f) >= 3L && identical(f[[1L]], quote(`~`))) {
      rhs <- f[[3L]]
      rhs_is_intercept_only <- is.numeric(rhs) && length(rhs) == 1L &&
        !is.na(rhs[1]) && rhs[1] == 1
      if (!rhs_is_intercept_only) {
        group_original_before_validation <- tryCatch({
          if (!is.null(data)) {
            eval(rhs, envir = data, enclos = parent.frame())
          } else {
            fe <- environment(f)
            if (is.null(fe)) fe <- parent.frame()
            eval(rhs, envir = fe)
          }
        }, error = function(e) NULL)
      }
    }
  }
  
  # Validate inputs using validation function shared with plot_density, plot_cdf, plot_freq
  # If formula input, use validate_plot
  if (is_formula_input) {
    validated <- validate_plot(formula, NULL, data, func_name = "plot_cdf", require_group = FALSE, data_name = data_name)
  } else {
    # Not a formula - use resolved names from evaluate_variable_arguments
    validated <- list(
      y = formula,
      group = NULL,
      y_name = formula_resolved$name,
      group_name = NULL,
      y_name_raw = formula_resolved$name_raw,
      group_name_raw = NULL,
      data_name = data_name
    )
  }
  y <- validated$y
  group <- validated$group
  y_name <- validated$y_name
  group_name <- validated$group_name
  y_name_raw <- validated$y_name_raw
  group_name_raw <- validated$group_name_raw
  data_name <- validated$data_name
  
  # Store original group for factor level checking (before NA removal)
  # Use the version captured before validation if available (to preserve factor levels)
  group_original <- if (!is.null(group_original_before_validation)) {
    group_original_before_validation
  } else {
    group
  }
  
  # Check if group is provided
  has_group <- !is.null(group)
  
  # Drop missing data
  if (has_group) {
    isnagroup=is.na(group)
    isnay=is.na(y)
    group=group[!isnagroup & !isnay]
    y=y[!isnagroup & !isnay]
    
    n.nagroup = sum(isnagroup)
    n.nay = sum(isnay)
    
    if (n.nagroup>0) message2("plot_cdf() says: dropped ",n.nagroup," observations with missing '",group_name_raw,"' values",col='red2')
    if (n.nay>0) message2("plot_cdf() says: dropped ",n.nay," observations with missing '",y_name_raw,"' values",col='red2')
    
    # Get unique groups and apply ordering
    unique_groups <- unique(group)
    n_groups <- length(unique_groups)
    
    # Check if order = -1 (reverse default order)
    reverse_order <- FALSE
    if (!is.null(order) && length(order) == 1 && is.numeric(order) && order == -1) {
      reverse_order <- TRUE
      order <- NULL  # Process as default, then reverse
    }
    
    if (!is.null(order)) {
      # User specified custom order
      # Validate that order contains all groups
      missing_groups <- setdiff(unique_groups, order)
      extra_groups <- setdiff(order, unique_groups)
      
      if (length(missing_groups) > 0) {
        stop(sprintf("plot_cdf(): 'order' is missing group(s): %s", 
                     paste(missing_groups, collapse = ", ")), call. = FALSE)
      }
      if (length(extra_groups) > 0) {
        warning(sprintf("plot_cdf(): 'order' contains group(s) not in data: %s", 
                       paste(extra_groups, collapse = ", ")))
      }
      
      # Use the specified order (only groups that exist in data)
      unique_x <- order[order %in% unique_groups]
    } else if (is.factor(group_original)) {
      # Respect factor levels
      factor_levels <- levels(group_original)
      # Only include levels that actually appear in the data
      unique_x <- factor_levels[factor_levels %in% unique_groups]
    } else {
      # Default: sort alphabetically/numerically
      unique_x <- sort(unique_groups)
    }
    
    # Reverse order if order = -1 was specified
    if (reverse_order) {
      unique_x <- rev(unique_x)
    }
  } else {
    # No group variable - just drop missing y values
    isnay=is.na(y)
    y=y[!isnay]
    
    n.nay = sum(isnay)
    if (n.nay>0) message2("plot_cdf() says: dropped ",n.nay," observations with missing '",y_name_raw,"' values",col='red2')
    
    unique_x <- NULL
    n_groups <- 1
  }
  
  # Initialize return values for tests (when 2 groups)
  ks_test_result <- NULL
  quantile_regression_25 <- NULL
  quantile_regression_50 <- NULL
  quantile_regression_75 <- NULL
  warnings_list <- list()
  
  # Get default colors based on number of groups
  default_colors <- get.colors(n_groups)
  
  # Helper function to extract parameter value for a group
  get_param <- function(param_name, group_idx) {
    if (param_name %in% names(dots)) {
      param_val <- dots[[param_name]]
      if (length(param_val) == 1) {
        return(param_val)
      } else if (length(param_val) >= group_idx) {
        return(param_val[group_idx])
      } else {
        return(param_val[1])  # Recycle if shorter
      }
    }
    return(NULL)
  }
  
  # Compute ECDFs for each group (or single ECDF if no group)
  ecdf_list <- list()
  y_ranges <- list()
  
  if (has_group) {
    for (i in seq_along(unique_x)) {
      group_val <- unique_x[i]
      y_group <- y[group == group_val]
      if (length(y_group) > 0) {
        ecdf_list[[i]] <- ecdf(y_group)
        y_ranges[[i]] <- range(y_group)
      }
    }
  } else {
    # No group - single ECDF for all data
    if (length(y) > 0) {
      ecdf_list[[1]] <- ecdf(y)
      y_ranges[[1]] <- range(y)
    }
  }
  
  # Determine overall range for plotting
  all_y <- unlist(y_ranges)
  y_min <- min(all_y, na.rm = TRUE)
  y_max <- max(all_y, na.rm = TRUE)
  y_range <- y_max - y_min
  y_lim <- c(y_min - 0.05 * y_range, y_max + 0.05 * y_range)

  # Create sequence for plotting ECDF
  y_seq <- seq(y_min, y_max, length.out = 1000)

  # Helper function for NULL coalescing
  `%||%` <- function(x, y) if (is.null(x)) y else x
  
  # Plot first ECDF to set up the plot
  if (length(ecdf_list) > 0) {
    first_ecdf <- ecdf_list[[1]]
    first_y_vals <- first_ecdf(y_seq)
    
    # Get parameters for first group
    col1 <- get_param("col", 1) %||% default_colors[1]
    lwd1 <- get_param("lwd", 1) %||% 4  # Default lwd=4
    lty1 <- get_param("lty", 1) %||% 1
    type1 <- get_param("type", 1) %||% "s"
    pch1 <- get_param("pch", 1)
    
    # Build plot arguments
    # Set main title if not provided
      if (!"main" %in% names(dots)) {
        if (has_group) {
          main_title <- paste0("Comparing Distribution of '", y_name, "' by '", group_name, "'")
        } else {
          main_title <- paste0("Distribution of '", y_name, "'")
        }
      } else {
        main_title <- dots$main
      }
    # Set font and size for main title if not provided
      font_main <- if ("font.main" %in% names(dots)) dots$font.main else 2
      cex_main <- if ("cex.main" %in% names(dots)) dots$cex.main else 1.38
    
    # Set xlab if not provided
      xlab_title <- if ("xlab" %in% names(dots)) dots$xlab else y_name
    
    # Set ylab if not provided
      ylab_title <- if ("ylab" %in% names(dots)) dots$ylab else "% of observations"
    
    # Set default ylim if not provided (extend to accommodate legend above plot if groups exist)
      if (!"ylim" %in% names(dots)) {
        if (has_group && n_groups > 1) {
          default_ylim <- c(0, 1.25)  # Reserve top 20% for legend (1.25 = 1 / 0.8)
        } else {
          default_ylim <- c(0, 1)
        }
      } else {
        default_ylim <- dots$ylim
      }
      
    # Set default xlim if not provided (add padding to prevent clipping at edges)
      if (!"xlim" %in% names(dots)) {
        x_range <- y_max - y_min
        default_xlim <- c(y_min - 0.05 * x_range, y_max + 0.05 * x_range)
      } else {
        default_xlim <- dots$xlim
      }
    
    # Ensure adequate top margin for main title and legend
    # Only save/restore what we modify; restoring full `par()` can reset `mfg`
    # and break multi-panel plotting via `par(mfrow=...)`.
    old_mar <- par("mar")
    old_mgp <- par("mgp")
    on.exit(par(mar = old_mar, mgp = old_mgp), add = TRUE)
    if (!"mar" %in% names(dots)) {
      # Increase top margin if it's too small (less than 3 lines to accommodate title and legend)
      # Add 1 line to the left margin
      new_mar <- c(old_mar[1], old_mar[2] + 1, old_mar[3], old_mar[4])
      if (new_mar[3] < 3) {
        new_mar[3] <- 3
      }
      par(mar = new_mar)
    }
    # Move ylab 0.75 lines to the left
    if (!"mgp" %in% names(dots)) {
      new_mgp <- c(old_mgp[1], old_mgp[2] - 0.75, old_mgp[3])
      par(mgp = new_mgp)
    }
    
    # Remove vectorized parameters and data from dots for plot()
    # Also remove xlab, ylab, main since we handle them separately
    plot_dots <- dots
    vectorized_params <- c("col", "lwd", "lty", "type", "pch", "data")
    plot_params_to_remove <- c(vectorized_params, "xlab", "ylab", "main")
    plot_dots[plot_params_to_remove] <- NULL
    
    plot_args <- list(x = y_seq, y = first_y_vals, 
                      type = type1, col = col1, lwd = lwd1, lty = lty1,
                      xlab = xlab_title, 
                      ylab = ylab_title,
                      main = main_title,
                      font.main = font_main,
                      cex.main = cex_main,
                      xlim = default_xlim,
                      ylim = default_ylim,
                      font.lab = 2, cex.lab = 1.2, las = 1,
                      yaxt = "n",  # Suppress default y-axis to draw custom percentage axis
                      xaxt = "n")  # Suppress default x-axis to redraw with adjusted label position
    if (!is.null(pch1)) plot_args$pch <- pch1
    
    # Set up plot
    do.call(plot, c(plot_args, plot_dots))
    
    # Draw custom y-axis with percentage labels
    y_ticks <- seq(0, 1, by = 0.25)
    y_labels <- paste0(y_ticks * 100, "%")
    axis(2, at = y_ticks, labels = y_labels, las = 1)
    
    # Draw x-axis with labels moved lower (padj moves labels down)
    axis(1, padj = 0.5)
    
    # Add remaining ECDFs
    if (length(ecdf_list) > 1) {
      for (i in 2:length(ecdf_list)) {
        ecdf_fun <- ecdf_list[[i]]
        y_vals <- ecdf_fun(y_seq)
        
        # Get parameters for this group
        coli <- get_param("col", i) %||% default_colors[i]
        lwdi <- get_param("lwd", i) %||% 4  # Default lwd=4
        ltyi <- get_param("lty", i) %||% 1
        typei <- get_param("type", i) %||% "s"
        pchi <- get_param("pch", i)
        
        # Build lines arguments
        lines_args <- list(x = y_seq, y = y_vals, type = typei, 
                          col = coli, lwd = lwdi, lty = ltyi)
        if (!is.null(pchi)) lines_args$pch <- pchi
        
        do.call(lines, lines_args)
      }
    }
    
    # Add legend on top (only if there are groups)
    if (has_group && n_groups > 1) {
      legend_cols <- sapply(1:length(ecdf_list), function(i) get_param("col", i) %||% default_colors[i])
      legend_lwds <- sapply(1:length(ecdf_list), function(i) get_param("lwd", i) %||% 4)
      legend_ltys <- sapply(1:length(ecdf_list), function(i) get_param("lty", i) %||% 1)
      
      # Calculate sample sizes for each group and add to legend labels
      group_ns <- sapply(1:length(ecdf_list), function(i) {
        length(y[group == unique_x[i]])
      })
      
      # Create legend labels with sample sizes (newline before sample size, lowercase n)
      legend_labels <- paste0(as.character(unique_x), "\n(n=", group_ns, ")")
      
      legend("top", legend = legend_labels, 
             col = legend_cols, lwd = legend_lwds, lty = legend_ltys,
             horiz = TRUE, bty = "n")
    }
    
    # If exactly 2 groups, perform KS test and quantile regression tests
      if (n_groups == 2) {
        # Get data for both groups
        y1 <- y[group == unique_x[1]]
        y2 <- y[group == unique_x[2]]
        
        # Kolmogorov-Smirnov test (capture warnings about ties)
        ks_test <- withCallingHandlers(
          ks.test(y1, y2),
          warning = function(w) {
            if (grepl("p-value will be approximate in the presence of ties", w$message, ignore.case = TRUE)) {
              warnings_list$ks_ties <<- w$message
              invokeRestart("muffleWarning")
            }
          }
        )
        ks_test_result <- ks_test
        
        # Quantile probabilities (used for both computation and display)
        quantile_probs <- c(0.25, 0.50, 0.75)
        
        # Add horizontal lines at 25%, 50%, and 75% of cumulative probability (only if show.quantiles is TRUE)
        if (show.quantiles) {
          abline(h = quantile_probs, lty = 2, col = "gray80")
        }
        
        # Quantile regression tests at 25th, 50th, and 75th percentiles
        # Always compute quantile regressions (for return value), but only display if show.quantiles is TRUE
        if (requireNamespace("quantreg", quietly = TRUE)) {
        # Show message about independence assumption (only once per session)
        if (!exists("plot_cdf_message_shown", envir = .statuser_state, inherits = FALSE)) {
          message2("The p-values are based on quantile regressions that assume all observations are independent", col = "red")
          assign("plot_cdf_message_shown", TRUE, envir = .statuser_state)
        }
        
        # Create data frame for quantile regression
        df_qr <- data.frame(y = y, x_group = as.numeric(group == unique_x[2]))
        
        # Calculate quantiles for each group (only needed if show.quantiles is TRUE for display)
        if (show.quantiles) {
          q1_25 <- quantile(y1, probs = 0.25, na.rm = TRUE)
          q1_50 <- quantile(y1, probs = 0.50, na.rm = TRUE)
          q1_75 <- quantile(y1, probs = 0.75, na.rm = TRUE)
          q2_25 <- quantile(y2, probs = 0.25, na.rm = TRUE)
          q2_50 <- quantile(y2, probs = 0.50, na.rm = TRUE)
          q2_75 <- quantile(y2, probs = 0.75, na.rm = TRUE)
        }
        
        # Get ECDF functions for finding intersections
        ecdf1 <- ecdf_list[[1]]
        ecdf2 <- ecdf_list[[2]]
        
        # Test quantiles and get p-values, store models as separate objects
        quantile_pvals <- character(length(quantile_probs))
        quantile_regression_25 <- NULL
        quantile_regression_50 <- NULL
        quantile_regression_75 <- NULL
        
        for (i in seq_along(quantile_probs)) {
          tau <- quantile_probs[i]
          tryCatch({
            # Capture warnings about non-uniqueness in quantile regression
            qr_model <- withCallingHandlers(
              quantreg::rq(y ~ x_group, data = df_qr, tau = tau),
              warning = function(w) {
                if (grepl("non.*unique|nonunique", w$message, ignore.case = TRUE)) {
                  warning_key <- paste0("quantile_regression_", tau, "_nonunique")
                  warnings_list[[warning_key]] <<- w$message
                  invokeRestart("muffleWarning")
                }
              }
            )
            # Store model first (before summary, so it's stored even if summary fails)
            qr_summary <- withCallingHandlers(
              summary(qr_model, se = "iid"),
              warning = function(w) {
                if (grepl("non.*unique|nonunique", w$message, ignore.case = TRUE)) {
                  warning_key <- paste0("quantile_regression_", tau, "_summary_nonunique")
                  warnings_list[[warning_key]] <<- w$message
                  invokeRestart("muffleWarning")
                }
              }
            )
            qr_p <- qr_summary$coefficients[2, 4]  # p-value for x_group coefficient
            quantile_pvals[i] <- format_pvalue(qr_p, include_p = TRUE)
            
            # Store model with appropriate name
            if (tau == 0.25) {
              quantile_regression_25 <- qr_model
            } else if (tau == 0.50) {
              quantile_regression_50 <- qr_model
            } else if (tau == 0.75) {
              quantile_regression_75 <- qr_model
            }
          }, error = function(e) {
            quantile_pvals[i] <- "NA"
          })
        }
        
        # Get plot boundaries
        usr <- par("usr")
        x_range <- usr[2] - usr[1]
        y_range <- usr[4] - usr[3]
        
        # Add p-values just above the horizontal lines (only if show.quantiles is TRUE)
        if (show.quantiles) {
          for (i in seq_along(quantile_probs)) {
            text(x = usr[1] + 0.02 * x_range, y = quantile_probs[i] + 0.02, 
                 labels = quantile_pvals[i],
                 adj = c(0, 0.5), cex = 0.8, font = 2)
          }
          
          # Add value labels next to CDF lines
          # Position labels based on which CDF is to the left/right at each quantile
          # Use adj to control alignment and add offset to separate labels
          usr <- par("usr")
          x_range <- usr[2] - usr[1]
          label_offset <- 0.05 / 3 * x_range  # ~1.67% of plot width for separation
          
          # 25th percentile
          if (q1_25 < q2_25) {
            # Group 1 is to the left, right-align its label and offset left
            text(x = q1_25 - label_offset, y = 0.25 + 0.02, labels = round(q1_25, 2),
                 adj = c(1, 0.5), cex = 0.8, font = 2, col = legend_cols[1])
            # Group 2 is to the right, left-align its label and offset right
            text(x = q2_25 + label_offset, y = 0.25 + 0.02, labels = round(q2_25, 2),
                 adj = c(0, 0.5), cex = 0.8, font = 2, col = legend_cols[2])
          } else {
            # Group 2 is to the left, right-align its label and offset left
            text(x = q2_25 - label_offset, y = 0.25 + 0.02, labels = round(q2_25, 2),
                 adj = c(1, 0.5), cex = 0.8, font = 2, col = legend_cols[2])
            # Group 1 is to the right, left-align its label and offset right
            text(x = q1_25 + label_offset, y = 0.25 + 0.02, labels = round(q1_25, 2),
                 adj = c(0, 0.5), cex = 0.8, font = 2, col = legend_cols[1])
          }
          
          # 50th percentile
          if (q1_50 < q2_50) {
            # Group 1 is to the left, right-align its label and offset left
            text(x = q1_50 - label_offset, y = 0.50 + 0.02, labels = round(q1_50, 2),
                 adj = c(1, 0.5), cex = 0.8, font = 2, col = legend_cols[1])
            # Group 2 is to the right, left-align its label and offset right
            text(x = q2_50 + label_offset, y = 0.50 + 0.02, labels = round(q2_50, 2),
                 adj = c(0, 0.5), cex = 0.8, font = 2, col = legend_cols[2])
          } else {
            # Group 2 is to the left, right-align its label and offset left
            text(x = q2_50 - label_offset, y = 0.50 + 0.02, labels = round(q2_50, 2),
                 adj = c(1, 0.5), cex = 0.8, font = 2, col = legend_cols[2])
            # Group 1 is to the right, left-align its label and offset right
            text(x = q1_50 + label_offset, y = 0.50 + 0.02, labels = round(q1_50, 2),
                 adj = c(0, 0.5), cex = 0.8, font = 2, col = legend_cols[1])
          }
          
          # 75th percentile
          if (q1_75 < q2_75) {
            # Group 1 is to the left, right-align its label and offset left
            text(x = q1_75 - label_offset, y = 0.75 + 0.02, labels = round(q1_75, 2),
                 adj = c(1, 0.5), cex = 0.8, font = 2, col = legend_cols[1])
            # Group 2 is to the right, left-align its label and offset right
            text(x = q2_75 + label_offset, y = 0.75 + 0.02, labels = round(q2_75, 2),
                 adj = c(0, 0.5), cex = 0.8, font = 2, col = legend_cols[2])
          } else {
            # Group 2 is to the left, right-align its label and offset left
            text(x = q2_75 - label_offset, y = 0.75 + 0.02, labels = round(q2_75, 2),
                 adj = c(1, 0.5), cex = 0.8, font = 2, col = legend_cols[2])
            # Group 1 is to the right, left-align its label and offset right
            text(x = q1_75 + label_offset, y = 0.75 + 0.02, labels = round(q1_75, 2),
                 adj = c(0, 0.5), cex = 0.8, font = 2, col = legend_cols[1])
          }
        }
      }
      
      # Add KS test results in bottom right corner (only if show.ks is TRUE)
        if (show.ks) {
          ks_d <- round(ks_test$statistic, 3)
          ks_p <- format_pvalue(ks_test$p.value, include_p = TRUE)
          
          usr <- par("usr")
          x_range <- usr[2] - usr[1]
          y_range <- usr[4] - usr[3]
          # Format KS p-value (format_pvalue with include_p=TRUE gives "p = .05")
          ks_p_formatted <- format_pvalue(ks_test$p.value, include_p = TRUE)
          # Format KS results: "Kolmogorov-Smirnov\nD = xx\np = p"
          # format_pvalue already includes "p = ", so we use it directly
          ks_values <- paste0("Kolmogorov-Smirnov\nD = ", ks_d, "\n", ks_p_formatted)
          # Position in bottom right corner
          text(x = usr[2] - 0.02 * x_range, y = usr[3] + 0.02 * y_range, 
               labels = ks_values,
               adj = c(1, 0), cex = 0.8, font = 1)
        }
    }
  }
  
  # Return ECDFs and test results
  if (has_group) {
    names(ecdf_list) <- as.character(unique_x)
  } else {
    names(ecdf_list) <- "all"
  }
  
  # Build return list
  result <- list(ecdfs = ecdf_list)
  
  # Add test results if 2 groups
  if (has_group && n_groups == 2) {
    result$ks_test <- ks_test_result
    # Add quantile regression models as separate named objects
    if (!is.null(quantile_regression_25)) {
      result$quantile_regression_25 <- quantile_regression_25
    }
    if (!is.null(quantile_regression_50)) {
      result$quantile_regression_50 <- quantile_regression_50
    }
    if (!is.null(quantile_regression_75)) {
      result$quantile_regression_75 <- quantile_regression_75
    }
  }
  
  # Add warnings if any were captured
  if (length(warnings_list) > 0) {
    result$warnings <- warnings_list
  }
  
  invisible(result)
}

#plot_cdf_001
test_that("plot_cdf runs without errors", {
  y <- rnorm(100)
  group <- rep(c("A", "B", "C"), c(30, 40, 30))
  
  # Should not throw errors
  expect_error(plot_cdf(y ~ group), NA)
  
  # Should return invisible list with ECDFs
  result <- plot_cdf(y ~ group)
  expect_true(is.list(result))
  expect_true("ecdfs" %in% names(result))
  expect_equal(length(result$ecdfs), 3)
  expect_true(all(sapply(result$ecdfs, is.function)))
})

#plot_cdf_002
test_that("plot_cdf handles data frame input", {
  df <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
  
  # Use formula syntax
  expect_error(plot_cdf(value ~ group, data = df), NA)
  
  # Check return value
  result <- plot_cdf(value ~ group, data = df)
  expect_equal(length(result$ecdfs), 2)
})

#plot_cdf_003
test_that("plot_cdf accepts df$col ~ df$col2 without bare column names in env", {
  df1 <- data.frame(
    y1 = rnorm(60),
    x1 = rep(c("A", "B", "C"), each = 20)
  )
  expect_error(plot_cdf(df1$y1 ~ df1$x1), NA)
  result <- plot_cdf(df1$y1 ~ df1$x1)
  expect_equal(length(result$ecdfs), 3)
})

#plot_cdf_004
test_that("plot_cdf handles formula syntax", {
  df <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
  
  expect_error(plot_cdf(value ~ group, data = df), NA)
  
  result <- plot_cdf(value ~ group, data = df)
  expect_equal(length(result$ecdfs), 2)
})

#plot_cdf_005
test_that("plot_cdf handles missing values", {
  y <- c(rnorm(90), rep(NA, 10))
  group <- rep(c("A", "B"), 50)
  
  # Should handle NAs gracefully
  expect_error(plot_cdf(y ~ group), NA)
  
  result <- plot_cdf(y ~ group)
  expect_equal(length(result$ecdfs), 2)
})

#plot_cdf_006
test_that("plot_cdf handles custom parameters", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  # Custom colors
  expect_error(plot_cdf(y ~ group, col = c("red", "blue")), NA)
  
  # Custom line width
  expect_error(plot_cdf(y ~ group, lwd = 2), NA)
  
  # Custom line type
  expect_error(plot_cdf(y ~ group, lty = c(1, 2)), NA)
})

#plot_cdf_007
test_that("plot_cdf handles show.ks parameter", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  expect_error(plot_cdf(y ~ group, show.ks = TRUE), NA)
  expect_error(plot_cdf(y ~ group, show.ks = FALSE), NA)
  
  # With 2 groups, KS test should be performed
  result <- plot_cdf(y ~ group, show.ks = TRUE)
  if (length(unique(group)) == 2) {
    expect_true("ks_test" %in% names(result))
  }
})

#plot_cdf_008
test_that("plot_cdf handles show.quantiles parameter", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  expect_error(plot_cdf(y ~ group, show.quantiles = TRUE), NA)
  expect_error(plot_cdf(y ~ group, show.quantiles = FALSE), NA)
})

#plot_cdf_009
test_that("plot_cdf handles different numbers of groups", {
  y <- rnorm(100)
  
  # Two groups
  group2 <- rep(c("A", "B"), 50)
  expect_error(plot_cdf(y ~ group2), NA)
  
  # Three groups
  group3 <- rep(c("A", "B", "C"), c(30, 40, 30))
  expect_error(plot_cdf(y ~ group3), NA)
  
  # Four groups
  group4 <- rep(c("A", "B", "C", "D"), c(25, 25, 25, 25))
  expect_error(plot_cdf(y ~ group4), NA)
})

#plot_cdf_010
test_that("plot_cdf returns correct structure", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  result <- plot_cdf(y ~ group)
  
  # Check structure
  expect_true(is.list(result))
  expect_true("ecdfs" %in% names(result))
  expect_true(is.list(result$ecdfs))
  
  # Check ECDF functions
  expect_true(all(sapply(result$ecdfs, is.function)))
  expect_equal(names(result$ecdfs), c("A", "B"))
  
  # ECDFs should be monotonic
  test_vals <- seq(min(y), max(y), length.out = 10)
  for (ecdf_func in result$ecdfs) {
    vals <- ecdf_func(test_vals)
    expect_true(all(diff(vals) >= 0))
  }
})

#plot_cdf_011
test_that("plot_cdf error message shows correct dataset name", {
  # Create a dataset with a specific name
  IV5 <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
  
  # Try to access a variable that doesn't exist - should show "IV5" not "data"
  expect_error(
    plot_cdf(nonexistent ~ group, data = IV5),
    'Variable "nonexistent" not found in dataset "IV5"',
    fixed = TRUE
  )
})

# ============================================================================
# EDGE CASES
# ============================================================================

#plot_cdf_012
test_that("plot_cdf handles single group", {
  y <- rnorm(50)
  group <- rep("A", 50)  # Only one group
  
  result <- plot_cdf(y ~ group)
  
  expect_true(is.list(result))
  expect_equal(length(result$ecdfs), 1)
})

#plot_cdf_013
test_that("plot_cdf handles all-NA values gracefully", {
  y <- c(rnorm(40), rep(NA, 10))
  group <- rep(c("A", "B"), 25)
  
  # Should handle without error
  result <- plot_cdf(y ~ group)
  
  expect_true(is.list(result))
  expect_equal(length(result$ecdfs), 2)
})

#plot_cdf_014
test_that("plot_cdf handles very small samples per group", {
  y <- c(1, 2, 3, 4, 5, 6)
  group <- c("A", "A", "A", "B", "B", "B")
  
  result <- plot_cdf(y ~ group)
  
  expect_true(is.list(result))
  expect_equal(length(result$ecdfs), 2)
})

#plot_cdf_015
test_that("plot_cdf handles order parameter for groups", {
  y <- rnorm(100)
  group <- rep(c("A", "B", "C"), c(30, 40, 30))
  
  # Custom group order
  expect_error(plot_cdf(y ~ group, order = c("C", "A", "B")), NA)
  result <- plot_cdf(y ~ group, order = c("C", "A", "B"))
  expect_true(is.list(result))
  
  # Check that ecdfs are in specified order
  expect_equal(names(result$ecdfs), c("C", "A", "B"))
})

#plot_cdf_016
test_that("plot_cdf order = -1 reverses default order", {
  y <- rnorm(100)
  group <- rep(c("A", "B", "C"), c(30, 40, 30))
  
  # Default order is A, B, C (sorted)
  result_default <- plot_cdf(y ~ group)
  expect_equal(names(result_default$ecdfs), c("A", "B", "C"))
  
  # order = -1 should reverse to C, B, A
  result_reversed <- plot_cdf(y ~ group, order = -1)
  expect_true(is.list(result_reversed))
  expect_equal(names(result_reversed$ecdfs), c("C", "B", "A"))
})

#plot_cdf_017
test_that("plot_cdf respects factor levels for groups when order is NULL", {
  y <- rnorm(100)
  group <- factor(rep(c("A", "B", "C"), c(30, 40, 30)),
                  levels = c("C", "A", "B"))
  
  result <- plot_cdf(y ~ group)
  expect_true(is.list(result))
  
  # Check that factor levels are respected (C, A, B)
  expect_equal(names(result$ecdfs), c("C", "A", "B"))
})

#plot_cdf_018
test_that("plot_cdf handles two-vector comparison", {
  y1 <- rnorm(50)
  y2 <- rnorm(50)
  
  # Should not throw errors
  expect_error(plot_cdf(y1, y2), NA)
  
  result <- plot_cdf(y1, y2)
  expect_true(is.list(result))
  expect_equal(length(result$ecdfs), 2)
  expect_true("ks_test" %in% names(result))
})

#plot_cdf_019
test_that("plot_cdf two-vector with custom parameters", {
  y1 <- rnorm(50)
  y2 <- rnorm(50)
  
  # Custom colors
  expect_error(plot_cdf(y1, y2, col = c("red", "blue")), NA)
  
  # Without KS test
  expect_error(plot_cdf(y1, y2, show.ks = FALSE), NA)
  
  # Without quantiles
  expect_error(plot_cdf(y1, y2, show.quantiles = FALSE), NA)
})

#plot_cdf_020
test_that("plot_cdf two-vector handles order parameter", {
  y1 <- rnorm(50)
  y2 <- rnorm(50)
  
  result <- plot_cdf(y1, y2, order = -1)
  expect_equal(names(result$ecdfs), c("y2", "y1"))
})

#plot_cdf_021
test_that("plot_cdf two-vector with different lengths", {
  y1 <- rnorm(50)
  y2 <- rnorm(30)  # Different length
  
  # Should handle different lengths
  expect_error(plot_cdf(y1, y2), NA)
  
  result <- plot_cdf(y1, y2)
  expect_equal(length(result$ecdfs), 2)
})

#plot_cdf_022
test_that("plot_cdf reserves space for legend with groups", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  # Should reserve space for legend
  expect_error(plot_cdf(y ~ group), NA)
  
  # Test with two-vector
  y1 <- rnorm(50)
  y2 <- rnorm(50)
  expect_error(plot_cdf(y1, y2), NA)
})

#plot_cdf_023
test_that("plot_cdf xlim adds padding", {
  # Test that xlim includes padding to prevent edge clipping
  y1 <- c(1, 2, 3, 4, 5)
  y2 <- c(1, 2, 3)
  
  expect_error(plot_cdf(y1, y2), NA)
  
  # Single variable
  x <- c(1, 2, 3, 4, 5)
  expect_error(plot_cdf(x), NA)
})

# Package initialization
# This file is executed when the package is loaded

.onLoad <- function(libname, pkgname) {
  # Package initialization code goes here
  # Currently no initialization needed, but this file is here for future use
  invisible()
}

#' @importFrom utils packageVersion
.onAttach <- function(libname, pkgname) {

     #Startup msgs
      #Version and URL for website
         #packageStartupMessage ("Attached: 'statuser' (Version: ",utils::packageVersion('statuser'),  ")") 
          
      #While developing:
      #  packageStartupMessage(
      #   "#######################################################\n",
      #                        "This Version 2026 05 19 - 04.57AM\n" )
        


  invisible()
}

#' Clear cache used by stimulus.plot() to reuse resampling results from identical previous calls.
#'
#' When `stimulus.plot()` with `plot.type = 'effects'` is run, resamples are saved in
#' package state. If an identical call is run again (same data, dv, condition, number of
#' simulations, etc.), stored results are loaded instead of re-calculated. Force
#' recalculation by clearing the cache with this function.
#'
#' @export
clear_stimulus_cache <- function() {
  .statuser_state$stimulus_cache <- list()
  message2("Cache for stimulus.plot() has been cleared.")
}

# Tests for stimulus.plot(), stimulus.beeswarm(), and clear_stimulus_cache()

stimulus_test_data <- function() {
  fp <- testthat::test_path("data", "salerno_slepian_jpsp_study4.csv")
  df1 <- read.csv(fp)
  df1$cond <- ifelse(df1$intent == 1, "Intentional", "Unintentional")
  df1$stimulus2 <- paste0(df1$stimulus, sample(1:6, size = nrow(df1), replace = TRUE))
  df2 <- subset(df1, stimulus %in% unique(df1$stimulus[1:6]))
  list(df1 = df1, df2 = df2)
}

stimulus_compared_data <- function(df1) {
  df1$stim_comp <- ifelse(
    df1$cond == "Intentional",
    paste0("A", df1$stimulus),
    paste0("B", df1$stimulus)
  )
  df1
}

stimulus_observed_numeric_cols <- function() {
  c("effect", "t", "df", "p", "ciL", "ciH")
}

expect_stimulus_effects_matches_saved <- function(actual, saved, tolerance = 1e-4) {
  expect_equal(names(actual), names(saved))
  expect_equal(nrow(actual$observed), nrow(saved$observed))
  expect_equal(actual$observed$stimulus, saved$observed$stimulus)
  num <- stimulus_observed_numeric_cols()
  for (col in num) {
    expect_equal(actual$observed[[col]], saved$observed[[col]], tolerance = tolerance)
  }
  cond_cols <- setdiff(names(actual$observed), c("stimulus", num))
  expect_equal(sort(cond_cols), sort(setdiff(names(saved$observed), c("stimulus", num))))
  for (col in cond_cols) {
    expect_equal(actual$observed[[col]], saved$observed[[col]], tolerance = tolerance)
  }
  expect_equal(actual$p.hetero, saved$p.hetero)
  expect_equal(actual$under.null, saved$under.null, tolerance = tolerance)
  expect_equal(as.matrix(actual$resamples), as.matrix(saved$resamples), tolerance = tolerance)
}

expect_stimulus_means_matches_saved <- function(actual, saved, tolerance = 1e-4) {
  expect_equal(names(actual), names(saved))
  expect_equal(nrow(actual), nrow(saved))
  expect_equal(actual$stimulus, saved$stimulus)
  num <- stimulus_observed_numeric_cols()
  for (col in num) {
    expect_equal(actual[[col]], saved[[col]], tolerance = tolerance)
  }
  cond_cols <- setdiff(names(actual), c("stimulus", num))
  expect_equal(sort(cond_cols), sort(setdiff(names(saved), c("stimulus", num))))
  for (col in cond_cols) {
    expect_equal(actual[[col]], saved[[col]], tolerance = tolerance)
  }
}

#stimulus.plot_001
test_that("stimulus.plot means returns expected structure", {
  dat <- stimulus_test_data()
  grDevices::pdf(tempfile(fileext = ".pdf"))
  on.exit(grDevices::dev.off(), add = TRUE)
  clear_stimulus_cache()

  res <- stimulus.plot(
    data = dat$df1, dv = "rev", stimulus = "stimulus", condition = "cond",
    plot.type = "means", watermark = FALSE
  )

  expect_true(is.data.frame(res))
  expect_true(all(c("stimulus", "effect", "ciL", "ciH") %in% names(res)))
  expect_gt(nrow(res), 0)
})

#stimulus.plot_002
test_that("stimulus.plot effects returns expected structure", {
  dat <- stimulus_test_data()
  grDevices::pdf(tempfile(fileext = ".pdf"))
  on.exit(grDevices::dev.off(), add = TRUE)
  clear_stimulus_cache()
  set.seed(2024)

  res <- stimulus.plot(
    data = dat$df1, dv = "rev", stimulus = "stimulus", condition = "cond",
    plot.type = "effects", simtot = 20, participant = "id", watermark = FALSE, seed = 2024
  )

  expect_true(is.list(res))
  expect_true(all(c("observed", "under.null", "resamples", "p.hetero") %in% names(res)))
  expect_true(is.data.frame(res$observed))
})

#stimulus.plot_003
test_that("stimulus.plot errors for compared-stimulus design", {
  dat <- stimulus_test_data()
  df_comp <- stimulus_compared_data(dat$df1)
  grDevices::pdf(tempfile(fileext = ".pdf"))
  on.exit(grDevices::dev.off(), add = TRUE)

  msgs <- character(0)
  out <- withCallingHandlers(
    withRestarts(
      stimulus.plot(
        data = df_comp, dv = "rev", stimulus = "stim_comp", condition = "cond",
        plot.type = "means", watermark = FALSE
      ),
      abort = function(...) "aborted"
    ),
    message = function(m) {
      msgs <<- c(msgs, conditionMessage(m))
      invokeRestart("muffleMessage")
    }
  )

  expect_identical(out, "aborted")
  expect_true(any(grepl("stimulus.beeswarm", msgs, fixed = TRUE)))
})

#stimulus.plot_004
test_that("stimulus.beeswarm runs on compared-stimulus subset", {
  dat <- stimulus_test_data()
  grDevices::pdf(tempfile(fileext = ".pdf"))
  on.exit(grDevices::dev.off(), add = TRUE)
  clear_stimulus_cache()
  set.seed(2024)

  res <- stimulus.beeswarm(
    data = dat$df2, dv = "rev", stimulus = "stimulus", condition = "cond",
    simtot = 20, watermark = FALSE
  )

  expect_true(is.matrix(res) || is.data.frame(res))
  expect_equal(ncol(res), 2)
})

#stimulus.plot_005
test_that("clear_stimulus_cache resets package cache", {
  dat <- stimulus_test_data()
  grDevices::pdf(tempfile(fileext = ".pdf"))
  on.exit(grDevices::dev.off(), add = TRUE)
  clear_stimulus_cache()
  set.seed(2024)

  invisible(stimulus.plot(
    data = dat$df1, dv = "rev", stimulus = "stimulus", condition = "cond",
    plot.type = "effects", simtot = 20, participant = "id", watermark = FALSE, seed = 2024
  ))
  expect_gt(length(statuser:::sp_stimulus_cache()), 0L)

  clear_stimulus_cache()
  expect_equal(statuser:::sp_stimulus_cache(), list())
})

#stimulus.plot_006
test_that("Salerno & Slepian effects with simtot=20 matches saved reference", {
  dat <- stimulus_test_data()
  grDevices::pdf(tempfile(fileext = ".pdf"))
  on.exit(grDevices::dev.off(), add = TRUE)
  clear_stimulus_cache()
  set.seed(2024)

  r1 <- stimulus.plot(
    data = dat$df1, dv = "rev", stimulus = "stimulus", condition = "cond",
    plot.type = "effects", simtot = 20, participant = "id", watermark = FALSE, seed = 2024
  )

  fp <- testthat::test_path("data", "stimulus_saved_results", "r1.rds")
  skip_if_not(file.exists(fp), "saved r1 fixture missing")
  expect_stimulus_effects_matches_saved(r1, readRDS(fp))
})

#stimulus.plot_007
test_that("Salerno & Slepian means matches saved reference", {
  dat <- stimulus_test_data()
  grDevices::pdf(tempfile(fileext = ".pdf"))
  on.exit(grDevices::dev.off(), add = TRUE)
  clear_stimulus_cache()

  r2 <- stimulus.plot(
    data = dat$df1, dv = "rev", stimulus = "stimulus", condition = "cond",
    plot.type = "means", watermark = FALSE
  )

  fp <- testthat::test_path("data", "stimulus_saved_results", "r2.rds")
  skip_if_not(file.exists(fp), "saved r2 fixture missing")
  expect_stimulus_means_matches_saved(r2, readRDS(fp))
})

#stimulus.plot_008
test_that("Salerno & Slepian model=all subset matches saved reference", {
  skip_if_not(requireNamespace("lmerTest", quietly = TRUE))
  dat <- stimulus_test_data()
  grDevices::pdf(tempfile(fileext = ".pdf"))
  on.exit(grDevices::dev.off(), add = TRUE)
  clear_stimulus_cache()
  set.seed(2024)

  r3 <- tryCatch(
    stimulus.plot(
      data = dat$df1, dv = "rev", stimulus = "stimulus", condition = "cond",
      plot.type = "effects", simtot = 20, model = "all", participant = "id",
      watermark = FALSE, seed = 2024
    ),
    error = function(e) {
      skip(paste("Mixed-model fit unavailable:", conditionMessage(e)))
    }
  )
  r3.list <- list(
    r3$model.results$m.mean,
    r3$model.results$m.ci,
    r3$model.results$m.labels,
    r3$model.results$m.p
  )

  fp <- testthat::test_path("data", "stimulus_saved_results", "r3.rds")
  skip_if_not(file.exists(fp), "saved r3 fixture missing")
  saved <- readRDS(fp)
  expect_equal(r3.list[[3]], saved[[3]])
  expect_equal(r3.list[[1]], saved[[1]], tolerance = 1e-4)
  expect_equal(r3.list[[2]], saved[[2]], tolerance = 1e-4)
  expect_equal(r3.list[[4]], saved[[4]], tolerance = 1e-4)
})

#' Plot frequencies for a variable (histogram without binning)
#'
#' Creates a frequency plot showing the frequency of every observed value, optionaly by group. 
#' Most frequent values are labeled by default.
#'
#' @param formula Two possible uses (similar to \code{t.test()}):
#'   \itemize{
#'     \item {Single Variable (possibly by subgroup)}: \code{plot_freq(y)} or \code{plot_freq(y~x)}
#'     \item {Contrast Two Variables}: \code{plot_freq(y1, y2)}
#'   }
#' @param y2 optional second variable when contrasting two variables \code{plot_freq(y1,y2)}
#' @param data An optional data frame containing the variables in the formula.
#' @param freq Logical. If TRUE (default), displays frequencies. If FALSE, displays percentages.
#' @param order Controls the order in which groups appear in the plot and legend. 
#'   Use \code{-1} to reverse the default order. Alternatively, provide a vector specifying
#'   the exact order (e.g., \code{c("B", "A", "C")}). If \code{NULL} (default), groups are 
#'   ordered by their factor levels (if the grouping variable is a factor) or sorted 
#'   alphabetically/numerically. Only applies when using grouped plots or comparing two variables.
#' @param col Color for the bars. 
#' @param lwd Line width for the frequency bars. Default is 9.
#' @param width Numeric. Width of the frequency bars. If NULL (default), width is automatically calculated based on the spacing between values.
#' @param value.labels Controls value labeling. If numeric, shows labels for the \code{value.labels}
#'   highest-frequency values (including ties). Use \code{-1} or \code{"all"} to show all labels and
#'   \code{0} to show none. Use \code{"auto"} to label all values when there are 30 or fewer
#'   unique values; otherwise label the single most frequent value. For backward compatibility, \code{TRUE}
#'   is treated as \code{-1} and \code{FALSE} as \code{0}.
#' @param ticks.max Integer. Maximum number of unique x values to label on the x-axis. If there are more
#'   than \code{ticks.max} unique values, \code{pretty()} ticks are used instead of labeling every value.
#' @param show.x.value Either \code{"auto"} (default), \code{TRUE}, or \code{FALSE}. If enabled, draws a
#'   small \code{x=<value>} label just above each frequency value label. When \code{"auto"}, x labels are
#'   shown only when plot_freq is not already labeling all x values on the x-axis.
#' 
#' @param show.legend Logical. If TRUE (default), displays a legend when \code{group} is specified. If FALSE, no legend is shown.
#' @param legend.title Character string. Title for the legend when \code{group} is specified. If NULL (default), no title is shown.
#' @param col.text Color for the value labels. If not specified, uses \code{col} for non-grouped plots or group colors for grouped plots.
#' @param ... Pass on any argument accepted by \code{plot()} e.g., \code{xlab='x-axis'} , \code{main='Distribution of X'}
#'
#' @return Invisibly returns a data frame with values and their frequencies.
#'
#' @details
#' This function creates a frequency plot where each observed value is shown
#' with its frequency. Unlike a standard histogram, there is no binning, unlike
#' a barplot, non-observed values of the variable are shown with 0 frequency 
#' instead of skipped.
#'
#' @examples
#' # Simple example
#' x <- c(1, 1, 2, 2, 2, 5, 5)
#' plot_freq(x)
#'
#' # Pass on some common \code{plot()} arguments
#' plot_freq(x, col = "steelblue", xlab = "Value", ylab = "Frequency",ylim=c(0,7))
#'
#' # Add to an existing plot
#' plot_freq(x, col = "dodgerblue")
#'
#'
#' # Compare two vectors
#' y1 <- c(1, 1, 2, 2, 2, 5, 5)
#' y2 <- c(1, 2, 2, 3, 3, 3)
#' plot_freq(y1, y2)
#'
#' # Using a data frame with grouping
#' df <- data.frame(value = c(1, 1, 2, 2, 2, 5, 5), group = c("A", "A", "A", "B", "B", "A", "B"))
#' plot_freq(value ~ 1, data = df)  # single variable
#' plot_freq(value ~ group, data = df)  # with grouping
#'
#' # Control group order in legend and plot
#' plot_freq(value ~ group, data = df, order = c("B", "A"))  # B first, then A
#' plot_freq(value ~ group, data = df, order = -1)  # Reverse default order
#'

#' @export
#'
#: 1 plot_freq: capture call (NSE-safe) -> normalize args -> dispatch -> compute grid -> draw -> return
#: 2 get_plot_freq_x_ticks: choose x-axis tick strategy (all unique vs pretty())
#: 3 topk_with_ties_mask: choose which bars receive frequency labels (top-k with ties)
#: 4 compute_show_x_value: decide whether to draw x=<value> labels above freq labels
#: 5 draw_freq_bars_and_labels: shared base-graphics drawing loop (bars + labels)
#: 6 grouped path: build (n_unique x n_groups) matrices + legend, then draw via helper
#: 7 non-grouped path: build single-group matrices, then draw via helper
#----------------------------------
# plot_freq (exported) ----
plot_freq <- function(formula, y2=NULL, data=NULL, freq=TRUE, order=NULL, col='dodgerblue', lwd=9, width=NULL, value.labels='auto', ticks.max=30, show.x.value="auto", show.legend=TRUE, legend.title=NULL, col.text=NULL, ...) {
  # 0. Capture the call before any evaluation (NSE-safe)
  # We intentionally use sys.call() in addition to match.call(): match.call() will
  # rename partial matches to formal argument names, which breaks detection of
  # user-supplied argument names (notably: ylim= can partially match y2=).
  mc <- match.call()
  # sys.call() preserves original supplied names; match.call() renames partial
  # matches to the formal name (e.g. ylim= becomes y2= in mc, never ylim=).
  sc <- sys.call()
  sc_names <- names(as.list(sc))[-1]
  # If the user wrote ylim= and did not explicitly pass y2=, R may have already
  # bound ylim's value to formal y2 via partial matching. We detect that case so
  # we can (a) avoid evaluating mc$y2 as an NSE variable and (b) forward ylim to
  # plot() via dots later.
  ylim_matched_to_y2 <- "ylim" %in% sc_names && !"y2" %in% sc_names
  
  # Resolve first argument (formula or first vector)
  formula_resolved <- evaluate_variable_arguments(
    arg_expr = mc$formula,
    arg_name = "formula",
    data = data,
    calling_env = parent.frame(),
    func_name = "plot_freq",
    allow_null = FALSE
  )
  
  # Resolve second argument if present (for two-vector mode).
  # Skip when ylim_matched_to_y2: mc$y2 holds a ..N symbol in wrapper contexts
  # which must not be evaluated as a column name.
  y2_resolved <- if (!is.null(mc$y2) && !ylim_matched_to_y2) {
    evaluate_variable_arguments(
      arg_expr = mc$y2,
      arg_name = "y2",
      data = data,
      calling_env = parent.frame(),
      func_name = "plot_freq",
      allow_null = TRUE
    )
  } else {
    list(value = NULL, name = NULL, name_raw = NULL, was_symbol = FALSE)
  }
  
  # Now overwrite the arguments with resolved values.
  # IMPORTANT: Do not force evaluation of y2 unless we are in the special case
  # where R partially matched ylim= into y2. In normal two-vector usage with
  # data=, y2 may be a bare symbol that must be resolved from data.
  y2_before_resolved <- if (isTRUE(ylim_matched_to_y2)) y2 else NULL
  formula <- formula_resolved$value
  y2 <- y2_resolved$value

  # Extract additional arguments
  dots <- list(...)
  
  # Internal-only: allow overlaying on an existing plot frame
  # Passed via ... as .overlay by internal callers; we remove it from dots so it
  # is never forwarded to plot().
    overlay <- isTRUE(dots$.overlay)
    dots$.overlay <- NULL

  # 1. Normalize and validate user-facing arguments
  # value.labels accepts numeric (k), 0 (none), -1/"all" (all), or "auto".
  # Internally we normalize it to a single integer in {-1, 0, 1, 2, ...} and keep
  # a separate flag to remember whether the user requested "auto" (for messaging).
    value_labels_auto <- FALSE
    if (is.character(value.labels)) {
      if (length(value.labels) != 1 || is.na(value.labels)) {
        stop("plot_freq(): 'value.labels' must be numeric, TRUE/FALSE, or one of: \"all\", \"auto\"", call. = FALSE)
      }
      if (value.labels == "all") {
        value.labels <- -1
      } else if (value.labels == "auto") {
        value_labels_auto <- TRUE
        value.labels <- 1
      } else {
        stop("plot_freq(): 'value.labels' must be numeric, TRUE/FALSE, or one of: \"all\", \"auto\"", call. = FALSE)
      }
    }
    if (isTRUE(value.labels)) value.labels <- -1
    if (identical(value.labels, FALSE)) value.labels <- 0
    if (length(value.labels) != 1 || is.na(value.labels) || !is.numeric(value.labels)) {
      stop("plot_freq(): 'value.labels' must be numeric, TRUE/FALSE, or one of: \"all\", \"auto\"", call. = FALSE)
    }
    if (value.labels < -1) {
      stop("plot_freq(): 'value.labels' must be -1 (all), 0 (none), or a positive integer", call. = FALSE)
    }
    if (value.labels > 0) {
      value.labels <- as.integer(value.labels)
    }

  # ticks.max controls when we stop labeling every unique x on the x-axis and
  # switch to a smaller pretty() tick set.
    if (length(ticks.max) != 1 || is.na(ticks.max) || !is.numeric(ticks.max)) {
      stop("plot_freq(): 'ticks.max' must be a single positive numeric value", call. = FALSE)
    }
    if (ticks.max < 1) {
      stop("plot_freq(): 'ticks.max' must be >= 1", call. = FALSE)
    }
    ticks.max <- as.integer(ticks.max)

  # show.x.value controls whether we annotate x=<value> above frequency labels.
  # "auto" means: show only when the x-axis is not already labeling all unique x.
    if (is.character(show.x.value)) {
      if (length(show.x.value) != 1 || is.na(show.x.value) || show.x.value != "auto") {
        stop("plot_freq(): 'show.x.value' must be \"auto\", TRUE, or FALSE", call. = FALSE)
      }
    } else if (is.logical(show.x.value)) {
      if (length(show.x.value) != 1 || is.na(show.x.value)) {
        stop("plot_freq(): 'show.x.value' must be \"auto\", TRUE, or FALSE", call. = FALSE)
      }
    } else {
      stop("plot_freq(): 'show.x.value' must be \"auto\", TRUE, or FALSE", call. = FALSE)
    }

  # 2. Internal constants/helpers (keep behavior, improve readability)
    VALUE_LABELS_AUTO_UNIQUE_CUTOFF <- 30
    
    # Emit the standard message shown when value.labels="auto" and the plot has
    # more unique values than the auto cutoff.
    warn_value_labels_auto_many <- function() {
      message2("plot_freq() says: because there are more than 30 unique values, frequency was printed only for the mode. Set  `value.labels` to modify this behavior")
    }
    
    # Return a logical mask selecting the bars that should receive frequency
    # labels when value.labels is a positive integer k (top-k, including ties).
    topk_with_ties_mask <- function(fs, k) {
      # fs: numeric vector, k: positive integer
      non_zero <- fs > 0
      if (!any(non_zero)) return(rep(FALSE, length(fs)))
      if (sum(non_zero) <= k) return(non_zero)
      cutoff <- sort(fs[non_zero], decreasing = TRUE)[k]
      non_zero & (fs >= cutoff)
    }
    
    # Decide whether to draw x=<value> above the frequency labels.
    # user_provided_xaxt is TRUE when the user directly controls x-axis ticks.
    compute_show_x_value <- function(show_x_value, user_provided_xaxt, n_unique, ticks_max) {
      if (is.character(show_x_value) && show_x_value == "auto") {
        return((!user_provided_xaxt) && (n_unique > ticks_max))
      }
      if (isTRUE(show_x_value)) {
        return((user_provided_xaxt) || (n_unique > ticks_max))
      }
      FALSE
    }

    # Shared drawing helper for grouped and non-grouped plots.
    # Expects a common (n_unique x n_groups) representation plus per-group offsets.
    draw_freq_bars_and_labels <- function(x_unique,
                                          heights_by_group,
                                          offsets,
                                          width,
                                          bar_cols,
                                          label_masks_by_group,
                                          label_text_by_group,
                                          show_x_value,
                                          col_text = NULL) {
      n_groups <- ncol(heights_by_group)
      # Draw bars
        for (i in seq_len(n_groups)) {
          fs_i <- heights_by_group[, i]
          non_zero <- fs_i > 0
          if (any(non_zero)) {
            for (j in which(non_zero)) {
              x_val <- x_unique[j]
              freq_val <- fs_i[j]
              x_center <- x_val + offsets[i]
              x_left <- x_center - width/2
              x_right <- x_center + width/2
              
              polygon(x = c(x_left, x_right, x_right, x_left),
                      y = c(0, 0, freq_val, freq_val),
                      col = bar_cols[i], border = bar_cols[i])
            }
          }
        }
      
      # Draw labels
        for (i in seq_len(n_groups)) {
          mask_i <- label_masks_by_group[, i]
          if (any(mask_i)) {
            x_vals <- x_unique[mask_i] + offsets[i]
            y_vals <- heights_by_group[mask_i, i]
            
            label_color <- if (!is.null(col_text)) col_text else bar_cols[i]
            text(x = x_vals, y = y_vals, labels = label_text_by_group[mask_i, i],
                 cex = 0.8, pos = 3, col = label_color)
            
            if (show_x_value) {
              text(x = x_vals, y = y_vals,
                   labels = paste0("x=", x_unique[mask_i]),
                   cex = 0.7, pos = 3, offset = 1.2, col = "gray77")
            }
          }
        }
    }

  # R's partial matching binds ylim= to formal y (since "ylim" begins with "y").
  # Move it to dots so it reaches plot() correctly.
  if (ylim_matched_to_y2 && !"ylim" %in% names(dots)) {
    dots$ylim <- y2_before_resolved
    y2 <- NULL
  }

  # Decide x-axis tick positions for plot_freq
  # - If there are <= max_unique_ticks unique x values, we label them all.
  # - Otherwise we use pretty() on the plotting x-range and then clip.
    get_plot_freq_x_ticks <- function(x_values, dots, max_unique_ticks = 30) {
      # Prefer user-supplied xlim (or computed dots$xlim) when available
        xlim_used <- if ("xlim" %in% names(dots)) dots$xlim else range(x_values, finite = TRUE)
        xlim_used <- as.numeric(xlim_used)
        if (length(xlim_used) != 2 || any(!is.finite(xlim_used))) {
          xlim_used <- range(x_values, finite = TRUE)
        }

      # If few unique values, label them all; otherwise use pretty() on the range
        x_ticks <- if (length(x_values) <= max_unique_ticks) {
          x_values
        } else {
          pretty(xlim_used, n = 10)
        }

      # Clip ticks to the plotting x-range (avoid drawing far outside)
        x_ticks <- x_ticks[x_ticks >= min(xlim_used) & x_ticks <= max(xlim_used)]
        x_ticks
    }
  
  #----------------------------------
  # Dispatch: two-vector vs formula mode ----
  # Two-vector mode (plot_freq(y1, y2)) is implemented by converting to a grouped
  # data.frame and delegating to the formula path to reuse the same plotting code.
  #----------------------------------
  # Check if we're in two-vector comparison mode (formula is vector, y is vector)
  if (!is.null(y2) && !inherits(formula, "formula")) {
    # Two-vector comparison mode: plot_freq(y1, y2)
    
    y1_name <- formula_resolved$name
    y2_name <- y2_resolved$name
    
    # Validate inputs
    if (!is.numeric(formula) || !is.vector(formula)) {
      stop(sprintf("plot_freq(): First argument must be a numeric vector"), call. = FALSE)
    }
    if (!is.numeric(y2) || !is.vector(y2)) {
      stop(sprintf("plot_freq(): Second argument must be a numeric vector"), call. = FALSE)
    }
    
    # Create a data frame and recursively call with grouped syntax
    df <- data.frame(
      value = c(formula, y2),
      group = c(rep(y1_name, length(formula)), rep(y2_name, length(y2))),
      stringsAsFactors = FALSE
    )
    
    # Forward all arguments to the grouped version
    return(plot_freq(value ~ group, data = df, freq = freq, order = order, 
                     col = col, lwd = lwd, width = width, 
                     value.labels = value.labels, ticks.max = ticks.max, show.x.value = show.x.value, 
                     show.legend = show.legend, legend.title = legend.title, 
                     col.text = col.text, ...))
  }
  
  #----------------------------------
  # Standard mode: formula syntax ----
  #----------------------------------
  # Validate formula early if it is one
  validate_formula(formula, data, func_name = "plot_freq", calling_env = parent.frame())
  
  # Check if formula is actually a formula or a vector
  is_formula_input <- inherits(formula, "formula")
  
  # Capture data name for error messages
  data_expr <- mc$data
  data_name <- if (!is.null(data_expr)) {
    data_name_val <- deparse(data_expr)
    gsub('^"|"$', '', data_name_val)
  } else {
    NULL
  }
  
  # Capture original group before validation (to preserve factor levels)
  group_original_before_validation <- NULL
  if (is_formula_input && inherits(formula, "formula")) {
    f <- formula
    if (length(f) >= 3L && identical(f[[1L]], quote(`~`))) {
      rhs <- f[[3L]]
      rhs_is_intercept_only <- is.numeric(rhs) && length(rhs) == 1L &&
        !is.na(rhs[1]) && rhs[1] == 1
      if (!rhs_is_intercept_only) {
        group_original_before_validation <- tryCatch({
          if (!is.null(data)) {
            eval(rhs, envir = data, enclos = parent.frame())
          } else {
            fe <- environment(f)
            if (is.null(fe)) fe <- parent.frame()
            eval(rhs, envir = fe)
          }
        }, error = function(e) NULL)
      }
    }
  }
  
  # Validate inputs using validation function shared with plot_density, plot_cdf, plot_freq
  # If formula input, use validate_plot
  if (is_formula_input) {
    validated <- validate_plot(formula, NULL, data, func_name = "plot_freq", require_group = FALSE, data_name = data_name)
  } else {
    # Not a formula - use resolved names from evaluate_variable_arguments
    validated <- list(
      y = formula,
      group = NULL,
      y_name = formula_resolved$name,
      group_name = NULL,
      y_name_raw = formula_resolved$name_raw,
      group_name_raw = NULL,
      data_name = data_name
    )
  }
  x <- validated$y  # Note: validate_plot uses 'y' but we use 'x'
  group <- validated$group
  x_name <- validated$y_name
  group_name <- validated$group_name
  x_name_raw <- validated$y_name_raw
  group_name_raw <- validated$group_name_raw
  
  # Store original group for factor level checking (before NA removal)
  # Use the version captured before validation if available (to preserve factor levels)
  group_original <- if (!is.null(group_original_before_validation)) {
    group_original_before_validation
  } else {
    group
  }
  
  # Drop missing data
  if (!is.null(group)) {
    isnagroup=is.na(group)
    isnax=is.na(x)
    group=group[!isnagroup & !isnax]
    x=x[!isnagroup & !isnax]
    
    n.nagroup = sum(isnagroup)
    n.nax = sum(isnax)
    
    if (n.nagroup>0) message2("plot_freq() says: dropped ",n.nagroup," observations with missing '",group_name_raw,"' values",col='red2')
    if (n.nax>0) message2("plot_freq() says: dropped ",n.nax," observations with missing '",x_name_raw,"' values",col='red2')
  } else {
    # No group variable - just drop missing x values
    isnax=is.na(x)
    x=x[!isnax]
    
    n.nax = sum(isnax)
    if (n.nax>0) message2("plot_freq() says: dropped ",n.nax," observations with missing '",x_name_raw,"' values",col='red2')
  }
  
  #----------------------------------
  # Grouped path (x ~ group) ----
  #----------------------------------
  if (!is.null(group)) {
    # Validate group argument
    if (length(group) != length(x)) {
      stop("'group' must have the same length as 'x'")
    }
    
    # Determine group ordering
    unique_groups <- unique(group)
    n_groups <- length(unique_groups)
    if (n_groups < 2 || n_groups > 4) {
      stop("'group' must have 2, 3, or 4 unique values")
    }
    
    # Check if order = -1 (reverse default order)
    reverse_order <- FALSE
    if (!is.null(order) && length(order) == 1 && is.numeric(order) && order == -1) {
      reverse_order <- TRUE
      order <- NULL  # Process as default, then reverse
    }
    
    if (!is.null(order)) {
      # User specified custom order
      # Validate that order contains all groups
      missing_groups <- setdiff(unique_groups, order)
      extra_groups <- setdiff(order, unique_groups)
      
      if (length(missing_groups) > 0) {
        stop(sprintf("plot_freq(): 'order' is missing group(s): %s", 
                     paste(missing_groups, collapse = ", ")), call. = FALSE)
      }
      if (length(extra_groups) > 0) {
        warning(sprintf("plot_freq(): 'order' contains group(s) not in data: %s", 
                       paste(extra_groups, collapse = ", ")))
      }
      
      # Use the specified order (only groups that exist in data)
      unique_by <- order[order %in% unique_groups]
    } else if (is.factor(group_original)) {
      # Respect factor levels
      factor_levels <- levels(group_original)
      # Only include levels that actually appear in the data
      unique_by <- factor_levels[factor_levels %in% unique_groups]
    } else {
      # Default: sort alphabetically/numerically
      unique_by <- sort(unique_groups)
    }
    
    # Reverse order if order = -1 was specified
    if (reverse_order) {
      unique_by <- rev(unique_by)
    }
    
    # Use provided colors if valid, otherwise use default colors for groups
    if (length(col) == n_groups && is.character(col)) {
      group_cols <- col
    } else {
      group_cols <- get.colors(n_groups)
    }
    
    # Compute frequencies using cross-tabulation
    all_xs <- sort(unique(x))
    freq_table <- table(x, group)
    
    # Get column names from table (these are the unique values of group in table order)
    table_by_cols <- colnames(freq_table)
    # Match table columns to unique_by order to ensure correct group assignment
    col_indices <- match(unique_by, table_by_cols)
    
    # Check for any NA values in col_indices (shouldn't happen but defensive programming)
    if (any(is.na(col_indices))) {
      stop(sprintf("plot_freq(): Internal error - group matching failed. unique_by: %s, table_by_cols: %s",
                   paste(unique_by, collapse=", "), paste(table_by_cols, collapse=", ")), call. = FALSE)
    }
    
    # Convert to list format for each group
    group_freqs <- list()
    group_freqs_original <- list()  # Store original frequencies for return value
    total <- length(x)  # Total sample size
    for (i in seq_len(n_groups)) {
      # Extract frequencies for this group, ensuring all x values are included
      group_fs <- numeric(length(all_xs))
      x_in_group <- x[group == unique_by[i]]
      counts <- table(x_in_group)
      idx <- match(names(counts), all_xs)
      group_fs[idx] <- as.numeric(counts)
      
      # Store original frequencies for return value
      group_freqs_original[[i]] <- list(xs = all_xs, fs = group_fs)
      
      # Convert to percentages if requested (relative to this group's sample size)
      if (freq == FALSE) {
        group_total <- sum(group_fs)  # Sample size for this group
        group_fs <- (group_fs / group_total) * 100
      }
      
      group_freqs[[i]] <- list(xs = all_xs, fs = group_fs)
    }
    
    # Find overall max frequency for ylim (already in percentages if freq=FALSE)
    max_fs <- max(sapply(group_freqs, function(gf) max(gf$fs, na.rm = TRUE)), na.rm = TRUE)
    
    # Calculate bar width if not provided (needed for xlim calculation and later use)
    if (is.null(width)) {
      if (length(all_xs) > 1) {
        min_spacing <- min(diff(sort(all_xs)))
        width_calc <- min_spacing * 0.2  # 20% of minimum spacing
      } else {
        width_calc <- 0.15  # fallback
      }
    } else {
      width_calc <- width
    }
    
    # Default for overlay mode: treat x-axis labeling as user-controlled
    user_provided_xaxt <- TRUE
    
    # Only set up plot if not overlaying on an existing frame
    if (!overlay) {
      
      # Set default xlim if not set, with padding for bar width
      if (!"xlim" %in% names(dots)) {
        # Bars are drawn with group-specific center offsets, so the outer edge can extend
        # beyond min/max(all_xs) by more than width/2.
        #
        # - 2 groups: outer edge at x ± 1.0*width
        # - 3 groups: outer edge at x ± 1.5*width
        # - 4 groups: outer edge at x ± 2.0*width
        outer_extent <- (n_groups / 2) * width_calc
        padding <- outer_extent * 1.1  # Add 10% extra padding
        dots$xlim <- c(min(all_xs) - padding, max(all_xs) + padding)
      }
      
      # Set default xlab if not provided
      if (!"xlab" %in% names(dots)) dots$xlab <- x_name
      
      # Set default ylab if not provided
      if (!"ylab" %in% names(dots) && freq) dots$ylab <- "Frequency"
      if (!"ylab" %in% names(dots) && !freq) dots$ylab <- "% of Observations"
      
      # Set default main title if not provided
      if (!"main" %in% names(dots)) {
        dots$main <- paste0("Distribution of ", x_name, "")
      }
      
      # Set default ylim to start at 0 if not provided
      if (!"ylim" %in% names(dots)) {
        y_max <- max_fs
        # Reserve top 20% for legend if legend will be shown
        if (show.legend) {
          y_max <- y_max * 1.25  # Increase by 25% to make room for legend
        }
        dots$ylim <- c(0, y_max)
      }
      
      # Remove default axis padding to eliminate gap at bottom
      if (!"yaxs" %in% names(dots)) dots$yaxs <- "i"
      
      # Only set xaxs = "i" if xlim is not provided (to allow padding when xlim is set)
      if (!"xaxs" %in% names(dots) && !"xlim" %in% names(dots)) dots$xaxs <- "i"
      
      # Add las=1 to dots if not provided
      if (!"las" %in% names(dots)) dots$las <- 1
      
      # Set axis label formatting (xlab/ylab, not tick labels)
      if (!"font.lab" %in% names(dots)) dots$font.lab <- 2
      if (!"cex.lab" %in% names(dots)) dots$cex.lab <- 1.2
      
      # Set main title formatting (15% bigger than default)
      if (!"cex.main" %in% names(dots)) dots$cex.main <- 1.38
      
      # Set type='n' to create plot frame without drawing points (unless user specifies type)
      if (!"type" %in% names(dots)) dots$type <- "n"
      
      # Track if user provided yaxt - if not, we'll draw custom axis
      user_provided_yaxt <- "yaxt" %in% names(dots)
      if (!user_provided_yaxt) dots$yaxt <- "n"
      
      # Track if user provided xaxt - if not, we'll draw custom axis with all x values
      user_provided_xaxt <- "xaxt" %in% names(dots)
      if (!user_provided_xaxt) dots$xaxt <- "n"
      
      # Ensure adequate top margin for main title and (N=...) text
      old_mar <- par("mar")
      on.exit(par(mar = old_mar), add = TRUE)
      if (!"mar" %in% names(dots)) {
        # Increase top margin if it's too small (less than 5 lines to accommodate title and N)
        if (old_mar[3] < 5) {
          par(mar = c(old_mar[1], old_mar[2], 5, old_mar[4]))
        }
      }
      
      # Plot the frequencies (empty plot frame)
      plot_args <- c(list(x = all_xs, y = rep(0, length(all_xs))), dots)
      do.call(plot, plot_args)
      
      # Calculate total sample size and add it below the main title
      # Main title is typically at line 3, so position (N=...) at line 0.75 (between positions used for grouped and non-grouped plots)
      tot <- length(x)
      mtext(paste0("(N=", tot, ")"), side = 3, line = 0.75, font = 3, cex = 0.9)
      
      # Draw custom x-axis with all x values (if we suppressed default)
      if (!user_provided_xaxt) {
        existing_ticks <- get_plot_freq_x_ticks(all_xs, dots, max_unique_ticks = ticks.max)
        axis(1, at = existing_ticks, las = 1)
      }
      
      # Draw custom y-axis with tickmarks at 0, midpoint, and maximum (if we suppressed default)
      if (!user_provided_yaxt) {
        # Use max_fs for tick calculation (actual data range, not expanded ylim)
        # max_fs is already in percentages if freq=FALSE
        y_max_plot <- max_fs
        if ("ylim" %in% names(dots)) {
          y_max_plot <- dots$ylim[2]
        }
        
        if (freq == FALSE) {
          # When showing percentages, use pretty() to generate reasonable tick marks
          # max_fs is already in percentages
          pct_range <- c(0, y_max_plot)
          pct_ticks <- pretty(pct_range, n = 5)
          pct_ticks <- pct_ticks[pct_ticks >= 0 & pct_ticks <= y_max_plot + 0.1]
          
          # Create labels with % sign
          y_labels <- paste0(pct_ticks, "%")
          
          # Ticks are positioned at percentage values (since group_freqs$fs is in percentages)
          axis(2, at = pct_ticks, labels = y_labels, las = 1)
        } else {
          # Use pretty() to generate nice tick intervals for frequency mode
          freq_range <- c(0, y_max_plot)
          y_ticks <- pretty(freq_range, n = 5)
          # Only keep ticks that are >= 0 and <= y_max_plot (with small tolerance for rounding)
          y_ticks <- y_ticks[y_ticks >= 0 & y_ticks <= y_max_plot + 0.1]
          
          axis(2, at = y_ticks, las = 1)
        }
      }
    }

    # Decide whether to show x=<value> labels above frequencies
      do_show_x_value <- compute_show_x_value(show.x.value, user_provided_xaxt, length(all_xs), ticks.max)
    
    # Use the width calculated earlier (or user-provided width)
    if (is.null(width)) {
      width <- width_calc
    }
    
    # Calculate offsets for each group so bars touch exactly
    # Centers are equally spaced by `width`, symmetric around 0.
    offsets <- (seq_len(n_groups) - (n_groups + 1) / 2) * width

    # Prepare the common representation used by draw_freq_bars_and_labels():
    # heights_by_group, label masks, and label text are all (n_unique x n_groups).
      heights_by_group <- do.call(cbind, lapply(group_freqs, function(gf) gf$fs))

    # Determine value-label masks and label text (n_unique x n_groups)
      label_masks_by_group <- matrix(FALSE, nrow = length(all_xs), ncol = n_groups)
      label_text_by_group <- matrix("", nrow = length(all_xs), ncol = n_groups)
      if (value.labels != 0) {
        value_labels_effective <- value.labels
        if (isTRUE(value_labels_auto) && length(all_xs) <= VALUE_LABELS_AUTO_UNIQUE_CUTOFF) {
          value_labels_effective <- -1
        }
        if (isTRUE(value_labels_auto) && length(all_xs) > VALUE_LABELS_AUTO_UNIQUE_CUTOFF) {
          warn_value_labels_auto_many()
        }

        for (i in seq_len(n_groups)) {
          fs_i <- heights_by_group[, i]
          if (value_labels_effective == -1) {
            label_masks_by_group[, i] <- fs_i > 0
          } else if (value_labels_effective > 0) {
            label_masks_by_group[, i] <- topk_with_ties_mask(fs_i, value_labels_effective)
          }
          if (any(label_masks_by_group[, i])) {
            if (freq == FALSE) {
              label_text_by_group[label_masks_by_group[, i], i] <- paste0(round(fs_i[label_masks_by_group[, i]], 0), "%")
            } else {
              label_text_by_group[label_masks_by_group[, i], i] <- as.character(fs_i[label_masks_by_group[, i]])
            }
          }
        }
      }

    # Draw bars and labels
      draw_freq_bars_and_labels(
        x_unique = all_xs,
        heights_by_group = heights_by_group,
        offsets = offsets,
        width = width,
        bar_cols = group_cols,
        label_masks_by_group = label_masks_by_group,
        label_text_by_group = label_text_by_group,
        show_x_value = do_show_x_value,
        col_text = col.text
      )
    
    # Add legend showing groups and colors
    if (!overlay && show.legend) {
      # Calculate sample sizes for each group
      group_ns <- sapply(seq_len(n_groups), function(i) {
        length(x[group == unique_by[i]])
      })
      
      # Create legend labels with sample sizes, aligned so N=xxx is at same position
      group_names <- as.character(unique_by)
      # Calculate text width needed for alignment
      # Measure the width of group names plus some padding
      temp_cex <- 1.2  # Match the legend cex
      text_widths <- strwidth(group_names, cex = temp_cex, units = "user")
      max_name_width <- max(text_widths)
      # Calculate width needed for the full label format: "name (N=xxx)"
      # We'll set text.width to accommodate the longest name plus "(N=xxx)" part
      sample_n_text <- paste0(" (N=", max(group_ns), ")")
      sample_n_width <- strwidth(sample_n_text, cex = temp_cex, units = "user")
      total_text_width <- max_name_width + sample_n_width
      
      # Format labels with padding to align N=xxx
      padded_labels <- sapply(seq_len(n_groups), function(i) {
        name_width <- text_widths[i]
        padding_needed <- max_name_width - name_width
        # Estimate number of spaces needed (using space width)
        space_width <- strwidth(" ", cex = temp_cex, units = "user")
        n_spaces <- max(1, round(padding_needed / space_width))
        paste0(group_names[i], strrep(" ", n_spaces), " (N=", group_ns[i], ")")
      })
      legend_labels <- padded_labels
      legend_cols <- group_cols[seq_len(n_groups)]
      
      # Add legend with text.width to ensure consistent alignment
      # Position at top center since we reserved 20% of space
      legend_args <- list("top", legend = legend_labels, fill = legend_cols, 
                          bty = "n", inset = 0.02, cex=1.2, text.width = total_text_width, horiz = FALSE)
      if (!is.null(legend.title)) {
        legend_args$title <- legend.title
        legend_args$title.font <- 2
      }
      do.call(legend, legend_args)
    }
    
    # Return frequencies or percentages invisibly (full table with separate columns for each group)
    # Build data frame with value column and one column per group
    result_df <- data.frame(value = all_xs, stringsAsFactors = FALSE)
    for (i in seq_len(n_groups)) {
      # Use percentages if freq=FALSE, otherwise use frequencies
      if (freq == FALSE) {
        result_df[[as.character(unique_by[i])]] <- group_freqs[[i]]$fs
      } else {
        result_df[[as.character(unique_by[i])]] <- group_freqs_original[[i]]$fs
      }
    }
    return(invisible(result_df))
  }
  
  #----------------------------------
  # Non-grouped path (x only) ----
  #----------------------------------
  # Calculate frequencies for each unique value
  freq_table <- table(x)
  xs <- as.numeric(names(freq_table))
  fs <- as.numeric(freq_table)
  fs_original <- fs  # Store original frequencies for return value
  fsp=fs
  
  # Convert frequencies to percentages if requested
  total <- sum(fs)
  if (freq==FALSE)
  {
    fs <- (fs / total) * 100  # Convert to percentages
    fsp <- paste0(round(fs, 0),"%")
  }
  
  # Calculate bar width if not provided (needed for xlim calculation and later use)
  if (is.null(width)) {
    if (length(xs) > 1) {
      min_spacing <- min(diff(sort(xs)))
      width_calc <- min_spacing * 0.2  # 20% of minimum spacing
    } else {
      width_calc <- 0.15  # fallback
    }
  } else {
    width_calc <- width
  }
    
  # Default for overlay mode: treat x-axis labeling as user-controlled
  user_provided_xaxt <- TRUE
  
  # Only set up plot if not overlaying on an existing frame
  if (!overlay) {
    #########################################################
    #Default figure parameters if not set
      
      # xlim if not set - add padding for bar width
          if (!"xlim" %in% names(dots)) {
            # For single variable, bars are centered, so add half-width padding
            padding <- width_calc * 0.6  # 60% of width for padding
            dots$xlim <- c(min(xs) - padding, max(xs) + padding)
          }
      
      # Set default xlab if not provided
          if (!"xlab" %in% names(dots)) dots$xlab <- x_name
      
      # Set default ylab if not provided
          if (!"ylab" %in% names(dots) && freq) dots$ylab <- "Frequency"
          if (!"ylab" %in% names(dots) && !freq) dots$ylab <- "% of Observations"
          
      # Set default main title if not provided
         if (!"main" %in% names(dots)) dots$main <- paste("Distribution of", x_name)
      
      # Set default ylim to start at 0 if not provided
          if (!"ylim" %in% names(dots)) {
            y_max <- max(fs, na.rm = TRUE)
            # Add a small buffer so the tallest bar doesn't touch the top border
            if (is.finite(y_max) && y_max > 0) {
              y_max <- y_max + max(1, y_max * 0.10)
            }
            dots$ylim <- c(0, y_max)
          }
        
          # Remove default axis padding to eliminate gap at bottom
            if (!"yaxs" %in% names(dots)) dots$yaxs <- "i"
            
          # Only set xaxs = "i" if xlim is not provided (to allow padding when xlim is set)
            if (!"xaxs" %in% names(dots) && !"xlim" %in% names(dots)) dots$xaxs <- "i"
          
          # Add las=1 to dots if not provided
            if (!"las" %in% names(dots)) dots$las <- 1
          
          # Set axis label formatting (xlab/ylab, not tick labels)
            if (!"font.lab" %in% names(dots)) dots$font.lab <- 2
            if (!"cex.lab" %in% names(dots)) dots$cex.lab <- 1.2
          
          # Set main title formatting (15% bigger than default)
          if (!"cex.main" %in% names(dots)) dots$cex.main <- 1.38
          
          # Set type='n' to create plot frame without drawing points (unless user specifies type)
            if (!"type" %in% names(dots)) dots$type <- "n"
          
          # Track if user provided yaxt - if not, we'll draw custom axis
            user_provided_yaxt <- "yaxt" %in% names(dots)
            if (!user_provided_yaxt) dots$yaxt <- "n"
            
          # Track if user provided xaxt - if not, we'll draw custom axis with all x values
            user_provided_xaxt <- "xaxt" %in% names(dots)
            if (!user_provided_xaxt) dots$xaxt <- "n"
    #########################################################
    
      # Ensure adequate top margin for main title and (N=...) text
      old_mar <- par("mar")
      on.exit(par(mar = old_mar), add = TRUE)
      if (!"mar" %in% names(dots)) {
        # Increase top margin if it's too small (less than 5 lines to accommodate title and N)
        if (old_mar[3] < 5) {
          par(mar = c(old_mar[1], old_mar[2], 5, old_mar[4]))
        }
      }
              
                  
    # Plot the frequencies (empty plot frame)
        plot_args <- c(list(x = xs, y = fs), dots)
        do.call(plot, plot_args)
      
    # Calculate total sample size and add it below the main title
    # Main title is typically at line 3, so position (N=...) at line 0.75 (between positions used for grouped and non-grouped plots)
        tot <- total  # Use the original total (before percentage conversion)
        mtext(paste0("(N=", tot, ")"), side = 3, line = 0.75, font = 3, cex = 0.9)
        
    # Draw custom x-axis with all x values (if we suppressed default)
      if (!user_provided_xaxt) {
        existing_ticks <- get_plot_freq_x_ticks(xs, dots, max_unique_ticks = ticks.max)
        axis(1, at = existing_ticks, las = 1)
      }
            
    # Draw custom y-axis with tickmarks at 0, midpoint, and maximum (if we suppressed default)
      if (!user_provided_yaxt) {
        # Get y-axis range (fs is now in percentages if freq=FALSE)
        y_max <- max(fs, na.rm = TRUE)
        if ("ylim" %in% names(dots)) {
          y_max_plot <- dots$ylim[2]
        } else {
          y_max_plot <- y_max
        }
        
        if (freq == FALSE) {
          # When showing percentages, use pretty() to generate reasonable tick marks
          # fs is already in percentages, so y_max_plot is also in percentages
          pct_range <- c(0, y_max_plot)
          pct_ticks <- pretty(pct_range, n = 5)
          # Only keep ticks that are >= 0 and <= y_max_plot (with small tolerance for rounding)
          pct_ticks <- pct_ticks[pct_ticks >= 0 & pct_ticks <= y_max_plot + 0.1]
          
          # Create labels with % sign
          y_labels <- paste0(pct_ticks, "%")
          
          # Ticks are positioned at percentage values (since fs is in percentages)
          axis(2, at = pct_ticks, labels = y_labels, las = 1)
        } else {
          # Use pretty() to generate nice tick intervals for frequency mode
          freq_range <- c(0, y_max_plot)
          y_ticks <- pretty(freq_range, n = 5)
          # Only keep ticks that are >= 0 and <= y_max_plot (with small tolerance for rounding)
          y_ticks <- y_ticks[y_ticks >= 0 & y_ticks <= y_max_plot + 0.1]
          
          axis(2, at = y_ticks, las = 1)
        }
      }
  }

  # Decide whether to show x=<value> labels above frequencies
    do_show_x_value <- compute_show_x_value(show.x.value, user_provided_xaxt, length(xs), ticks.max)
    
  # Use the width calculated earlier (or user-provided width)
  if (is.null(width)) {
    width <- width_calc
  }
  
  # Prepare (n_unique x 1) matrix for shared drawing
    heights_by_group <- matrix(fs, ncol = 1)
    bar_cols <- rep(col, 1)
    offsets <- 0

  # Determine value-label mask and label text (n_unique x 1)
    label_masks_by_group <- matrix(FALSE, nrow = length(xs), ncol = 1)
    label_text_by_group <- matrix("", nrow = length(xs), ncol = 1)
    if (value.labels != 0) {
      value_labels_effective <- value.labels
      if (isTRUE(value_labels_auto) && length(xs) <= VALUE_LABELS_AUTO_UNIQUE_CUTOFF) {
        value_labels_effective <- -1
      }
      if (isTRUE(value_labels_auto) && length(xs) > VALUE_LABELS_AUTO_UNIQUE_CUTOFF) {
        warn_value_labels_auto_many()
      }

      if (value_labels_effective == -1) {
        label_masks_by_group[, 1] <- fs > 0
      } else if (value_labels_effective > 0) {
        # Use fs_rank for selection (percent vs counts), but show fsp as the label
        fs_rank <- if (freq == FALSE) fs else fs_original
        label_masks_by_group[, 1] <- topk_with_ties_mask(fs_rank, value_labels_effective)
      }

      if (any(label_masks_by_group[, 1])) {
        label_text_by_group[label_masks_by_group[, 1], 1] <- as.character(fsp[label_masks_by_group[, 1]])
      }
    }

  # Draw bars and labels
    draw_freq_bars_and_labels(
      x_unique = xs,
      heights_by_group = heights_by_group,
      offsets = offsets,
      width = width,
      bar_cols = bar_cols,
      label_masks_by_group = label_masks_by_group,
      label_text_by_group = label_text_by_group,
      show_x_value = do_show_x_value,
      col_text = col.text
    )
    
  # Return frequencies or percentages invisibly
  if (freq == FALSE) {
    # Return percentages
    invisible(data.frame(value = xs, percentage = fs))
  } else {
    # Return frequencies
    invisible(data.frame(value = xs, frequency = fs_original))
  }
}

#plot_freq_001
test_that("plot_freq runs without errors", {
  x <- c(1, 1, 2, 2, 2, 5, 5)
  
  # Should not throw errors (single variable uses x ~ 1)
  expect_error(plot_freq(x ~ 1), NA)
  
  # Should return invisible data frame
  result <- plot_freq(x ~ 1)
  expect_true(is.data.frame(result))
  expect_true("value" %in% names(result) || "x" %in% names(result))
})

#plot_freq_002
test_that("plot_freq handles data frame input", {
  df <- data.frame(value = c(1, 1, 2, 2, 2, 5, 5))
  
  # Use formula syntax
  expect_error(plot_freq(value ~ 1, data = df), NA)
  
  result <- plot_freq(value ~ 1, data = df)
  expect_true(is.data.frame(result))
})

#plot_freq_003
test_that("plot_freq handles grouping", {
  x <- c(1, 1, 2, 2, 2, 5, 5)
  group <- c("A", "A", "A", "B", "B", "A", "B")
  
  expect_error(plot_freq(x ~ group), NA)
  
  result <- plot_freq(x ~ group)
  expect_true(is.data.frame(result))
})

#plot_freq_004
test_that("plot_freq handles group with data frame", {
  df <- data.frame(
    value = c(1, 1, 2, 2, 2, 5, 5),
    group = c("A", "A", "A", "B", "B", "A", "B")
  )
  
  # Use formula syntax
  expect_error(plot_freq(value ~ group, data = df), NA)
  
  result <- plot_freq(value ~ group, data = df)
  expect_true(is.data.frame(result))
})

#plot_freq_005
test_that("plot_freq accepts df$col ~ df$col2 without bare column names in env", {
  df1 <- data.frame(
    value = c(1, 1, 2, 2, 2, 5, 5),
    grp = c("A", "A", "A", "B", "B", "A", "B")
  )
  expect_error(plot_freq(df1$value ~ df1$grp), NA)
  result <- plot_freq(df1$value ~ df1$grp)
  expect_true(is.data.frame(result))
})

#plot_freq_006
test_that("plot_freq handles freq parameter", {
  x <- c(1, 1, 2, 2, 2)
  
  # Frequency plot
  expect_error(plot_freq(x ~ 1, freq = TRUE), NA)
  
  # Proportion plot
  expect_error(plot_freq(x ~ 1, freq = FALSE), NA)
})

#plot_freq_007
test_that("plot_freq handles custom parameters", {
  x <- c(1, 1, 2, 2, 2)
  
  # Custom color
  expect_error(plot_freq(x ~ 1, col = "red"), NA)
  
  # Custom line width
  expect_error(plot_freq(x ~ 1, lwd = 5), NA)
  
  # Custom width
  expect_error(plot_freq(x ~ 1, width = 0.5), NA)
})

#plot_freq_008
test_that("plot_freq handles value.labels parameter", {
  x <- c(1, 1, 2, 2, 2)
  
  # With labels
  expect_error(plot_freq(x ~ 1, value.labels = TRUE), NA)
  
  # Without labels
  expect_error(plot_freq(x ~ 1, value.labels = FALSE), NA)
  
  # Character options
  expect_error(plot_freq(x ~ 1, value.labels = "all"), NA)
  expect_error(plot_freq(x ~ 1, value.labels = "auto"), NA)
  
  # Top-N labels
  expect_error(plot_freq(x ~ 1, value.labels = 5), NA)
  
  # None / all
  expect_error(plot_freq(x ~ 1, value.labels = 0), NA)
  expect_error(plot_freq(x ~ 1, value.labels = -1), NA)
  
  # Grouped path also accepts numeric / logical
  df <- data.frame(value = x, group = c("A", "A", "B", "B", "A"))
  expect_error(plot_freq(value ~ group, data = df, value.labels = 2), NA)
  expect_error(plot_freq(value ~ group, data = df, value.labels = TRUE), NA)
  expect_error(plot_freq(value ~ group, data = df, value.labels = FALSE), NA)
  expect_error(plot_freq(value ~ group, data = df, value.labels = "all"), NA)
  expect_error(plot_freq(value ~ group, data = df, value.labels = "auto"), NA)
  
  # In grouped plots, numeric value.labels applies per group (still should not error)
  df2 <- data.frame(value = c(1, 1, 1, 2, 2, 3, 3, 4),
                    group = c("A", "A", "A", "A", "B", "B", "B", "B"))
  expect_error(plot_freq(value ~ group, data = df2, value.labels = 1), NA)
  
  # Labeling does not affect returned data
  r1 <- plot_freq(value ~ group, data = df2, value.labels = 1)
  r0 <- plot_freq(value ~ group, data = df2, value.labels = 0)
  expect_equal(r1, r0)
  
  # Invalid inputs
  expect_error(plot_freq(x ~ 1, value.labels = -2))
  expect_error(plot_freq(x ~ 1, value.labels = NA))
  expect_error(plot_freq(x ~ 1, value.labels = c(1, 2)))
  expect_error(plot_freq(x ~ 1, value.labels = "nope"))
})

#plot_freq_009
test_that("plot_freq handles show.x.value parameter", {
  x <- c(1, 1, 2, 2, 2, 3, 3)
  df <- data.frame(value = x, group = rep(c("A", "B"), length.out = length(x)))
  
  # Non-grouped: x= only where frequency labels are drawn
  expect_error(plot_freq(x ~ 1, value.labels = 2, show.x.value = "auto"), NA)
  expect_error(plot_freq(x ~ 1, value.labels = 0, show.x.value = "auto"), NA)
  
  # Grouped
  expect_error(plot_freq(value ~ group, data = df, value.labels = 2, show.x.value = "auto"), NA)
  
  # Explicit TRUE/FALSE still accepted
  expect_error(plot_freq(x ~ 1, show.x.value = TRUE), NA)
  expect_error(plot_freq(x ~ 1, show.x.value = FALSE), NA)
})

#plot_freq_010
test_that("plot_freq handles ticks.max parameter", {
  x <- 1:50
  expect_error(plot_freq(x ~ 1, ticks.max = 5), NA)
})

#plot_freq_011
test_that("plot_freq handles show.legend parameter", {
  x <- c(1, 1, 2, 2, 2)
  group <- c("A", "A", "B", "B", "A")
  
  # With legend
  expect_error(plot_freq(x ~ group, show.legend = TRUE), NA)
  
  # Without legend
  expect_error(plot_freq(x ~ group, show.legend = FALSE), NA)
})

#plot_freq_012
test_that("plot_freq returns correct structure", {
  x <- c(1, 1, 2, 2, 2, 5, 5)
  
  result <- plot_freq(x ~ 1)
  
  # Check structure
  expect_true(is.data.frame(result))
  
  # Should have at least value and frequency columns
  expect_true(ncol(result) >= 2)
  expect_true(nrow(result) > 0)
})

#plot_freq_013
test_that("plot_freq handles missing values", {
  x <- c(1, 1, 2, NA, 2, 5, 5)
  
  # Should handle NAs gracefully
  expect_error(plot_freq(x ~ 1), NA)
  
  result <- plot_freq(x ~ 1)
  expect_true(is.data.frame(result))
})

#plot_freq_014
test_that("plot_freq error message shows correct dataset name", {
  # Create a dataset with a specific name
  IV5 <- data.frame(value = c(1, 1, 2, 2, 2, 5, 5))
  
  # Try to access a variable that doesn't exist - should show "IV5" not "data"
  expect_error(
    plot_freq(nonexistent ~ 1, data = IV5),
    'Variable "nonexistent" not found in dataset "IV5"',
    fixed = TRUE
  )
})

#plot_freq_015
test_that("plot_freq handles order parameter for groups", {
  df <- data.frame(
    value = c(1, 1, 2, 2, 2, 5, 5, 1, 5),
    group = c("A", "A", "A", "B", "B", "A", "B", "B", "A")
  )
  
  # Custom group order
  expect_error(plot_freq(value ~ group, data = df, order = c("B", "A")), NA)
  result <- plot_freq(value ~ group, data = df, order = c("B", "A"))
  expect_true(is.data.frame(result))
  
  # Check that columns are in specified order
  expect_equal(names(result), c("value", "B", "A"))
})

#plot_freq_016
test_that("plot_freq respects factor levels for groups when order is NULL", {
  # Create factor with specific level order
  df <- data.frame(
    value = c(1, 1, 2, 2, 2, 5, 5, 1, 5),
    group = factor(c("A", "A", "A", "B", "B", "A", "B", "B", "A"),
                   levels = c("B", "A"))
  )
  
  expect_error(plot_freq(value ~ group, data = df), NA)
  result <- plot_freq(value ~ group, data = df)
  expect_true(is.data.frame(result))
  
  # Check that factor levels are respected (B before A)
  expect_equal(names(result), c("value", "B", "A"))
})

#plot_freq_017
test_that("plot_freq order parameter validates group names", {
  df <- data.frame(
    value = c(1, 1, 2, 2, 2, 5, 5),
    group = c("A", "A", "A", "B", "B", "A", "B")
  )
  
  # Missing a group in order
  expect_error(
    plot_freq(value ~ group, data = df, order = c("A")),
    "order.*missing group"
  )
  
  # Extra group in order (should warn, not error)
  expect_warning(
    plot_freq(value ~ group, data = df, order = c("A", "B", "C")),
    "order.*contains group.*not in data"
  )
})

#plot_freq_018
test_that("plot_freq order works with 3 groups", {
  df <- data.frame(
    value = rep(c(1, 2, 5), 6),
    group = rep(c("A", "B", "C"), each = 6)
  )
  
  # Custom order for 3 groups
  expect_error(plot_freq(value ~ group, data = df, order = c("C", "A", "B")), NA)
  result <- plot_freq(value ~ group, data = df, order = c("C", "A", "B"))
  expect_true(is.data.frame(result))
  
  # Check column order
  expect_equal(names(result), c("value", "C", "A", "B"))
})

#plot_freq_019
test_that("plot_freq default group order is sorted", {
  df <- data.frame(
    value = c(1, 1, 2, 2, 2, 5, 5, 1, 5),
    group = c("C", "C", "C", "A", "A", "C", "A", "A", "C")
  )
  
  # Without order parameter, groups should be sorted (A before C)
  result <- plot_freq(value ~ group, data = df)
  expect_true(is.data.frame(result))
  expect_equal(names(result), c("value", "A", "C"))
})

#plot_freq_020
test_that("plot_freq order = -1 reverses default order", {
  df <- data.frame(
    value = c(1, 1, 2, 2, 2, 5, 5, 1, 5),
    group = c("A", "A", "A", "B", "B", "A", "B", "B", "A")
  )
  
  # Default order is A, B (sorted)
  result_default <- plot_freq(value ~ group, data = df)
  expect_equal(names(result_default), c("value", "A", "B"))
  
  # order = -1 should reverse to B, A
  result_reversed <- plot_freq(value ~ group, data = df, order = -1)
  expect_true(is.data.frame(result_reversed))
  expect_equal(names(result_reversed), c("value", "B", "A"))
})

#plot_freq_021
test_that("plot_freq order = -1 reverses factor levels", {
  df <- data.frame(
    value = c(1, 1, 2, 2, 2, 5, 5, 1, 5),
    group = factor(c("A", "A", "A", "B", "B", "A", "B", "B", "A"),
                   levels = c("A", "B"))
  )
  
  # Default respects factor levels: A, B
  result_default <- plot_freq(value ~ group, data = df)
  expect_equal(names(result_default), c("value", "A", "B"))
  
  # order = -1 should reverse factor levels to B, A
  result_reversed <- plot_freq(value ~ group, data = df, order = -1)
  expect_true(is.data.frame(result_reversed))
  expect_equal(names(result_reversed), c("value", "B", "A"))
})

#plot_freq_022
test_that("plot_freq order = -1 works with 3 groups", {
  df <- data.frame(
    value = rep(c(1, 2, 5), 6),
    group = rep(c("A", "B", "C"), each = 6)
  )
  
  # Default order is A, B, C (sorted)
  result_default <- plot_freq(value ~ group, data = df)
  expect_equal(names(result_default), c("value", "A", "B", "C"))
  
  # order = -1 should reverse to C, B, A
  result_reversed <- plot_freq(value ~ group, data = df, order = -1)
  expect_true(is.data.frame(result_reversed))
  expect_equal(names(result_reversed), c("value", "C", "B", "A"))
})

#plot_freq_023
test_that("plot_freq handles two-vector comparison", {
  y1 <- c(1, 1, 2, 2, 2, 5, 5)
  y2 <- c(1, 2, 2, 3, 3, 3)
  
  # Should not throw errors
  expect_error(plot_freq(y1, y2), NA)
  
  result <- plot_freq(y1, y2)
  expect_true(is.data.frame(result))
  
  # Should have columns for value and both vector names
  expect_equal(ncol(result), 3)
  expect_true("value" %in% names(result))
})

#plot_freq_024
test_that("plot_freq two-vector comparison returns correct structure", {
  y1 <- c(1, 1, 2, 2, 2)
  y2 <- c(2, 3, 3, 3)
  
  result <- plot_freq(y1, y2)
  
  # Check structure
  expect_true(is.data.frame(result))
  expect_equal(ncol(result), 3)
  expect_true("value" %in% names(result))
  expect_true("y1" %in% names(result))
  expect_true("y2" %in% names(result))
})

#plot_freq_025
test_that("plot_freq two-vector comparison with custom parameters", {
  y1 <- c(1, 1, 2, 2, 2)
  y2 <- c(2, 3, 3, 3)
  
  # Custom colors
  expect_error(plot_freq(y1, y2, col = c("red", "blue")), NA)
  
  # Percentages
  expect_error(plot_freq(y1, y2, freq = FALSE), NA)
  
  # Without legend
  expect_error(plot_freq(y1, y2, show.legend = FALSE), NA)
})

#plot_freq_026
test_that("plot_freq two-vector comparison handles order parameter", {
  y1 <- c(1, 1, 2, 2, 2)
  y2 <- c(2, 3, 3, 3)
  
  # Default order (alphabetical)
  result_default <- plot_freq(y1, y2)
  expect_equal(names(result_default), c("value", "y1", "y2"))
  
  # Reverse order
  result_reversed <- plot_freq(y1, y2, order = -1)
  expect_equal(names(result_reversed), c("value", "y2", "y1"))
  
  # Custom order
  result_custom <- plot_freq(y1, y2, order = c("y2", "y1"))
  expect_equal(names(result_custom), c("value", "y2", "y1"))
})

#plot_freq_027
test_that("plot_freq two-vector comparison handles missing values", {
  y1 <- c(1, 1, 2, NA, 2)
  y2 <- c(2, NA, 3, 3)
  
  # Should handle NAs gracefully
  expect_error(plot_freq(y1, y2), NA)
  
  result <- plot_freq(y1, y2)
  expect_true(is.data.frame(result))
})

#plot_freq_028
test_that("plot_freq two-vector comparison validates inputs", {
  y1 <- c(1, 1, 2, 2, 2)
  
  # Non-existent variable name as second argument
  expect_error(
    plot_freq(y1, "not_a_vector"),
    "Could not find variable"
  )
  
  # Non-existent variable name as first argument
  expect_error(
    plot_freq("not_a_vector", y1),
    "Could not find variable"
  )
})

#plot_freq_029
test_that("plot_freq two-vector uses deduced variable names", {
  y1 <- c(1, 1, 2, 2, 2)
  y2 <- c(2, 3, 3, 3)
  
  result <- plot_freq(y1, y2)
  expect_true(is.data.frame(result))
  expect_true("y1" %in% names(result))
  expect_true("y2" %in% names(result))
})

#plot_freq_030
test_that("plot_freq two-vector can resolve columns from data", {
  set.seed(1)
  y1 <- sample(100, replace = TRUE)
  y2 <- sample(100, replace = TRUE)
  df1 <- data.frame(a = y1, b = y2)
  
  expect_error(plot_freq(a, b, data = df1), NA)
})

#plot_freq_031
test_that("plot_freq two-vector order parameter works", {
  y1 <- c(1, 1, 2, 2, 2)
  y2 <- c(2, 3, 3, 3)
  
  # Reversed order
  result <- plot_freq(y1, y2, order = -1)
  expect_equal(names(result), c("value", "y2", "y1"))
  
  # Explicit order
  result2 <- plot_freq(y1, y2, order = c("y2", "y1"))
  expect_equal(names(result2), c("value", "y2", "y1"))
})

#plot_freq_032
test_that("plot_freq two-vector handles continuous data", {
  # Test with continuous data (runif, rnorm) which have floating point values
  set.seed(123)
  y1 <- runif(50)
  y2 <- rnorm(50)
  
  # Should not throw errors with continuous data
  expect_error(plot_freq(y1, y2), NA)
  
  result <- plot_freq(y1, y2)
  expect_true(is.data.frame(result))
})

#plot_freq_033
test_that("plot_freq xlim includes all bars", {
  # Test that bars at edges are not clipped
  y1 <- c(1, 1, 2, 2, 3)
  y2 <- c(1, 4, 4, 4)
  
  # Should include padding so bars at min (1) and max (4) are visible
  expect_error(plot_freq(y1, y2), NA)
  
  # Test with single variable at edges
  x <- c(1, 1, 5, 5)
  expect_error(plot_freq(x), NA)
})

#plot_freq_034
test_that("plot_freq uses pretty x-ticks for many unique values", {
  # With >30 unique values, plot_freq should still run (and not attempt to label every x)
    x <- 1:40
    expect_error(plot_freq(x, ylim = c(0, 10)), NA)
    
  # Grouped path also should run
    df <- data.frame(value = 1:40, group = rep(c("A", "B"), each = 20))
    expect_error(plot_freq(value ~ group, data = df, ylim = c(0, 10)), NA)
})

#plot_freq_035
test_that("plot_freq reserves space for legend", {
  y1 <- c(1, 1, 2, 2, 3)
  y2 <- c(1, 4, 4, 4)
  
  # With legend (default)
  expect_error(plot_freq(y1, y2, show.legend = TRUE), NA)
  
  # Without legend
  expect_error(plot_freq(y1, y2, show.legend = FALSE), NA)
})

#plot_freq_036
test_that("plot_freq two-vector supports col and freq args", {
  y1 <- c(1, 1, 2, 2, 2)
  y2 <- c(2, 3, 3, 3)
  
  # col argument works
  expect_error(plot_freq(y1, y2, col = c("blue", "pink")), NA)
  
  # Percentages work
  result_pct <- plot_freq(y1, y2, freq = FALSE)
  expect_true(is.data.frame(result_pct))
})

#' Enhanced alternative to message()
#' 
#' Add options to set color and to end execution of code (to be used as error message)
#'
#' @param ... Message content to be printed. Multiple arguments are pasted together.
#' @param col text color. Default is "cyan". 
#' @param font Integer. 1 for plain text (default), 2 for bold text.
#' @param stop Logical. If TRUE, stops execution (like \code{stop()}) but without printing "Error:".
#'
#' @details
#' This function prints colored messages to the console. If ANSI color codes are supported
#' by the terminal, the message will be colored. Otherwise, it will be printed as plain text.
#' If \code{stop = TRUE}, execution will be halted after printing the message.
#'
#' @examples
#' message2("This is a plain cyan message", col = "cyan", font = 1)
#' message2("This is a bold cyan message", col = "cyan", font = 2)
#' message2("This is a bold red message", col = "red", font = 2)
#' \donttest{
#' cat("this will be shown")
#' try(message2("This stops execution", stop = TRUE), silent = TRUE)
#' cat("this will be shown after the try")
#' }
#'
#' @return No return value, called for side effects. Prints a colored message 
#'   to the console. If \code{stop = TRUE}, execution is halted after printing 
#'   the message.
#'
#' @importFrom grDevices col2rgb
#' @export
message2 <- function(..., col = "cyan", font = 1, stop = FALSE) {
  
  # Check if ANSI is supported
  supportsANSI <- isTRUE(getOption("crayon.enabled", default = TRUE)) && 
                  (Sys.getenv("TERM") != "" || .Platform$OS.type == "windows")
  
  # Check for 256-color support
  supports256 <- FALSE
  if (requireNamespace("crayon", quietly = TRUE)) {
    supportsANSI <- crayon::has_color()
    # Check if terminal supports 256 colors
    # Most modern terminals support 256 colors if they support colors at all
    if (supportsANSI) {
      term <- Sys.getenv("TERM", "")
      # Most xterm-compatible terminals support 256 colors
      supports256 <- term != "" && (
        grepl("256", term) || 
        grepl("xterm", term, ignore.case = TRUE) ||
        grepl("screen", term, ignore.case = TRUE) ||
        grepl("tmux", term, ignore.case = TRUE) ||
        .Platform$OS.type == "windows"  # Windows Terminal supports 256 colors
      )
    }
  } else if (supportsANSI) {
    # Basic check without crayon
    term <- Sys.getenv("TERM", "")
    supports256 <- term != "" && (
      grepl("256", term) || 
      grepl("xterm", term, ignore.case = TRUE) ||
      .Platform$OS.type == "windows"
    )
  }
  
  # Helper function to convert RGB to 256-color ANSI code
  rgb_to_256color <- function(rgb_vals) {
    r <- rgb_vals[1]
    g <- rgb_vals[2]
    b <- rgb_vals[3]
    
    # If all components are equal and close, use grayscale (232-255)
    if (abs(r - g) < 3 && abs(g - b) < 3 && abs(r - b) < 3) {
      gray_val <- round((r + g + b) / 3)
      # Map to grayscale range 232-255
      gray_index <- round((gray_val / 255) * 23) + 232
      return(paste0("38;5;", gray_index))
    }
    
    # Otherwise, use 6x6x6 color cube (16-231)
    # Each component is quantized to 6 levels: 0, 95, 135, 175, 215, 255
    r_level <- round((r / 255) * 5)
    g_level <- round((g / 255) * 5)
    b_level <- round((b / 255) * 5)
    
    # Clamp to valid range
    r_level <- max(0, min(5, r_level))
    g_level <- max(0, min(5, g_level))
    b_level <- max(0, min(5, b_level))
    
    # Calculate index: 16 + 36*r + 6*g + b
    color_index <- 16 + 36 * r_level + 6 * g_level + b_level
    return(paste0("38;5;", color_index))
  }
  
  # Map basic ANSI color names to ANSI codes (for exact matches)
  basic_ansi_colors <- list(
    black = "30",
    red = "31",
    green = "32",
    yellow = "33",
    blue = "34",
    magenta = "35",
    cyan = "36",
    white = "37",
    gray = "90",
    grey = "90"
  )
  
  # RGB values for ANSI colors (standard and bright variants)
  # Standard colors (30-37)
  ansi_rgb_standard <- list(
    black = c(0, 0, 0),
    red = c(187, 0, 0),      # Dark red
    green = c(0, 187, 0),    # Dark green
    yellow = c(187, 187, 0), # Dark yellow
    blue = c(0, 0, 187),     # Dark blue
    magenta = c(187, 0, 187), # Dark magenta
    cyan = c(0, 187, 187),   # Dark cyan
    white = c(187, 187, 187) # Dark white/gray
  )
  
  # Bright colors (90-97)
  ansi_rgb_bright <- list(
    black = c(85, 85, 85),   # Dark gray
    red = c(255, 85, 85),     # Bright red
    green = c(85, 255, 85),   # Bright green
    yellow = c(255, 255, 85), # Bright yellow
    blue = c(85, 85, 255),    # Bright blue
    magenta = c(255, 85, 255), # Bright magenta
    cyan = c(85, 255, 255),   # Bright cyan
    white = c(255, 255, 255)  # White
  )
  
  # ANSI codes for standard and bright colors
  ansi_codes_standard <- c("30", "31", "32", "33", "34", "35", "36", "37")
  ansi_codes_bright <- c("90", "91", "92", "93", "94", "95", "96", "97")
  ansi_color_names <- names(ansi_rgb_standard)
  
  color_lower <- tolower(col)
  color_code <- NULL
  use_256color <- FALSE
  
  # Try exact match with basic ANSI colors first
  if (color_lower %in% names(basic_ansi_colors)) {
    color_code <- basic_ansi_colors[[color_lower]]
  } else {
    # Try to convert any R color name to RGB
    tryCatch({
      # Convert color name to RGB (handles all R color names)
      rgb_vals <- col2rgb(col)[, 1]
      
      # If 256-color mode is supported, use it for better color accuracy
      if (supports256) {
        color_code <- rgb_to_256color(rgb_vals)
        use_256color <- TRUE
      } else {
        # Fall back to 16-color mode: find closest ANSI color
        min_dist <- Inf
        closest_code <- "36"  # Default to cyan
        
        # Check standard colors
        for (i in seq_along(ansi_color_names)) {
          ansi_rgb <- ansi_rgb_standard[[i]]
          # Euclidean distance in RGB space
          dist <- sqrt(sum((rgb_vals - ansi_rgb)^2))
          if (dist < min_dist) {
            min_dist <- dist
            closest_code <- ansi_codes_standard[i]
          }
        }
        
        # Check bright colors
        for (i in seq_along(ansi_color_names)) {
          ansi_rgb <- ansi_rgb_bright[[i]]
          dist <- sqrt(sum((rgb_vals - ansi_rgb)^2))
          if (dist < min_dist) {
            min_dist <- dist
            closest_code <- ansi_codes_bright[i]
          }
        }
        
        color_code <- closest_code
      }
    }, error = function(e) {
      # If col2rgb fails, default to basic ANSI cyan (same as col='cyan')
      color_code <<- "36"
    })
  }
  
  # Fallback to basic ANSI cyan if still null (same as col='cyan')
  if (is.null(color_code)) {
    color_code <- "36"
  }
  
  # Build ANSI escape sequence
  if (supportsANSI) {
    if (use_256color) {
      # 256-color mode: \033[38;5;COLORm (or \033[1;38;5;COLORm for bold)
      if (font == 2) {
        ansi_start <- paste0("\033[1;", color_code, "m")
      } else {
        ansi_start <- paste0("\033[", color_code, "m")
      }
    } else {
      # 16-color mode: \033[COLORm (or \033[1;COLORm for bold)
      if (font == 2) {
        ansi_start <- paste0("\033[1;", color_code, "m")
      } else {
        ansi_start <- paste0("\033[", color_code, "m")
      }
    }
    ansi_end <- "\033[0m"
  } else {
    ansi_start <- ""
    ansi_end <- ""
  }
  
  # Create message
  msg_parts <- list(...)
  msg_text <- paste(msg_parts, collapse = "")
  
  # Combine message parts with ANSI codes if supported
  if (supportsANSI) {
    # Wrap message with ANSI codes, start on new line
    msg <- paste0("\n", ansi_start, msg_text, ansi_end)
  } else {
    # Start on new line even without ANSI
    msg <- paste0("\n", msg_text)
  }
  
  # Create message object and print
  msg_obj <- simpleMessage(msg)
  message(msg_obj)
  
  # Stop execution if requested (without printing "Error:")
  if (stop) {
    opt <- options(show.error.messages = FALSE)
    on.exit(options(opt))
    stop(msg_text)
  }
  
  invisible()
}

#message2_001
test_that("message2 prints messages without errors", {
  # Should not throw errors
  expect_error(message2("Test message"), NA)
  expect_error(message2("Test", "message", "with", "multiple", "parts"), NA)
})


#message2_002
test_that("message2 handles different colors", {
  # Test various color options
  expect_error(message2("Red message", col = "red"), NA)
  expect_error(message2("Blue message", col = "blue"), NA)
  expect_error(message2("Cyan message", col = "cyan"), NA)
  expect_error(message2("Green message", col = "green"), NA)
  expect_error(message2("Custom color", col = "dodgerblue"), NA)
})

#message2_003
test_that("message2 handles font parameter", {
  # Font 1 (plain)
  expect_error(message2("Plain text", font = 1), NA)
  
  # Font 2 (bold)
  expect_error(message2("Bold text", font = 2), NA)
})

#message2_004
test_that("message2 handles stop parameter", {
  # When stop=TRUE, should stop execution
  expect_error(message2("This stops", stop = TRUE), class = "simpleError")
  
  # When stop=FALSE, should not stop
  expect_error(message2("This doesn't stop", stop = FALSE), NA)
})


#message2_005
test_that("message2 handles multiple message parts", {
  # Multiple arguments should be combined
  expect_error(message2("Part", "1", "and", "Part", "2"), NA)
  
  # With numeric values
  expect_error(message2("Value:", 42), NA)
  
  # With mixed types
  expect_error(message2("Count:", 5, "items"), NA)
})

#message2_006
test_that("message2 returns invisibly", {
  result <- message2("Test")
  expect_null(result)
})

#' Resize Images
#'
#' Saves images to PNG with a specified width. As input it accepts (SVG, PDF, EPS, JPG, JPEG, TIF, TIFF, BMP, PNG)
#' Saves to subdirectory '/resized' within input folder (or same directory as file if input is a single file)
#'
#' @param path Character string. Path to a folder containing image files, or path to a single image file.
#' @param width Numeric vector. Target width(s) in pixels for the output PNG files.
#'   Can be a single value (recycled for all files) or a vector matching the number
#'   of files found.
#'
#' @return Invisibly returns TRUE on success.
#'
#' @details
#' This function:
#' \itemize{
#'   \item Searches for image files with extensions: svg, pdf, eps, jpg, jpeg, tif, tiff, bmp, png
#'   \item Creates a "resized" subfolder in the target directory if it doesn't exist
#'   \item Converts each file to PNG format at the specified width(s)
#'   \item Saves output files as: \code{originalname_width.png} in the resized subfolder
#' }
#'
#' Supported input formats:
#' \itemize{
#'   \item Vector graphics: SVG, PDF, EPS (rasterized using rsvg/magick)
#'   \item Raster images: JPG, JPEG, TIF, TIFF, BMP, PNG
#' }
#'
#' @note
#' Dependencies required: \code{rsvg}, \code{magick}, and \code{tools} (base R).
#' SVG files are rasterized using \code{rsvg::rsvg()}, while PDF/EPS and other
#' formats are handled by \code{magick::image_read()}.
#'
#' @examples
#' \donttest{
#' # Create a temporary PNG file and resize it
#' tmp_png <- tempfile(fileext = ".png")
#' grDevices::png(tmp_png, width = 400, height = 300)
#' old_par <- graphics::par(no.readonly = TRUE)
#' graphics::par(mar = c(2, 2, 1, 1))
#' graphics::plot(1:2, 1:2, type = "n")
#' grDevices::dev.off()
#' graphics::par(old_par)
#' resize_images(tmp_png, width = 80)
#' }
#'
#' @export
resize_images <- function(path, width) {

  # Dependencies: rsvg, magick, tools

  

  # Check if input is a file or folder

  if (file.exists(path) && !dir.exists(path)) {

    # Input is a single file

    files <- path

    # Output directory is the same directory as the file

    outdir <- file.path(dirname(path), "resized")

  } else if (dir.exists(path)) {

    # Input is a folder

    # Find files in folder

    files <- list.files(

      path,

      pattern = "\\.(svg|pdf|eps|jpg|jpeg|tif|tiff|bmp|png)$",

      ignore.case  = TRUE,

      full.names   = TRUE

    )

    

    # Output directory is subfolder within input folder

    outdir <- file.path(path, "resized")

  } else {

    stop("Path does not exist or is not a valid file or folder.")

  }

  

  # Ensure output subfolder exists

  if (!dir.exists(outdir)) dir.create(outdir)

  

  if (length(files) == 0) stop("No supported image/figure files found.")

  

  # Recycle width if scalar

  if (length(width) == 1) {

    width <- rep(width, length(files))

  } else if (length(width) != length(files)) {

    # Print error message in red
    message2("Error", col = "red", font = 2)
    message2("Found ", length(files), " image(s), but ", length(width), " width(s) were provided.", col = "red")
    
    # List files with counter
    for (j in seq_along(files)) {
      message2("  ", j, ". ", basename(files[j]), col = "red")
    }
    
    stop("Width vector length must match number of files or be scalar.")

  }

  

  for (i in seq_along(files)) {

    f <- files[i]

    ext <- tolower(tools::file_ext(f))

    

    # Read + rasterize SVG/PDF/EPS

    if (ext == "svg") {

      img <- magick::image_read(rsvg::rsvg(f))

    } else if (ext %in% c("pdf", "eps")) {

      img <- magick::image_read(f)

    } else {

      img <- magick::image_read(f)

    }

    

    # Resize

    resized <- magick::image_resize(img, paste0(width[i], "x"))

    

    # Output file in resized/

    out <- file.path(

      outdir,

      paste0(tools::file_path_sans_ext(basename(f)), "_", width[i], ".png")

    )

    

    magick::image_write(resized, out, format = "png")
    
    # Print progress message in blue
      new_name <- basename(out)
      message2("Resized '", new_name, "'", col = "blue")

  }

  

  invisible(TRUE)

}

#resize_images_001
test_that("resize_images handles non-existent folder gracefully", {
  # Non-existent folder should error or handle gracefully
  expect_error(resize_images("nonexistent_folder_12345", width = 800), 
               class = "error")
})

#resize_images_002
test_that("resize_images handles empty folder", {
  # Create temporary empty folder
  temp_dir <- tempfile()
  dir.create(temp_dir)
  
  # Should handle empty folder (will error because no images found)
  # This test ensures it errors gracefully
  expect_error(resize_images(temp_dir, width = 800), 
               "No supported image")
  
  # Cleanup
  unlink(temp_dir, recursive = TRUE)
})

# Note: Full testing of resize_images requires actual image files
# which is complex in a test environment. These tests cover basic
# error handling and edge cases.

#resize_images_003
test_that("resize_images accepts width parameter", {
  # Test that function signature accepts width
  # Full functionality requires image files
  temp_dir <- tempfile()
  dir.create(temp_dir)
  
  # Should accept single width (will error because no images, but that's expected)
  expect_error(resize_images(temp_dir, width = 800), 
               "No supported image")
  
  # Should accept multiple widths (will error because no images, but that's expected)
  expect_error(resize_images(temp_dir, width = c(800, 1200)), 
               "No supported image")
  
  # Cleanup
  unlink(temp_dir, recursive = TRUE)
})

# Note: Testing actual image conversion would require:
# 1. Creating test image files
# 2. Verifying output files are created
# 3. Checking image dimensions
# This is complex and may require additional dependencies
# The current tests verify basic parameter handling

#' Clear Plot, Global Environment, and Console
#'
#' Clears plot, global environment, and console. On first use the user is
#' prompted to authorize clearing the environment to comply with CRAN rules.
#'
#' @return Invisibly returns NULL. Prints a colored confirmation message.
#'
#' @details
#' This function performs three cleanup operations:
#' \itemize{
#'   \item \strong{Plot}: Closes all open graphics devices (except the null device)
#'   \item \strong{Global environment}: Removes all objects from the global environment
#'   \item \strong{Console}: Clears the console screen (only in interactive sessions)
#' }
#'
#' \code{clear()} will not modify the global environment unless you have
#' previously typed "yes" when prompted. If you do not type "yes", you are asked
#' again next time; only "yes" is remembered for future sessions.
#'
#' \strong{Warning}: This function deletes all objects in the global environment.
#' Save anything that you wish to keep before running.
#'
#' @examples
#' \donttest{
#' # Interactive use: clear workspace, console, and plots
#' # First run may prompt; once you type "yes", your preference is saved.
#' clear()
#' }
#'
#' @export
clear <- function() {
  pref <- clear_allow_global_preference()
  if (!isTRUE(pref)) {
    if (interactive()) {
      msg <- "To allow clear() to clear your environment, console, and plot, type \"yes\"\n(you will not be asked again if you say \"yes\").\n"
      message2(msg, col = "red2")
      ans <- trimws(tolower(readline(prompt = "")))
      if (!identical(ans, "yes")) {
        message2("You did not write 'yes', so clear() will not work.", col = "red2")
        return(invisible(NULL))  # End without clearing; will prompt again next time
      }
      clear_save_allow_global_preference(TRUE)
    } else {
      warning("clear() will not modify the global environment. Set the preference interactively first.")
      return(invisible(NULL))
    }
  }

  # Clear environment
  rm(list = ls(envir = .GlobalEnv), envir = .GlobalEnv)
  
  # Clear console (only in interactive sessions)
  if (interactive()) {
    cat("\014")
  }
  
  # Clear all plots (close all devices except null device)
  # Close devices one at a time, checking each iteration
  dev_list <- dev.list()
  if (!is.null(dev_list) && length(dev_list) > 0) {
    for (dev_num in dev_list) {
      if (dev_num != 1) {  # Don't close the null device
        tryCatch({
          dev.off(which = dev_num)
        }, error = function(e) {
          # Device may have been closed already, ignore error
        })
      }
    }
  }
  
  # Print confirmation message
  
  message2("statuser::clear()", font=2,col = "red2")
  message2("Cleared plot, global environment, and console",col = "red2")
  
  invisible(NULL)
}

# --- Persistent preference for clearing .GlobalEnv (internal) ---

clear_config_dir <- function() {
  tryCatch(
    tools::R_user_dir("statuser", "config"),
    error = function(e) file.path(Sys.getenv("HOME", "~"), ".config", "R", "statuser")
  )
}

clear_allow_global_preference <- function() {
  dir <- clear_config_dir()
  path <- file.path(dir, "clear_allow_global")
  if (!file.exists(path)) return(NA)
  out <- trimws(readLines(path, n = 1L, warn = FALSE))
  if (identical(out, "TRUE")) return(TRUE)
  if (identical(out, "FALSE")) return(FALSE)
  NA
}

clear_save_allow_global_preference <- function(value) {
  dir <- clear_config_dir()
  if (!dir.exists(dir)) dir.create(dir, recursive = TRUE, showWarnings = FALSE)
  path <- file.path(dir, "clear_allow_global")
  writeLines(as.character(value), path)
}

# Internal helpers for stimulus.plot() and stimulus.beeswarm()
# (Vendored from the `stimulus` GitHub package, adapted for statuser.)

utils::globalVariables(c("m1", "m2", "m3", "m1.cluster"))


# ---- from utils.r ----

#0 Stimulus cache (package state)
  sp_stimulus_cache <- function() {
    if (is.null(.statuser_state$stimulus_cache)) {
      .statuser_state$stimulus_cache <- list()
    }
    .statuser_state$stimulus_cache
  }




#Functions

#1 round2()        - Rounding
#2 exit()          - message to show when executoin ends early
#3 clean_string()  - Remove everything except letters, numbers, and underscores
#4 tidy.t()        - t.test outputs to data.frame
#5 formatted.p () - turn p-value into a formatted string (e.g. p=.023 or p<.001)
#6 get.ci()       - Get confidence interval from model
#7 add_named_if_exists() 
#8 namedList()    - automatically name objects in listAutomatically name elements in list with name of the objects in the list
#9 format_percent() - format number as a percent
#10 auto.decimals() - Number of decimals determined automatically
#11 get.md5()       - Compute the MD5 hash of a dataframe
#12 does.cache.d.exist() - Does cache exist
#13 eval2()         - Evaluate string as formula
#14 eval.arguments()
#15 message2()     - show message on screen with 'groundhog' color (not red, blue or black, which are not usually friendly texts)
#16 round_smart()  - automatically set number of digits
#17 Format format_msg() - turns a long character string into a fixed withd with possible header and pre text.
#18 get.counter.interval() - set how many interactions in counter before output is shown 
 


#----------------------------------

#1 Rounding
  round2 <- function(x, digits = 2) { 
    ncode <- paste0("%.", digits, "f")
    sub("^(-?)0.", "\\1.", sprintf(ncode, x))
  }


  
#2 gstop (uses statuser exit)
    gstop <- function(msg, format = FALSE) {
    if (format == TRUE) msg <- format_msg(msg)
    exit(paste0(msg, "\n----------------------------------------"))
    }
    
    
  
#3 Clean string
   clean_string <- function(input_string) {
      #Remove everything except letters, numbers, and underscores
      #cleaned_string <- gsub("[^A-Za-z0-9._]", "", input_string)
       cleaned_string <- gsub("\"", "", input_string)

  return(cleaned_string)
   }
   
   
   
#4 tidy.t
   
  tidy_t = function(t)
  {
  row=data.frame(
                 m1     = t$estimate[1],
                 m2     = t$estimate[2],
                 effect = t$estimate[1]-t$estimate[2],
                 t      = t$statistic,
                 df     = t$parameter,
                 p      = t$p.value,
                 ciL    = t$conf.int[1],
                 ciH    = t$conf.int[2], 
                 row.names = NULL)    
  return(row)    
    
  }
  

#5 formatted p-value
    formatted.p <- function(p) {
      format_single_p <- function(single_p) {
        p.clean <- round(single_p, 3)           # Round it
        if (p.clean==0) return ("p<.0001")
        p.clean <- substr(p.clean, 2, 6)        # Drop the 0
        p.clean <- paste0("=", p.clean)
        if (single_p < .0001) p.clean <- "<.0001"
        if (single_p > .9999) p.clean <- ">.9999"
        p.clean <- paste0("p", p.clean)
        return(p.clean)
      }
  
  # Apply the function to each element of the input vector
  result <- sapply(p, format_single_p)
  return(result)
}


  
#6 Confidence interval from model
     get.ci=function(m)
      {
      coe=summary(m)$coefficients 
      b=coe[2,1]                   #point estimate
      se=coe[2,2]                  #SE
      tc = qt(.975,df=coe[2,3])    #look-up t-distribution for 95% CI for those d.f.
      ci=c(b-tc*se , b+tc*se)
      return(ci)
     }
     
     
#7 Add to list if object exists
   add_named_if_exists <- function(obj_name, lst) {
  if (exists(obj_name, envir = parent.frame())) {
    obj_value <- get(obj_name, envir = parent.frame())
    lst[[obj_name]] <- obj_value
  }
  return(lst)
   }
   
   
#8 Automatically name elements in list with name of the objects in the list
    #https://stackoverflow.com/questions/16951080/can-lists-be-created-that-name-themselves-based-on-input-object-names
    namedList <- function(...) {
      L <- list(...)
      snm <- sapply(substitute(list(...)),deparse)[-1]
      if (is.null(nm <- names(L))) nm <- snm
      if (any(nonames <- nm=="")) nm[nonames] <- snm[nonames]
      setNames(L,nm)
    }
    
#9 Format as percent
format_percent <- function(x,decimals) {
  if (decimals=='auto') decimals <- auto.decimals(x * 100)
  formatted_number <- sapply(1:length(x), function(i) {
    paste0(formatC(x[i] * 100, format = "f", digits = decimals), "%")
  })
  return(formatted_number)
}
    
    
#10 Number of decimals
    auto.decimals <- function(x) {
      
      sapply(x, function(num) {
      num=abs(num)
      if (is.na(num)) {
        return(NA)
      } else if (num > 100) {
        return(0)
      } else if (num > 10) {
        return(1)
      } else if (num > 0.1) {
        return(2)
      } else if (num > 0.01) {
        return(3)
      } else {
        return(4)
      }
    })
  }

    
#11 # Function to compute the MD5 hash of a dataframe
  get.md5 <- function(df) {
    df_serialized <- serialize(df, NULL)
    md5_hash <- digest::digest(df_serialized, algo = "md5")
    return(md5_hash)
  }
  
#12 Does cache exist
  does.cache.d.exist = function(md5k)
  {
    cache <- sp_stimulus_cache()
    key_exists <- md5k %in% names(cache)
    return(key_exists)
  }
  
#13 eval2
 eval2 <- function(s)  eval(parse(text=s),  parent.frame())  #Added "parent.frame() because otherwise eval2() will not call the right objects within another function: Fix was found here; https://www.rdocumentation.org/packages/base/versions/3.5.1/topics/eval


#14 eval.arguments
 eval.arguments = function(model_string, dv, condition, stimulus, participant,dataname)
   {
     model_string <- gsub("dv", dv, model_string)
     model_string <- gsub("condition", condition, model_string)
     model_string <- gsub("stimulus", stimulus, model_string)
     model_string <- gsub(",data=df2", '', model_string)
     #model_string <- gsub("df2", dataname, model_string,fixed=TRUE)
     model_string <- gsub("participant", participant, model_string)
    return(model_string)
 }
 
#17 round_smart: dynamic number of digits
  round_smart <- function(x) {
    abs_x <- abs(x)  # Use the absolute value
  
    if (abs_x >= 0.01) {
      # Always show 2 decimals
      return(formatC(x, format = "f", digits = 2))
    } else if (abs_x >= 0.00001) {
      # Show the first non-zero decimal
      non_zero_digits <- sub("0\\.", "", sub(".*?([1-9]+.*)", "\\1", formatC(abs_x, format = "f", digits = 5)))
      return(formatC(x, format = "f", digits = nchar(non_zero_digits)))
    } else {
      # For numbers smaller than 0.00001
      return("<0.00001")
    }
  }
  
  
#18 Format msg
format_msg <- function(msg,width=70, header='IMPORTANT.', pre="| ")
    {
    #Line counter
    j<-0
    #Lines with formatted message starts empty
      msg.lines=c()
    #Turn message into vector of words
      msg.left <- strsplit(msg,' ')[[1]]

    #Loop over lines
      while (length(msg.left)>0)
      {
     j=j+1
     msg.lines[j]=''

    #loop over words
      while (nchar(msg.lines[j]) + nchar(msg.left[1]) <width)
      {
      new.word <- msg.left[1]
      msg.left <- msg.left[-1]
      if (regexpr('\n', new.word)>0) break   #skip line if \n is found
      msg.lines[j] <- paste0(msg.lines[j],new.word," ")   #add the next word
      
      if (length(msg.left)==0) break
    }
      msg.lines[j]<- paste0(pre,"    ", msg.lines[j] ) 
      if (length(msg.left)==0) break
    }
      
  #formatted 
    #Add |  
      msg.lines <- gsub("\n", "\n|", msg.lines)
      
      
    #Join al
      msg.formatted <- paste0(msg.lines,collapse="\n")
      
    #Add header
      msg.formatted <- paste0(pre,header,"\n",msg.formatted)
      
    #Add ------------- on top
      sep.line <- c(paste0(rep('-',width+5)) , "\n" )
      msg.formatted<-c(sep.line, msg.formatted)
    
    return(msg.formatted)
}



#18 Set how often feedback of interactions is shown 
  get.counter.interval = function(seconds)
  {
  # Predefined set of possible intervals
      intervals <- c(1,5, 10,20,25,50,seq(100,1000,100))
    
  # Calculate approximate interval to achieve roughly a 10-second print frequency
    target_time <- 3 #we want to show feedback every 3 seconds
    approx_interval <- target_time / seconds
    
  # Find the closest interval from the predefined set
    return(intervals[which.min(abs(intervals - approx_interval))])
    
  }
  
  
  

# ---- from check1.R ----

#Function used to validate inputs to functions in this package, checks type and length of variable


#1) Auxiliary - Integer?    
    is.integer2 = function(x) all(floor(x)==x)

    
    #Color
    valid_color <- function(col1) {
        # Try converting the color to RGB; if it fails, it's not a valid color
        tryCatch({
          col2rgb(col1)
          TRUE  # If no error, the color is valid
        }, error = function(e) {
          message("Error: '", col1, "' is not a valid color.")
          FALSE  # If error, the color is invalid
        })
      }
            

#2) check1(): function that evaluates type and length of a given argument in the function
      check1 = function(f,var, type.check,  nu.check = -1,args_passed)
      {
        
        #varname
          varname <- deparse(substitute(var))
          
        #If was not assigned a value 
          if (!varname %in% args_passed) return(TRUE)
        
        
      
        #Unique.values
          nu=length(unique(var))
          if (nu != nu.check & nu.check!= -1) {
              exit(paste0(f,"() says: the argument '",varname,"' must have '",nu.check,"' unique values, but it has '",nu,"'")) 
          }
            
      
        #Type integer
          if (type.check=='integer')
            {
            if (is.integer2(var)==FALSE) {
              exit(paste0(f,"() says: the argument '",varname,"' must be an integer, but '",var, "' isn't."))
            }
              
              }
        
        #Type character
          if (type.check=='character')
            {
            if (is.character(var)==FALSE) exit(paste0(f,"() says: the argument '",varname,"' must be a character variable but '",var, "' isn't."))
          }
        
        #Type numeric
          if (type.check=='numeric')
            {
            if (is.numeric(var)==FALSE) exit(paste0(f,"() says: the argument '",varname,"' must be a numeric, but '",var, "' isn't."))
          }
        #Type loical
          if (type.check=='logical')
          {
            
            if (is.logical(var)==FALSE)  exit(paste0(f,"() says: the argument '",varname,"' must be either TRUE or FALSE, but '",var, "' is neither."))
          }
          
        #Type color
          if (type.check=='color')
          {
            is.color <- valid_color(var) 
            if (!is.color) exit(paste0(f,"() says: the argument '",varname,"' must be a valid color but '",var, "' isn't recognized as a color."))
            
            }
            
          
      }
        
        
      
      

# ---- from validate.stimulus.plot.R ----

validate.stimulus.plot=function(plot.type, data,dv, condition, stimulus, save.as,  svg.width, svg.height,  sort.by,  flip.conditions,model, 
                                  overall.estimate, overall.ci,overall.p, overall.label,ylab1, ylab2, xlab1, xlab2,
                                  decimals, null.method,dv.is.percentage, legend.title,simtot, watermark, seed, ylim,args_passed)
    {
  
  #1 Function to identify source of error
      f = 'statuser::stimulus.plot'
      
  #2 Check all arguments are of the appropriate length and type; see file check1.R
      check1(f, plot.type, 'character', 1,args_passed)
      check1(f, save.as, 'character', 1,args_passed)
      check1(f, svg.width, 'numeric', 1,args_passed)
      check1(f, svg.height, 'numeric', 1,args_passed)
      check1(f, sort.by, 'charcter', 1,args_passed)
      check1(f, flip.conditions, 'logical', 1,args_passed)
      check1(f, overall.estimate, 'numeric', 1,args_passed)
      check1(f, overall.ci, 'numeric', -1,args_passed)       #Checked below, and it need not be equal to 2.    
      check1(f, overall.p, 'numeric', 1,args_passed)
      check1(f, overall.label, 'character', 1,args_passed)
      check1(f, ylab1, 'character', 1,args_passed)
      check1(f, ylab2, 'character', 1,args_passed)
      check1(f, xlab1, 'character', 1,args_passed)
      check1(f, xlab2, 'character', 1,args_passed)
      check1(f, decimals, 'numeric', 1,args_passed)
      check1(f, null.method, 'character', 1,args_passed)
      check1(f, dv.is.percentage, 'logical', 1,args_passed)
      check1(f, legend.title, 'character', 1,args_passed)
      check1(f, simtot, 'integer', 1,args_passed)
      check1(f, watermark, 'logical', 1,args_passed)
      check1(f, seed, 'numeric', 1,args_passed)
      check1(f, ylim, 'numeric', 2,args_passed)
      
      
  #3 Check with limited set of values
      if (!plot.type %in% c('means','effects'))     exit(paste0(f,"() says: the argument 'plot.type' must be either 'effects' or 'means', you entered '",plot.type,"'"))
      if (!null.method %in% c('shuffle', 'demean', 'demeans')) exit(paste0(f,"() says: the argument 'null.method' must be either 'shuffle' or 'demean', you entered '",null.method,"'"))
      if (any(!model %in% c('all','regression','intercepts','slopes'))) {
          exit(paste0(f,"() says:If the the argument 'model' is set, it must include only a subset of the following four values:\n ",
             "'all','regression','intercepts','slopes' "))
      }
      

  #4 Custome checks
      check.save.as(save.as) #function 5 here

  #5 No stimulus plot for compared design
      t = table(data[,stimulus],data[,condition])
      matched = mean(t[,1]*t[,2]>0) > .5  #check that at least half the stimulus ids in both conditions
      
      if (matched==FALSE) {
              exit(format_msg(paste0(
              f,"() says: The stimulus variable ('", stimulus,"') does not have the same values ",
              "across conditions. If you have a compared-stimulus design, with different ",
              "and unmatched stimuli across condition, use stimulus.beeswarm(). If you do ",
              "have a treated- or matched-stimulus design, then check that you have a matching ",
              "stimulus identifier for the pairs of stimuli across conditions."),header='Cannot do Stimulus Plot for compared-stimulus designs'))
      
      }
      
  #6 overall
      n1=length(overall.estimate)
      n2=length(overall.ci)
      n3=length(overall.p)
      n4=length(overall.label)

      if (length(unique(c(n1,n3,n4)))>1 & n1>0) exit("The 'overall' arguments (estimate, p, and label) must have the same legnth")
      if (n2!=2*n1) exit(paste0(f,"() says: Make sure that there are twice the number of values in overall.ci as in overall.estimate"))
     

      
      
  }

  
      
      
      

# ---- from validate.data.R ----



#Function - Validate data

  validate.data = function(f, data, dv, condition, stimulus, sort.by,participant,dataname)
  {
        if (missing(data))      exit(paste0(f," says: you must specify a dataframe"))
        if (missing(dv))        exit(paste0(f," says: you must specify the dependent variable ('dv')"))
        if (missing(condition)) exit(paste0(f," says: you must specify the condition variable ('condition')"))
        if (missing(stimulus))  exit(paste0(f," says: you must specify the stimulus variable ('stimulus')"))
    
    
      n1=names(data)
      if (!dv %in% n1)        exit(paste0(f,"() says the dv ('",dv,    "') is not in the dataset '",dataname,"'."))
      if (!condition %in% n1) exit(paste0(f,"() says the condition variable ('",condition,"') is not in the dataset '",dataname,"'."))
      if (!stimulus %in% n1)  exit(paste0(f,"() says the stimulus variable ('",stimulus,"') is not in the dataset '",dataname,"'."))
      if (!sort.by %in% c(n1,"")    ) exit(paste0(f," says the sort.by variable ('",sort.by,"') is not in the dataset '",dataname,"'."))
      if (!participant %in% c(n1,"")) exit(paste0(f," says the participant variable ('",participant,"') is not in the dataset '",dataname,"'."))
  }
  
  

# ---- from validate.dots.R ----

  validate.dots=function(f,...)
  {
    #Get the arguments  
      dot_args <- list(...)
      #dot_args <- paste0('"', dot_args, '"')

      
    # Get the list of valid arguments for plot
      valid_plot_args <- names(formals(graphics::plot.default))
  
    # Check if all names in ... are valid plot arguments
       invalid_args <- setdiff(names(dot_args), valid_plot_args)
  
  # If there are any invalid arguments, throw an error
  if (length(invalid_args) > 0) {
    
    invalid_args <- paste0('"', invalid_args, '"')
    
    exit(paste0("These arguments are neither part of ", f, "(), nor Base R plot():\n",
        paste(invalid_args, collapse = ", ")))
  }
  }

# ---- from validate.R ----


  
#FUnction 3 - validate dots

  
  
#Function 4 - Validate beeswarm
    validate.beeswarm=function(data,  dv, stimulus, condition, 
                        flip.conditions, 
                        dv.is.percentage,
                        simtot,
                        confidence,
                        ylim,
                        ylab1,
                        ylab2,
                        xlab1,
                        xlab2,
                        dot.spacing,
                        col1,
                        col2,
                        main,
                        watermark,
                        save.as,
                        svg.width,
                        svg.height,
                        args_passed)
    {
      f <- 'statuser::stimulus.beeswarm'
      check1(f, flip.conditions, 'logical', 1, args_passed)
      check1(f, dv.is.percentage, 'logical', 1, args_passed)
      check1(f, simtot, 'integer', 1, args_passed)
      check1(f, ylab1, 'character', 1, args_passed)
      check1(f, ylab2, 'character', 1, args_passed)
      check1(f, xlab1, 'character', 1, args_passed)
      check1(f, watermark, 'logical', 1, args_passed)
      check1(f, confidence, 'numeric', 1, args_passed)
      if (svg.height != '') check1(f, svg.height, 'numeric', 1, args_passed)
      if (svg.width != '') check1(f, svg.width, 'numeric', 1, args_passed)
      check.save.as(save.as)
      check.confidence(confidence)
    }
    
    
  check.save.as=function(save.as)
  {
    if (save.as!='')  
    {
           call_stack <- sys.calls()
  
        # Check the second last entry in the stack, which is the calling function
          if (length(call_stack) >= 2) {
          caller <- as.character(call_stack[[length(call_stack) - 2]][[1]])
          }
     extension= tools::file_ext(save.as)  
     
     if (!extension %in% c('svg','png')) exit(paste0(caller,"() says: 'save.as' must have extension .svg or .png"))
    }
  }
  
  check.confidence=function(confidence)
  {
     # Get the call stack
          call_stack <- sys.calls()
  
        # Check the second last entry in the stack, which is the calling function
          if (length(call_stack) >= 2) {
          caller <- as.character(call_stack[[length(call_stack) - 2]][[1]])
          }
    
    if (confidence<5.1 | confidence>=99.9) exit(paste0(caller,"() says: 'confidence' must be between 5.1 and 99.9; for 95% confidence use 95, not .95"))
  
    
  }
    
    

# ---- from get.means.by.condition.R ----


  

  get.means.condition <- function(data, dv, stimulus, condition,sort.by,flip.conditions) {
      

    #  Process stimulus and condition values
        #Stimulus
          stimulus.all=unique(data[,stimulus])
          
        #Condition
          ucond=sort(unique(data[,condition]))
          if (flip.conditions==FALSE) data[,condition]=factor(data[,condition], levels=ucond)
          if (flip.conditions==TRUE)  data[,condition]=factor(data[,condition], levels=rev(ucond))
          
    
         
    #2 Compute means and CI via t-test
        
      # Split data by `stimulus`
          split_data <- split(data, data[,stimulus])
          
          # Apply t-test for each split, then extract p-value and statistic
          t.all_list <- lapply(split_data, function(sub_data) {
            test <- t.test(sub_data[,dv] ~sub_data[,condition])
            tk=tidy_t(test)
            tk
          })
          
        # Combine the results into a dataframe
          t.all <- do.call(rbind, t.all_list)
          t.all <- data.frame(stimulus = rownames(t.all), t.all, row.names = NULL)
          
              
      # Rename the means columns
        names(t.all)[2:3] <- c(paste0(condition,"_",ucond[1]), paste0(condition,"_",ucond[2]))
        names(t.all)[1] <-stimulus 
      # Sort rows
          #Default: effect size
            if (sort.by=='') {
              t.all <- t.all[order(t.all$effect), ]
            }
        
          #Else, by sort.by
            if (sort.by!='') 
            {
              
              #Is the sort.by value unique to each stimulus (e.g., alphabetical order)
                t = table(data[,stimulus],data[,sort.by])
                
                #If there are just as many cells with frequencies >0 as there are stimuli, then it is unique
                  item.unique = FALSE
                  if (sum(t!=0) == length(unique(data[,stimulus]))) item.unique = TRUE
        
        
              #If unique
                if (item.unique==TRUE)
                {
                  #Dataframe with unique values of sort.by for each stimulus
                    sort.by.data <- unique(data[,c(stimulus,sort.by)])

                }
                
              #If not unique it's numeric, so we compute the mean
                if (item.unique==FALSE)
                  {
                  #Compute mean by item
                    sort.by.data <- aggregate(data[, sort.by],list(data[, stimulus]), mean)
                    names(sort.by.data)=c(stimulus,sort.by) 
                } #End if sort.by is not unique to each stimulus
                  #Merge with sort.by
                    t.all = merge(t.all, sort.by.data,by=stimulus)

             
                  
                
        #Sort it
            t.all <- t.all[order(t.all[,sort.by]), ]

        
        } #End if sort.by is not null
                  
              

              
       
                
      return(t.all)
  }
  
 

# ---- from get.null.shuffle.R ----



 get.null.shuffle = function(data,  dv, stimulus, condition, participant,simtot=100,flip.conditions,obs)
  {
    
    #Residualize stimulus effects
      m0 = lm(data[,dv]~factor(data[,stimulus]))
      data$r=residuals(m0)
      
      
    #Compute means on residuals
      message2("Will conduct ",simtot," resamples to estimate expected heterogeneity under null of homogeneity.")
      
      means.all=matrix(nrow=simtot,ncol=length(unique(data[,stimulus])))
      for (k in 1:simtot)
      {
        if (k==1) t1=Sys.time()
        
      #Shuffle item within condition
        data[,paste0(stimulus,"_shuffled")]  <- ave(data[,stimulus], data[,condition], FUN = function(x) sample(x))
      
      #Get means of residualized dv on shuffled stimuli
        tk   = get.means.condition(data=data,dv='r',stimulus=paste0(stimulus,"_shuffled"),condition=condition,sort.by='',flip.conditions = flip.conditions)
            
      #Extract estimates
        means.all[k,] = sort(tk$effect)
        
      #Counter
        if (k==1) {
          t2=Sys.time()
          seconds <- as.numeric(difftime(t2, t1, units = "secs"))
          counter.interval= get.counter.interval(seconds)
          }
        
        if (k%%counter.interval==0) cat('...',k)
    }
        cat("\n")
      
  #Compute the full set of sorted effect sizes
   dM=colMeans(means.all) 
   dL=apply(means.all,2,quantile,.025) 
   dH=apply(means.all,2,quantile,.975) 
  
  #Full resamples saved
   under.null.resamples=data.frame(means.all)
   names(under.null.resamples)   =paste0('stimulus_',1:ncol(under.null.resamples))
   rownames(under.null.resamples)=paste0('resample_',1:nrow(under.null.resamples))
   
  #Compute heterogeneity p-value
    d.obs  = obs$effect
    e2.obs = sum((dM-d.obs)^2)
    
    #Resampled
      e2.rows <- (sweep(under.null.resamples, 2, dM, FUN = "-"))^2
      e2.resamples <- rowSums(e2.rows)
    
    #p-value
      p.hetero = mean(e2.resamples >= e2.obs)
      p.hetero_text = formatted.p(p.hetero)
     if (p.hetero == 0) {
        rounded_value <- 1 / simtot
        
        # Check if rounded_value is less than 0.0001
        if (rounded_value < 0.0001) {
          p.hetero_text <- 'p<.0001'
        } else {
          p.hetero_text <- paste0('p<', format(rounded_value, digits = 1, scientific = FALSE))
        }
      }

      

  #Output
    list(under.null.summary   = data.frame(low=dL, mean=dM, high=dH),
         e2.obs=e2.obs, e2.resamples=e2.resamples,
         p.hetero=p.hetero,
         p.hetero_text=p.hetero_text,
         under.null.resamples = under.null.resamples )

  }
    
 

# ---- from get.null.demean.R ----

#Generates the expected distribution of effects by forcing the null without assuming homoskedsticity
#Only the mean is adjusted to force the null, allowing each stimulus to have its own variance\






 get.null.demean = function(data,  dv, stimulus, condition, participant,simtot=100,flip.conditions,obs)
  {
    #1 Unique conditions and participant ids
      uc=sort(unique(data[,condition]))
      ui=unique(data[,participant])
      nui=length(ui)
    
    #2 Make null true, all means the same
      #Observed means
        means.obs = get.means.condition(data,dv,stimulus,condition,sort.by='',FALSE)

      #Means for condition 1 & 2 for each stimulus
        m1.obs=means.obs[,1]
        m2.obs=means.obs[,2]
        
      #Overall means for conditions 1 & 2
        m1.all=mean(m1.obs)
        m2.all=mean(m2.obs)
      
      #Compute gap between condition mean and overall mean for each stimulus*condition
        gap1=m1.obs-m1.all
        gap2=m2.obs-m2.all
      
      #Turn gaps to dataframe that is merged with the data to have the gaps in the full data
        df.gaps = data.frame(means.obs[,stimulus],gap1,gap2)
        names(df.gaps)[1]=stimulus
        data=merge(data, df.gaps, by=stimulus)
        
      #Generate the null
        data[,'dv.null'] = ifelse(data[,condition]==uc[1], data[,dv]-data[,'gap1'], data[,dv]-data[,'gap2'])
        
      #Sampling error by randomly drawing participants
        means.all=matrix(nrow=simtot,ncol=length(unique(data[,stimulus])))
        for (k in 1:simtot)
       {
         
        #Generate data
            ids=sample(ui,replace=TRUE)
            length(unique(ids))
            
            data.null.list=list()
            j=1
            for (idk in ids)
            {
              data.null.list[[j]]=data[data[,participant]==idk,]
              j=j+1
            }
            data.boot <- do.call(rbind, data.null.list)
  
        #Get null.means
          mk   = get.means.condition(data=data.boot,dv='dv.null',stimulus=stimulus,condition=condition,sort.by='',flip.conditions = flip.conditions)
              
        #Extract estimates
          means.all[k,] = sort(mk$effect)
          
        #Counter
          if (k%%50==0) cat('...',k)
        }
      
      
      
  #Compute the full set of sorted effect sizes
   dM=colMeans(means.all) 
   dL=apply(means.all,2,quantile,.025) 
   dH=apply(means.all,2,quantile,.975) 
  
  #Full resamples saved
   under.null.resamples=data.frame(means.all)
   names(under.null.resamples)   =paste0('stimulus_',1:ncol(under.null.resamples))
   rownames(under.null.resamples)=paste0('resample_',1:nrow(under.null.resamples))
   
  #Compute heterogeneity p-value
    d.obs  = means.obs$effect
    e2.obs = sum((dM-d.obs)^2)
    
  #Resampled
      e2.rows <- (sweep(under.null.resamples, 2, dM, FUN = "-"))^2
      e2.resamples <- rowSums(e2.rows)
    
  #p-value
      p.hetero = mean(e2.resamples >= e2.obs)
      p.hetero_text = formatted.p(p.hetero)
      if (p.hetero==0) p.hetero_text = paste0('p<',1/simtot)
      

  #Output
    list(under.null.summary   = data.frame(low=dL, mean=dM, high=dH),
         e2.obs=e2.obs, e2.resamples=e2.resamples,
         p.hetero=p.hetero,
         p.hetero_text=p.hetero_text,
         under.null.resamples = under.null.resamples )

  }
    
 

# ---- from get.model.results.R ----

  #1 Make variable structure compatible with lmer which cannot do df[,dv]
  #2 Shared parameters
  #3 Regression
  #4 Stimulus intercepts
  #5 Stimulus slopes
  #6 Results



 get.model.results=function(df, dataname, dv, stimulus, condition, participant,model,flip.conditions)
{
#-------------------------------------------------------
  
           
  
          
  #1 Make variable structure compatible with lmer which cannot do df[,dv]
       df2=df
       df2$dv=df[,dv]
       df2$stimulus   = df[,stimulus]
       df2$condition  = df[,condition]
       if (participant !='') df2$participant  = df[,participant]

       ucond=sort(unique(df2$condition))
       if (flip.conditions==TRUE)    df2$condition=factor(df2$condition, levels=ucond)
       if (flip.conditions==FALSE)  df2$condition=factor(df2$condition, levels=rev(ucond))
      

#-------------------------------------------------------

  #2 Shared parameters
      crossed = FALSE
      if (participant!='')
          {  
            t=table(df2$participant,df2$condition)
            crossed <- sum(t[,1]*t[,2]>0)>0
      }
       
      #Stimulus
          stimulus.all=unique(df[,stimulus])
 
#-------------------------------------------------------

  #3 Regression
     if (any(c('all', 'regression') %in% model)) {
          message2("stimulus.plot() says:")
               
      #Fixed effects for participant only if they get different conditions
         #Set up teh regression
            if (crossed==FALSE)  m1.text = "m1=lm(dv~condition+factor(stimulus),data=df2)"
            if (crossed==TRUE)   m1.text = "m1=lm(dv~condition+factor(stimulus)+factor(participant),data=df2)"
            
            
          #Show feedback on screen
            m1.text.formatted= eval.arguments (m1.text, dv, condition, stimulus, participant, dataname)
            message(" ")
            message2("Estimating regression:\n    ",m1.text.formatted)
            
        #Evaluate the regression
            eval2(m1.text)

        #Get mean effect for condition and its  ci
            m1.mean = summary(m1)$coefficients[2,1]
            se =summary(m1)$coefficients[2,2]
            deg.free = m1$df.residual
            tc = qt(.975,df=deg.free)
            m1.p    = summary(m1)$coefficients[2,4]
            m1.ci = c(m1.mean - tc*se, m1.mean+tc*se)

       
        #Cluster by participant if needed
            if (participant!='') {
              m1.cluster.text = "lmtest::coeftest(m1,vcov=sandwich::vcovCL,type='HC3',cluster=~participant)"
          
             #Show feedback on screen
                m1.cluster.text.formatted = eval.arguments(m1.cluster.text, dv, condition, stimulus, participant, dataname)
                message(" ")
                message2("  Clustering the standard errors:\n    ",m1.cluster.text.formatted)
           
             #Estimate SE
                m1.cluster = try(eval2(m1.cluster.text))
                
                  if (class(m1.cluster) %in% 'try-error')
                    {
                    #Change to hc1
                      m1.cluster.text = "lmtest::coeftest(m1,vcov=sandwich::vcovCL,type='HC1',cluster=~participant)"

                    #Show feedback on scren
                      message2("Couldn't estimate HC3, will try HC1")
                      m1.cluster.text.formatted = eval.arguments(m1.cluster.text, dv, condition, stimulus, participant, dataname)
                      message(" ")
                      message2("  Clustering the standard errors:\n    ",m1.cluster.text.formatted)
                      
                    #Estimate new one
                      m1.cluster = eval2(m1.cluster.text)

                    }
                
                se = m1.cluster[2,2]
                m1.ci = c(m1.mean - tc*se, m1.mean+tc*se)
                m1.p = m1.cluster[2,4]
              
            } #End if clustering
            
        #Save 
            m.mean = m1.mean
            m.ci   = m1.ci  
            m.labels='Regression'
            m.p = m1.p
       } #End if regression 
    
   
#-------------------------------------------------------
     
  #4 Stimulus intercepts
     if (any(c('intercepts','all') %in% model)) 
     {


          #Run random model
           #Set up text
            if (participant!='') m2.text = "lmerTest::lmer(dv~condition+(1|stimulus)+(1|participant),data=df2)"
            if (participant=='') m2.text = "lmerTest::lmer(dv~condition+(1|stimulus),data=df2)"
            
          #Show feedback on screen
            m2.text.formatted= eval.arguments (m2.text, dv, condition, stimulus, participant, dataname)
            message(" ")
            message2("Estimating random intercepts model:\n    ",m2.text.formatted)
            m2=eval2(m2.text)

          #Get mean effect for condition and its  ci
            m2.mean = summary(m2)$coefficients[2,1]
            m2.ci  =  get.ci(m2)  #see utils.r function #6
            m2.p =  summary(m2)$coefficients[2,5]
            
          #Add
            lab='Random Intercepts'
            m.mean <- if (exists("m.mean")) c(m.mean, m2.mean)   else m2.mean
            m.ci   <- if (exists("m.ci"))   c(m.ci  , m2.ci)     else m2.ci
            m.labels <- if (exists("m.labels")) c(m.labels,lab)  else lab
            m.p       <- if (exists("m.p"))     c(m.p ,m2.p) else m2.p

              
    } #End if intercepts

#-------------------------------------------------------
      
#5 Stimulus slopes
     if (any(c('slopes','all') %in% model)) {

          #Run random model
            if (participant!='') m3.text ="lmerTest::lmer(dv~condition+(1+condition|stimulus)+(1|participant),data=df2)"
            if (participant=='') m3.text = "lmerTest::lmer(dv~condition+(1+condition|stimulus),data=df2)"
            message(" ")
            m3.text.formatted= eval.arguments (m3.text, dv, condition, stimulus, participant, dataname)
            message2("Estimating random slopes model:\n    ",m3.text.formatted)
            m3=eval2(m3.text)

          #Get mean effect for condition and its  ci
            m3.mean = summary(m3)$coefficients[2,1]
            m3.ci  =  get.ci(m3)  #see utils.r function #6
            m3.p =  summary(m3)$coefficients[2,5]

           #Add
            lab='Random Slopes'
            m.mean   <- if (exists("m.mean"))   c(m.mean, m3.mean) else m3.mean
            m.ci     <- if (exists("m.ci"))     c(m.ci  , m3.ci)   else m3.ci
            m.labels <- if (exists("m.labels")) c(m.labels,lab)    else lab
            m.p      <- if (exists("m.p"))     c(m.p , m3.p)      else m3.p

      } #End if slopes   
      
#-------------------------------------------------------
      
#6 Results

    #Summary results for plotting
      results=namedList(m.mean, m.ci, m.labels, m.p)
      
    #Full models
      if (exists('m1'))         results$regression = m1
      if (exists('m1.cluster')) results$regression.clustered_errors = m1.cluster
      if (exists('m2'))         results$random_intercepts = m2
      if (exists('m3'))         results$random_slopes = m3
  
       return(results)
}
      

# ---- from get.maxmin.confidence.R ----



#Does resampling under null of all stimuli having the same distribution, 
#see  stimulus.beeswarm.R 

    get.maxmin.confidence = function(data,  dv, stimulus, condition, simtot=500,confidence=95,ms1,ms2,dc1,dc2)
    {
        
        #how many stimuli
          n1=nrow(ms1)
          n2=nrow(ms2)
          
        #Empty matrices
          ms1.boot=matrix(nrow=simtot,ncol=n1)
          ms2.boot=matrix(nrow=simtot,ncol=n2)
        
          
        #Bootstrap itself under null of equal distributions
          datak=data
          message2("Will run ",simtot," resamples to compute confidence band")
          for (bk in 1:simtot)
          {
          #Shuffle the stimulus
            datak[,stimulus]=unsplit(lapply(split(datak[,stimulus], data[,condition]), sample), datak[,condition])

            ms1.boot[bk,]=sort(aggregate(datak[dc1,dv],list(datak[dc1,stimulus]),mean)$x)
            ms2.boot[bk,]=sort(aggregate(datak[dc2,dv],list(datak[dc2,stimulus]),mean)$x)
            
            if (bk %% 100==0) cat("...",bk)
          }
        
        #Set quantiles for confidence level required
            qL = ((100-confidence)/2)/100
            qH = 1-qL
            
        #Compute quantiles 
            b1L = quantile(ms1.boot[,1],qL)
            b1H = quantile(ms1.boot[,n1],qH)
            b2L = quantile(ms2.boot[,1],qL)
            b2H = quantile(ms2.boot[,n2],qH)
          
        #Output
            output=namedList(b1L,b1H,b2L,b2H)
            return(output)
      
    }

# ---- from stimulus.plot.means.R ----

 stimulus.plot.means = function(data, dv, condition, stimulus, 
                    participant, sort.by,
                    flip.conditions,
                    ylab1, ylab2,  xlab1, xlab2, 
                    decimals, 
                    dv.is.percentage,
                    legend.title, col1,col2,ylim,main,...)
    
      {

    #Grab the arguments passed on to ...  
     args = list(...)
      
     
    #Hard code the colors
      col1='black'
      col2='red4'
      
    
      
    #0 Is it a matched design?
          t = table(data[,stimulus],data[,condition])
          matched = FALSE
          if (mean(t[,1]*t[,2]>0) >.5 ) matched=TRUE
          
              #This computes the frequency of stimuli id by condition
              #if it is compared, a given ID appears only in one condition
              #if it is matched, it appears in both
              #this classifies a design as matched if 50%+
              #of stimuli appear in both conditions
             

    #1 Get the means by condition
          means.obs = get.means.condition(data=data,dv=dv,stimulus=stimulus,flip.conditions=flip.conditions, condition=condition,sort.by=sort.by)

   #2 local names
      n=nrow(means.obs)/2

      
      #Condition
       ucond=sort(unique(data[,condition]))
       cond1= paste0(condition,"_",ucond[1])
       cond2= paste0(condition,"_",ucond[2])
      
      
        y1 = means.obs[,cond1] #Condition 1
        y2 = means.obs[,cond2] #Condition 2
        label1  =  sub(paste0(condition,"_"), "", cond1)
        label2  =  sub(paste0(condition,"_"), "", cond2)
      
      
      #Which value is higher for each stimulus
        bh=pmax(y1 , y2)
        bl=pmin(y1 , y2)  
      


      #Overall means
        m1 = mean(y1)
        m2 = mean(y2)

      
    #3 ylim: range of y values in the plot
      if (length(ylim)<2)
      {
        ylim = range(c(y1,y2))
        dy = diff(ylim)
        ylim[2]=ylim[2]+.4*dy  #Give a 25% buffer on top (for the legend)
        ylim[1]=ylim[1]-.03*dy  #give a 3% buffer below, for the value labels
      }
    #4 Margins
      #Get current margins
        mar.before =  par("mar")
        mar.after  =  mar.before
         

      #Only change margin if not the default (so users can set own in)
        custom_mar <- getOption("graphics.par")$mar   #see if user has set different margins by default
        if (is.null(custom_mar))  mar.default = c(5.1, 4.1, 4.1, 2.1)
        if (!is.null(custom_mar)) mar.default = custom_mar
        
        
        max.x.label = max(nchar(unique(data[,stimulus])))
        xlabel.buffer = max(0,max.x.label)*.3
         
        if (all(mar.before==mar.default))
        {
        #4.1 Bottom
             mar.after[1] = mar.before[1] + xlabel.buffer
        
        #4.2 Top
          #Drop top margin if there is no main header
            mar.after[3] = ifelse (main=='',1,2)
        
        #4.3 Left
           width.y.label = nchar(max(pretty(y1)))
           mar.after[2] = max(width.y.label/3.5, 4)
           if (ylab2!='') mar.after[2]= mar.after[2] + 1
           if (dv.is.percentage==TRUE) {
             
             mar.after[2] = mar.after[2] + 1
             
           }
          
        #4.4 Assign it
           par(mar=mar.after)
           
        } 
           
  #5 black dots
       n=length(y1)
       if (dv.is.percentage==FALSE) plot(y1,pch=16,ylim=ylim,          xaxt='n',xlab='',las=1,ylab='',xlim=c(.5-.015*n,n+4 + n*.0125),cex=1.5, xaxs='i',...)
       if (dv.is.percentage==TRUE)  plot(y1,pch=16,ylim=ylim, yaxt='n',xaxt='n',xlab='',las=1,ylab='',xlim=c(.5-.015*n,n+4 + n*.0125),cex=1.5, xaxs='i',...)
       
  #6 Segments
    e=mean(means.obs$effect)    
    lty=ifelse( (y1 - y2)*(m1-m2)>0 ,1,2)
    col12=ifelse( (y1 - y2)*(m1-m2)>0 ,col1, col2)
    segments(x0=1:n, x1=1:n,y0=y1, y1=y2,lty=lty,col=col12)
    
  #7 White dots   
      points(y2,pch=21,col='black',bg='white',cex=1.5)
 
  #8 Redo black dots to cover any red lines
    points(y1,pch=16,cex=1.5)
    
  #9 Value labels
    
 
    #Color for text
      col.h = ifelse(bh==y1,adjustcolor(col1,.5),adjustcolor(col2,.91))
      col.l = ifelse(bl==y2,adjustcolor(col2,.5),adjustcolor(col1,.91))
      col.h=col.l='black'

     
    #Labels themselves
      if (dv.is.percentage==FALSE)
      {
      text(1:n,bh,round2(bh,auto.decimals(bh)),col=col.h,cex=.75,pos=3)
      text(1:n,bl,round2(bl,auto.decimals(bl)),col=col.l,cex=.75,pos=1)
      }
      
      if (dv.is.percentage==TRUE)
      {
      text(1:n,bh,format_percent(bh,decimals),col=col.h,cex=.75,pos=3)
      text(1:n,bl,format_percent(bl,decimals),col=col.l,cex=.75,pos=1)
      }
   
  #10 Overall means
    segments(x0=n+3, x1=n+3,y0=m1, y1=m2)
    
    points(n+3,m1,pch=16,cex=1.5*1.5) 
    points(n+3,m2,pch=21,cex=1.5*1.5,col='black',bg='white')
    
    axis(side=1,at=n+3,"MEAN",font=2)
    
    
    d.pos=c(3,1)
    if (m1<m2) d.pos=rev(d.pos)
    if (dv.is.percentage==FALSE)
      {
      text(n+3,m1 , round2(m1,auto.decimals(m1)),cex=.8,col='black',pos=d.pos[1])
      text(n+3,m2 , round2(m2,auto.decimals(m1)),cex=.8,col='black',pos=d.pos[2])
      }
    
    
      if (dv.is.percentage==TRUE)
      {
      text(n+3,m1 , format_percent(m1,decimals),cex=.8,col='black',pos=d.pos[1])
      text(n+3,m2 , format_percent(m2,decimals),cex=.8,col='black',pos=d.pos[2])
      }


  #11 Y axis
      if (ylab1=='') ylab1=dv
        
    
      if (!"yaxt" %in% names(args))
      {
      mtext(side=2,line=mar.after[2]-1.5,font=2,cex=1.2,ylab1)
      mtext(side=2,line=mar.after[2]-2.5,font=3,cex=1,ylab2)
      }

    
      if (dv.is.percentage==TRUE)
      {
          ys=pretty(c(y1,y2))
          if (dv.is.percentage==TRUE) axis(side=2,at=ys,paste0(ys*100,"%"),las=1)
      }
    
  #12 X-axis
    
    #12.1 Stimuli labels
        text(1:n,par('usr')[3] , paste0(means.obs[,stimulus],"  "),srt=80,xpd=TRUE,adj=1,col=col12)
       
    #12.2 Headers
        if (xlab2=="" & sort.by=='') xlab2='(sorted by effect size)'
        if (xlab2=="" & sort.by!='') xlab2=paste0('(sorted by ',sort.by,')')

            #For matched stimuli, the default is the above text
        
        mtext(side=1,line=2.7 + xlabel.buffer , font=2,cex=1.2,xlab1)
        mtext(side=1,line=3.7 + xlabel.buffer   ,font=3,cex=1,xlab2)

        
  #13 Legend
        labels=c(label1,label2)
        if (flip.conditions) labels=rev(labels)
        #leg1 = legend('topleft',pch=c(16,1), labels ,inset=.05,bty='n',cex=1.3, y.intersp = 1.5)
        
        #Legend title?
        if (legend.title=='') leg1 = legend('topleft',pch=c(16,1), labels ,inset=.02,bty='n',cex=1.3, y.intersp = 1)
        if (legend.title!='') leg1 = legend('topleft',pch=c(16,1), labels ,inset=.02,bty='n',cex=1.3, title.cex = 1.3,  x.intersp = 0.5, y.intersp = 1,title=legend.title,title.font=2, text.width = strwidth("W"))
        
  #14 Main header
    if (main!=''){
      mtext(side=3,line=.5, font=2,cex=1.5,main)
    }
      
        
        
  #14 Return margins to where they were
    par(mar=mar.before)
    
    return(means.obs)
  
  }
  

# ---- from stimulus.plot.effects.R ----

 stimulus.plot.effects=function(data,  dv, stimulus, condition, participant ,
                                dataname,
                                overall.estimate,
                                overall.ci,
                                overall.label, 
                                overall.p,
                                model,
                                sort.by, 
                                flip.conditions, 
                                null.method='shuffle',
                                decimals, 
                                dv.is.percentage,
                                ylab1,ylab2,xlab1,xlab2,
                                simtot,
                                ylim,
                                seed, 
                                main,
                                ...)
    {
    

         
         
  #------------------------------------
    #0 colors
      col.null1   = 'dodgerblue'
      col.null2   =  adjustcolor(col.null1,.1)
      col.ci      = 'gray50'
      col.overall = 'purple'
    
    #1 Grab the arguments passed on to ...  
      args = list(...)
      args_to_drop=c(decimals, dv.is.percentage,ylab1,ylab2,xlab1,xlab2)
      args <- args[!(names(args) %in% args_to_drop)]

    #2 Compute means by stimulus
       obs = get.means.condition(data=data,dv=dv,stimulus=stimulus,condition=condition,sort.by=sort.by,flip.conditions=flip.conditions)

      #Localize stimulus variables
        d = obs$effect
        ciL = obs$ciL
        ciH = obs$ciH
        n = length(d)
        label2  =  sub(paste0("^",condition,"_"), "", names(obs[3]))
        label1  =  sub(paste0("^",condition,"_"), "", names(obs[2]))
       
        if (flip.conditions)
        {
          l1 = label1
          l2 = label2
          label1 = l2
          label2 = l1
          }
        
    #3 Get the null distribution  (only if sort.by is not set)
          d0=rep(d,length(unique(data[,stimulus])))   #make it equal to d just to help with code below, e.g., ylim=range(...)
          dnull=data.frame(low=d0,high=d0,mean=d0)
        
        #Resampling if sort.by is not specified
            if (sort.by=='') 
                  {
              
                  #get new call's md5
                     #Arguments
                        mc <- match.call(expand.dots = FALSE)
                        args <- as.list(mc)[-1]
                        args <- lapply(args, eval, envir = parent.frame())
                        md5.args = get.md5(args)
                    
                     #dataframe
                      md5.data   = get.md5(data)  
                      
                    #Combine for single md5 
                      md5s=paste0(md5.args ,  md5.data)

                    
                  #if dnull for that md5s has been saved, load it
                    if (does.cache.d.exist(md5s)) 
                    {
                      list_resamples = sp_stimulus_cache()[[md5s]]
                      
                      #If it was not called by itself when re-runnig for showing on the screen
                      
                        if (sys.parent() < 4) {
                              #NOTE: we don't want to show this message when the user sets save.as='fig1.svg'
                              #and we run the function a second time, so here we check whether the call is nested 
                              #which occurs if is already stack 4, meaning the call has already gone through 4 opeartions
                            
                                message2("*Recycled results*:\n",
                               "You had run this same analysis before with all the same variables and options.\n",
                               "We are re-using stored results from the previous call.\n",
                               "To force new calculations clear your cache running: 'clear_stimulus_cache()'")
                                }
                      
                  #else run it                  
                    } else  {
                      if (null.method=='shuffle')
                        {
                        set.seed(seed)
                        list_resamples = get.null.shuffle (data=data, dv=dv, stimulus=stimulus, condition=condition, participant=participant,simtot=simtot,flip.conditions=flip.conditions,obs=obs)
                        }
                        
                      if (null.method=='demean')
                        {
                        set.seed(seed)
                        list_resamples = get.null.demean (data=data, dv=dv, stimulus=stimulus, condition=condition, participant=participant,simtot=simtot,flip.conditions=flip.conditions,obs=obs)
                        }
                   
                  #Save
                      cache <- sp_stimulus_cache()
                      cache[[md5s]] <- list_resamples
                      .statuser_state$stimulus_cache <- cache
                    
                }

                  
                      
            } #End if sort.by!=''
          
          
        #Extract the dnull object with summary
          dnull =  list_resamples$under.null.summary

        #Extract p-hetero
          p.hetero_text = list_resamples$p.hetero_text
          
          
    #4 ylim: range of y values in the plot
      
      if (length(ylim)<2)
      {
      ylim = range(c(ciL,ciH,dnull))
      dy = diff(ylim)
      
      ylim[2]=ylim[2]+.35*dy  #Give a 28% buffer on top (for the legend)
      ylim[1]=ylim[1]-.03*dy  #give a 3% buffer below, for the value labels
      }
          
    #5 get models if specified
       if (length(model)>0)
       {
        model.results = get.model.results(data, dataname, dv, stimulus, condition, participant,model,flip.conditions)

        overall.estimate  = model.results$m.mean
        overall.ci        = model.results$m.ci
        if (length(overall.label)==0) overall.label= model.results$m.labels
        overall.p         = model.results$m.p
  
       }

    #5 xlim 
      n1 = length(overall.estimate)
      xmax = ifelse(n1 > 0, length(d) + n1 +2, length(d)+1)
      xlim = c(0,xmax)
      
 
    #6 Margins
      #Get current margins
        mar.before =  par("mar")
        mar.after  =  mar.before
         
      #Only change margin if not the default (so users can set own)
        custom_mar <- getOption("graphics.par")$mar   #see if user has set different margins by default
        if (is.null(custom_mar))  mar.default = c(5.1, 4.1, 2, 2.1)
        if (!is.null(custom_mar)) mar.default = custom_mar
        
        
          #Get current margins
            mar.before =  par("mar")
            mar.after  =  mar.before
            

          #Label calculations for bottom margin 
            max.length = max(nchar(unique(data[,stimulus])))
            xlabel.buffer = max(0,max.length-3)*.3

          #Bottom
                max.x.label = max(nchar(unique(data[,stimulus])))
                xlabel.buffer = max(0,max.x.label)*.3
                mar.after[1] = mar.before[1] + xlabel.buffer
          
          #Top
            #Drop top margin if there is no main header
              mar.after[3] = ifelse (main=="",1,2)

          #Left
               width.y.label = nchar(max(d))
               mar.after[2] = max(width.y.label/3, 5.1)
               if (ylab2!='') mar.after[2]= mar.after[2] + 1
              
          #Implement
             par(mar=mar.after)
        
    
  #6 Black dots
     if (dv.is.percentage==FALSE)  {
        plot(d,         pch=16,ylim=ylim,xaxt='n',xlab='',las=1,ylab='', cex=1.5, xlim=xlim, xaxs='i', ...)
     }
        
      
     if (dv.is.percentage==TRUE )  {
       plot(d,yaxt='n', pch=16,ylim=ylim,xaxt='n',xlab='',las=1,ylab='', cex=1.5, xlim=xlim, xaxs='i', ...)
      }


      #horizontal line
        abline(h=0,lty=3,col='gray66')
    
   
  #7 Plot the null and its CI
    if (sort.by=='')
    {
    points(dnull$mean,type='l',col=col.null1,lty=2)
    polygon(x=c(1:n,n:1),y=c(dnull$low , rev(dnull$high)),col=col.null2,border=NA)
    }
  
  #8 Value labels

    #set position   
      y.text=d 
      
   #print them
    
      #How many decimals to show?
        if (decimals=='auto') d.decimals=auto.decimals(d)
        if (decimals!='auto') d.decimals=decimals
      
      if (dv.is.percentage==FALSE) text(1:n,d ,round2(d , d.decimals),col='blue4',cex=.65,pos=4)
      if (dv.is.percentage==TRUE)  text(1:n,d ,format_percent(d,decimals), col='blue4',cex=.65,pos=4) #utils.R #9 <-- 'format_percent()'

     
  #7 CI
      arrows(x0=1:n, x1=1:n, y0=ciL,y1=ciH,col=adjustcolor('gray60',.8),code=3,angle=90,length=.02)

      
  #9 Y axis
      if (ylab1=='') ylab1=paste0("Effect on '",dv,"'")

      if (!"yaxt" %in% names(args))
      {
      mtext(side=2,line=mar.after[2]-1.5,font=2,cex=1.2,ylab1)
      mtext(side=2,line=mar.after[2]-2.5,font=3,cex=1,ylab2)
      
      #Y labels for percentages
        ys=pretty(c(ciL,ciH))
        if (dv.is.percentage==TRUE) axis(side=2,at=ys,paste0(ys*100,"%"),las=1)
      }

  
      
    #10 x-axis
      #Skip if xaxt='n' is set
      if (!'xaxt' %in% args)
      {
    
      #10.1 Stimuli labels
        text(1:n,par('usr')[3] , paste0(obs[,stimulus],"  "),srt=80,xpd=TRUE,adj=1)
        

    #14.2 Headers
        if (xlab2=="" & sort.by=='') xlab2='(sorted by effect size)'
        if (xlab2=="" & sort.by!='') xlab2=paste0('(sorted by ',sort.by,')')
        mtext(side=1,line=2.7 + xlabel.buffer , font=2,cex=1.2,xlab1, at=n/2)
        mtext(side=1,line=3.7 + xlabel.buffer   ,font=3,cex=1,xlab2, at=n/2)
      }
      
        
  #15 Legend
        if (sort.by=="")
        {
        
        #Band & p-value
           
        if (null.method=='shuffle') null_text =  'same distribution for all stimuli'
        if (null.method=='demean')  null_text  = 'same means for all stimuli'
        
        leg1 = legend('topleft',
                      bty='n',
                      pch=c(16,NA,NA,NA,NA), 
                      lty=c(NA,1,2,1,NA),
                      lwd=c(NA,1,1,14,NA),
                      col=c('black', col.ci, col.null1 , col.null2,NA),
                      c(paste0("Observed effect: '",label1,"' - '",label2,"'"),
                        "95 CI for observed effect (t-test for this stimulus)",
                        paste0("Expected under null of ",null_text),
                        "95% confidence band under null",
                        paste0("Heterogeneity test: ",p.hetero_text," (N=",simtot, " resamples)")),
                        inset=.03)
        } else {
            leg1 = legend('topleft',
                      bty='n',
                      pch=c(16,NA), 
                      lty=c(NA,1),
                      lwd=c(NA,1),
                      col=c('black',col.ci),
                      c(paste0('Observed effect: ',label1," - ",label2),
                        "95 CI for observed effect"),
                        inset=.03)
          
        }
          
          
  #16 Overall
      if (n1 > 0)
      {
        
        xs=(n+1):(n+n1)+1
        
      
      
      #Markers
        points( x=xs,
                y= overall.estimate,
                pch=16,
                cex=1.5*1.5,
                col=col.overall)
          
      #CI
        arrows(x0=xs,x1=xs, 
               y0=overall.ci[seq(1,n1*2,2)], y1=overall.ci[seq(2,n1*2,2)],
               col=col.overall, code=3, length=.03, angle=90)
        
      #Labels
         overall.label =  gsub("\\n", " \n", overall.label)
         text(xs,par('usr')[3] , paste0(overall.label," ") ,srt=80,xpd=TRUE,adj=1,col=col.overall)


      #p-value
         text(xs,max(overall.ci),pos=3,formatted.p(overall.p),col=col.overall,cex=.7,font=2)

     #"Overall" 
         y.overall=par('usr')[4]- 0.18* (par('usr')[4] - par('usr')[3])
         text(mean(xs),y.overall,pos=3,"Overall",cex=1.2,font=2)

     #Vertical separator
           abline(v= n+1 ,lwd=2) 

     #Overall value label
            if (dv.is.percentage==FALSE)  text(xs,overall.estimate , round(overall.estimate, auto.decimals(overall.estimate)),    cex=.65,col='purple',pos=4)
            if (dv.is.percentage==TRUE)   text(xs,overall.estimate , format_percent(overall.estimate, decimals),                  cex=.65,col='purple',pos=4)      
      }
      
      
      
  #Redo dots
    points(d, pch=16,cex=1.5)
      
  #Main header
    if (main!=''){
      mtext(side=3,line=.5, font=2,cex=1.5,main)
    }
      
    par(mar=mar.before)
    
    
    
    
    
  #Results
    results = list(observed=obs, p.hetero=p.hetero_text, under.null=dnull,resamples = list_resamples$under.null.resamples)
    if (exists('model.results')) results$model.results= model.results
    return(results)     
    
        
    }#End of function
  
 

# Internal helpers for interprobe()
# (Vendored from the deprecated `interacting` package, adapted for statuser.)

# lm/glm store the model frame in $model; estimatr::lm_robust / lm2 does not — use model.frame()
ip_model_frame <- function(model) {
  mf <- tryCatch(stats::model.frame(model), error = function(e) NULL)
  if (!is.null(mf) && is.data.frame(mf)) {
    return(mf)
  }
  m <- model[["model"]]
  if (!is.null(m) && is.data.frame(m)) {
    return(m)
  }
  NULL
}

# Distinguish model = fitted object vs model = "lm"/"linear"/stats::lm (estimate with lm2 inside interprobe).
ip_interprobe_parse_model_arg <- function(model) {
  if (is.null(model)) {
    return(list(model = NULL, estimate_linear = FALSE))
  }
  if (is.function(model) && identical(model, stats::lm)) {
    return(list(model = NULL, estimate_linear = TRUE))
  }
  if (is.character(model) && length(model) == 1L) {
    s <- trimws(model[[1L]])
    if (nzchar(s) && tolower(s) %in% c("lm", "linear")) {
      return(list(model = NULL, estimate_linear = TRUE))
    }
  }
  if (inherits(model, "factor")) {
    s <- trimws(as.character(model)[[1L]])
    if (nzchar(s) && tolower(s) %in% c("lm", "linear")) {
      return(list(model = NULL, estimate_linear = TRUE))
    }
  }
  list(model = model, estimate_linear = FALSE)
}

# Bare model = linear / model = lm: resolve in caller env (fitted object vs lm2 keyword) without forcing promise.
ip_interprobe_resolve_bare_model_symbol <- function(mc, env) {
  default <- list(handled = FALSE, model = NULL, estimate_linear = FALSE)
  if (!"model" %in% names(mc)) {
    return(default)
  }
  ma <- mc[["model"]]
  if (!is.name(ma)) {
    return(default)
  }
  nm <- as.character(ma)
  if (!nm %in% c("linear", "lm")) {
    return(default)
  }
  fit_classes <- c("lm", "glm", "gam", "lm2", "lm_robust")
  if (nm == "linear") {
    if (!exists(nm, envir = env, inherits = TRUE)) {
      return(list(handled = TRUE, model = NULL, estimate_linear = TRUE))
    }
    val <- tryCatch(get(nm, envir = env, inherits = TRUE), error = function(e) NULL)
    if (is.null(val)) {
      return(list(handled = TRUE, model = NULL, estimate_linear = TRUE))
    }
    if (inherits(val, fit_classes)) {
      return(list(handled = TRUE, model = val, estimate_linear = FALSE))
    }
    return(list(handled = TRUE, model = NULL, estimate_linear = TRUE))
  }
  if (!exists(nm, envir = env, inherits = TRUE)) {
    return(list(handled = TRUE, model = NULL, estimate_linear = TRUE))
  }
  val <- tryCatch(get(nm, envir = env, inherits = TRUE), error = function(e) NULL)
  if (is.null(val)) {
    return(list(handled = TRUE, model = NULL, estimate_linear = TRUE))
  }
  if (is.function(val) && identical(val, stats::lm)) {
    return(list(handled = TRUE, model = NULL, estimate_linear = TRUE))
  }
  if (inherits(val, fit_classes)) {
    return(list(handled = TRUE, model = val, estimate_linear = FALSE))
  }
  list(handled = TRUE, model = NULL, estimate_linear = TRUE)
}

ip_adjustcolor2 <- function(col, dark) {
  new_cols <- c()
  for (dj in dark) {
    rgb_val <- grDevices::col2rgb(col)
    new_rgb_val <- rgb_val * dj + (1 - dj) * 255
    new_rgb_val <- pmax(0, pmin(255, new_rgb_val))
    new_col <- grDevices::rgb(new_rgb_val[1], new_rgb_val[2], new_rgb_val[3], maxColorValue = 255)
    new_cols <- c(new_cols, new_col)
  }
  new_cols
}

ip_line_seg <- function(x, y, lwd, col, g, lty = 1, type = "l") {
  g <- pmax(g, 0.15)
  lwd <- pmax(lwd, 0.75)
  n <- length(x)
  for (k in 1:(n - 1)) {
    graphics::lines(
      x = c(x[k], x[k + 1]),
      y = c(y[k], y[k + 1]),
      type = type,
      lwd = lwd[k],
      col = ip_adjustcolor2(col, (g[k] + g[k + 1]) / 2),
      lty = lty
    )
  }
}

ip_get_breaks <- function(cut_var) {
  interval_matrix <- do.call(
    rbind,
    strsplit(gsub("\\[|\\]|\\(|\\)", "", levels(cut_var)), ",", fixed = TRUE)
  )
  df_intervals <- as.data.frame(interval_matrix, stringsAsFactors = FALSE)
  df_intervals$V1 <- as.numeric(df_intervals$V1)
  df_intervals$V2 <- as.numeric(df_intervals$V2)
  names(df_intervals) <- c("from", "to")
  df_intervals
}

ip_sync_factors <- function(data, ndj) {
  for (var in names(data)) {
    if (is.factor(data[[var]])) {
      ndj[[var]] <- factor(ndj[[var]], levels = levels(data[[var]]))
    }
  }
  ndj
}

ip_add_covariates_at_mean <- function(newdata, data) {
  missing_vars <- setdiff(names(data), names(newdata))
  mean_values <- list()

  for (var in missing_vars) {
    if (is.numeric(data[[var]])) {
      mean_values[[var]] <- mean(data[[var]], na.rm = TRUE)
    } else {
      mean_values[[var]] <- as.character(data[[var]][1])
    }
  }

  for (var in names(mean_values)) {
    newdata[[var]] <- mean_values[[var]]
  }

  newdata
}

ip_round2 <- function(x, min.d = 2, max.d = 3) {
  x_char <- as.character(x)
  fnz <- regexpr("[1-9]", sub(".*\\.", "", x_char))
  d_round <- max(min(max.d, fnz), min.d)
  rounded <- round(x, d_round)
  formatC(rounded, format = "f", digits = min.d)
}

ip_clean_string <- function(input_string) {
  gsub("[^A-Za-z0-9_]", "", input_string)
}

ip_is_integer2 <- function(x) all(floor(x) == x)

ip_check1 <- function(var, varname, length.check, type.check) {
  if (length(var) != length.check) {
    exit(
      paste0(
        "interprobe() says the argument '", varname, "' must be of length '",
        length.check, "'\n",
        "but it is of length '", length(var), "'"
      )
    )
  }

  if (type.check == "integer") {
    if (ip_is_integer2(var) == FALSE) {
      exit(paste0("interprobe() says the argument '", varname, "' must be an integer, but '", var, "' isn't."))
    }
  }

  if (type.check == "character") {
    if (is.character(var) == FALSE) {
      exit(paste0("interprobe() says the argument '", varname, "' must be a character variable but '", var, "' isn't."))
    }
  }

  if (type.check == "numeric") {
    if (is.numeric(var) == FALSE) {
      exit(paste0("interprobe() says the argument '", varname, "' must be a numeric, but '", var, "' isn't."))
    }
  }

  if (type.check == "logical") {
    if (is.logical(var) == FALSE) {
      exit(paste0("interprobe() says the argument '", varname, "' must be a logical, but '", var, "' isn't."))
    }
  }
}

ip_check_gam_error <- function(model) {
  if ("try-error" %in% class(model)) {
    message2(
      paste0(
        "interprobe() says: The GAM model could not be estimated. ",
        "This often occurs when predictors have few possible values. ",
        "Try setting the argument 'k' to a low value like k=3 or k=2."
      ),
      col = "red",
      stop = FALSE
    )
    exit("interprobe() stopped due to GAM estimation failure.")
  }
}

ip_validate_input_combinations <- function(data, model, x, y, z) {
  input.xz <- input.data <- input.xyz <- input.model <- FALSE

  if (!is.null(model) & !is.null(y)) {
    y <- NULL
    message("interprobe() says: You specified both 'model' and 'y', will ignore 'y' and use DV in the model.")
  }

  if (!is.null(data)) input.data <- TRUE
  if (!is.null(model)) input.model <- TRUE
  if (!is.null(x) & !is.null(z) & !is.null(y)) input.xyz <- TRUE
  if (!is.null(x) & !is.null(z) & is.null(y)) input.xz <- TRUE

  if (input.data + input.model == 2) {
    exit("interprobe() says: You may include either a data or a model argument, but you included both.")
  }

  if (input.data == TRUE & input.xyz == FALSE) {
    exit(
      paste0(
        "interprobe says: you specified a dataset but not x,y,z, recall that:\n",
        "x: focal predictor\n",
        "z: moderator\n",
        "y: dependent variable"
      )
    )
  }

  if (input.model == TRUE & input.xz == FALSE) {
    message(
      paste0(
        "interprobe says: you specified a model but not x and z, recall that:\n",
        "x: focal predictor\n",
        "z: moderator"
      )
    )
    exit("interprobe() stopped due to missing x/z variable names.")
  }

  if (input.model == TRUE & input.xyz == TRUE) {
    message("interprobe says: you specified both a model and the y argument.\nPlease specify only one of them")
    exit("interprobe() stopped due to incompatible inputs.")
  }

  if (input.data + input.xyz + input.model == 0) {
    message(
      paste0(
        "interprobe says:\nYou must specify the variables to be used through one of these 3 combinations:\n",
        "1) 'data' and variable names for 'x', 'z', 'y'\n",
        "2) vectors 'x', 'z', and 'y'\n",
        "3) 'model' and variable names 'x' and 'z'"
      )
    )
    exit("interprobe() stopped due to missing inputs.")
  }

  list2(input.data, input.xyz, input.model, input.xz)
}

ip_validate_arguments <- function(
  x, z, y,
  model, data,
  k,
  spotlights, spotlight.labels,
  histogram,
  max.unique, n.bin.continuous, n.max,
  xlab, ylab1, ylab2, main1, main2,
  cols,
  draw,
  legend.round,
  xlim,
  save.as,
  xvar, zvar, yvar,
  x.ticks, y1.ticks, y2.ticks,
  moderator.on.x.axis
) {
  if (!is.null(data)) {
    dataname <- deparse(substitute(data))
    if (!inherits(data, "data.frame")) {
      exit(paste0("interprobe says(): the 'data' argument must be a data.frame, but '", dataname, "' is not a data.frame."))
    }

    n1 <- names(data)
    if (!xvar %in% n1) exit(paste0("interprobe() says the focal variable ('", xvar, "') is not in the dataset '", dataname, "'."))
    if (!zvar %in% n1) exit(paste0("interprobe() says the moderator variable ('", zvar, "') is not in the dataset '", dataname, "'."))
    if (!yvar %in% n1) exit(paste0("interprobe() says the dependent variable ('", yvar, "') is not in the dataset '", dataname, "'."))
  }

  if (
    any(class(model) %in% c("logical", "integer", "numeric", "data.frame", "factor")) |
      any(class(x) %in% c("lm", "glm", "gam", "lm2", "lm_robust")) |
      any(class(z) %in% c("lm", "glm", "gam", "lm2", "lm_robust")) |
      any(class(y) %in% c("lm", "glm", "gam", "lm2", "lm_robust"))
  ) {
    exit(
      paste0(
        "interprobe() says:\n",
        "There is a problem with the set of arguments provided to the function.\n",
        "If you are providing a model as input, make sure to reference it explictly\n",
        "and to enter the variable names in quotes.\n",
        "\n   For example:\n      lm1=lm(y~cond*age)\n      interprobe(model=lm1,x='cond',z='age') "
      )
    )
  }

  if (is.null(model) & is.null(data)) {
    if (length(x) != length(z)) exit("interprobe says(): x and z must have the same length")
    if (length(x) != length(y)) exit("interprobe says(): x and y must have the same length")
  }

  if (!is.null(model)) {
    modelname <- attr(model, "interprobe_modelname")
    if (is.null(modelname) || !is.character(modelname) || length(modelname) != 1) {
      modelname <- deparse(substitute(model))
    }
    if (!inherits(model, c("lm", "glm", "gam", "lm2", "lm_robust"))) {
      exit("interprobe() says you provided a model but it is not lm, glm, gam, lm2, or lm_robust")
    }
    vars <- all.vars(terms(model))[-1]
    if (!xvar %in% vars) exit(paste0("interprobe() says the focal variable x ('", xvar, "') is not in the model '", modelname, "'"))
    if (!zvar %in% vars) exit(paste0("interprobe() says the moderator variable z ('", zvar, "') is not in the model '", modelname, "'"))
  }

  if (!is.null(k)) ip_check1(k, "k", 1, "integer")

  if (!is.null(spotlights) && length(spotlights) != 3) {
    exit("interprobe() says the argument 'spotlights' must be of length 3")
  }
  if (!is.null(spotlight.labels) & length(spotlight.labels) != 3) {
    exit("interprobe() says the argument 'spotlight.labels' must be  of length 3")
  }

  if (!is.logical(histogram) && length(histogram) == 1) {
    exit("interprobe() says, The argument 'histogram' must be TRUE or FALSE and of length 1")
  }

  ip_check1(max.unique, "max.unique", 1, "integer")
  ip_check1(n.bin.continuous, "n.bin.continuous", 1, "integer")
  ip_check1(n.max, "n.max", 1, "integer")

  ip_check1(xlab, "xlab", 1, "character")
  ip_check1(ylab1, "ylab1", 1, "character")
  ip_check1(ylab2, "ylab2", 1, "character")
  ip_check1(main1, "main1", 1, "character")
  ip_check1(main2, "main2", 1, "character")
  ip_check1(legend.round, "legend.round", 2, "integer")
  ip_check1(cols, "cols", 3, "character")

  draw2 <- draw
  if (identical(draw2, "simple.slopes")) draw2 <- "simple slopes"
  if (!draw2 %in% c("both", "simple slopes", "jn")) {
    exit(
      paste0(
        "interprobe() says that the argument 'draw' must be one of:\n",
        "  - 'both'\n",
        "  - 'simple slopes' (or legacy: 'simple.slopes')\n",
        "  - 'jn'"
      )
    )
  }

  if (!is.null(xlim)) ip_check1(xlim, "xlim", 2, "numeric")

  if (!is.null(save.as)) {
    extension <- tools::file_ext(save.as)
    if (!extension %in% c("svg", "png")) exit("interprobe() says 'save.as' must be either a .png or .svg format.")
  }

  if (!is.null(model)) {
    mf <- ip_model_frame(model)
    if (is.null(mf)) {
      exit("interprobe() says: could not recover the model frame from the fitted model (try stats::model.frame()).")
    }
    x_model <- mf[, xvar]
    nux <- length(unique(x_model))
    model_txt <- ip_clean_string(deparse(substitute(model)))

    if (nux <= 3 & !inherits(x_model, "factor")) {
      exit(
        paste0(
          "ERROR.\n   ",
          "interprobe() says the focal predictor x ('", xvar, "') has only \n   ",
          nux, " possible values. Make sure to define it as a factor before\n   ",
          "estimating the model '", model_txt, "'. You can do that by running\n   ",
          "df$", xvar, " <- factor(df$", xvar, "), where df is the name of the \n   ",
          "data frame containing  '", xvar, "'."
        )
      )
    }

    formula.txt <- paste0(as.character(model$call), collapse = " ")
    if (as.numeric(regexpr("factor\\(", formula.txt)) > 0) {
      exit(
        paste0(
          "interprobe() says: you defined a variable as factor\n   ",
          "within the '", model_txt, "' call. This creates problems.\n   ",
          "Please define factor variables as factors in \n   ",
          "the data before estimating the model \n   e.g., df$x <- factor(df$x))."
        )
      )
    }
  }

  if (!is.null(x.ticks) && !class(x.ticks) %in% c("numeric", "data.frame", "integer")) {
    exit(paste0("interprobe() says: the argument 'x.ticks' must be either a numeric vector or a dataframe, but it is '", class(x.ticks), "'."))
  }
  if (!is.null(y1.ticks) && !class(y1.ticks) %in% c("numeric", "data.frame", "integer")) {
    exit(paste0("interprobe() says: the argument 'y1.ticks' must be either a numeric vector or a dataframe, but it is '", class(y1.ticks), "'."))
  }
  if (!is.null(y2.ticks) && !class(y2.ticks) %in% c("numeric", "data.frame", "integer")) {
    exit(paste0("interprobe() says: the argument 'y2.ticks' must be either a numeric vector or a dataframe, but it is '", class(y2.ticks), "'."))
  }

  if (!is.null(y1.ticks) && inherits(y1.ticks, "data.frame") && ncol(y1.ticks) != 2) {
    exit(paste0("interprobe() says: the argument 'y1.ticks' must be a dataframe with 2 columns, but it has '", ncol(y1.ticks), "' columns."))
  }
  if (!is.null(y2.ticks) && inherits(y2.ticks, "data.frame") && ncol(y2.ticks) != 2) {
    exit(paste0("interprobe() says: the argument 'y2.ticks' must be a dataframe with 2 columns, but it has '", ncol(y2.ticks), "' columns."))
  }

  ip_check1(moderator.on.x.axis, "moderator.on.x.axis", 1, "logical")

  if (length(unique(c(xvar, zvar, yvar))) < 3) {
    exit("interprobe() says: you seem to have entered the same variable twice")
  }
}

ip_estimate_model <- function(nux, data, k, xvar, zvar, yvar, engine = c("gam", "lm2")) {
  engine <- match.arg(engine)

  if (engine == "lm2") {
    if (nux <= 3) {
      data[, xvar] <- factor(data[, xvar])
    }
    fo <- stats::as.formula(paste(yvar, "~", xvar, "*", zvar))
    return(lm2(fo, data = data, notes = FALSE))
  }

  k_if_specified <- ifelse(is.null(k), "", paste0(",k=", k))

  if (nux <= 3) {
    model.text <- paste0(
      'try(mgcv::gam(',
      yvar, "~",
      "s(", zvar, ",by=", xvar, k_if_specified, ")+",
      xvar,
      ', data=data,method="REML"),silent=TRUE)'
    )
    data[, xvar] <- factor(data[, xvar])
    model <- eval(parse(text = model.text))
    ip_check_gam_error(model)
  }

  if (nux >= 4) {
    model.text <- paste0(
      'try(mgcv::gam(',
      yvar, "~",
      "s(", zvar, k_if_specified, ")+",
      "s(", xvar, k_if_specified, ")+",
      "ti(", xvar, ",", zvar, k_if_specified, ')',
      ',method="REML", data=data),silent=TRUE)'
    )
    model <- eval(parse(text = model.text))
    ip_check_gam_error(model)
  }

  model
}

ip_format_p_apa <- function(p, digits = 3, eps = 1e-3) {
  if (!is.finite(p)) return("p = NA")
  if (p < eps) return("p < .001")
  paste0("p = ", formatC(p, format = "f", digits = digits))
}

ip_get_linear_interaction_test_apa <- function(data, xvar, zvar, yvar) {
  fo <- stats::as.formula(paste(yvar, "~", xvar, "*", zvar))
  fit <- tryCatch(lm2(fo, data = data, notes = FALSE), error = function(e) NULL)
  if (is.null(fit)) return("linear model: unavailable")
  tbl <- attr(fit, "statuser_table")
  if (is.null(tbl)) return("linear model: unavailable")
  term <- paste0(xvar, ":", zvar)
  idx <- which(tbl$term == term)
  if (length(idx) != 1) return("linear model: unavailable")
  tval <- as.numeric(tbl$t[idx])
  df <- as.numeric(tbl$df[idx])
  p <- as.numeric(tbl$p.value[idx])
  if (!is.finite(tval) || !is.finite(df)) return("linear model: unavailable")
  paste0(
    "linear model: t(", formatC(df, format = "f", digits = 0), ") = ",
    formatC(tval, format = "f", digits = 2), ", ",
    ip_format_p_apa(p), " (robust errors: HC3)"
  )
}

ip_get_linear_interaction_test_apa_from_lm2 <- function(fit, xvar, zvar) {
  if (is.null(fit)) return("linear model: unavailable")
  tbl <- attr(fit, "statuser_table")
  if (is.null(tbl)) return("linear model: unavailable")
  term <- paste0(xvar, ":", zvar)
  idx <- which(tbl$term == term)
  if (length(idx) != 1) return("linear model: unavailable")
  tval <- as.numeric(tbl$t[idx])
  df <- as.numeric(tbl$df[idx])
  p <- as.numeric(tbl$p.value[idx])
  if (!is.finite(tval) || !is.finite(df)) return("linear model: unavailable")
  paste0(
    "linear model: t(", formatC(df, format = "f", digits = 0), ") = ",
    formatC(tval, format = "f", digits = 2), ", ",
    ip_format_p_apa(p), " (robust errors: HC3)"
  )
}

ip_get_gam_interaction_test_apa <- function(model, xvar, zvar) {
  sm <- tryCatch(summary(model), error = function(e) NULL)
  if (is.null(sm) || is.null(sm$s.table)) return("GAM: unavailable")
  st <- sm$s.table
  rn <- rownames(st)
  if (is.null(rn)) return("GAM: unavailable")

  preferred <- c(
    paste0("ti(", xvar, ",", zvar, ")"),
    paste0("ti(", zvar, ",", xvar, ")")
  )
  idx <- match(preferred, rn)
  idx <- idx[!is.na(idx)]
  if (length(idx) < 1) {
    hits <- which(grepl(xvar, rn, fixed = TRUE) & grepl(zvar, rn, fixed = TRUE))
    if (length(hits) == 0) return("GAM: unavailable")
    idx <- hits
  }

  row <- idx[1]
  p <- as.numeric(st[row, "p-value"])
  edf <- if ("edf" %in% colnames(st)) as.numeric(st[row, "edf"]) else NA_real_
  refdf <- if ("Ref.df" %in% colnames(st)) as.numeric(st[row, "Ref.df"]) else NA_real_
  fval <- if ("F" %in% colnames(st)) as.numeric(st[row, "F"]) else NA_real_

  if (is.finite(fval) && is.finite(edf) && is.finite(refdf)) {
    return(paste0(
      "GAM: F(", formatC(edf, format = "f", digits = 2), ", ",
      formatC(refdf, format = "f", digits = 2), ") = ",
      formatC(fval, format = "f", digits = 2), ", ",
      ip_format_p_apa(p)
    ))
  }

  paste0("GAM: ", ip_format_p_apa(p))
}

ip_compute_slopes_continuous <- function(spotlights, data, xs, zs, model, xvar, zvar, moderator.on.x.axis) {
  simple.slopes <- list()
  j <- 1

  if (moderator.on.x.axis == FALSE) {
    for (zj in spotlights) {
      ndj <- expand.grid(z = zj, x = xs)
      names(ndj) <- c(zvar, xvar)
      ndj <- ip_add_covariates_at_mean(ndj, data)
      options(warn = -1)
      simple.slopes[[j]] <- marginaleffects::predictions(model, newdata = ndj)
      options(warn = 0)
      j <- j + 1
    }
    return(simple.slopes)
  }

  if (moderator.on.x.axis == TRUE) {
    for (xj in spotlights) {
      ndj <- expand.grid(z = zs, x = xj)
      names(ndj) <- c(zvar, xvar)
      ndj <- ip_add_covariates_at_mean(ndj, data)
      options(warn = -1)
      simple.slopes[[j]] <- marginaleffects::predictions(model, newdata = ndj)
      options(warn = 0)
      j <- j + 1
    }
    return(simple.slopes)
  }
}

ip_compute_slopes_discrete <- function(ux, zs, model, xvar, zvar) {
  simple.slopes <- list()
  j <- 1

  for (xj in ux) {
    ndj <- expand.grid(z = zs, x = xj)
    names(ndj) <- c(zvar, xvar)
    data <- ip_model_frame(model)
    ndj <- ip_add_covariates_at_mean(ndj, data)
    options(warn = -1)
    simple.slopes[[j]] <- marginaleffects::predictions(model, newdata = ndj, by = zvar)
    options(warn = 0)
    j <- j + 1
  }

  simple.slopes
}

ip_compute_jn_continuous <- function(spotlights, data, xs, zs, model, xvar, zvar, moderator.on.x.axis) {
  if (moderator.on.x.axis == FALSE) {
    jn <- list()
    j <- 1
    for (zj in spotlights) {
      ndj <- expand.grid(z = zj, x = xs)
      ndj <- ip_add_covariates_at_mean(ndj, data)
      names(ndj)[1:2] <- c(zvar, xvar)
      options(warn = -1)
      jn[[j]] <- data.frame(marginaleffects::slopes(model, newdata = ndj, var = xvar))
      jn[[j]][, zvar] <- zj
      options(warn = 0)
      j <- j + 1
    }
    return(jn)
  }

  if (moderator.on.x.axis == TRUE) {
    jn <- list()
    j <- 1
    for (xj in spotlights) {
      ndj <- expand.grid(x = xj, z = zs)
      ndj <- ip_add_covariates_at_mean(ndj, data)
      names(ndj)[1:2] <- c(xvar, zvar)
      options(warn = -1)
      jn[[j]] <- data.frame(marginaleffects::slopes(model, newdata = ndj, var = xvar))
      jn[[j]][, xvar] <- xj
      options(warn = 0)
      j <- j + 1
    }
    return(jn)
  }
}

ip_compute_jn_discrete <- function(ux, zs, model, xvar, zvar) {
  floodlight <- list()
  j <- 1

  for (xj in ux[-1]) {
    ndj <- expand.grid(z = zs, x = c(as.character(ux[1]), xj))
    names(ndj)[1:2] <- c(zvar, xvar)
    data <- ip_model_frame(model)
    ndj <- ip_add_covariates_at_mean(ndj, data)
    options(warn = -1)
    floodlight[[j]] <- data.frame(marginaleffects::slopes(model = model, newdata = ndj, by = zvar))
    floodlight[[j]][, xvar] <- xj
    options(warn = 0)
    floodlight[[j]] <- floodlight[[j]][floodlight[[j]]$term == xvar, , drop = FALSE]
    j <- j + 1
  }

  floodlight
}

ip_make_fxz <- function(data, n.bin.continuous, moderation, nux, max.unique, spotlights, xvar, zvar, moderator.on.x.axis) {
  uz <- sort(unique(data[, zvar]))
  nuz <- length(unique(data[, zvar]))

  if (nux > max.unique) {
    if (moderator.on.x.axis == FALSE) {
      xbins <- cut(
        data[, xvar],
        n.bin.continuous,
        include.lowest = TRUE,
        labels = paste0("xbin_", 1:(n.bin.continuous))
      )
      x1bins <- cut(data[, xvar], n.bin.continuous, include.lowest = TRUE)

      cuts <- c()
      cuts[1] <- (spotlights[1] + spotlights[2]) / 2
      cuts[2] <- (spotlights[3] + spotlights[2]) / 2
      zbins <- cut(data[, zvar], breaks = c(-Inf, cuts, Inf), labels = paste0("zbin_", 1:3), include.lowest = TRUE)
      fxz <- table(xbins, zbins)
      return(list2(fxz, x1bins))
    }

    if (moderator.on.x.axis == TRUE) {
      if (nuz > max.unique) {
        xbins <- cut(
          data[, zvar],
          n.bin.continuous,
          include.lowest = TRUE,
          labels = paste0("xbin_", 1:(n.bin.continuous))
        )
        x1bins <- cut(data[, zvar], n.bin.continuous, include.lowest = TRUE)
      }
      if (nuz <= max.unique) {
        xbins <- data[, zvar]
        x1bins <- data[, zvar]
      }

      cuts <- c()
      cuts[1] <- (spotlights[1] + spotlights[2]) / 2
      cuts[2] <- (spotlights[3] + spotlights[2]) / 2
      zbins <- cut(data[, xvar], breaks = c(-Inf, cuts, Inf), labels = paste0("zbin_", 1:3), include.lowest = TRUE)
      fxz <- table(xbins, zbins)
      return(list2(fxz, x1bins))
    }
  }

  if (nux > 3 & nux <= max.unique) {
    cuts <- c()
    cuts[1] <- (spotlights[1] + spotlights[2]) / 2
    cuts[2] <- (spotlights[3] + spotlights[2]) / 2
    zbins <- cut(data[, zvar], breaks = c(-Inf, cuts, Inf), labels = paste0("zbin_", 1:3), include.lowest = TRUE)
    fxz <- table(data[, xvar], zbins)
    return(list2(fxz))
  }

  if (nux <= 3 & moderation == "continuous") {
    zbins <- cut(
      data[, zvar],
      n.bin.continuous,
      include.lowest = TRUE,
      labels = paste0("zbin_", 1:(n.bin.continuous))
    )
    x1bins <- cut(data[, zvar], n.bin.continuous, include.lowest = TRUE)
    fxz <- table(zbins, data[, xvar])
    return(list2(fxz, x1bins))
  }

  if (nux <= 3 & moderation == "discrete") {
    fxz <- t(table(data[, xvar], data[, zvar]))
    return(list(fxz = fxz))
  }
}

ip_draw_histogram <- function(fxz.list, focal, moderation, x1s, nux1, cols, ylim, xlim, max.unique) {
  breaks <- NULL

  fxz <- fxz.list$fxz
  rs <- rowSums(fxz)
  rs <- ifelse(rs > 1000000, paste0(round(rs / 1000000, 1), "M"), rs)
  rs <- ifelse(rs > 1000, paste0(round(rs / 1000, 1), "k"), rs)

  x.axis.bins <- nrow(fxz)
  lines.total <- ncol(fxz)

  y0 <- par("usr")[3]
  x0 <- par("usr")[1]
  x1 <- par("usr")[2]
  yd <- y0 + (0.09 + lines.total * 0.07) * diff(ylim) * 0.85

  graphics::polygon(
    x = c(x0, x0, x1, x1),
    border = NA,
    y = c(y0, yd, yd, y0),
    col = grDevices::adjustcolor("gray77", 0.25)
  )
  xc <- (x1 + x0) / 2
  yd2 <- (yd - y0) * 1.05 + y0
  graphics::text(xc, yd2, "Number of Observations", font = 3, cex = 1.1, pos = 1)

  if (nux1 > max.unique) {
    breaks <- ip_get_breaks(fxz.list$x1bins)
    y0 <- par("usr")[3]
    y1 <- y0 + 0.05 * lines.total * diff(ylim)

    fxz2 <- apply(fxz, 1, cumsum)
    fxz2 <- fxz2 / max(fxz2[nrow(fxz2), ])
    fxz2 <- rbind(rep(0, ncol(fxz2)), fxz2)
    fxz2 <- fxz2 * (y1 - y0) + y0

    for (j in 1:x.axis.bins) {
      for (m in 1:lines.total) {
        x1s2 <- c(breaks$from[j], breaks$from[j], breaks$to[j], breaks$to[j])
        ys <- c(fxz2[m, j], fxz2[m + 1, j], fxz2[m + 1, j], fxz2[m, j])
        graphics::polygon(x = x1s2, y = ys, col = ip_adjustcolor2(cols[m], 0.6))
      }
    }

    graphics::text(rowMeans(breaks), y1, rs, pos = 3, cex = 0.8, font = 3, col = "gray38")
  }

  if (nux1 <= max.unique) {
    y0 <- par("usr")[3]
    y1 <- y0 + 0.05 * lines.total * diff(ylim)
    fxz2 <- (fxz / max(fxz)) * (y1 - y0) + y0

    x.width <- (x1s[2] - x1s[1])
    for (j in 1:lines.total) {
      xj <- x1s + x.width * (j - 2) * 0.1
      graphics::segments(x0 = xj, x1 = xj, y0 = y0, y1 = fxz2[, j], lwd = 4, col = cols[j])
    }
    graphics::text(x1s, y1, rs, pos = 3, cex = 0.8, font = 3, col = "gray38")
  }

  breaks
}

ip_make_plot <- function(
  type, xlab, ylab, main, res, histogram, data, xs, zs, gr, spotlights, cols, spotlight.labels,
  focal, moderation, max.unique, fxz.list, nux, nuz, xvar, zvar, xlim, ylim, legend.title,
  x.ticks, y.ticks, moderator.on.x.axis
) {
  ylim.set.by.user <- !is.null(ylim)

  if (moderator.on.x.axis == FALSE) {
    if (xlab == "") xlab <- xvar
    n.lines <- length(spotlights)
    x1.range <- range(data[, xvar])
    x1s <- xs
    nux1 <- nux
    x1.axis <- "x"
  }

  if (moderator.on.x.axis == TRUE) {
    if (xlab == "") xlab <- zvar

    if (type == "simple.slopes" & focal != "categorical") n.lines <- length(spotlights)
    if (type == "simple.slopes" & focal == "categorical") n.lines <- nux

    if (type == "jn" & focal == "categorical") n.lines <- nux - 1
    if (type == "jn" & focal != "categorical") n.lines <- length(spotlights)

    x1.range <- range(data[, zvar])
    x1s <- zs
    nux1 <- nuz
    x1.axis <- "z"
  }

  if (!is.null(y.ticks) & inherits(y.ticks, "data.frame")) {
    yline.adj <- max(nchar(y.ticks[, 2])) / 3.25
    m0 <- par("mar")
    m1 <- m0
    m1[2] <- m1[2] + yline.adj
    par(mar = m1)
  }

  res.df <- do.call(rbind, res)

  if (is.null(ylim)) ylim <- range(res.df[, c("conf.low", "conf.high")])
  ylim1 <- ylim[1]
  ylim2 <- ylim[2]

  if (type == "jn" & !ylim.set.by.user) {
    if (ylim1 * ylim2 > 0) {
      if (ylim1 > 0) ylim[1] <- 0 - 0.02 * diff(ylim)
      if (ylim1 < 0) ylim[2] <- 0 + 0.02 * diff(ylim)
    }
  }

  if (!ylim.set.by.user) ylim[2] <- ylim[2] + 0.3 * diff(ylim)
  if (histogram == TRUE & !ylim.set.by.user) ylim[1] <- ylim[1] - (0.15 + n.lines * 0.07) * diff(ylim)

  if (is.null(xlim)) xlim <- x1.range

  xaxt <- "s"
  yaxt <- "s"
  if (!is.null(x.ticks)) xaxt <- "n"
  if (!is.null(y.ticks)) yaxt <- "n"

  ylab.plotted <- ylab
  if (!is.null(y.ticks)) ylab.plotted <- ""

  graphics::plot(
    x1s, res[[1]]$estimate,
    type = "n",
    xlab = xlab,
    ylab = ylab.plotted,
    las = 1,
    ylim = ylim,
    xlim = xlim,
    yaxt = "n",
    cex.lab = 1.3,
    font.lab = 2,
    xaxt = xaxt,
    yaxt = yaxt
  )

  if (yaxt == "s") graphics::axis(2, at = pretty(ylim)[c(-1, -2)], las = 1)

  if (!is.null(x.ticks)) {
    if (!is.data.frame(x.ticks)) graphics::axis(side = 1, at = x.ticks)
    if (is.data.frame(x.ticks)) graphics::axis(side = 1, at = x.ticks[, 1], x.ticks[, 2])
  }

  if (!is.null(y.ticks)) {
    if (!is.data.frame(y.ticks)) graphics::axis(side = 2, at = y.ticks, las = 1)
    if (is.data.frame(y.ticks)) graphics::axis(side = 2, at = y.ticks[, 1], y.ticks[, 2], las = 1)
  }

  if (type == "jn") {
    graphics::axis(2, at = 0, las = 1)
    graphics::abline(h = 0, lty = 2, col = "gray77")
  }

  col.seg <- cols
  if (focal == "categorical" & type == "jn") col.seg <- col.seg[-1]

  for (j in 1:n.lines) {
    g1 <- as.numeric(gr[, j])
    if (nux1 > max.unique) g <- rep(g1, each = length(x1s) / length(g1))
    if (nux1 <= max.unique) g <- g1

    ip_line_seg(x1s, res[[j]]$estimate, lwd = 4 * g, col = col.seg[j], g = g)

    graphics::polygon(
      x = c(x1s, rev(x1s)),
      y = c(res[[j]]$conf.high, rev(res[[j]]$conf.low)),
      col = grDevices::adjustcolor(col.seg[j], 0.1),
      border = NA
    )

    if (nux1 <= max.unique) {
      graphics::points(x1s, res[[j]]$estimate, col = ip_adjustcolor2(col.seg[j], pmax(g, 0.2)), pch = 16, cex = 1.5)
    }
  }

  graphics::mtext(side = 3, line = 1.5, font = 2, cex = 1.5, main)

  if (x1.axis == "x") {
    if (is.null(legend.title)) legend.title <- zvar
    graphics::legend(
      "top",
      inset = 0.01,
      bty = "n",
      lwd = 8,
      col = cols[1:n.lines],
      title = legend.title,
      title.font = 2,
      legend = spotlight.labels
    )
  }

  if (x1.axis == "z") {
    if (is.null(legend.title)) legend.title <- paste0("Focal predictor ('", xvar, "')")

    if (type == "simple.slopes" & focal == "categorical") {
      graphics::legend("top", inset = 0.01, bty = "n", lwd = 5, col = cols[1:n.lines], title = legend.title, title.font = 2, legend = xs)
    }

    if (type == "simple.slopes" & focal != "categorical") {
      graphics::legend("top", inset = 0.01, bty = "n", lwd = 5, col = cols[1:n.lines], title = legend.title, title.font = 2, legend = spotlight.labels)
    }

    if (type == "jn" & focal == "categorical") {
      graphics::legend("top", inset = 0.01, bty = "n", lwd = 5, col = cols[-1], title = legend.title, title.font = 2, legend = paste0(xs[-1], " - ", xs[1]))
    }

    if (type == "jn" & focal == "continuous") {
      graphics::legend("top", inset = 0.01, bty = "n", lwd = 5, col = cols, title = legend.title, title.font = 2, legend = spotlight.labels)
    }
  }

  if (!is.null(y.ticks)) graphics::mtext(side = 2, line = par("mar")[2] - 1, font = 2, cex = 1.5, ylab)

  breaks <- NULL
  if (histogram == TRUE) {
    breaks <- ip_draw_histogram(fxz.list, focal, moderation = "continuous", x1s, nux1, cols, ylim, xlim, max.unique)
  }

  breaks
}

ip_get_regions_jn <- function(j, xvar, zvar, focal, probe.bins) {
  j$k <- 1:nrow(j)
  js <- j[j$conf.low * j$conf.high > 0, ]
  output <- "Johnson-Neyman Regions of Significance\n"

  if (nrow(js) == 0) {
    output <- paste0(output, "The effect of ", xvar, " is not significant for any value of the moderator considered")
    return(output)
  }

  if (focal != "categorical") {
    breaks <- c(TRUE, diff(js$k) != 1 | diff(js$z) != 0 | diff(sign(js$marginal.effect)) != 0)
  }

  if (focal == "categorical") {
    same.contrast <- c(js$contrast[-1] == js$contrast[-nrow(js)])
    breaks <- c(TRUE, diff(js$k) != 1 | !same.contrast | diff(sign(js$marginal.effect)) != 0)
  }

  js$batch <- cumsum(breaks)
  j2 <- do.call(
    rbind,
    by(js, js$batch, function(sub_j) {
      sub_j[c(1, nrow(sub_j)), ]
    })
  )

  if (focal != "categorical") {
    uz <- unique(j2$z)
    k <- 0
    for (zk in uz) {
      j3 <- j2[j2[, zvar] == zk, ]
      xs <- j3[, xvar]
      xs <- data.frame(matrix(xs, ncol = 2, byrow = TRUE))
      names(xs) <- c("from.x", "to.x")
      sk <- sign(j3$marginal.effect)
      signk <- sk[seq(1, length(sk), by = 2)]
      xs$sign.text <- ifelse(signk == 1, "positive", "negative")

      if (nrow(xs) > 0) {
        for (rowk in 1:nrow(xs)) {
          k <- k + 1
          xr <- xs[rowk, ]
          output <- paste0(
            output,
            k, ") When '", zvar, "' = ", round(zk, 2), ", the effect of '", xvar, "' is ", xr$sign.text,
            " in the range of '", xvar, "': [", round(xr$from.x, 2), " to ", round(xr$to.x, 2), "]\n"
          )
        }
      }
    }
  }

  if (focal == "categorical") {
    ucon <- unique(j2$contrast)
    k <- 0
    for (conk in ucon) {
      j3 <- j2[j2[, "contrast"] == conk, ]
      zs <- j3[, zvar]
      zs <- data.frame(matrix(zs, ncol = 2, byrow = TRUE))
      names(zs) <- c("from.z", "to.z")
      sk <- sign(j3$marginal.effect)
      signk <- sk[seq(1, length(sk), by = 2)]
      zs$sign.text <- ifelse(signk == 1, "positive", "negative")

      conk.clean <- gsub("mean\\(", "", conk)
      conk.clean <- gsub("\\)", "", conk.clean)

      if (nrow(zs) > 0) {
        for (rowk in 1:nrow(zs)) {
          k <- k + 1
          zr <- zs[rowk, ]
          output <- paste0(
            output,
            k, ") The contrast for '", xvar, "' of ", conk.clean, " is ", zr$sign.text,
            " in the range of '", zvar, "': [", round(zr$from.z, 2), " to ", round(zr$to.z, 2), "]\n"
          )
        }
      }
    }
  }

  output <- paste0(
    output,
    "\n\nNotes:\n",
    "1) Regions of significance reported by interprobe() never include values outside\n",
    "range of observed data.\n",
    "2) These regions are precise to 1/", probe.bins, " of the range of x-values. For \n",
    "greater precision (and slower estimation) set 'probe.bins' to a value greater than '", probe.bins, "'."
  )

  output
}

#' Plot GAM Model 
#'
#' Plots fitted GAM values for focal predictor,
#' keeping any other predictors in the model at a specified quantile  (default: median)
#'
#' @param model A GAM model object fitted using \code{mgcv::gam()}.
#' @param predictor Character string specifying the name of the predictor variable
#'   to plot on the x-axis.
#' @param quantile.others Number between 1 and 99 for quantile
#'   at which all other predictors are held constant. Default is 50 (median).
#' @param col Color for the prediction line. Default is "blue4".
#' @param bg Background color for the confidence band. Default is
#'   \code{adjustcolor('dodgerblue', .2)}.
#' @param plot2 How to plot the distribution in the lower plot. Options: \code{'auto'} (default,
#'   auto-select based on number of unique values), \code{'freq'} (always plot frequencies),
#'   \code{'density'} (always plot the density) or \code{'none'} (neither). When \code{'auto'}, 
#'   plots frequencies with predictor has less than 30 unique values, density otherwise.
#' @param col2 Color for the lines/bars in the bottom distribution plot. Default is "dodgerblue"
#' @param bg2 Background color for the bottom distribution plot. Default is "gray90".
#' @param ... Additional arguments passed to \code{plot()} and \code{lines()}.
#'
#' @return Invisibly returns a list containing:
#' \itemize{
#'   \item \code{predictor_values}: The sequence of predictor values used
#'   \item \code{predicted}: The predicted values
#'   \item \code{se}: The standard errors
#'   \item \code{lower}: Lower confidence bound (predicted - 2*se)
#'   \item \code{upper}: Upper confidence bound (predicted + 2*se)
#' }
#'
#' @examples
#' \donttest{
#' library(mgcv)
#' # Fit a GAM model
#' data(mtcars)
#' mtcars$cyl <- factor(mtcars$cyl)  # Convert to factor before fitting GAM
#' model <- gam(mpg ~ s(hp) + s(wt) + cyl, data = mtcars)
#'
#' # Plot effect of hp (with other variables at median)
#' plot_gam(model, "hp")
#'
#' # Plot effect of hp (with other variables at 25th percentile)
#' plot_gam(model, "hp", quantile.others = 25)
#'
#' # Customize plot
#' plot_gam(model, "hp", main = "Effect of Horsepower", col = "blue", lwd = 2)
#' }
#'
#' @importFrom mgcv gam
#' @export
plot_gam <- function(model, predictor, quantile.others = 50, 
                     col = "blue4", bg = adjustcolor('dodgerblue', .2), 
                     plot2 = 'auto', col2 = NULL, bg2 = "gray90", ...) {
  # Check if model is from mgcv::gam()
  if (!inherits(model, "gam")) {
    stop("'model' must be a GAM model object fitted using mgcv::gam()")
  }
  
  # Check if mgcv is available
  if (!requireNamespace("mgcv", quietly = TRUE)) {
    stop("Package 'mgcv' is required. Please install it with: install.packages('mgcv')")
  }
  
  # Check if factor() is used in the model formula
  # This causes issues with predict() - variables should be converted to factor before gam()
  model_formula <- formula(model)
  formula_char <- paste(deparse(model_formula), collapse = " ")
  # Check for factor( in the formula (factor is lowercase in R)
  if (grepl("\\bfactor\\s*\\(", formula_char, ignore.case = FALSE)) {
    message2("plot_gam() says: A variable in the GAM formula was included with 'factor()'.\n",
         "Please, instead, convert that variable to factor  before running the GAM model\n",
         "\nExample: Instead of gam(y ~ factor(x), data = df), do:\n",
         "    df$x <- factor(df$x)\n",
         "    gam(y ~ x, data = df)", col = 'red', stop = TRUE)
  }
  
  # Validate quantile.others
  if (!is.numeric(quantile.others) || length(quantile.others) != 1) {
    stop("'quantile.others' must be a single numeric value")
  }
  if (quantile.others < 1 || quantile.others > 99) {
    stop("'quantile.others' must be between 1 and 99")
  }
  
  # Validate predictor
  if (!is.character(predictor) || length(predictor) != 1) {
    stop("'predictor' must be a single character string")
  }
  
  # Validate plot2
  if (!is.character(plot2) || length(plot2) != 1) {
    stop("'plot2' must be a single character string")
  }
  if (!plot2 %in% c('auto', 'freq', 'density', 'none')) {
    stop("'plot2' must be one of: 'auto', 'freq', 'density', 'none'")
  }
  
  # Extract model data
  # Try model$model first, then model.frame() as fallback
  model_data <- model$model
  if (is.null(model_data)) {
    # Try to get model frame from the model
    tryCatch({
      model_data <- model.frame(model)
    }, error = function(e) {
      stop("Model data not found. Please refit the model with 'keepData = TRUE' or ensure model data is available.")
    })
  }
  
  # Get the original data if available (from model call)
  # This helps ensure we have the right variable structure
  original_data <- NULL
  if (!is.null(model$call$data)) {
    tryCatch({
      original_data <- eval(model$call$data, envir = environment(formula(model)))
    }, error = function(e) {
      # If we can't get original data, use model_data
      original_data <- model_data
    })
  } else {
    original_data <- model_data
  }
  
  # Get all variable names from the model frame
  all_vars <- names(model_data)
  
  # Remove response variable (first column is typically the response)
  response_var <- all_vars[1]
  predictor_vars <- all_vars[-1]
  
  # Check if predictor exists in model data
  if (!predictor %in% predictor_vars) {
    stop(sprintf("Predictor '%s' not found in model variables. Available variables: %s",
                 predictor, paste(predictor_vars, collapse = ", ")))
  }
  
  # Get other variables (all predictors except the one we're plotting)
  other_vars <- setdiff(predictor_vars, predictor)
  
  # Create new dataset for prediction
  # Start with the predictor variable: 100 equally spaced values between min and max
  predictor_values <- model_data[[predictor]]
  predictor_min <- min(predictor_values, na.rm = TRUE)
  predictor_max <- max(predictor_values, na.rm = TRUE)
  predictor_seq <- seq(predictor_min, predictor_max, length.out = 100)
  n_rows <- length(predictor_seq)
  
  # CRITICAL: Replicate the entire model frame structure first
  # This preserves ALL attributes, factor structures, and internal mgcv requirements
  new_data <- model_data[rep(1, n_rows), , drop = FALSE]
  
  # Now modify the predictor variable
  new_data[[predictor]] <- predictor_seq
  
  # Set other variables to their quantile/default values
  for (var in other_vars) {
    # Get variable from model_data (this is what mgcv sees)
    var_values <- model_data[[var]]
    
    # Handle different variable types
    if (is.factor(var_values)) {
      # For factors, use the first/lowest factor level
      first_level <- levels(var_values)[1]
      # Create factor with first level, preserving all levels
      new_data[[var]] <- factor(rep(first_level, n_rows), levels = levels(var_values))
    } else if (is.character(var_values)) {
      # For character vectors, use the most common value
      var_levels <- table(var_values)
      most_common <- names(var_levels)[which.max(var_levels)]
      new_data[[var]] <- rep(most_common, n_rows)
    } else if (is.numeric(var_values)) {
      # For numeric variables, use the specified quantile
      quantile_value <- quantile(var_values, probs = quantile.others / 100, na.rm = TRUE)
      new_data[[var]] <- rep(as.numeric(quantile_value), n_rows)
    } else {
      # For other types, try to use the most common value
      warning(sprintf("Variable '%s' has unsupported type. Using most common value.", var))
      var_levels <- table(var_values)
      most_common <- names(var_levels)[which.max(var_levels)]
      new_data[[var]] <- rep(most_common, n_rows)
    }
  }
  
  # Extract only predictor variables (exclude response) for prediction
  new_data <- new_data[, predictor_vars, drop = FALSE]
  
  # Make predictions with standard errors
  pred_result <- predict(model, newdata = new_data, se.fit = TRUE)
  predicted <- pred_result$fit
  se <- pred_result$se.fit
  
  # Calculate confidence bounds (2 standard errors)
  lower <- predicted - 2 * se
  upper <- predicted + 2 * se
  
  # Extract additional arguments for plotting
  dots <- list(...)
  
  # Remove col and bg from dots if present (use formal arguments instead)
  if ("col" %in% names(dots)) {
    col <- dots$col
    dots$col <- NULL
  }
  if ("bg" %in% names(dots)) {
    bg <- dots$bg
    dots$bg <- NULL
  }
  
  # Set default labels if not provided
  dots <- set_default(dots, "xlab", predictor)
  dots <- set_default(dots, "ylab", response_var)
  
  # Set default main title if not provided
  main_title_text <- if ("main" %in% names(dots)) dots$main else paste0("GAM Predicting '", response_var, "' with '", predictor,"'")
  
  # Extract model formula for subtitle (will be added after plot)
  model_formula_text <- paste(deparse(formula(model)), collapse = " ")
  
  # Set main='' so we can use mtext() for both titles
  dots$main <- ""
  
  # Set default formatting
  dots <- set_default(dots, "font.lab", 2)
  dots <- set_default(dots, "cex.lab", 1.2)
  dots <- set_default(dots, "las", 1)
  
  # Set default main title formatting if not provided
  dots <- set_default(dots, "font.main", 2)  # bold
  dots <- set_default(dots, "cex.main", 1.2)
  
  # Determine ylim if not provided
  if (!"ylim" %in% names(dots)) {
    dots$ylim <- range(c(lower, upper), na.rm = TRUE)
  }
  
  # Set default xlim if not provided
  if (!"xlim" %in% names(dots)) {
    dots$xlim <- range(predictor_seq, na.rm = TRUE)
  }
  
  # Determine if we need to plot distribution and which method to use
  plot_distribution <- plot2 != 'none'
  use_plot_freq <- FALSE
  use_plot_density <- FALSE
  
  # Extract predictor data once if needed for distribution plot
  predictor_data <- NULL
  if (plot_distribution) {
    predictor_data <- model_data[[predictor]]
    n_unique <- length(unique(predictor_data))
    
    if (plot2 == 'freq') {
      use_plot_freq <- TRUE
    } else if (plot2 == 'density') {
      use_plot_density <- TRUE
    } else if (plot2 == 'auto') {
      # Auto-select: use plot_freq if <= 30 unique values, else plot_density
      if (n_unique <= 30) {
        use_plot_freq <- TRUE
      } else {
        use_plot_density <- TRUE
      }
    }
  }
  
  # Save/restore only what we change in par(); restoring full `par()` can reset `mfg`
  # and break multi-panel plotting via `par(mfrow=...)` in the caller.
  old_mar <- par("mar")
  old_mfrow <- par("mfrow")
  old_mfcol <- par("mfcol")
  on.exit({
    par(mar = old_mar)
    # plot_gam() may use layout() internally; clear it so caller's mfrow can work again
    layout(1)
    par(mfrow = old_mfrow, mfcol = old_mfcol)
  }, add = TRUE)
  
  # Set up layout for two panels if distribution plotting is requested
  if (plot_distribution) {
    # Set up two panels: top for GAM plot (75%), bottom for distribution (25%)
    # Use layout to control spacing (no gap between panels)
    layout(matrix(c(1, 2), nrow = 2, ncol = 1), heights = c(0.75, 0.25))
    
    # Set margins: remove bottom margin from top plot, remove top margin from bottom plot
    # Top margin needs to accommodate main title and subtitle
    par(mar = c(0, 4.1, 5.1, 2.1))  # Top plot: no bottom margin, extra top margin for title
  } else {
    # When no bottom plot, ensure we have enough top margin for title and subtitle
    if (old_mar[3] < 5) {
      par(mar = c(old_mar[1], old_mar[2], 5.1, old_mar[4]))
    }
  }
  
  # Suppress x-axis on top plot if distribution is plotted below
  if (plot_distribution) {
    dots$xaxt <- "n"
  }
  
  # Plot predicted values
  plot_args <- c(list(x = predictor_seq, y = predicted, type = "l"), dots)
  do.call(plot, plot_args)
  
  # Add main title and subtitle using mtext()
  # Get main title cex to calculate subtitle size
  main_cex <- if ("cex.main" %in% names(dots)) dots$cex.main else 1.2
  main_font <- if ("font.main" %in% names(dots)) dots$font.main else 2
  subtitle_cex <- main_cex * 0.9  # 10% smaller (90% of main size)
  
  # Add main title at line=2
  mtext(main_title_text, 
        side = 3, 
        line = 2, 
        cex = main_cex, 
        font = main_font)
  
  # Add subtitle with model formula at line=1 (below main title)
  mtext(model_formula_text, 
        side = 3, 
        line = 1, 
        cex = subtitle_cex, 
        font = 3,  # italic
        col = "gray70")
  
  # Add confidence bands
  polygon(c(predictor_seq, rev(predictor_seq)), 
          c(lower, rev(upper)),
          col = bg,
          border = NA)
  
  # Redraw the prediction line on top
  lines(predictor_seq, predicted, col = col, ...)
  
  # Plot distribution in bottom panel if requested
  if (plot_distribution) {
    # Switch to bottom panel
    par(mar = c(5.1, 4.1, 0, 2.1))  # Bottom plot: no top margin
    
    # Determine xlim for distribution plot (use same as main plot)
    if ("xlim" %in% names(dots)) {
      xlim_dist <- dots$xlim
    } else {
      xlim_dist <- range(predictor_seq, na.rm = TRUE)
    }
    
    if (use_plot_freq) {
      # For plot_freq, determine ylim from frequencies
      freq_table <- table(predictor_data)
      max_freq <- max(freq_table, na.rm = TRUE)
      ylim_dist <- c(0, max_freq)
      # Add extra space at top if value labels might be shown (plot_freq default behavior)
      if (max_freq > 0) {
        ylim_dist[2] <- max_freq + max(1, max_freq * 0.15)  # Add 15% or at least 1 unit
      }
      
      # Get ylab cex from GAM plot to match size
      ylab_cex <- if ("cex.lab" %in% names(dots)) dots$cex.lab else 1.2
      
      # Initialize bottom plot with background
      init_bottom_plot(xlim = xlim_dist, 
                       ylim = ylim_dist,
                       xlab = predictor, 
                       ylab = "",  # Will use mtext instead
                       bg = bg2, 
                       cex.lab = ylab_cex)
      
      # Use plot_freq() with .overlay=TRUE to draw on the background
      # Set col2 if provided, otherwise use plot_freq default
      plot_freq_args <- list(formula = predictor_data, 
                             xlab = predictor,  # Predictor variable name
                             main = "",
                             xlim = xlim_dist,
                             .overlay = TRUE)
      if (!is.null(col2)) {
        plot_freq_args$col <- col2
      }
      do.call(plot_freq, plot_freq_args)
      
      # Draw axes after plot_freq (since .overlay=TRUE doesn't draw axes)
      axis(1)  # x-axis
      axis(2, las = 1)  # y-axis on left side
      
      # Add Frequency label on left side, aligned with top plot's ylab and in gray like tick numbers
      mtext(side = 2, text = "Frequency", line = 2.5, font = 2, cex = ylab_cex, col = "gray30")
    } else if (use_plot_density) {
      # For plot_density, compute density manually to determine ylim
      density_obj <- density(predictor_data)
      ylim_density <- c(0, max(density_obj$y) * 1.3)
      
      # Get ylab cex from GAM plot to match size
      ylab_cex <- if ("cex.lab" %in% names(dots)) dots$cex.lab else 1.2
      
      # Initialize bottom plot with background
      init_bottom_plot(xlim = xlim_dist, 
                       ylim = ylim_density,
                       xlab = predictor, 
                       ylab = "",  # Will use mtext instead
                       bg = bg2, 
                       cex.lab = ylab_cex)
      
      # Manually plot the density curve
      # Use col2 if provided, otherwise default to "dodgerblue"
      density_col <- if (!is.null(col2)) col2 else "dodgerblue"
      lines(density_obj, col = density_col, lwd = 4)
      
      # Add axes
      axis(1)  # x-axis
      axis(4, las = 1)  # y-axis on right side to avoid conflict with GAM plot
      
      # Add Density label on left side, aligned with top plot's ylab and in gray like tick numbers
      mtext(side = 2, text = "Density", line = 2.5, font = 2, cex = ylab_cex, col = "gray30")
    }
  }
  
  # Return results invisibly
  result <- list(
    predictor_values = predictor_seq,
    predicted = predicted,
    se = se,
    lower = lower,
    upper = upper
  )
  
  invisible(result)
}

# Tests for GAM-related functions: twolines, plot_gam, scatter.gam
# These require mgcv package and GAM models

#scatter.gam_001
test_that("scatter.gam runs without errors", {
  skip_if_not_installed("mgcv")
  
  x <- rnorm(100)
  y <- 2*x + rnorm(100)
  
  # Should run without errors
  expect_error(scatter.gam(x, y), NA)
  
  # Should return GAM model invisibly
  result <- scatter.gam(x, y)
  expect_true(inherits(result, "gam"))
})

#scatter.gam_002
test_that("scatter.gam handles data frame input", {
  skip_if_not_installed("mgcv")
  
  df <- data.frame(x = rnorm(100), y = 2*rnorm(100) + rnorm(100))
  
  expect_error(scatter.gam(x, y, data = df), NA)
})

#scatter.gam_003
test_that("scatter.gam handles data.dots parameter", {
  skip_if_not_installed("mgcv")
  
  x <- rnorm(100)
  y <- 2*x + rnorm(100)
  
  expect_error(scatter.gam(x, y, data.dots = TRUE), NA)
  expect_error(scatter.gam(x, y, data.dots = FALSE), NA)
})

#scatter.gam_004
test_that("scatter.gam handles three.dots parameter", {
  skip_if_not_installed("mgcv")
  
  x <- rnorm(100)
  y <- 2*x + rnorm(100)
  
  expect_error(scatter.gam(x, y, three.dots = TRUE), NA)
  expect_error(scatter.gam(x, y, three.dots = FALSE), NA)
})

#scatter.gam_005
test_that("scatter.gam handles k parameter", {
  skip_if_not_installed("mgcv")
  
  x <- rnorm(100)
  y <- 2*x + rnorm(100)
  
  expect_error(scatter.gam(x, y, k = 5), NA)
  expect_error(scatter.gam(x, y, k = 10), NA)
})

#scatter.gam_006
test_that("scatter.gam throws error for invalid data", {
  skip_if_not_installed("mgcv")
  
  # Invalid data frame
  expect_error(scatter.gam(x, y, data = list(x = 1:10, y = 1:10)))
  
  # Missing column
  df <- data.frame(x = rnorm(100))
  expect_error(scatter.gam(x, y, data = df))
})

#scatter.gam_007
test_that("scatter.gam accepts formula syntax with and without data", {
  skip_if_not_installed("mgcv")
  
  # Formula with data= (y ~ x, data = df)
  df <- data.frame(x = rnorm(80), y = 2 * rnorm(80) + rnorm(80))
  expect_error(scatter.gam(y ~ x, data = df), NA)
  
  # Formula with $ syntax and no data (LHS/RHS evaluated in calling env)
  expect_error(scatter.gam(df$y ~ df$x), NA)
})

#twolines_001
test_that("twolines accepts df$y ~ df$x without data=", {
  skip_if_not_installed("mgcv")
  
  df <- data.frame(x = rnorm(80), y = 2 * rnorm(80) + rnorm(80))
  
  pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(dev.off(), add = TRUE)
  
  expect_error(twolines(df$y ~ df$x, quiet = TRUE), NA)
})

#twolines_002
test_that("twolines errors if df$y ~ df$x is combined with data=", {
  skip_if_not_installed("mgcv")
  
  df <- data.frame(x = rnorm(80), y = 2 * rnorm(80) + rnorm(80))
  expect_error(twolines(df$y ~ df$x, data = df, quiet = TRUE), "Do not combine", fixed = TRUE)
})

#scatter.gam_008
test_that("scatter.gam accepts ylim via ... without duplicate-argument error", {
  skip_if_not_installed("mgcv")
  
  x <- rnorm(50)
  y <- 2 * x + rnorm(50)
  expect_error(scatter.gam(x, y, ylim = c(-5, 5)), NA)
})

#plot_gam_001
test_that("plot_gam throws error for non-GAM model", {
  # Should error if model is not a GAM
  expect_error(plot_gam(lm(mpg ~ hp, data = mtcars), "hp"), 
               "must be a GAM model")
})

#plot_gam_002
test_that("plot_gam validates predictor parameter", {
  skip_if_not_installed("mgcv")
  pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(dev.off(), add = TRUE)

  # Create a simple GAM model
  x <- rnorm(100)
  y <- 2*x + rnorm(100)
  model <- mgcv::gam(y ~ s(x))
  
  # Valid predictor
  expect_error(plot_gam(model, "x"), NA)
  
  # Invalid predictor (not character)
  expect_error(plot_gam(model, 1), "must be a single character string")
  
  # Invalid predictor (not in model)
  expect_error(plot_gam(model, "nonexistent"), "not found")
})

#plot_gam_003
test_that("plot_gam validates quantile.others parameter", {
  skip_if_not_installed("mgcv")
  pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(dev.off(), add = TRUE)

  x <- rnorm(100)
  y <- 2*x + rnorm(100)
  model <- mgcv::gam(y ~ s(x))
  
  # Valid quantile
  expect_error(plot_gam(model, "x", quantile.others = 50), NA)
  expect_error(plot_gam(model, "x", quantile.others = 25), NA)
  expect_error(plot_gam(model, "x", quantile.others = 75), NA)
  
  # Invalid quantile (out of range)
  expect_error(plot_gam(model, "x", quantile.others = 0), "between 1 and 99")
  expect_error(plot_gam(model, "x", quantile.others = 100), "between 1 and 99")
})

#plot_gam_004
test_that("plot_gam returns correct structure", {
  skip_if_not_installed("mgcv")
  pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(dev.off(), add = TRUE)

  x <- rnorm(100)
  y <- 2*x + rnorm(100)
  model <- mgcv::gam(y ~ s(x))
  
  result <- plot_gam(model, "x")
  
  expect_true(is.list(result))
  expect_true("predictor_values" %in% names(result))
  expect_true("predicted" %in% names(result))
  expect_true("se" %in% names(result))
  expect_true("lower" %in% names(result))
  expect_true("upper" %in% names(result))
})

#twolines_001
test_that("twolines runs without errors", {
  skip_if_not_installed("mgcv")
  
  # Create U-shaped data
  set.seed(123)
  x <- rnorm(100)
  y <- -x^2 + rnorm(100)
  data <- data.frame(x = x, y = y)
  
  # Should run without errors
  expect_error(twolines(y ~ x, data = data, graph = 0), NA)
  
  # Should return a list
  result <- twolines(y ~ x, data = data, graph = 0)
  expect_true(is.list(result))
})

#twolines_002
test_that("twolines handles graph parameter", {
  skip_if_not_installed("mgcv")
  
  x <- rnorm(100)
  y <- -x^2 + rnorm(100)
  data <- data.frame(x = x, y = y)
  
  # With graph = 0 (no plot)
  expect_error(twolines(y ~ x, data = data, graph = 0), NA)
  
  # With graph = 1 (plot)
  expect_error(twolines(y ~ x, data = data, graph = 1), NA)
})

#twolines_003
test_that("twolines handles quiet parameter", {
  skip_if_not_installed("mgcv")
  
  x <- rnorm(100)
  y <- -x^2 + rnorm(100)
  data <- data.frame(x = x, y = y)
  
  expect_error(twolines(y ~ x, data = data, quiet = TRUE, graph = 0), NA)
  expect_error(twolines(y ~ x, data = data, quiet = FALSE, graph = 0), NA)
})

#twolines_004
test_that("twolines handles covariates", {
  skip_if_not_installed("mgcv")
  
  x <- rnorm(100)
  z <- rnorm(100)
  y <- -x^2 + 0.5*z + rnorm(100)
  data <- data.frame(x = x, y = y, z = z)
  
  expect_error(twolines(y ~ x + z, data = data, graph = 0), NA)
})

#twolines_005
test_that("twolines returns correct structure", {
  skip_if_not_installed("mgcv")
  
  x <- rnorm(100)
  y <- -x^2 + rnorm(100)
  data <- data.frame(x = x, y = y)
  
  result <- twolines(y ~ x, data = data, graph = 0)
  
  expect_true(is.list(result))
  # Check for key elements
  expect_true("b1" %in% names(result))
  expect_true("b2" %in% names(result))
  expect_true("p1" %in% names(result))
  expect_true("p2" %in% names(result))
  expect_true("u.sig" %in% names(result))
})

# ============================================================================
# ADDITIONAL GAM TESTS
# ============================================================================

#scatter.gam_009
test_that("scatter.gam returns sensible fitted values", {
  skip_if_not_installed("mgcv")
  
  set.seed(123)
  x <- seq(-3, 3, length.out = 100)
  y <- 2 * x + rnorm(100, sd = 0.5)  # Linear relationship
  
  result <- scatter.gam(x, y)
  
  # Fitted values should exist
  expect_true(!is.null(result$fitted.values))
  
  # For linear data, fitted should correlate highly with y
  expect_true(cor(result$fitted.values, y) > 0.8)
})

#plot_gam_005
test_that("plot_gam predicted values are reasonable", {
  skip_if_not_installed("mgcv")
  pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(dev.off(), add = TRUE)

  set.seed(456)
  x <- rnorm(100)
  y <- 2 * x + rnorm(100)
  model <- mgcv::gam(y ~ s(x))
  
  result <- plot_gam(model, "x")
  
  # Predicted values should be finite
  expect_true(all(is.finite(result$predicted)))
  
  # Standard errors should be positive
  expect_true(all(result$se > 0))
  
  # Lower bound should be less than upper bound
  expect_true(all(result$lower < result$upper))
})

#plot_gam_006
test_that("plot_gam works with multiple predictors", {
  skip_if_not_installed("mgcv")
  pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(dev.off(), add = TRUE)

  set.seed(789)
  x1 <- rnorm(100)
  x2 <- rnorm(100)
  y <- x1 + x2 + rnorm(100)
  model <- mgcv::gam(y ~ s(x1) + s(x2))
  
  # Should work for both predictors
  result1 <- plot_gam(model, "x1")
  result2 <- plot_gam(model, "x2")
  
  expect_true(is.list(result1))
  expect_true(is.list(result2))
})

#twolines_006
test_that("twolines detects U-shaped relationship", {
  skip_if_not_installed("mgcv")
  
  set.seed(111)
  x <- rnorm(200)
  y <- x^2 + rnorm(200, sd = 0.5)  # Clear U-shape
  data <- data.frame(x = x, y = y)
  
  result <- twolines(y ~ x, data = data, graph = 0)
  
  # For U-shaped data:
  # - b1 (left slope) should be negative
  # - b2 (right slope) should be positive
  expect_true(result$b1 < 0 || result$b2 > 0)
})

#twolines_007
test_that("twolines detects inverted U-shaped relationship", {
  skip_if_not_installed("mgcv")
  
  set.seed(222)
  x <- rnorm(200)
  y <- -x^2 + rnorm(200, sd = 0.5)  # Inverted U-shape
  data <- data.frame(x = x, y = y)
  
  result <- twolines(y ~ x, data = data, graph = 0)
  
  # For inverted U-shaped data:
  # - b1 (left slope) should be positive
  # - b2 (right slope) should be negative
  expect_true(result$b1 > 0 || result$b2 < 0)
})

#' Describe a variable, optionally by groups
#'
#' Returns a dataframe with one row per group.
#'
#' The dependent variable (`y`) must be numeric. If you pass an existing
#' non-numeric variable (e.g., character, factor), `desc_var()` will stop with
#' a clear error indicating that the variable must be numeric.
#'
#' @param y A numeric vector of values, a column name (character string or unquoted) if \code{data} is provided,
#'   or a formula of the form \code{y ~ x} or \code{y ~ x1 + x2} (for multiple grouping variables).
#' @param group Optional grouping variable, if not provided computed for the full data.
#'   Ignored if \code{y} is a formula.
#' @param data Optional data frame containing the variable(s).
#' @param digits Number of decimal places to round to. Default is 3.
#'
#' @return A data frame with one row per group (or one row if no group is specified) containing:
#'   \itemize{
#'     \item \code{group}: Group identifier
#'     \item \code{mean}: Mean
#'     \item \code{sd}: Standard deviation
#'     \item \code{se}: Standard error
#'     \item \code{median}: Median
#'     \item \code{min}: Minimum
#'     \item \code{max}: Maximum
#'     \item \code{mode}: Most frequent value
#'     \item \code{freq_mode}: Frequency of mode
#'     \item \code{mode2}: 2nd most frequent value
#'     \item \code{freq_mode2}: Frequency of 2nd mode
#'     \item \code{n.total}: Number of observations
#'     \item \code{n.missing}: Number of observations with missing (NA) values
#'     \item \code{n.unique}: Number of unique values
#'   }
#'   Columns may also carry a \code{"label"} attribute, which provides a short
#'   human-readable description of each column.
#'
#'
#' @examples
#' # With grouping
#' df <- data.frame(y = rnorm(100), group = rep(c("A", "B"), 50))
#' desc_var(y, group, data = df)
#'
#' # Without grouping (full dataset)
#' desc_var(y, data = df)
#'
#' # Direct vectors
#' y <- rnorm(100)
#' group <- rep(c("A", "B"), 50)
#' desc_var(y, group)
#'
#' # With custom decimal places
#' desc_var(y, group, data = df, digits = 2)
#'
#' # Using formula syntax: y ~ x
#' desc_var(y ~ group, data = df)
#'
#' # Using formula syntax with multiple grouping variables: y ~ x1 + x2
#' df2 <- data.frame(y = rnorm(200), x1 = rep(c("A", "B"), 100), x2 = rep(c("X", "Y"), each = 100))
#' desc_var(y ~ x1 + x2, data = df2)
#'
#' @export
desc_var <- function(y, group = NULL, data = NULL, digits = 3) {
  
  # FUNCTION OUTLINE:
  # 1. Check if y is a formula and extract variables (formula vs non-formula syntax)
  # 2. Validate inputs (numeric check, length checks, perfect overlap check)
  # 3. Define helper function to compute statistics for a vector
  # 4. Compute statistics (either for full dataset or by group)
  # 5. Round numeric columns to specified decimal places
  # 6. Add descriptive labels to columns as attributes
  # 7. Return result dataframe
  
  # Helper function to format error messages with conditional newline
  format_msg <- function(msg) {
    total_length <- nchar(msg) + nchar("desc_var() says: ")
    if (total_length > 40) {
      return(paste0("desc_var() says:\n", msg))
    } else {
      return(paste0("desc_var() says: ", msg))
    }
  }
  
  # Helper function to validate that variables have the same length
  validate_lengths <- function(vars, y_len, var_names = NULL) {
    for (i in seq_along(vars)) {
      if (length(vars[[i]]) != y_len) {
        message2(format_msg("All variables must have the same length"), col = 'red', stop = TRUE)
      }
    }
  }
  
  # Helper function to extract variable from data or environment
  extract_var <- function(var_name, data, calling_env, error_msg) {
    if (!is.null(data)) {
      if (!var_name %in% names(data)) {
        message2(format_msg(sprintf("'%s' not found in data", var_name)), col = 'red', stop = TRUE)
      }
      return(data[[var_name]])
    } else {
      tryCatch({
        get(var_name, envir = calling_env)
      }, error = function(e) {
        message2(format_msg(sprintf("'%s' not found", var_name)), col = 'red', stop = TRUE)
      })
    }
  }
  
  # Helper function to extract and validate grouping variables
  extract_group_vars <- function(group_var_names, data, calling_env, y_len) {
    if (is.null(group_var_names) || length(group_var_names) == 0) {
      return(list(group_list = NULL, use_separate_cols = FALSE))
    }
    
    # Extract grouping variables
    group_list <- lapply(group_var_names, function(gv) {
      extract_var(gv, data, calling_env, sprintf("'%s' not found", gv))
    })
    names(group_list) <- group_var_names
    
    # Validate all variables have the same length
    validate_lengths(group_list, y_len)
    
    # For multiple grouping variables, keep them separate
    # For single grouping variable, create a single group vector
    use_separate_cols <- length(group_var_names) > 1
    
    return(list(group_list = group_list, use_separate_cols = use_separate_cols))
  }
  
  # 1. Check if y is a formula and extract variables
  # Use unevaluated call to detect formula so we never evaluate y (avoids
  # "restarting interrupted promise evaluation" when the object does not exist)
  calling_env <- parent.frame()
  call_match <- match.call()
  y_quote <- call_match[["y"]]
  is_formula <- is.call(y_quote) && identical(y_quote[[1]], as.symbol("~"))

  if (is_formula) {
    # If formula uses `$`, do not allow data= (use either one style or the other).
    .assert_no_dollar_with_data(y, data, func_name = "desc_var")

    # If formula uses `$` and data is NULL, normalize into a clean data+formula pair.
    if (is.null(data) && .formula_contains_dollar(y)) {
      normalized <- .normalize_dollar_formula_to_data_and_formula(
        formula = y,
        calling_env = calling_env,
        func_name = "desc_var"
      )
      y <- normalized$formula
      data <- normalized$data
    }

    # Parse formula: y ~ x or y ~ x1 + x2 (evaluate only now that we know it's a formula)
    formula_vars <- all.vars(y)
    
    if (length(formula_vars) < 1) {
      message2(format_msg("Formula must have at least one variable"), col = 'red', stop = TRUE)
    }
    
    # First variable is response, rest are grouping variables
    y_var_name <- formula_vars[1]
    group_var_names <- if (length(formula_vars) > 1) formula_vars[-1] else NULL
    y_name <- y_var_name  # For consistent error messages
    
    # Validate data frame if provided
    if (!is.null(data) && !is.data.frame(data)) {
      message2(format_msg("'data' must be a data frame"), col = 'red', stop = TRUE)
    }
    
    # Extract response variable
    y <- extract_var(y_var_name, data, calling_env, sprintf("'%s' not found", y_var_name))
    y_len <- length(y)
    
    # Extract grouping variables
    group_result <- extract_group_vars(group_var_names, data, calling_env, y_len)
    group_list <- group_result$group_list
    use_separate_group_cols <- group_result$use_separate_cols
    
    # Create single group vector if single grouping variable, otherwise keep separate
    if (!is.null(group_list)) {
      if (use_separate_group_cols) {
        group <- NULL  # Keep separate, don't create interaction
      } else {
        group <- group_list[[1]]  # Single grouping variable
      }
    } else {
      group <- NULL
    }
    
  } else {
    # Not a formula: use match.call() and deparse(substitute()) like other functions
    group_was_provided <- !missing(group)
    
    # Get variable names from the call (before any evaluation)
    y_expr <- call_match$y
    y_name_raw <- if (!is.null(y_expr)) deparse(y_expr) else "y"
    # Clean variable name: remove df$ prefix if present
    y_name <- if (grepl("\\$", y_name_raw)) {
      strsplit(y_name_raw, "\\$")[[1]][length(strsplit(y_name_raw, "\\$")[[1]])]
    } else {
      y_name_raw
    }
    
    group_name <- NULL
    group_name_raw <- NULL
    if (group_was_provided) {
      group_expr <- call_match$group
      group_name_raw <- if (!is.null(group_expr)) deparse(group_expr) else "group"
      # Clean variable name: remove df$ prefix if present
      group_name <- if (grepl("\\$", group_name_raw)) {
        strsplit(group_name_raw, "\\$")[[1]][length(strsplit(group_name_raw, "\\$")[[1]])]
      } else {
        group_name_raw
      }
    }
    
    # Validate data frame if provided
    if (!is.null(data) && !is.data.frame(data)) {
      message2(format_msg("'data' must be a data frame"), col = 'red', stop = TRUE)
    }
    
    # Extract y variable
    if (!is.null(data)) {
      # Data provided: extract from data frame
      if (!y_name %in% names(data)) {
        message2(format_msg(sprintf("'%s' not found in data", y_name)), col = 'red', stop = TRUE)
      }
      y <- data[[y_name]]
    } else {
      # No data: evaluate from calling environment
      # If the call was df$x (or other call), always eval that expression — do not
      # use get("x"), which can pick up a wrong object when bare `x` exists.
      if (is.call(y_expr)) {
        y <- tryCatch(
          eval(y_expr, envir = calling_env),
          error = function(e) {
            message2(
              format_msg(sprintf("'%s' could not be evaluated", y_name_raw)),
              col = "red",
              stop = TRUE
            )
          }
        )
      } else {
        y_exists <- exists(y_name, envir = calling_env, inherits = TRUE)
        if (!y_exists) {
          y_eval <- tryCatch(
            eval(y_expr, envir = calling_env),
            error = function(e) NULL
          )
          if (is.null(y_eval)) {
            message2(format_msg(sprintf("'%s' not found", y_name)), col = 'red', stop = TRUE)
          } else {
            y <- y_eval
          }
        } else {
          y <- get(y_name, envir = calling_env)
        }
      }
    }
    
    y_len <- length(y)
    
    # Extract group variable if provided
    if (group_was_provided) {
      if (!is.null(data)) {
        # Data provided: extract from data frame
        if (!group_name %in% names(data)) {
          message2(format_msg(sprintf("'%s' not found in data", group_name)), col = 'red', stop = TRUE)
        }
        group <- data[[group_name]]
      } else {
        # No data: evaluate from calling environment (same rule as y: calls like df$g first)
        if (is.call(group_expr)) {
          group <- tryCatch(
            eval(group_expr, envir = calling_env),
            error = function(e) {
              message2(
                format_msg(sprintf("'%s' could not be evaluated", group_name_raw)),
                col = "red",
                stop = TRUE
              )
            }
          )
        } else {
          group_exists <- exists(group_name, envir = calling_env, inherits = TRUE)
          if (!group_exists) {
            if (is.vector(group)) {
              # group is already evaluated, use it as-is
            } else {
              message2(format_msg(sprintf("'%s' not found", group_name)), col = 'red', stop = TRUE)
            }
          } else {
            group <- get(group_name, envir = calling_env)
          }
        }
      }
      
      # Validate length
      if (!is.null(group)) {
        validate_lengths(list(group), y_len)
      }
    } else {
      group <- NULL
    }
    
    # For non-formula case, always use single group column
    group_list <- NULL
    use_separate_group_cols <- FALSE
  }
  
  # 2. Validate inputs
  # 2.1. Validate that y is numeric
  if (!is.numeric(y)) {
    y_class <- paste(class(y), collapse = "/")
    message2(
      format_msg(sprintf("'%s' must be numeric; currently %s", y_name, y_class)),
      col = "red",
      stop = TRUE
    )
  }
  
  # 2.2. Check that multiple grouping variables do not overlap perfectly (formula syntax only)
  if (use_separate_group_cols && !is.null(group_list) && length(group_list) > 1) {
    # Check all pairs of grouping variables for perfect overlap
    for (i in 1:(length(group_list) - 1)) {
      for (j in (i + 1):length(group_list)) {
        var1 <- group_list[[i]]
        var2 <- group_list[[j]]
        # Perfect overlap: one-to-one mapping means unique combinations equals both unique value counts
        # This means each value in var1 maps to exactly one value in var2, and vice versa
        n_unique_combos <- length(unique(paste(var1, var2, sep = "|")))
        n_unique_v1 <- length(unique(var1))
        n_unique_v2 <- length(unique(var2))
        # Perfect overlap occurs when: n_unique_combos == n_unique_v1 == n_unique_v2
        # This ensures one-to-one mapping (no many-to-one or one-to-many)
        if (n_unique_combos == n_unique_v1 && n_unique_combos == n_unique_v2 && n_unique_v1 == n_unique_v2) {
          var1_name <- names(group_list)[i]
          var2_name <- names(group_list)[j]
          message2(format_msg(sprintf("Multiple grouping variables should not overlap perfectly: '%s' and '%s' overlap perfectly", var1_name, var2_name)), col = 'red', stop = TRUE)
        }
      }
    }
  }
    
    
  
  # 3. Define helper function to compute statistics for a vector
  compute_stats <- function(x) {
    n_total <- length(x)  # Total count including missing values
    na_count <- sum(is.na(x))
    x <- x[!is.na(x)]  # Remove NAs for statistics computation
    n <- length(x)  # Count of non-missing values
    
    # Handle empty case (all values are missing)
    if (n == 0) {
      return(list(
        n = as.integer(n_total),  # Total count including missing
        mean = NA_real_,
        sd = NA_real_,
        se = NA_real_,
        median = NA_real_,
        missing = as.integer(na_count),
        unique = 0L,
        min = NA_real_,
        max = NA_real_,
        mode = NA_real_,
        freq_mode = NA_integer_,
        mode2 = NA_real_,
        freq_mode2 = NA_integer_
      ))
    }
    
    # Compute basic statistics
    mean_val <- mean(x)
    sd_val <- sd(x)
    se_val <- sd_val / sqrt(n)
    median_val <- median(x)
    min_val <- min(x)
    max_val <- max(x)
    
    # Compute number of unique values
    n_unique <- length(unique(x))
    
    # Compute mode statistics only if there are repeated values
    has_repeats <- n_unique < n
    
    if (has_repeats) {
      tab <- table(x)
      ord <- order(tab, decreasing = TRUE)
      mode_val <- as.numeric(names(tab)[ord[1]])
      freq_mode <- as.integer(tab[ord[1]])
      mode2nd_val <- if (length(ord) > 1) as.numeric(names(tab)[ord[2]]) else NA_real_
      freq_mode2nd <- if (length(ord) > 1) as.integer(tab[ord[2]]) else NA_integer_
    } else {
      mode_val <- NA_real_
      freq_mode <- NA_integer_
      mode2nd_val <- NA_real_
      freq_mode2nd <- NA_integer_
    }
    
    # Return consistent structure
    return(list(
      n = as.integer(n_total),  # Total count including missing values
      mean = as.numeric(mean_val),
      sd = as.numeric(sd_val),
      se = as.numeric(se_val),
      median = as.numeric(median_val),
      missing = as.integer(na_count),
      unique = as.integer(n_unique),
      min = as.numeric(min_val),
      max = as.numeric(max_val),
      mode = as.numeric(mode_val),
      freq_mode = as.integer(freq_mode),
      mode2 = as.numeric(mode2nd_val),
      freq_mode2 = as.integer(freq_mode2nd)
    ))
  }
  
  # 4. Compute statistics
  has_grouping <- !is.null(group) || (use_separate_group_cols && !is.null(group_list))
  
  if (!has_grouping) {
    # No grouping: compute for full dataset
    stats <- compute_stats(y)
    
    # Build base data frame (counts at end: n, n.missing, n.unique)
    result_df <- data.frame(
      group = "All",
      mean = stats$mean,
      sd = stats$sd,
      se = stats$se,
      median = stats$median,
      min = stats$min,
      max = stats$max,
      mode = stats$mode,
      freq_mode = stats$freq_mode,
      mode2 = stats$mode2,
      freq_mode2 = stats$freq_mode2,
      n.total = stats$n,
      n.missing = stats$missing,
      n.unique = stats$unique,
      stringsAsFactors = FALSE
    )
    
  } else {
    # Grouping: compute for each group
    result_list <- list()
    
    if (use_separate_group_cols) {
      # Multiple grouping variables: iterate over unique combinations
      # Create a data frame to get unique combinations efficiently
      group_df <- as.data.frame(group_list, stringsAsFactors = FALSE)
      unique_combos <- unique(group_df)
      
      # Check for missing combinations (combinations that don't exist in data)
      all_possible_combos <- expand.grid(lapply(group_list, function(x) sort(unique(x))), stringsAsFactors = FALSE)
      all_possible_str <- do.call(paste, c(all_possible_combos, sep = "|"))
      unique_combos_str <- do.call(paste, c(unique_combos, sep = "|"))
      missing_indices <- which(!all_possible_str %in% unique_combos_str)
      
      if (length(missing_indices) > 0) {
        missing_combos_df <- all_possible_combos[missing_indices, , drop = FALSE]
        # Format missing combinations as readable strings
        missing_str <- apply(missing_combos_df, 1, function(row) {
          paste(paste(names(missing_combos_df), row, sep = "="), collapse = ", ")
        })
        missing_list <- paste(missing_str, collapse = "\n")
        message2(format_msg(sprintf("Some possible group combinations are not observed:\n%s", missing_list)), col = 'blue')
      }
      
      # Build data frames for each combination
      for (i in seq_len(nrow(unique_combos))) {
        combo <- unique_combos[i, , drop = FALSE]
        # Create mask: all grouping variables match this combination
        mask <- Reduce(`&`, lapply(names(group_list), function(nm) {
          group_list[[nm]] == combo[[nm]]
        }))
        y_group <- y[mask]
        stats <- compute_stats(y_group)
        
        # Build data frame with grouping variables as separate columns (counts at end)
        result_row <- data.frame(
          combo,
          mean = stats$mean,
          sd = stats$sd,
          se = stats$se,
          median = stats$median,
          min = stats$min,
          max = stats$max,
          mode = stats$mode,
          freq_mode = stats$freq_mode,
          mode2 = stats$mode2,
          freq_mode2 = stats$freq_mode2,
          n.total = stats$n,
          n.missing = stats$missing,
          n.unique = stats$unique,
          stringsAsFactors = FALSE
        )
        
        result_list[[length(result_list) + 1]] <- result_row
      }
    } else {
      # Single group column: use group vector directly
      unique_groups <- sort(unique(group))
      
      # Second pass: build data frames
      for (g in unique_groups) {
        y_group <- y[group == g]
        stats <- compute_stats(y_group)
        
        # Build data frame (counts at end: n, n.missing, n.unique)
        result_row <- data.frame(
          group = as.character(g),
          mean = stats$mean,
          sd = stats$sd,
          se = stats$se,
          median = stats$median,
          min = stats$min,
          max = stats$max,
          mode = stats$mode,
          freq_mode = stats$freq_mode,
          mode2 = stats$mode2,
          freq_mode2 = stats$freq_mode2,
          n.total = stats$n,
          n.missing = stats$missing,
          n.unique = stats$unique,
          stringsAsFactors = FALSE
        )
        
        result_list[[length(result_list) + 1]] <- result_row
      }
    }
    
    result_df <- do.call(rbind, result_list)
    
    # Check if any group has 0 observations
    if (any(result_df$n.total == 0)) {
      zero_groups <- sum(result_df$n.total == 0)
      if (zero_groups == 1) {
        message2(format_msg("1 group has 0 observations"), col = "red2")
      } else {
        message2(format_msg(sprintf("%d groups have 0 observations", zero_groups)), col = "red2")
      }
    }
    
    # Sort by grouping variables
      if (use_separate_group_cols && !is.null(group_list)) {
        # Sort by grouping variables from right to left in formula
        # y ~ x1 + x2 + x3  => sort by x3, then x2, then x1
        sort_cols <- rev(names(group_list))
        result_df <- result_df[do.call(order, result_df[sort_cols]), , drop = FALSE]
      } else if (!is.null(group)) {
        # Sort by single group column
        result_df <- result_df[order(result_df$group), , drop = FALSE]
      }
      
  }
  
  # 5. Round numeric columns to specified decimal places
    numeric_cols <- c("mean", "sd", "se", "median", "mode", "mode2", "min", "max")
    numeric_cols <- numeric_cols[numeric_cols %in% names(result_df)]
    result_df[numeric_cols] <- lapply(result_df[numeric_cols], round, digits = digits)
  
  # 5b. Replace NA mode values with "-" and flag if all modes are NA
    all_modes_na <- all(is.na(result_df$mode))
    # Convert mode columns to character and replace NA with "-"
    result_df$mode <- ifelse(is.na(result_df$mode), "-", as.character(result_df$mode))
    result_df$freq_mode <- ifelse(is.na(result_df$freq_mode), "-", as.character(result_df$freq_mode))
    result_df$mode2 <- ifelse(is.na(result_df$mode2), "-", as.character(result_df$mode2))
    result_df$freq_mode2 <- ifelse(is.na(result_df$freq_mode2), "-", as.character(result_df$freq_mode2))
    
  # 6. Add descriptive labels to columns as attributes
    label_list <- list(
      group = "Group identifier",
      mean = "Mean",
      sd = "Standard deviation",
      se = "Standard error",
      median = "Median (50th percentile)",
      min = "Minimum value",
      max = "Maximum value",
      mode = "Most frequent value",
      freq_mode = "Frequency of mode",
      mode2 = "2nd most frequent value",
      freq_mode2 = "Frequency of 2nd mode",
      n.total = "Number of observations",
      n.missing = "Number of observations with missing values (NA)",
      n.unique = "Number of unique values"
    )
  
  # Add labels for individual grouping variable columns if they exist
    if (use_separate_group_cols && !is.null(group_list)) {
      for (gv in names(group_list)) {
        if (gv %in% names(result_df)) {
          label_list[[gv]] <- paste("Grouping variable:", gv)
        }
      }
    }
  
  # Only include labels for columns that exist
    var_labels(result_df) <- label_list
  
  rownames(result_df) <- NULL
  
  # 7. Add class and attributes for print method, then return
  attr(result_df, "all_modes_na") <- all_modes_na
  attr(result_df, "y_name") <- y_name
  class(result_df) <- c("desc_var", class(result_df))
  return(result_df)
}

#' Print method for desc_var objects
#'
#' @param x An object of class \code{desc_var}
#' @param ... Additional arguments passed to print.data.frame
#'
#' @return Invisibly returns the original object
#' @export
print.desc_var <- function(x, ...) {
  # Remove the desc_var class temporarily to use default data.frame printing
  class(x) <- class(x)[class(x) != "desc_var"]
  
  print(x, ...)
  
  # Print note if all modes were NA
  if (isTRUE(attr(x, "all_modes_na"))) {
    y_name <- attr(x, "y_name")
    if (is.null(y_name) || y_name == "") y_name <- "y"
    cat(sprintf("\nNote: mode not reported because all values of %s are unique\n", y_name))
  }
  
  invisible(x)
}

#desc_var_001
test_that("desc_var computes statistics correctly", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  result <- desc_var(y, group)
  
  expect_true(is.data.frame(result))
  expect_equal(nrow(result), 2)
  expect_true("group" %in% names(result))
  expect_true("n.total" %in% names(result))
  expect_true("mean" %in% names(result))
  expect_true("sd" %in% names(result))
  expect_true("median" %in% names(result))
})

#desc_var_002
test_that("desc_var handles data frame input", {
  df <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
  
  result <- desc_var(value, group, data = df)
  expect_equal(nrow(result), 2)
  expect_equal(as.character(result$group), c("A", "B"))
})

#desc_var_003
test_that("desc_var works when column names match parameter names (y, group)", {
  # This test catches a bug where using column names that match function 

  # parameter names (y, group) caused incorrect variable extraction
  df <- data.frame(y = rnorm(100), group = rep(c("A", "B"), 50))
  
  # Should not error with "All variables must have the same length"
  result <- desc_var(y, group, data = df)
  
  expect_equal(nrow(result), 2)
  expect_equal(as.character(result$group), c("A", "B"))
  expect_equal(sum(as.numeric(result$n.total)), 100)
})

#desc_var_004
test_that("desc_var handles no grouping", {
  y <- rnorm(100)
  
  result <- desc_var(y)
  expect_equal(nrow(result), 1)
  expect_equal(as.character(result$group), "All")
})

#desc_var_005
test_that("desc_var handles missing values", {
  y <- c(rnorm(90), rep(NA, 10))
  group <- rep(c("A", "B"), 50)
  
  result <- desc_var(y, group)
  expect_true(all(result$n.missing >= 0))
  expect_true(sum(result$n.missing) == 10)
})

#desc_var_006
test_that("desc_var respects digits parameter", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  result1 <- desc_var(y, group, digits = 2)
  result2 <- desc_var(y, group, digits = 4)
  
  # Check that rounding is applied (mean should be rounded)
  expect_true(is.numeric(result1$mean))
  expect_true(is.numeric(result2$mean))
  
  # With different digits, values should differ (unless they happen to round the same)
  # This is a weak test, but ensures digits parameter is used
  expect_true(is.data.frame(result1))
  expect_true(is.data.frame(result2))
})

#desc_var_007
test_that("desc_var returns all expected columns", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  result <- desc_var(y, group)
  
  expected_cols <- c("group", "n.total", "mean", "sd", "se", "median", "n.missing", "n.unique",
                     "mode", "freq_mode", "mode2", "freq_mode2",
                     "min", "max")
  
  expect_true(all(expected_cols %in% names(result)))
})

#desc_var_008
test_that("desc_var handles empty groups", {
  y <- c(rnorm(50), rep(NA, 50))
  group <- rep(c("A", "B"), 50)
  
  # Should handle gracefully
  result <- desc_var(y, group)
  expect_true(is.data.frame(result))
  expect_equal(nrow(result), 2)
})

#desc_var_009
test_that("desc_var handles formula syntax", {
  df <- data.frame(y = rnorm(100), group = rep(c("A", "B"), 50))
  
  result <- desc_var(y ~ group, data = df)
  expect_equal(nrow(result), 2)
  expect_equal(as.character(result$group), c("A", "B"))
})

#desc_var_010
test_that("desc_var handles multiple grouping variables", {
  df <- data.frame(
    y = rnorm(200),
    x1 = rep(c("A", "B"), 100),
    x2 = rep(c("X", "Y"), each = 100)
  )
  
  result <- desc_var(y ~ x1 + x2, data = df)
  expect_equal(nrow(result), 4)  # 2 x 2 = 4 combinations
  expect_true("x1" %in% names(result))
  expect_true("x2" %in% names(result))
  expect_false("group" %in% names(result))  # Should use separate columns, not "group"
})

#desc_var_011
test_that("desc_var sorts results by grouping variables", {
  df <- data.frame(
    y = rnorm(200),
    x1 = rep(c("B", "A"), 100),  # Intentionally out of order
    x2 = rep(c("Y", "X"), each = 100)
  )
  
  result <- desc_var(y ~ x1 + x2, data = df)
  # Results should be sorted by x2 then x1 (right-to-left in formula)
  expect_equal(as.character(result$x1), c("A", "B", "A", "B"))
  expect_equal(as.character(result$x2), c("X", "X", "Y", "Y"))
})

#desc_var_012
test_that("desc_var detects missing group combinations", {
  df <- data.frame(
    y = rnorm(100),
    x1 = rep(c("A", "B"), 50),
    x2 = rep(c("X", "Y"), each = 50)
  )
  # Remove one combination
  df_missing <- df[!(df$x1 == "B" & df$x2 == "Y"), ]
  
  # Should produce a message about missing combinations
  expect_message(
    desc_var(y ~ x1 + x2, data = df_missing),
    "Some possible group combinations are not observed"
  )
})

#desc_var_013
test_that("desc_var detects perfectly overlapping grouping variables", {
  df <- data.frame(
    y = rnorm(100),
    x1 = rep(c("A", "B"), 50),
    x2 = rep(c("X", "Y"), 50)
  )
  
  # Should stop with a specific error message
  expect_error(
    desc_var(y ~ x1 + x2, data = df),
    "Multiple grouping variables should not overlap perfectly"
  )
})

# ============================================================================
# EDGE CASES
# ============================================================================

#desc_var_014
test_that("desc_var handles single observation per group", {
  df <- data.frame(
    y = c(1, 2, 3),
    group = c("A", "B", "C")
  )
  
  result <- desc_var(y ~ group, data = df)
  
  expect_equal(nrow(result), 3)
  expect_true(all(result$n.total == 1))
  # SD should be NA for single observations
  expect_true(all(is.na(result$sd)))
})

#desc_var_015
test_that("desc_var handles all-NA values in y", {
  df <- data.frame(
    y = rep(NA_real_, 20),
    group = rep(c("A", "B"), 10)
  )
  
  result <- desc_var(y ~ group, data = df)
  
  expect_equal(nrow(result), 2)
  # Missing should equal total observations per group
  expect_true(all(as.numeric(result$n.missing) == 10))
  # n (non-missing) should be 0
  # Note: desc_var may define n differently - just check structure is valid
  expect_true(is.data.frame(result))
})

#desc_var_016
test_that("desc_var handles single group", {
  y <- rnorm(50)
  
  result <- desc_var(y)
  
  expect_equal(nrow(result), 1)
  expect_equal(as.character(result$group), "All")
  expect_equal(as.numeric(result$n.total), 50)
})

#desc_var_017
test_that("desc_var correctly computes all statistics", {
  # Use known data for exact verification
  y <- c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
  
  result <- desc_var(y)
  
  expect_equal(as.numeric(result$n.total), 10)
  expect_equal(as.numeric(result$mean), 5.5, tolerance = 1e-2)  # Allow for rounding
  expect_equal(as.numeric(result$median), 5.5, tolerance = 1e-2)
  expect_equal(as.numeric(result$min), 1, tolerance = 1e-2)
  expect_equal(as.numeric(result$max), 10, tolerance = 1e-2)
  # SD is rounded by desc_var, so use tolerance
  expect_equal(as.numeric(result$sd), sd(y), tolerance = 0.01)
})

#desc_var_018
test_that("desc_var digits parameter actually rounds differently", {
  y <- c(1.23456789, 2.34567891, 3.45678912)
  
  result2 <- desc_var(y, digits = 2)
  result4 <- desc_var(y, digits = 4)
  
  # Mean with 2 digits should be different from 4 digits
  # 2.35 (2 digits) vs 2.3457 (4 digits)
  expect_true(nchar(format(result2$mean, nsmall = 2)) < nchar(format(result4$mean, nsmall = 4)) ||
              result2$mean != result4$mean)
})

#desc_var_019
test_that("desc_var errors clearly for non-numeric y", {
  df <- data.frame(
    y_char = as.character(1:10),
    y_factor = factor(1:10),
    group = rep(c("A", "B"), each = 5)
  )

  # Non-formula, data frame input
  expect_error(
    desc_var(y_char, group, data = df),
    "'y_char' must be numeric; currently character"
  )

  # Formula syntax
  expect_error(
    desc_var(y_char ~ group, data = df),
    "'y_char' must be numeric; currently character"
  )

  # Factor response
  expect_error(
    desc_var(y_factor ~ group, data = df),
    "'y_factor' must be numeric; currently factor"
  )
})

#desc_var_020
test_that("desc_var still errors when variable truly does not exist", {
  df <- data.frame(
    y = rnorm(10),
    group = rep(c("A", "B"), each = 5)
  )

  expect_error(
    desc_var(nonexistent, group, data = df),
    "'nonexistent' not found in data"
  )

  expect_error(
    desc_var(nonexistent ~ group, data = df),
    "'nonexistent' not found in data"
  )
})

#desc_var_021
test_that("desc_var evaluates df$col expressions before numeric validation", {
  df <- data.frame(x = 1:10)

  # Numeric expression works
  result <- desc_var(df$x)
  expect_true(is.data.frame(result))
  expect_equal(as.numeric(result$mean), 5.5)

  # Existing but non-numeric expression gives numeric-type error (not not-found)
  df$x <- "a"
  expect_error(
    desc_var(df$x),
    "'x' must be numeric; currently character"
  )
})

#' Scatter Plot with GAM Smooth Line
#'
#' Creates a scatter plot with a GAM (Generalized Additive Model) smooth line.
#' Supports both \code{scatter.gam(x, y)} and \code{scatter.gam(y ~ x)}.
#'
#' @param x A numeric vector of x values, or a formula of the form \code{y ~ x}.
#' @param y A numeric vector of y values. Not used if \code{x} is a formula.
#' @param data.dots Logical. If TRUE, displays data on scatterplot
#' @param three.dots Logical. If TRUE, divides x into tertiles and puts markers on the average  x & y for each
#' @param data An optional data frame containing the variables \code{x} and \code{y}.
#' @param k Optional integer specifying the basis dimension for the smooth term
#'   in the GAM model (passed to \code{s(x, k=k)}). If NULL (default), uses the
#'   default basis dimension.
  #' @param plot.dist Character string specifying how to plot the distribution of \code{x}
  #'   underneath the scatter plot. Options: \code{NULL} (default, auto-select based on
  #'   number of unique values), \code{"none"} (no distribution plot), \code{"plot_freq"}
  #'   (always use \code{plot_freq()}), or \code{"hist"} (always use \code{hist()}).
  #'   When \code{NULL}, uses \code{plot_freq()} if there are 25 or fewer unique values,
  #'   otherwise uses \code{hist()}.
#' @param dot.pch Plotting character for data points when \code{data.dots = TRUE}.
#'   Default is 16 (filled circle).
#' @param dot.col Color for data points when \code{data.dots = TRUE}. Default is
#'   \code{adjustcolor('gray', 0.7)} (semi-transparent gray).
#' @param jitter Logical. If TRUE, applies a small amount of jitter to data points
#'   to reduce overplotting. Default is FALSE.
#' @param ... Additional arguments passed to \code{plot()} and \code{gam()}.
#'   Common plot arguments include:
#'   \itemize{
#'     \item \code{main}: Custom title for the plot (e.g., \code{main = "My Title"})
#'     \item \code{col}: Color of the GAM smooth line (e.g., \code{col = "red"})
#'     \item \code{lwd}: Line width of the GAM smooth line (e.g., \code{lwd = 2})
#'     \item \code{xlim}, \code{ylim}: Axis limits (e.g., \code{xlim = c(0, 10)})
#'     \item \code{xlab}, \code{ylab}: Axis labels (e.g., \code{xlab = "Age"})
#'   }
#'
#' @return Invisibly returns the fitted GAM model object.
#'
#' @details
#' This function fits a GAM model with a smooth term for x and plots the fitted
#' smooth line. The function uses the \code{mgcv} package's \code{gam()} function.
#'
#' When \code{three.dots = TRUE}, the x variable is divided into three equal-sized
#' groups (tertiles), and the mean x and y values for each group are plotted as
#' points. This provides a simple summary of the relationship across the range of x.
#'
#' @examples
#' # Generate sample data for examples
#' x <- rnorm(100)
#' y <- 2*x + rnorm(100)
#'
#' # Plot GAM smooth line only
#' scatter.gam(x, y)
#'
#' # Equivalent call using formula syntax (y ~ x)
#' scatter.gam(y ~ x)
#'
#' # Include scatter plot with underlying data points behind the GAM line
#' scatter.gam(x, y, data.dots = TRUE)
#'
#' # Include summary points showing mean x and y for each tertile bin
#' scatter.gam(x, y, three.dots = TRUE)
#'
#' # Customize the plot with a custom title, line color, and line width
#' scatter.gam(x, y, data.dots = TRUE, col = "red", lwd = 2, main = "GAM Fit")
#'
#' # Control smoothness of the GAM line by specifying the basis dimension
#' scatter.gam(x, y, k = 10)
#'
#' @seealso \code{\link[stats]{scatter.smooth}} for a simpler loess-based scatter plot smoother.
#'
#' @importFrom mgcv gam
#' @export
scatter.gam <- function(x, y, data.dots = TRUE, three.dots = FALSE, data = NULL, k = NULL, plot.dist = NULL, 
                        dot.pch = 16, dot.col = adjustcolor('gray', 0.7), jitter = FALSE, ...) {
  # Check if x is a formula (y ~ x syntax)
  is_formula <- tryCatch(inherits(x, "formula"), error = function(e) FALSE)
  
  if (is_formula) {
    # Formula syntax: y ~ x (supports both bare names and expressions like mpg$hwy ~ mpg$displ)
    formula_vars <- all.vars(x)
    # Environment for evaluation: data first if provided, then formula env
    formula_env <- environment(x)
    if (is.null(formula_env)) {
      formula_env <- parent.frame()
    }
    eval_env <- if (!is.null(data)) {
      if (!is.data.frame(data)) {
        stop("scatter.gam(): 'data' must be a data frame", call. = FALSE)
      }
      list2env(data, parent = formula_env)
    } else {
      formula_env
    }

    if (length(formula_vars) == 2) {
      # Two variable names: use names for labels and extract from data or env
      y_var_name <- formula_vars[1]
      x_var_name <- formula_vars[2]
      if (!is.null(data)) {
        if (!y_var_name %in% names(data)) {
          stop(sprintf("scatter.gam(): Variable \"%s\" not found in dataset", y_var_name), call. = FALSE)
        }
        if (!x_var_name %in% names(data)) {
          stop(sprintf("scatter.gam(): Variable \"%s\" not found in dataset", x_var_name), call. = FALSE)
        }
        y <- data[[y_var_name]]
        x <- data[[x_var_name]]
      } else {
        y_exists <- exists(y_var_name, envir = formula_env, inherits = TRUE)
        x_exists <- exists(x_var_name, envir = formula_env, inherits = TRUE)
        if (!y_exists) {
          stop(sprintf("scatter.gam(): Could not find variable '%s'", y_var_name), call. = FALSE)
        }
        if (!x_exists) {
          stop(sprintf("scatter.gam(): Could not find variable '%s'", x_var_name), call. = FALSE)
        }
        y <- get(y_var_name, envir = formula_env, inherits = TRUE)
        x <- get(x_var_name, envir = formula_env, inherits = TRUE)
      }
      y_name <- y_var_name
      x_name <- x_var_name
    } else if (length(x) >= 3) {
      # LHS ~ RHS with complex expressions (e.g. mpg$hwy ~ mpg$displ): evaluate both sides
      form <- x
      y <- eval(form[[2]], envir = eval_env)
      x <- eval(form[[3]], envir = eval_env)
      if (!is.numeric(y) || !is.numeric(x)) {
        stop("scatter.gam(): Formula sides must evaluate to numeric vectors", call. = FALSE)
      }
      # Use variable names only (strip data frame prefix e.g. mtcars$hp -> hp)
      y_name <- paste(deparse(form[[2]], width.cutoff = 60L), collapse = " ")
      x_name <- paste(deparse(form[[3]], width.cutoff = 60L), collapse = " ")
      y_name <- sub("^.*\\$", "", trimws(y_name))
      x_name <- sub("^.*\\$", "", trimws(x_name))
    } else {
      stop("scatter.gam(): Formula must have exactly two terms: y ~ x", call. = FALSE)
    }
  } else {
    # Standard syntax: x, y
    # Capture x and y names for labels (before potentially overwriting)
    x_name_raw <- deparse(substitute(x))
    y_name_raw <- deparse(substitute(y))
    # Remove quotes if present
    x_name_raw <- gsub('^"|"$', '', x_name_raw)
    y_name_raw <- gsub('^"|"$', '', y_name_raw)
    # Clean variable names: remove df$ prefix if present
    x_name <- if (grepl("\\$", x_name_raw)) {
      strsplit(x_name_raw, "\\$")[[1]][length(strsplit(x_name_raw, "\\$")[[1]])]
    } else {
      x_name_raw
    }
    y_name <- if (grepl("\\$", y_name_raw)) {
      strsplit(y_name_raw, "\\$")[[1]][length(strsplit(y_name_raw, "\\$")[[1]])]
    } else {
      y_name_raw
    }
    
    # Handle data frame if provided
    if (!is.null(data)) {
      if (!is.data.frame(data)) {
        stop("scatter.gam(): 'data' must be a data frame", call. = FALSE)
      }
      
      # Extract columns from data frame
      # Use raw names for column lookup (they may include df$ prefix)
      if (!x_name_raw %in% names(data)) {
        stop(sprintf("scatter.gam(): Column '%s' not found in data", x_name_raw), call. = FALSE)
      }
      if (!y_name_raw %in% names(data)) {
        stop(sprintf("scatter.gam(): Column '%s' not found in data", y_name_raw), call. = FALSE)
      }
      
      x <- data[[x_name_raw]]
      y <- data[[y_name_raw]]
    }
  }
  
  # Extract additional arguments
  dots <- list(...)
  
  # Extract plot.dist from dots if it was passed via ... (for backward compatibility)
  if ("plot.dist" %in% names(dots)) {
    plot.dist <- dots$plot.dist
    dots$plot.dist <- NULL
  }
  
  # Check for required package
  if (!requireNamespace("mgcv", quietly = TRUE)) {
    stop("Package 'mgcv' is required. Please install it with: install.packages('mgcv')")
  }
  
  # Fit GAM model
  # Define GAM argument names (to avoid duplication)
  gam_arg_names <- c("family", "method", "optimizer", "control", 
                     "scale", "select", "knots", "sp", "min.sp",
                     "H", "gamma", "fit", "paraPen", "G", "in.out",
                     "drop.unused.levels", "drop.intercept", "nthreads",
                     "cluster", "mustart", "etastart", "offset", "subset",
                     "na.action", "start", "model", "x", "y")
  
  # Separate gam arguments from plot arguments
  gam_args <- dots[names(dots) %in% gam_arg_names]
  plot_args <- dots[!names(dots) %in% gam_arg_names]
  
  # Build GAM formula with optional k parameter
  if (!is.null(k)) {
    gam_formula <- y ~ s(x, k = k)
  } else {
    gam_formula <- y ~ s(x)
  }
  g1 <- do.call(mgcv::gam, c(list(formula = gam_formula), gam_args))
  
  # Create grid for smooth line prediction
  # Use a single shared xlim for both panels to keep x-axes aligned
  # Prefer user-provided xlim if available, otherwise use data range (no buffer)
  xlim_common <- if ("xlim" %in% names(plot_args)) plot_args$xlim else range(x, na.rm = TRUE)
  newdat <- data.frame(x = seq(xlim_common[1], xlim_common[2], length.out = 400))
  yh <- predict(g1, newdata = newdat)
  
  # Calculate three-way spline points if requested
  if (three.dots == TRUE) {
    xq <- cut(x, 3)
    # aggregate returns data frame with Group.1 (factor levels) and x (aggregated values)
    x3_agg <- aggregate(x, list(xq), mean, na.rm = TRUE)
    y3_agg <- aggregate(y, list(xq), mean, na.rm = TRUE)
    x3_means <- x3_agg$x  # Mean x values for each tertile
    y3_means <- y3_agg$x  # Mean y values for each tertile
  }
  
  # Determine ylim (user ylim from ... overrides computed value to avoid duplicate-argument error)
  if ("ylim" %in% names(plot_args)) {
    ylim <- plot_args$ylim
    plot_args$ylim <- NULL
  } else if (data.dots == TRUE && three.dots == TRUE) {
    ylim <- range(c(y, yh, y3_means), na.rm = TRUE)
  } else if (data.dots == TRUE) {
    ylim <- range(y, na.rm = TRUE)
  } else if (three.dots == TRUE) {
    ylim <- range(c(yh, y3_means), na.rm = TRUE)
  } else {
    ylim <- range(yh, na.rm = TRUE)
  }
  
  # Determine if we need to plot distribution and which method to use
  plot_distribution <- is.null(plot.dist) || (plot.dist != "none")
  use_plot_freq <- FALSE
  use_hist <- FALSE
  
  if (plot_distribution) {
    if (!is.null(plot.dist) && plot.dist == "plot_freq") {
      use_plot_freq <- TRUE
    } else if (!is.null(plot.dist) && plot.dist == "hist") {
      use_hist <- TRUE
    } else {
      # Auto-select (when plot.dist is NULL): use plot_freq if 25 or fewer unique values
      n_unique <- length(unique(x))
      if (n_unique <= 25) {
        use_plot_freq <- TRUE
      } else {
        use_hist <- TRUE
      }
    }
  }
  
  # Save current par settings and restore on exit
  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par), add = TRUE)
  
  # Set up layout for two panels if distribution plotting is requested
  if (plot_distribution) {
    # Set up two panels: top for scatter plot, bottom for distribution
    # Use layout to control spacing (no gap between panels)
    layout(matrix(c(1, 2), nrow = 2, ncol = 1), heights = c(2, 1))
    
    # Set margins: remove bottom margin from top plot, remove top margin from bottom plot
    # Use same left and right margins for both plots to ensure x-axes align
    # Left margin: 5.1 to accommodate y-axis label on top plot
    # Right margin: 4.1 to accommodate y-axis labels on bottom plot (axis 4)
    par(mar = c(0, 5.1, 3, 4.1))  # Top plot: no bottom margin, 3 lines top margin, aligned margins
  } else {
    # Set top margin to 3 lines when not plotting distribution
    par(mar = c(old_par$mar[1], old_par$mar[2], 3, old_par$mar[4]))
  }
  
  # Set default labels if not provided
  if (!"xlab" %in% names(plot_args)) plot_args$xlab <- x_name
  if (!"ylab" %in% names(plot_args)) plot_args$ylab <- y_name
  
  # Set axis label formatting and orientation
  if (!"font.lab" %in% names(plot_args)) plot_args$font.lab <- 2
  if (!"cex.lab" %in% names(plot_args)) plot_args$cex.lab <- 1.2
  if (!"las" %in% names(plot_args)) plot_args$las <- 1
  
  # Suppress x-axis on top plot if distribution is plotted below
  if (plot_distribution) {
    plot_args$xaxt <- "n"
  }
  
  # Extract main title if provided, use default if not
  user_main <- plot_args$main
  plot_args$main <- NULL  # Remove main from plot_args to avoid double title
  
  # Create plot frame first (without drawing the line yet)
  # We'll draw the line after the data points so it appears on top
  plot_args_frame <- c(list(x = newdat$x, y = yh, type = 'n', ylim = ylim, xlim = xlim_common), 
                       plot_args)
  do.call(plot, plot_args_frame)
  
  # Add title on top - use user-provided main if specified, otherwise default
  if (!is.null(user_main)) {
    mtext(side = 3, text = user_main, line = 1, font = 2, cex = 1.2)
  } else {
    mtext(side = 3, text = paste0("Scatter GAM - ", x_name, " & ", y_name), 
          line = 1, font = 2, cex = 1.2)
  }
  
  # Add data points if requested (draw these first, before the line)
  if (data.dots == TRUE) {
    # Detect dense data and adjust visualization accordingly
    n_points <- length(x)
    n_unique_x <- length(unique(x))
    density_ratio <- n_points / n_unique_x
    
    # Auto-adjust transparency and jitter for dense data
    auto_jitter <- jitter
    auto_dot_col <- dot.col
    auto_cex <- 0.8  # Default point size
    
    # If data is dense (many points per unique x value), use more transparency and auto-jitter
    if (density_ratio > 10 && !jitter) {
      auto_jitter <- TRUE
      # Make points more transparent for dense data
      if (identical(dot.col, adjustcolor('gray', 0.7))) {
        # Only adjust if using default color
        auto_dot_col <- adjustcolor('gray', 0.3)  # Much more transparent
        auto_cex <- 0.5  # Smaller points for dense data
      }
    } else if (density_ratio > 5 && !jitter) {
      # Moderate density - still use jitter
      auto_jitter <- TRUE
      if (identical(dot.col, adjustcolor('gray', 0.7))) {
        auto_dot_col <- adjustcolor('gray', 0.4)
        auto_cex <- 0.6
      }
    }
    
    # Apply jitter if requested or auto-enabled
    if (auto_jitter) {
      # Calculate jitter amount based on data range
      x_range <- diff(range(x, na.rm = TRUE))
      y_range <- diff(range(y, na.rm = TRUE))
      x_jitter <- x_range * 0.01  # 1% of x range
      y_jitter <- y_range * 0.01  # 1% of y range
      x_plot <- x + runif(length(x), -x_jitter, x_jitter)
      y_plot <- y + runif(length(y), -y_jitter, y_jitter)
    } else {
      x_plot <- x
      y_plot <- y
    }
    points(x_plot, y_plot, pch = dot.pch, col = auto_dot_col, cex = auto_cex)
  }
  
  # Add three-way spline points if requested
  if (three.dots == TRUE) {
    points(x3_means, y3_means, pch = 16, cex = 1.3)
  }
  
  # Draw smooth line on top of data points
  lines(newdat$x, yh, col = 'blue', lwd = 3)
  
  # Plot distribution in bottom panel if requested
  if (plot_distribution) {
    # Switch to bottom panel
    # Use same left and right margins as top plot to ensure x-axes align
    par(mar = c(5.1, 5.1, 0, 4.1))  # Bottom plot: no top margin, aligned margins with top plot
    
    # Determine xlim for distribution plot
    # Use the same xlim as the scatter plot to keep both panels aligned
    xlim_dist <- xlim_common
    
    # Prepare distribution plot arguments - only pass essential arguments
    # plot_freq() and hist() have their own defaults, so we only override what's necessary
    dist_plot_args <- list(
      xlab = x_name,
      ylab = "",  # Will use mtext instead
      main = "",
      xlim = xlim_dist
    )
    
    # Optionally pass through some formatting arguments if they were specified
      # but let plot_freq/hist use their defaults otherwise
    if ("font.lab" %in% names(plot_args)) dist_plot_args$font.lab <- plot_args$font.lab
    if ("cex.lab" %in% names(plot_args)) dist_plot_args$cex.lab <- plot_args$cex.lab
    if ("las" %in% names(plot_args)) dist_plot_args$las <- plot_args$las
    
    if (use_plot_freq) {
      # Calculate ylim from frequencies for background plot (10% buffer at top)
      freq_table <- table(x)
      max_freq <- max(freq_table, na.rm = TRUE)
      ylim_freq <- c(0, if (max_freq > 0) max_freq * 1.10 else 0)
      
      # Get cex.lab for consistency
      cex_lab <- if ("cex.lab" %in% names(plot_args)) plot_args$cex.lab else 1.2
      
      # Initialize bottom plot with gray80 background
      init_bottom_plot(xlim = xlim_dist,
                       ylim = ylim_freq,
                       xlab = x_name,
                       ylab = "",  # Will use mtext instead
                       bg = "gray95",
                       cex.lab = cex_lab)
      
      # Use plot_freq() with .overlay=TRUE to draw on the background
      # Pass x as first positional argument (formula parameter) without naming it
      plot_freq_args <- list(x,
                             xlab = x_name,
                             main = "",
                             xlim = xlim_dist,
                             .overlay = TRUE)
      if ("col" %in% names(dist_plot_args)) plot_freq_args$col <- dist_plot_args$col
      do.call(plot_freq, plot_freq_args)
      
      # Draw axes after plot_freq (since .overlay=TRUE doesn't draw axes)
      axis(1)  # x-axis
      axis(2, las = 1, col = "gray40", col.axis = "gray40")  # y-axis on left side with gray40 color
      
      # Add Frequency label on left side next to the plot with gray40 color
      # Use line = 3 to match the default ylab position from plot() (mgp[1] = 3)
      mtext(side = 2, text = "Frequency", line = 3, font = 2, cex = cex_lab, col = "gray40")
    } else if (use_hist) {
      # Use hist() for distribution plot of x (10% buffer on ylim)
      h <- hist(x, plot = FALSE, breaks = "Sturges")
      max_count <- max(h$counts, na.rm = TRUE)
      ylim_hist <- c(0, if (max_count > 0) max_count * 1.10 else 0)
      hist_args <- c(list(x = x, yaxt = "n", ylim = ylim_hist, breaks = h$breaks), dist_plot_args)
      do.call(hist, hist_args)
      axis(2, las = 1, col = "gray40", col.axis = "gray40")  # y-axis on left side with gray40 color
      
      # Add Frequency label on left side next to the plot with gray40 color
      # Use line = 3 to match the default ylab position from plot() (mgp[1] = 3)
      cex_lab <- if ("cex.lab" %in% names(plot_args)) plot_args$cex.lab else 1.2
      mtext(side = 2, text = "Frequency", line = 3, font = 2, cex = cex_lab, col = "gray40")
    }
  }
  
  # Return GAM model invisibly
  invisible(g1)
}

# Utility functions (not exported)

# Package-level state (avoid global options)
.statuser_state <- new.env(parent = emptyenv())
#0 Set default values in a list
set_default <- function(x, name, value) {
  if (!name %in% names(x)) x[[name]] <- value
  x
}

# Helper function to initialize bottom plot with background
init_bottom_plot <- function(xlim, ylim, xlab, ylab, bg, cex.lab) {
  plot(NA, NA, 
       xlim = xlim, 
       ylim = ylim,
       xlab = xlab, 
       ylab = ylab,
       main = "", 
       xaxt = "n", 
       yaxt = "n",
       bty = "o",  # Show border
       font.lab = 2, 
       cex.lab = cex.lab)
  usr <- par("usr")
  rect(usr[1], usr[3], usr[2], usr[4], col = bg, border = NA)
  box()
}
  


#1 Get colors

      get.colors <- function(k) {
                  if (k==1) return('steelblue')
          
          if (k == 2) {
            return(c("firebrick", "dodgerblue"))
          } else if (k == 3) {
            return( c('orange1','red1','red4'))
          } else if (k == 4) {
            return(c('orange1','red1', 'red4','black'))
          } else {
            # For 5+ groups, use a combination of colors that cycle
            # Start with the 4-group palette and add more colors
            base_colors <- c(  "orange1", "orange3", "red2", "red4",
                             "dodgerblue", "dodgerblue3", "blue1", "blue4",
                             "green", "darkgreen",  "darkorchid", "purple4","gray88", "gray11")
            return(base_colors[1:k])
          }
      }

      
      
#2 Nice exit
  exit <- function(msg, col='red2',font=2) {
    message2(msg,col=col,font=font)
    invokeRestart("abort") 
  }

#3 Determine decimal places for group values
  group_decimals <- function(v) {
    v <- abs(v[is.finite(v)])
    if (!length(v)) return(2)
    
    x <- min(v)
    if (x < 1)  return(3)
    if (x < 10) return(2)
    return(1)
  }

#3 Determine decimal places for group values
  group_decimals <- function(v) {
    v <- abs(v[is.finite(v)])
    if (!length(v)) return(2)
    
    x <- min(v)
    if (x < 1)  return(3)
    if (x < 10) return(2)
    return(1)
  }

#4 Evaluate variable arguments with NSE support
#' Evaluate Variable Arguments with NSE Support
#'
#' Resolves bare symbols (unquoted names) or quoted strings to actual data,
#' either from a data frame or the calling environment. This enables both
#' \code{plot_density(DV1, data=df)} and \code{plot_density("DV1", data=df)}
#' to work identically.
#'
#' @param arg_expr Unevaluated expression (from match.call() or substitute())
#' @param arg_name Character string. Name of the argument (for error messages)
#' @param data Optional data frame to look up columns
#' @param calling_env Environment to evaluate symbols if data is NULL
#' @param func_name Character string. Name of calling function (for error messages)
#' @param allow_null Logical. If TRUE, NULL is a valid input. Default FALSE.
#'
#' @return A list with:
#' \itemize{
#'   \item \code{value}: The actual data (vector, formula, or NULL)
#'   \item \code{name}: Clean name for labels (e.g., "DV1")
#'   \item \code{name_raw}: Raw name for error messages (e.g., "df$DV1")
#'   \item \code{was_symbol}: Logical. TRUE if input was an unquoted name
#' }
#'
#' @keywords internal
evaluate_variable_arguments <- function(arg_expr, 
                                        arg_name = "arg",
                                        data = NULL, 
                                        calling_env = parent.frame(),
                                        func_name = "function",
                                        allow_null = FALSE) {
  
  # Handle NULL input
  if (is.null(arg_expr)) {
    if (allow_null) {
      return(list(
        value = NULL,
        name = NULL,
        name_raw = NULL,
        was_symbol = FALSE
      ))
    } else {
      stop(sprintf("%s(): '%s' is required", func_name, arg_name), call. = FALSE)
    }
  }
  
  # Get the expression as a string for name extraction
  expr_str <- deparse(arg_expr, width.cutoff = 500L)
  expr_str <- paste(expr_str, collapse = "")
  
  # Determine what type of expression this is
  is_symbol <- is.symbol(arg_expr)
  # Check if it's a character constant (quoted string like "DV1")
  is_string <- is.character(arg_expr) || (is.call(arg_expr) && length(arg_expr) == 1 && is.character(arg_expr[[1]]))
  # For character constants captured by match.call, they appear as actual character values
  if (!is_symbol && is.character(arg_expr) && length(arg_expr) == 1) {
    is_string <- TRUE
    string_val <- arg_expr
  } else {
    string_val <- NULL
  }
  
  # Initialize return values
  var_value <- NULL
  var_name_raw <- expr_str
  var_name <- expr_str
  was_symbol <- is_symbol
  
  # Case 1: Bare symbol (unquoted name like DV1)
  if (is_symbol) {
    symbol_name <- as.character(arg_expr)
    
    if (!is.null(data)) {
      # Look up in data frame
      if (symbol_name %in% names(data)) {
        var_value <- data[[symbol_name]]
        var_name <- symbol_name
        var_name_raw <- symbol_name
      } else {
        stop(sprintf('%s(): Column "%s" not found in data', func_name, symbol_name), call. = FALSE)
      }
    } else {
      # Evaluate in calling environment
      if (exists(symbol_name, envir = calling_env, inherits = TRUE)) {
        var_value <- get(symbol_name, envir = calling_env, inherits = TRUE)
        var_name <- symbol_name
        var_name_raw <- symbol_name
      } else {
        stop(sprintf("%s(): Could not find variable '%s'", func_name, symbol_name), call. = FALSE)
      }
    }
  }
  
  # Case 2: Quoted string (like "DV1")
  else if (is_string && !is.null(string_val)) {
    string_name <- string_val
    
    if (!is.null(data)) {
      # Look up in data frame
      if (string_name %in% names(data)) {
        var_value <- data[[string_name]]
        var_name <- string_name
        var_name_raw <- string_name
      } else {
        stop(sprintf('%s(): Column "%s" not found in data', func_name, string_name), call. = FALSE)
      }
    } else {
      # String without data frame - look up in environment
      if (exists(string_name, envir = calling_env, inherits = TRUE)) {
        var_value <- get(string_name, envir = calling_env, inherits = TRUE)
        var_name <- string_name
        var_name_raw <- string_name
      } else {
        stop(sprintf("%s(): Could not find variable '%s'", func_name, string_name), call. = FALSE)
      }
    }
  }
  
  # Case 3: Complex expression (formula, vector, df$col, etc.)
  else {
    # Evaluate the expression
    var_value <- tryCatch({
      eval(arg_expr, envir = calling_env)
    }, error = function(e) {
      stop(sprintf("%s(): Error evaluating '%s': %s", func_name, expr_str, e$message), call. = FALSE)
    })
    
    # Extract a clean name for labels.
    # - df$col            -> "col"
    # - df$col[df$col<k]  -> "col<k"   (avoid trailing ']' in plot titles)
      if (grepl("\\[", expr_str, fixed = FALSE)) {
        base_part <- sub("\\[.*$", "", expr_str)
        cond_part <- sub("^.*\\[", "", expr_str)
        cond_part <- sub("\\]$", "", cond_part)
        
        # Base name: last token after $, otherwise the full base_part.
          base_name <- if (grepl("\\$", base_part)) {
            parts <- strsplit(base_part, "\\$")[[1]]
            parts[length(parts)]
          } else {
            base_part
          }
        
        # Condition: remove whitespace and drop repeated LHS like "df$col" or "col".
          cond_compact <- gsub("\\s+", "", cond_part)
          base_compact <- gsub("\\s+", "", base_part)
          base_name_compact <- gsub("\\s+", "", base_name)
          
          cond_clean <- cond_compact
          cond_clean <- gsub(base_compact, "", cond_clean, fixed = TRUE)
          cond_clean <- gsub(base_name_compact, "", cond_clean, fixed = TRUE)
          
          var_name <- paste0(base_name, cond_clean)
      } else if (grepl("\\$", expr_str)) {
        parts <- strsplit(expr_str, "\\$")[[1]]
        var_name <- parts[length(parts)]
      } else {
        var_name <- expr_str
      }
    var_name_raw <- expr_str
  }
  
  
  return(list(
    value = var_value,
    name = var_name,
    name_raw = var_name_raw,
    was_symbol = was_symbol
  ))
}

#' Enhanced alternative to t.test()
#' 
#'  The basic t-test function in R, \code{\link[stats]{t.test}}, does not report 
#'  the observed difference of means, does not stipulate which mean
#'  is subtracted from which (i.e., whether it computed A-B or B-A), and
#'  presents the test results on the console in a verbose unorganized 
#'  paragraph of text. \code{t.test2} improves on all those counts, and 
#'  in addition, it reports the number of observations per group and if any observations 
#'  are missing it issues a warning. It returns a dataframe instead of a list.
#'  The function is also registered as \code{t.test2} for the \code{\link[base]{t}}
#'  generic to satisfy S3 registration checks while keeping direct calls to
#'  \code{t.test2(...)} unchanged.
#'
#' @param x The first data argument passed to \code{\link[stats]{t.test}}. This can
#'   be a numeric vector or a formula.
#' @param ... Arguments passed to \code{\link[stats]{t.test}}
#'
#' @return A data frame with class \code{c("t.test2", "data.frame")} containing 
#'   a single row with the following columns:
#'   \describe{
#'     \item{mean columns}{One or two columns containing group means, named after 
#'       the input variables (e.g., \code{men}, \code{women}) or \code{Group 1}, 
#'       \code{Group 2} for long names.}
#'     \item{diff column}{For two-sample tests, the difference between means 
#'       (e.g., \code{men-women}).}
#'     \item{d}{For two-sample independent tests only, Cohen's d computed as the
#'       mean difference divided by the pooled standard deviation.}
#'     \item{ci}{The confidence level as a string (e.g., "95 percent").}
#'     \item{ci.L, ci.H}{Lower and upper bounds of the confidence interval.}
#'     \item{t}{The t-statistic.}
#'     \item{df}{Degrees of freedom.}
#'     \item{p.value}{The p-value.}
#'     \item{N columns}{Sample sizes, named \code{N(group1)}, \code{N(group2)} or 
#'       \code{N1}, \code{N2}. For paired tests, a single \code{N} column.}
#'     \item{correlation}{For paired tests only, the correlation between pairs.}
#'   }
#'   Attributes store additional information including missing value counts and 
#'   test type (one-sample, two-sample, paired, Welch vs. Student).
#'
#' @importFrom stats cor
#' @usage NULL
#' @rawNamespace S3method(t,test2)
#'
#' @examples
#' # Two-sample t-test
#' men <- rnorm(100, mean = 5, sd = 1)
#' women <- rnorm(100, mean = 4.8, sd = 1)
#' t.test2(men, women)
#'
#' # Paired t-test
#' x <- rnorm(50, mean = 5, sd = 1)
#' y <- rnorm(50, mean = 5.2, sd = 1)
#' t.test2(x, y, paired = TRUE)
#'
#' # One-sample t-test
#' data <- rnorm(100, mean = 5, sd = 1)
#' t.test2(data, mu = 0)
#'
#' # Formula syntax
#' data <- data.frame(y = rnorm(100), group = rep(c("A", "B"), 50))
#' t.test2(y ~ group, data = data)
#'
#' @export t.test2
t.test2 <- function(x, ...) {
  
  # Get dots_list for validation and later use (extracting group names, etc.)
  dots_list <- list(...)
  
  # Validate formula early if first argument is one
  if (!missing(x)) {
    validate_formula(x, dots_list$data, func_name = "t.test2", calling_env = parent.frame())
    # Do not allow combining `$` formulas with data= (user should pick one style).
    if (inherits(x, "formula")) {
      .assert_no_dollar_with_data(x, dots_list$data, func_name = "t.test2")
    }
  }
  
  # Run t.test with provided arguments (avoiding do.call to prevent expensive deparse)
  tt_result <- stats::t.test(x, ...)
  
  # Determine test type
  is_paired <- grepl("Paired", tt_result$method, ignore.case = TRUE)
  is_one_sample <- length(tt_result$estimate) == 1 && !is_paired
  
  # Extract means
  mean1 <- if (length(tt_result$estimate) >= 1) as.numeric(tt_result$estimate[1]) else NA_real_
  mean2 <- if (length(tt_result$estimate) >= 2) as.numeric(tt_result$estimate[2]) else NA_real_
  
  # Extract test statistics
  t_stat <- as.numeric(tt_result$statistic)
  df <- as.numeric(tt_result$parameter)
  p_value <- as.numeric(tt_result$p.value)
  
  # Extract confidence interval
  conf_int <- tt_result$conf.int
  ci_L <- if (!is.null(conf_int) && length(conf_int) >= 1) as.numeric(conf_int[1]) else NA_real_
  ci_H <- if (!is.null(conf_int) && length(conf_int) >= 2) as.numeric(conf_int[2]) else NA_real_
  conf_level <- if (!is.null(conf_int)) attr(conf_int, "conf.level") else NA_real_
  ci_level <- if (!is.na(conf_level)) paste0(100 * conf_level, "%") else NA_character_
  
  # Calculate difference of means
  diff <- if (!is.na(mean1) && !is.na(mean2)) mean1 - mean2 else NA_real_
  
  # Initialize group names and sample sizes
  group1 <- NA_character_
  group2 <- NA_character_
  group_var_name <- NA_character_  # Store grouping variable name for integer groups
  N1 <- NA_integer_
  N2 <- NA_integer_
  NA1 <- NA_integer_
  NA2 <- NA_integer_
  NA_paired <- NA_integer_  # Number of dropped pairs for paired tests
  corr_value <- NA_real_
  
  # Track extracted raw group vectors for effect size computation (two-sample independent only)
  d_x <- NULL
  d_y <- NULL
  
  # Extract data to get group names and sample sizes
  call_args <- sys.call()
  calling_env <- parent.frame()
  dots_list <- list(...)
  
  # Helper function to extract variable name from expression
  extract_var_name <- function(expr) {
    # Fast path for simple symbols (common case) - avoid expensive deparse()
    if (is.symbol(expr)) {
      return(as.character(expr))
    }
    # For complex expressions (e.g., df$col), use deparse
    expr_str <- deparse(expr, width.cutoff = 500)
    if (grepl("\\$", expr_str)) {
      var_name <- sub(".*\\$", "", expr_str)
      return(trimws(var_name))
    }
    return(trimws(expr_str))
  }
  
  # Helper function to check if variables were read from environment when data= was specified
  # Takes a list of variable info: list(list(name="var1", from_env=TRUE, from_data=FALSE), ...)
  # Returns warning message or NULL (warning is printed by print method at the end)
  check_env_vars <- function(var_info_list, data_arg, call_args) {
    if (is.null(data_arg) || !is.data.frame(data_arg)) {
      return(NULL)
    }
    
    # Collect variables that need warnings
    impacted_vars <- character(0)
    for (var_info in var_info_list) {
      if (!is.null(var_info$name) && isTRUE(var_info$from_env) && !isTRUE(var_info$from_data)) {
        impacted_vars <- c(impacted_vars, var_info$name)
      }
    }
    
    # If any variables are impacted, return the warning message
    if (length(impacted_vars) > 0) {
      data_name <- if ("data" %in% names(call_args)) {
        deparse(call_args$data)
      } else {
        "data"
      }
      
      var_list <- paste0("'", impacted_vars, "'", collapse = ", ")
      if (length(impacted_vars) == 1) {
        return(sprintf("Warning: you specified %s and data='%s' but that variable is not in that data set.\nThe t-test was run reading %s directly from the R environment.", var_list, data_name, var_list))
      } else {
        return(sprintf("Warning: you specified %s and data='%s' but those variables are not in that data set.\nThe t-test was run reading %s directly from the R environment.", var_list, data_name, var_list))
      }
    }
    return(NULL)
  }
  
  # Variable to store env warning message (will be added as attribute)
  env_warning_msg <- NULL
  
  # Try to extract data and group names
  tryCatch({
    # Check if it's formula syntax
    if (length(call_args) >= 2 && is.call(call_args[[2]]) && 
        as.character(call_args[[2]][[1]]) %in% c("~", "formula")) {
      # Formula syntax: y ~ group
      formula <- eval(call_args[[2]], envir = calling_env)
      data_arg <- if ("data" %in% names(call_args)) eval(call_args$data, envir = calling_env) else NULL

      # Do not allow combining `$` formulas with data= (user should pick one style).
      .assert_no_dollar_with_data(formula, data_arg, func_name = "t.test2")

      # If formula uses `$` and no data= is given, normalize into a clean data+formula pair.
      if (is.null(data_arg) && .formula_contains_dollar(formula)) {
        normalized <- .normalize_dollar_formula_to_data_and_formula(
          formula = formula,
          calling_env = calling_env,
          func_name = "t.test2"
        )
        formula <- normalized$formula
        data_arg <- normalized$data
      }
      
      # Extract variable names from formula
      y_var_name <- extract_var_name(formula[[2]])
      group_var_name <- extract_var_name(formula[[3]])
      
      # Check if the formula uses $ notation (e.g., df$minutes.watched ~ df$cond)
      y_expr_str <- deparse(formula[[2]], width.cutoff = 500)
      group_expr_str <- deparse(formula[[3]], width.cutoff = 500)
      y_uses_dollar <- grepl("\\$", y_expr_str)
      group_uses_dollar <- grepl("\\$", group_expr_str)
      
      # Extract y and group variables
      # Match t.test() behavior: check environment first, then data frame
      # This is important because t.test() uses environment variables if they exist,
      # even when data= is specified
      
      # For $ notation, evaluate the full expression directly
      if (y_uses_dollar) {
        y_var <- eval(formula[[2]], envir = calling_env)
        y_from_env <- TRUE
        y_from_data <- FALSE
      } else {
        y_from_env <- exists(y_var_name, envir = calling_env, inherits = TRUE)
        y_from_data <- !is.null(data_arg) && is.data.frame(data_arg) && y_var_name %in% names(data_arg)
        
        if (y_from_env) {
          y_var <- eval(as.name(y_var_name), envir = calling_env)
        } else if (y_from_data) {
          y_var <- data_arg[[y_var_name]]
        } else {
          y_var <- eval(as.name(y_var_name), envir = calling_env)  # Will throw error if not found
        }
      }
      
      if (group_uses_dollar) {
        group_var <- eval(formula[[3]], envir = calling_env)
        group_from_env <- TRUE
        group_from_data <- FALSE
      } else {
        group_from_env <- exists(group_var_name, envir = calling_env, inherits = TRUE)
        group_from_data <- !is.null(data_arg) && is.data.frame(data_arg) && group_var_name %in% names(data_arg)
        
        if (group_from_env) {
          group_var <- eval(as.name(group_var_name), envir = calling_env)
        } else if (group_from_data) {
          group_var <- data_arg[[group_var_name]]
        } else {
          group_var <- eval(as.name(group_var_name), envir = calling_env)  # Will throw error if not found
        }
      }
      
      # Check if data= is specified but variables were read from environment instead
      env_warning_msg <- check_env_vars(
        list(
          list(name = y_var_name, from_env = y_from_env, from_data = y_from_data),
          list(name = group_var_name, from_env = group_from_env, from_data = group_from_data)
        ),
        data_arg, call_args
      )
      
      # Get unique groups
      unique_groups <- sort(unique(group_var))
      if (length(unique_groups) == 2) {
        # Check if groups are integers
        g1_val <- unique_groups[1]
        g2_val <- unique_groups[2]
        
        # Check if both are integers (numeric and equal to their integer value)
        is_g1_int <- is.numeric(g1_val) && !is.na(g1_val) && g1_val == as.integer(g1_val)
        is_g2_int <- is.numeric(g2_val) && !is.na(g2_val) && g2_val == as.integer(g2_val)
        
        if (is_g1_int && is_g2_int) {
          # Format as "varname=value"
          group1 <- paste0(group_var_name, "=", as.character(g1_val))
          group2 <- paste0(group_var_name, "=", as.character(g2_val))
        } else {
          group1 <- as.character(g1_val)
          group2 <- as.character(g2_val)
        }
        
        # Important: exclude NA group values; otherwise logical subsetting keeps NA rows
        # and inflates missing counts within both groups.
        g1_data <- y_var[!is.na(group_var) & group_var == unique_groups[1]]
        g2_data <- y_var[!is.na(group_var) & group_var == unique_groups[2]]
        
        d_x <- g1_data
        d_y <- g2_data
        
        N1 <- sum(!is.na(g1_data))
        N2 <- sum(!is.na(g2_data))
        NA1 <- sum(is.na(g1_data))
        NA2 <- sum(is.na(g2_data))
        
        # Recalculate means from data
        mean1 <- mean(g1_data, na.rm = TRUE)
        mean2 <- mean(g2_data, na.rm = TRUE)
        diff <- mean1 - mean2
        
        # Calculate correlation for paired tests and track dropped pairs
        if (is_paired && length(g1_data) == length(g2_data)) {
          complete <- complete.cases(g1_data, g2_data)
          NA_paired <- sum(!complete)  # Number of pairs dropped due to missing values
          if (sum(complete) > 1) {
            corr_value <- cor(g1_data[complete], g2_data[complete])
          }
        }
      }
    } else {
      # Standard syntax: x, y or just x
      x_arg <- NULL
      y_arg <- NULL
      x_expr <- NULL
      y_expr <- NULL
      x_from_env <- FALSE
      x_from_data <- FALSE
      y_from_env <- FALSE
      y_from_data <- FALSE
      x_var_name <- NULL
      y_var_name <- NULL
      data_arg <- if ("data" %in% names(call_args)) eval(call_args$data, envir = calling_env) else NULL
      
      # Find x and y in the call
      if ("x" %in% names(call_args)) {
        x_arg <- eval(call_args$x, envir = calling_env)
        x_expr <- call_args$x
        # For named arguments, assume they come from environment (they're evaluated directly)
        if (is.symbol(x_expr)) {
          x_var_name <- as.character(x_expr)
          x_from_env <- exists(x_var_name, envir = calling_env, inherits = TRUE)
          x_from_data <- !is.null(data_arg) && is.data.frame(data_arg) && x_var_name %in% names(data_arg)
        }
      }
      
      if ("y" %in% names(call_args)) {
        y_arg <- eval(call_args$y, envir = calling_env)
        y_expr <- call_args$y
        # For named arguments, assume they come from environment (they're evaluated directly)
        if (is.symbol(y_expr)) {
          y_var_name <- as.character(y_expr)
          y_from_env <- exists(y_var_name, envir = calling_env, inherits = TRUE)
          y_from_data <- !is.null(data_arg) && is.data.frame(data_arg) && y_var_name %in% names(data_arg)
        }
      }
      
      # If not found as named arguments, try positional arguments
      if (is.null(x_arg) && length(call_args) >= 2) {
        arg2_name <- names(call_args)[2]
        if (is.null(arg2_name) || arg2_name == "") {
          x_expr <- call_args[[2]]
          
          if (is.symbol(x_expr)) {
            x_var_name <- as.character(x_expr)
            # Match t.test() behavior: check environment first, then data frame
            x_from_env <- exists(x_var_name, envir = calling_env, inherits = TRUE)
            x_from_data <- !is.null(data_arg) && is.data.frame(data_arg) && x_var_name %in% names(data_arg)
            
            if (x_from_env) {
              x_arg <- eval(x_expr, envir = calling_env)
            } else if (x_from_data) {
              x_arg <- data_arg[[x_var_name]]
            } else {
              x_arg <- eval(x_expr, envir = calling_env)  # Will throw error if not found
            }
          } else {
            x_arg <- eval(x_expr, envir = calling_env)
          }
        }
      }
      
      if (is.null(y_arg) && length(call_args) >= 3) {
        arg3_name <- names(call_args)[3]
        if (is.null(arg3_name) || arg3_name == "") {
          y_expr <- call_args[[3]]
          
          if (is.symbol(y_expr)) {
            y_var_name <- as.character(y_expr)
            # Match t.test() behavior: check environment first, then data frame
            y_from_env <- exists(y_var_name, envir = calling_env, inherits = TRUE)
            y_from_data <- !is.null(data_arg) && is.data.frame(data_arg) && y_var_name %in% names(data_arg)
            
            if (y_from_env) {
              y_arg <- eval(y_expr, envir = calling_env)
            } else if (y_from_data) {
              y_arg <- data_arg[[y_var_name]]
            } else {
              y_arg <- eval(y_expr, envir = calling_env)  # Will throw error if not found
            }
          } else {
            y_arg <- eval(y_expr, envir = calling_env)
          }
        }
      }
      
      # Check if data= is specified but variables were read from environment instead
      env_warning_msg <- check_env_vars(
        list(
          list(name = x_var_name, from_env = x_from_env, from_data = x_from_data),
          list(name = y_var_name, from_env = y_from_env, from_data = y_from_data)
        ),
        data_arg, call_args
      )
      
      # Extract group names (only for two-sample tests)
      if (!is_one_sample) {
        if (!is.null(x_expr)) {
          group1 <- extract_var_name(x_expr)
        } else if (!is.null(x_arg)) {
          group1 <- "x"
        }
        
        if (!is.null(y_expr)) {
          group2 <- extract_var_name(y_expr)
        } else if (!is.null(y_arg)) {
          group2 <- "y"
        }
      }
      
      # Calculate sample sizes and means from data
      if (!is.null(x_arg) && is.numeric(x_arg)) {
        N1 <- sum(!is.na(x_arg))
        NA1 <- sum(is.na(x_arg))
        # Recalculate mean from data (important for paired tests)
        mean1 <- mean(x_arg, na.rm = TRUE)
        d_x <- x_arg
      }
      
      if (!is.null(y_arg) && is.numeric(y_arg)) {
        N2 <- sum(!is.na(y_arg))
        NA2 <- sum(is.na(y_arg))
        # Recalculate mean from data (important for paired tests)
        mean2 <- mean(y_arg, na.rm = TRUE)
        d_y <- y_arg
      }
      
      # For paired tests, calculate correlation and track dropped pairs
      if (is_paired && !is.null(x_arg) && !is.null(y_arg) && 
          is.numeric(x_arg) && is.numeric(y_arg) &&
          length(x_arg) == length(y_arg)) {
        complete <- complete.cases(x_arg, y_arg)
        NA_paired <- sum(!complete)  # Number of pairs dropped due to missing values
        if (sum(complete) > 1) {
          corr_value <- cor(x_arg[complete], y_arg[complete])
        }
        # Recalculate diff from cleaned data
        if (sum(complete) > 0) {
          diff <- mean(x_arg[complete] - y_arg[complete], na.rm = TRUE)
        }
      } else if (!is.na(mean1) && !is.na(mean2)) {
        # Recalculate diff for non-paired tests
        diff <- mean1 - mean2
      }
    }
  }, error = function(e) {
    # If we can't extract data, use defaults
    # Group names will remain NA or "x"/"y"
  })
  
  # Set group1 and group2 to NA for one-sample tests
  if (is_one_sample) {
    group1 <- NA_character_
    group2 <- NA_character_
  }
  
  # Compute Cohen's d (two-sample independent only; pooled SD)
  d <- NA_real_
  if (!isTRUE(is_one_sample) && !isTRUE(is_paired)) {
    if (!is.null(d_x) && !is.null(d_y) && is.numeric(d_x) && is.numeric(d_y)) {
      x_complete <- d_x[!is.na(d_x)]
      y_complete <- d_y[!is.na(d_y)]
      
      n1 <- length(x_complete)
      n2 <- length(y_complete)
      
      if (n1 >= 2 && n2 >= 2) {
        s1 <- stats::sd(x_complete)
        s2 <- stats::sd(y_complete)
        
        if (!is.na(s1) && !is.na(s2) && is.finite(s1) && is.finite(s2)) {
          denom_df <- n1 + n2 - 2
          if (denom_df > 0) {
            sp <- sqrt(((n1 - 1) * (s1^2) + (n2 - 1) * (s2^2)) / denom_df)
            if (!is.na(sp) && is.finite(sp) && sp > 0) {
              d <- diff / sp
            }
          }
        }
      }
    }
  }
  
  #Decide suffix for groups
  ng <- if (!is.na(group1) && !is.na(group2)) max(nchar(group1), nchar(group2), na.rm = TRUE) else 0
  
  # Store original group names before potentially replacing them
  orig_group1 <- group1
  orig_group2 <- group2
  show_group_mapping <- FALSE
  
  # Handle one-sample tests separately
  if (is_one_sample) {
    name_1 <- "mean"
    name_2 <- NA_character_
    name_N1 <- "N"
    name_N2 <- NA_character_
  } else if (ng <= 8 && !is.na(group1) && !is.na(group2)) {
    name_1 <- group1
    name_2 <- group2
    name_N1 <- paste0("N(", group1, ")")
    name_N2 <- paste0("N(", group2, ")")
  } else {
    name_1 <- "Group 1"
    name_2 <- "Group 2"
    name_N1 <- "N1"
    name_N2 <- "N2"
    show_group_mapping <- TRUE
  }

  # Build data.frame with dynamic column names
  # Use a list first, then convert to dataframe to allow dynamic column names
  result_list <- list()
  result_list[[name_1]] <- mean1
  
  # For one-sample tests, don't add group2 or diff columns
  if (!is_one_sample) {
    result_list[[name_2]] <- mean2
    
    # Create diff column name as "group1-group2" (e.g., "men-women")
    # If using "Group 1" and "Group 2", use "1-2" instead
    if (!is.na(mean2)) {
      if (name_1 == "Group 1" && name_2 == "Group 2") {
        diff_col_name <- "1-2"
      } else {
        diff_col_name <- paste0(name_1, "-", name_2)
      }
      result_list[[diff_col_name]] <- diff
    }
    
    # Effect size (two-sample independent only)
    if (!isTRUE(is_paired)) {
      result_list$d <- d
    }
  }
  
  result_list$ci <- ci_level
  result_list$ci.L <- ci_L
  result_list$ci.H <- ci_H
  result_list$t <- t_stat
  result_list$df <- df
  result_list$p.value <- p_value
  
  # For paired tests, only show one N column (both sample sizes are the same)
  if (is_paired) {
    result_list$N <- N1
    # Add correlation for paired tests with name "r(var1,var2)"
    corr_col_name <- paste0("r(", name_1, ",", name_2, ")")
    result_list[[corr_col_name]] <- corr_value
  } else if (is_one_sample) {
    # For one-sample tests, only show N (not N1 or N2)
    result_list[[name_N1]] <- N1
  } else {
    result_list[[name_N1]] <- N1
    result_list[[name_N2]] <- N2
  }
  
  # Convert list to dataframe
  # Use check.names = FALSE to preserve spaces in column names (e.g., "N A" instead of "N.A")
  result <- as.data.frame(result_list, stringsAsFactors = FALSE, check.names = FALSE)
  
  # Determine method type (Welch vs Student)
  is_welch <- grepl("Welch", tt_result$method, ignore.case = TRUE)
  method_type <- if (is_welch) "welch" else "student"
  
  # Store attributes for print method
  attr(result, "is_one_sample") <- is_one_sample
  attr(result, "is_paired") <- is_paired
  attr(result, "method_type") <- method_type
  attr(result, "NA1") <- NA1
  attr(result, "NA2") <- NA2
  attr(result, "NA_paired") <- NA_paired
  attr(result, "group1") <- group1
  attr(result, "group2") <- group2
  attr(result, "name_1") <- name_1
  attr(result, "name_2") <- name_2
  attr(result, "name_N1") <- name_N1
  attr(result, "name_N2") <- name_N2
  attr(result, "show_group_mapping") <- show_group_mapping
  if (show_group_mapping) {
    attr(result, "orig_group1") <- orig_group1
    attr(result, "orig_group2") <- orig_group2
  }
  attr(result, "env_warning") <- env_warning_msg
  
  # Add class for print method
  class(result) <- c("t.test2", class(result))
  
  return(result)
}

# Backend equivalence battery: compare statuser outputs to underlying engines.

#t.test2_027
test_that("backend: t.test2 matches stats::t.test across core scenarios", {
  set.seed(101)
  
  x <- rnorm(60, mean = 0.2)
  y <- rnorm(55, mean = 0.5)
  
  scenarios <- list(
    list(label = "two_sample_welch", ref = list(x = x, y = y), args = list()),
    list(label = "two_sample_student", ref = list(x = x, y = y), args = list(var.equal = TRUE)),
    list(label = "paired", ref = list(x = x[1:50], y = y[1:50]), args = list(paired = TRUE)),
    list(label = "one_sample", ref = list(x = x, y = NULL), args = list(mu = 0.1)),
    list(label = "alt_less", ref = list(x = x, y = y), args = list(alternative = "less")),
    list(label = "conf_99", ref = list(x = x, y = y), args = list(conf.level = 0.99))
  )
  
  for (sc in scenarios) {
    if (is.null(sc$ref$y)) {
      tt_ref <- do.call(stats::t.test, c(list(x = sc$ref$x), sc$args))
      tt2 <- do.call(statuser::t.test2, c(list(sc$ref$x), sc$args))
    } else {
      tt_ref <- do.call(stats::t.test, c(list(x = sc$ref$x, y = sc$ref$y), sc$args))
      tt2 <- do.call(statuser::t.test2, c(list(sc$ref$x, sc$ref$y), sc$args))
    }
    
    be_expect_equal_num(tt2$p.value, tt_ref$p.value, tol = 1e-10)
    be_expect_equal_num(tt2$t, as.numeric(tt_ref$statistic), tol = 1e-10)
    be_expect_equal_num(tt2$df, as.numeric(tt_ref$parameter), tol = 1e-10)
    be_expect_equal_num(tt2$ci.L, tt_ref$conf.int[1], tol = 1e-10)
    be_expect_equal_num(tt2$ci.H, tt_ref$conf.int[2], tol = 1e-10)
  }
})

#t.test2_028
test_that("backend: t.test2 formula interface matches stats::t.test", {
  set.seed(102)
  
  n <- 120
  g <- factor(rep(c("A", "B"), each = n / 2), levels = c("A", "B"))
  y <- rnorm(n, mean = ifelse(g == "B", 0.3, 0))
  df <- data.frame(y = y, g = g)
  
  tt_ref <- stats::t.test(y ~ g, data = df)
  tt2 <- statuser::t.test2(y ~ g, data = df)
  
  be_expect_equal_num(tt2$p.value, tt_ref$p.value, tol = 1e-10)
  be_expect_equal_num(tt2$t, as.numeric(tt_ref$statistic), tol = 1e-10)
  be_expect_equal_num(tt2$df, as.numeric(tt_ref$parameter), tol = 1e-10)
  be_expect_equal_num(tt2$ci.L, tt_ref$conf.int[1], tol = 1e-10)
  be_expect_equal_num(tt2$ci.H, tt_ref$conf.int[2], tol = 1e-10)
})

#lm2_051
test_that("backend: lm2 matches estimatr::lm_robust coefficients and inference", {
  testthat::skip_if_not_installed("estimatr")
  
  set.seed(201)
  n <- 300
  x1 <- rnorm(n)
  x2 <- rnorm(n)
  y <- 1 + 0.5 * x1 - 0.2 * x2 + rnorm(n)
  df <- data.frame(y = y, x1 = x1, x2 = x2)
  
  m2 <- statuser::lm2(y ~ x1 + x2, data = df)
  ref <- estimatr::lm_robust(y ~ x1 + x2, data = df, se_type = "HC3")
  
  be_expect_equal_num(unname(m2$coefficients), unname(ref$coefficients), tol = 1e-10)
  be_expect_equal_num(unname(m2$std.error), unname(ref$std.error), tol = 1e-10)
  be_expect_equal_num(unname(m2$statistic), unname(ref$statistic), tol = 1e-10)
  be_expect_equal_num(unname(m2$p.value), unname(ref$p.value), tol = 1e-10)
  be_expect_equal_num(unname(m2$df), unname(ref$df), tol = 1e-10)
})

#interprobe_012
test_that("backend: interprobe JN output matches marginaleffects slopes and regions of significance", {
  testthat::skip_if_not_installed("marginaleffects")
  testthat::skip_if_not_installed("mgcv")
  
  set.seed(301)
  n <- 500
  x <- rnorm(n)
  z <- rnorm(n)
  y <- x * z + rnorm(n)
  df <- data.frame(y = y, x = x, z = z)
  
  m <- stats::lm(y ~ x * z, data = df)
  
  spotlights <- c(-1, 0, 1)
  probe_bins <- 40
  zs <- seq(min(df$z), max(df$z), length.out = probe_bins)
  
  grDevices::pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(grDevices::dev.off(), add = TRUE)
  
  res <- statuser::interprobe(
    model = m,
    x = "x",
    z = "z",
    draw = "jn",
    histogram = FALSE,
    quiet = TRUE,
    spotlights = spotlights,
    probe.bins = probe_bins
  )
  
  jn_ref_list <- lapply(spotlights, function(xj) {
    nd <- data.frame(x = xj, z = zs)
    marginaleffects::slopes(m, newdata = nd, var = "x")
  })
  jn_ref <- do.call(rbind, jn_ref_list)
  jn_ref$x <- rep(spotlights, each = probe_bins)
  
  jn_stat <- res$johnson.neyman
  
  jn_ref <- jn_ref[order(jn_ref$x, jn_ref$z), ]
  jn_stat <- jn_stat[order(jn_stat$x, jn_stat$z), ]
  
  testthat::expect_equal(nrow(jn_ref), nrow(jn_stat))
  
  be_expect_equal_num(jn_stat$marginal.effect, jn_ref$estimate, tol = 1e-8)
  be_expect_equal_num(jn_stat$conf.low, jn_ref$conf.low, tol = 1e-8)
  be_expect_equal_num(jn_stat$conf.high, jn_ref$conf.high, tol = 1e-8)
  
  sig_ref <- be_sig_from_ci(jn_ref$conf.low, jn_ref$conf.high)
  sig_stat <- be_sig_from_ci(jn_stat$conf.low, jn_stat$conf.high)
  testthat::expect_identical(sig_stat, sig_ref)
})

#interprobe_013
test_that("backend: interprobe internal interaction test p-values match returned testing models (binary x)", {
  testthat::skip_if_not_installed("marginaleffects")
  testthat::skip_if_not_installed("mgcv")
  
  set.seed(302)
  n <- 240
  z <- rnorm(n)
  x <- rep(c(0, 1), each = n / 2)
  y <- 0.2 + 0.5 * x + 0.8 * z + 0.9 * x * z + rnorm(n)
  
  grDevices::pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(grDevices::dev.off(), add = TRUE)
  
  out <- capture.output(
    res <- statuser::interprobe(
      x, z, y,
      quiet = FALSE,
      draw = "jn",
      histogram = FALSE,
      spotlights = c(-1, 0, 1),
      probe.bins = 30
    )
  )
  
  testthat::expect_true(is.list(res))
  testthat::expect_true("gam_results_testing" %in% names(res))
  
  gam_p <- NA_real_
  sm <- summary(res$gam_results_testing)
  st <- sm$s.table
  rn <- rownames(st)
  if (!is.null(rn)) {
    hits <- which(
      grepl("ti(", rn, fixed = TRUE) &
        grepl("x", rn, fixed = TRUE) &
        grepl("z", rn, fixed = TRUE)
    )
    if (length(hits) > 0) {
      gam_p <- as.numeric(st[hits[1], "p-value"])
    }
  }
  
  gam_line <- out[grepl("^\\s*GAM:", out)]
  gam_line <- if (length(gam_line) > 0) gam_line[1] else NA_character_
  gam_p_reported <- be_parse_p_from_apa_line(gam_line)
  
  # If output was of the form p < .001, we parse as 0.001 and only check the direction.
  if (is.finite(gam_p_reported) && identical(gam_p_reported, 0.001)) {
    testthat::expect_true(is.finite(gam_p))
    testthat::expect_true(gam_p < 0.001)
  } else {
    be_expect_equal_num(gam_p_reported, gam_p, tol = 1e-10)
  }
  
  # Linear interaction p-value: use lm2_results if available, otherwise compute from lm on the same data.
  lin_line <- out[grepl("^\\s*linear model:", out)]
  lin_line <- if (length(lin_line) > 0) lin_line[1] else NA_character_
  lin_p_reported <- be_parse_p_from_apa_line(lin_line)
  
  lin_p_ref <- NA_real_
  if ("lm2_results" %in% names(res) && !is.null(res$lm2_results)) {
    tbl <- attr(res$lm2_results, "statuser_table")
    if (!is.null(tbl) && "term" %in% names(tbl) && "p.value" %in% names(tbl)) {
      idx2 <- match(c("x:z", "z:x"), tbl$term)
      idx2 <- idx2[!is.na(idx2)]
      if (length(idx2) > 0) lin_p_ref <- as.numeric(tbl$p.value[idx2[1]])
    }
  }
  if (!is.finite(lin_p_ref)) {
    df <- data.frame(y = y, x = x, z = z)
    m <- stats::lm(y ~ x * z, data = df)
    lin_p_ref <- be_extract_linear_interaction_p(m, "x", "z")
  }
  
  if (is.finite(lin_p_reported) && identical(lin_p_reported, 0.001)) {
    testthat::expect_true(is.finite(lin_p_ref))
    testthat::expect_true(lin_p_ref < 0.001)
  } else {
    be_expect_equal_num(lin_p_reported, lin_p_ref, tol = 1e-10)
  }
})

# Tests for t.test2() function

# ============================================================================
# BASIC FUNCTIONALITY
# ============================================================================

#t.test2_001
test_that("t.test2 is exported and returns correct class", {
  x <- rnorm(50)
  y <- rnorm(50)
  result <- t.test2(x, y)
  
  expect_s3_class(result, "t.test2")
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 1)
})

#t.test2_002
test_that("t.test2 returns all expected columns", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  result <- t.test2(y ~ group)
  
  # Essential columns

  expect_true("p.value" %in% names(result))
  expect_true("t" %in% names(result))
  expect_true("df" %in% names(result))
  expect_true("ci.L" %in% names(result))
  expect_true("ci.H" %in% names(result))
  expect_true("d" %in% names(result))
  
  # Attributes
  expect_true(!is.null(attr(result, "method_type")))
  expect_true(attr(result, "method_type") %in% c("student", "welch"))
})

# ============================================================================
# INPUT SYNTAX VARIATIONS
# ============================================================================

#t.test2_003
test_that("t.test2 handles formula syntax with and without data", {
  # With data argument
  df <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
  result1 <- t.test2(value ~ group, data = df)
  expect_true("A" %in% names(result1))
  expect_true("B" %in% names(result1))
  

  # Without data argument (variables in environment)
  value <- rnorm(100)
  group <- rep(c("X", "Y"), 50)
  result2 <- t.test2(value ~ group)
  expect_true("X" %in% names(result2))
  expect_true("Y" %in% names(result2))
})

#t.test2_004
test_that("t.test2 accepts df$col ~ df$col2 without bare column names in env", {
  df1 <- data.frame(y1 = rnorm(100), x1 = rep(c("A", "B"), 50))
  expect_error(t.test2(df1$y1 ~ df1$x1), NA)
  result <- t.test2(df1$y1 ~ df1$x1)
  expect_s3_class(result, "t.test2")
  expect_true("A" %in% names(result))
  expect_true("B" %in% names(result))
})

#t.test2_006
test_that("t.test2 errors if df$y ~ df$g is combined with data=", {
  df1 <- data.frame(y1 = rnorm(100), x1 = rep(c("A", "B"), 50))
  expect_error(t.test2(df1$y1 ~ df1$x1, data = df1), "Do not combine", fixed = TRUE)
})

#t.test2_005
test_that("t.test2 handles two-vector syntax", {
  x1 <- rnorm(50, mean = 5)
  x2 <- rnorm(50, mean = 4.8)
  
  result <- t.test2(x1, x2)
  
  expect_true("x1" %in% names(result))
  expect_true("x2" %in% names(result))
  expect_true("x1-x2" %in% names(result))
  expect_true("d" %in% names(result))
  expect_true("N(x1)" %in% names(result))
  expect_true("N(y2)" %in% names(result) || "N(x2)" %in% names(result))
})

#t.test2_006
test_that("t.test2 handles paired tests", {
  set.seed(12)
  y1 <- rnorm(100)
  y2 <- y1 + rnorm(100, sd = 0.5)  # Correlated
  
  result <- t.test2(y1, y2, paired = TRUE)
  
  expect_true(attr(result, "is_paired"))
  expect_true("N" %in% names(result))  # Single N for paired
  expect_true("r(y1,y2)" %in% names(result))  # Correlation
  expect_false("d" %in% names(result))
})

#t.test2_007
test_that("t.test2 handles one-sample test", {
  x <- rnorm(50, mean = 5)
  
  result <- t.test2(x, mu = 5)
  
  expect_true("mean" %in% names(result))
  expect_true("N" %in% names(result))
  expect_false("Group 1" %in% names(result))
  expect_false(any(grepl("-", names(result), fixed = TRUE)))  # No diff column
  expect_false("d" %in% names(result))
  
  output <- capture.output(print(result))
  expect_true(any(grepl("One sample", output)))
})

# ============================================================================
# COLUMN NAMING CONVENTIONS
# ============================================================================

#t.test2_008
test_that("t.test2 uses group values as column names for short names", {
  set.seed(12)
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  result <- t.test2(y ~ group)
  
  expect_true("A" %in% names(result))
  expect_true("B" %in% names(result))
  expect_true("A-B" %in% names(result))
  expect_true("N(A)" %in% names(result))
  expect_true("N(B)" %in% names(result))
})

#t.test2_009
test_that("t.test2 uses Group 1/2 format for long group names", {
  set.seed(12)
  y <- rnorm(100)
  group <- rep(c("Low construal", "High construal"), 50)
  
  result <- t.test2(y ~ group)
  
  expect_true("Group 1" %in% names(result))
  expect_true("Group 2" %in% names(result))
  expect_true("1-2" %in% names(result))
  expect_true("N1" %in% names(result))
  expect_true("N2" %in% names(result))
  
  # Print should show group mapping
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  expect_true(grepl("Group 1:", output_text))
  expect_true(grepl("Group 2:", output_text))
})

#t.test2_010
test_that("t.test2 formats 0/1 grouping as varname=value", {
  set.seed(12)
  y <- rnorm(100)
  x4 <- rep(0:1, 50)
  
  result <- t.test2(y ~ x4)
  
  expect_true("x4=0" %in% names(result))
  expect_true("x4=1" %in% names(result))
  expect_true("x4=0-x4=1" %in% names(result))
  expect_true("N(x4=0)" %in% names(result))
  expect_true("N(x4=1)" %in% names(result))
})

#t.test2_011
test_that("t.test2 uses Group 1/2 for long variable names with 0/1", {
  set.seed(12)
  y <- rnorm(100)
  xlongversion <- rep(0:1, 50)
  
  result <- t.test2(y ~ xlongversion)
  
  expect_true("Group 1" %in% names(result))
  expect_true("Group 2" %in% names(result))
  
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  expect_true(grepl("xlongversion=0", output_text))
  expect_true(grepl("xlongversion=1", output_text))
})

# ============================================================================
# MISSING DATA HANDLING
# ============================================================================

#t.test2_012
test_that("t.test2 reports missing data for two-sample tests", {
  set.seed(12)
  y <- rnorm(100)
  y[c(10, 20, 60)] <- NA  # Missing in both groups
  group <- rep(c("A", "B"), 50)
  
  result <- t.test2(y ~ group)
  
  expect_true(attr(result, "NA1") > 0 || attr(result, "NA2") > 0)
  
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  expect_true(grepl("missing", output_text, ignore.case = TRUE))
})

#t.test2_012b
test_that("t.test2 does not double-count missing when group has NA", {
  set.seed(12)
  y <- rnorm(20)
  group <- rep(0:1, each = 10)
  
  # One missing value in each group
  y[2] <- NA
  y[12] <- NA
  
  # Additional rows with missing group should not be attributed to either group
  y <- c(y, 10, 11, 12)
  group <- c(group, NA, NA, NA)
  
  result <- t.test2(y ~ group)
  
  expect_equal(attr(result, "NA1"), 1L)
  expect_equal(attr(result, "NA2"), 1L)
})

#t.test2_013
test_that("t.test2 reports missing data for paired tests", {
  set.seed(12)
  y1 <- rnorm(100)
  y2 <- rnorm(100)
  y1[c(50, 60)] <- NA
  
  result <- t.test2(y1, y2, paired = TRUE)
  
  expect_true(attr(result, "NA_paired") > 0)
  
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  expect_true(grepl("pairs were dropped due to missing values", output_text))
})

# ============================================================================
# CONSISTENCY WITH BASE t.test()
# ============================================================================

#t.test2_014
test_that("t.test2 produces identical results to t.test", {
  set.seed(100)
  x <- rnorm(50, mean = 5)
  y <- rnorm(50, mean = 5.5)
  
  tt1 <- t.test(x, y)
  tt2 <- t.test2(x, y)
  
  expect_equal(tt2$p.value, tt1$p.value, tolerance = 1e-10)
  expect_equal(tt2$t, as.numeric(tt1$statistic), tolerance = 1e-10)
  expect_equal(tt2$df, as.numeric(tt1$parameter), tolerance = 1e-10)
  expect_equal(tt2$ci.L, tt1$conf.int[1], tolerance = 1e-10)
  expect_equal(tt2$ci.H, tt1$conf.int[2], tolerance = 1e-10)
})

#t.test2_015
test_that("t.test2 matches t.test for paired and one-sample", {
  set.seed(104)
  
  # Paired
  x <- rnorm(30, mean = 10)
  y <- x + rnorm(30, mean = 0.5, sd = 0.5)
  
  tt1_paired <- t.test(x, y, paired = TRUE)
  tt2_paired <- t.test2(x, y, paired = TRUE)
  expect_equal(tt2_paired$p.value, tt1_paired$p.value, tolerance = 1e-10)
  
  # One-sample
  z <- rnorm(50, mean = 5)
  tt1_one <- t.test(z, mu = 5)
  tt2_one <- t.test2(z, mu = 5)
  expect_equal(tt2_one$p.value, tt1_one$p.value, tolerance = 1e-10)
})

#t.test2_016
test_that("t.test2 matches t.test with formula syntax", {
  df <- data.frame(
    value = c(rnorm(30, mean = 5), rnorm(30, mean = 5.5)),
    group = rep(c("A", "B"), each = 30)
  )
  
  tt1 <- t.test(value ~ group, data = df)
  tt2 <- t.test2(value ~ group, data = df)
  
  expect_equal(tt2$p.value, tt1$p.value, tolerance = 1e-10)
})

# ============================================================================
# EDGE CASES
# ============================================================================

#t.test2_017
test_that("t.test2 handles small samples", {
  # Very small (n=3)
  result1 <- t.test2(c(1, 2, 3), c(4, 5, 6))
  expect_s3_class(result1, "t.test2")
  expect_true(!is.na(result1$p.value))
  
  # Minimum (n=2)
  result2 <- t.test2(c(1, 2), c(3, 4))
  expect_s3_class(result2, "t.test2")
  expect_true(!is.na(result2$p.value))
})

#t.test2_018
test_that("t.test2 handles extreme cases", {
  set.seed(107)
  
  # Equal means (p should be large)
  x_eq <- rnorm(100, mean = 5)
  y_eq <- rnorm(100, mean = 5)
  result_eq <- t.test2(x_eq, y_eq)
  expect_true(result_eq$p.value > 0.01)
  
  # Very different means (p should be small)
  x_diff <- rnorm(50, mean = 0)
  y_diff <- rnorm(50, mean = 10)
  result_diff <- t.test2(x_diff, y_diff)
  expect_true(result_diff$p.value < 0.001)
  
  # Zero variance in one group
  x_zero <- rep(5, 10)
  y_var <- rnorm(10, mean = 6)
  result_zero <- t.test2(x_zero, y_var)
  expect_s3_class(result_zero, "t.test2")
})

#t.test2_019
test_that("t.test2 handles different grouping variable types", {
  set.seed(108)
  y <- rnorm(40)
  
  # Factor
  result_factor <- t.test2(y ~ factor(rep(c("A", "B"), 20)))
  expect_s3_class(result_factor, "t.test2")
  
  # Logical
  result_logical <- t.test2(y ~ rep(c(TRUE, FALSE), 20))
  expect_s3_class(result_logical, "t.test2")
  
  # Numeric 0/1
  result_numeric <- t.test2(y ~ rep(0:1, 20))
  expect_s3_class(result_numeric, "t.test2")
})

# ============================================================================
# REGRESSION TESTS
# ============================================================================

#t.test2_020
test_that("t.test2 uses environment variables when data= not specified", {
  # Test that t.test2 can use environment variables without data=
  set.seed(12)
  n <- 100
  
  # Variables in environment
  y_env <- round(rnorm(n * 2, mean = 100, sd = 4), 0)
  group_env <- rep(c("A", "B"), n)
  
  # Call without data= argument to use environment variables
  result <- t.test2(y_env ~ group_env)
  
  # Should use environment variables (A/B groups)
  expect_true("A" %in% names(result))
  expect_true("B" %in% names(result))
  
  # Means should be ~100 (from environment)
  expect_true(result[["A"]] > 90)
  expect_true(result[["B"]] > 90)
})

# ============================================================================
# SNAPSHOT TESTS FOR OUTPUT FORMAT
# ============================================================================

#t.test2_021
test_that("t.test2 print output for two-sample test is stable", {
  # Use fixed seed for reproducible output
  set.seed(42)
  x <- round(rnorm(30, mean = 100, sd = 15), 1)
  y <- round(rnorm(30, mean = 105, sd = 15), 1)
  
  result <- t.test2(x, y)
  
  expect_snapshot(print(result))
})

#t.test2_022
test_that("t.test2 print output for formula syntax is stable", {
  set.seed(42)
  df <- data.frame(
    value = round(c(rnorm(25, mean = 50), rnorm(25, mean = 55)), 1),
    group = rep(c("Control", "Treatment"), each = 25)
  )
  
  result <- t.test2(value ~ group, data = df)
  
  expect_snapshot(print(result))
})

#t.test2_023
test_that("t.test2 print output for paired test is stable", {
  set.seed(42)
  before <- round(rnorm(20, mean = 100, sd = 10), 1)
  after <- round(before + rnorm(20, mean = 5, sd = 3), 1)
  
  result <- t.test2(before, after, paired = TRUE)
  
  expect_snapshot(print(result))
})

#t.test2_024
test_that("t.test2 print output for one-sample test is stable", {
  set.seed(42)
  x <- round(rnorm(30, mean = 100, sd = 15), 1)
  
  result <- t.test2(x, mu = 100)
  
  expect_snapshot(print(result))
})

#t.test2_025
test_that("t.test2 print output with missing data is stable", {
  set.seed(42)
  x <- round(rnorm(30, mean = 100, sd = 15), 1)
  y <- round(rnorm(30, mean = 105, sd = 15), 1)
  x[c(5, 10, 15)] <- NA  # Add missing values
  
  result <- t.test2(x, y)
  
  expect_snapshot(print(result))
})

#t.test2_026
test_that("t.test2 print output with long group names is stable", {
  set.seed(42)
  df <- data.frame(
    score = round(rnorm(40, mean = 75, sd = 10), 1),
    condition = rep(c("Experimental Group A", "Control Group B"), each = 20)
  )
  
  result <- t.test2(score ~ condition, data = df)
  
  expect_snapshot(print(result))
})

# Two-line test (same as https://webstimate.org/twolines 0.52)
# Last update 2025 12 23
# Written by Uri Simonsohn (urisohn@gmail.com)
#


#
# FUNCTIONS DEFINED IN THIS SCRIPT:
# 1. twolines() - Main function: Tests for u-shaped relationships using two-line regression
# 2. reg2() - Helper function: Performs interrupted regression with robust standard errors


#' Two-Lines Test of U-Shapes
#'
#' Implements the two-lines test for U-shaped (or inverted U-shaped) relationships 
#'introduced by Simonsohn (2018).
#'
#' Reference: 
#' Simonsohn, Uri (2018) "Two lines: A valid alternative to the invalid testing of 
#' U-shaped relationships with quadratic regressions." AMPPS, 538-555. 
#' \doi{10.1177/2515245918805755}
#'
#' @param f A formula object specifying the model (e.g., y ~ x1 + x2 + x3).
#'   The first predictor is the one tested for a u-shaped relationship.
#' @param graph Integer. If 1 (default), produces a plot. If 0, no plot is generated.
#' @param link Character string specifying the link function for the GAM model.
#'   Default is "gaussian".
#' @param data An optional data frame containing the variables in the formula.
#'   If not provided, variables are evaluated from the calling environment.
#' @param pngfile Optional character string. If provided, saves the plot to a PNG file
#'   with the specified filename.
#' @param quiet Logical. If TRUE, suppresses the Robin Hood details messages.
#'   Default is FALSE.
#'
#' @return A list containing:
#' \itemize{
#'   \item All elements from \code{reg2()}: \code{b1}, \code{b2}, \code{p1}, \code{p2},
#'     \code{z1}, \code{z2}, \code{u.sig}, \code{xc}, \code{glm1}, \code{glm2}, \code{rob1},
#'     \code{rob2}, \code{msg}, \code{yhat.smooth}
#'   \item \code{yobs}: Observed y values (adjusted for covariates if present)
#'   \item \code{y.hat}: Fitted values from GAM
#'   \item \code{y.ub}, \code{y.lb}: Upper and lower bounds for fitted values
#'   \item \code{y.most}: Most extreme fitted value
#'   \item \code{x.most}: x-value associated with most extreme fitted value
#'   \item \code{f}: Formula as character string
#'   \item \code{bx1}, \code{bx2}: Linear and quadratic coefficients from preliminary quadratic regression
#'   \item \code{minx}: Minimum x value
#'   \item \code{midflat}: Median of flat region
#'   \item \code{midz1}, \code{midz2}: Z-statistics at midpoint
#' }
#'
#' @details
#' The test beings fitting a GAM model, predicting y with a smooth of x, and optionally with covariates. 
#' It identifies the interior most extreme value of fitted y, and adjusts from the matching x-value 
#' to set the breakpoint relying on the Robin Hood procedure introduced also by Simonsohn (2018).
#' It then estimates the (once) interrupted regression using that breakpoint,
#' and reports the slope and significance of the average slopes at either side of it. A U-shape 
#' is significant if the slopes are of opposite sign and are both individually significant.
#' 

#'
#' @examples
#' \donttest{
#' # Simple example with simulated data
#' set.seed(123)
#' x <- rnorm(100)
#' y <- -x^2 + rnorm(100)
#' data <- data.frame(x = x, y = y)
#' result <- twolines(y ~ x, data = data)
#'
#' # With covariates
#' z <- rnorm(100)
#' y <- -x^2 + 0.5*z + rnorm(100)
#' data <- data.frame(x = x, y = y, z = z)
#' result <- twolines(y ~ x + z, data = data)
#' 
#' # Without data argument (variables evaluated from environment)
#' x <- rnorm(100)
#' y <- -x^2 + rnorm(100)
#' result <- twolines(y ~ x)
#' }
#'
#' @importFrom mgcv gam
#' @export
twolines <- function(f, graph = 1, link = "gaussian", data = NULL, pngfile = "", quiet = FALSE) {
  #OUTLINE
  #1. Validate inputs
  #2. Extract variable names from formula
  #3. Drop missing values
  #4. Grab key variables
  #5. Set up GAM formula for smoothing
  #6. Run GAM smoother
  #7. Generate yobs (adjusted for covariates if present)
  #8. Get fitted values and standard errors from GAM
  #9. Determine shape (u-shaped vs inverted u-shaped) using quadratic regression
  #10. Find middle 80% of data and restrict analysis to that range
  #11. Find most extreme fitted value and flat regions
  #12. Run preliminary two-line regression at midpoint
  #13. Adjust breakpoint using Robin Hood procedure
  #14. Run final two-line regression
  #15. Combine and return results
  
  #1. Validate inputs and handle data
    # Get calling environment for evaluating variables
    calling_env <- parent.frame()

    # If formula uses `$`, do not allow data= (use either one style or the other).
    .assert_no_dollar_with_data(f, data, func_name = "twolines")

    # If formula uses `$` and data is NULL, normalize into a clean data+formula pair.
    if (is.null(data) && .formula_contains_dollar(f)) {
      normalized <- .normalize_dollar_formula_to_data_and_formula(
        formula = f,
        calling_env = calling_env,
        func_name = "twolines"
      )
      f <- normalized$formula
      data <- normalized$data
    }
    
    # Extract variable names from formula
    vars <- all.vars(f)
    y.f <- vars[1]  # DV
    x.f <- vars[2]  # Variable on which the u-shape shall be tested
    
    # Number of variables
    var.count <- length(vars)  # How many predictors in addition to the key predictor?
    
    # If data is NULL, create it from the calling environment
    if (is.null(data)) {
      # Check if variables exist before evaluating
      missing_vars <- c()
      for (var in vars) {
        if (!exists(var, envir = calling_env, inherits = TRUE)) {
          missing_vars <- c(missing_vars, var)
        }
      }
      
      if (length(missing_vars) > 0) {
        if (length(missing_vars) == 1) {
          stop(sprintf("twolines(): Could not find variable '%s'", missing_vars[1]), call. = FALSE)
        } else {
          stop(sprintf("twolines(): Could not find variables: %s", paste(missing_vars, collapse = ", ")), call. = FALSE)
        }
      }
      
      # Variables exist, create data frame from environment
      # Create a list first, then convert to data frame
      data_list <- list()
      for (var in vars) {
        data_list[[var]] <- eval(as.name(var), envir = calling_env)
      }
      data <- as.data.frame(data_list)
    } else {
      # Data provided: validate it's a data frame
      if (!is.data.frame(data)) {
        stop("twolines(): 'data' must be a data frame", call. = FALSE)
      }
    }
    
    # Entire model, except the first predictor
    if (var.count > 2) {
      nox.f <- drop.terms(terms(f), dropx = 1, keep.response = TRUE)
    }
  
  #3. Drop missing values
    # Vector with columns associated with those variable names in the dataset
    cols <- c()
    for (var in vars) {
      if (!var %in% names(data)) {
        stop(sprintf("twolines(): Variable '%s' not found in dataset", var), call. = FALSE)
      }
      cols <- c(cols, which(names(data) == var))
    }
    
    # Set of complete observations
    full.rows <- complete.cases(data[, cols, drop = FALSE])
    
    # Drop missing rows
    data <- data[full.rows, , drop = FALSE]
  
  #4. Grab key variables
    xu <- data[[x.f]]  # xu is the key predictor predicted to be u-shaped
    yu <- data[[y.f]]  # yu is the dv
  
  #5. Set up GAM formula for smoothing
    # Count number of unique x values
    unique.x <- length(unique(xu))   # How many unique values x has
    
    # New function segment for x
    sx.f <- paste0("s(", x.f, ",bs='cr', k=min(10,", unique.x, "))")
  
  #6. Run GAM smoother
    # Define the formula to be run based on whether there are covariates
    if (var.count > 2) {
      gam.f <- paste0(format(nox.f), "+", sx.f)      # with covariates
    }
    if (var.count == 2) {
      gam.f <- paste0(y.f, " ~", sx.f)                  # without
    }
    
    # Now run it
    gams <- mgcv::gam(as.formula(gam.f), data = data, family = link)  # so this is a general additive model with the main specification entered
    # but we make the first predictor, the one that will be tested for having a u-shaped effect
    # be estimated with a completely flexible functional form.
  
  #7. Generate yobs (adjusted for covariates if present)
    # If no covariates, yobs is the actually observed data
    if (var.count == 2) {
      yobs <- yu
    }
    
    # If covariates present, yobs is the fitted value with u(x) at mean, need new.data() with variables at means
    if (var.count > 2) {
      # Put observed data into data frame
      data.obs <- data[, all.vars(f), drop = FALSE]
      
      # Drop observations with missing values on any of the variables
      data.obs <- na.omit(data.obs)
      
      # Create data where xu is at sample means to get residuals based on rest of models to act as yobs
      data.xufixed <- data.obs
      data.xufixed[[x.f]] <- mean(data.obs[[x.f]])   # Note, the 1st predictor is always the one hypothesized to be u-shaped
      # replace it with the mean value of the predictor
      
      # Get yobs with covariates
      # First the fitted value
      yhat.xufixed <- mgcv::predict.gam(gams, newdata = data.xufixed)        # get fitted values at means for covariates
      
      # Subtract fitted value from observed y
      yobs <- yu - yhat.xufixed
      
      # Create data where u(x) is obs, and all else at sample means
      data.otherfixed <- data.obs     # start with original value
      
      # Replace all covariates with their mean for fitting data at sample means
      for (i in 3:var.count) {
        var.name <- all.vars(f)[i]
        data.otherfixed[[var.name]] <- mean(data.obs[[var.name]])
      }
    }  # End if covariates are present to compute yobs
  
  #8. Get fitted values and standard errors from GAM
    # Get predicted values into list
    if (var.count > 2) {
      g.fit <- mgcv::predict.gam(gams, newdata = data.otherfixed, se.fit = TRUE)  # predict with covariates at means
    }
    if (var.count == 2) {
      g.fit <- mgcv::predict.gam(gams, se.fit = TRUE)
    }
    
    # Take out the fitted itself
    y.hat <- g.fit$fit
    
    # Now the SE
    y.se <- g.fit$se.fit
  
  #9. Determine shape (u-shaped vs inverted u-shaped) using quadratic regression
    # Determine if function is at first decreasing (potential u-shape) vs. increasing (potentially inverted U)
    # using quadratic regression to know if we are looking for max or min
    xu2 <- xu^2                                                  # Square x term, xu is the 1st predictor
    # Create data frame for quadratic regression
    qdata <- data.frame(xu = xu, xu2 = xu2)
    qdata[[y.f]] <- yu
    if (var.count > 2) {
      # Add covariates
      for (var in all.vars(f)[-(1:2)]) {
        qdata[[var]] <- data[[var]]
      }
      lmq.f <- update(nox.f, ~ xu + xu2 + .)           # Add to function with covariates (put first, before covariates)
    }
    if (var.count == 2) {
      lmq.f <- as.formula(paste0(y.f, " ~ xu + xu2"))
    }
    lmq <- lm(lmq.f, data = qdata)            # Estimate the quadratic regression
    bqs <- lmq$coefficients                   # Get the point estimates
    bx1 <- bqs[2]                             # point estimate for effect of x
    bx2 <- bqs[3]                             # point estimate for effect of x^2
    x0 <- min(xu)                             # lowest x-value
    s0 <- bx1 + 2 * bx2 * x0                  # estimated slope at the lowest x-value
    if (s0 > 0) {
      shape <- 'inv-ushape'                   # if the quadratic is increasing at the lowest point, the could be inverted u-shape
    }
    if (s0 <= 0) {
      shape <- 'ushape'                       # if it is decreasing, then it could be a regular u-shape
    }
  
  #10. Find middle 80% of data and restrict analysis to that range
    # Get the middle 80% of data to avoid an extreme cutoff
    x10 <- quantile(xu, .1)
    x90 <- quantile(xu, .9)
    middle <- (xu > x10 & xu < x90)       # Don't consider extreme values for cutoff
    x.middle <- xu[middle]
    
    # Restrict y.hat to middle
    y.hat <- y.hat[middle]
    y.se <- y.se[middle]
  
  #11. Find most extreme fitted value and flat regions
    # Find upper and lower band
    y.ub <- y.hat + y.se            # +SE is for flat max
    y.lb <- y.hat - y.se            # -SE is for flat min
    
    # Find most extreme y-hat
    if (shape == 'inv-ushape') {
      y.most <- max(y.hat)   # if potentially inverted u-shape, use the highest y-hat as the most extreme
    }
    if (shape == 'ushape') {
      y.most <- min(y.hat)   # if potential u-shaped, then the lowest instead
    }
    
    # x-value associated with the most extreme value
    x.most <- x.middle[match(y.most, y.hat)]
    
    # Find flat regions
    if (shape == 'inv-ushape') {
      flat <- (y.ub > y.most)
    }
    if (shape == 'ushape') {
      flat <- (y.lb < y.most)
    }
    xflat <- x.middle[flat]
  
  #12. Run preliminary two-line regression at midpoint
    # First an interrupted regression at the midpoint of the flat region
    rmid <- reg2(f, xc = median(xflat), graph = 0, data = data)  # Two line regression at the median point of flat maximum
    
    # Get z1 and z2, statistical strength of both lines at the midpoint
    z1 <- abs(rmid$z1)
    z2 <- abs(rmid$z2)
  
  #13. Adjust breakpoint using Robin Hood procedure
    # Adjust breakpoint based on z1, z2
    xc <- quantile(xflat, z2 / (z1 + z2))
    
    # Print Robin Hood details (unless quiet = TRUE)
    if (!quiet) {
      xcn <- round(as.numeric(xc), 2)
      qp <- z2 / (z1 + z2)
      
      # Header message with font=2
      message2("Robin Hood calculations (see Simonsohn, 2018)", col = "blue4", font = 2)
      
      # Combine all details into a single message
          msg <- paste0(
            "Explaining why ", x.f, " = ", xcn, " is used as the breakpoint\n\n",
            "Most extreme value of fitted '", y.f, "' with GAM obtained at '", x.f, "' = ", round(x.most, 2), "\n",
            "Local range of values considered for breakpoint '", x.f, "': [", round(min(xflat), 2), ", ", round(max(xflat), 2), "]\n",
            "t-values for two lines at '", x.f, "' = ", round(x.most, 2), ":\n",
            "   t1 = ", round(z1, 2), "\n",
            "   t2 = ", round(z2, 2), "\n",
            "We compute t2/(t1+t2) = ", round(qp, 2), "\n",
            "We then lookup the value of '", x.f, "' at that quantile, so we run\n",
            "quantile(", x.f, ", ", round(qp, 2), ") = ", round(as.numeric(xc), 2), 
            " and set that value, ", round(as.numeric(xc), 2), ", as the breakpoint."
          )
          message2(msg, col = "blue4")
      
      message2("\nNote: you may turn off this message by setting `quiet=TRUE`", col = "red2")
      
    }
  
  #14. Run final two-line regression
    # Save to png? (option set at the beginning by giving png a name)
    if (pngfile != "") {
      png(pngfile, width = 2000, height = 1500, res = 300)
    }
    
    # Run the two lines
    res <- reg2(f, xc = xc, graph = graph, data = data)
    
    # Save to png? (close)
    if (pngfile != "") {
      dev.off()
    }
  
  #15. Combine and return results
    # Add other results obtained before to the output (some of these are read by the server and included in the app)
    res$yobs <- yobs
    res$y.hat <- y.hat
    res$y.ub <- y.ub
    res$y.lb <- y.lb
    res$y.most <- y.most
    res$x.most <- x.most
    res$f <- format(f)
    res$bx1 <- bx1           # linear effect in quadratic regression
    res$bx2 <- bx2           # quadratic
    res$minx <- min(xu)       # lowest x value
    res$midflat <- median(xflat)
    res$midz1 <- abs(rmid$z1)
    res$midz2 <- abs(rmid$z2)
    
    invisible(res)
}  # End function


#' Two-Line Interrupted Regression
#'
#' Performs an interrupted regression analysis with heteroskedastic robust standard errors.
#' This function fits two regression lines with a breakpoint at a specified value of the
#' first predictor variable.
#'
#' @param f A formula object specifying the model (e.g., y ~ x1 + x2 + x3).
#'   The first predictor is the one on which the u-shape is tested.
#' @param xc Numeric value specifying where to set the breakpoint.
#' @param graph Integer. If 1 (default), produces a plot. If 0, no plot is generated.
#' @param family Character string specifying the family for the GLM model.
#'   Default is "gaussian" for OLS regression. Use "binomial" for probit models.
#' @param data A data frame containing the variables in the formula.
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{b1}, \code{b2}: Slopes of the two regression lines
#'   \item \code{p1}, \code{p2}: P-values for the slopes
#'   \item \code{z1}, \code{z2}: Z-statistics for the slopes
#'   \item \code{u.sig}: Indicator (0/1) for whether u-shape is significant
#'   \item \code{xc}: The breakpoint value
#'   \item \code{glm1}, \code{glm2}: The fitted GLM models
#'   \item \code{rob1}, \code{rob2}: Robust coefficient test results
#'   \item \code{msg}: Messages about standard error computation
#'   \item \code{yhat.smooth}: Fitted smooth values (if graph=1)
#' }
#'
#' @details
#' This function fits two interrupted regression lines with heteroskedastic robust
#' standard errors using the sandwich package. The first predictor variable is split
#' at the breakpoint \code{xc}, creating separate slopes before and after the breakpoint.
#'
#' @importFrom mgcv gam
#' @importFrom sandwich vcovHC
#' @importFrom lmtest coeftest
#' @keywords internal
reg2 <- function(f, xc, graph = 1, family = "gaussian", data = NULL) {
  #OUTLINE
  #1. Validate data
  #2. Extract variable names from formula
  #3. Grab key variables
  
  #1. Validate data
    if (is.null(data)) {
      stop("reg2(): 'data' argument is required", call. = FALSE)
    }
    if (!is.data.frame(data)) {
      stop("reg2(): 'data' must be a data frame", call. = FALSE)
    }
  
  #2. Extract variable names from formula
  #3. Set up GAM formula for smoothing (accommodates discrete values)
  #4. Create interrupted regression variables (xlow1, xhigh1, high1, xlow2, xhigh2, high2)
  #5. Generate formulas for two regression lines
  #6. Run GLM regressions
  #7. Compute robust standard errors (HC3, fallback to HC1 if needed)
  #8. Extract slopes, z-statistics, and p-values
  #9. Determine if u-shape is significant
  #10. Plot results (if graph=1)
  #11. Return results
  
  #1. Extract variable names from formula
    y.f <- all.vars(f)[1]  # DV
    x.f <- all.vars(f)[2]  # Variable on which the u-shape shall be tested
    
    # Number of variables
    var.count <- length(all.vars(f))  # How many total variables, including y and x
    
    # Entire model, except the first predictor
    if (var.count > 2) {
      nox.f <- drop.terms(terms(f), dropx = 1, keep.response = TRUE)
    }
  
  #2. Grab key variables
    xu <- data[[x.f]]  # xu is the key predictor predicted to be u-shaped
    yu <- data[[y.f]]  # yu is the dv
  
  #3. Set up GAM formula for smoothing (accommodates discrete values)
    # Replace formula for key predictor so that it accommodates possibly discrete values
    # gam() breaks down if x has few possible values unless one restricts it, done automatically
    unique.x <- length(unique(xu))  # How many unique values x has
    sx.f <- paste0("s(", x.f, ",bs='cr', k=min(10,", unique.x, "))")
  
  #4. Create interrupted regression variables
    # xc is included in the first line
    xlow1 <- ifelse(xu <= xc, xu - xc, 0)   # xlow = x - xc when x < xc, 0 otherwise
    xhigh1 <- ifelse(xu > xc, xu - xc, 0)    # xhigh = x - xc when x > xc, 0 otherwise
    high1 <- ifelse(xu > xc, 1, 0)           # high dummy, allows interruption
    
    # Now include xc in second line
    xlow2 <- ifelse(xu < xc, xu - xc, 0)
    xhigh2 <- ifelse(xu >= xc, xu - xc, 0)
    high2 <- ifelse(xu >= xc, 1, 0)
  
  #5. Generate formulas for two regression lines
    # Create data frame with interrupted regression variables
    reg.data <- data.frame(
      xlow1 = xlow1, xhigh1 = xhigh1, high1 = high1,
      xlow2 = xlow2, xhigh2 = xhigh2, high2 = high2
    )
    reg.data[[y.f]] <- yu
    
    # Add covariates if present
    if (var.count > 2) {
      for (var in all.vars(f)[-(1:2)]) {
        reg.data[[var]] <- data[[var]]
      }
      glm1.f <- update(nox.f, ~ xlow1 + xhigh1 + high1 + .)
      glm2.f <- update(nox.f, ~ xlow2 + xhigh2 + high2 + .)
    }
    
    # If there were no covariates, just run the 3 variable model
    if (var.count == 2) {
      glm1.f <- as.formula(paste0(y.f, " ~ xlow1 + xhigh1 + high1"))
      glm2.f <- as.formula(paste0(y.f, " ~ xlow2 + xhigh2 + high2"))
    }
  
  #6. Run GLM regressions
    glm1 <- glm(glm1.f, data = reg.data, family = family)
    glm2 <- glm(glm2.f, data = reg.data, family = family)
  
  #7. Compute robust standard errors (HC3, fallback to HC1 if needed)
    # Compute robust standard errors
    rob1 <- lmtest::coeftest(glm1, vcov = sandwich::vcovHC(glm1, "HC3"))
    rob2 <- lmtest::coeftest(glm2, vcov = sandwich::vcovHC(glm2, "HC3"))
    
    # Sometimes HC3 gives NA values (for very sparse or extreme data), check and if that's the case change method
    msg <- ""
    if (is.na(rob1[2, 4])) {
      rob1 <- lmtest::coeftest(glm1, vcov = sandwich::vcovHC(glm1, "HC1"))
      msg <- paste0(msg, "\nFor line 1 the heteroskedastic standard errors HC3 resulted in an error thus we used HC1 instead.")
    }
    if (is.na(rob2[2, 4])) {
      rob2 <- lmtest::coeftest(glm2, vcov = sandwich::vcovHC(glm2, "HC1"))
      msg <- paste0(msg, "\nFor line 2 the heteroskedastic standard errors HC3 resulted in an error thus we used HC1 instead.")
    }
  
  #8. Extract slopes, z-statistics, and p-values
    # Slopes
    b1 <- as.numeric(rob1[2, 1])
    b2 <- as.numeric(rob2[3, 1])
    
    # Test statistics, z-values
    z1 <- as.numeric(rob1[2, 3])
    z2 <- as.numeric(rob2[3, 3])
    
    # p-values
    p1 <- as.numeric(rob1[2, 4])
    p2 <- as.numeric(rob2[3, 4])
  
  #9. Determine if u-shape is significant
    u.sig <- ifelse(b1 * b2 < 0 & p1 < .05 & p2 < .05, 1, 0)
  
  #10. Plot results (if graph=1)
    if (graph == 1) {
      # Set up colors and parameters
      pch.dot <- 1          # Dot for scatterplot (data)
      col.l1 <- 'dodgerblue3'
      col.l2 <- 'firebrick'      # Color of straight line 2
      col.fit <- 'gray50'   # Color of fitted smooth line
      col.div <- "green3"   # Color of vertical line
      lty.l1 <- 1           # Type of line 1
      lty.l2 <- 1           # Type of line 2
      lty.fit <- 2          # Type of smoothed line
      
      # Estimate smoother
      if (var.count > 2) {
        gam.f <- paste0(format(nox.f), "+", sx.f)   # add the modified smoother version of x into the formula
        gams <- mgcv::gam(as.formula(gam.f), data = data, family = family)  # now actually run the smoother
      }
      
      if (var.count == 2) {
        gams <- mgcv::gam(as.formula(paste0(y.f, " ~", sx.f)), data = data, family = family)  # now actually run the smoother
      }
      
      # Get dots of raw data
      # If no covariates, there are two variables, and y.dots is the y values
      if (var.count == 2) {
        yobs <- yu
      }
      
      # If covariates present, yobs is the fitted value with u(x) at mean, need new.data() with variables at means
      if (var.count > 2) {
        # Put observed data into data frame
        data.obs <- data[, all.vars(f), drop = FALSE]
        
        # Drop observations with missing values on any of the variables
        data.obs <- na.omit(data.obs)
        
        # Create data where u(x) is at sample means to get residuals based on rest of models to act as yobs
        data.xufixed <- data.obs
        data.xufixed[[x.f]] <- mean(data.obs[[x.f]])   # Note, the 1st predictor is always the one hypothesized to be u-shaped
        # replace it with the mean value of the predictor
        
        # Create data where u(x) is obs, and all else at sample means
        data.otherfixed <- data.obs     # start with original value
        
        # Replace all RHS with mean, except the u(x)
        # changed on 2018 11 23 to allow having factors() as predictors, their "midpoint" value is used
        for (i in 3:var.count) {  # loop over covariates
          var.name <- all.vars(f)[i]
          xt <- sort(data.obs[[var.name]])  # create auxiliary variable that has those values sorted
          n <- length(xt)          # see how many observations there are
          xm <- xt[round(n/2, 0)]   # take the midpoint value (this will work with ordinal and factor data, but with factor it can be arbitrary)
          data.otherfixed[[var.name]] <- xm  # Replace with that value in the dataset used for fitted value
        }
        
        # Get yobs with covariates
        # First the fitted value
        yhat.xufixed <- mgcv::predict.gam(gams, newdata = data.xufixed)
        
        # Subtract fitted value from observed y, and shift it with constant so that it has same mean as original y
        yobs <- yu - yhat.xufixed
        yobs <- yobs + mean(yu) - mean(yobs)  # Adjust to have the same mean
      }  # End if for covariates that requires computes y.obs instead of using real y.
      
      # Get yhat.smooth
      # Without covariates, just fit the observed data
      if (var.count == 2) {
        yhat.smooth <- mgcv::predict.gam(gams)
      }
      
      # With covariates, fit at observed means
      if (var.count > 2) {
        yhat.smooth <- mgcv::predict.gam(gams, newdata = data.otherfixed)
      }
      
      # Subtract fitted value from observed y
      offset3 <- mean(yobs - yhat.smooth)
      yhat.smooth <- yhat.smooth + offset3
      
      # Coordinates for top and bottom end of chart
      y1 <- max(yobs, yhat.smooth)  # highest point
      y0 <- min(yobs, yhat.smooth)  # lowest point
      yr <- y1 - y0                  # range
      y0 <- y0 - .3 * yr               # new lowest. 30% lower
      
      # xs
      x1 <- max(xu)
      x0 <- min(xu)
      xr <- x1 - x0
      
      # Plot
      old_par <- par(no.readonly = TRUE)
      on.exit(par(old_par), add = TRUE)
      par(mar = c(5.4, 4.1, .5, 2.1))
      plot(xu[xu < xc], yobs[xu < xc], cex = .75, col = col.l1, pch = pch.dot, las = 1,
           ylim = c(y0, y1),
           xlim = c(min(xu), max(xu)),
           xlab = "",
           ylab = "")  # Range of y has extra 30% to add labels
      
      points(xu[xu > xc], yobs[xu > xc], cex = .75, col = col.l2)
      
      # Axis labels
      mtext(side = 1, line = 2.75, x.f, font = 2)
      mtext(side = 2, line = 2.75, y.f, font = 2)
      
      # Smoothed line
      lines(xu[order(xu)], yhat.smooth[order(xu)], col = col.fit, lty = 2, lwd = 2)
      
      # Arrow 1
      xm1 <- (xc + x0) / 2
      x0.arrow.1 <- xm1 - .1 * xr
      x1.arrow.1 <- xm1 + .1 * xr
      y0.arrow.1 <- y0 + .1 * yr
      y1.arrow.1 <- y0 + .1 * yr + b1 * (x1.arrow.1 - x0.arrow.1)
      
      # Move arrow if it is too short
      if (x0.arrow.1 < x0 + .1 * xr) {
        x0.arrow.1 <- x0
      }
      
      # Move arrow if it covers text
      gap.1 <- (min(y0.arrow.1, y1.arrow.1) - (y0 + .1 * yr))
      if (gap.1 < 0) {
        y0.arrow.1 <- y0.arrow.1 - gap.1
        y1.arrow.1 <- y1.arrow.1 - gap.1
      }
      
      arrows(x0 = x0.arrow.1, x1 = x1.arrow.1, y0 = y0.arrow.1, y1 = y1.arrow.1, col = col.l1, lwd = 2)
      
      # Text under arrow 1
      xm1 <- max(xm1, x0 + .20 * xr)
      text(xm1, y0 + .025 * yr,
           paste0("Average slope 1:\nb = ", round(b1, 2), ", z = ", round(z1, 2), ", ", format_pvalue(p1, include_p = TRUE)),
           col = col.l1)
      
      # Arrow 2
      x0.arrow.2 <- xc + (x1 - xc) / 2 - .1 * xr
      x1.arrow.2 <- xc + (x1 - xc) / 2 + .1 * xr
      y0.arrow.2 <- y1.arrow.1
      y1.arrow.2 <- y0.arrow.2 + b2 * (x1.arrow.2 - x0.arrow.2)
      
      gap.2 <- (min(y0.arrow.2, y1.arrow.2) - (y0 + .1 * yr))
      if (gap.2 < 0) {
        y0.arrow.2 <- y0.arrow.2 - gap.2
        y1.arrow.2 <- y1.arrow.2 - gap.2
      }
      
      # Shorten arrow if it is too close to the end
      x1.arrow.2 <- min(x1.arrow.2, x1)
      if (x0.arrow.2 < xc) {
        x0.arrow.2 <- xc
      }
      
      xm2 <- xc + (x1 - xc) / 2
      xm2 <- min(xm2, x1 - .2 * xr)
      
      arrows(x0 = x0.arrow.2, x1 = x1.arrow.2, y0 = y0.arrow.2, y1 = y1.arrow.2, col = col.l2, lwd = 2)
      text(xm2, y0 + .025 * yr,
           paste0("Average slope 2:\nb = ", round(b2, 2), ", z = ", round(z2, 2), ", ", format_pvalue(p2, include_p = TRUE)),
           col = col.l2)
      
      # Division line
      lines(c(xc, xc), c(y0 + .35 * yr, y1), col = col.div, lty = lty.fit)
      text(xc, y0 + .3 * yr, round(xc, 2), col = col.div)
    }  # End: if graph == 1
  
  #11. Return results
    # list with results
    res <- list(b1 = b1, p1 = p1, b2 = b2, p2 = p2, u.sig = u.sig, xc = xc, z1 = z1, z2 = z2,
                glm1 = glm1, glm2 = glm2, rob1 = rob1, rob2 = rob2, msg = msg)
    
    if (graph == 1) {
      res$yhat.smooth <- yhat.smooth
    }
    
    # output it
    res
}  # End of reg2() function

#' Enhanced alternative to lm()
#'
#' Runs a linear regression with better defaults (robust SE), and richer & better 
#' formatted output than \code{lm}. For robust and clustered errors it relies on \code{\link[estimatr]{lm_robust}}. 
#' The output reports classical and robust errors, number of missing observations per
#' variable, an effect size column (standardized regression coefficient), and a red.flag column per variable 
#' flagging the need to conduct specific diagnostics. It relies by default on HC3 for standard errors;
#' \code{lm_robust} relies on HC2 (and Stata's 'reg y x, robust' on HC1), which can have
#' inflated false-positive rates in smaller samples (Long & Ervin, 2000). 
#' @param ... same arguments as \code{\link[base]{lm}}, plus the arguments below
#' @param se_type The type of standard error to use. Default is \code{"HC3"}.
#'   Without clusters: \code{"HC0"}, \code{"HC1"}, \code{"HC2"}, or \code{"HC3"}.
#'   When \code{clusters} is specified, \code{se_type} is automatically set to \code{"CR2"}.
#' @param notes Logical. If TRUE (default), print explanatory notes below the table
#'   when the result is printed.
#' @param clusters An optional variable indicating clusters for cluster-robust standard 
#'   errors. When specified, \code{se_type} is automatically set to \code{"CR2"} 
#'   (bias-reduced cluster-robust estimator). Passed to \code{\link[estimatr]{lm_robust}}.
#' @param fixed_effects An optional right-sided formula containing the fixed effects 
#'   to be projected out (absorbed) before estimation. Useful for models with many 
#'   fixed effect groups (e.g., \code{~ firm_id} or \code{~ firm_id + year}). 
#'   Passed to \code{\link[estimatr]{lm_robust}}.
#' @param ... Additional arguments passed to \code{\link[estimatr]{lm_robust}}.
#'
#' @details
#' Robust standard errors and clustered standard errors are computed using 
#' \code{\link[estimatr]{lm_robust}}; see the documentation of that function for details (using by default CR2 errors)
#' The output shows both standard errors and when clustering errors it reports all three.
#' The red.flag column is based on the difference between robust and classical standard errors.
#'
#' The \code{red.flag} column provides diagnostic warnings:
#' \itemize{
#'   \item \code{!}, \code{!!}, \code{!!!}: Robust and classical standard errors differ by 
#'     more than 25\%, 50\%, or 100\%, respectively. Large differences may suggest model 
#'     misspecification or outliers (but they may also be benign). When encountering a red flag,
#'     authors should plot the distributions to look for outliers or skewed data, and use \code{\link{scatter.gam}}
#'     to look for possible nonlinearities in the relevant variables.
#'     King & Roberts (2015) propose a higher cutoff, at 100\%, and a bootstrapped significance test; 
#'     \code{statuser} does not follow either recommendation. The former seems too liberal, the 
#'     latter too time consuming to include in every regression, plus the focus here is on individual variables rather than joint tests.
#'   \item \code{X}: For interaction terms, the component variables are correlated (|r| > 0.3 or p < .05),
#'     which means the interaction term is likely to be biased. See Simonsohn (2024) "Interacting with curves"
#'     \doi{10.1177/25152459231207787}.
#' }
#'
#' @references
#' King, G., & Roberts, M. E. (2015). How robust standard errors expose methodological 
#' problems they do not fix, and what to do about it. \emph{Political Analysis}, 23(2), 159-179.
#'
#' Long, J. S., & Ervin, L. H. (2000). Using heteroscedasticity consistent standard errors 
#' in the linear regression model. \emph{The American Statistician}, 54(3), 217-224.
#'
#' Simonsohn, U. (2024). Interacting with curves: How to validly test and probe 
#' interactions in the real (nonlinear) world. \emph{Advances in Methods and Practices in 
#' Psychological Science}, 7(1), 1-22.
#'
#' @examples
#' # Basic usage with data argument
#' lm2(mpg ~ wt + hp, data = mtcars)
#'
#' # Without data argument (variables from environment)
#' y <- mtcars$mpg
#' x1 <- mtcars$wt
#' x2 <- mtcars$hp
#' lm2(y ~ x1 + x2)
#'
#' # RED FLAG EXAMPLES
#' 
#' # Example 1: red flag catches a nonlinearity
#' # True model is quadratic: y = x^2
#' set.seed(123)
#' x <- runif(200, -3, 3)
#' y <- x^2 + rnorm(200, sd = 2)
#' 
#' # lm2() shows red flag due to misspecification
#' lm2(y ~ x)
#' 
#' # Follow up with scatter.gam() to diagnose it
#' scatter.gam(x, y)
#'
#' # Example 2: red flag catches an outlier in y
#' # True model is y = x, but one observation has a very large y value
#' set.seed(123)
#' x <- sort(rnorm(200))
#' y <- round(x + rnorm(200, sd = 2), 1)
#' y[200] <- 100  # Outlier
#' 
#' # lm2() flags x
#' lm2(y ~ x)
#' 
#' # Look at distribution of y to spot the outlier
#' plot_freq(y)
#'
#' # Example 3: red flag catches an outlier in one predictor
#' # True model is y = x1 + x2, but x2 has an extreme value
#' set.seed(123)
#' x1 <- round(rnorm(200),.1)
#' x2 <- round(rnorm(200),.1)
#' y <- x1 + x2 + rnorm(200, sd = 0.5)
#' x2[200] <- 50  # Outlier in x2
#' 
#' # lm2() flags x2 (but not x1)
#' lm2(y ~ x1 + x2)
#' 
#' # Look at distribution of x2 to spot the outlier
#' plot_freq(x2)
#'
#' # CLUSTERED STANDARD ERRORS
#' # When observations are grouped (e.g., students within schools),
#' # use clusters to account for within-group correlation
#' set.seed(123)
#' n_clusters <- 20
#' n_per_cluster <- 15
#' cluster_id <- rep(1:n_clusters, each = n_per_cluster)
#' cluster_effect <- rnorm(n_clusters, sd = 2)[cluster_id]
#' x <- rnorm(n_clusters * n_per_cluster)
#' y <- 1 + 0.5 * x + cluster_effect + rnorm(n_clusters * n_per_cluster)
#' mydata <- data.frame(y = y, x = x, cluster_id = cluster_id)
#' 
#' # Clustered SE (CR2) - note the SE.cluster column in output
#' lm2(y ~ x, data = mydata, clusters = cluster_id)
#'
#' # FIXED EFFECTS
#' # Use fixed_effects to absorb group-level variation (e.g., firm or year effects)
#' # This is useful for panel data or when you have many fixed effect levels
#' set.seed(456)
#' n_firms <- 30
#' n_years <- 5
#' firm_id <- rep(1:n_firms, each = n_years)
#' year <- rep(2018:2022, times = n_firms)
#' firm_effect <- rnorm(n_firms, sd = 3)[firm_id]
#' x <- rnorm(n_firms * n_years)
#' y <- 2 + 0.8 * x + firm_effect + rnorm(n_firms * n_years)
#' panel <- data.frame(y = y, x = x, firm_id = factor(firm_id), year = factor(year))
#' 
#' # Absorb firm fixed effects (coefficient on x is estimated, firm dummies are not shown)
#' lm2(y ~ x, data = panel, fixed_effects = ~ firm_id)
#' 
#' # Two-way fixed effects (firm and year)
#' lm2(y ~ x, data = panel, fixed_effects = ~ firm_id + year)
#'
#' @return An object of class \code{c("lm2", "lm_robust", "lm")}. This inherits 
#'   from \code{\link[estimatr]{lm_robust}} and can be used with packages like 
#'   \code{marginaleffects}. The object contains all components of an \code{lm_robust} 

#'   object plus additional attributes:
#'   \describe{
#'     \item{statuser_table}{A data frame with columns: \code{term}, \code{estimate}, 
#'       \code{SE.robust}, \code{SE.classical}, \code{t}, \code{df}, \code{p.value}, 
#'       \code{B} (standardized coefficient), and optionally \code{SE.cluster} when 
#'       clustered standard errors are used.}
#'     \item{classical_fit}{The underlying \code{lm} object with classical standard errors.}
#'     \item{na_counts}{Integer vector of missing value counts per variable.}
#'     \item{n_missing}{Total number of observations excluded due to missing values.}
#'     \item{has_clusters}{Logical indicating whether clustered standard errors were used.}
#'   }
#'   When printed, displays a formatted regression table with robust and classical 
#'   standard errors, effect sizes, and diagnostic red flags.
#'
#' @seealso \code{\link[estimatr]{lm_robust}}, \code{\link{scatter.gam}}
#' @usage NULL
#' @export lm2
lm2 <- function(formula, data = NULL, se_type = "HC3", notes = TRUE, 
                clusters = NULL, fixed_effects = NULL, ...) {
  
  # Capture the call
  cl <- match.call()
  
  # Check that estimatr is available
  if (!requireNamespace("estimatr", quietly = TRUE)) {
    stop("Package 'estimatr' is required for lm2(). Please install it with: install.packages('estimatr')")
  }
  
  # Capture clusters expression for non-standard evaluation
  clusters_expr <- substitute(clusters)
  has_clusters <- !missing(clusters) && !is.null(clusters_expr) && !identical(clusters_expr, quote(NULL))
  
  # Evaluate clusters in data context if provided
  clusters_vec <- NULL
  if (has_clusters && !is.null(data)) {
    clusters_vec <- tryCatch(
      eval(clusters_expr, envir = data, enclos = parent.frame()),
      error = function(e) {
        # Try evaluating in parent frame if not in data
        eval(clusters_expr, envir = parent.frame(2))
      }
    )
  } else if (has_clusters) {
    clusters_vec <- eval(clusters_expr, envir = parent.frame())
  }
  
  # Validate formula early
  validate_formula(formula, data, func_name = "lm2", calling_env = parent.frame())
  
  # Validate inputs and construct data frame if needed
  validated <- validate_lm2(
    formula = formula,
    data = data,
    se_type = se_type,
    se_type_missing = missing(se_type),
    dots = list(..., clusters = clusters_vec),
    calling_env = parent.frame()
  )
  
  data <- validated$data
  se_type <- validated$se_type
  

  # Build arguments for lm_robust (only include non-NULL optional args)
  lm_robust_args <- list(formula = formula, data = data, se_type = se_type, ...)
  if (has_clusters) lm_robust_args$clusters <- clusters_vec
  if (!is.null(fixed_effects)) lm_robust_args$fixed_effects <- fixed_effects
  
  # Run lm_robust with specified se_type
  robust_fit <- do.call(estimatr::lm_robust, lm_robust_args)
  
  # Build enhanced object that inherits from lm_robust
  # This allows packages like marginaleffects to work with it seamlessly
  
  # Extract components from lm_robust object
  term_names <- names(robust_fit$coefficients)
  estimates <- as.numeric(robust_fit$coefficients)
  
  # When clusters are used, we need both clustered SE and HC3 robust SE
  # The clustered SE comes from robust_fit (CR2)
  # The HC3 robust SE comes from a separate run without clusters
  # This lets us flag heteroskedasticity/outliers separately from clustering effects
  if (has_clusters) {
    cluster_se <- robust_fit$std.error
    # t-values and p-values based on clustered SE
    t_values <- robust_fit$statistic
    p_values <- robust_fit$p.value
    df_values <- robust_fit$df
    # Run HC3 without clusters to get heteroskedasticity-robust SE (for red flag only)
    hc3_args <- list(formula = formula, data = data, se_type = "HC3", ...)
    if (!is.null(fixed_effects)) hc3_args$fixed_effects <- fixed_effects
    hc3_fit <- do.call(estimatr::lm_robust, hc3_args)
    robust_se <- hc3_fit$std.error
  } else {
    cluster_se <- NULL
    robust_se <- robust_fit$std.error
    t_values <- robust_fit$statistic
    p_values <- robust_fit$p.value
    df_values <- robust_fit$df
  }
  
  # Run classical SEs using the same model specification (including fixed effects)
  classical_args <- list(formula = formula, data = data, se_type = "classical", ...)
  if (!is.null(fixed_effects)) classical_args$fixed_effects <- fixed_effects
  # Ensure classical SEs are not clustered
  if ("clusters" %in% names(classical_args)) classical_args$clusters <- NULL
  classical_fit <- do.call(estimatr::lm_robust, classical_args)
  classical_se <- classical_fit$std.error
  
  # Calculate standardized coefficients (beta weights)
  # Beta = b * (SD_x / SD_y)
  # For intercept, beta is NA
  model_data <- stats::model.frame(formula, data = data)
  y <- model_data[[1]]
  sd_y <- stats::sd(y, na.rm = TRUE)
  
  # Calculate total missing observations (rows excluded due to NAs)
  n_original <- nrow(data)
  n_used <- robust_fit$nobs
  n_missing <- n_original - n_used
  
  # Missing DV count (from original data)
  y_name <- all.vars(formula)[1]
  n_missing_y <- if (!is.null(y_name) && y_name %in% names(data)) {
    sum(is.na(data[[y_name]]))
  } else {
    NA_integer_
  }

  # Fixed effects info (for printing)
  fe_terms <- character(0)
  fe_k <- integer(0)
  fe_missing <- integer(0)
  if (!is.null(fixed_effects)) {
    fe_terms <- all.vars(fixed_effects)
    fe_k <- sapply(fe_terms, function(term) {
      if (!term %in% names(data)) return(NA_integer_)
      v <- data[[term]]
      length(unique(v[!is.na(v)]))
    })
    fe_missing <- sapply(fe_terms, function(term) {
      if (!term %in% names(data)) return(NA_integer_)
      sum(is.na(data[[term]]))
    })
    names(fe_k) <- fe_terms
    names(fe_missing) <- fe_terms
  }
  
  # Calculate standardized coefficients, means/percentages, and NA counts
  standardized_coefs <- numeric(length(term_names))
  mean_or_pct <- numeric(length(term_names))
  mean_is_pct <- logical(length(term_names))  # TRUE when mean_or_pct is 0-100 % (factor/factor*factor)
  na_counts <- integer(length(term_names))
  names(standardized_coefs) <- term_names
  names(mean_or_pct) <- term_names
  names(mean_is_pct) <- term_names
  names(na_counts) <- term_names
  
  # Get model matrix for dummy variable percentages
  # Try from classical_fit first, then robust_fit
  model_matrix <- tryCatch(
    stats::model.matrix(classical_fit),
    error = function(e) {
      tryCatch(
        stats::model.matrix(robust_fit),
        error = function(e2) NULL
      )
    }
  )
  
  # Get original data to count NAs (before model.frame removes them)
  # We need to look at the raw data columns
  for (i in seq_along(term_names)) {
    term <- term_names[i]
    if (term == "(Intercept)") {
      standardized_coefs[i] <- NA_real_
      mean_or_pct[i] <- NA_real_
      na_counts[i] <- NA_integer_  # NA count not meaningful for intercept
    } else {
      # Get the coefficient
      b <- estimates[i]
      
      # Check if this is an interaction term (contains ":")
      if (grepl(":", term)) {
        # Interaction term: compute b * sd(x1) * sd(x2) * ... / sd(y)
        components <- strsplit(term, ":")[[1]]
        sd_product <- 1
        all_numeric <- TRUE
        
        for (comp in components) {
          if (comp %in% names(model_data)) {
            x_comp <- model_data[[comp]]
            if (is.numeric(x_comp)) {
              sd_product <- sd_product * stats::sd(x_comp, na.rm = TRUE)
            } else {
              # Factor component - can't compute SD
              all_numeric <- FALSE
              break
            }
          } else {
            # Component not found (likely a factor level like "groupB")
            all_numeric <- FALSE
            break
          }
        }
        
        if (all_numeric) {
          standardized_coefs[i] <- b * sd_product / sd_y
        } else {
          standardized_coefs[i] <- NA_real_
        }
        
        # Compute mean for interactions: 
        # For x*z where z is factor: mean of x for that factor level
        # For x*z where both numeric: mean of x*z product
        if (length(components) == 2) {
          # Try to identify numeric vs factor components
          comp1 <- components[1]
          comp2 <- components[2]
          
          # Helper to get variable and check if it's a factor level
          get_var_info <- function(comp_name) {
            # Direct numeric variable
            if (comp_name %in% names(model_data)) {
              v <- model_data[[comp_name]]
              if (is.numeric(v)) return(list(type = "numeric", var = v, filter = NULL))
              if (is.factor(v)) return(list(type = "factor", var = v, filter = NULL))
            }
            # Factor level term (e.g., "groupTreatment")
            for (var_name in names(model_data)) {
              if (is.factor(model_data[[var_name]]) || is.character(model_data[[var_name]])) {
                if (startsWith(comp_name, var_name)) {
                  level_name <- substring(comp_name, nchar(var_name) + 1)
                  factor_var <- model_data[[var_name]]
                  if (level_name %in% levels(factor(factor_var))) {
                    return(list(type = "factor_level", var = factor_var, filter = level_name))
                  }
                }
              }
            }
            return(NULL)
          }
          
          info1 <- get_var_info(comp1)
          info2 <- get_var_info(comp2)
          
          if (!is.null(info1) && !is.null(info2)) {
            # Case 1: numeric * factor_level -> mean of numeric where factor == level
            if (info1$type == "numeric" && info2$type == "factor_level") {
              filter_idx <- info2$var == info2$filter
              mean_or_pct[i] <- mean(info1$var[filter_idx], na.rm = TRUE)
            } else if (info1$type == "factor_level" && info2$type == "numeric") {
              filter_idx <- info1$var == info1$filter
              mean_or_pct[i] <- mean(info2$var[filter_idx], na.rm = TRUE)
            # Case 2: numeric * numeric -> mean of product
            } else if (info1$type == "numeric" && info2$type == "numeric") {
              mean_or_pct[i] <- mean(info1$var * info2$var, na.rm = TRUE)
            # Case 3: factor_level * factor_level -> percentage where both conditions met
            } else if (info1$type == "factor_level" && info2$type == "factor_level") {
              filter_idx <- (info1$var == info1$filter) & (info2$var == info2$filter)
              mean_or_pct[i] <- mean(filter_idx, na.rm = TRUE) * 100
              mean_is_pct[i] <- TRUE
            } else {
              mean_or_pct[i] <- NA_real_
            }
          } else {
            mean_or_pct[i] <- NA_real_
          }
        } else {
          # 3+ way interactions: skip for now
          mean_or_pct[i] <- NA_real_
        }
        na_counts[i] <- NA_integer_  # NA count not meaningful for interactions
        
      } else if (term %in% names(model_data)) {
        # Simple term in model data: compute b * sd(x) / sd(y)
        x <- model_data[[term]]
        if (is.numeric(x)) {
          sd_x <- stats::sd(x, na.rm = TRUE)
          standardized_coefs[i] <- b * (sd_x / sd_y)
          mean_or_pct[i] <- mean(x, na.rm = TRUE)
        } else {
          standardized_coefs[i] <- NA_real_
          mean_or_pct[i] <- NA_real_
        }
        # Count NAs in original data column
        if (term %in% names(data)) {
          na_counts[i] <- sum(is.na(data[[term]]))
        } else {
          na_counts[i] <- NA_integer_
        }
      } else {
        # For factor levels (e.g., "groupB"), get standardized coef and % from model matrix or model frame
        # Try to get percentage from model matrix (dummy variable) first
        if (!is.null(model_matrix) && term %in% colnames(model_matrix)) {
          dummy_col <- model_matrix[, term]
          # Calculate standardized coefficient: b * sd(dummy) / sd(y)
          sd_dummy <- stats::sd(dummy_col, na.rm = TRUE)
          standardized_coefs[i] <- b * sd_dummy / sd_y
          
          # Check if this is an ordered factor polynomial contrast (.L, .Q, .C, .^n)
          # These have non-0/1 values so percentage doesn't make sense
          is_polynomial_contrast <- grepl("\\.(L|Q|C|\\^[0-9]+)$", term)
          
          if (is_polynomial_contrast) {
            # Polynomial contrasts: mean/pct not meaningful
            mean_or_pct[i] <- NA_real_
          } else {
            # Treatment contrasts (dummy 0/1): calculate percentage of 1s
            mean_or_pct[i] <- mean(dummy_col, na.rm = TRUE) * 100
            mean_is_pct[i] <- TRUE
          }
          
          # Count NAs in the original factor variable
          # Find the factor variable name by checking which data column this term came from
          found_na_count <- FALSE
          if (!is.null(data)) {
            for (var_name in names(data)) {
              if (is.factor(data[[var_name]]) || is.character(data[[var_name]])) {
                if (startsWith(term, var_name)) {
                  na_counts[i] <- sum(is.na(data[[var_name]]))
                  found_na_count <- TRUE
                  break
                }
              }
            }
          }
          if (!found_na_count) {
            na_counts[i] <- 0L
          }
        } else {
          # Fallback: infer factor level from term (e.g. "groupB" -> variable "group", level "B")
          # and compute % from model frame (works when model.matrix fails or colnames don't match)
          found_pct <- FALSE
          for (var_name in names(model_data)) {
            if (var_name == term) next
            v <- model_data[[var_name]]
            if (is.factor(v) || is.character(v)) {
              levs <- levels(factor(v))
              for (lev in levs[-1L]) {  # skip baseline (first level)
                if (term == paste0(var_name, lev)) {
                  mean_or_pct[i] <- mean(v == lev, na.rm = TRUE) * 100
                  mean_is_pct[i] <- TRUE
                  found_pct <- TRUE
                  # Standardized coef from dummy
                  dummy_col <- as.numeric(v == lev)
                  sd_dummy <- stats::sd(dummy_col, na.rm = TRUE)
                  standardized_coefs[i] <- b * sd_dummy / sd_y
                  break
                }
              }
              if (found_pct) break
            }
          }
          if (!found_pct) {
            standardized_coefs[i] <- NA_real_
            mean_or_pct[i] <- NA_real_
            mean_is_pct[i] <- FALSE
          }
          # NA count for factor variable
          if (!is.null(data)) {
            found_na_count <- FALSE
            for (var_name in names(data)) {
              if (is.factor(data[[var_name]]) || is.character(data[[var_name]])) {
                if (startsWith(term, var_name)) {
                  na_counts[i] <- sum(is.na(data[[var_name]]))
                  found_na_count <- TRUE
                  break
                }
              }
            }
            if (!found_na_count) na_counts[i] <- NA_integer_
          } else {
            na_counts[i] <- NA_integer_
          }
        }
      }
    }
  }
  
  # Build display table data frame (used by print method)
  statuser_table <- data.frame(
    term = term_names,
    estimate = estimates,
    SE.robust = robust_se,
    SE.classical = classical_se[term_names],
    t = t_values,
    df = df_values,
    p.value = p_values,
    B = standardized_coefs,
    mean_or_pct = mean_or_pct,
    mean_is_pct = mean_is_pct,
    row.names = NULL,
    stringsAsFactors = FALSE
  )
  
  # Add SE.cluster column if clusters were used
  if (!is.null(cluster_se)) {
    statuser_table$SE.cluster <- cluster_se
  }
  
  # Start with the robust_fit object so we inherit from lm_robust
  # This ensures compatibility with marginaleffects and other packages
  result <- robust_fit
  
  # Store the statuser-specific attributes
 attr(result, "statuser_table") <- statuser_table
  attr(result, "lm2_call") <- cl
  attr(result, "classical_fit") <- classical_fit
  attr(result, "na_counts") <- na_counts
  attr(result, "n_missing") <- n_missing
  attr(result, "n_missing_y") <- n_missing_y
  attr(result, "y_name") <- y_name
  attr(result, "notes") <- notes
  attr(result, "has_clusters") <- has_clusters
  attr(result, "fe_terms") <- fe_terms
  attr(result, "fe_k") <- fe_k
  attr(result, "fe_missing") <- fe_missing
  
  # Prepend "lm2" class so our print method is used, but keep lm_robust inheritance
  class(result) <- c("lm2", class(result))
  
  return(result)
}

#' Print method for lm2 objects
#'
#' @param x An object of class \code{lm2}
#' @param notes Logical. If TRUE (default), print explanatory notes below the table.
#'   If not specified, uses the value set when \code{lm2()} was called.
#' @param ... Additional arguments (ignored)
#'
#' @return Invisibly returns the original object
#' @export
print.lm2 <- function(x, notes = NULL, ...) {
  
  # Use notes attribute from lm2() call if not explicitly specified
  if (is.null(notes)) {
    notes <- attr(x, "notes")
    if (is.null(notes)) notes <- TRUE  
  }
  
  # Get the statuser table from attribute
  tbl <- attr(x, "statuser_table")
  
  # Helper: smart rounding based on magnitude
  # >=100: 1 decimal, >=10: 2 decimals, >=0.01: 3 decimals
  # <0.01: show 2 significant non-zero digits (e.g., .000042)
  smart_round <- function(val) {
    if (is.na(val)) return(NA_character_)
    abs_val <- abs(val)
    if (abs_val >= 100) {
      decimals <- 1
      return(format(round(val, decimals), nsmall = decimals))
    } else if (abs_val >= 10) {
      decimals <- 2
      return(format(round(val, decimals), nsmall = decimals))
    } else if (abs_val >= 0.01 || abs_val == 0) {
      decimals <- 3
      return(format(round(val, decimals), nsmall = decimals))
    } else {
      # For values < 0.01, show 2 significant digits
      # e.g., 0.000042323 -> .000042
      sign_char <- if (val < 0) "-" else ""
      # Find how many decimal places needed for 2 sig figs
      log_val <- floor(log10(abs_val))
      decimals <- -log_val + 1  # +1 to get 2 sig figs
      rounded <- round(abs_val, decimals)
      formatted <- format(rounded, nsmall = decimals, scientific = FALSE)
      # Remove leading zero
      formatted <- sub("^0\\.", ".", formatted)
      return(paste0(sign_char, formatted))
    }
  }
  
  # Helper: format p-value (no leading zero, <.0001 if tiny)
  format_p <- function(p) {
    if (is.na(p)) return(NA_character_)
    if (p < 0.0001) return("<.0001")
    # Round to 4 decimals, remove leading zero, add space to align with "<.0001"
    p_rounded <- round(p, 4)
    p_str <- format(p_rounded, nsmall = 4, scientific = FALSE)
    p_str <- sub("^0\\.", " .", p_str)  # Replace leading 0 with space for alignment
    p_str
  }
  
  # Get NA counts from attribute before modifying display_df
  na_counts <- attr(x, "na_counts")
  
  # -------------------------------------------------------------------------
  # Helper: Parse factor terms and identify baseline levels
  # Returns a list with:
  #   - formatted_terms: vector of formatted term names (e.g., "group (B)")
  #   - baseline_info: list of baseline rows to insert
  #   - is_factor_level: logical vector indicating which terms are factor levels
  # -------------------------------------------------------------------------
  parse_factor_terms <- function(tbl, xlevels) {
    n_terms <- nrow(tbl)
    formatted_terms <- tbl$term
    is_factor_level <- logical(n_terms)
    baseline_info <- list()  # Will store info about baseline rows to add
    
    if (is.null(xlevels) || length(xlevels) == 0) {
      return(list(
        formatted_terms = formatted_terms,
        baseline_info = baseline_info,
        is_factor_level = is_factor_level
      ))
    }
    
    # Helper: Replace factor level in a term component
    # e.g., "group2B" with var_name="group2", lev="B" -> "group2 (B)"
    replace_factor_level <- function(component, var_name, lev) {
      expected <- paste0(var_name, lev)
      if (component == expected) {
        return(paste0(var_name, " (", lev, ")"))
      }
      return(NULL)
    }
    
    # Helper: Check if term is an ordered factor polynomial contrast
    # e.g., "group3_ord.L" -> "group3_ord (linear)"
    # Returns NULL if not a match, or the formatted name if it is
    check_ordered_contrast <- function(term, var_name) {
      # Ordered factors use .L, .Q, .C, .^4, .^5, etc. for polynomial contrasts
      contrast_map <- c(
        ".L" = "linear",
        ".Q" = "quadratic", 
        ".C" = "cubic"
      )
      
      for (suffix in names(contrast_map)) {
        expected <- paste0(var_name, suffix)
        if (term == expected) {
          return(paste0(var_name, " (", contrast_map[suffix], ")"))
        }
      }
      
      # Check for higher order polynomials: .^4, .^5, etc.
      pattern <- paste0("^", gsub("([.|()\\^{}+*?$\\[\\]\\\\])", "\\\\\\1", var_name), "\\.\\^(\\d+)$")
      if (grepl(pattern, term)) {
        degree <- sub(pattern, "\\1", term)
        return(paste0(var_name, " (^", degree, ")"))
      }
      
      return(NULL)
    }
    
    # Track which factors we've seen (to add baseline only once)
    factors_seen <- character(0)
    
    for (i in seq_len(n_terms)) {
      term <- tbl$term[i]
      
      # Skip intercept
      if (term == "(Intercept)") next
      
      # Check if this is an interaction term
      if (grepl(":", term)) {
        # Split into components and try to format each
        components <- strsplit(term, ":")[[1]]
        new_components <- components
        
        for (ci in seq_along(components)) {
          comp <- components[ci]
          # Try to match against factor levels
          for (var_name in names(xlevels)) {
            levels_vec <- xlevels[[var_name]]
            for (lev in levels_vec[-1]) {  # Skip baseline level (won't appear in interactions)
              replacement <- replace_factor_level(comp, var_name, lev)
              if (!is.null(replacement)) {
                new_components[ci] <- replacement
                break
              }
            }
            # Also check for ordered factor contrasts in interactions
            ordered_replacement <- check_ordered_contrast(comp, var_name)
            if (!is.null(ordered_replacement)) {
              new_components[ci] <- ordered_replacement
            }
          }
        }
        formatted_terms[i] <- paste(new_components, collapse = ":")
      } else {
        # Non-interaction term: check each factor variable
        matched <- FALSE
        for (var_name in names(xlevels)) {
          levels_vec <- xlevels[[var_name]]
          
          # First check for ordered factor polynomial contrasts (e.g., varname.L, varname.Q)
          ordered_replacement <- check_ordered_contrast(term, var_name)
          if (!is.null(ordered_replacement)) {
            is_factor_level[i] <- TRUE
            formatted_terms[i] <- ordered_replacement
            matched <- TRUE
            # Note: ordered factors don't have a baseline level to show
            break
          }
          
          # Check if this term matches varname + level (treatment contrasts)
          for (lev in levels_vec) {
            expected_name <- paste0(var_name, lev)
            if (term == expected_name) {
              # This is a factor level term
              is_factor_level[i] <- TRUE
              formatted_terms[i] <- paste0(var_name, " (", lev, ")")
              
              # If this is the first time we see this factor, record the baseline
              if (!(var_name %in% factors_seen)) {
                factors_seen <- c(factors_seen, var_name)
                baseline_level <- levels_vec[1]  # First level is baseline
                baseline_info[[length(baseline_info) + 1]] <- list(
                  var_name = var_name,
                  baseline_level = baseline_level,
                  insert_before = i  # Insert baseline row before this row
                )
              }
              matched <- TRUE
              break
            }
          }
          if (matched) break
        }
      }
    }
    
    return(list(
      formatted_terms = formatted_terms,
      baseline_info = baseline_info,
      is_factor_level = is_factor_level
    ))
  }
  
  # Get factor levels from the model
  xlevels <- x$xlevels
  if (is.null(xlevels)) {
    # Fallback to classical fit
    classical_fit <- attr(x, "classical_fit")
    if (!is.null(classical_fit)) {
      xlevels <- classical_fit$xlevels
    }
  }
  
  # Parse factor terms
  factor_info <- parse_factor_terms(tbl, xlevels)
  
  # Get model data for correlation checks and baseline % (lm_robust may not have $model)
  model_data <- x$model
  if (is.null(model_data)) {
    # Fallback: reconstruct from formula and data
    model_data <- tryCatch(
      stats::model.frame(x$terms, data = attr(x, "classical_fit")$model),
      error = function(e) NULL
    )
  }
  if (is.null(model_data)) {
    # Second fallback: use data from the call (e.g. lm2(y ~ x + group, data = d))
    cl <- attr(x, "lm2_call")
    if (!is.null(cl) && !is.null(cl$data)) {
      model_data <- tryCatch(
        stats::model.frame(x$terms, data = eval(cl$data, envir = parent.frame())),
        error = function(e) NULL
      )
    }
  }

  # Build model matrix for factor level handling (if available)
  model_matrix <- tryCatch(
    stats::model.matrix(x),
    error = function(e) {
      classical_fit <- attr(x, "classical_fit")
      if (!is.null(classical_fit)) {
        tryCatch(stats::model.matrix(classical_fit), error = function(e2) NULL)
      } else {
        NULL
      }
    }
  )
  
  # Calculate SE flag: ! if robust and classical differ by >25%, !! if >50%, !!! if >100%
  # Also check for interaction term correlations and store r(x,z) values
  se_flag <- character(nrow(tbl))
  cor_rxz <- character(nrow(tbl))  # Store formatted correlation for r(x,z) column
  has_interactions <- FALSE
  has_se_flags <- FALSE      # Track if any SE-related flags (!, !!, !!!)
  has_cor_flags <- FALSE     # Track if any correlation-related flags (X)
  
  for (i in seq_len(nrow(tbl))) {
    flags <- character(0)
    cor_rxz[i] <- "--"  # Default for non-interaction terms
    
    # Check SE difference
    se_classical <- tbl$SE.classical[i]
    se_robust <- tbl$SE.robust[i]
    if (!is.na(se_classical) && !is.na(se_robust) && se_classical != 0) {
      ratio_diff <- abs(se_classical - se_robust) / se_classical
      if (ratio_diff > 1) {
        flags <- c(flags, "!!!")
        has_se_flags <- TRUE
      } else if (ratio_diff > 0.5) {
        flags <- c(flags, "!!")
        has_se_flags <- TRUE
      } else if (ratio_diff > 0.25) {
        flags <- c(flags, "!")
        has_se_flags <- TRUE
      }
    }
    
    # Check if this is an interaction term (contains ":")
    term <- tbl$term[i]
    if (grepl(":", term)) {
      has_interactions <- TRUE
      # Extract the component variable names
      components <- strsplit(term, ":")[[1]]
      
      # Check correlation between components if we have 2 components
      if (length(components) == 2 && !is.null(model_data)) {
        var1_name <- components[1]
        var2_name <- components[2]
        
        # Helper function to get numeric variable for correlation
        # Handles both direct numeric variables and factor level terms
        get_numeric_var <- function(comp_name, model_data, model_matrix) {
          # First check if it's directly in model_data as numeric
          if (comp_name %in% names(model_data)) {
            v <- model_data[[comp_name]]
            if (is.numeric(v)) return(v)
            # If it's a factor with 2 levels, convert to 0/1
            if (is.factor(v) && length(levels(v)) == 2) {
              return(as.numeric(v) - 1)
            }
          }
          # Check if it's a factor level term (e.g., "groupTreatment")
          # Try to find the original factor variable
          for (var_name in names(model_data)) {
            if (is.factor(model_data[[var_name]]) || is.character(model_data[[var_name]])) {
              if (startsWith(comp_name, var_name)) {
                # Extract the level name
                level_name <- substring(comp_name, nchar(var_name) + 1)
                factor_var <- model_data[[var_name]]
                if (level_name %in% levels(factor(factor_var))) {
                  # Convert to 0/1 for this specific level
                  return(as.numeric(factor_var == level_name))
                }
              }
            }
          }
          # Try model matrix as fallback
          if (!is.null(model_matrix) && comp_name %in% colnames(model_matrix)) {
            return(model_matrix[, comp_name])
          }
          return(NULL)
        }
        
        var1 <- get_numeric_var(var1_name, model_data, model_matrix)
        var2 <- get_numeric_var(var2_name, model_data, model_matrix)
        
        # Calculate correlation if both are numeric
        if (!is.null(var1) && !is.null(var2) && is.numeric(var1) && is.numeric(var2)) {
          cor_test <- tryCatch(
            stats::cor.test(var1, var2),
            error = function(e) NULL
          )
          
          if (!is.null(cor_test)) {
            cor_val <- cor_test$estimate  # Keep sign for display
            cor_p <- cor_test$p.value
            
            # Format correlation: 2 decimals, † for p<.1, * for p<.05, ** for p<.01
            cor_formatted <- format(round(cor_val, 2), nsmall = 2)
            if (!is.na(cor_p) && cor_p < 0.01) {
              cor_formatted <- paste0(cor_formatted, "**")
            } else if (!is.na(cor_p) && cor_p < 0.05) {
              cor_formatted <- paste0(cor_formatted, "*")
            } else if (!is.na(cor_p) && cor_p < 0.1) {
              cor_formatted <- paste0(cor_formatted, "\u2020")  # † character
            }
            cor_rxz[i] <- cor_formatted
            
            # X if correlation |r| > .3 OR p < .05
            if ((!is.na(cor_val) && abs(cor_val) > 0.3) || (!is.na(cor_p) && cor_p < 0.05)) {
              flags <- c(flags, "X")
              has_cor_flags <- TRUE
            }
          }
        }
      }
    }
    
    se_flag[i] <- paste(flags, collapse = " ")
  }
  
  # Format B column, using "  --" for intercept or NA (with extra padding for alignment)
  B_formatted <- sapply(seq_along(tbl$B), function(i) {
    if (tbl$term[i] == "(Intercept)") return("  --")
    val <- smart_round(tbl$B[i])
    if (is.na(val)) return("  --")
    val
  })
  
  # Format mean column: mean for numeric vars, % only when mean_is_pct is TRUE (factor levels / factor*factor)
  mean_is_pct <- if (!is.null(tbl$mean_is_pct)) tbl$mean_is_pct else rep(FALSE, nrow(tbl))
  mean_formatted <- sapply(seq_along(tbl$mean_or_pct), function(i) {
    if (tbl$term[i] == "(Intercept)") return("--")
    if (is.na(tbl$mean_or_pct[i])) return("--")
    val <- tbl$mean_or_pct[i]
    if (mean_is_pct[i]) {
      paste0(format(round(val, 1), nsmall = 1), "%")
    } else {
      smart_round(val)
    }
  })
  
  # Format missing column, using "--" for intercept instead of NA
  missing_formatted <- if (!is.null(na_counts) && length(na_counts) == nrow(tbl)) {
    sapply(seq_along(na_counts), function(i) {
      if (tbl$term[i] == "(Intercept)") return("--")
      if (is.na(na_counts[i])) return("--")
      as.character(na_counts[i])
    })
  } else {
    NULL
  }
  
  # Build display dataframe from scratch with desired column order
  # Add leading padding spaces to values for better column separation
  pad <- function(x, n) paste0(strrep(" ", n), x)
  
  # Right-align values within column, then add trailing space
  right_align <- function(x, trail = 1) {
    max_width <- max(nchar(x))
    formatted <- format(x, width = max_width, justify = "right")
    paste0(formatted, strrep(" ", trail))
  }
  
  # Check if we have clustered SE
  has_clusters <- attr(x, "has_clusters")
  if (is.null(has_clusters)) has_clusters <- FALSE
  
  # Format columns
  # Add significance stars to estimates: † p<.1, * p<.05, ** p<.01
  # Pad stars to consistent width (2 chars) for alignment
  estimate_vals <- sapply(seq_along(tbl$estimate), function(i) {
    est <- smart_round(tbl$estimate[i])
    p <- tbl$p.value[i]
    if (is.na(p)) return(paste0(est, "  "))
    if (p < 0.01) {
      paste0(est, "**")
    } else if (p < 0.05) {
      paste0(est, "* ")
    } else if (p < 0.1) {
      paste0(est, "\u2020 ")  # † character + space
    } else {
      paste0(est, "  ")
    }
  })
  t_vals <- sapply(tbl$t, smart_round)
  
  # Check if df varies across coefficients
  df_vals <- tbl$df
  df_varies <- length(unique(round(df_vals, 2))) > 1
  
  # Format term names: use parsed factor terms (e.g., "group (B)"), with 2 trailing spaces
  # Only lowercase "(Intercept)" -> "intercept", preserve original case for all other terms
  term_names_formatted <- sapply(seq_along(factor_info$formatted_terms), function(i) {
    term <- factor_info$formatted_terms[i]
    term <- gsub("^\\(Intercept\\)$", "intercept", term)
    paste0(term, "  ")
  })
  
  if (has_clusters) {
    # With clusters: SE.cluster, SE.robust (HC3), SE.classical
    if (df_varies) {
      # df varies: include as column between t.value and p.value
      display_df <- data.frame(
        term = term_names_formatted,
        estimate = right_align(estimate_vals, 1),
        SE.cluster = pad(sapply(tbl$SE.cluster, smart_round), 2),
        SE.robust = pad(sapply(tbl$SE.robust, smart_round), 2),
        SE.classical = pad(sapply(tbl$SE.classical, smart_round), 3),
        t.value = right_align(t_vals, 1),
        df = pad(sapply(df_vals, function(d) format(round(d, 1), nsmall = 1)), 1),
        p.value = pad(sapply(tbl$p.value, format_p), 1),
        std.estimate = pad(right_align(B_formatted, 0), 3),
        `  mean` = right_align(mean_formatted, 1),
        stringsAsFactors = FALSE,
        check.names = FALSE
      )
    } else {
      display_df <- data.frame(
        term = term_names_formatted,
        estimate = right_align(estimate_vals, 1),
        SE.cluster = pad(sapply(tbl$SE.cluster, smart_round), 2),
        SE.robust = pad(sapply(tbl$SE.robust, smart_round), 2),
        SE.classical = pad(sapply(tbl$SE.classical, smart_round), 3),
        t.value = right_align(t_vals, 1),
        p.value = pad(sapply(tbl$p.value, format_p), 1),
        std.estimate = pad(right_align(B_formatted, 0), 3),
        `  mean` = right_align(mean_formatted, 1),
        stringsAsFactors = FALSE,
        check.names = FALSE
      )
    }
  } else {
    # Without clusters: SE.robust, SE.classical
    if (df_varies) {
      # df varies: include as column between t.value and p.value
      display_df <- data.frame(
        term = term_names_formatted,
        estimate = right_align(estimate_vals, 1),
        SE.robust = pad(sapply(tbl$SE.robust, smart_round), 2),
        SE.classical = pad(sapply(tbl$SE.classical, smart_round), 3),
        t.value = right_align(t_vals, 1),
        df = pad(sapply(df_vals, function(d) format(round(d, 1), nsmall = 1)), 1),
        p.value = pad(sapply(tbl$p.value, format_p), 1),
        std.estimate = pad(right_align(B_formatted, 0), 3),
        `  mean` = right_align(mean_formatted, 1),
        stringsAsFactors = FALSE,
        check.names = FALSE
      )
    } else {
      display_df <- data.frame(
        term = term_names_formatted,
        estimate = right_align(estimate_vals, 1),
        SE.robust = pad(sapply(tbl$SE.robust, smart_round), 2),
        SE.classical = pad(sapply(tbl$SE.classical, smart_round), 3),
        t.value = right_align(t_vals, 1),
        p.value = pad(sapply(tbl$p.value, format_p), 1),
        std.estimate = pad(right_align(B_formatted, 0), 3),
        `  mean` = right_align(mean_formatted, 1),
        stringsAsFactors = FALSE,
        check.names = FALSE
      )
    }
  }
  
  # Add missing column if available
  if (!is.null(missing_formatted)) {
    display_df$missing <- pad(missing_formatted, 2)
  }
  
  # Add r(x,z) column for interaction correlations (only if there are interactions)
  if (has_interactions) {
    display_df$`r(x,z)` <- pad(right_align(cor_rxz, 0), 2)
  }
  
  # Add red flag column at the end (use "--" when no flag)
  display_df$red.flag <- pad(ifelse(se_flag == "", "--", se_flag), 4)
  
  # Insert baseline rows for factor variables (in reverse order to maintain correct positions)
  if (length(factor_info$baseline_info) > 0) {
    # Process in reverse order so row indices remain valid
    for (j in rev(seq_along(factor_info$baseline_info))) {
      bl <- factor_info$baseline_info[[j]]
      baseline_term <- paste0(bl$var_name, " (", bl$baseline_level, ")  ")
      
      # Create a baseline row with "[baseline]" placeholder
      baseline_row <- display_df[1, , drop = FALSE]  # Copy structure
      baseline_row$term <- baseline_term
      # Set all numeric columns to "[baseline]" indicator
      for (col in names(baseline_row)) {
        if (col == "term") next
        baseline_row[[col]] <- ""
      }
      baseline_row$estimate <- "[baseline]"
      
      # Calculate percentage for the baseline level (show in mean column like other factor levels)
      factor_col <- NULL
      if (!is.null(model_data) && bl$var_name %in% names(model_data)) {
        factor_col <- model_data[[bl$var_name]]
      }
      if (is.null(factor_col)) {
        # Fallback: get data from call (lm_robust often has no $model)
        cl <- attr(x, "lm2_call")
        if (!is.null(cl) && !is.null(cl$data)) {
          data_from_call <- tryCatch(eval(cl$data, envir = parent.frame()), error = function(e) NULL)
          if (!is.null(data_from_call) && bl$var_name %in% names(data_from_call)) {
            factor_col <- data_from_call[[bl$var_name]]
            # Align to model: use same rows as used in fit (model frame may subset)
            mf <- tryCatch(stats::model.frame(x$terms, data = data_from_call), error = function(e) NULL)
            if (!is.null(mf) && bl$var_name %in% names(mf)) factor_col <- mf[[bl$var_name]]
          }
        }
      }
      if (!is.null(factor_col)) {
        baseline_pct <- mean(factor_col == bl$baseline_level, na.rm = TRUE) * 100
        baseline_pct_str <- paste0(format(round(baseline_pct, 1), nsmall = 1), "%")
        existing_width <- max(nchar(display_df$`  mean`))
        baseline_row$`  mean` <- paste0(format(baseline_pct_str, width = existing_width - 1, justify = "right"), " ")
      }
      
      # Count missing observations for this factor variable
      # Get the NA count from an existing level of this factor in tbl
      baseline_na_count <- 0L
      for (k in seq_len(nrow(tbl))) {
        # Look for a term that starts with this factor's variable name
        if (startsWith(tbl$term[k], bl$var_name)) {
          # Get the na_count for this term from the na_counts attribute
          if (!is.null(na_counts) && !is.na(na_counts[k])) {
            baseline_na_count <- na_counts[k]
            break
          }
        }
      }
      baseline_row$missing <- paste0("  ", baseline_na_count)
      
      # Insert before the first level of this factor
      insert_pos <- bl$insert_before
      if (insert_pos == 1) {
        display_df <- rbind(baseline_row, display_df)
      } else if (insert_pos > nrow(display_df)) {
        display_df <- rbind(display_df, baseline_row)
      } else {
        display_df <- rbind(
          display_df[1:(insert_pos - 1), , drop = FALSE],
          baseline_row,
          display_df[insert_pos:nrow(display_df), , drop = FALSE]
        )
      }
    }
  }

  # Add fixed-effects rows (after coefficients) and adjust intercept label
  fe_terms <- attr(x, "fe_terms")
  fe_k <- attr(x, "fe_k")
  fe_missing <- attr(x, "fe_missing")
  has_fe <- !is.null(fe_terms) && length(fe_terms) > 0
  if (has_fe) {
    # Ensure intercept row is present when fixed effects are absorbed
    intercept_idx <- which(trimws(display_df$term) == "intercept")
    if (length(intercept_idx) == 0) {
      intercept_row <- setNames(as.list(rep("", length(names(display_df)))), names(display_df))
      intercept_row[["term"]] <- "intercept  "
      intercept_row[["estimate"]] <- "[dropped]"
      for (col in setdiff(names(display_df), c("term", "estimate"))) {
        if (col == "missing") {
          intercept_row[[col]] <- pad("--", 2)
        } else if (col == "SE.classical") {
          intercept_row[[col]] <- pad("--", 3)
        } else if (col %in% c("SE.robust", "SE.cluster")) {
          intercept_row[[col]] <- pad("--", 2)
        } else if (col == "t.value") {
          intercept_row[[col]] <- right_align("--", 1)
        } else if (col == "df") {
          intercept_row[[col]] <- pad("--", 1)
        } else if (col == "p.value") {
          intercept_row[[col]] <- pad("--", 1)
        } else if (col == "std.estimate") {
          intercept_row[[col]] <- pad(right_align("--", 0), 3)
        } else if (col == "  mean") {
          intercept_row[[col]] <- right_align("--", 1)
        } else if (col == "r(x,z)") {
          intercept_row[[col]] <- pad(right_align("--", 0), 2)
        } else if (col == "red.flag") {
          intercept_row[[col]] <- pad("--", 4)
        } else {
          intercept_row[[col]] <- "--"
        }
      }
      display_df <- rbind(as.data.frame(intercept_row, stringsAsFactors = FALSE, check.names = FALSE), display_df)
      intercept_idx <- 1
    }
    # Update intercept estimate to indicate absorption
    if (length(intercept_idx) > 0) {
      display_df$estimate[intercept_idx] <- "[dropped]"
    }
    
    make_fe_row <- function(term, k, missing) {
      row <- setNames(as.list(rep("", length(names(display_df)))), names(display_df))
      row[["term"]] <- paste0(term, "  ")
      estimate_val <- if (is.na(k)) "df=NA" else paste0("df=", k)
      row[["estimate"]] <- right_align(estimate_val, 1)
      
      for (col in setdiff(names(display_df), c("term", "estimate"))) {
        if (col == "missing") {
          missing_val <- if (is.na(missing)) "--" else as.character(missing)
          row[[col]] <- pad(missing_val, 2)
        } else if (col == "SE.classical") {
          row[[col]] <- pad("--", 3)
        } else if (col %in% c("SE.robust", "SE.cluster")) {
          row[[col]] <- pad("--", 2)
        } else if (col == "t.value") {
          row[[col]] <- right_align("--", 1)
        } else if (col == "df") {
          row[[col]] <- pad("--", 1)
        } else if (col == "p.value") {
          row[[col]] <- pad("--", 1)
        } else if (col == "std.estimate") {
          row[[col]] <- pad(right_align("--", 0), 3)
        } else if (col == "  mean") {
          row[[col]] <- right_align("--", 1)
        } else if (col == "r(x,z)") {
          row[[col]] <- pad(right_align("--", 0), 2)
        } else if (col == "red.flag") {
          row[[col]] <- pad("--", 4)
        } else {
          row[[col]] <- "--"
        }
      }
      as.data.frame(row, stringsAsFactors = FALSE, check.names = FALSE)
    }
    
    fe_rows <- lapply(fe_terms, function(term) {
      k <- if (!is.null(fe_k) && term %in% names(fe_k)) fe_k[[term]] else NA_integer_
      miss <- if (!is.null(fe_missing) && term %in% names(fe_missing)) fe_missing[[term]] else NA_integer_
      make_fe_row(term, k, miss)
    })
    fe_df <- do.call(rbind, fe_rows)
    display_df <- rbind(display_df, fe_df)
  }
  
  # Use term values as row names, then remove the term column
  rownames(display_df) <- display_df$term
  display_df$term <- NULL
  
  # Print the original call (use lm2_call attribute which stores the lm2() call)
  cat("Call: ")
  print(attr(x, "lm2_call"))
  cat("\n")
  
  # Print the table
  print(display_df, right = FALSE)
  
  # Print model summary info
  cat("\n")
  cat("N =", x$nobs, " | ")
  cat("missing =", attr(x, "n_missing"), " | ")
  if (!df_varies) {
    cat("df =", round(tbl$df[1], 0), " | ")
  }
  cat("R\u00b2 =", format(round(x$r.squared, 3), nsmall = 3), " | ")
  if (has_clusters) {
    cat("SE type: CR2 (cluster)\n")
  } else {
    cat("SE type:", x$se_type, "\n")
  }
  if (notes) {
    cat("\nNotes:\n")
    n_missing_y <- attr(x, "n_missing_y")
    y_name <- attr(x, "y_name")
    if (!is.null(n_missing_y) && !is.na(n_missing_y) && n_missing_y > 0) {
      y_label <- if (!is.null(y_name) && !is.na(y_name)) y_name else "DV"
      cat("  - ", n_missing_y, " observations have missing \"", y_label, "\" values\n", sep = "")
    }
    show_dagger <- any(!is.na(tbl$p.value) & tbl$p.value >= 0.05 & tbl$p.value < 0.1)
    show_star <- any(!is.na(tbl$p.value) & tbl$p.value >= 0.01 & tbl$p.value < 0.05)
    show_double_star <- any(!is.na(tbl$p.value) & tbl$p.value < 0.01)
    sig_parts <- character(0)
    if (show_dagger) sig_parts <- c(sig_parts, "\u2020 p<.1")
    if (show_star) sig_parts <- c(sig_parts, "* p<.05")
    if (show_double_star) sig_parts <- c(sig_parts, "** p<.01")
    sig_basis <- if (has_clusters) "SE.cluster" else "SE.robust"
    if (length(sig_parts) > 0) {
      cat("  - ", paste(sig_parts, collapse = ", "), " (based on ", sig_basis, ")\n", sep = "")
    } else {
      cat("  - All estimates are p>.1 (based on ", sig_basis, ")\n", sep = "")
    }
    if (has_clusters) {
      cat("  - SE.robust (HC3) used only to contrast with SE.classical to flag observations\n")
    }
    cat("  - std.estimate is the standardized coefficient: beta = b * sd(x) / sd(y)\n")
    if (has_interactions) {
      cat("  - mean: for main effects mean of x; for x*z interaction where z is factor, mean of x when z==1\n")
    } else {
      cat("  - mean: for numeric variables, mean of x; for factors, % of observations\n")
    }
    cat("  - missing: number of observations excluded due to missing values\n")
    if (has_interactions) {
      cat("  - r(x,z): correlation between interacted variables\n")
    }
    # Build red.flag note based on which flags are present
    if (has_se_flags || has_cor_flags) {
      cat("  - red.flag:\n")
      # SE flags explanation (only if present)
      if (has_se_flags) {
        present_se_flags <- intersect(
          c("!", "!!", "!!!"),
          unique(unlist(strsplit(se_flag, " ", fixed = TRUE)))
        )
        if (length(present_se_flags) > 0) {
          thresholds <- c("!" = "25%", "!!" = "50%", "!!!" = "100%")
          thresh_text <- paste(thresholds[present_se_flags], collapse = ", ")
          flags_text <- paste(present_se_flags, collapse = ", ")
          cat("     ", flags_text, ": robust & classical SE differ by more than ", thresh_text, "\n", sep = "")
        }
        # Get actual variable names for suggestions
        dv_name <- attr(x, "y_name")
        dv_label <- if (!is.null(dv_name) && !is.na(dv_name)) dv_name else "DV"
        
        # Find which variables have flags (flagged variables)
        flagged_vars <- character(0)
        for (fi in seq_along(se_flag)) {
          flag_val <- trimws(se_flag[fi])
          if (flag_val != "" && flag_val != "--") {
            term_name <- tbl$term[fi]
            if (term_name != "(Intercept)" && !grepl(":", term_name)) {
              flagged_vars <- c(flagged_vars, term_name)
            }
          }
        }
        flagged_label <- if (length(flagged_vars) > 0) paste(flagged_vars, collapse = ", ") else "flagged variable(s)"
        
        cat("     - Suggestion 1: to evaluate possible extreme skew or outliers:\n")
        cat("       plot_density() or plot_freq() for ", dv_label, " and for ", flagged_label, "\n", sep = "")
        cat("     - Suggestion 2: to evaluate possible nonlinearity, do plot_gam(", dv_label, " ~ ", flagged_label, ")\n", sep = "")
      }
      # Correlation flags explanation (only if present)
      if (has_cor_flags) {
        cat("     X: interacted variables are correlated, interaction term is likely to be biased\n")
        cat("        See Simonsohn (2024) \"Interacting with curves\" https://doi.org/10.1177/25152459231207787\n")
      }
    } else {
      if (has_interactions) {
        cat("  - red.flag: none (robust and classical SE are similar and interacted terms are not correlated)\n")
      } else {
        cat("  - red.flag: none (robust and classical SE are similar)\n")
      }
    }
    # Fixed effects note if absorbed (after red.flag)
    fe_terms <- attr(x, "fe_terms")
    if (!is.null(fe_terms) && length(fe_terms) > 0) {
      fe_vars <- paste(fe_terms, collapse = ", ")
      fe_note <- "- <vars> were entered as absorbed fixed effects; df is the number of unique values."
      fe_note <- gsub("<vars>", fe_vars, fe_note, fixed = TRUE)
      cat("  ", fe_note, "\n", sep = "")
    }
    cat("  - To avoid these notes, lm2(..., notes=FALSE)\n")
  }
  
  invisible(x)
}

#' Summary method for lm2 objects
#'
#' @param object An object of class \code{lm2}
#' @param ... Additional arguments passed to \code{\link{print.lm2}}
#'
#' @return Invisibly returns the original object
#' @export
summary.lm2 <- function(object, ...) {
  print(object, ...)
  message2("print() and summary() show the same information for lm2()", col = "blue")
  invisible(object)
}

#' Predict method for lm2 objects
#'
#' @param object An object of class \code{lm2}
#' @param newdata An optional data frame in which to look for variables with 
#'   which to predict. If omitted, the original model data is used.
#' @param ... Additional arguments passed to \code{\link[estimatr]{predict.lm_robust}},
#'   including \code{se.fit} and \code{interval}.
#'
#' @return A vector of predicted values (or a list with \code{fit} and \code{se.fit} 
#'   if \code{se.fit = TRUE}, or a matrix with \code{fit}, \code{lwr}, \code{upr} 
#'   if \code{interval} is specified)
#' @export
predict.lm2 <- function(object, newdata, ...) {
  # If newdata is not provided, use the original model data
  if (missing(newdata)) {
    # Get the original data from the model frame
    newdata <- model.frame(object)
  }
  # Remove lm2 class temporarily and call predict on the lm_robust object
  class(object) <- setdiff(class(object), "lm2")
  predict(object, newdata = newdata, ...)
}

# Tests for lm2() function

#lm2_001
test_that("lm2 returns statuser output by default", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt + hp, data = mtcars)
  
  # Check it has the correct classes (lm2 first, then inherits from lm_robust)
  expect_s3_class(result, "lm2")
  expect_s3_class(result, "lm_robust")
  
  # Check the statuser_table attribute exists and has expected columns
  tbl <- attr(result, "statuser_table")
  expect_true(!is.null(tbl))
  expected_cols <- c("term", "estimate", "SE.robust", "SE.classical", "t", "df", "p.value", "B")
  expect_true(all(expected_cols %in% names(tbl)))
  
  # Check correct number of rows (intercept + 2 predictors)
  expect_equal(nrow(tbl), 3)
  
  # Check term names
  expect_equal(tbl$term, c("(Intercept)", "wt", "hp"))
})

#lm2_002
test_that("lm2 uses HC3 standard errors by default", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt + hp, data = mtcars)
  
  # Check se_type (it's a list element since lm2 inherits from lm_robust)
  expect_equal(result$se_type, "HC3")
})

#lm2_003
test_that("lm2 standardized coefficients are calculated correctly", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt + hp, data = mtcars)
  tbl <- attr(result, "statuser_table")
  
  # Intercept should have NA for standardized coefficient
  expect_true(is.na(tbl$B[tbl$term == "(Intercept)"]))
  
  # Other coefficients should have non-NA standardized values
  expect_false(is.na(tbl$B[tbl$term == "wt"]))
  expect_false(is.na(tbl$B[tbl$term == "hp"]))
  
  # Manually calculate standardized coefficient for 'wt'
  b_wt <- tbl$estimate[tbl$term == "wt"]
  sd_wt <- sd(mtcars$wt)
  sd_mpg <- sd(mtcars$mpg)
  expected_B_wt <- b_wt * (sd_wt / sd_mpg)
  
  expect_equal(tbl$B[tbl$term == "wt"], expected_B_wt, tolerance = 1e-10)
})

#lm2_004
test_that("lm2 robust and classical SEs differ", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt + hp, data = mtcars)
  tbl <- attr(result, "statuser_table")
  
  # Robust and classical SEs should generally differ
  # (they might be similar but not exactly equal)
  expect_false(all(tbl$SE.robust == tbl$SE.classical))
})

#lm2_005
test_that("lm2 passes additional arguments to lm_robust", {
  skip_if_not_installed("estimatr")
  
  # Test with different se_type
  result_hc2 <- lm2(mpg ~ wt + hp, data = mtcars, se_type = "HC2")
  result_hc3 <- lm2(mpg ~ wt + hp, data = mtcars, se_type = "HC3")
  
  tbl_hc2 <- attr(result_hc2, "statuser_table")
  tbl_hc3 <- attr(result_hc3, "statuser_table")
  
  # SEs should differ between HC2 and HC3
  expect_false(all(tbl_hc2$SE.robust == tbl_hc3$SE.robust))
})

#lm2_006
test_that("lm2 stores model attributes", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt + hp, data = mtcars)
  
  # Check that it inherits from lm_robust (so it has these as list elements, not attributes)
  expect_true(!is.null(result$r.squared))
  expect_true(!is.null(result$adj.r.squared))
  expect_true(!is.null(result$nobs))
  
  # Check statuser-specific attributes exist
  expect_true(!is.null(attr(result, "statuser_table")))
  expect_true(!is.null(attr(result, "classical_fit")))
})

#lm2_007
test_that("lm2 predict matches lm_robust predict with clustering", {
  skip_if_not_installed("estimatr")
  
  # Create clustered data
  set.seed(123)
  cluster_id <- rep(1:10, each = 20)
  x <- rnorm(200)
  y <- x + rnorm(200) + rnorm(10)[cluster_id]
  test_data <- data.frame(y = y, x = x, cluster_id = cluster_id)
  
  # Fit with lm2 and lm_robust
 m1 <- lm2(y ~ x, data = test_data, clusters = cluster_id)
  m2 <- estimatr::lm_robust(y ~ x, data = test_data, clusters = cluster_id)
  
  # Get predictions with SEs
  yh1 <- predict(m1, newdata = test_data, se.fit = TRUE)
  yh2 <- predict(m2, newdata = test_data, se.fit = TRUE)
  
  # Fitted values should be identical
  expect_equal(mean(yh1$fit == yh2$fit), 1)
  
  # Standard errors should be identical
  expect_equal(mean(yh1$se.fit == yh2$se.fit), 1)
})

#lm2_008
test_that("lm2 predict matches lm_robust predict without clustering", {
  skip_if_not_installed("estimatr")
  
  # Fit with lm2 and lm_robust
  m1 <- lm2(mpg ~ wt + hp, data = mtcars)
  m2 <- estimatr::lm_robust(mpg ~ wt + hp, data = mtcars, se_type = "HC3")
  
  # Get predictions with SEs
  yh1 <- predict(m1, newdata = mtcars, se.fit = TRUE)
  yh2 <- predict(m2, newdata = mtcars, se.fit = TRUE)
  
  # Fitted values should be identical
  expect_equal(mean(yh1$fit == yh2$fit), 1)
  
  # Standard errors should be identical
  expect_equal(mean(yh1$se.fit == yh2$se.fit), 1)
})

#lm2_009
test_that("lm2 predict without newdata matches lm predict", {
  skip_if_not_installed("estimatr")
  
  # Fit with lm2 and lm
  m1 <- lm2(mpg ~ wt + hp, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  
  # Get predictions without newdata
  yh1 <- predict(m1)
  yh2 <- predict(m2)
  
  # Fitted values should be identical
  expect_equal(unname(yh1), unname(yh2), tolerance = 1e-10)
})

#lm2_010
test_that("lm2 predict without newdata matches lm_robust predict with newdata", {
  skip_if_not_installed("estimatr")
  
  # Fit with lm2 and lm_robust
  m1 <- lm2(mpg ~ wt + hp, data = mtcars)
  m2 <- estimatr::lm_robust(mpg ~ wt + hp, data = mtcars, se_type = "HC3")
  
  # lm2 without newdata vs lm_robust with newdata
  yh1 <- predict(m1)
  yh2 <- predict(m2, newdata = mtcars)
  
  # Fitted values should be identical
  expect_equal(unname(yh1), unname(yh2), tolerance = 1e-10)
})

#lm2_011
test_that("lm2 predict with se.fit=TRUE works without newdata", {
  skip_if_not_installed("estimatr")
  
  # Fit with lm2 and lm_robust
  m1 <- lm2(mpg ~ wt + hp, data = mtcars)
  m2 <- estimatr::lm_robust(mpg ~ wt + hp, data = mtcars, se_type = "HC3")
  
  # lm2 without newdata but with se.fit=TRUE
  yh1 <- predict(m1, se.fit = TRUE)
  yh2 <- predict(m2, newdata = mtcars, se.fit = TRUE)
  
  # Fitted values should be identical
  expect_equal(unname(yh1$fit), unname(yh2$fit), tolerance = 1e-10)
  
  # Standard errors should be identical
  expect_equal(unname(yh1$se.fit), unname(yh2$se.fit), tolerance = 1e-10)
})

#lm2_012
test_that("lm2 is compatible with marginaleffects", {
  skip_if_not_installed("estimatr")
  skip_if_not_installed("marginaleffects")
  
  # lm2 should work with marginaleffects since it inherits from lm_robust
  result <- lm2(mpg ~ wt + hp, data = mtcars)
  
  # Test that slopes() works
  slopes_result <- marginaleffects::slopes(result)
  expect_true(inherits(slopes_result, "data.frame"))
  expect_true(nrow(slopes_result) > 0)
  
  # Test that predictions() works
  pred_result <- marginaleffects::predictions(result)
  expect_true(inherits(pred_result, "data.frame"))
  expect_equal(nrow(pred_result), nrow(mtcars))
})

# ============================================================================
# RED FLAG DETECTION TESTS
# ============================================================================

#lm2_013
test_that("lm2 shows red flag (!) when SE differs by >25%", {
  skip_if_not_installed("estimatr")
  
  # Create data with moderate heteroskedasticity
  set.seed(42)
  x <- 1:50
  y <- 2 * x + rnorm(50, sd = x * 0.3)  # Variance increases with x
  
  result <- lm2(y ~ x)
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  

  # Should show at least one red flag indicator (!, !!, or !!!)
  expect_true(
    grepl("!", output_text, fixed = TRUE),
    info = "Expected red flag (!) for heteroskedastic data"
  )
})

#lm2_014
test_that("lm2 shows severe red flag (!!!) for extreme heteroskedasticity", {
  skip_if_not_installed("estimatr")
  
  # Create data with severe heteroskedasticity (SE should differ by >100%)
  # Use stronger heteroskedasticity by making variance grow with x^2
  set.seed(42)
  x <- 1:100
  y <- x + rnorm(100, sd = x^1.5)  # Very strong heteroskedasticity
  
  result <- lm2(y ~ x)
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  
  # Should show at least one red flag (!, !!, or !!!)
  expect_true(
    grepl("!", output_text, fixed = TRUE),
    info = "Expected red flag for heteroskedasticity"
  )
})

#lm2_015
test_that("lm2 shows X flag for significantly correlated interaction terms", {
  skip_if_not_installed("estimatr")
  
  # Create data with highly correlated predictors in an interaction
  set.seed(456)
  n <- 200
  x1 <- rnorm(n)
  x2 <- x1 + rnorm(n, sd = 0.2)  # x2 highly correlated with x1
  y <- x1 + x2 + x1 * x2 + rnorm(n)
  
  result <- lm2(y ~ x1 * x2)
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  
  # Should show X flag in red.flag column for the interaction term
  # (correlation significance stars now appear in r(x,z) column separately)
  expect_true(
    grepl("\\bX\\b", output_text),
    info = "Expected X flag for significantly correlated interaction terms"
  )
  
  # Additionally check that significant correlation is shown in r(x,z) column
  expect_true(
    grepl("r\\(x,z\\)", output_text),
    info = "Expected r(x,z) column for interaction models"
  )
})

#lm2_016
test_that("lm2 shows X flag for moderately correlated interaction terms", {
  skip_if_not_installed("estimatr")
  
  # Create data with moderately correlated predictors (|r| > 0.3)
  # Using larger sample size to ensure correlation is detectable
  set.seed(789)
  n <- 100
  x1 <- rnorm(n)
  x2 <- 0.5 * x1 + rnorm(n, sd = 0.8)  # Moderate correlation ~0.5
  y <- x1 + x2 + x1 * x2 + rnorm(n)
  
  result <- lm2(y ~ x1 * x2)
  output <- capture.output(print(result))
  
  # Find the line with the interaction term's red.flag column
  # The X flag appears when |r| > 0.3 or p < .05 for the correlation
  interaction_line <- grep("x1:x2", output, value = TRUE)
  
  # Should show X flag in the red.flag column for the interaction
  has_x_flag <- any(grepl("\\bX\\b", interaction_line))
  expect_true(
    has_x_flag,
    info = "Expected X flag for correlated interaction terms"
  )
})

#lm2_017
test_that("lm2 does not show interaction flags for uncorrelated terms", {
  skip_if_not_installed("estimatr")
  
  # Create data with uncorrelated predictors
  set.seed(111)
  n <- 100
  x1 <- rnorm(n)
  x2 <- rnorm(n)  # Independent of x1
  y <- x1 + x2 + x1 * x2 + rnorm(n)
  
  result <- lm2(y ~ x1 * x2)
  output <- capture.output(print(result))
  
  # Find the line with the interaction term
  interaction_line <- grep("x1:x2", output, value = TRUE)
  
  # Should not show X or X* in the red.flag column for the interaction
  # The red.flag column is the last column, so check it doesn't contain X as a flag
  if (length(interaction_line) > 0) {
    # The interaction line should have x1:x2 in the term name
    # Check that X* flag is not present (X by itself is part of the term name)
    # X* would indicate significant correlation between predictors
    expect_false(
      any(grepl("X\\*", interaction_line)),
      info = "Should not show X* flag for uncorrelated interaction terms"
    )
  }
})

# ============================================================================
# CLUSTERED STANDARD ERRORS TESTS
# ============================================================================

#lm2_018
test_that("lm2 with clusters shows SE.cluster column", {
  skip_if_not_installed("estimatr")
  
  # Create clustered data
  set.seed(222)
  cluster_id <- rep(1:20, each = 10)
  x <- rnorm(200)
  cluster_effect <- rnorm(20)[cluster_id]
  y <- x + cluster_effect + rnorm(200)
  test_data <- data.frame(y = y, x = x, cluster_id = cluster_id)
  
  result <- lm2(y ~ x, data = test_data, clusters = cluster_id)
  
  # Check that SE.cluster is in the statuser_table
  tbl <- attr(result, "statuser_table")
  expect_true("SE.cluster" %in% names(tbl))
  
  # SE.cluster should differ from SE.robust
  expect_false(all(tbl$SE.cluster == tbl$SE.robust))
})

#lm2_019
test_that("lm2 with clusters prints cluster info", {
  skip_if_not_installed("estimatr")
  
  set.seed(333)
  cluster_id <- rep(1:15, each = 10)
  x <- rnorm(150)
  y <- x + rnorm(15)[cluster_id] + rnorm(150)
  test_data <- data.frame(y = y, x = x, cluster_id = cluster_id)
  
  result <- lm2(y ~ x, data = test_data, clusters = cluster_id)
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  
  # Should mention CR2 (cluster-robust) SE type
  expect_true(
    grepl("CR2", output_text),
    info = "Should mention CR2 for clustered SE"
  )
  
  # Should mention cluster in output
  expect_true(
    grepl("cluster", output_text, ignore.case = TRUE),
    info = "Should mention 'cluster' in output"
  )
})

#lm2_020
test_that("lm2 clustered SE is larger than robust SE when there's clustering", {
  skip_if_not_installed("estimatr")
  
  # Create data with strong within-cluster correlation
  set.seed(444)
  cluster_id <- rep(1:10, each = 30)
  x <- rnorm(300)
  cluster_effect <- rnorm(10, sd = 3)[cluster_id]  # Strong cluster effects
  y <- x + cluster_effect + rnorm(300, sd = 0.5)
  test_data <- data.frame(y = y, x = x, cluster_id = cluster_id)
  
  result <- lm2(y ~ x, data = test_data, clusters = cluster_id)
  tbl <- attr(result, "statuser_table")
  
  # For the x coefficient, clustered SE should be larger than robust SE
  # due to the within-cluster correlation
  x_row <- tbl[tbl$term == "x", ]
  expect_true(
    x_row$SE.cluster > x_row$SE.robust,
    info = "Clustered SE should be larger when there's within-cluster correlation"
  )
})

# ============================================================================
# PRINT METHOD TESTS
# ============================================================================

#lm2_021
test_that("print.lm2 shows notes by default", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt, data = mtcars)
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  
  expect_true(grepl("Notes:", output_text))
  expect_true(grepl("std.estimate", output_text))
  expect_true(grepl("red.flag", output_text))
})

#lm2_022
test_that("print.lm2 respects notes=FALSE", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt, data = mtcars)
  output <- capture.output(print(result, notes = FALSE))
  output_text <- paste(output, collapse = "\n")
  
  expect_false(grepl("Notes:", output_text))
})

#lm2_023
test_that("print.lm2 respects notes=FALSE from lm2 call", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt, data = mtcars, notes = FALSE)
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  
  expect_false(grepl("Notes:", output_text))
})

#lm2_024
test_that("print.lm2 shows Call correctly", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt + hp, data = mtcars)
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  
  expect_true(grepl("Call:", output_text))
  expect_true(grepl("lm2", output_text))
})

#lm2_025
test_that("print.lm2 shows model summary statistics", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt, data = mtcars)
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  
  expect_true(grepl("N =", output_text))
  expect_true(grepl("R²", output_text) || grepl("R2", output_text))
  expect_true(grepl("missing", output_text))
})

#lm2_026
test_that("print.lm2 shows interaction notes when model has interactions", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt * hp, data = mtcars)
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  
  # Should show interaction-specific notes
  expect_true(
    grepl("X:", output_text) || grepl("interaction", output_text, ignore.case = TRUE),
    info = "Should show interaction-related notes"
  )
})

# ============================================================================
# COEFFICIENT CONSISTENCY TESTS
# ============================================================================

#lm2_027
test_that("lm2 produces same coefficients as lm", {
  skip_if_not_installed("estimatr")
  
  result_lm2 <- lm2(mpg ~ wt + hp, data = mtcars)
  result_lm <- lm(mpg ~ wt + hp, data = mtcars)
  
  expect_equal(
    unname(coef(result_lm2)),
    unname(coef(result_lm)),
    tolerance = 1e-10
  )
})

#lm2_028
test_that("lm2 produces same estimates as lm_robust", {
  skip_if_not_installed("estimatr")
  
  # Include fixed effects to match absorbed model behavior
  set.seed(101)
  n_groups <- 20
  n_per_group <- 10
  group <- factor(rep(1:n_groups, each = n_per_group))
  x <- rnorm(n_groups * n_per_group)
  y <- 1 + 0.7 * x + rnorm(n_groups)[group] + rnorm(n_groups * n_per_group)
  fe_data <- data.frame(y = y, x = x, group = group)
  
  result_lm2 <- lm2(y ~ x, data = fe_data, fixed_effects = ~ group)
  result_lr <- estimatr::lm_robust(y ~ x, data = fe_data, fixed_effects = ~ group, se_type = "HC3")
  
  expect_equal(
    unname(coef(result_lm2)),
    unname(coef(result_lr)),
    tolerance = 1e-10
  )
})

#lm2_029
test_that("lm2 produces same R-squared as lm", {
  skip_if_not_installed("estimatr")
  
  result_lm2 <- lm2(mpg ~ wt + hp, data = mtcars)
  result_lm <- lm(mpg ~ wt + hp, data = mtcars)
  
  expect_equal(
    result_lm2$r.squared,
    summary(result_lm)$r.squared,
    tolerance = 1e-10
  )
})

#lm2_030
test_that("lm2 classical SE matches lm SE", {
  skip_if_not_installed("estimatr")
  
  result_lm2 <- lm2(mpg ~ wt + hp, data = mtcars)
  result_lm <- lm(mpg ~ wt + hp, data = mtcars)
  
  tbl <- attr(result_lm2, "statuser_table")
  lm_se <- summary(result_lm)$coefficients[, "Std. Error"]
  
  expect_equal(
    tbl$SE.classical,
    unname(lm_se),
    tolerance = 1e-10
  )
})

#lm2_031
test_that("lm2 mean column shows mean for numeric, % for factor levels", {
  skip_if_not_installed("estimatr")

  set.seed(1)
  n <- 50
  d <- data.frame(
    x = runif(n, 0, 10),
    group = factor(sample(c("A", "B", "C"), n, replace = TRUE)),
    y = 0
  )
  d$y <- 2 * d$x + as.numeric(d$group) + rnorm(n)

  m <- lm2(y ~ x + group, data = d, notes = FALSE)
  tbl <- attr(m, "statuser_table")

  expect_true("x" %in% tbl$term)
  if ("mean_is_pct" %in% names(tbl)) {
    # x is numeric: display as mean, not %
    expect_false(tbl$mean_is_pct[tbl$term == "x"])
    # at least one factor level term displayed as %
    expect_true(any(tbl$mean_is_pct))
  }
})

# ============================================================================
# EDGE CASES
# ============================================================================

#lm2_032
test_that("lm2 handles missing values correctly", {
  skip_if_not_installed("estimatr")
  
  # Create data with missing values
  data_with_na <- mtcars
  data_with_na$mpg[c(1, 5, 10)] <- NA
  data_with_na$wt[c(3, 7)] <- NA
  
  result <- lm2(mpg ~ wt + hp, data = data_with_na)
  
  # Should report missing observations
  expect_true(attr(result, "n_missing") > 0)
  
  # Output should mention missing
  output <- capture.output(print(result))
  output_text <- paste(output, collapse = "\n")
  expect_true(grepl("missing", output_text))
})

#lm2_033
test_that("lm2 handles single predictor", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt, data = mtcars)
  
  tbl <- attr(result, "statuser_table")
  expect_equal(nrow(tbl), 2)  # intercept + 1 predictor
})

#lm2_034
test_that("lm2 handles many predictors", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt + hp + drat + qsec + vs + am, data = mtcars)
  
  tbl <- attr(result, "statuser_table")
  expect_equal(nrow(tbl), 7)  # intercept + 6 predictors
})

# ============================================================================
# SNAPSHOT TESTS FOR OUTPUT FORMAT
# ============================================================================

#lm2_035
test_that("lm2 print output format is stable", {
  skip_if_not_installed("estimatr")
  
  # Use mtcars for reproducible output
  result <- lm2(mpg ~ wt + hp, data = mtcars, notes = FALSE)
  
  expect_snapshot(print(result))
})

#lm2_036
test_that("lm2 print output with notes is stable", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt, data = mtcars, notes = TRUE)
  
  expect_snapshot(print(result))
})

#lm2_037
test_that("lm2 print output with interaction is stable", {
  skip_if_not_installed("estimatr")
  
  result <- lm2(mpg ~ wt * hp, data = mtcars, notes = FALSE)
  
  expect_snapshot(print(result))
})

# ============================================================================
# TESTS WITHOUT DATA ARGUMENT (VARIABLES FROM ENVIRONMENT)
# ============================================================================

#lm2_038
test_that("lm2 works without data argument (variables from environment)", {
  skip_if_not_installed("estimatr")
  
  # Without data argument (variables from environment)
  y <- mtcars$mpg
  x1 <- mtcars$wt
  x2 <- mtcars$hp
  
  result <- lm2(y ~ x1 + x2)
  
  # Check it has the correct classes
  expect_s3_class(result, "lm2")
  expect_s3_class(result, "lm_robust")
  
  # Check the statuser_table attribute exists and has expected columns
  tbl <- attr(result, "statuser_table")
  expect_true(!is.null(tbl))
  expect_equal(nrow(tbl), 3)  # intercept + 2 predictors
  expect_equal(tbl$term, c("(Intercept)", "x1", "x2"))
})

#lm2_039
test_that("lm2 without data matches estimatr::lm_robust without data", {
  skip_if_not_installed("estimatr")
  
  y <- mtcars$mpg
  x1 <- mtcars$wt
  x2 <- mtcars$hp
  
  m1 <- lm2(y ~ x1 + x2)
  m2 <- estimatr::lm_robust(y ~ x1 + x2, se_type = "HC3")
  
  # Coefficients should match
  expect_equal(unname(coef(m1)), unname(coef(m2)), tolerance = 1e-10)
  
  # R-squared should match
  expect_equal(m1$r.squared, m2$r.squared, tolerance = 1e-10)
})

# ============================================================================
# MARGINALEFFECTS COMPATIBILITY TESTS
# ============================================================================

#lm2_040
test_that("lm2 slopes match lm_robust slopes with data argument", {
  skip_if_not_installed("estimatr")
  skip_if_not_installed("marginaleffects")
  
  m1 <- lm2(mpg ~ wt + hp, data = mtcars)
  m2 <- estimatr::lm_robust(mpg ~ wt + hp, data = mtcars, se_type = "HC3")
  
  slopes1 <- marginaleffects::slopes(m1)
  slopes2 <- marginaleffects::slopes(m2)
  
  # Estimates should match
  expect_equal(slopes1$estimate, slopes2$estimate, tolerance = 1e-10)
  
  # Standard errors should match
  expect_equal(slopes1$std.error, slopes2$std.error, tolerance = 1e-10)
})

#lm2_041
test_that("lm2 slopes match lm_robust slopes without data argument", {
  skip_if_not_installed("estimatr")
  skip_if_not_installed("marginaleffects")
  
  y <- mtcars$mpg
  x1 <- mtcars$wt
  x2 <- mtcars$hp
  
  m1 <- lm2(y ~ x1 + x2)
  m2 <- estimatr::lm_robust(y ~ x1 + x2, se_type = "HC3")
  
  slopes1 <- marginaleffects::slopes(m1)
  slopes2 <- marginaleffects::slopes(m2)
  
  # Estimates should match
  expect_equal(slopes1$estimate, slopes2$estimate, tolerance = 1e-10)
  
  # Standard errors should match
  expect_equal(slopes1$std.error, slopes2$std.error, tolerance = 1e-10)
})

#lm2_042
test_that("lm2 comparisons work with marginaleffects", {
  skip_if_not_installed("estimatr")
  skip_if_not_installed("marginaleffects")
  
  result <- lm2(mpg ~ wt + hp, data = mtcars)
  
  # Test comparisons() works
  comp_result <- marginaleffects::comparisons(result)
  expect_true(inherits(comp_result, "data.frame"))
  expect_true(nrow(comp_result) > 0)
})

#lm2_043
test_that("lm2 avg_slopes works with marginaleffects", {
  skip_if_not_installed("estimatr")
  skip_if_not_installed("marginaleffects")
  
  result <- lm2(mpg ~ wt + hp, data = mtcars)
  
  # Test avg_slopes() works
  avg_result <- marginaleffects::avg_slopes(result)
  expect_true(inherits(avg_result, "data.frame"))
  expect_equal(nrow(avg_result), 2)  # 2 predictors
})

# ============================================================================
# BROOM COMPATIBILITY TESTS
# ============================================================================

#lm2_044
test_that("lm2 works with broom::tidy", {
  skip_if_not_installed("estimatr")
  skip_if_not_installed("broom")
  
  m1 <- lm2(mpg ~ wt + hp, data = mtcars)
  
  # broom::tidy should work (via estimatr's method)
  tidy_result <- broom::tidy(m1)
  
  expect_true(inherits(tidy_result, "data.frame"))
  expect_true("term" %in% names(tidy_result))
  expect_true("estimate" %in% names(tidy_result))
  expect_true("std.error" %in% names(tidy_result))
  expect_equal(nrow(tidy_result), 3)  # intercept + 2 predictors
})

#lm2_045
test_that("lm2 works with broom::glance", {
  skip_if_not_installed("estimatr")
  skip_if_not_installed("broom")
  
  m1 <- lm2(mpg ~ wt + hp, data = mtcars)
  
  # broom::glance should work
  glance_result <- broom::glance(m1)
  
  expect_true(inherits(glance_result, "data.frame"))
  expect_equal(nrow(glance_result), 1)
  expect_true("r.squared" %in% names(glance_result))
  expect_true("nobs" %in% names(glance_result))
})

#lm2_046
test_that("lm2 works with broom::augment", {
  skip_if_not_installed("estimatr")
  skip_if_not_installed("broom")
  
  m1 <- lm2(mpg ~ wt + hp, data = mtcars)
  
  # broom::augment is not supported for lm_robust objects
  # so we just verify that fitted values can be extracted another way
  fitted_vals <- predict(m1, newdata = mtcars)
  expect_equal(length(fitted_vals), nrow(mtcars))
})

# ============================================================================
# MODELSUMMARY COMPATIBILITY TESTS
# ============================================================================

#lm2_047
test_that("lm2 works with modelsummary", {
  skip_if_not_installed("estimatr")
  skip_if_not_installed("modelsummary")
  
  m1 <- lm2(mpg ~ wt + hp, data = mtcars)
  
  # modelsummary should not error
  output <- capture.output(
    modelsummary::modelsummary(m1, output = "markdown")
  )
  
  expect_true(length(output) > 0)
  output_text <- paste(output, collapse = "\n")
  
  # Should contain coefficient info
  expect_true(grepl("wt", output_text, ignore.case = TRUE))
})

#lm2_048
test_that("modelsummary can compare lm2 and lm_robust models", {
  skip_if_not_installed("estimatr")
  skip_if_not_installed("modelsummary")
  
  m1 <- lm2(mpg ~ wt + hp, data = mtcars)
  m2 <- estimatr::lm_robust(mpg ~ wt + hp, data = mtcars, se_type = "HC3")
  
  # modelsummary should handle mixed model types
  output <- capture.output(
    modelsummary::modelsummary(list("lm2" = m1, "lm_robust" = m2), output = "markdown")
  )
  
  expect_true(length(output) > 0)
})

#lm2_049
test_that("modelsummary can compare multiple lm2 models", {
  skip_if_not_installed("estimatr")
  skip_if_not_installed("modelsummary")
  
  m1 <- lm2(mpg ~ wt, data = mtcars)
  m2 <- lm2(mpg ~ wt + hp, data = mtcars)
  m3 <- lm2(mpg ~ wt + hp + drat, data = mtcars)
  
  # modelsummary should handle multiple lm2 models
  output <- capture.output(
    modelsummary::modelsummary(
      list("Model 1" = m1, "Model 2" = m2, "Model 3" = m3), 
      output = "markdown"
    )
  )
  
  expect_true(length(output) > 0)
  output_text <- paste(output, collapse = "\n")
  
  # Should contain all model names
  expect_true(grepl("Model 1", output_text))
  expect_true(grepl("Model 2", output_text))
  expect_true(grepl("Model 3", output_text))
})

# ============================================================================
# COEFFICIENT EXTRACTION COMPATIBILITY TESTS
# ============================================================================

#lm2_050
test_that("lm2 coef extraction works like lm_robust", {
  skip_if_not_installed("estimatr")
  
  m1 <- lm2(mpg ~ wt + hp, data = mtcars)
  m2 <- estimatr::lm_robust(mpg ~ wt + hp, data = mtcars, se_type = "HC3")
  
  # coef() should work
  expect_equal(coef(m1), coef(m2))
  
  # vcov() should work
  expect_equal(vcov(m1), vcov(m2))
  
  # confint() should work
  expect_equal(confint(m1), confint(m2))
})

# Tests for interprobe()

#interprobe_001
test_that("interprobe works with lm() model input and returns expected structure", {
  skip_if_not_installed("marginaleffects")

  set.seed(111)
  n <- 600
  x1 <- rnorm(n)
  z1 <- rnorm(n, mean = 10, sd = 2)
  y.raw <- x1 * z1
  e <- rnorm(n, sd = sd(y.raw))
  y1 <- y.raw + e

  df <- data.frame(x1, y1, z1)
  lm1 <- lm(y1 ~ x1 * z1, data = df)

  grDevices::pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(grDevices::dev.off(), add = TRUE)

  res <- interprobe(
    model = lm1,
    x = "x1",
    z = "z1",
    draw = "jn",
    histogram = FALSE,
    quiet = TRUE,
    spotlights = c(-1, 0, 1),
    probe.bins = 30
  )

  expect_true(is.list(res))
  expect_true(all(c("simple.slopes", "johnson.neyman", "frequencies") %in% names(res)))
  expect_true(is.data.frame(res$johnson.neyman))
  expect_true(nrow(res$johnson.neyman) > 0)

  # For a linear model, dy/dx does not depend on x itself (only on z due to interaction),
  # so the marginal effects curves should be (nearly) identical across the x-spotlights.
  jn <- res$johnson.neyman
  expect_true("marginal.effect" %in% names(jn))
  expect_true("x1" %in% names(jn))
  expect_true("z1" %in% names(jn))

  eff_by_x <- split(jn$marginal.effect, jn$x1)
  expect_true(length(eff_by_x) == 3)
  expect_equal(unname(eff_by_x[[1]]), unname(eff_by_x[[2]]), tolerance = 1e-8)
  expect_equal(unname(eff_by_x[[2]]), unname(eff_by_x[[3]]), tolerance = 1e-8)
})

#interprobe_002
test_that("interprobe returns fitted model when estimating internally", {
  skip_if_not_installed("marginaleffects")
  skip_if_not_installed("mgcv")
  skip_if_not_installed("estimatr")

  set.seed(333)
  n <- 250
  x <- rnorm(n)
  z <- rnorm(n)
  y <- x * z + rnorm(n)

  grDevices::pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(grDevices::dev.off(), add = TRUE)

  res <- interprobe(
    x, z, y,
    quiet = TRUE,
    draw = "jn",
    histogram = FALSE,
    spotlights = c(-1, 0, 1),
    probe.bins = 30
  )

  expect_true(is.list(res))
  expect_true("gam_results" %in% names(res))
  expect_s3_class(res$gam_results, "gam")
  expect_true("lm2_results" %in% names(res))
  expect_s3_class(res$lm2_results, "lm2")

  cl <- attr(res$lm2_results, "lm2_call")
  expect_true(!is.null(cl))
  call_txt <- gsub("\\s+", " ", paste(deparse(cl), collapse = " "))
  expect_equal(call_txt, "lm2(y ~ x * z)")
})

#interprobe_003
test_that("binary x returns gam_results_testing (ti-based testing GAM)", {
  skip_if_not_installed("marginaleffects")
  skip_if_not_installed("mgcv")
  skip_if_not_installed("estimatr")

  set.seed(555)
  n <- 200
  z <- rnorm(n)
  x <- rep(c(0, 1), each = n / 2)
  y <- x + z + rnorm(n, sd = 2)

  grDevices::pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(grDevices::dev.off(), add = TRUE)

  res <- interprobe(
    x, z, y,
    quiet = TRUE,
    draw = "jn",
    histogram = FALSE,
    spotlights = c(-1, 0, 1),
    probe.bins = 30
  )

  expect_true(is.list(res))
  expect_true("gam_results_testing" %in% names(res))
  expect_s3_class(res$gam_results_testing, "gam")
})

#interprobe_004
test_that("quiet=TRUE suppresses interaction APA printing", {
  skip_if_not_installed("marginaleffects")
  skip_if_not_installed("mgcv")

  set.seed(444)
  n <- 200
  x <- rnorm(n)
  z <- rnorm(n)
  y <- x * z + rnorm(n)

  grDevices::pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(grDevices::dev.off(), add = TRUE)

  out <- capture.output(
    interprobe(
      x, z, y,
      quiet = TRUE,
      draw = "jn",
      histogram = FALSE,
      spotlights = c(-1, 0, 1),
      probe.bins = 30
    )
  )

  expect_false(any(grepl("p-value for the interaction", out, fixed = TRUE)))
})

#interprobe_005
test_that("interprobe works with lm2() model input", {
  skip_if_not_installed("marginaleffects")
  skip_if_not_installed("estimatr")

  set.seed(111)
  n <- 600
  x1 <- rnorm(n)
  z1 <- rnorm(n, mean = 10, sd = 2)
  y.raw <- x1 * z1
  e <- rnorm(n, sd = sd(y.raw))
  y1 <- y.raw + e

  df <- data.frame(x1, y1, z1)
  lm2_1 <- lm2(y1 ~ x1 * z1, data = df, notes = FALSE)

  grDevices::pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(grDevices::dev.off(), add = TRUE)

  res <- interprobe(
    model = lm2_1,
    x = "x1",
    z = "z1",
    draw = "jn",
    histogram = FALSE,
    quiet = TRUE,
    spotlights = c(-1, 0, 1),
    probe.bins = 30
  )

  expect_true(is.list(res))
  expect_true(all(c("simple.slopes", "johnson.neyman", "frequencies") %in% names(res)))
  expect_true(nrow(res$johnson.neyman) > 0)
  jn <- res$johnson.neyman
  eff_by_x <- split(jn$marginal.effect, jn$x1)
  expect_true(length(eff_by_x) == 3)
  expect_equal(unname(eff_by_x[[1]]), unname(eff_by_x[[2]]), tolerance = 1e-8)
  expect_equal(unname(eff_by_x[[2]]), unname(eff_by_x[[3]]), tolerance = 1e-8)
})

#interprobe_006
test_that("interprobe with data fits lm2 when model = linear or lm sentinel", {
  skip_if_not_installed("marginaleffects")
  skip_if_not_installed("estimatr")

  set.seed(111)
  n <- 600
  x1 <- rnorm(n)
  z1 <- rnorm(n, mean = 10, sd = 2)
  y.raw <- x1 * z1
  e <- rnorm(n, sd = sd(y.raw))
  y1 <- y.raw + e

  df <- data.frame(x1, y1, z1)

  grDevices::pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(grDevices::dev.off(), add = TRUE)

  res_linear <- interprobe(
    data = df,
    x = "x1",
    z = "z1",
    y = "y1",
    model = "linear",
    draw = "jn",
    histogram = FALSE,
    quiet = TRUE,
    spotlights = c(-1, 0, 1),
    probe.bins = 30
  )

  expect_true(is.list(res_linear))
  expect_true(nrow(res_linear$johnson.neyman) > 0)
  jn <- res_linear$johnson.neyman
  eff_by_x <- split(jn$marginal.effect, jn$x1)
  expect_true(length(eff_by_x) == 3)
  expect_equal(unname(eff_by_x[[1]]), unname(eff_by_x[[2]]), tolerance = 1e-8)
  expect_equal(unname(eff_by_x[[2]]), unname(eff_by_x[[3]]), tolerance = 1e-8)

  res_lm_sentinel <- interprobe(
    data = df,
    x = "x1",
    z = "z1",
    y = "y1",
    model = lm,
    draw = "jn",
    histogram = FALSE,
    quiet = TRUE,
    spotlights = c(-1, 0, 1),
    probe.bins = 30
  )

  expect_equal(nrow(res_lm_sentinel$johnson.neyman), nrow(res_linear$johnson.neyman))
})

#interprobe_007
test_that("interprobe(x, z, y vectors) works with model = linear", {
  skip_if_not_installed("marginaleffects")
  skip_if_not_installed("estimatr")

  set.seed(111)
  n <- 600
  x <- rnorm(n)
  z <- rnorm(n, mean = 10, sd = 2)
  y.raw <- x * z
  e <- rnorm(n, sd = sd(y.raw))
  y <- y.raw + e

  grDevices::pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(grDevices::dev.off(), add = TRUE)

  res <- interprobe(
    x, z, y,
    model = "linear",
    quiet = TRUE,
    draw = "jn",
    histogram = FALSE,
    spotlights = c(-1, 0, 1),
    probe.bins = 30
  )

  expect_true(is.list(res))
  expect_true(nrow(res$johnson.neyman) > 0)
})

#interprobe_008
test_that("bare model = linear is keyword when linear not in calling env", {
  skip_if_not_installed("marginaleffects")
  skip_if_not_installed("estimatr")

  # Child of .GlobalEnv: no `linear` binding here; interprobe's parent.frame() is this env.
  res <- local(
    {
      set.seed(111)
      n <- 200
      x <- rnorm(n)
      z <- rnorm(n)
      y <- x * z + rnorm(n)
      grDevices::pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
      on.exit(grDevices::dev.off(), add = TRUE)
      interprobe(
        x, z, y,
        model = linear,
        quiet = TRUE,
        draw = "jn",
        histogram = FALSE,
        spotlights = c(-1, 0, 1),
        probe.bins = 30
      )
    },
    envir = new.env(parent = globalenv())
  )

  expect_true(is.list(res))
  expect_true(nrow(res$johnson.neyman) > 0)
})

#interprobe_009
test_that("bare model = linear uses fitted object when linear exists", {
  skip_if_not_installed("marginaleffects")
  skip_if_not_installed("estimatr")

  res <- local(
    {
      set.seed(111)
      n <- 200
      x1 <- rnorm(n)
      z1 <- rnorm(n)
      y1 <- x1 * z1 + rnorm(n)
      df <- data.frame(x1, z1, y1)
      linear <- lm2(y1 ~ x1 * z1, data = df, notes = FALSE)
      grDevices::pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
      on.exit(grDevices::dev.off(), add = TRUE)
      interprobe(
        model = linear,
        x = "x1",
        z = "z1",
        draw = "jn",
        histogram = FALSE,
        quiet = TRUE,
        spotlights = c(-1, 0, 1),
        probe.bins = 30
      )
    },
    envir = new.env(parent = globalenv())
  )

  expect_true(is.list(res))
  expect_true(nrow(res$johnson.neyman) > 0)
})

#interprobe_010
test_that("bare model = lm on vectors uses stats::lm as keyword", {
  skip_if_not_installed("marginaleffects")
  skip_if_not_installed("estimatr")

  set.seed(111)
  n <- 200
  x <- rnorm(n)
  z <- rnorm(n)
  y <- x * z + rnorm(n)

  grDevices::pdf(file = tempfile(fileext = ".pdf"), width = 7, height = 7)
  on.exit(grDevices::dev.off(), add = TRUE)

  res <- interprobe(
    x, z, y,
    model = lm,
    quiet = TRUE,
    draw = "jn",
    histogram = FALSE,
    spotlights = c(-1, 0, 1),
    probe.bins = 30
  )

  expect_true(is.list(res))
  expect_true(nrow(res$johnson.neyman) > 0)
})

#interprobe_011
test_that("error message shows original model name (linear)", {
  set.seed(123)
  x <- rnorm(200)
  z <- rnorm(200)
  y <- x * z + rnorm(200)
  x2 <- rnorm(200)

  linear <- lm(y ~ x2 * z)

  msgs <- character(0)
  out <- withCallingHandlers(
    withRestarts(
      interprobe(x, z, y, model = linear, quiet = TRUE),
      abort = function(...) "aborted"
    ),
    message = function(m) {
      msgs <<- c(msgs, conditionMessage(m))
      invokeRestart("muffleMessage")
    }
  )

  expect_identical(out, "aborted")
  expect_true(any(grepl("model 'linear'", msgs, fixed = TRUE)))
})

#' Make stimulus plots as in "Stimulus Sampling Reimagined"
#'
#' Build stimulus plots comparing results for individual stimuli in an experiment
#' (Simonsohn, Montealegre, & Evangelidis, 2024).
#'
#' @param plot.type Either `"means"` or `"effects"`; determines what is plotted on the y-axis.
#' @param data Data frame containing variables to be analyzed.
#' @param dv Name of the dependent variable (e.g., `dv = 'y'`); quotes are not required.
#' @param condition Name of the variable containing the condition indicator.
#' @param stimulus Name of the variable containing the stimulus ID.
#' @param participant Name of the variable containing participant IDs; necessary for valid
#'   inference when `plot.type = 'effects'` and each participant provided more than one observation.
#' @param save.as File path for saving the figure (`.svg` or `.png`, optional).
#' @param svg.width Optional width when saving to SVG/PNG.
#' @param svg.height Optional height when saving to SVG/PNG.
#' @param sort.by Variable to sort stimuli by. Defaults to sorting by observed effect size.
#' @param flip.conditions If `TRUE`, subtract the first condition from the second instead of the default.
#' @param model Method used to compute overall average: `'regression'`, `'intercepts'`,
#'   `'slopes'`, and/or `'all'`.
#' @param overall.estimate Scalar or vector of overall average effect computed outside statuser.
#' @param overall.ci Confidence interval bounds for `overall.estimate`.
#' @param overall.p P-value for overall average effect computed outside statuser.
#' @param overall.label Label for overall averages on the x-axis.
#' @param ylab1,ylab2 Labels on the y-axis (optional).
#' @param xlab1,xlab2 Labels on the x-axis (optional).
#' @param decimals Number of decimals for value labels (`"auto"` by default).
#' @param null.method Null method for effects plots: `"shuffle"` or `"demean"`.
#' @param dv.is.percentage If `TRUE`, format the dependent variable as percentages.
#' @param legend.title Text with title above legend (optional).
#' @param simtot Number of resamples for heterogeneity under null (effects plots).
#' @param watermark If `TRUE`, display package version watermark.
#' @param seed Seed used when resampling for effects plots.
#' @param ylim Optional y-axis limits.
#' @param main Plot title.
#' @param ... Additional arguments passed to [graphics::plot.default()].
#'
#' @return Invisibly, a data frame (means plot) or list (effects plot).
#' @importFrom digest digest
#' @importFrom grDevices dev.off png svg
#' @importFrom graphics mtext
#' @importFrom utils packageVersion
#' @importFrom stats ave qt residuals t.test
#' @import lmerTest
#' @export
stimulus.plot <- function(
  plot.type = 'means',
  data, dv, condition, stimulus,
  participant = '',
  save.as = '',
  svg.width = '',
  svg.height = '',
  sort.by = '',
  flip.conditions = FALSE,
  model = c(),
  overall.estimate = c(),
  overall.ci = c(),
  overall.p = c(),
  overall.label = c(),
  ylab1 = '',
  ylab2 = '',
  xlab1 = 'Stimuli',
  xlab2 = '',
  decimals = 'auto',
  null.method = 'shuffle',
  dv.is.percentage = FALSE,
  legend.title = '',
  simtot = 1000,
  watermark = TRUE,
  seed = 2024,
  ylim = c(),
  main = '',
  ...
) {

#: 1 stimulus.plot: validate -> prepare -> plot -> watermark
  args_passed <- as.list(match.call())[-1]

  if (!"data.frame" %in% class(data)) {
    exit(paste0("stimulus.plot() says: the argument data must be a data.frame, but '", deparse(substitute(data)), "' is not a dataframe."))
  }
  data <- data.frame(data)

  dataname <- clean_string(deparse(substitute(data)))

  if (null.method == 'demeans') null.method <- 'demean'

  validate.stimulus.plot(
    plot.type, data, dv, condition, stimulus,
    save.as, svg.width, svg.height, sort.by, flip.conditions,
    model, overall.estimate, overall.ci, overall.p, overall.label,
    ylab1, ylab2, xlab1, xlab2, decimals, null.method,
    dv.is.percentage, legend.title, simtot, watermark, seed, ylim,
    args_passed
  )

  f <- 'statuser::stimulus.plot'
  validate.dots(f, ...)

  dv <- clean_string(deparse(substitute(dv)))
  condition <- clean_string(deparse(substitute(condition)))
  stimulus <- clean_string(deparse(substitute(stimulus)))
  sort.by <- clean_string(deparse(substitute(sort.by)))

  validate.data(f, data, dv, condition, stimulus, sort.by, participant, dataname)

  data <- data[, names(data) %in% c('r', dv, condition, stimulus, participant), drop = FALSE]

  n1 <- nrow(data)
  data <- data[!is.na(data[, stimulus]) & !is.na(data[, dv]) & !is.na(data[, condition]), , drop = FALSE]
  if (participant != '') data <- data[!is.na(data[, participant]), , drop = FALSE]
  n2 <- nrow(data)
  if (n2 < n1) message2('stimulus.plot() says: ', n1 - n2, ' observations were dropped because of missing values.')

  t <- table(data[, stimulus], data[, condition])
  drop.stim <- t[t[, 1] == 0 | t[, 2] == 0, , drop = FALSE]
  if (nrow(drop.stim) > 0) {
    drop.stim.rows <- data[, stimulus] %in% rownames(drop.stim)
    data <- data[!drop.stim.rows, , drop = FALSE]
    n3 <- nrow(data)
    message2('stimulus.plot() says: ', n2 - n3, ' observations were dropped because their value of "', stimulus, '" appears in only one condition')
  }

  if (save.as != '') {
    filename <- save.as
    dir.create(dirname(filename), recursive = TRUE, showWarnings = FALSE)
    extension <- tools::file_ext(filename)
    ns <- length(unique(data[, stimulus]))
    nm <- length(model)
    w <- 5 + (ns + nm * 1.5) * .4
    h <- 5 * (1 + max(nchar(unique(data[, stimulus]))) / 40)
    if (svg.width != '') w <- svg.width
    if (svg.height != '') h <- svg.height
    if (extension == 'svg') grDevices::svg(filename, w, h)
    if (extension == 'png') grDevices::png(filename, w * 1000, h * 1000, res = 1000)
  }

  if (plot.type == 'means') {
    res <- stimulus.plot.means(
      data = data, dv = dv, condition = condition, stimulus = stimulus,
      participant = participant, sort.by = sort.by, flip.conditions = flip.conditions,
      ylab1 = ylab1, ylab2 = ylab2, xlab1 = xlab1, xlab2 = xlab2,
      decimals = decimals, dv.is.percentage = dv.is.percentage,
      legend.title = legend.title, ylim = ylim, main = main, ...
    )
  }

  if (plot.type == 'effects') {
    res <- stimulus.plot.effects(
      data = data, dv = dv, condition = condition, stimulus = stimulus,
      dataname = dataname, model = model,
      overall.estimate = overall.estimate, overall.ci = overall.ci,
      overall.p = overall.p, overall.label = overall.label,
      participant = participant, sort.by = sort.by,
      flip.conditions = flip.conditions, null.method = null.method,
      ylab1 = ylab1, ylab2 = ylab2, xlab1 = xlab1, xlab2 = xlab2,
      dv.is.percentage = dv.is.percentage, simtot = simtot,
      decimals = decimals, ylim = ylim, seed = seed, main = main, ...
    )
  }

  if (watermark == TRUE) {
    stim_vrs <- paste0("{statuser v", utils::packageVersion('statuser'), "}")
    graphics::mtext(side = 1, line = -1, cex = .7, stim_vrs, col = 'gray66', adj = 0, outer = TRUE)
  }

  if (save.as != '') grDevices::dev.off()

  invisible(res)
}

#' Print method for table2 output with centered column variable name
#'
#' @param x An object of class \code{table2}
#' @param ... Additional arguments passed to print
#'
#' @return Invisibly returns the input object \code{x}. Called for its side effect 
#'   of printing a formatted cross-tabulation table to the console. The output 
#'   includes frequencies, optional relative frequencies (row, column, or overall 
#'   proportions), and chi-squared test results when applicable.
#'
#' @export
print.table2 <- function(x, ...) {
  # Check if x is the new list format (has freq, prop, chisq elements)
  if (is.list(x) && "freq" %in% names(x) && "prop" %in% names(x) && "chisq" %in% names(x)) {
    # Print freq table (without chi-square - we'll print that at the end)
    freq_to_print <- x$freq
    # Mark as frequency (not proportion) to avoid triggering legacy freq+prop logic
    attr(freq_to_print, "is_frequency") <- TRUE
    class(freq_to_print) <- c("table2", class(freq_to_print))
    
    if (!is.null(x$prop)) {
      cat("\n1. Frequencies")
    }
    print.table2(freq_to_print, ...)
    
    # Print prop table if present
    if (!is.null(x$prop)) {
      prop_to_print <- x$prop
      # Mark as proportion for formatting, but no original_frequency to avoid legacy logic
      attr(prop_to_print, "is_proportion") <- TRUE
      attr(prop_to_print, "proportion_digits") <- 3
      class(prop_to_print) <- c("table2", class(prop_to_print))
      
      # Get prop_type from list attributes
      prop_type_list <- attr(x, "prop_type")
      
      # Pass prop_type to the proportion table for adding totals
      attr(prop_to_print, "prop_type") <- prop_type_list
      
      # Build proportion header based on prop type
      cat("\n2. ")
      if (!is.null(prop_type_list) && prop_type_list == 0) {
        cat("Relative frequencies: overall")
      } else if (!is.null(prop_type_list) && prop_type_list == 1) {
        cat("Relative frequencies: by row")
      } else if (!is.null(prop_type_list) && prop_type_list == 2) {
        cat("Relative frequencies: by column")
      } else {
        cat("Relative frequencies")
      }
      print.table2(prop_to_print, ...)
    }
    
    # Print chi-square test at the end if present
    if (!is.null(x$chisq)) {
      chi_test <- x$chisq
      n_dims <- length(dim(x$freq))
      # Print header based on dimension
      if (n_dims == 1) {
        cat("Chi-squared test, null: equality of proportions\n")
      } else {
        cat("Chi-squared test, null: independence\n")
      }
      
      chi_stat <- as.numeric(chi_test$statistic)
      chi_df <- as.numeric(chi_test$parameter)
      chi_p <- as.numeric(chi_test$p.value)
      
      chi_stat_formatted <- if (!is.na(chi_stat)) sprintf("%.2f", round(chi_stat, 2)) else "NA"
      chi_df_formatted <- if (!is.na(chi_df)) sprintf("%.0f", round(chi_df, 0)) else "NA"
      
      chi_p_formatted <- if (!is.na(chi_p)) {
        format_pvalue(chi_p, include_p = TRUE)
      } else {
        "p = NA"
      }
      
      apa_string <- paste0("\u03C7\u00B2(", chi_df_formatted, ") = ", chi_stat_formatted, ", ", chi_p_formatted)
      cat(paste0(apa_string, "\n"))
      
      # Show warning if expected counts are low
      if (isTRUE(attr(chi_test, "low_expected"))) {
        message2(col = "red", "Warning: some cells have less than 5 expected observations.\nThis makes the chi-square approximation less accurate; the p-value is questionable.")
      }
    }
    
    return(invisible(x))
  }
  
  # Legacy format: x is a table/matrix with attributes
  # Get dimension names
  dimn <- dimnames(x)
  dim_names <- names(dimn)
  n_dims <- length(dim(x))
  
  # Check if this is a proportion table (check once)
  # Use !is.null() instead of isTRUE() to be more robust
  is_proportion <- !is.null(attr(x, "is_proportion")) && isTRUE(attr(x, "is_proportion"))
  is_frequency <- !is.null(attr(x, "is_frequency")) && isTRUE(attr(x, "is_frequency"))
  proportion_digits <- attr(x, "proportion_digits")
  if (is.null(proportion_digits)) proportion_digits <- 3
  
  # Get original frequency table if this is a proportion table
  orig_freq <- attr(x, "original_frequency")
  prop_type <- attr(x, "prop_type")
  var1_name <- attr(x, "var1_name")
  var2_name <- attr(x, "var2_name")
  
  # If this is a proportion table, first print the frequency table
  if (is_proportion && !is.null(orig_freq)) {
    # Create a temporary table2 object from the original frequency table
    # This will have the same dimnames structure but without proportion formatting
    freq_table <- orig_freq
    # Copy dimnames from original (before totals were added)
    dimnames(freq_table) <- dimnames(orig_freq)
    # Mark this as a frequency table (not proportion) so it formats correctly
    attr(freq_table, "is_frequency") <- TRUE
    # Ensure it's NOT marked as proportion
    attr(freq_table, "is_proportion") <- FALSE
    # Copy chi_test attribute from the proportion table to freq_table so it prints after frequency table
    chi_test_from_prop <- attr(x, "chi_test")
    if (!is.null(chi_test_from_prop)) {
      attr(freq_table, "chi_test") <- chi_test_from_prop
    }
    # Add class for custom printing if it has the right structure
    if (length(dim(freq_table)) == 2 || length(dim(freq_table)) == 3) {
      freq_dimn <- dimnames(freq_table)
      if (!is.null(names(freq_dimn)) && any(nchar(names(freq_dimn)) > 0)) {
        class(freq_table) <- c("table2", class(freq_table))
      } else {
        # Even without dimension names, add class to ensure custom printing
        class(freq_table) <- c("table2", class(freq_table))
      }
    }
    # Print header for frequency table
    cat("\n1. Frequencies")
    # Print the frequency table (this will also print the chi-square test)
    print.table2(freq_table, ...)
    
    # Print header for proportion table
    cat("\n2. ")
    if (prop_type == 0) {
      cat("Relative frequencies: overall")
    } else if (prop_type == 1) {
      cat("Relative frequencies: by row")
    } else if (prop_type == 2) {
      cat("Relative frequencies: by column")
    }
    # Remove chi_test attribute from proportion table so it doesn't print again
    attr(x, "chi_test") <- NULL
  }
  
  # Handle 3D tables
  if (n_dims == 3) {
    if (is.null(dim_names) || length(dim_names) != 3) {
      # Use base print instead of NextMethod() to avoid issues with recursive calls
      x_for_print <- x
      class(x_for_print) <- setdiff(class(x_for_print), "table2")
      print(x_for_print)
      return(invisible(x))
    }
    
    third_var_name <- dim_names[3]
    third_labels <- dimn[[3]]
    
    # Print each slice of the 3D table
    for (k in seq_along(third_labels)) {
      # Print the third dimension header (e.g., "heat = high" instead of ", , heat = high")
      cat("\n")
      cat(third_var_name, " = ", third_labels[k], "\n", sep = "")
      
      # Extract the 2D slice
      slice <- x[, , k, drop = FALSE]
      # Remove the third dimension to make it a 2D table
      dim(slice) <- dim(slice)[1:2]
      dimnames(slice) <- dimn[1:2]
      
      # For proportion tables, add Total row/column based on prop_type
      if (is_proportion && !is.null(prop_type)) {
        slice_dimn <- dimnames(slice)
        dimn_names <- names(slice_dimn)
        
        if (prop_type == 0) {
          # Overall proportions: add both Total row and Total column
          n_rows <- nrow(slice)
          n_cols <- ncol(slice)
          
          # Add Total row
          col_totals <- colSums(slice, na.rm = TRUE)
          slice <- rbind(slice, col_totals)
          slice_dimn[[1]] <- c(slice_dimn[[1]], "Total")
          
          # Add Total column
          row_totals <- rowSums(slice, na.rm = TRUE)
          slice <- cbind(slice, row_totals)
          slice_dimn[[2]] <- c(slice_dimn[[2]], "Total")
          
          names(slice_dimn) <- dimn_names
          dimnames(slice) <- slice_dimn
        } else if (prop_type == 1) {
          # Row proportions: add Total column with 1.0 for each row
          n_rows <- nrow(slice)
          summary_col <- matrix(round(1.0, proportion_digits), nrow = n_rows, ncol = 1)
          slice <- cbind(slice, summary_col)
          slice_dimn[[2]] <- c(slice_dimn[[2]], "Total")
          names(slice_dimn) <- dimn_names
          dimnames(slice) <- slice_dimn
        } else if (prop_type == 2) {
          # Column proportions: add Total row with 1.0 for each column
          n_cols <- ncol(slice)
          summary_row <- matrix(round(1.0, proportion_digits), nrow = 1, ncol = n_cols)
          slice <- rbind(slice, summary_row)
          slice_dimn[[1]] <- c(slice_dimn[[1]], "Total")
          names(slice_dimn) <- dimn_names
          dimnames(slice) <- slice_dimn
        }
      }
      
      class(slice) <- c("table2", class(slice))
      # Copy attributes
      if (is_proportion) {
        attr(slice, "is_proportion") <- TRUE
        attr(slice, "proportion_digits") <- proportion_digits
        # Copy original frequency slice if available
        if (!is.null(orig_freq)) {
          freq_slice <- orig_freq[, , k, drop = FALSE]
          dim(freq_slice) <- dim(freq_slice)[1:2]
          attr(slice, "original_frequency") <- freq_slice
          attr(slice, "prop_type") <- prop_type
          attr(slice, "var1_name") <- var1_name
          attr(slice, "var2_name") <- var2_name
        }
      }
      
      # Print the 2D slice using the same print method
      print.table2(slice, ...)
    }
    
    return(invisible(x))
  }
  
  # Handle 1D tables
  if (n_dims == 1) {
    # For 1D tables, print using base table print method
    # We can't use NextMethod() here because it fails when called recursively
    # with the table2 class added. Instead, temporarily remove the table2 class.
    x_for_print <- x
    class(x_for_print) <- setdiff(class(x_for_print), "table2")
    cat("\n")
    print(x_for_print)
    
    # Check if we have chi_test attribute and print if available
    # BUT skip if this is being called from the list format handler (is_frequency or is_proportion set)
    # because the list handler prints chi-square at the end
    chi_test <- attr(x, "chi_test")
    if (!is.null(chi_test) && !is.null(chi_test$statistic) && !is.null(chi_test$p.value) &&
        !is_frequency && !is_proportion) {
      # Print chi-square test with label (left-aligned)
      cat("\nChi-squared test, null: equality of proportions\n")
      chi_stat <- as.numeric(chi_test$statistic)
      chi_df <- as.numeric(chi_test$parameter)
      chi_p <- as.numeric(chi_test$p.value)
      
      chi_stat_formatted <- if (!is.na(chi_stat)) sprintf("%.2f", round(chi_stat, 2)) else "NA"
      chi_df_formatted <- if (!is.na(chi_df)) sprintf("%.0f", round(chi_df, 0)) else "NA"
      
      # Format p-value using format_pvalue (same approach as t.test2)
      chi_p_formatted <- if (!is.na(chi_p)) {
        # format_pvalue returns "p = .05", "p < .0001", or "p > .9999" format
        format_pvalue(chi_p, include_p = TRUE)
      } else {
        "p = NA"
      }
      
      # Create APA formatted string: chi-squared(df) = value, p = value
      apa_string <- paste0("\u03c7\u00b2(", chi_df_formatted, ") = ", chi_stat_formatted, ", ", chi_p_formatted)
      
      # Print left-aligned APA formatting
      cat(paste0(apa_string, "\n"))
    }
    return(invisible(x))
  }
  
  # Handle 2D tables (original code)
  # If we don't have dimension names and it's not a proportion table and not a frequency table, fall back to default print
  # But if it's a frequency table (marked with is_frequency), we want to use our custom formatting
  if ((is.null(dim_names) || length(dim_names) != 2) && !is_proportion && !is_frequency) {
    # Use base print instead of NextMethod() to avoid issues with recursive calls
    x_for_print <- x
    class(x_for_print) <- setdiff(class(x_for_print), "table2")
    print(x_for_print)
    return(invisible(x))
  }
  
  # For proportion tables or frequency tables without dimension names, use default print but with custom formatting
  if (is.null(dim_names) || length(dim_names) != 2) {
    # Still use base print but the formatting should be handled by the proportion logic
    # Actually, we need dimension names to format properly, so fall back
    x_for_print <- x
    class(x_for_print) <- setdiff(class(x_for_print), "table2")
    print(x_for_print)
    return(invisible(x))
  }
  
  row_var_name <- dim_names[1]
  col_var_name <- dim_names[2]
  
  # Get row and column labels
  row_labels <- dimn[[1]]
  col_labels <- dimn[[2]]
  
  # Calculate widths for formatting
  # First, calculate width of column labels
  col_label_widths <- nchar(col_labels)
  max_col_label_width <- max(col_label_widths, na.rm = TRUE)
  
  # Helper function to format proportion values
  format_proportion <- function(val, digits) {
    if (is.na(val)) {
      return("NA")
    }
    # For values >= 1.0 (like totals), show with leading digit: 1.000
    if (val >= 1.0) {
      return(sprintf(paste0("%.", digits, "f"), val))
    }
    # For values < 1.0, show without leading zero: .100, .000, etc.
    val_rounded <- round(val * (10^digits))
    return(sprintf(paste0(".%0", digits, "d"), val_rounded))
  }
  
  # Calculate width of actual data values - format each value and check width
  max_data_width <- 0
  for (i in seq_along(row_labels)) {
    for (j in seq_along(col_labels)) {
      val <- x[i, j]
      # Format as integer if NOT a proportion table (frequencies should be integers)
      if (is_proportion) {
        val_str <- format_proportion(val, proportion_digits)
      } else {
        # For frequencies, format as integer (no decimals)
        if (is.na(val)) {
          val_str <- "NA"
        } else {
          # Force integer formatting - use %d which always shows integers
          val_int <- as.integer(round(val))
          val_str <- sprintf("%d", val_int)
        }
      }
      max_data_width <- max(max_data_width, nchar(val_str), na.rm = TRUE)
    }
  }
  
  # Use the maximum of label width and data width for column formatting
  max_col_width <- max(max_col_label_width, max_data_width)
  
  row_label_width <- max(nchar(row_labels), na.rm = TRUE)
  row_var_width <- nchar(row_var_name)
  
  # Column spacing (R typically uses 2 spaces between columns)
  col_spacing <- 2
  
  # Calculate total width of all columns
  n_cols <- length(col_labels)
  total_col_width <- n_cols * max_col_width + (n_cols - 1) * col_spacing
  
  # Calculate spacing to center column variable name over the data columns
  col_var_width <- nchar(col_var_name)
  col_var_spacing <- max(0, floor((total_col_width - col_var_width) / 2))
  
  # Total width needed for row label area (variable name + space + label + spacing)
  total_row_label_width <- row_var_width + 1 + row_label_width + col_spacing
  
  # Print column variable name centered over columns
  cat("\n")
  cat(strrep(" ", total_row_label_width + col_var_spacing))
  cat(col_var_name)
  cat("\n")
  
  # Print column labels
  cat(strrep(" ", total_row_label_width))
  # Build column labels as formatted values
  col_label_values <- character(length(col_labels))
  for (i in seq_along(col_labels)) {
    col_label_values[i] <- sprintf(paste0("%", max_col_width, "s"), col_labels[i])
  }
  # Join with spacing and print
  spacing_str <- strrep(" ", col_spacing)
  cat(paste(col_label_values, collapse = spacing_str))
  cat("\n")
  
  # Print rows with row variable name on left margin
  for (i in seq_along(row_labels)) {
    # Print row variable name and label on first row, or just label with spacing for alignment
    if (i == 1) {
      cat(sprintf(paste0("%-", row_var_width, "s"), row_var_name))
      cat(" ")
      cat(sprintf(paste0("%-", row_label_width + col_spacing, "s"), row_labels[i]))
    } else {
      cat(strrep(" ", row_var_width + 1))
      cat(sprintf(paste0("%-", row_label_width + col_spacing, "s"), row_labels[i]))
    }
    
    # Print values (right-aligned like R does)
    # Build the row as a vector of formatted values
    row_values <- character(length(col_labels))
    for (j in seq_along(col_labels)) {
      val <- x[i, j]
      # Format as integer if NOT a proportion table (frequencies should be integers)
      if (is_proportion) {
        val_str <- format_proportion(val, proportion_digits)
      } else {
        # For frequencies, format as integer (no decimals)
        if (is.na(val)) {
          val_str <- "NA"
        } else {
          # Force integer formatting - use %d which always shows integers
          val_int <- as.integer(round(val))
          val_str <- sprintf("%d", val_int)
        }
      }
      # Right-align the value within max_col_width
      row_values[j] <- sprintf(paste0("%", max_col_width, "s"), val_str)
    }
    # Join with spacing and print
    spacing_str <- strrep(" ", col_spacing)
    cat(paste(row_values, collapse = spacing_str))
    cat("\n")
  }
  
  cat("\n")
  
  # Print chi-square test result if available
  chi_test <- attr(x, "chi_test")
  if (!is.null(chi_test) && !is.null(chi_test$statistic) && !is.null(chi_test$p.value)) {
    # Print header for 2D tables (independence test)
    cat("Chi-squared test, null: independence\n")
    # Format chi-square statistic to 2 decimal places
    chi_stat <- as.numeric(chi_test$statistic)
    chi_df <- as.numeric(chi_test$parameter)
    chi_p <- as.numeric(chi_test$p.value)
    
    chi_stat_formatted <- if (!is.na(chi_stat)) sprintf("%.2f", round(chi_stat, 2)) else "NA"
    chi_df_formatted <- if (!is.na(chi_df)) sprintf("%.0f", round(chi_df, 0)) else "NA"
    
    # Format p-value using format_pvalue (same approach as t.test2)
    chi_p_formatted <- if (!is.na(chi_p)) {
      # format_pvalue returns "p = .05", "p < .0001", or "p > .9999" format
      format_pvalue(chi_p, include_p = TRUE)
    } else {
      "p = NA"
    }
    
    # Create APA formatted string: chi-squared(df) = value, p = value
    # Use Unicode chi-square symbol
    apa_string <- paste0("\u03c7\u00b2(", chi_df_formatted, ") = ", chi_stat_formatted, ", ", chi_p_formatted)
    
    # Print left-aligned APA formatting
    cat(paste0(apa_string, "\n"))
  }
  
  invisible(x)
}

#' Print method for t.test2 output
#'
#' @param x An object of class \code{t.test2}
#' @param ... Additional arguments passed to print
#'
#' @return Invisibly returns the input object \code{x}. Called for its side effect 
#'   of printing a formatted t-test summary to the console, including means, 
#'   confidence intervals, test statistics, p-values, sample sizes, and 
#'   APA-formatted results.
#'
#' @export
print.t.test2 <- function(x, ...) {
  # Ensure x is a data frame
  if (!is.data.frame(x)) {
    # Remove the t.test2 class temporarily to avoid infinite recursion
    class(x) <- setdiff(class(x), "t.test2")
    return(print(x, ...))
  }
  
  # Get attributes
  is_one_sample <- attr(x, "is_one_sample")
  is_paired <- attr(x, "is_paired")
  method_type <- attr(x, "method_type")
  NA1 <- attr(x, "NA1")
  NA2 <- attr(x, "NA2")
  group1 <- attr(x, "group1")
  group2 <- attr(x, "group2")
  name_1 <- attr(x, "name_1")
  name_2 <- attr(x, "name_2")
  
  # Determine test title
  if (isTRUE(is_one_sample)) {
    title_text <- "One sample Student t-test"
  } else if (isTRUE(is_paired)) {
    title_text <- "Two sample paired t-test"
  } else if (identical(method_type, "welch")) {
    title_text <- "Two sample Welch t-test"
  } else {
    title_text <- "Two sample Student t-test"
  }
  
  # Print left-aligned title
  cat(paste0(title_text, "\n\n"))
  
  # Get N column names before removing class
  name_N1 <- attr(x, "name_N1")
  name_N2 <- attr(x, "name_N2")
  
  # Show missing value notes if present
  NA1 <- if (is.null(NA1) || is.na(NA1)) 0L else NA1
  NA2 <- if (is.null(NA2) || is.na(NA2)) 0L else NA2
  
  # For one-sample tests, remove group2, diff column, and N2 columns before formatting
  # For paired tests, remove N2 column (only show N)
  if (isTRUE(is_one_sample)) {
    # Get column names to remove
    name_2 <- attr(x, "name_2")
    name_N2 <- attr(x, "name_N2")
    name_1 <- attr(x, "name_1")
    cols_to_remove <- c()
    
    # Find diff column (format: "group1-group2" or "1-2" if using Group 1/Group 2)
    if (!is.null(name_1) && !is.null(name_2)) {
      if (name_1 == "Group 1" && name_2 == "Group 2") {
        diff_col_name <- "1-2"
      } else {
        diff_col_name <- paste0(name_1, "-", name_2)
      }
      if (diff_col_name %in% names(x)) {
        cols_to_remove <- c(cols_to_remove, diff_col_name)
      }
    }
    
    if (!is.null(name_2) && name_2 %in% names(x)) {
      cols_to_remove <- c(cols_to_remove, name_2)
    }
    if (!is.null(name_N2) && name_N2 %in% names(x)) {
      cols_to_remove <- c(cols_to_remove, name_N2)
    }
    # Create display dataframe without these columns
    display_x <- x[, !names(x) %in% cols_to_remove, drop = FALSE]
  } else if (isTRUE(is_paired)) {
    # For paired tests, remove N2 column (only show N)
    name_N2 <- attr(x, "name_N2")
    cols_to_remove <- c()
    if (!is.null(name_N2) && name_N2 %in% names(x)) {
      cols_to_remove <- c(cols_to_remove, name_N2)
    }
    display_x <- x[, !names(x) %in% cols_to_remove, drop = FALSE]
  } else {
    display_x <- x
    
    # If Cohen's d is present, place it after the N columns in the printed table
    if ("d" %in% names(display_x)) {
      n_cols <- character(0)
      if (!is.null(name_N1) && name_N1 %in% names(display_x)) {
        n_cols <- c(n_cols, name_N1)
      }
      if (!is.null(name_N2) && name_N2 %in% names(display_x)) {
        n_cols <- c(n_cols, name_N2)
      }
      
      if (length(n_cols) == 2) {
        base_cols <- setdiff(names(display_x), "d")
        n_pos <- match(n_cols, base_cols)
        n_pos <- n_pos[!is.na(n_pos)]
        
        if (length(n_pos) > 0) {
          insert_after <- max(n_pos)
          after_cols <- if (insert_after < length(base_cols)) base_cols[(insert_after + 1):length(base_cols)] else character(0)
          display_x <- display_x[, c(base_cols[seq_len(insert_after)], "d", after_cols), drop = FALSE]
        }
      }
    }
  }
  
  # Format numeric columns using group_decimals
  # Collect all numeric values (means, diff, ci.L, ci.H, t, df, corr) to determine decimals
  numeric_values <- c()
  for (col in names(display_x)) {
    if (col != "ci" && is.numeric(display_x[[col]])) {
      numeric_values <- c(numeric_values, display_x[[col]])
    }
  }
  
  # Determine number of decimals
  decimals <- group_decimals(numeric_values)
  
  # Format numeric columns (except N columns which are integers, and df which has 1 decimal)
  for (col in names(display_x)) {
    if (is.numeric(display_x[[col]])) {
      if (col == "df") {
        # df always has 1 decimal
        display_x[[col]] <- round(display_x[[col]], 1)
      } else if (col == "N" || grepl("^N", col) || grepl("^N ", col) || grepl("^N\\(", col)) {
        # N columns are integers (including plain "N" for paired tests)
        display_x[[col]] <- round(display_x[[col]], 0)
      } else if (col == "p.value") {
        # p.value is formatted separately using format_pvalue
        if (!is.na(display_x[[col]])) {
          formatted_p <- format_pvalue(display_x[[col]], include_p = FALSE)
          formatted_p <- gsub("= ", "", formatted_p)
          formatted_p <- gsub("< ", "<", formatted_p)
          formatted_p <- gsub("> ", ">", formatted_p)
          display_x[[col]] <- formatted_p
        }
      } else {
        # All other numeric columns use group_decimals
        display_x[[col]] <- round(display_x[[col]], decimals)
      }
    }
  }
  
  # Print the dataframe
  # Remove the t.test2 class temporarily to avoid infinite recursion
  class(display_x) <- setdiff(class(display_x), "t.test2")
  print.data.frame(display_x, row.names = FALSE, ...)
  
  # Show group mapping if group names were replaced (but not for one-sample tests)
  show_group_mapping <- attr(x, "show_group_mapping")
  if (isTRUE(show_group_mapping) && !isTRUE(is_one_sample)) {
    orig_group1 <- attr(x, "orig_group1")
    orig_group2 <- attr(x, "orig_group2")
    if (!is.null(orig_group1) && !is.null(orig_group2)) {
      cat(paste0("\nGroup 1: ", orig_group1, "\n"))
      cat(paste0("Group 2: ", orig_group2, "\n"))
    }
  }
  
  # Add APA formatting: t(df)=t.value, p=p.value
  t_value <- x$t
  df_value <- x$df
  p_value <- x$p.value
  
  # Format t-value to 2 decimal places
  t_formatted <- if (!is.na(t_value)) sprintf("%.2f", round(t_value, 2)) else "NA"
  
  # Format df to 1 decimal place (already rounded in display_x, but use original for APA)
  df_formatted <- if (!is.na(df_value)) sprintf("%.1f", round(df_value, 1)) else "NA"
  
  # Format p-value using format_pvalue (same approach as used elsewhere in this function)
  p_formatted <- if (!is.na(p_value)) {
    # format_pvalue returns "p = .05", "p < .0001", or "p > .9999" format
    format_pvalue(p_value, include_p = TRUE)
  } else {
    "p = NA"
  }
  
  # Create APA formatted string
  apa_string <- paste0("t(", df_formatted, ") = ", t_formatted, ", ", p_formatted)
  
  # Print left-aligned APA formatting
  cat(paste0("\nAPA Style:\n", apa_string, "\n"))
  
  # Check if this is a paired test
  if (isTRUE(is_paired)) {
    # Paired test - show missing pairs
    NA_paired <- attr(x, "NA_paired")
    NA_paired <- if (is.null(NA_paired) || is.na(NA_paired)) 0L else NA_paired
    
    if (NA_paired > 0) {
      # Get N value from original dataframe (before formatting)
      N_col <- if ("N" %in% names(x)) "N" else names(x)[grepl("^N", names(x))][1]
      if (length(N_col) > 0 && N_col %in% names(x)) {
        N <- x[[N_col]]
        total_pairs <- if (!is.na(N)) N + NA_paired else NA_integer_
        
        if (!is.na(total_pairs)) {
          cat(paste0("\nnote: ", NA_paired, " of ", total_pairs, " pairs were dropped due to missing values\n"))
        } else {
          cat(paste0("\nnote: ", NA_paired, " pairs were dropped due to missing values\n"))
        }
      } else {
        cat(paste0("\nnote: ", NA_paired, " pairs were dropped due to missing values\n"))
      }
    }
  } else if (isTRUE(is_one_sample)) {
    # One-sample test - only show missing values for the single variable
    if (NA1 > 0) {
      # Get N value from original dataframe (before formatting)
      N1_col <- names(x)[grepl("^N", names(x))][1]  # Find first column starting with "N"
      if (length(N1_col) > 0 && N1_col %in% names(x)) {
        N1 <- x[[N1_col]]
        total_1 <- if (!is.na(N1)) N1 + NA1 else NA_integer_
        
        if (!is.na(total_1)) {
          cat(paste0("\nnote: ", NA1, " of ", total_1, " values are missing\n"))
        } else {
          cat(paste0("\nnote: ", NA1, " values are missing\n"))
        }
      } else {
        cat(paste0("\nnote: ", NA1, " values are missing\n"))
      }
    }
  } else {
    # Two-sample test - show missing values for both variables
    has_any_missing <- (NA1 > 0 || NA2 > 0)
    
    if (has_any_missing) {
      # Get N values from original dataframe (before formatting)
      N1 <- if (!is.null(name_N1) && name_N1 %in% names(x)) x[[name_N1]] else NA_integer_
      N2 <- if (!is.null(name_N2) && name_N2 %in% names(x)) x[[name_N2]] else NA_integer_
      
      # Calculate totals (N + missing)
      total_1 <- if (!is.na(N1)) N1 + NA1 else NA_integer_
      total_2 <- if (!is.na(N2)) N2 + NA2 else NA_integer_
      
      # Use original group names for the note (not "Group 1" and "Group 2")
      orig_group1 <- attr(x, "orig_group1")
      orig_group2 <- attr(x, "orig_group2")
      show_group_mapping <- attr(x, "show_group_mapping")
      
      # If group names were replaced, use original names; otherwise use current names
      if (isTRUE(show_group_mapping) && !is.null(orig_group1) && !is.null(orig_group2)) {
        col1_name <- orig_group1
        col2_name <- orig_group2
      } else {
        col1_name <- if (!is.null(name_1) && !is.na(name_1)) name_1 else group1
        col2_name <- if (!is.null(name_2) && !is.na(name_2)) name_2 else group2
        # Fallback to "Group 1" and "Group 2" if still empty
        if (is.na(col1_name) || col1_name == "") col1_name <- "Group 1"
        if (is.na(col2_name) || col2_name == "") col2_name <- "Group 2"
      }
      
      # Check if group names are in "varname=value" format
      is_varname_format1 <- grepl("=", col1_name)
      is_varname_format2 <- grepl("=", col2_name)
      
      # Warning about missing data
      # Convert "varname=value" to "varname==value" for display
      col1_display <- gsub("=", "==", col1_name)
      col2_display <- gsub("=", "==", col2_name)
      
      if (!is.na(total_1) && !is.na(total_2)) {
        if (is_varname_format1 && is_varname_format2) {
          # Format: "when 'x4==0' there are k out of N values missing, and when 'x4==1' there are k2 out of N2 values missing"
          cat(paste0("\nnote: when '", col1_display, "' there are ", NA1, " out of ", total_1, 
                     " values missing, and when '", col2_display, "' there are ", NA2, " out of ", total_2, " values missing\n"))
        } else {
          # Original format
          cat(paste0("\nnote: '", col1_name, "' is missing ", NA1, " of ", total_1, 
                     " values, while '", col2_name, "' is missing ", NA2, " of ", total_2, "\n"))
        }
      } else if (!is.na(total_1)) {
        if (is_varname_format1) {
          cat(paste0("\nnote: when '", col1_display, "' there are ", NA1, " out of ", total_1, " values missing\n"))
        } else {
          cat(paste0("\nnote: '", col1_name, "' is missing ", NA1, " of ", total_1, " values\n"))
        }
      } else if (!is.na(total_2)) {
        if (is_varname_format2) {
          cat(paste0("\nnote: when '", col2_display, "' there are ", NA2, " out of ", total_2, " values missing\n"))
        } else {
          cat(paste0("\nnote: '", col2_name, "' is missing ", NA2, " of ", total_2, " values\n"))
        }
      }
    }
  }
  
  # Print environment variable warning at the end if present
  env_warning <- attr(x, "env_warning")
  if (!is.null(env_warning)) {
    cat("\n")
    message2("t.test2() says:", col = "red", font = 2)
    message2(env_warning, col = "red")
  }
  
  # Restore class before returning
  class(x) <- c("t.test2", class(x))
  
  # Return invisibly
  invisible(x)
}

#' Probe interactions robustly to nonlinearities
#'
#' Probes an interaction by estimating (or accepting) a model and computing:
#' - simple slopes ("spotlights") using predicted values
#' - Johnson-Neyman ("jn") curves using marginal effects
#'
#' Designed for GAM models but works with any model supported by `marginaleffects`
#' (including `lm`, `glm`, `mgcv::gam`, and `lm2` / `estimatr::lm_robust`).
#'
#' @param x The focal predictor. Can be a name (bare or quoted) when `data` or `model`
#'   is provided, or a numeric/factor vector when probing from vectors.
#' @param z The moderator. Same options as `x`.
#' @param y The dependent variable. Same options as `x`. Not required when `model` is supplied.
#' @param model By default `interprobe` estimates a GAM model predicting `y` with `x` and `z`.
#' You can instead probe a linear interaction by setting model=linear. You can also probe a 
#' model of your choice by running it separately, saving the output, and submitting it as the model 
#' argument to  interprobe. This is the way to include covariates for a probed interaction.
#' @param data Optional data frame containing `x`, `z`, and `y`.
#' @param moderator.on.x.axis Logical. If TRUE (default), moderator (`z`) is shown on the x-axis.
#' @param k Integer. Smoothness parameter passed to `mgcv::gam()` when estimating with the default
#'   GAM engine. 
#' @param spotlights Numeric vector of length 3. Values at which curves are computed.
#' @param spotlight.labels Character vector of length 3. Labels for the legend.
#' @param histogram Logical. If TRUE (default), show sample size distribution under the plot.
#' @param max.unique Integer. Threshold for treating a variable as continuous vs discrete.
#' @param n.bin.continuous Integer. Number of bins used in histogram when binning continuous values.
#' @param n.max Integer. Sample size at which line darkness/width saturates.
#' @param xlab Character. X-axis label.
#' @param cols Character vector of length 3. Colors for the three curves.
#' @param ylab1 Character. Y-axis label for simple slopes panel.
#' @param ylab2 Character. Y-axis label for JN panel.
#' @param main1 Character. Title for simple slopes panel.
#' @param main2 Character. Title for JN panel.
#' @param legend.round Integer vector length 2. Min/max decimals in legend.
#' @param draw Which plots to draw: `"both"` (default), `"simple slopes"` (or legacy `"simple.slopes"`), or `"jn"`.
#' @param save.as Optional file path to save plot (`.png` or `.svg`).
#' @param xlim Numeric vector length 2. X-axis limits.
#' @param ylim1 Numeric vector length 2. Y-axis limits for simple slopes.
#' @param ylim2 Numeric vector length 2. Y-axis limits for JN.
#' @param legend.simple.slopes Optional legend title for simple slopes.
#' @param legend.jn Optional legend title for JN.
#' @param x.ticks Optional custom x-axis ticks.
#' @param y1.ticks Optional custom y-axis ticks for panel 1.
#' @param y2.ticks Optional custom y-axis ticks for panel 2.
#' @param quiet Logical. If TRUE, reduces console output.
#' @param probe.bins Integer. Resolution for probing curves (larger = smoother/slower).
#'
#' @return Invisibly returns a list with:
#' \itemize{
#'   \item \code{simple.slopes}: data.frame of predicted values and confidence intervals
#'   \item \code{johnson.neyman}: data.frame of marginal effects and confidence intervals
#'   \item \code{frequencies}: data.frame with bin frequencies used for shading/histogram
#'   \item \code{gam_results}: the fitted GAM model when estimated inside \code{interprobe()}
#'   \item \code{gam_results_testing}: when \code{interprobe()} estimates a GAM internally and \code{x}
#'     has exactly 2 unique values, a separate GAM fit used for interaction testing (with a \code{ti()}
#'     term and numeric coding of \code{x})
#'   \item \code{lm2_results}: when \code{interprobe()} estimates a GAM internally, also returns the
#'     corresponding linear fit \code{lm2(y ~ x * z)} (or \code{NULL} if package \code{estimatr}
#'     is not installed)
#' }
#'
#' @export
interprobe <- function( x = NULL, z = NULL, y = NULL,
  model = NULL,  data = NULL,
  moderator.on.x.axis = TRUE,
  k = NULL,
  spotlights = NULL, spotlight.labels = NULL,
  histogram = TRUE, max.unique = 11,n.bin.continuous = 10,n.max = 50,
  xlab = "", ylab1 = "", ylab2 = "",
  cols = c("red4", "dodgerblue", "green4"),
  main1 = "GAM Simple Slopes", main2 = "GAM Johnson-Neyman",
  legend.round = c(1, 4), draw = "both",
  save.as = NULL,
  xlim = NULL, ylim1 = NULL, ylim2 = NULL,x.ticks = NULL,   y1.ticks = NULL, y2.ticks = NULL, 
  legend.simple.slopes = NULL, legend.jn = NULL,
  quiet = FALSE,
  probe.bins = 100
) {
  xvar <- ip_clean_string(deparse(substitute(x)))
  zvar <- ip_clean_string(deparse(substitute(z)))
  yvar <- ip_clean_string(deparse(substitute(y)))

  mc <- match.call(expand.dots = FALSE)
  model_label <- NULL
  if ("model" %in% names(mc)) {
    model_label <- paste0(deparse(mc[["model"]]), collapse = " ")
  }
  rs <- ip_interprobe_resolve_bare_model_symbol(mc, parent.frame(1))
  if (isTRUE(rs$handled)) {
    model <- rs$model
    estimate_linear <- rs$estimate_linear
  } else {
    pm <- ip_interprobe_parse_model_arg(model)
    model <- pm$model
    estimate_linear <- pm$estimate_linear
  }
  if (!is.null(model) && !is.null(model_label)) {
    attr(model, "interprobe_modelname") <- model_label
  }

  if (!is.null(data)) {
    if (is.null(x) | is.null(z) | is.null(y)) exit("interprobe says(): you must specify 'x', 'z' and 'y'")
    x <- xvar
    z <- zvar
    y <- yvar
  }

  # Run before ip_validate_arguments() so missing x/z with model=... hits the
  # intended message instead of failing on focal variable ('NULL') not in model.
  v <- ip_validate_input_combinations(data, model, x, y, z)
  if (v$input.model == TRUE) yvar <- all.vars(terms(model))[1]

  ip_validate_arguments(
    x, z, y,
    model, data,
    k,
    spotlights, spotlight.labels,
    histogram,
    max.unique, n.bin.continuous, n.max,
    xlab, ylab1, ylab2, main1, main2,
    cols,
    draw,
    legend.round,
    xlim,
    save.as,
    xvar, zvar, yvar,
    x.ticks, y1.ticks, y2.ticks,
    moderator.on.x.axis
  )

  if (!is.null(data)) {
    x <- xvar
    z <- zvar
    y <- yvar
  }

  if (quiet == FALSE) {
    cat(paste0("Probing the interaction of '", xvar, "' * '", zvar, "'\n"))
  }

  if (v$input.data == FALSE & v$input.xyz == TRUE) {
    data.text <- paste0("data = data.frame(", xvar, ",", zvar, ",", yvar, ")")
    data <- eval(parse(text = data.text), envir = parent.frame())
  }

  if (v$input.model == TRUE) {
    data <- ip_model_frame(model)
    if (is.null(data)) {
      exit("interprobe() says: could not recover the model frame from the fitted model (try stats::model.frame()).")
    }
  }

  ux <- sort(unique(data[, xvar]))
  uz <- sort(unique(data[, zvar]))
  nux <- length(ux)
  nuz <- length(uz)

  if (nux == 1) exit(paste0("interprobe says: there is only one observed value for the focal (x) variable, '", xvar, "'"))
  if (nuz == 1) exit(paste0("interprobe says: there is only one observed value for the moderator (z) variable, '", zvar, "'"))

  if (nux > max.unique) focal <- "continuous"
  if (nux <= max.unique & nux > 3) focal <- "discrete"
  if (nux <= 3) focal <- "categorical"

  if (nuz > max.unique) moderation <- "continuous"
  if (nuz <= max.unique) moderation <- "discrete"

  if (nux <= 3 & moderator.on.x.axis == FALSE) {
    message(
      paste0(
        "interprobe() says: Less than 3 unique values for x variable ('", xvar, "').\n",
        "Will ignore request to have it on the x-axis"
      )
    )
    moderator.on.x.axis <- TRUE
  }

  if (moderation == "continuous") zs <- seq(min(data[, zvar]), max(data[, zvar]), length.out = probe.bins)
  if (moderation == "discrete") zs <- uz

  if (focal == "continuous") xs <- seq(min(data[, xvar]), max(data[, xvar]), length.out = probe.bins)
  if (focal != "continuous") xs <- ux

  gam_results_testing <- NULL
  lm2_results_override <- NULL

  if (v$input.model == FALSE) {
    engine <- if (estimate_linear) "lm2" else "gam"
    model <- ip_estimate_model(nux, data, k, xvar, zvar, yvar, engine = engine)
    if (!is.null(model) && !is.null(model_label)) {
      attr(model, "interprobe_modelname") <- model_label
    }
    if (identical(engine, "gam") && nux == 2) {
      # Separate GAM for interaction testing: keep x numeric (0/1) and use ti(x,z)
      xF <- factor(data[[xvar]])
      xNum <- as.numeric(xF == levels(xF)[2])
      data_testing <- data
      data_testing[[xvar]] <- xNum

      kz <- if (is.null(k)) 10 else k
      gam_testing_formula <- stats::as.formula(
        paste0(
          yvar, " ~ ", xvar,
          " + s(", zvar, ",k=", kz, ")",
          " + ti(", zvar, ",by=", xvar, ",k=", kz, ")"
        )
      )
      gam_results_testing <- tryCatch(
        mgcv::gam(gam_testing_formula, data = data_testing, method = "REML"),
        error = function(e) NULL
      )

      if (requireNamespace("estimatr", quietly = TRUE)) {
        lm2_formula <- stats::as.formula(paste(yvar, "~", xvar, "*", zvar))
        lm2_results_override <- tryCatch(
          {
            fit_lm2 <- lm2(lm2_formula, data = data_testing, notes = FALSE)
            attr(fit_lm2, "lm2_call") <- as.call(list(as.name("lm2"), lm2_formula))
            fit_lm2
          },
          error = function(e) NULL
        )
      }
    }

    if (quiet == FALSE && identical(engine, "gam")) {
      cat("p-value for the interaction:\n")
      linear_txt <- if (!is.null(lm2_results_override)) {
        ip_get_linear_interaction_test_apa_from_lm2(lm2_results_override, xvar, zvar)
      } else {
        ip_get_linear_interaction_test_apa(data, xvar, zvar, yvar)
      }
      cat("  ", linear_txt, "\n", sep = "")
      gam_for_print <- if (!is.null(gam_results_testing)) gam_results_testing else model
      cat("  ", ip_get_gam_interaction_test_apa(gam_for_print, xvar, zvar), "\n\n", sep = "")
    }
  }

  if (is.null(spotlights) & focal != "categorical") {
    if (moderator.on.x.axis == FALSE) spotvar <- data[, zvar]
    if (moderator.on.x.axis == TRUE) spotvar <- data[, xvar]

    spotlights <- stats::quantile(spotvar, c(0.15, 0.5, 0.85), type = 3)

    if (is.null(spotlight.labels)) {
      spotlight.labels <- paste0(
        c("15th percentile (", "50th percentile (", "85th percentile ("),
        c(ip_round2(as.numeric(spotlights), max.d = legend.round[2], min.d = legend.round[1])),
        c(")", ")", ")")
      )
    }
  }

  if (is.null(spotlight.labels)) spotlight.labels <- as.numeric(spotlights)

  if (nux <= 3) simple.slopes <- ip_compute_slopes_discrete(ux, zs, model, xvar, zvar)
  if (nux > 3) simple.slopes <- ip_compute_slopes_continuous(spotlights, data, xs, zs, model, xvar, zvar, moderator.on.x.axis)

  if (nux <= 3) jn <- ip_compute_jn_discrete(ux, zs, model, xvar, zvar)
  if (nux > 3) jn <- ip_compute_jn_continuous(spotlights, data, xs, zs, model, xvar, zvar, moderator.on.x.axis)

  fxz.list <- ip_make_fxz(data, n.bin.continuous, moderation, nux, max.unique, spotlights, xvar, zvar, moderator.on.x.axis)
  fxz <- fxz.list$fxz

  gr <- fxz
  for (j in 1:ncol(fxz)) gr[, j] <- pmin(fxz[, j] / n.max, 1)

  df1 <- data.frame(do.call(rbind, simple.slopes))
  df2 <- data.frame(do.call(rbind, jn))

  df1 <- df1[, !names(df1) %in% c("rowid", "y", "s.value", "p.value", "statistic", yvar)]
  df2 <- df2[, !names(df2) %in% c("rowid", "y", "s.value", "statistic", "term", "predicted_lo", "predicted_hi", "predicted", yvar)]

  names(df1)[names(df1) == "estimate"] <- "y.hat"
  names(df2)[names(df2) == "estimate"] <- "marginal.effect"

  if (ncol(fxz) == 2) frequencies <- data.frame(bin = rownames(fxz), f1 = fxz[, 1], f2 = fxz[, 2], row.names = NULL)
  if (ncol(fxz) == 3) frequencies <- data.frame(bin = rownames(fxz), f1 = fxz[, 1], f2 = fxz[, 2], f3 = fxz[, 3], row.names = NULL)

  output <- list(simple.slopes = df1, johnson.neyman = df2, frequencies = frequencies)
  if (v$input.model == FALSE) {
    output$gam_results <- model
    if (!is.null(gam_results_testing)) {
      output$gam_results_testing <- gam_results_testing
    }
    if (identical(engine, "gam")) {
      output$lm2_results <- if (!is.null(lm2_results_override)) {
        lm2_results_override
      } else {
        tryCatch(
          {
            if (!requireNamespace("estimatr", quietly = TRUE)) {
              NULL
            } else {
              fo_lm2 <- stats::as.formula(paste(yvar, "~", xvar, "*", zvar))
              fit_lm2 <- lm2(fo_lm2, data = data, notes = FALSE)
              # Make printed Call reflect the actual interaction formula.
              attr(fit_lm2, "lm2_call") <- as.call(list(as.name("lm2"), fo_lm2))
              fit_lm2
            }
          },
          error = function(e) NULL
        )
      }
    }
  }

  if (v$input.model == TRUE) {
    if (!inherits(model, "gam")) {
      linear.st <- ""
      if (inherits(model, "lm")) linear.st <- "Linear "
      if (main1 == "GAM Simple Slopes") main1 <- paste0(linear.st, "Simple Slopes")
      if (main2 == "GAM Johnson-Neyman") main2 <- paste0(linear.st, "Johnson-Neyman")
    }
  }

  if (ylab1 == "") ylab1 <- yvar
  if (ylab2 == "") ylab2 <- paste0("Marginal effect of ", xvar)

  draw2 <- draw
  if (identical(draw2, "simple.slopes")) draw2 <- "simple slopes"

  if (!is.null(save.as)) {
    extension <- tools::file_ext(save.as)

    if (draw2 == "both") {
      if (extension == "svg") grDevices::svg(save.as, width = 14, height = 7)
      if (extension == "png") grDevices::png(save.as, width = 14000, height = 7000, res = 1000)
      par(mfrow = c(1, 2))
      par(oma = c(0, 1, 0, 0))
    } else {
      if (extension == "svg") grDevices::svg(save.as, width = 7, height = 7)
      if (extension == "png") grDevices::png(save.as, width = 7000, height = 7000, res = 1000)
      par(oma = c(0, 1, 0, 0))
    }

    if (draw2 %in% c("both", "simple slopes")) {
      ip_make_plot(
        type = "simple.slopes",
        xlab, ylab1, main1, simple.slopes, histogram, data, xs, zs, gr, spotlights, cols, spotlight.labels,
        focal, moderation, max.unique, fxz.list, nux, nuz, xvar, zvar, xlim, ylim1, legend.title = legend.simple.slopes,
        x.ticks, y1.ticks, moderator.on.x.axis
      )
    }

    if (draw2 %in% c("both", "jn")) {
      ip_make_plot(
        type = "jn",
        xlab, ylab2, main2, jn, histogram, data, xs, zs, gr, spotlights, cols, spotlight.labels,
        focal, moderation, max.unique, fxz.list, nux, nuz, xvar, zvar, xlim, ylim2, legend.title = legend.jn,
        x.ticks, y2.ticks, moderator.on.x.axis
      )
    }

    message(paste0("The figures have been saved to '", save.as, "'"))
    dev.off()
  }

  old_par <- par(no.readonly = TRUE)
  par(oma = c(0, 1, 0, 0))

  if (draw2 == "both") {
    par(mfrow = c(1, 2))
    on.exit(par(old_par))
  }

  breaks <- NULL

  if (draw2 %in% c("simple slopes", "both")) {
    breaks <- ip_make_plot(
      type = "simple.slopes",
      xlab, ylab1, main1, simple.slopes, histogram, data, xs, zs, gr, spotlights, cols, spotlight.labels,
      focal, moderation, max.unique, fxz.list, nux, nuz, xvar, zvar, xlim, ylim1, legend.title = legend.simple.slopes,
      x.ticks, y1.ticks, moderator.on.x.axis
    )
  }

  if (draw2 %in% c("jn", "both")) {
    breaks <- ip_make_plot(
      type = "jn",
      xlab, ylab2, main2, jn, histogram, data, xs, zs, gr, spotlights, cols, spotlight.labels,
      focal, moderation, max.unique, fxz.list, nux, nuz, xvar, zvar, xlim, ylim2, legend.title = legend.jn,
      x.ticks, y2.ticks, moderator.on.x.axis
    )
  }

  regions.jn <- "N/A"
  if (draw2 != "simple slopes" & quiet == FALSE & focal == "categorical") {
    regions.jn <- ip_get_regions_jn(df2, xvar, zvar, focal, probe.bins)
    cat(regions.jn)
  }

  if (!is.null(breaks)) {
    output$frequencies$bin_from <- breaks$from
    output$frequencies$bin_to <- breaks$to
  }

  invisible(output)
}

# Helpers: `all.vars()` lists RHS of `$` (column names) as if they were free
# variables — they must not be required to exist as bindings (see t.test2).
#' @noRd
.collect_dollar_rhs_symbols <- function(expr) {
  acc <- character()
  walk <- function(e) {
    if (is.call(e)) {
      op <- e[[1L]]
      if (identical(op, quote(`$`)) || identical(op, as.name("$"))) {
        if (length(e) >= 3L) {
          rh <- e[[3L]]
          if (is.symbol(rh) || is.name(rh)) {
            acc <<- c(acc, as.character(rh))
          }
        }
      }
      if (length(e) >= 2L) {
        for (i in 2L:length(e)) walk(e[[i]])
      }
    }
  }
  walk(expr)
  unique(acc)
}

#' @noRd
formula_symbols_required_in_env <- function(formula) {
  av <- all.vars(formula)
  setdiff(av, .collect_dollar_rhs_symbols(formula))
}

.extract_formula_side_label <- function(expr) {
  if (is.symbol(expr) || is.name(expr)) {
    return(as.character(expr))
  }
  es <- deparse(expr, width.cutoff = 500L)
  if (grepl("\\$", es)) {
    return(trimws(sub(".*\\$", "", es)))
  }
  trimws(es)
}

#' Validate Formula Variables
#'
#' Checks if the input is a formula and validates that all variables mentioned
#' in the formula exist either in the provided data frame or in the environment.
#' This is a lightweight validation function that should be called early in functions
#' that accept formula syntax.
#'
#' @param formula A potential formula object to validate (can be any object).
#' @param data An optional data frame containing the variables.
#' @param func_name Character string. Name of the calling function (for error messages).
#' @param calling_env The environment in which to look for variables if data is not provided.
#'   Defaults to parent.frame().
#'
#' @return Returns NULL invisibly. Stops with an error if validation fails.
#'
#' @keywords internal
validate_formula <- function(formula, data = NULL, func_name = "function", calling_env = parent.frame()) {
  # Capture the call to get the actual expression passed (for better error messages)
  parent_call <- sys.call(-1)
  formula_expr <- if (!is.null(parent_call) && length(parent_call) >= 2) {
    parent_call[[2]]  # Get the formula argument expression
  } else {
    NULL
  }
  
  # First, check if we can even access the object
  # This catches cases like df$var where df doesn't exist
  obj_result <- tryCatch(
    {
      obj <- force(formula)
      list(success = TRUE, obj = obj)
    },
    error = function(e) {
      list(success = FALSE, error = e$message)
    }
  )
  
  if (!obj_result$success) {
    stop(obj_result$error, call. = FALSE)
  }
  
  # Check if the result is NULL (which happens when df$column where column doesn't exist)
  if (is.null(obj_result$obj)) {
    # Try to extract df name and column name from the expression
    if (!is.null(formula_expr) && is.call(formula_expr) && length(formula_expr) >= 3) {
      if (as.character(formula_expr[[1]]) == "$") {
        df_name <- as.character(formula_expr[[2]])
        col_name <- as.character(formula_expr[[3]])
        stop(sprintf("%s(): Column '%s' not found in '%s'", func_name, col_name, df_name), call. = FALSE)
      }
    }
    # Fallback error if we can't parse the expression
    stop(sprintf("%s(): The input evaluated to NULL", func_name), call. = FALSE)
  }
  
  # Now check if input is actually a formula
  if (!inherits(obj_result$obj, "formula")) {
    # Not a formula, return silently (allow non-formula inputs)
    return(invisible(NULL))
  }
  
  # From here on, use obj_result$obj instead of formula
  formula <- obj_result$obj
  
  formula_env <- environment(formula)
  if (is.null(formula_env)) {
    formula_env <- calling_env
  }
  
  # With data=: verify by evaluation in data context (supports df$col ~ df$col2).
  # With env only: require bindings only for symbols that are not `$` column names.
  if (!is.null(data)) {
    if (!is.data.frame(data)) {
      stop(sprintf("%s(): 'data' must be a data frame", func_name), call. = FALSE)
    }
    parent_call <- sys.call(-1)
    data_name <- if (!is.null(parent_call) && "data" %in% names(parent_call)) {
      deparse(parent_call$data)
    } else {
      "data"
    }
    data_name <- gsub('^"|"$', '', data_name)
    tmp_env <- list2env(as.list(data, use.names = TRUE), parent = calling_env)
    # Prefer evaluation (handles df$col). If evaluation fails (e.g. x1+x2 with factors),
    # fall back to: bare symbols in the expression must be columns of data.
    eval_formula_side <- function(expr) {
      ev <- tryCatch(
        list(ok = TRUE, val = suppressWarnings(eval(expr, envir = tmp_env))),
        error = function(e) list(ok = FALSE, err = e)
      )
      if (isTRUE(ev$ok)) {
        return(invisible(ev$val))
      }
      vars_in <- all.vars(expr)
      miss <- character(0)
      for (v in vars_in) {
        if (!v %in% names(data) && !exists(v, envir = calling_env, inherits = TRUE)) {
          miss <- c(miss, v)
        }
      }
      if (length(miss) > 0) {
        stop(sprintf('%s(): Variable "%s" not found in dataset "%s"', func_name, miss[1], data_name), call. = FALSE)
      }
      invisible(NULL)
    }
    if (length(formula) >= 2L) eval_formula_side(formula[[2L]])
    if (length(formula) >= 3L) eval_formula_side(formula[[3L]])
  } else {
    must_exist <- formula_symbols_required_in_env(formula)
    missing_vars <- character(0)
    for (var in must_exist) {
      if (!exists(var, envir = formula_env, inherits = TRUE)) {
        missing_vars <- c(missing_vars, var)
      }
    }
    if (length(missing_vars) > 0) {
      if (length(missing_vars) == 1) {
        stop(sprintf("%s(): Could not find variable '%s'", func_name, missing_vars[1]), call. = FALSE)
      } else {
        missing_list <- paste0("'", missing_vars, "'", collapse = ", ")
        stop(sprintf("%s(): Could not find variables: %s", func_name, missing_list), call. = FALSE)
      }
    }
  }
  
  invisible(NULL)
}

#' Validate Inputs for Plotting Functions
#'
#' Validates inputs for plotting functions that accept either formula syntax
#' (y ~ group) or standard syntax (y, group), with optional data frame.
#'
#' @param y A numeric vector, column name, or formula of the form y ~ group.
#' @param group A vector used to group the data, or a column name if data is provided.
#'   Can be NULL for functions where group is optional.
#' @param data An optional data frame containing the variables.
#' @param func_name Character string. Name of the calling function (for error messages).
#' @param require_group Logical. If TRUE, group is required. If FALSE, group can be NULL.
#' @param data_name Character string. Name of the data argument (for error messages). 
#'   If NULL, will attempt to infer from the call.
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{y}: Validated y variable (numeric vector)
#'   \item \code{group}: Validated group variable (if provided, otherwise NULL)
#'   \item \code{y_name}: Clean variable name for y (for labels)
#'   \item \code{group_name}: Clean variable name for group (for labels)
#'   \item \code{y_name_raw}: Raw variable name for y (for error messages)
#'   \item \code{group_name_raw}: Raw variable name for group (for error messages)
#'   \item \code{data_name}: Name of data argument (for error messages)
#' }
#'
#' @keywords internal
validate_plot <- function(y, group = NULL, data = NULL, func_name = "plot", require_group = TRUE, data_name = NULL) {
  # Capture data name for error messages (before potentially overwriting)
  # If data_name not provided, try to infer it from the call
  if (is.null(data_name)) {
    # Try to get it from parent frame (the calling function)
    parent_call <- sys.call(-1)
    parent_func <- sys.function(-1)
    if (!is.null(parent_call) && !is.null(parent_func)) {
      # Wrap in tryCatch in case match.call fails (e.g., in test contexts)
      parent_mc <- tryCatch({
        match.call(definition = parent_func, call = parent_call)
      }, error = function(e) NULL)
      if (!is.null(parent_mc) && "data" %in% names(parent_mc)) {
        data_expr <- parent_mc$data
        if (!is.null(data_expr)) {
          data_name <- deparse(data_expr)
          # Remove quotes if present
          data_name <- gsub('^"|"$', '', data_name)
        }
      }
    }
    # Fallback to "data" if we couldn't determine it
    if (is.null(data_name)) {
      data_name <- "data"
    }
  }
  
  # Check if y is a formula
  # Use tryCatch to avoid "object not found" error if y is a symbol in data
  is_formula <- tryCatch(inherits(y, "formula"), error = function(e) FALSE)
  
  if (is_formula) {
    # Formula syntax: evaluate lhs/rhs (supports df$col ~ df$col2).
    fo <- y
    if (length(fo) < 2L || !identical(fo[[1L]], quote(`~`))) {
      stop(sprintf("%s(): Invalid formula", func_name), call. = FALSE)
    }
    if (require_group && length(fo) < 3L) {
      stop(sprintf("%s(): Formula must have exactly two variables: response ~ group", func_name), call. = FALSE)
    }
    formula_env <- environment(fo)
    if (is.null(formula_env)) {
      formula_env <- parent.frame()
    }
    enc <- parent.frame()
    if (length(fo) == 2L) {
      lhs_expr <- fo[[2L]]
      y_name_raw <- deparse(lhs_expr, width.cutoff = 500L)
      y_name <- .extract_formula_side_label(lhs_expr)
      group <- NULL
      group_name_raw <- NULL
      group_name <- NULL
      if (!is.null(data)) {
        if (!is.data.frame(data)) {
          stop(sprintf("%s(): 'data' must be a data frame", func_name), call. = FALSE)
        }
        y <- tryCatch(
          eval(lhs_expr, envir = data, enclos = enc),
          error = function(e) stop(sprintf("%s(): %s", func_name, e$message), call. = FALSE)
        )
      } else {
        y <- tryCatch(
          eval(lhs_expr, envir = formula_env),
          error = function(e) stop(sprintf("%s(): %s", func_name, e$message), call. = FALSE)
        )
      }
    } else {
      lhs_expr <- fo[[2L]]
      rhs_expr <- fo[[3L]]
      y_name_raw <- deparse(lhs_expr, width.cutoff = 500L)
      y_name <- .extract_formula_side_label(lhs_expr)
      rhs_is_intercept_only <- is.numeric(rhs_expr) && length(rhs_expr) == 1L &&
        !is.na(rhs_expr[1]) && rhs_expr[1] == 1
      if (!require_group && rhs_is_intercept_only) {
        group <- NULL
        group_name_raw <- NULL
        group_name <- NULL
        if (!is.null(data)) {
          if (!is.data.frame(data)) {
            stop(sprintf("%s(): 'data' must be a data frame", func_name), call. = FALSE)
          }
          y <- tryCatch(
            eval(lhs_expr, envir = data, enclos = enc),
            error = function(e) stop(sprintf("%s(): %s", func_name, e$message), call. = FALSE)
          )
        } else {
          y <- tryCatch(
            eval(lhs_expr, envir = formula_env),
            error = function(e) stop(sprintf("%s(): %s", func_name, e$message), call. = FALSE)
          )
        }
      } else {
        group_name_raw <- deparse(rhs_expr, width.cutoff = 500L)
        group_name <- .extract_formula_side_label(rhs_expr)
        if (!is.null(data)) {
          if (!is.data.frame(data)) {
            stop(sprintf("%s(): 'data' must be a data frame", func_name), call. = FALSE)
          }
          y <- tryCatch(
            eval(lhs_expr, envir = data, enclos = enc),
            error = function(e) stop(sprintf("%s(): %s", func_name, e$message), call. = FALSE)
          )
          group <- tryCatch(
            eval(rhs_expr, envir = data, enclos = enc),
            error = function(e) stop(sprintf("%s(): %s", func_name, e$message), call. = FALSE)
          )
        } else {
          y <- tryCatch(
            eval(lhs_expr, envir = formula_env),
            error = function(e) stop(sprintf("%s(): %s", func_name, e$message), call. = FALSE)
          )
          group <- tryCatch(
            eval(rhs_expr, envir = formula_env),
            error = function(e) stop(sprintf("%s(): %s", func_name, e$message), call. = FALSE)
          )
        }
      }
    }
  } else {
    # Standard syntax: y, group
    # Use match.call() to capture the actual expressions passed, not parameter names
    mc <- match.call()
    y_expr <- mc$y
    group_expr <- mc$group
    
    # Capture y name for xlab (before potentially overwriting y)
    if (!is.null(y_expr)) {
      y_name_raw <- deparse(y_expr)
    } else {
      y_name_raw <- deparse(substitute(y))
    }
    # Remove quotes if present (handles both y = "col" and y = col)
    y_name_raw <- gsub('^"|"$', '', y_name_raw)
    y_name <- if (grepl("\\$", y_name_raw)) {
      strsplit(y_name_raw, "\\$")[[1]][length(strsplit(y_name_raw, "\\$")[[1]])]
    } else {
      y_name_raw
    }
    
    # Capture group name for legend title (before potentially overwriting group)
    if (!is.null(group_expr)) {
      group_name_raw <- deparse(group_expr)
      # Remove quotes if present (handles both group = "col" and group = col)
      group_name_raw <- gsub('^"|"$', '', group_name_raw)
      group_name <- if (grepl("\\$", group_name_raw)) {
        strsplit(group_name_raw, "\\$")[[1]][length(strsplit(group_name_raw, "\\$")[[1]])]
      } else {
        group_name_raw
      }
    } else {
      group_name_raw <- NULL
      group_name <- NULL
    }
    
    # Handle data frame if provided
    if (!is.null(data)) {
      if (!is.data.frame(data)) {
        stop(sprintf("%s(): 'data' must be a data frame", func_name), call. = FALSE)
      }
      
      # Extract columns from data frame
      # Use raw names for column lookup (they may include df$ prefix)
      if (!y_name_raw %in% names(data)) {
        stop(sprintf("%s(): Column \"%s\" not found in dataset \"%s\"", func_name, y_name_raw, data_name), call. = FALSE)
      }
      if (!is.null(group_name_raw) && !group_name_raw %in% names(data)) {
        stop(sprintf("%s(): Column \"%s\" not found in dataset \"%s\"", func_name, group_name_raw, data_name), call. = FALSE)
      }
      
      y <- data[[y_name_raw]]
      if (!is.null(group_name_raw)) {
        group <- data[[group_name_raw]]
      }
    }
  }
  
  # Validate that y is a numeric vector
  if (!is.numeric(y) || !is.vector(y)) {
    stop(sprintf("%s(): '%s' must be a numeric vector, and '%s' is not", func_name, ifelse(is_formula, "y", y_name_raw), y_name_raw), call. = FALSE)
  }
  
  # Validate group if required
  if (require_group && is.null(group)) {
    stop(sprintf("%s(): 'group' argument is required", func_name), call. = FALSE)
  }
  
  # Convert factor to character if needed
  if (!is.null(group) && is.factor(group)) {
    group <- as.character(group)
  }
  
  # Validate group type and length if provided
  if (!is.null(group)) {
    if (!is.vector(group)) {
      stop(sprintf("%s(): 'group' must be a vector", func_name), call. = FALSE)
    }
    if (length(y) != length(group)) {
      stop(sprintf("%s(): 'y' and 'group' must have the same length (y has %d, group has %d)", func_name, length(y), length(group)), call. = FALSE)
    }
  }
  
  # Return validated inputs
  list(
    y = y,
    group = group,
    y_name = y_name,
    group_name = group_name,
    y_name_raw = y_name_raw,
    group_name_raw = group_name_raw,
    data_name = data_name
  )
}

#' Validate Inputs for table2() Function
#'
#' Validates inputs for table2() function that accepts multiple variables via ...
#' with optional data frame.
#'
#' @param ... One or more variables to be tabulated.
#' @param data An optional data frame containing the variables.
#' @param func_name Character string. Name of the calling function (for error messages).
#'   Default is "table2".
#' @param data_name Character string. Name of the data argument (for error messages).
#'   If NULL, will attempt to infer from the call.
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{dots}: List of evaluated variables (ready for base::table)
#'   \item \code{dot_expressions}: List of expressions (for variable name extraction)
#'   \item \code{data_name}: Name of data argument (for error messages)
#' }
#'
#' @keywords internal
validate_table2 <- function(..., data = NULL, func_name = "table2", data_name = NULL) {
  # Capture data name for error messages
  # If data_name not provided, try to infer it from the call
  if (is.null(data_name)) {
    # Try to get it from parent frame (the calling function)
    parent_call <- sys.call(-1)
    parent_func <- sys.function(-1)
    if (!is.null(parent_call) && !is.null(parent_func)) {
      # Wrap in tryCatch in case match.call fails (e.g., in test contexts)
      parent_mc <- tryCatch({
        match.call(definition = parent_func, call = parent_call)
      }, error = function(e) NULL)
      if (!is.null(parent_mc) && "data" %in% names(parent_mc)) {
        data_expr <- parent_mc$data
        if (!is.null(data_expr)) {
          data_name <- deparse(data_expr)
          # Remove quotes if present
          data_name <- gsub('^"|"$', '', data_name)
        }
      }
    }
    # Fallback to "data" if we couldn't determine it
    if (is.null(data_name)) {
      data_name <- "data"
    }
  }
  
  # Handle data argument and capture expressions
  # If data is provided, evaluate unquoted variable names in the data context
  if (!is.null(data)) {
    if (!is.data.frame(data)) {
      stop(sprintf("%s(): 'data' must be a data frame", func_name), call. = FALSE)
    }
    
    # Capture expressions before evaluation
    dot_expressions <- as.list(substitute(list(...)))[-1L]
    
    # Evaluate each expression in the data context
    dots_list <- lapply(dot_expressions, function(expr) {
      # Check if it's a symbol (unquoted variable name)
      if (is.symbol(expr) || is.name(expr)) {
        var_name <- as.character(expr)
        if (var_name %in% names(data)) {
          return(data[[var_name]])
        } else {
          stop(sprintf("%s(): Column '%s' not found in dataset '%s'", func_name, var_name, data_name), call. = FALSE)
        }
      } else {
        # It's an expression (like df$var), evaluate it with data in context
        tryCatch({
          return(eval(expr, envir = data, enclos = parent.frame()))
        }, error = function(e) {
          # Try to extract variable name from expression for better error message
          var_name <- tryCatch({
            if (is.call(expr) && length(expr) >= 3) {
              op <- expr[[1]]
              if (identical(op, quote(`$`)) || identical(op, as.name("$"))) {
                as.character(expr[[3]])
              } else {
                deparse(expr)
              }
            } else {
              deparse(expr)
            }
          }, error = function(e2) "variable")
          stop(sprintf("%s(): Could not evaluate '%s' in dataset '%s': %s", func_name, var_name, data_name, e$message), call. = FALSE)
        })
      }
    })
    
    # Convert to list for base::table
    dots <- dots_list
  } else {
    # No data argument: use dots as-is
    dots <- list(...)
    dot_expressions <- as.list(substitute(list(...)))[-1L]
  }
  
  # Validate that all variables have the same length
  if (length(dots) > 1) {
    lengths <- sapply(dots, length)
    if (length(unique(lengths)) > 1) {
      # Extract variable names from expressions for better error message
      var_names <- sapply(dot_expressions, function(expr) {
        # Check if it's a symbol (variable name)
        if (is.symbol(expr) || is.name(expr)) {
          return(as.character(expr))
        }
        # Check if it's a dataframe column reference: df$var
        if (is.call(expr) && length(expr) >= 3) {
          op <- expr[[1]]
          if (identical(op, quote(`$`)) || identical(op, as.name("$"))) {
            return(as.character(expr[[3]]))
          }
        }
        # Fallback to deparsed expression
        return(deparse(expr))
      })
      
      # Build informative message showing each variable and its length
      var_info <- paste(sprintf("  %s (n=%d)", var_names, lengths), collapse = "\n")
      msg <- paste0(func_name, "(): All variables must have the same length, but these variables have different lengths:\n", var_info)
      message2(msg, col = "red", stop = TRUE)
    }
  }
  
  # Return validated inputs
  list(
    dots = dots,
    dot_expressions = dot_expressions,
    data_name = data_name
  )
}

#' Validate Grouping Variable for t.test2()
#'
#' Validates that a grouping variable exists and has exactly 2 levels for t-test.
#'
#' @param group_var_name Character string. Name of the grouping variable (for error messages).
#' @param data An optional data frame containing the variable.
#' @param calling_env The environment in which to look for the variable if data is not provided.
#' @param data_name Character string. Name of the data argument (for error messages).
#'   If NULL, will attempt to infer from the call.
#'
#' @return The validated grouping variable (numeric or factor vector). Stops execution with an error message if validation fails.
#'
#' @keywords internal
validate_t.test2 <- function(group_var_name, data = NULL, calling_env = parent.frame(), data_name = NULL) {
  # Capture data name for error messages (before potentially overwriting)
  # If data_name not provided, try to infer it from the call
  if (is.null(data_name)) {
    # Try to get it from parent frame (the calling function)
    parent_call <- sys.call(-1)
    parent_func <- sys.function(-1)
    if (!is.null(parent_call) && !is.null(parent_func)) {
      # Wrap in tryCatch in case match.call fails (e.g., in test contexts)
      parent_mc <- tryCatch({
        match.call(definition = parent_func, call = parent_call)
      }, error = function(e) NULL)
      if (!is.null(parent_mc) && "data" %in% names(parent_mc)) {
        data_expr <- parent_mc$data
        if (!is.null(data_expr)) {
          data_name <- deparse(data_expr)
          # Remove quotes if present
          data_name <- gsub('^"|"$', '', data_name)
        }
      }
    }
    # Fallback to "data" if we couldn't determine it
    if (is.null(data_name)) {
      data_name <- "data"
    }
  }
  
  # Check if variable exists in data or environment
  if (!is.null(data)) {
    # Data provided: check if variable exists in data frame
    if (!is.data.frame(data)) {
      message2("t.test2() says: 'data' must be a data frame", col = "red", stop = TRUE)
    }
    
    # Check if variable exists in data
    if (!group_var_name %in% names(data)) {
      message2(sprintf("t.test2() says: '%s' is not a variable in %s", group_var_name, data_name), col = "red", stop = TRUE)
    }
    
    # Extract variable from data
    group_var <- data[[group_var_name]]
  } else {
    # No data: check if variable exists before evaluating
    group_exists <- exists(group_var_name, envir = calling_env, inherits = TRUE)
    
    if (!group_exists) {
      message2(sprintf("t.test2() says: Could not find variable '%s'", group_var_name), col = "red", stop = TRUE)
    }
    
    # Variable exists, now evaluate
    group_var <- eval(as.name(group_var_name), envir = calling_env)
  }
  
  # Validate grouping variable has exactly 2 levels
  unique_groups <- unique(group_var)
  n_levels <- length(unique_groups[!is.na(unique_groups)])
  if (n_levels != 2) {
    message2(sprintf("t.test2() says: The grouping variable '%s' has %d unique values, cannot run t-test (requires exactly 2 levels).", group_var_name, n_levels), col = "red", stop = TRUE)
  }
  
  return(group_var)
}

#' Validate Inputs for lm2() Function
#'
#' Validates se_type and clusters arguments for lm2().
#' Also handles creating a data frame from vectors if data is not provided.
#'
#' @param formula A formula specifying the model.
#' @param data An optional data frame containing the variables.
#' @param se_type The type of standard error to use.
#' @param se_type_missing Logical. Whether se_type was not explicitly provided by user.
#' @param dots Additional arguments passed to lm_robust (to check for clusters).
#' @param calling_env The environment in which to look for variables if data is not provided.
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{data}: The data frame to use (either provided or constructed from vectors)
#'   \item \code{se_type}: The validated/adjusted se_type
#'   \item \code{has_clusters}: Logical indicating if clusters are being used
#' }
#'
#' @keywords internal
validate_lm2 <- function(formula, data = NULL, se_type = "HC3", se_type_missing = TRUE, 
                         dots = list(), calling_env = parent.frame()) {
  
  # Check if clusters are specified (and not NULL)
  has_clusters <- "clusters" %in% names(dots) && !is.null(dots$clusters)
  
  # Validate se_type based on whether clusters are used
  if (has_clusters) {
    # If clusters specified, se_type cannot be specified (we hardcode CR2)
    if (!se_type_missing && toupper(se_type) != "HC3") {
      message2("lm2() says: When using clusters, se_type cannot be specified. lm2() uses CR2 for clustered standard errors.", col = "red")
      invokeRestart("abort")
    }
    se_type <- "CR2"
  } else {
    # Without clusters, only HC0-HC3 are valid
    valid_se_types <- c("HC0", "HC1", "HC2", "HC3")
    if (!toupper(se_type) %in% valid_se_types) {
      message2(paste0("lm2() says: se_type must be one of: ", paste(valid_se_types, collapse = ", "), 
           ". Got: '", se_type, "'"), col = "red")
      invokeRestart("abort")
    }
    # Normalize to uppercase
    se_type <- toupper(se_type)
  }
  
  # If data is not provided, try to construct it from vectors in the environment
  if (is.null(data)) {
    # Extract variable names from formula (omit `$` RHS column names; see t.test2 / validate_formula)
    formula_vars <- all.vars(formula)
    must_exist <- formula_symbols_required_in_env(formula)
    
    # Check if all required bindings exist in the calling environment
    all_exist <- all(sapply(must_exist, function(v) exists(v, envir = calling_env, inherits = TRUE)))
    
    if (!all_exist) {
      missing_vars <- must_exist[!sapply(must_exist, function(v) exists(v, envir = calling_env, inherits = TRUE))]
      message2(paste0("lm2() says: Could not find variable(s): ", paste(missing_vars, collapse = ", "), 
           ". Either provide a 'data' argument or ensure variables exist in the environment."), col = "red")
      invokeRestart("abort")
    }
    
    # Get all variables from environment
    var_list <- lapply(formula_vars, function(v) {
      eval(as.name(v), envir = calling_env)
    })
    names(var_list) <- formula_vars
    
    # Check all have the same length
    lengths <- sapply(var_list, length)
    if (length(unique(lengths)) > 1) {
      length_info <- paste(sapply(seq_along(formula_vars), function(i) {
        paste0(formula_vars[i], " (", lengths[i], ")")
      }), collapse = ", ")
      message2(paste0("lm2() says: All variables must have the same length. Lengths: ", length_info), col = "red")
      invokeRestart("abort")
    }
    
    # Create data frame
    data <- as.data.frame(var_list, stringsAsFactors = FALSE)
    
    # Also check clusters variable if specified
    if (has_clusters) {
      cluster_var <- dots$clusters
      # If clusters is a symbol/name, evaluate it
      if (is.symbol(cluster_var) || is.name(cluster_var)) {
        cluster_name <- as.character(cluster_var)
        if (!exists(cluster_name, envir = calling_env, inherits = TRUE)) {
          message2(paste0("lm2() says: Could not find clusters variable: ", cluster_name), col = "red")
          invokeRestart("abort")
        }
        cluster_values <- eval(cluster_var, envir = calling_env)
        if (length(cluster_values) != nrow(data)) {
          message2("lm2() says: clusters variable must have the same length as other variables", col = "red")
          invokeRestart("abort")
        }
        data[[cluster_name]] <- cluster_values
      }
    }
  }
  
  list(
    data = data,
    se_type = se_type,
    has_clusters = has_clusters
  )
}

#validate_plot_001
test_that("validate_plot handles standard syntax correctly", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  result <- validate_plot(y, group, require_group = TRUE)
  expect_equal(length(result$y), 100)
  expect_equal(length(result$group), 100)
  expect_true(is.numeric(result$y))
  expect_equal(result$y_name_raw, "y")
  expect_equal(result$group_name_raw, "group")
})

#validate_plot_002
test_that("validate_plot handles data frame input", {
  df <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
  
  result <- validate_plot("value", "group", data = df, require_group = TRUE)
  expect_equal(length(result$y), 100)
  expect_equal(result$y_name, "value")
  expect_equal(result$group_name, "group")
})

#validate_plot_003
test_that("validate_plot handles formula syntax", {
  df <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
  
  result <- validate_plot(value ~ group, data = df, require_group = TRUE)
  expect_equal(length(result$y), 100)
  expect_equal(result$y_name, "value")
  expect_equal(result$group_name, "group")
})

#validate_plot_004
test_that("validate_plot handles formula syntax without data", {
  value <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  result <- validate_plot(value ~ group, require_group = TRUE)
  expect_equal(length(result$y), 100)
  expect_equal(result$y_name, "value")
  expect_equal(result$group_name, "group")
})

#validate_plot_005
test_that("validate_plot throws error for invalid inputs", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  # Wrong length
  expect_error(validate_plot(y, group[1:50], require_group = TRUE))
  
  # Non-numeric y
  expect_error(validate_plot(letters[1:10], rep(c("A", "B"), 5), require_group = TRUE))
  
  # Missing group when required
  expect_error(validate_plot(y, NULL, require_group = TRUE))
  
  # Missing column in data
  df <- data.frame(value = rnorm(100))
  expect_error(validate_plot("value", "group", data = df, require_group = TRUE))
  
  # Invalid data (not a data frame)
  expect_error(validate_plot("value", "group", data = list(value = 1:10), require_group = TRUE))
})

#validate_plot_006
test_that("validate_plot handles optional group", {
  y <- rnorm(100)
  
  # Should work when group is optional
  result <- validate_plot(y, NULL, require_group = FALSE)
  expect_equal(length(result$y), 100)
  expect_null(result$group)
})

#validate_plot_007
test_that("validate_plot handles df$var syntax", {
  df <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
  
  result <- validate_plot(df$value, df$group, require_group = TRUE)
  expect_equal(length(result$y), 100)
  expect_equal(result$y_name, "value")
  expect_equal(result$group_name, "group")
})

# ============================================================================
# ERROR MESSAGE QUALITY TESTS
# ============================================================================

#validate_plot_008
test_that("validate_plot error includes variable name", {
  df <- data.frame(value = rnorm(10))
  
  expect_error(
    validate_plot(missing_var ~ group, data = df),
    "missing_var"
  )
})

#validate_plot_009
test_that("validate_plot error message includes function name", {
  y <- letters[1:10]  # Non-numeric
  
  expect_error(
    validate_plot(y, NULL, func_name = "my_func", require_group = FALSE),
    "my_func"
  )
})

#validate_plot_010
test_that("validate_plot error for length mismatch includes lengths", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 25)  # Length 50
  
  expect_error(
    validate_plot(y, group, require_group = TRUE),
    "100"
  )
  expect_error(
    validate_plot(y, group, require_group = TRUE),
    "50"
  )
})

# ============================================================================
# ADDITIONAL VALIDATION TESTS
# ============================================================================

#validate_plot_011
test_that("validate_plot handles formula with no group (y ~ 1)", {
  df <- data.frame(value = rnorm(50))
  
  # Formula with just y ~ 1 should work when group not required
  result <- validate_plot(value ~ 1, data = df, require_group = FALSE)
  expect_equal(length(result$y), 50)
  expect_null(result$group)
})

#validate_plot_012
test_that("validate_plot with explicit data_name parameter", {
  my_data <- data.frame(value = rnorm(50), group = rep("A", 50))
  
  # When data_name is explicitly provided, it should be used
  result <- validate_plot(value ~ group, data = my_data, require_group = TRUE, data_name = "my_data")
  expect_equal(result$data_name, "my_data")
})

#' Enhanced alternative to text()
#' 
#' Adds to text() optional background color and horizontal alignment (align='center')
#'
#' @param x,y coordinates for text placement 
#' @param labels text to display
#' @param align alignment in relation to x coordinate ('left','center','right')
#' @param bg background color
#' @param cex character expansion factor
#' @param pad left/right padding in percentage (e.g., .03)
#' @param pad_v top/bottom padding in percentage (e.g., .25)
#' @param ... Additional arguments passed to \code{\link[graphics]{text}}.
#'
#' @return No return value, called for side effects. Adds text with an optional 
#'   background rectangle to an existing plot.
#'
#' @examples
#' # Create a simple plot
#' plot(1:10, 1:10, type = "n", main = "text2() - Alignment & Color")
#' 
#' # Alignment with respect to x=5
#' text2(5, 8, "align='left' from 5", align = "left", bg = "yellow1")
#' text2(5, 7, "align='right' from 5", align = "right", bg = "blue", col = "white")
#' text2(5, 6, "align='center' from 5", align = "center", bg = "black", col = "white")
#' abline(v = 5, lty = 2)
#' 
#' # Multiple labels with different alignments
#' text2(c(2, 5, 8), c(5, 5, 5), 
#'       labels = c("Left", "Center", "Right"),
#'       align = c("left", "center", "right"),
#'       bg = c("pink", "lightblue", "lightgreen"))
#' 
#' # Text with custom font color (passed through ...)
#' text2(5, 3, "Red Text", col = "red", bg = "white")
#' 
#' # Padding examples
#' plot(1:10, 1:10, type = "n", main = "Padding Examples")
#' 
#' # Default padding (pad=0.03, pad_v=0.25)
#' text2(5, 8, "Default padding", bg = "lightblue")
#' 
#' # More horizontal padding
#' text2(5, 6, "Wide padding", pad = 0.2, bg = "lightgreen")
#' 
#' # More vertical padding
#' text2(5, 4, "Tall padding", pad_v = 0.8, bg = "lightyellow")
#' 
#' # Both padding increased
#' text2(5, 2, "Extra padding", pad = 0.15, pad_v = 0.6, bg = "pink")
#' @usage NULL
#' @export
text2 <- function(x, y, labels, align="center", bg="papayawhip", cex=1, pad=0.03, pad_v=0.25,  ...) {

  dots <- list(...)
  n <- length(x)

  # Validate and recycle align argument
  valid_aligns <- c("left", "center", "right")
  if (length(align) == 1) {
    align <- match.arg(align, valid_aligns)
    align <- rep(align, length(x))
  } else {
    align <- match.arg(align, valid_aligns, several.ok = TRUE)
    if (length(align) != length(x)) {
      align <- rep(align, length.out = length(x))
    }
  }
  
  # compute text physical size in USER coordinates
  w <- strwidth(labels, cex = cex)
  h <- strheight(labels, cex = cex)

  # padding
  pad_w <- pad * w
  pad_h <- pad_v * h

  # Process each text label
  for (i in seq_along(x)) {
    # Determine horizontal adjustment based on alignment
    if (align[i] == "left") {
      adj_x <- 0
      rect_x1 <- x[i] - pad_w[i]
      rect_x2 <- x[i] + w[i] + pad_w[i]
    } else if (align[i] == "right") {
      adj_x <- 1
      rect_x1 <- x[i] - w[i] - pad_w[i]
      rect_x2 <- x[i] + pad_w[i]
    } else { # center
      adj_x <- 0.5
      rect_x1 <- x[i] - (w[i]/2 + pad_w[i])
      rect_x2 <- x[i] + (w[i]/2 + pad_w[i])
    }
    
    # Handle vectorized bg
    bg_i <- if (length(bg) > 1) bg[i] else bg
    
    # draw background rectangle
    rect(rect_x1,
         y[i] - (h[i]/2 + pad_h[i]),
         rect_x2,
         y[i] + (h[i]/2 + pad_h[i]),
         col = bg_i, border = NA)

    # draw text with appropriate alignment
    # Per-point `text()` calls: vector args in ... must be subset (else only
    # `col[1]` etc. apply to every label — wrong for plot_means value colors).
    text_args <- list(
      x = x[i],
      y = y[i],
      labels = labels[i],
      cex = cex,
      adj = c(adj_x, 0.5)
    )
    if (length(dots)) {
      for (nm in names(dots)) {
        v <- dots[[nm]]
        lv <- length(v)
        if (!lv) next
        if (lv == n) {
          text_args[[nm]] <- v[i]
        } else if (lv == 1L) {
          text_args[[nm]] <- v
        } else {
          text_args[[nm]] <- v[((i - 1L) %% lv) + 1L]
        }
      }
    }
    do.call(graphics::text, text_args)
  }
}

#text2_001
test_that("text2 runs without errors", {
  # Create a simple plot first
  plot(1:10, 1:10, type = "n")
  
  # Should not throw errors
  expect_error(text2(5, 5, "Test"), NA)
  expect_error(text2(c(2, 8), c(3, 7), c("Left", "Right")), NA)
})

#text2_002
test_that("text2 handles different alignments", {
  plot(1:10, 1:10, type = "n")
  
  # Left alignment
  expect_error(text2(2, 8, "Left", align = "left"), NA)
  
  # Center alignment (default)
  expect_error(text2(5, 8, "Center", align = "center"), NA)
  
  # Right alignment
  expect_error(text2(8, 8, "Right", align = "right"), NA)
})

#text2_003
test_that("text2 handles vectorized alignments", {
  plot(1:10, 1:10, type = "n")
  
  # Multiple labels with different alignments
  expect_error(text2(c(2, 5, 8), c(5, 5, 5),
                      labels = c("Left", "Center", "Right"),
                      align = c("left", "center", "right")), NA)
})

#text2_004
test_that("text2 handles custom background colors", {
  plot(1:10, 1:10, type = "n")
  
  # Single color
  expect_error(text2(5, 5, "Test", bg = "yellow"), NA)
  
  # Multiple colors
  expect_error(text2(c(2, 8), c(5, 5), c("A", "B"), bg = c("red", "blue")), NA)
})

#text2_005
test_that("text2 handles cex parameter", {
  plot(1:10, 1:10, type = "n")
  
  # Different sizes
  expect_error(text2(5, 5, "Small", cex = 0.5), NA)
  expect_error(text2(5, 7, "Large", cex = 2), NA)
})

#text2_006
test_that("text2 handles pad parameters", {
  plot(1:10, 1:10, type = "n")
  
  # Custom padding
  expect_error(text2(5, 5, "Test", pad = 0.1), NA)
  expect_error(text2(5, 7, "Test", pad_v = 0.5), NA)
})

#text2_007
test_that("text2 handles additional arguments", {
  plot(1:10, 1:10, type = "n")
  
  # Pass through col argument
  expect_error(text2(5, 5, "Red text", col = "red", bg = "white"), NA)
  
  # Pass through font argument
  expect_error(text2(5, 7, "Bold text", font = 2, bg = "yellow"), NA)
})

#text2_008
test_that("text2 applies vector col per label (not only col[1])", {
  plot(1:10, 1:10, type = "n")
  expect_error(
    text2(
      c(2, 4, 6, 8), c(3, 5, 7, 9),
      c("a", "b", "c", "d"),
      bg = c("orange", "gray90", "gray20", "black"),
      col = c("black", "black", "white", "white")
    ),
    NA
  )
})

#text2_009
test_that("text2 handles multiple labels", {
  plot(1:10, 1:10, type = "n")
  
  # Multiple labels
  expect_error(text2(c(2, 5, 8), c(3, 5, 7),
                      labels = c("Label1", "Label2", "Label3")), NA)
})

#text2_010
test_that("text2 recycles bg color", {
  plot(1:10, 1:10, type = "n")
  
  # Single bg color should be recycled
  expect_error(text2(c(2, 5, 8), c(3, 5, 7),
                      labels = c("A", "B", "C"),
                      bg = "yellow"), NA)
})

#text2_011
test_that("text2 handles edge cases", {
  plot(1:10, 1:10, type = "n")
  
  # Single point
  expect_error(text2(5, 5, "Single"), NA)
  
  # Empty labels (should error or handle gracefully)
  # This might error, which is acceptable
  # expect_error(text2(5, 5, ""), NA)
})

#' Format P-Values for Display 
#'
#' Formats p-values for clean display in figures and tables. e.g., p = .0231, p<.0001
#'
#' @param p A numeric vector of p-values to format.
#' @param digits Number of decimal places to round to. Default is 4.
#' @param include_p Logical. If TRUE, includes "p" prefix before the formatted
#'   value (e.g., "p = .05"). Default is FALSE.
#'
#' @return A character vector of formatted p-values.
#'
#' @examples
#' # Basic usage
#' format_pvalue(0.05)
#' format_pvalue(0.0001)
#' 
#' # More rounding
#' format_pvalue(0.0001,digits=2)
#'
#' # Vector input
#' format_pvalue(c(0.05, 0.001, 0.00001, 0.99))
#'
#' # With p prefix
#' format_pvalue(0.05, include_p = TRUE)
#'
#' @name format_pvalue
#' @export format_pvalue
format_pvalue <- function(p, digits = 4, include_p = FALSE) {
  # Handle NA values
  is_na <- is.na(p)
  result <- character(length(p))
  
  # Create p prefix if requested
  p_prefix <- if (include_p) "p " else ""
  
  # Handle edge cases first

  # Create dynamic threshold strings based on digits (e.g., ".01" for digits=2, ".001" for digits=3)
  min_threshold <- 10^(-digits)
  max_threshold <- 1 - 10^(-digits)
  min_str <- sub("^0\\.", ".", format(min_threshold, nsmall = digits, scientific = FALSE))
  max_str <- sub("^0\\.", ".", format(max_threshold, nsmall = digits, scientific = FALSE))
  
  result[p < min_threshold] <- paste0(p_prefix, "< ", min_str)
  result[p > max_threshold] <- paste0(p_prefix, "> ", max_str)
  
  # Handle regular p-values
  regular <- !is_na & p >= 10^(-digits) & p <= (1 - 10^(-digits))
  
  if (any(regular)) {
    p.clean <- round(p[regular], digits)
    p.clean <- format(p.clean, nsmall = digits, scientific = FALSE)
    # Remove leading zero
    p.clean <- sub("^0\\.", ".", p.clean)
    # Always add equals sign
    p.clean <- paste0(p_prefix, "= ", p.clean)
    result[regular] <- p.clean
  }
  
  # Handle NA values
  result[is_na] <- NA_character_
  
  return(result)
}

#format_pvalue_001
test_that("format_pvalue formats basic p-values correctly", {
  # Note: format_pvalue uses format() with nsmall=digits, so it always shows full decimal places
  expect_equal(format_pvalue(0.05), "= .0500")
  expect_equal(format_pvalue(0.001), "= .0010")
  expect_equal(format_pvalue(0.5), "= .5000")
  expect_equal(format_pvalue(0.1234), "= .1234")
})

#format_pvalue_002
test_that("format_pvalue handles edge cases", {
  expect_equal(format_pvalue(0.00001), "< .0001")
  expect_equal(format_pvalue(0.99999), "> .9999")
  expect_equal(format_pvalue(0), "< .0001")
  expect_equal(format_pvalue(1), "> .9999")
  expect_equal(format_pvalue(0.0001), "= .0001")
  expect_equal(format_pvalue(0.9999), "= .9999")
})

#format_pvalue_003
test_that("format_pvalue handles NA values", {
  result <- format_pvalue(c(0.05, NA, 0.01))
  expect_equal(result[1], "= .0500")
  expect_true(is.na(result[2]))
  expect_equal(result[3], "= .0100")
  
  # All NA
  result <- format_pvalue(c(NA, NA))
  expect_true(all(is.na(result)))
})

#format_pvalue_004
test_that("format_pvalue works with include_p parameter", {
  expect_equal(format_pvalue(0.05, include_p = TRUE), "p = .0500")
  expect_equal(format_pvalue(0.001, include_p = TRUE), "p = .0010")
  expect_equal(format_pvalue(0.00001, include_p = TRUE), "p < .0001")
  expect_equal(format_pvalue(0.99999, include_p = TRUE), "p > .9999")
})

#format_pvalue_005
test_that("format_pvalue handles vector input", {
  result <- format_pvalue(c(0.05, 0.001, 0.00001))
  expect_equal(length(result), 3)
  expect_equal(result[1], "= .0500")
  expect_equal(result[2], "= .0010")
  expect_equal(result[3], "< .0001")
})

#format_pvalue_006
test_that("format_pvalue respects digits parameter", {
  expect_equal(format_pvalue(0.12345, digits = 2), "= .12")
  expect_equal(format_pvalue(0.12345, digits = 3), "= .123")
  expect_equal(format_pvalue(0.12345, digits = 4), "= .1235")
  expect_equal(format_pvalue(0.1, digits = 1), "= .1")
})

#format_pvalue_007
test_that("format_pvalue handles boundary values correctly", {
  # Test values just above and below thresholds
  expect_equal(format_pvalue(0.00009, digits = 4), "< .0001")
  expect_equal(format_pvalue(0.0001, digits = 4), "= .0001")
  expect_equal(format_pvalue(0.9999, digits = 4), "= .9999")

  expect_equal(format_pvalue(0.99991, digits = 4), "> .9999")
})

#format_pvalue_008
test_that("format_pvalue threshold adapts to digits parameter", {
  # Very small p-value should use threshold based on digits
  expect_equal(format_pvalue(0.0000001, digits = 2), "< .01")
  expect_equal(format_pvalue(0.0000001, digits = 3), "< .001")
  expect_equal(format_pvalue(0.0000001, digits = 4), "< .0001")
  
  # Very large p-value should use threshold based on digits
  expect_equal(format_pvalue(0.9999999, digits = 2), "> .99")
  expect_equal(format_pvalue(0.9999999, digits = 3), "> .999")
  expect_equal(format_pvalue(0.9999999, digits = 4), "> .9999")
  
  # With include_p
  expect_equal(format_pvalue(0.0000001, digits = 2, include_p = TRUE), "p < .01")
  expect_equal(format_pvalue(0.0000001, digits = 3, include_p = TRUE), "p < .001")
})

# ============================================================================
# ADDITIONAL EDGE CASES
# ============================================================================

#format_pvalue_009
test_that("format_pvalue handles negative values", {
  # Negative p-values are invalid but function should handle gracefully
  # (or throw appropriate error)
  result <- tryCatch(
    format_pvalue(-0.05),
    error = function(e) "error"
  )
  # Should either produce a result or error gracefully
  expect_true(!is.null(result))
})

#format_pvalue_010
test_that("format_pvalue handles values > 1", {
  # p-values > 1 are invalid but function should handle gracefully
  result <- tryCatch(
    format_pvalue(1.5),
    error = function(e) "error"
  )
  expect_true(!is.null(result))
})

#format_pvalue_011
test_that("format_pvalue handles empty vector", {
  result <- format_pvalue(numeric(0))
  expect_equal(length(result), 0)
})

#format_pvalue_012
test_that("format_pvalue handles single NA", {
  result <- format_pvalue(NA_real_)
  expect_true(is.na(result))
})

#format_pvalue_013
test_that("format_pvalue includes_p works with vectors", {
  result <- format_pvalue(c(0.05, 0.001), include_p = TRUE)
  expect_true(all(grepl("^p", result)))
})

#' Plot density of a variable, optionally by another variable
#'
#' Plots the distribution of a variable by group, simply: \code{plot_density(y ~ x)}
#'
#' @param formula Two possible uses (similar to \code{t.test()}):
#'   \itemize{
#'     \item {Single Variable (possibly by subgroup)}: \code{plot_density(y)} or \code{plot_density(y~x)}
#'     \item {Contrast Two Variables}: \code{plot_density(y1, y2)}
#'   }
#' @param y2 optional second variable when contrasting two variables \code{plot_density(y1,y2)}
#' @param data An optional data frame containing the variables in the formula.
#' @param order Controls the order in which groups appear in the plot and legend. 
#'   Use \code{-1} to reverse the default order. Alternatively, provide a vector specifying
#'   the exact order (e.g., \code{c("B", "A", "C")}). If \code{NULL} (default), groups are 
#'   ordered by their factor levels (if the grouping variable is a factor) or sorted 
#'   alphabetically/numerically. Only applies when using grouped plots.
#' @param show_means Logical. If TRUE (default), shows points at means.
#' @param ... Additional arguments passed to plotting functions.

#' @details
#' Plot parameters like
#' \code{col}, \code{lwd}, \code{lty}, and \code{pch} can be specified as:
#' \itemize{
#'   \item A single value: applied to all groups
#'   \item A vector: applied to groups in order of unique \code{group} values
#' }
#'
#'
#' @examples
#' # Basic usage with formula syntax (no grouping)
#' y <- rnorm(100)
#' plot_density(y)
#'
#' # With grouping variable
#' group <- rep(c("A", "B", "C"), c(30, 40, 30))
#' plot_density(y ~ group)
#'
#' # With custom colors (scalar - same for all)
#' plot_density(y ~ group, col = "blue")
#'
#' # With custom colors (vector - different for each group)
#' plot_density(y ~ group, col = c("red", "green", "blue"))
#'
#' # Multiple parameters
#' plot_density(y ~ group, col = c("red", "green", "blue"), lwd = c(1, 2, 3))
#'
#' # With line type
#' plot_density(y ~ group, col = c("red", "green", "blue"), lty = c(1, 2, 3), lwd = 2)
#'
#' # Using data frame
#' df <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
#' plot_density(value ~ group, data = df)
#' plot_density(value ~ group, data = df, col = c("red", "blue"))
#'
#' # Compare two vectors
#' y1 <- rnorm(50)
#' y2 <- rnorm(50, mean = 1)
#' plot_density(y1, y2)
#'
#' @return Invisibly returns a list with the following element:
#'   \describe{
#'     \item{densities}{A named list of density objects (class \code{"density"}), 
#'       one for each group. Each density object contains \code{x} (evaluation points), 
#'       \code{y} (density estimates), \code{bw} (bandwidth), and other components 
#'       as returned by \code{\link[stats]{density}}. If no grouping variable is 
#'       provided, the list contains a single element named \code{"all"}.}
#'   }
#'   The function is primarily called for its side effect of creating a plot.
#'
#' @export
plot_density <- function(formula, y2 = NULL, data = NULL, order = NULL, show_means = TRUE, ...) {
  #OUTLINE
  #1. Capture variable names for labels
  #2. Extract and handle parameters
  #3. Handle data frame input
  #4. Drop missing data
  #5. Get unique groups
  #6. Initialize return values
  #7. Get default colors based on number of groups
  #8. Helper function: extract parameter value for a group
  #9. Separate density() arguments from plotting arguments
  #10. Compute densities for each group
  #11. Determine overall range for plotting
  #12. Helper function: NULL coalescing
  #13. Plot first density to set up the plot
  #14. Get parameters for first group
  #15. Build plot arguments
  #16. Set up plot
  #17. Add remaining densities
  #18. Add points at y=0 and x=mean with labels
  #19. Add vertical segments at means (if show.means=TRUE and 2-3 groups)
  #20. Add legend
  #21. Return densities
  
  #0. CAPTURE UNEVALUATED ARGUMENTS FIRST (before ANY evaluation!)
  mc <- match.call()
  
  # Resolve first argument (formula or first vector)
  formula_resolved <- evaluate_variable_arguments(
    arg_expr = mc$formula,
    arg_name = "formula",
    data = data,
    calling_env = parent.frame(),
    func_name = "plot_density",
    allow_null = FALSE
  )
  
  # Resolve second argument if present (for two-vector mode)
  y2_resolved <- if (!is.null(mc$y2)) {
    evaluate_variable_arguments(
      arg_expr = mc$y2,
      arg_name = "y2",
      data = data,
      calling_env = parent.frame(),
      func_name = "plot_density",
      allow_null = TRUE
    )
  } else {
    list(value = NULL, name = NULL, name_raw = NULL, was_symbol = FALSE)
  }
  
  # Now overwrite the arguments with resolved values
  formula <- formula_resolved$value
  y2 <- y2_resolved$value
  
  #1. Extract and handle parameters
  # Extract plotting parameters from ...
    dots <- list(...)
    
  # Check if we're in two-vector comparison mode (formula is vector, y is vector)
  if (!is.null(y2) && !inherits(formula, "formula")) {
    # Two-vector comparison mode: plot_density(y1, y2)
    # Use resolved names
    y1_name <- formula_resolved$name
    y2_name <- y2_resolved$name
    
    # Validate inputs
    if (!is.numeric(formula) || !is.vector(formula)) {
      stop(sprintf("plot_density(): First argument '%s' must be a numeric vector", y1_name), call. = FALSE)
    }
    if (!is.numeric(y2) || !is.vector(y2)) {
      stop(sprintf("plot_density(): Second argument '%s' must be a numeric vector", y2_name), call. = FALSE)
    }
    
    # Create a data frame and recursively call with grouped syntax
    df <- data.frame(
      value = c(formula, y2),
      group = c(rep(y1_name, length(formula)), rep(y2_name, length(y2))),
      stringsAsFactors = FALSE
    )
    
    # Forward all arguments to the grouped version
    return(plot_density(value ~ group, data = df, order = order, 
                       show_means = show_means, ...))
  }
    
    # Extract show.means parameter (from dots, as it's not a formal parameter)
    show_mean_segments <- if ("show.means" %in% names(dots)) dots$show.means else TRUE
    dots$show.means <- NULL  # Remove from dots so it doesn't get passed to plot functions
  
  # Validate formula early if it is one
  validate_formula(formula, data, func_name = "plot_density", calling_env = parent.frame())
  
  # Check if formula is actually a formula or a vector
  is_formula_input <- inherits(formula, "formula")
  
  # Capture data name for error messages
  data_expr <- mc$data
  data_name <- if (!is.null(data_expr)) {
    data_name_val <- deparse(data_expr)
    gsub('^"|"$', '', data_name_val)
  } else {
    NULL
  }
  
  # Capture original group before validation (to preserve factor levels)
  group_original_before_validation <- NULL
  if (is_formula_input && inherits(formula, "formula")) {
    f <- formula
    if (length(f) >= 3L && identical(f[[1L]], quote(`~`))) {
      rhs <- f[[3L]]
      rhs_is_intercept_only <- is.numeric(rhs) && length(rhs) == 1L &&
        !is.na(rhs[1]) && rhs[1] == 1
      if (!rhs_is_intercept_only) {
        group_original_before_validation <- tryCatch({
          if (!is.null(data)) {
            eval(rhs, envir = data, enclos = parent.frame())
          } else {
            fe <- environment(f)
            if (is.null(fe)) fe <- parent.frame()
            eval(rhs, envir = fe)
          }
        }, error = function(e) NULL)
      }
    }
  }
  
  #2. Validate inputs using validation function shared with plot_density, plot_cdf, plot_freq
  # If formula input, use validate_plot
  if (is_formula_input) {
    validated <- validate_plot(formula, NULL, data, func_name = "plot_density", require_group = FALSE, data_name = data_name)
  } else {
    # Not a formula - use resolved names from evaluate_variable_arguments
    validated <- list(
      y = formula,
      group = NULL,
      y_name = formula_resolved$name,
      group_name = NULL,
      y_name_raw = formula_resolved$name_raw,
      group_name_raw = NULL,
      data_name = data_name
    )
  }
  y <- validated$y
  group <- validated$group
  y_name <- validated$y_name
  group_name <- validated$group_name
  y_name_raw <- validated$y_name_raw
  group_name_raw <- validated$group_name_raw
  
  # Store original group for factor level checking (before NA removal)
  # Use the version captured before validation if available (to preserve factor levels)
  group_original <- if (!is.null(group_original_before_validation)) {
    group_original_before_validation
  } else {
    group
  }
  
  #3. Drop missing data
    if (!is.null(group)) {
      isnagroup=is.na(group)
      isnay=is.na(y)
      group=group[!isnagroup & !isnay]
      y=y[!isnagroup & !isnay]
      
      n.nagroup = sum(isnagroup)
      n.nay = sum(isnay)
      
      if (n.nagroup>0) message2("plot_density() says: dropped ",n.nagroup," observations with missing '",group_name_raw,"' values",col='red2')
      if (n.nay>0) message2("plot_density() says: dropped ",n.nay," observations with missing '",y_name_raw,"' values",col='red2')
    } else {
      isnay=is.na(y)
      y=y[!isnay]
      
      n.nay = sum(isnay)
      if (n.nay>0) message2("plot_density() says: dropped ",n.nay," observations with missing '",y_name_raw,"' values",col='red2')
    }
  
  #5. Get unique groups (if group is provided)
    if (!is.null(group)) {
      unique_groups <- unique(group)
      n_groups <- length(unique_groups)
      
      # Check if order = -1 (reverse default order)
      reverse_order <- FALSE
      if (!is.null(order) && length(order) == 1 && is.numeric(order) && order == -1) {
        reverse_order <- TRUE
        order <- NULL  # Process as default, then reverse
      }
      
      if (!is.null(order)) {
        # User specified custom order
        # Validate that order contains all groups
        missing_groups <- setdiff(unique_groups, order)
        extra_groups <- setdiff(order, unique_groups)
        
        if (length(missing_groups) > 0) {
          stop(sprintf("plot_density(): 'order' is missing group(s): %s", 
                       paste(missing_groups, collapse = ", ")), call. = FALSE)
        }
        if (length(extra_groups) > 0) {
          warning(sprintf("plot_density(): 'order' contains group(s) not in data: %s", 
                         paste(extra_groups, collapse = ", ")))
        }
        
        # Use the specified order (only groups that exist in data)
        unique_x <- order[order %in% unique_groups]
      } else if (is.factor(group_original)) {
        # Respect factor levels
        factor_levels <- levels(group_original)
        # Only include levels that actually appear in the data
        unique_x <- factor_levels[factor_levels %in% unique_groups]
      } else {
        # Default: sort alphabetically/numerically
        unique_x <- sort(unique_groups)
      }
      
      # Reverse order if order = -1 was specified
      if (reverse_order) {
        unique_x <- rev(unique_x)
      }
    } else {
      unique_x <- NULL
      n_groups <- 0
    }
  
  #6. Initialize return values
  
  #7. Get default colors based on number of groups
    if (is.null(group)) {
      default_colors <- get.colors(1) #See utils.R
    } else {
      default_colors <- get.colors(n_groups) #See utils.R
    }
  
  #8. Helper function: extract parameter value for a group
  # Helper function to extract parameter value for a group
    get_param <- function(param_name, group_idx) {
      if (param_name %in% names(dots)) {
        param_val <- dots[[param_name]]
        if (length(param_val) == 1) {
          return(param_val)
        } else if (length(param_val) >= group_idx) {
          return(param_val[group_idx])
        } else {
          return(param_val[1])  # Recycle if shorter
        }
      }
      return(NULL)
    }
    
  #9. Separate density() vs plotting arguments
    density_args <- c("bw", "adjust", "kernel", "n", "from", "to", "na.rm", "weights")
    density_params <- list()
    plot_params <- dots
    
    for (arg in density_args) {
      if (arg %in% names(dots)) {
        density_params[[arg]] <- dots[[arg]]
        plot_params[[arg]] <- NULL
      }
    }
  
  #10. Compute densities for each group (or single density if no group)
    density_list <- list()
    y_ranges <- list()
    y_density_max <- list()
    
    if (is.null(group)) {
      # No grouping: compute single density
      if (length(y) > 0) {
        density_obj <- do.call(density, c(list(x = y), density_params))
        density_list[[1]] <- density_obj
        y_ranges[[1]] <- range(density_obj$x)
        y_density_max[[1]] <- max(density_obj$y)
      }
    } else {
      # With grouping: compute density for each group
      for (i in seq_along(unique_x)) {
        group_val <- unique_x[i]
        y_group <- y[group == group_val]
        if (length(y_group) > 0) {
          # Compute density with any density-specific arguments
          density_obj <- do.call(density, c(list(x = y_group), density_params))
          density_list[[i]] <- density_obj
          y_ranges[[i]] <- range(density_obj$x)
          y_density_max[[i]] <- max(density_obj$y)
        }
      }
    }
  
  #11. Determine overall range for plotting
    all_x <- unlist(lapply(density_list, function(d) range(d$x)))
    all_y_density <- unlist(y_density_max)
    x_min <- min(all_x, na.rm = TRUE)
    x_max <- max(all_x, na.rm = TRUE)
    x_range <- x_max - x_min
    x_lim <- c(x_min - 0.05 * x_range, x_max + 0.05 * x_range)
    y_max_density <- max(all_y_density, na.rm = TRUE)
    # Reserve space for legend if there's a group (legend will be shown)
    if (!is.null(group)) {
      y_lim_density <- c(0, y_max_density * 1.3)  # Add 30% space for legend
    } else {
      y_lim_density <- c(0, y_max_density * 1.1)  # Add 10% space for margins only
    }

  #12. Helper function: NULL coalescing
    `%||%` <- function(x, y) if (is.null(x)) y else x
  
  #13. Plot first density to set up the plot
    if (length(density_list) > 0) {
      first_density <- density_list[[1]]
    
    #14. Get parameters for first group
      col1 <- get_param("col", 1) %||% default_colors[1]
      lwd1 <- get_param("lwd", 1) %||% 4  # Default lwd=4
      lty1 <- get_param("lty", 1) %||% 1
      type1 <- get_param("type", 1) %||% "l"
      pch1 <- get_param("pch", 1)
      
    #15. Build plot arguments
    # Build plot arguments
    # Set main title if not provided
      if (!"main" %in% names(plot_params)) {
        if (!is.null(group)) {
          # With grouping: show grouping variable like plot_cdf
          main_title <- paste0("Comparing Distribution of '", y_name, "' by '", group_name, "'")
        } else {
          # No grouping: simple format
          main_title <- paste0("Distribution of '", y_name, "'")
        }
      } else {
        main_title <- plot_params$main
      }
    # Set font and size for main title if not provided
      font_main <- if ("font.main" %in% names(plot_params)) plot_params$font.main else 2
      cex_main <- if ("cex.main" %in% names(plot_params)) plot_params$cex.main else 1.3
    
    # Set xlab if not provided
      xlab_title <- if ("xlab" %in% names(plot_params)) plot_params$xlab else y_name
    
    # Set ylab if not provided
      ylab_title <- if ("ylab" %in% names(plot_params)) plot_params$ylab else "Probability Density"
    
    # Set default ylim if not provided
      if (!"ylim" %in% names(plot_params)) {
        default_ylim <- y_lim_density
        default_ylim[2]=default_ylim[2]*1.15
      } else {
        default_ylim <- plot_params$ylim
        # Ensure ylim always starts at 0
        default_ylim[1] <- 0
      }
    
    # Set default xlim if not provided
      if (!"xlim" %in% names(plot_params)) {
        default_xlim <- x_lim
      } else {
        default_xlim <- plot_params$xlim
      }
    
    # Remove vectorized parameters and data from plot_params for plot()
    # Also remove xlab, ylab, main, xlim, ylim since we handle them separately
    vectorized_params <- c("col", "lwd", "lty", "type", "pch", "data")
    plot_params_to_remove <- c(vectorized_params, "xlab", "ylab", "main", "xlim", "ylim", "font.main", "cex.main")
    plot_params[plot_params_to_remove] <- NULL
    
    # Track if user provided yaxt - if not, we'll draw custom axis with formatted labels
    user_provided_yaxt <- "yaxt" %in% names(plot_params)
    
    plot_args <- list(x = first_density, 
                      col = col1, lwd = lwd1, lty = lty1, type = type1,
                      xlab = xlab_title, 
                      ylab = ylab_title,
                      main = main_title,
                      font.main = font_main,
                      cex.main = cex_main,
                      xlim = default_xlim,
                      ylim = default_ylim,
                      yaxs = "i",  # Prevent padding below 0
                      font.lab = 2, cex.lab = 1.2, las = 1)
    if (!is.null(pch1)) plot_args$pch <- pch1
    if (!user_provided_yaxt) plot_args$yaxt <- "n"  # Suppress default y-axis
    
    #16. Set up plot
      # Ensure adequate top margin for main title and (n=...) text (like plot_freq)
      old_mar <- par("mar")
      on.exit(par(mar = old_mar), add = TRUE)
      if (!"mar" %in% names(plot_params)) {
        # Increase top margin if it's too small (less than 5 lines to accommodate title and N)
        if (old_mar[3] < 5) {
          par(mar = c(old_mar[1], old_mar[2], 5, old_mar[4]))
        }
      }
      
      # Set up plot
      do.call(plot, c(plot_args, plot_params))
      
      # Add (n=...) below main title (like plot_cdf notation)
      tot <- length(y)
      mtext(paste0("(n=", tot, ")"), side = 3, line = 0.75, font = 3, cex = 0.9)
      
      # Draw custom y-axis with formatted labels (skip leading zeros)
      if (!user_provided_yaxt) {
        # Generate nice tick marks
        y_ticks <- pretty(default_ylim, n = 5)
        # Only keep ticks that are >= 0 and <= y_max (with small tolerance for rounding)
        y_ticks <- y_ticks[y_ticks >= 0 & y_ticks <= default_ylim[2] + 0.1]
        
        # Format labels to skip leading zeros (e.g., 0.014 becomes .014)
        format_label <- function(x) {
          if (x == 0) {
            return("0")
          } else {
            # Convert to character and remove leading zero
            label <- as.character(x)
            # If it starts with "0.", replace with "."
            label <- sub("^0\\.", ".", label)
            return(label)
          }
        }
        y_labels <- sapply(y_ticks, format_label)
        
        axis(2, at = y_ticks, labels = y_labels, las = 1)
      }
    
    #17. Add remaining densities
      # Add remaining densities (only if there's grouping)
      if (!is.null(group) && length(density_list) > 1) {
        for (i in 2:length(density_list)) {
          density_obj <- density_list[[i]]
          
          # Get parameters for this group
          coli <- get_param("col", i) %||% default_colors[i]
          lwdi <- get_param("lwd", i) %||% 4  # Default lwd=4
          ltyi <- get_param("lty", i) %||% 1
          typei <- get_param("type", i) %||% "l"
          pchi <- get_param("pch", i)
          
          # Build lines arguments
          lines_args <- list(x = density_obj, 
                            col = coli, lwd = lwdi, lty = ltyi, type = typei)
          if (!is.null(pchi)) lines_args$pch <- pchi
          
          do.call(lines, lines_args)
        }
      }
    
    #18. Add points at y=0 and x=mean
      # Add points at y=0 and x=mean for each group (only if show_means is TRUE and there's grouping)
      if (show_means && !is.null(group)) {
        for (i in seq_along(density_list)) {
          group_val <- unique_x[i]
          y_group <- y[group == group_val]
          group_mean <- mean(y_group, na.rm = TRUE)
          
          # Get color for this group
          coli <- get_param("col", i) %||% default_colors[i]
          
          # Add point at (mean, 0)
          points(x = group_mean, y = 0, pch = 16, col = coli)
        }
      } else if (show_means && is.null(group)) {
        # No grouping: add single point at mean
        group_mean <- mean(y, na.rm = TRUE)
        coli <- get_param("col", 1) %||% default_colors[1]
        points(x = group_mean, y = 0, pch = 16, col = coli)
      }
    
    #19. Report means and t-test
      
      # Add vertical segments at mean values for non-grouped plots
        if (show_means && show_mean_segments && is.null(group)) {
          group_mean <- mean(y, na.rm = TRUE)
          coli <- get_param("col", 1) %||% default_colors[1]
          y_max_segment <- y_max_density * 1.1
          
          # Add vertical segment
          segments(x0 = group_mean, y0 = 0, x1 = group_mean, y1 = y_max_segment,
                   col = coli, lwd = 2)
          # Add M= label at top of segment
          text(x = group_mean, y = y_max_segment, 
               labels = paste0("M=", round(group_mean, 2)),
               pos = 3, col = coli, cex = 0.8, font = 2)
        }
      
      # Add vertical segments at mean values (only if show_means and show.means are TRUE and there's grouping)
        if (show_means && show_mean_segments && !is.null(group) && n_groups >= 2 && n_groups <= 3) {
          # Calculate all means
          all_means <- numeric(n_groups)
          for (i in seq_along(density_list)) {
            group_val <- unique_x[i]
            y_group <- y[group == group_val]
            all_means[i] <- mean(y_group, na.rm = TRUE)
          }
        
        # Sort means to determine low/mid/high
          sorted_indices <- order(all_means)
          sorted_means <- all_means[sorted_indices]
        
        # Get maximum density value for segment height (1.1 times highest density)
          y_max_segment <- y_max_density * 1.1
          
        # Add segments and labels based on number of groups
          if (n_groups == 2) {
            # Low mean: pos=2 (left), High mean: pos=4 (right)
            low_idx <- sorted_indices[1]
            high_idx <- sorted_indices[2]
            
          # Low mean segment
            coli_low <- get_param("col", low_idx) %||% default_colors[low_idx]
            segments(x0 = sorted_means[1], y0 = 0, x1 = sorted_means[1], y1 = y_max_segment,
                     col = coli_low, lwd = 2)
            text(x = sorted_means[1], y = y_max_segment, 
                 labels = paste0("M=", round(sorted_means[1], 2)),
                 pos = 2, col = coli_low, cex = 0.8, font = 2)
          
          # High mean segment
            coli_high <- get_param("col", high_idx) %||% default_colors[high_idx]
            segments(x0 = sorted_means[2], y0 = 0, x1 = sorted_means[2], y1 = y_max_segment,
                     col = coli_high, lwd = 2,lty=2)
            
            text(x = sorted_means[2], y = y_max_segment, 
                 labels = paste0("M=", round(sorted_means[2], 2)),
                 pos = 4, col = coli_high, cex = 0.8, font = 2)
            
        } else if (n_groups == 3) {
          # Show all three means: Low (pos=2), Mid (pos=3), High (pos=4)
          low_idx <- sorted_indices[1]
          mid_idx <- sorted_indices[2]
          high_idx <- sorted_indices[3]
          
          # Low mean segment
            coli_low <- get_param("col", low_idx) %||% default_colors[low_idx]
            segments(x0 = sorted_means[1], y0 = 0, x1 = sorted_means[1], y1 = y_max_segment,
                     col = coli_low, lwd = 2)
            text(x = sorted_means[1], y = y_max_segment, 
                 labels = paste0("M=", round(sorted_means[1], 2)),
                 pos = 2, col = coli_low, cex = 0.8, font = 2)
            
          # Mid mean segment
            coli_mid <- get_param("col", mid_idx) %||% default_colors[mid_idx]
            segments(x0 = sorted_means[2], y0 = 0, x1 = sorted_means[2], y1 = y_max_segment,
                     col = coli_mid, lwd = 2)
            text(x = sorted_means[2], y = y_max_segment, 
                 labels = paste0("M=", round(sorted_means[2], 2)),
                 pos = 3, col = coli_mid, cex = 0.8, font = 2)
            
          # High mean segment
            coli_high <- get_param("col", high_idx) %||% default_colors[high_idx]
            segments(x0 = sorted_means[3], y0 = 0, x1 = sorted_means[3], y1 = y_max_segment,
                     col = coli_high, lwd = 2, lty = 2)
            text(x = sorted_means[3], y = y_max_segment, 
                 labels = paste0("M=", round(sorted_means[3], 2)),
                 pos = 4, col = coli_high, cex = 0.8, font = 2)
          
          }
        }
    
    #20. Add legend (only if there's grouping, like plot_freq)
      if (!is.null(group)) {
        # Add legend on top with title showing x variable name
        # Calculate means and sample sizes for each group
        legend_cols <- sapply(1:length(density_list), function(i) get_param("col", i) %||% default_colors[i])
        legend_lwds <- sapply(1:length(density_list), function(i) get_param("lwd", i) %||% 4)
        legend_ltys <- sapply(1:length(density_list), function(i) get_param("lty", i) %||% 1)
        
        # Create legend labels with sample sizes (newline before sample size, lowercase n)
        group_ns <- sapply(1:length(density_list), function(i) {
          length(y[group == unique_x[i]])
        })
        
        # Format: just the group value, then (n=sample_size) on new line
        legend_labels <- paste0(as.character(unique_x), "\n(n=", group_ns, ")")
        
        legend("top", legend = legend_labels, 
               col = legend_cols, lwd = legend_lwds, lty = legend_ltys,
               horiz = TRUE, bty = "n")
      }
  }
  
  #21. Return densities
  # Return densities
    if (!is.null(group)) {
      names(density_list) <- as.character(unique_x)
    } else {
      names(density_list) <- "all"
    }
  
  # Build return list
    result <- list(densities = density_list)
  
  invisible(result)
}

#plot_density_001
test_that("plot_density runs without errors", {
  y <- rnorm(100)
  group <- rep(c("A", "B", "C"), c(30, 40, 30))
  
  # Should not throw errors
  expect_error(plot_density(y ~ group), NA)
  
  # Should return invisible list with densities
  result <- plot_density(y ~ group)
  expect_true(is.list(result))
  expect_true("densities" %in% names(result))
  expect_equal(length(result$densities), 3)
  expect_true(all(sapply(result$densities, inherits, "density")))
})

#plot_density_002
test_that("plot_density handles data frame input", {
  df <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
  
  # Use formula syntax
  expect_error(plot_density(value ~ group, data = df), NA)
  
  # Check return value
  result <- plot_density(value ~ group, data = df)
  expect_equal(length(result$densities), 2)
})

#plot_density_003
test_that("plot_density accepts df$col ~ df$col2 without bare column names in env", {
  df1 <- data.frame(
    y1 = rnorm(60),
    x1 = rep(c("A", "B", "C"), each = 20)
  )
  expect_error(plot_density(df1$y1 ~ df1$x1), NA)
  result <- plot_density(df1$y1 ~ df1$x1)
  expect_equal(length(result$densities), 3)
})

#plot_density_004
test_that("plot_density handles missing values", {
  y <- c(rnorm(90), rep(NA, 10))
  group <- rep(c("A", "B"), 50)
  
  # Should handle NAs gracefully
  expect_error(plot_density(y ~ group), NA)
  
  result <- plot_density(y ~ group)
  expect_equal(length(result$densities), 2)
})

#plot_density_005
test_that("plot_density handles custom parameters", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  # Custom colors
  expect_error(plot_density(y ~ group, col = c("red", "blue")), NA)
  
  # Custom line width
  expect_error(plot_density(y ~ group, lwd = 2), NA)
  
  # Custom line type
  expect_error(plot_density(y ~ group, lty = c(1, 2)), NA)
  
  # Multiple parameters
  expect_error(plot_density(y ~ group, col = c("red", "blue"), lwd = c(1, 2)), NA)
})

#plot_density_006
test_that("plot_density handles show.means parameter", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  expect_error(plot_density(y ~ group, show.means = TRUE), NA)
  expect_error(plot_density(y ~ group, show.means = FALSE), NA)
})

#plot_density_007
test_that("plot_density handles different numbers of groups", {
  y <- rnorm(100)
  
  # Two groups
  group2 <- rep(c("A", "B"), 50)
  expect_error(plot_density(y ~ group2), NA)
  
  # Three groups
  group3 <- rep(c("A", "B", "C"), c(30, 40, 30))
  expect_error(plot_density(y ~ group3), NA)
  
  # Four groups
  group4 <- rep(c("A", "B", "C", "D"), c(25, 25, 25, 25))
  expect_error(plot_density(y ~ group4), NA)
})

#plot_density_008
test_that("plot_density handles density() arguments", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  # Custom bandwidth
  expect_error(plot_density(y ~ group, bw = 0.5), NA)
  
  # Custom kernel
  expect_error(plot_density(y ~ group, kernel = "rectangular"), NA)
})

#plot_density_009
test_that("plot_density returns correct structure", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  result <- plot_density(y ~ group)
  
  # Check structure
  expect_true(is.list(result))
  expect_true("densities" %in% names(result))
  expect_true(is.list(result$densities))
  
  # Check density objects
  expect_true(all(sapply(result$densities, inherits, "density")))
  expect_equal(names(result$densities), c("A", "B"))
})

#plot_density_010
test_that("plot_density error message shows correct dataset name", {
  # Create a dataset with a specific name
  IV5 <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
  
  # Try to access a variable that doesn't exist - should show "IV5" not "data"
  expect_error(
    plot_density(nonexistent ~ group, data = IV5),
    'Variable "nonexistent" not found in dataset "IV5"',
    fixed = TRUE
  )
})

# ============================================================================
# EDGE CASES
# ============================================================================

#plot_density_011
test_that("plot_density handles single group", {
  y <- rnorm(50)
  group <- rep("Only", 50)
  
  result <- plot_density(y ~ group)
  
  expect_true(is.list(result))
  expect_equal(length(result$densities), 1)
})

#plot_density_012
test_that("plot_density handles many groups", {
  y <- rnorm(200)
  group <- rep(LETTERS[1:10], each = 20)  # 10 groups
  
  result <- plot_density(y ~ group)
  
  expect_true(is.list(result))
  expect_equal(length(result$densities), 10)
})

#plot_density_013
test_that("plot_density handles order parameter for groups", {
  y <- rnorm(100)
  group <- rep(c("A", "B", "C"), c(30, 40, 30))
  
  # Custom group order
  expect_error(plot_density(y ~ group, order = c("C", "A", "B")), NA)
  result <- plot_density(y ~ group, order = c("C", "A", "B"))
  expect_true(is.list(result))
  
  # Check that densities are in specified order
  expect_equal(names(result$densities), c("C", "A", "B"))
})

#plot_density_014
test_that("plot_density order = -1 reverses default order", {
  y <- rnorm(100)
  group <- rep(c("A", "B", "C"), c(30, 40, 30))
  
  # Default order is A, B, C (sorted)
  result_default <- plot_density(y ~ group)
  expect_equal(names(result_default$densities), c("A", "B", "C"))
  
  # order = -1 should reverse to C, B, A
  result_reversed <- plot_density(y ~ group, order = -1)
  expect_true(is.list(result_reversed))
  expect_equal(names(result_reversed$densities), c("C", "B", "A"))
})

#plot_density_015
test_that("plot_density respects factor levels for groups when order is NULL", {
  y <- rnorm(100)
  group <- factor(rep(c("A", "B", "C"), c(30, 40, 30)),
                  levels = c("C", "A", "B"))
  
  result <- plot_density(y ~ group)
  expect_true(is.list(result))
  
  # Check that factor levels are respected (C, A, B)
  expect_equal(names(result$densities), c("C", "A", "B"))
})

#plot_density_016
test_that("plot_density handles two-vector comparison", {
  y1 <- rnorm(50)
  y2 <- rnorm(50)
  
  # Should not throw errors
  expect_error(plot_density(y1, y2), NA)
  
  result <- plot_density(y1, y2)
  expect_true(is.list(result))
  expect_equal(length(result$densities), 2)
})

#plot_density_017
test_that("plot_density two-vector with custom parameters", {
  y1 <- rnorm(50)
  y2 <- rnorm(50)
  
  # Custom colors
  expect_error(plot_density(y1, y2, col = c("red", "blue")), NA)
  
  # Show means
  expect_error(plot_density(y1, y2, show_means = FALSE), NA)
})

#plot_density_018
test_that("plot_density two-vector handles order parameter", {
  y1 <- rnorm(50)
  y2 <- rnorm(50)
  
  result <- plot_density(y1, y2, order = -1)
  expect_equal(names(result$densities), c("y2", "y1"))
})

#plot_density_019
test_that("plot_density two-vector with different lengths", {
  y1 <- rnorm(50)
  y2 <- rnorm(30)  # Different length
  
  # Should handle different lengths
  expect_error(plot_density(y1, y2), NA)
  
  result <- plot_density(y1, y2)
  expect_equal(length(result$densities), 2)
})

#plot_density_020
test_that("plot_density reserves space for legend with groups", {
  y <- rnorm(100)
  group <- rep(c("A", "B"), 50)
  
  # Should reserve space for legend
  expect_error(plot_density(y ~ group), NA)
  
  # Test with two-vector
  y1 <- rnorm(50)
  y2 <- rnorm(50)
  expect_error(plot_density(y1, y2), NA)
})

# Variable labels (exported)
#
# 1. Get/set variable labels
#' Get or set variable labels
#'
#' Assign labels to describe variables in a data frame.
#' Can be called on a single variable or full data, to assign or to read existing
#' labels.
#'
#' For a vector, this reads/writes a base-R \code{"label"} attribute.
#' For a data frame, this reads/writes the \code{"label"} attribute of each
#' column.
#'
#' @param x A vector or a data frame.
#' @param value Character label(s) to assign.
#'
#' @return
#' - If \code{x} is a data frame: a named character vector with one entry per
#'   column (missing labels are returned as \code{NA_character_}).
#' - Otherwise: a single character string (or \code{NA_character_}).
#'
#' @examples
#' df <- data.frame(x = 1:3, y = 4:6)
#'
#' # Set labels for all columns
#' var_labels(df) <- c("this is x", "this is y")
#' var_labels(df)
#'
#' # Set a label for a single column
#' var_labels(df$x) <- "this is x"
#' var_labels(df$x)
#'
#' @export
  var_labels <- function(x) {
    if (is.data.frame(x)) {
      out <- vapply(x, function(col) {
        lab <- attr(col, "label", exact = TRUE)
        if (is.null(lab)) NA_character_ else as.character(lab)[1]
      }, character(1))
      return(stats::setNames(out, names(x)))
    }
    
    lab <- attr(x, "label", exact = TRUE)
    if (is.null(lab)) {
      return(NA_character_)
    }
    as.character(lab)[1]
  }

#' @rdname var_labels
#' @export
  `var_labels<-` <- function(x, value) {
    # 1. Validate value
      if (is.list(value)) {
        value <- unlist(value, recursive = FALSE, use.names = TRUE)
      }
      if (!is.atomic(value) || (!is.character(value) && !is.factor(value))) {
        stop("var_labels<-(): 'value' must be a character vector", call. = FALSE)
      }
      
      value_names <- names(value)
      value <- as.character(value)
      names(value) <- value_names
    
    # 2. Data frame case
      if (is.data.frame(x)) {
        n <- ncol(x)
        if (!n) {
          return(x)
        }
        
        if (!is.null(names(value)) && any(nzchar(names(value)))) {
          # 2a. Named assignment: match by name (ignore unknown names)
            common <- intersect(names(x), names(value))
            for (nm in common) {
              v <- value[[nm]]
              if (is.na(v) || identical(v, "")) {
                attr(x[[nm]], "label") <- NULL
              } else {
                attr(x[[nm]], "label") <- v
              }
            }
            return(x)
        }
        
        # 2b. Positional assignment: names()-like length rules
          if (length(value) == 1L && n > 1L) {
            value <- rep(value, n)
            names(value) <- NULL
          } else if (length(value) != n) {
            stop(sprintf("var_labels<-(): 'value' must have length 1 or %d", n), call. = FALSE)
          }
          
          for (i in seq_len(n)) {
            v <- value[[i]]
            if (is.na(v) || identical(v, "")) {
              attr(x[[i]], "label") <- NULL
            } else {
              attr(x[[i]], "label") <- v
            }
          }
          return(x)
      }
    
    # 3. Vector-like case
      v <- value[[1]]
      if (is.na(v) || identical(v, "")) {
        attr(x, "label") <- NULL
      } else {
        attr(x, "label") <- v
      }
      x
  }

#var_labels_001
test_that("var_labels gets/sets label for vectors", {
  x <- 1:3
  expect_true(is.na(var_labels(x)))
  
  var_labels(x) <- "this is x"
  expect_equal(var_labels(x), "this is x")
  
  var_labels(x) <- NA_character_
  expect_true(is.na(var_labels(x)))
})

#var_labels_002
test_that("var_labels gets/sets labels for data frames (positional)", {
  df <- data.frame(x = 1:3, y = 4:6)
  
  expect_equal(var_labels(df), c(x = NA_character_, y = NA_character_))
  
  var_labels(df) <- c("this is x", "this is y")
  expect_equal(var_labels(df), c(x = "this is x", y = "this is y"))
})

#var_labels_003
test_that("var_labels supports named assignment for data frames", {
  df <- data.frame(x = 1:3, y = 4:6)
  
  var_labels(df) <- c(x = "this is x")
  expect_equal(var_labels(df), c(x = "this is x", y = NA_character_))
})

#var_labels_004
test_that("var_labels recycles length-1 assignment for data frames", {
  df <- data.frame(x = 1:3, y = 4:6, z = 7:9)
  
  var_labels(df) <- "same"
  expect_equal(var_labels(df), c(x = "same", y = "same", z = "same"))
})

#' Enhanced alternative to list()
#' 
#' List with objects that are automatically named. 
#' 
#' list2(x , y)      is equivalent to list(x = x , y = y)
#' 
#' list2(x , y2 = y) is equivalent to list(x = x , y2 = y)
#' 
#' Based on: \url{https://stackoverflow.com/questions/16951080/can-lists-be-created-that-name-themselves-based-on-input-object-names}
#'
#' @param ... Objects to include in the list. Objects are automatically named 
#'   based on their variable names unless explicit names are provided.
#'
#' @return A named list. Each element is named after the variable passed to 
#'   the function (or the explicit name if provided). The structure is identical 
#'   to a standard R list created with \code{\link[base]{list}}.
#'
#' @examples
#' x <- 1:5
#' y <- letters[1:3]
#' z <- matrix(1:4, nrow = 2)
#'
#' # Create named list from objects
#' my_list <- list2(x, y, z)
#' names(my_list)  # "x" "y" "z"
#'
#' # Works with explicit names too
#' my_list2 <- list2(a = x, b = y)
#' names(my_list2)  # "a" "b"
#'
#' @export
list2 <- function(...) {
  L <- list(...)
  snm <- sapply(substitute(list(...)), deparse)[-1]
  
  if (is.null(nm <- names(L))) nm <- snm
  
  if (any(nonames <- nm == "")) nm[nonames] <- snm[nonames]
  
  setNames(L, nm)
}

#list2_001
test_that("list2 creates list with object names", {
  x <- 1:5
  y <- letters[1:3]
  z <- matrix(1:4, nrow = 2)
  
  result <- list2(x, y, z)
  
  expect_true(is.list(result))
  expect_equal(length(result), 3)
  expect_equal(names(result), c("x", "y", "z"))
  expect_equal(result$x, x)
  expect_equal(result$y, y)
  expect_equal(result$z, z)
})

#list2_002
test_that("list2 works with explicit names", {
  x <- 1:5
  y <- letters[1:3]
  
  result <- list2(a = x, b = y)
  
  expect_equal(names(result), c("a", "b"))
  expect_equal(result$a, x)
  expect_equal(result$b, y)
})

#list2_003
test_that("list2 works with mixed named and unnamed", {
  x <- 1:5
  y <- letters[1:3]
  z <- matrix(1:4, nrow = 2)
  
  result <- list2(a = x, y, z)
  
  expect_equal(names(result), c("a", "y", "z"))
  expect_equal(result$a, x)
  expect_equal(result$y, y)
  expect_equal(result$z, z)
})

#list2_004
test_that("list2 handles single object", {
  x <- 1:5
  
  result <- list2(x)
  
  expect_equal(length(result), 1)
  expect_equal(names(result), "x")
  expect_equal(result$x, x)
})

#list2_005
test_that("list2 handles empty call", {
  result <- list2()
  
  expect_true(is.list(result))
  expect_equal(length(result), 0)
})

#list2_006
test_that("list2 preserves object values", {
  x <- c(1, 2, 3)
  y <- list(a = 1, b = 2)
  z <- data.frame(col1 = 1:3, col2 = 4:6)
  
  result <- list2(x, y, z)
  
  expect_equal(result$x, x)
  expect_equal(result$y, y)
  expect_equal(result$z, z)
})

#list2_007
test_that("list2 works with complex expressions", {
  x <- 1:5
  
  # Should work even with expressions
  result <- list2(x, x + 1)
  
  expect_true(is.list(result))
  expect_equal(length(result), 2)
  # Second element should be x + 1
  expect_equal(result[[2]], x + 1)
})

#' Enhanced alternative to table() 
#'
#'  The function \code{\link[base]{table}} does not show variable 
#'  names when tabulating from a dataframe, requires running another
#'  function, \code{\link[base]{prop.table}},  to tabulate proportions 
#'  and yet another function, \code{\link[stats]{chisq.test}} to test difference of 
#'  proportions.  \code{table2} does what those three functions do, producing easier to 
#'  read output, and always shows variable names. 
#' @param ... same arguments as \code{\link[base]{table}}, plus the arguments shown below
#' @param prop report a table with:
#'   \itemize{
#'     \item \code{prop="all"}: Proportions for full table (each cell / total)
#'     \item \code{prop="row"}: Proportions by row  ('rows' also accepted)
#'     \item \code{prop="col"}: Proportions by columns ('cols', 'column', 'columns' also accepted)
#'   }
#' @param digits Number of decimal values to show for proportions 
#' @param chi Logical. If \code{TRUE}, performs a chi-square test on frequency table,
#' reports results in APA format
#' @param correct Logical. If \code{TRUE}, applies Yates' continuity correction 
#' for 2x2 tables in the chi-square test. Default is \code{FALSE} (no correction).
#' 
#'
#' @return A list (object of class "table2") with the following components:
#'   \itemize{
#'     \item \code{freq}: frequency table 
#'     \item \code{prop}: proportions table 
#'     \item \code{chisq}: chi-square test  
#'   }
#'
#' @examples
#' # Create example data
#' df <- data.frame(
#'   group = c("A", "A", "B", "B", "A"),
#'   status = c("X", "Y", "X", "Y", "X")
#' )
#'
#' # Enhanced table with variable names (2 variables)
#' table2(df$group, df$status)
#'
#' # Enhanced table with variable names (3 variables)
#' df3 <- data.frame(
#'   x = c("A", "A", "B", "B"),
#'   y = c("X", "Y", "X", "Y"),
#'   z = c("high", "low", "high", "low")
#' )
#' table2(df3$x, df3$y, df3$z)
#'
#' # Table with proportions
#' table2(df$group, df$status, prop = 'all')  # Overall proportions
#' table2(df$group, df$status, prop = 'row')  # Row proportions
#' table2(df$group, df$status, prop = 'col')  # Column proportions
#'
#' # Table with chi-square test
#' table2(df$group, df$status, chi = TRUE,prop='all')
#'
#' @usage NULL
#' @export
table2 <- function(..., data = NULL, exclude = if (useNA == "no") c(NA, NaN), 
                  useNA = c("no", "ifany", "always"), 
                  dnn = NULL, deparse.level = 1, prop = NULL, digits = 3, 
                  chi = FALSE, correct = FALSE) {
  
  # FUNCTION OUTLINE:
  # 1. Validate and process useNA and exclude arguments
  # 2. Capture function call expressions to extract variable names
  # 3. Call base::table() to create the contingency table
  # 4. Extract variable names from dataframe column references (df$var, df[["var"]], etc.)
  # 5. FORMAT HEADERS: Set variable names as dimension names (appear in margins when printed)
  # 6. FORMAT HEADERS: Keep row/column labels as values only (not "var=value")
  # 7. Handle proportion calculations if prop argument is provided
  # 8. Add marginal totals for proportion tables (Total rows/columns)
  # 9. Return the enhanced table object
  
  # TASK 1: Validate and process useNA and exclude arguments
  # Match useNA argument to handle default properly
  useNA <- match.arg(useNA)
  
  # Set exclude default based on useNA
  if (missing(exclude)) {
    exclude <- if (useNA == "no") c(NA, NaN) else NULL
  }
  
  # TASK 2: Validate inputs and handle data argument
  validated <- validate_table2(..., data = data, func_name = "table2")
  dots <- validated$dots
  dot_expressions <- validated$dot_expressions
  data_name <- validated$data_name
  
  # Track missing values for each variable
  var_lengths <- integer(length(dots))
  var_na_counts <- integer(length(dots))
  for (i in seq_along(dots)) {
    var_lengths[i] <- length(dots[[i]])
    var_na_counts[i] <- sum(is.na(dots[[i]]))
  }
  
  # Initialize return list elements
  freq <- NULL
  prop_out <- NULL
  chisq <- NULL
  
  # TASK 3: Call base table function
  # Let base::table handle dnn default (it uses list.names internally)
  if (is.null(dnn)) {
    result <- do.call(base::table, c(dots, list(exclude = exclude, useNA = useNA, 
                          deparse.level = deparse.level)))
  } else {
    result <- do.call(base::table, c(dots, list(exclude = exclude, useNA = useNA, 
                         dnn = dnn, deparse.level = deparse.level)))
  }
  
  # Store frequency table
  freq <- result
  
  # Helper function to extract variable name from an expression
  extract_var_name <- function(expr) {
    var_name <- ""
    # Check if it's a symbol (variable name) - works with or without data argument
    if (is.symbol(expr) || is.name(expr)) {
      var_name <- as.character(expr)
      return(var_name)
    }
    
    # Check if it's a dataframe column reference: df$var
    if (is.call(expr) && length(expr) >= 3) {
      op <- expr[[1]]
      
      # Handle comparison operators: show full expression like "var>10"
      # For expressions like df$var > 10, the operator is >, not $
      if (identical(op, quote(`>`)) || identical(op, quote(`<`)) || 
          identical(op, quote(`>=`)) || identical(op, quote(`<=`)) || 
          identical(op, quote(`==`)) || identical(op, quote(`!=`))) {
        # Extract variable name from left-hand side
        left_var <- extract_var_name(expr[[2]])
        # Get the operator as string
        op_str <- as.character(op)
        # Get the right-hand side value
        right_val <- deparse(expr[[3]])
        # Construct full expression like "var>10"
        full_expr <- paste0(left_var, op_str, right_val)
        # Truncate if longer than 30 characters
        if (nchar(full_expr) > 30) {
          full_expr <- paste0(substr(full_expr, 1, 27), "...")
        }
        return(full_expr)
      }
      
      # Handle df$var
      if (identical(op, quote(`$`)) || identical(op, as.name("$"))) {
        var_name <- as.character(expr[[3]])
      }
      # Handle df[["var"]] or df[, "var"] or df[, i]
      else if (identical(op, quote(`[`)) || identical(op, as.name("["))) {
        if (length(expr) >= 3) {
          col_expr <- expr[[3]]
          # Handle df[["var"]] - double bracket with character
          if (is.character(col_expr) && length(col_expr) == 1) {
            var_name <- col_expr
          }
          # Handle df[, "var"] - single bracket with character column name
          else if (is.call(col_expr) && identical(col_expr[[1]], quote(`[`)) && 
                   length(col_expr) >= 2 && is.character(col_expr[[2]])) {
            var_name <- col_expr[[2]]
          }
          # Handle df[, i] where i is a name or number
          else if (is.name(col_expr)) {
            # Try to evaluate to see if it's a character
            tryCatch({
              val <- eval(col_expr, parent.frame())
              if (is.character(val) && length(val) == 1) {
                var_name <- val
              }
            }, error = function(e) {})
          }
        }
      }
    }
    return(var_name)
  }
  
  # TASK 4: Extract variable names from dataframe column references
  # Initialize var_names
  var_names <- character(length(dots))
  
  # Enhance if we have 1, 2 or 3 dimensions matching the number of arguments
  n_dims <- length(dim(result))
  if ((n_dims == 1 && length(dots) == 1) || 
      (n_dims == 2 && length(dots) == 2) || 
      (n_dims == 3 && length(dots) == 3)) {
    
    # Try to extract variable names from expressions
    for (i in 1:n_dims) {
      expr <- dot_expressions[[i]]
      var_names[i] <- extract_var_name(expr)
    }
    
    # TASK 5 & 6: FORMAT HEADERS - Set variable names and labels
    # If we found variable names, enhance the dimnames
    if (any(nchar(var_names) > 0)) {
      dimn <- dimnames(result)
      
      # TASK 5: FORMAT HEADERS - Set variable names as dimension names
      # These appear in margins when printed (column var on top, row var on left)
      # For 3D tables, the third dimension name appears in the slice headers
      for (i in 1:n_dims) {
        if (nchar(var_names[i]) > 0) {
          names(dimn)[i] <- var_names[i]
        }
      }
      
      # TASK 6: FORMAT HEADERS - Keep row and column labels as just the values
      # Row labels: dimn[[1]] contains just values (e.g., "A", "B", "C")
      # Column labels: dimn[[2]] contains just values (e.g., "1", "2", "3")
      # Third dimension labels: dimn[[3]] contains just values (e.g., "high", "low")
      # The variable names will appear in the margins via names(dimnames)
      dimnames(result) <- dimn
      
      # Update freq to have the enhanced dimnames
      freq <- result
    }
  }
  
  # TASK 7: Perform chi-square test if requested
  # Initialize chi_test_attr to NULL
  chi_test_attr <- NULL
  
  if (isTRUE(chi)) {
    # Chi-square test can be performed on 1D or 2D frequency tables
    # For 1D: tests for given probabilities (goodness of fit)
    # For 2D: tests for independence
    n_dims_chi <- length(dim(result))
    if (n_dims_chi == 1 || n_dims_chi == 2) {
      # Perform chi-square test on the frequency table
      # Capture warning about small expected counts
      chi_warning <- NULL
      chi_test <- tryCatch({
        withCallingHandlers(
          stats::chisq.test(result, correct = correct),
          warning = function(w) {
            if (grepl("Chi-squared approximation may be incorrect", w$message)) {
              chi_warning <<- TRUE
              invokeRestart("muffleWarning")
            }
          }
        )
      }, error = function(e) {
        # If chi-square test fails (e.g., all zeros, insufficient data), return NULL
        NULL
      })
      # Store chi-square test result and warning flag
      attr(result, "chi_test") <- chi_test
      attr(chi_test, "low_expected") <- isTRUE(chi_warning)
      chi_test_attr <- chi_test
      chisq <- chi_test
    } else {
      # For 3D tables, chi-square test could be performed on each slice
      # For now, we'll skip it or perform on the first slice
      # Store NULL to indicate chi-square test not performed
      attr(result, "chi_test") <- NULL
      chi_test_attr <- NULL
    }
  }
  
  # TASK 8: Convert to proportions if requested
  # Initialize orig_freq to NULL (will be set if prop is requested)
  orig_freq <- NULL
  
  if (!is.null(prop)) {
    # Mark this as a proportion table
    attr(result, "is_proportion") <- TRUE
    attr(result, "proportion_digits") <- digits
    # Convert character values to numeric
    if (is.character(prop)) {
      prop_lower <- tolower(prop)
      if (prop_lower == "all") {
        prop <- 0
      } else if (prop_lower %in% c("row", "rows")) {
        prop <- 1
      } else if (prop_lower %in% c("col", "cols", "column", "columns")) {
        prop <- 2
      } else {
        stop("table2(): prop must be 0, 1, 2, 'all', 'row'/'rows', or 'col'/'cols'/'column'/'columns'", call. = FALSE)
      }
    }
    
    if (!prop %in% c(0, 1, 2)) {
      stop("table2(): prop must be 0, 1, 2, 'all', 'row', or 'column'", call. = FALSE)
    }
    
    # Get original dimnames before modification
    orig_dimn <- dimnames(result)
    n_dims_orig <- length(dim(result))
    
    # Save original frequency table before any modifications (for all prop types)
    orig_freq <- result
    
    # Track if original was 1D
    was_1d <- (n_dims_orig == 1)
    
    # Handle 1D tables: keep as 1D, just calculate proportions
    if (was_1d) {
      # For 1D tables, calculate proportions and return early
      total_sum <- sum(result, na.rm = TRUE)
      prop_result <- result / total_sum
      prop_result <- round(prop_result, digits = digits)
      
      # Keep the same dimnames as the frequency table
      dimnames(prop_result) <- dimnames(result)
      class(prop_result) <- "table"
      
      # Store proportion table
      prop_out <- prop_result
      
      # Build output list
      output <- list2(freq = freq, prop = prop_out, chisq = chisq)
      
      # Store proportion metadata
      attr(output, "prop_type") <- prop
      attr(output, "proportion_digits") <- digits
      var1_name <- if (!is.null(names(orig_dimn)) && length(names(orig_dimn)) >= 1 && 
                       !is.na(names(orig_dimn)[1]) && nchar(names(orig_dimn)[1]) > 0) {
        names(orig_dimn)[1]
      } else {
        ""
      }
      attr(output, "var1_name") <- var1_name
      attr(output, "var2_name") <- ""
      
      class(output) <- c("table2", class(output))
      return(output)
    }
    
    # Handle 3D tables: calculate proportions within each slice
    if (n_dims_orig == 3) {
      # For 3D tables, calculate proportions within each 2D slice
      prop_result <- result
      n_slices <- dim(result)[3]
      
      for (k in seq_len(n_slices)) {
        slice <- result[, , k]
        
        if (prop == 0) {
          # Overall proportions within slice
          slice_sum <- sum(slice, na.rm = TRUE)
          if (slice_sum > 0) {
            prop_result[, , k] <- slice / slice_sum
          }
        } else if (prop == 1) {
          # Row proportions within slice
          row_sums <- rowSums(slice, na.rm = TRUE)
          row_sums[row_sums == 0] <- 1
          prop_result[, , k] <- slice / row_sums
        } else if (prop == 2) {
          # Column proportions within slice
          col_sums <- colSums(slice, na.rm = TRUE)
          col_sums[col_sums == 0] <- 1
          prop_result[, , k] <- sweep(slice, 2, col_sums, "/")
        }
      }
      
      prop_result <- round(prop_result, digits = digits)
      
      # Keep the same dimnames as the frequency table
      dimnames(prop_result) <- dimnames(result)
      class(prop_result) <- "table"
      
      # Store proportion table
      prop_out <- prop_result
      
      # Get variable names
      var1_name <- if (!is.null(names(orig_dimn)) && length(names(orig_dimn)) >= 1 && 
                       !is.na(names(orig_dimn)[1]) && nchar(names(orig_dimn)[1]) > 0) {
        names(orig_dimn)[1]
      } else {
        ""
      }
      var2_name <- if (!is.null(names(orig_dimn)) && length(names(orig_dimn)) >= 2 && 
                       !is.na(names(orig_dimn)[2]) && nchar(names(orig_dimn)[2]) > 0) {
        names(orig_dimn)[2]
      } else {
        ""
      }
      
      # Build output list
      output <- list2(freq = freq, prop = prop_out, chisq = chisq)
      
      # Store proportion metadata
      attr(output, "prop_type") <- prop
      attr(output, "proportion_digits") <- digits
      attr(output, "var1_name") <- var1_name
      attr(output, "var2_name") <- var2_name
      
      class(output) <- c("table2", class(output))
      return(output)
    }
    
    # Get variable names from dimnames for cat messages (for 2D tables)
    var1_name <- if (length(dim(result)) == 2 && !is.null(names(orig_dimn)) && length(names(orig_dimn)) >= 1 && !is.na(names(orig_dimn)[1]) && nchar(names(orig_dimn)[1]) > 0) {
      names(orig_dimn)[1]
    } else {
      ""
    }
    var2_name <- if (length(dim(result)) == 2 && !is.null(names(orig_dimn)) && length(names(orig_dimn)) >= 2 && !is.na(names(orig_dimn)[2]) && nchar(names(orig_dimn)[2]) > 0) {
      names(orig_dimn)[2]
    } else {
      ""
    }
    
    if (prop == 0) {
      # Overall proportions: divide by sum of all cells
      total_sum <- sum(result, na.rm = TRUE)
      
      result <- result / total_sum
      # Round to specified number of digits
      result <- round(result, digits = digits)
      prop_type <- "overall proportions"
      
      # Add row and column with marginal proportions, 100% only in bottom right
      # Skip margins for 1D tables (they would just duplicate the single column)
      if (length(dim(result)) == 2 && !was_1d) {
        n_rows <- nrow(result)
        n_cols <- ncol(result)
        dimn <- dimnames(result)
        # Preserve names of dimnames
        dimn_names <- names(dimn)
        
        # Calculate column totals from original frequency table, then convert to proportions
        col_totals_freq <- colSums(orig_freq, na.rm = TRUE)
        col_totals <- col_totals_freq / total_sum
        col_totals <- round(col_totals, digits = digits)
        
        # TASK 8: Add summary row with column marginal proportions
        summary_row <- matrix(col_totals, nrow = 1, ncol = n_cols)
        result <- rbind(result, summary_row)
        dimn[[1]] <- c(dimn[[1]], "Total")  # Add "Total" to row labels
        
        # Calculate row totals from original frequency table, then convert to proportions
        row_totals_freq <- rowSums(orig_freq, na.rm = TRUE)
        row_totals <- row_totals_freq / total_sum
        row_totals <- round(row_totals, digits = digits)
        
        # TASK 8: Add summary column with row marginal proportions
        summary_col <- matrix(c(row_totals, round(1.0, digits = digits)), nrow = n_rows + 1, ncol = 1)
        # Bottom right corner is 1.0 - sum of all proportions
        result <- cbind(result, summary_col)
        dimn[[2]] <- c(dimn[[2]], "Total")  # Add "Total" to column labels
        # Restore names of dimnames
        names(dimn) <- dimn_names
        dimnames(result) <- dimn
      }
      
    } else if (prop == 1) {
      # Row proportions: each row sums to 1
      row_sums <- rowSums(result, na.rm = TRUE)
      # Avoid division by zero
      row_sums[row_sums == 0] <- 1
      result <- result / row_sums
      # Round to specified number of digits
      result <- round(result, digits = digits)
      prop_type <- "row proportions"
      
      # TASK 8: Add column with 1.0 for each row
      # Skip for 1D tables (would just duplicate the single column)
      if (length(dim(result)) == 2 && !was_1d) {
        n_rows <- nrow(result)
        dimn <- dimnames(result)
        # Preserve names of dimnames
        dimn_names <- names(dimn)
        summary_col <- matrix(round(1.0, digits = digits), nrow = n_rows, ncol = 1)  # 1.0 for each row
        result <- cbind(result, summary_col)
        dimn[[2]] <- c(dimn[[2]], "Total")  # Add "Total" to column labels
        # Restore names of dimnames
        names(dimn) <- dimn_names
        dimnames(result) <- dimn
      }
      
    } else if (prop == 2) {
      # Column proportions: each column sums to 1
      col_sums <- colSums(result, na.rm = TRUE)
      # Avoid division by zero
      col_sums[col_sums == 0] <- 1
      result <- sweep(result, 2, col_sums, "/")
      # Round to specified number of digits
      result <- round(result, digits = digits)
      prop_type <- "column proportions"
      
      # TASK 8: Add row with 1.0 for each column
      if (length(dim(result)) == 2) {
        n_cols <- ncol(result)
        dimn <- dimnames(result)
        # Preserve names of dimnames
        dimn_names <- names(dimn)
        summary_row <- matrix(round(1.0, digits = digits), nrow = 1, ncol = n_cols)  # 1.0 for each column
        result <- rbind(result, summary_row)
        dimn[[1]] <- c(dimn[[1]], "Total")  # Add "Total" to row labels
        # Restore names of dimnames
        names(dimn) <- dimn_names
        dimnames(result) <- dimn
      }
    }
    
    # Re-set attributes after all modifications (rbind/cbind may have created new objects)
    if (!is.null(prop)) {
      attr(result, "is_proportion") <- TRUE
      attr(result, "proportion_digits") <- digits
      attr(result, "original_frequency") <- orig_freq
      attr(result, "prop_type") <- prop
      attr(result, "var1_name") <- var1_name
      attr(result, "var2_name") <- var2_name
      # Restore chi_test attribute if it was saved before prop calculations
      if (!is.null(chi_test_attr)) {
        attr(result, "chi_test") <- chi_test_attr
      }
      # Store proportion table as plain table (no custom attributes)
      # Keep only dim, dimnames, class
      prop_out <- result
      attr(prop_out, "is_proportion") <- NULL
      attr(prop_out, "proportion_digits") <- NULL
      attr(prop_out, "original_frequency") <- NULL
      attr(prop_out, "prop_type") <- NULL
      attr(prop_out, "var1_name") <- NULL
      attr(prop_out, "var2_name") <- NULL
      attr(prop_out, "chi_test") <- NULL
      class(prop_out) <- "table"
    }
  }
  
  # TASK 9: Return as list with class table2
  output <- list2(freq = freq, prop = prop_out, chisq = chisq)
  
  # Store proportion metadata if prop was specified
  if (!is.null(prop)) {
    attr(output, "prop_type") <- prop
    attr(output, "proportion_digits") <- digits
    attr(output, "var1_name") <- var1_name
    attr(output, "var2_name") <- var2_name
  }
  
  # Store missing data information and variable names
  attr(output, "var_lengths") <- var_lengths
  attr(output, "var_na_counts") <- var_na_counts
  attr(output, "var_names") <- var_names
  
  class(output) <- c("table2", class(output))
  return(output)
}

#' Print method for table2 objects
#'
#' @param x An object of class \code{table2}
#' @param ... Additional arguments (ignored)
#'
#' @return Invisibly returns the original object
#' @export
print.table2 <- function(x, ...) {
  # Print the frequency table
  if (!is.null(x$freq)) {
    print(x$freq)
  }
  
  # Print proportion table if present
  if (!is.null(x$prop)) {
    cat("\n")
    prop_type <- attr(x, "prop_type")
    if (!is.null(prop_type)) {
      if (prop_type == "all" || prop_type == 0) {
        cat("Overall proportions:\n")
      } else if (prop_type == "row" || prop_type == 1 || prop_type == "rows") {
        cat("Row proportions:\n")
      } else if (prop_type == "col" || prop_type == 2 || 
                 prop_type == "cols" || prop_type == "column" || prop_type == "columns") {
        cat("Column proportions:\n")
      }
    }
    # Format and print proportion values with the specified number of digits
    digits <- attr(x, "proportion_digits")
    if (is.null(digits)) digits <- 3  # Default
    
    # Create a formatted version of the prop table with fixed decimal places
    prop_table <- x$prop
    
    # Convert to character matrix with fixed decimal places using formatC
    prop_values <- as.numeric(prop_table)
    prop_formatted <- formatC(prop_values, format = "f", digits = digits)
    
    # Restore dimensions and dimnames
    dim(prop_formatted) <- dim(prop_table)
    dimnames(prop_formatted) <- dimnames(prop_table)
    
    # Print as a formatted table
    print(noquote(prop_formatted))
  }
  
  # Print chi-square test if present
  if (!is.null(x$chisq)) {
    cat("\n")
    print(x$chisq)
  }
  
  # Print missing data notes (mirroring t.test2 style)
  var_na_counts <- attr(x, "var_na_counts")
  var_lengths <- attr(x, "var_lengths")
  var_names <- attr(x, "var_names")
  
  if (!is.null(var_na_counts) && !is.null(var_lengths) && !is.null(var_names)) {
    # Check if any variables have missing data
    has_missing <- any(var_na_counts > 0, na.rm = TRUE)
    
    if (has_missing) {
      cat("\n")
      missing_msgs <- character(0)
      for (i in seq_along(var_na_counts)) {
        if (!is.na(var_na_counts[i]) && var_na_counts[i] > 0) {
          var_name <- if (i <= length(var_names) && !is.na(var_names[i]) && var_names[i] != "") {
            paste0("'", var_names[i], "'")
          } else {
            paste0("variable ", i)
          }
          missing_msgs <- c(missing_msgs, 
                           paste0(var_name, " is missing ", var_na_counts[i], 
                                 " of ", var_lengths[i], " values"))
        }
      }
      if (length(missing_msgs) > 0) {
        cat("note:", paste(missing_msgs, collapse = ", while "), "\n")
      }
    }
  }
  
  invisible(x)
}

#table2_001
test_that("table2 creates basic table", {
  x <- c("A", "A", "B", "B", "A")
  y <- c("X", "Y", "X", "Y", "X")
  
  result <- table2(x, y)
  
  # Result is now a list with class table2
  expect_true(inherits(result, "table2"))
  expect_true(is.list(result))
  expect_true("freq" %in% names(result))
  expect_true("prop" %in% names(result))
  expect_true("chisq" %in% names(result))
  
  # freq should be the frequency table
  expect_true(inherits(result$freq, "table"))
  expect_equal(length(dim(result$freq)), 2)
  expect_equal(dim(result$freq), c(2, 2))
  
  # prop and chisq should be NULL when not requested
  expect_null(result$prop)
  expect_null(result$chisq)
})

#table2_002
test_that("table2 handles data frame column references", {
  df <- data.frame(
    group = c("A", "A", "B", "B", "A"),
    status = c("X", "Y", "X", "Y", "X")
  )
  
  result <- table2(df$group, df$status)
  
  expect_true(inherits(result, "table2"))
  expect_true(inherits(result$freq, "table"))
  expect_equal(length(dim(result$freq)), 2)
  
  # Check that dimension names are set on freq
  dim_names <- names(dimnames(result$freq))
  expect_true(!is.null(dim_names))
  expect_equal(length(dim_names), 2)
})

#table2_003
test_that("table2 handles three-way table", {
  df <- data.frame(
    x = c("A", "A", "B", "B"),
    y = c("X", "Y", "X", "Y"),
    z = c("high", "low", "high", "low")
  )
  
  result <- table2(df$x, df$y, df$z)
  
  expect_true(inherits(result, "table2"))
  expect_true(inherits(result$freq, "table"))
  expect_equal(length(dim(result$freq)), 3)
  expect_equal(dim(result$freq), c(2, 2, 2))
})

#table2_004
test_that("table2 handles prop parameter", {
  x <- c("A", "A", "B", "B", "A")
  y <- c("X", "Y", "X", "Y", "X")
  
  # Overall proportions
  result0 <- table2(x, y, prop = 0)
  expect_true(inherits(result0, "table2"))
  expect_true(!is.null(result0$prop))
  expect_true(is.numeric(result0$prop))
  expect_true(!is.null(result0$freq))
  
  # Row proportions
  result1 <- table2(x, y, prop = 1)
  expect_true(!is.null(result1$prop))
  expect_true(is.numeric(result1$prop))
  
  # Column proportions
  result2 <- table2(x, y, prop = 2)
  expect_true(!is.null(result2$prop))
  expect_true(is.numeric(result2$prop))
  
  # Character prop values
  result_row <- table2(x, y, prop = "row")
  expect_true(!is.null(result_row$prop))
  
  result_col <- table2(x, y, prop = "col")
  expect_true(!is.null(result_col$prop))
})

#table2_005
test_that("table2 prop prints with decimal formatting", {
  x <- c("A", "A", "B", "B", "A")
  y <- c("X", "Y", "X", "Y", "X")
  
  result <- table2(x, y, prop = 1)
  
  # Capture printed output
  output <- capture.output(print(result))
  

  # Should contain decimal proportions (e.g., .500, .333), not just 0s and 1s
  # Look for pattern like .XXX (decimal point followed by digits)
  expect_true(any(grepl("\\.[0-9]{3}", output)), 
              info = "Proportions should print with decimal formatting")
})

#table2_006
test_that("table2 handles digits parameter", {
  x <- c("A", "A", "B", "B", "A")
  y <- c("X", "Y", "X", "Y", "X")
  
  result <- table2(x, y, prop = 1, digits = 2)
  
  # Check that prop is numeric
  expect_true(!is.null(result$prop))
  expect_true(is.numeric(result$prop))
})

#table2_007
test_that("table2 handles useNA parameter", {
  x <- c("A", "A", NA, "B", "A")
  y <- c("X", "Y", "X", "Y", "X")
  
  # Without NA - should exclude NA values
  result_no <- table2(x, y, useNA = "no")
  dimnames_x_no <- dimnames(result_no$freq)[[1]]
  expect_false(any(is.na(dimnames_x_no)))
  
  # With NA if any - may include NA if present
  result_ifany <- table2(x, y, useNA = "ifany")
  # May or may not include NA depending on data
  
  # Always with NA - should include NA in dimnames when there are NAs in data
  result_always <- table2(x, y, useNA = "always")
  expect_true(!is.null(result_always$freq))
  expect_true(!is.null(dimnames(result_always$freq)))
  dimnames_x_always <- dimnames(result_always$freq)[[1]]
  expect_true(!is.null(dimnames_x_always))
  expect_true(length(dimnames_x_always) > 0)
  # When useNA="always" and there are NAs, NA should be in dimnames
  expect_true(any(is.na(dimnames_x_always)) || "<NA>" %in% dimnames_x_always)
})

#table2_008
test_that("table2 handles exclude parameter", {
  x <- c("A", "A", "B", "B", "A")
  y <- c("X", "Y", "X", "Y", "X")
  
  # Should work with exclude
  result <- table2(x, y, exclude = NULL)
  expect_true(inherits(result, "table2"))
  expect_true(inherits(result$freq, "table"))
})

#table2_009
test_that("table2 returns correct structure", {
  x <- c("A", "A", "B", "B")
  y <- c("X", "Y", "X", "Y")
  
  result <- table2(x, y)
  
  # Check structure - result is a list with table2 class
  expect_true(inherits(result, "table2"))
  expect_true(inherits(result$freq, "table"))
  expect_equal(length(dim(result$freq)), 2)
  
  # Check values sum correctly
  expect_equal(sum(result$freq), length(x))
})

#table2_010
test_that("table2 handles single vector", {
  x <- c("A", "A", "B", "B", "A")
  
  result <- table2(x)
  
  expect_true(inherits(result, "table2"))
  expect_true(inherits(result$freq, "table"))
  expect_equal(length(dim(result$freq)), 1)
  expect_equal(sum(result$freq), length(x))
})

#table2_011
test_that("table2 single vector prints without error", {
  # Regression test: table2() with one variable used to produce
  # "Error in NextMethod() : generic function not specified"
  x <- c("A", "A", "B", "B", "A")
  
  result <- table2(x)
  
  # Printing should not error
  expect_no_error(print(result))
  
  # Capture output and verify it contains expected values
  output <- capture.output(print(result))
  expect_true(any(grepl("A", output)))
  expect_true(any(grepl("B", output)))
})

#table2_012
test_that("table2 single vector with prop and chi prints chi-square only once", {
  # Regression test: chi-square was being printed twice for 1D tables with prop
  x <- c("A", "A", "A", "B", "B")
  
  result <- table2(x, prop = "all", chi = TRUE)
  
  # Capture output
  output <- capture.output(print(result))
  
  # Count occurrences of chi-square header
  chi_count <- sum(grepl("Chi-squared test", output))
  expect_equal(chi_count, 1, info = "Chi-square test should only be printed once")
  
  # Verify chi-square result is present (check for the pattern)
  expect_true(any(grepl("Chi-squared test", output)), 
              info = "Chi-square statistic should be in output")
})

#table2_013
test_that("table2 prop='ALL' Total row sums to 1.0", {
  x <- c("A", "A", "B", "B", "A")
  y <- c("X", "Y", "X", "Y", "X")
  
  # Test with prop='ALL' (overall proportions)
  result <- table2(x, y, prop = "ALL")
  prop_table <- result$prop
  
  # Check that the Total row (excluding the bottom right 1.0) sums to 1.0
  n_rows <- nrow(prop_table)
  n_cols <- ncol(prop_table)
  total_row <- prop_table[n_rows, -n_cols, drop = FALSE]
  total_row_sum <- sum(total_row, na.rm = TRUE)
  
  # Should sum to 1.0 (within rounding tolerance)
  expect_equal(total_row_sum, 1.0, tolerance = 0.001)
  
  # Also test with prop=0 (numeric equivalent)
  result0 <- table2(x, y, prop = 0)
  prop_table0 <- result0$prop
  n_rows0 <- nrow(prop_table0)
  n_cols0 <- ncol(prop_table0)
  total_row0 <- prop_table0[n_rows0, -n_cols0, drop = FALSE]
  total_row_sum0 <- sum(total_row0, na.rm = TRUE)
  expect_equal(total_row_sum0, 1.0, tolerance = 0.001)
})

#table2_014
test_that("table2 chi parameter returns chi-square test", {
  # Use larger sample to avoid chi-squared approximation warning
  set.seed(123)
  x <- sample(c("A", "B"), 100, replace = TRUE)
  y <- sample(c("X", "Y"), 100, replace = TRUE)
  
  # Without chi test
  result_no_chi <- table2(x, y)
  expect_null(result_no_chi$chisq)
  
  # With chi test
  result_chi <- table2(x, y, chi = TRUE)
  expect_true(!is.null(result_chi$chisq))
  expect_true(inherits(result_chi$chisq, "htest"))
  
  # Chi-square test with proportions
  result_chi_prop <- table2(x, y, prop = 1, chi = TRUE)
  expect_true(!is.null(result_chi_prop$chisq))
  expect_true(!is.null(result_chi_prop$prop))
  expect_true(!is.null(result_chi_prop$freq))
})

# ============================================================================
# EDGE CASES AND ADDITIONAL TESTS
# ============================================================================

#table2_015
test_that("table2 handles factors", {
  x <- factor(c("A", "A", "B", "B"))
  y <- factor(c("X", "Y", "X", "Y"))
  
  result <- table2(x, y)
  
  expect_true(inherits(result, "table2"))
  expect_equal(dim(result$freq), c(2, 2))
})

#table2_016
test_that("table2 handles numeric variables", {
  x <- c(1, 1, 2, 2)
  y <- c(10, 20, 10, 20)
  
  result <- table2(x, y)
  
  expect_true(inherits(result, "table2"))
  expect_true(inherits(result$freq, "table"))
})

#table2_017
test_that("table2 handles many categories", {
  x <- sample(LETTERS[1:5], 100, replace = TRUE)
  y <- sample(letters[1:4], 100, replace = TRUE)
  
  result <- table2(x, y)
  
  expect_equal(dim(result$freq), c(5, 4))
})

#table2_018
test_that("table2 chi-square test produces valid statistics", {
  set.seed(456)
  # Create data with known association
  x <- sample(c("A", "B"), 200, replace = TRUE)
  y <- ifelse(x == "A", 
              sample(c("X", "Y"), 200, replace = TRUE, prob = c(0.8, 0.2)),
              sample(c("X", "Y"), 200, replace = TRUE, prob = c(0.2, 0.8)))
  
  result <- table2(x, y, chi = TRUE)
  
  # Chi-square statistic should be positive
  expect_true(result$chisq$statistic > 0)
  # p-value should be between 0 and 1
  expect_true(result$chisq$p.value >= 0 && result$chisq$p.value <= 1)
})

#table2_019
test_that("table2 prop values sum correctly for row proportions", {
  x <- c("A", "A", "A", "B", "B")
  y <- c("X", "X", "Y", "X", "Y")
  
  result <- table2(x, y, prop = 1)  # Row proportions
  
  # Each row (excluding Total) should sum to 1
  prop_matrix <- result$prop
  n_rows <- nrow(prop_matrix)
  n_cols <- ncol(prop_matrix)
  
  # Check first row sums to 1 (excluding Total column)
  row1_sum <- sum(prop_matrix[1, 1:(n_cols-1)], na.rm = TRUE)
  expect_equal(row1_sum, 1.0, tolerance = 0.001)
})

#table2_020
test_that("table2 prop values sum correctly for column proportions", {
  x <- c("A", "A", "A", "B", "B")
  y <- c("X", "X", "Y", "X", "Y")
  
  result <- table2(x, y, prop = 2)  # Column proportions
  
  # Each column (excluding Total row) should sum to 1
  prop_matrix <- result$prop
  n_rows <- nrow(prop_matrix)
  n_cols <- ncol(prop_matrix)
  
  # Check first column sums to 1 (excluding Total row)
  col1_sum <- sum(prop_matrix[1:(n_rows-1), 1], na.rm = TRUE)
  expect_equal(col1_sum, 1.0, tolerance = 0.001)
})

# ============================================================================
# SNAPSHOT TESTS FOR OUTPUT FORMAT
# ============================================================================

#table2_021
test_that("table2 print output for frequency table is stable", {
  set.seed(42)
  gender <- sample(c("Male", "Female"), 100, replace = TRUE)
  response <- sample(c("Yes", "No", "Maybe"), 100, replace = TRUE)
  
  result <- table2(gender, response)
  
  expect_snapshot(print(result))
})

#table2_022
test_that("table2 print output with row proportions is stable", {
  set.seed(42)
  gender <- sample(c("Male", "Female"), 100, replace = TRUE)
  response <- sample(c("Yes", "No"), 100, replace = TRUE)
  
  result <- table2(gender, response, prop = 1)
  
  expect_snapshot(print(result))
})

#table2_023
test_that("table2 print output with column proportions is stable", {
  set.seed(42)
  gender <- sample(c("Male", "Female"), 100, replace = TRUE)
  response <- sample(c("Yes", "No"), 100, replace = TRUE)
  
  result <- table2(gender, response, prop = 2)
  
  expect_snapshot(print(result))
})

#table2_024
test_that("table2 print output with chi-square test is stable", {
  set.seed(42)
  gender <- sample(c("Male", "Female"), 100, replace = TRUE)
  response <- sample(c("Yes", "No"), 100, replace = TRUE)
  
  result <- table2(gender, response, chi = TRUE)
  
  expect_snapshot(print(result))
})

#table2_025
test_that("table2 print output for three-way table is stable", {
  set.seed(42)
  x <- sample(c("A", "B"), 60, replace = TRUE)
  y <- sample(c("X", "Y"), 60, replace = TRUE)
  z <- sample(c("High", "Low"), 60, replace = TRUE)
  
  result <- table2(x, y, z)
  
  expect_snapshot(print(result))
})

# ============================================================================
# CONSISTENCY TESTS WITH BASE R FUNCTIONS
# ============================================================================

#table2_026
test_that("table2 freq matches table() output", {
  set.seed(123)
  x <- sample(c("A", "B", "C"), 100, replace = TRUE)
  y <- sample(c("X", "Y"), 100, replace = TRUE)
  
  result <- table2(x, y)
  base_table <- table(x, y)
  

  # Frequencies should match exactly
  expect_equal(as.vector(result$freq), as.vector(base_table))
  
  # Dimensions should match
  expect_equal(dim(result$freq), dim(base_table))
})

#table2_027
test_that("table2 freq matches table() for single variable", {
  set.seed(456)
  x <- sample(c("A", "B", "C"), 50, replace = TRUE)
  
  result <- table2(x)
  base_table <- table(x)
  
  # Frequencies should match exactly
  expect_equal(as.vector(result$freq), as.vector(base_table))
})

#table2_028
test_that("table2 freq matches table() for three-way table", {
  set.seed(789)
  x <- sample(c("A", "B"), 80, replace = TRUE)
  y <- sample(c("X", "Y"), 80, replace = TRUE)
  z <- sample(c("High", "Low"), 80, replace = TRUE)
  
  result <- table2(x, y, z)
  base_table <- table(x, y, z)
  
  # Frequencies should match exactly
  expect_equal(as.vector(result$freq), as.vector(base_table))
  
  # Dimensions should match
  expect_equal(dim(result$freq), dim(base_table))
})

#table2_029
test_that("table2 prop='all' matches prop.table() overall proportions", {
  set.seed(111)
  x <- sample(c("A", "B", "C"), 100, replace = TRUE)
  y <- sample(c("X", "Y"), 100, replace = TRUE)
  
  result <- table2(x, y, prop = "all")
  base_table <- table(x, y)
  base_prop <- prop.table(base_table)
  
 # For prop="all", table2 adds both Total row and Total column
  # Extract inner part by excluding "Total" from both dimensions
  prop_matrix <- result$prop
  row_names <- rownames(prop_matrix)
  col_names <- colnames(prop_matrix)
  inner_rows <- row_names[row_names != "Total"]
  inner_cols <- col_names[col_names != "Total"]
  inner_prop <- prop_matrix[inner_rows, inner_cols]
  
  # table2 rounds to 3 decimal places for display
  # Compare element-by-element (same row/column order)
  for (r in inner_rows) {
    for (c in inner_cols) {
      expect_true(abs(inner_prop[r, c] - base_prop[r, c]) < 0.001,
                  info = paste("Mismatch at row", r, "col", c, 
                               "table2:", inner_prop[r, c], "prop.table:", base_prop[r, c]))
    }
  }
})

#table2_030
test_that("table2 prop=1 matches prop.table(margin=1) row proportions", {
  set.seed(222)
  x <- sample(c("A", "B", "C"), 100, replace = TRUE)
  y <- sample(c("X", "Y"), 100, replace = TRUE)
  
  result <- table2(x, y, prop = 1)
  base_table <- table(x, y)
  base_prop <- prop.table(base_table, margin = 1)
  
  # For prop=1 (row), table2 adds only Total column (no Total row)
  prop_matrix <- result$prop
  col_names <- colnames(prop_matrix)
  inner_cols <- col_names[col_names != "Total"]
  inner_prop <- prop_matrix[, inner_cols]
  
  # table2 rounds to 3 decimal places for display
  # Compare element-by-element (same row/column order)
  for (r in rownames(inner_prop)) {
    for (c in inner_cols) {
      expect_true(abs(inner_prop[r, c] - base_prop[r, c]) < 0.001,
                  info = paste("Mismatch at row", r, "col", c,
                               "table2:", inner_prop[r, c], "prop.table:", base_prop[r, c]))
    }
  }
})

#table2_031
test_that("table2 prop=2 matches prop.table(margin=2) column proportions", {
  set.seed(333)
  x <- sample(c("A", "B", "C"), 100, replace = TRUE)
  y <- sample(c("X", "Y"), 100, replace = TRUE)
  
  result <- table2(x, y, prop = 2)
  base_table <- table(x, y)
  base_prop <- prop.table(base_table, margin = 2)
  
  # For prop=2 (column), table2 adds only Total row (no Total column)
  prop_matrix <- result$prop
  row_names <- rownames(prop_matrix)
  inner_rows <- row_names[row_names != "Total"]
  inner_prop <- prop_matrix[inner_rows, ]
  
  # table2 rounds to 3 decimal places for display
  # Compare element-by-element (same row/column order)
  for (r in inner_rows) {
    for (c in colnames(inner_prop)) {
      expect_true(abs(inner_prop[r, c] - base_prop[r, c]) < 0.001,
                  info = paste("Mismatch at row", r, "col", c,
                               "table2:", inner_prop[r, c], "prop.table:", base_prop[r, c]))
    }
  }
})

#table2_032
test_that("table2 chi=TRUE matches chisq.test() results", {
  set.seed(444)
  x <- sample(c("A", "B", "C"), 150, replace = TRUE)
  y <- sample(c("X", "Y", "Z"), 150, replace = TRUE)
  
  result <- table2(x, y, chi = TRUE)
  base_table <- table(x, y)
  base_chisq <- chisq.test(base_table)
  
  # Chi-square statistic should match
  expect_equal(
    as.numeric(result$chisq$statistic), 
    as.numeric(base_chisq$statistic), 
    tolerance = 1e-10
  )
  
  # Degrees of freedom should match
  expect_equal(result$chisq$parameter, base_chisq$parameter)
  
  # p-value should match
  expect_equal(result$chisq$p.value, base_chisq$p.value, tolerance = 1e-10)
})

#table2_033
test_that("table2 chi=TRUE matches chisq.test() for 2x2 table", {
  set.seed(555)
  x <- sample(c("A", "B"), 100, replace = TRUE)
  y <- sample(c("X", "Y"), 100, replace = TRUE)
  
  # table2 defaults to correct=FALSE
  result <- table2(x, y, chi = TRUE)
  base_table <- table(x, y)
  base_chisq <- chisq.test(base_table, correct = FALSE)
  
  # Chi-square statistic should match
  expect_equal(
    as.numeric(result$chisq$statistic), 
    as.numeric(base_chisq$statistic), 
    tolerance = 1e-10
  )
  
  # p-value should match
  expect_equal(result$chisq$p.value, base_chisq$p.value, tolerance = 1e-10)
})

#table2_034
test_that("table2 correct parameter controls Yates continuity correction", {
  set.seed(556)
  x <- sample(c("A", "B"), 100, replace = TRUE)
  y <- sample(c("X", "Y"), 100, replace = TRUE)
  
  # Without correction (default for table2)
  result_uncorrected <- table2(x, y, chi = TRUE)
  base_uncorrected <- chisq.test(table(x, y), correct = FALSE)
  
  # With correction (explicit)
  result_corrected <- table2(x, y, chi = TRUE, correct = TRUE)
  base_corrected <- chisq.test(table(x, y), correct = TRUE)
  
  # Uncorrected results should match (default)
  expect_equal(
    as.numeric(result_uncorrected$chisq$statistic),
    as.numeric(base_uncorrected$statistic),
    tolerance = 1e-10
  )
  
  # Corrected results should match
  expect_equal(
    as.numeric(result_corrected$chisq$statistic),
    as.numeric(base_corrected$statistic),
    tolerance = 1e-10
  )
  
  # Corrected and uncorrected should differ for 2x2 tables
  expect_false(
    isTRUE(all.equal(
      as.numeric(result_corrected$chisq$statistic),
      as.numeric(result_uncorrected$chisq$statistic)
    ))
  )
})

#table2_035
test_that("table2 chi-square matches prop.test for 2x2 tables", {
  set.seed(557)
  x <- sample(c("A", "B"), 100, replace = TRUE)
  y <- sample(c("X", "Y"), 100, replace = TRUE)
  
  base_table <- table(x, y)
  
  # Without Yates correction (default for table2)
  result_uncorrected <- table2(x, y, chi = TRUE)
  prop_uncorrected <- prop.test(base_table, correct = FALSE)
  
  # Chi-square statistic should match prop.test (uncorrected)
  expect_equal(
    as.numeric(result_uncorrected$chisq$statistic),
    as.numeric(prop_uncorrected$statistic),
    tolerance = 1e-10
  )
  
  # p-value should match prop.test (uncorrected)
  expect_equal(
    result_uncorrected$chisq$p.value,
    prop_uncorrected$p.value,
    tolerance = 1e-10
  )
  
  # With Yates correction (explicit)
  result_corrected <- table2(x, y, chi = TRUE, correct = TRUE)
  prop_corrected <- prop.test(base_table, correct = TRUE)
  
  # Chi-square statistic should match prop.test
  expect_equal(
    as.numeric(result_corrected$chisq$statistic),
    as.numeric(prop_corrected$statistic),
    tolerance = 1e-10
  )
  
  # p-value should match prop.test
  expect_equal(
    result_corrected$chisq$p.value,
    prop_corrected$p.value,
    tolerance = 1e-10
  )
})

# Tests for comparison operators in table2
#table2_036
test_that("table2 handles comparison operators and shows full expression", {
  # Create test data
  df <- data.frame(
    x = c(rep(5, 30), rep(15, 70)),
    y = c(rep(3, 40), rep(12, 60))
  )
  
  # Test with > operator (1D table)
  result_gt <- table2(df$x > 10)
  expect_true("x>10" %in% names(dimnames(result_gt$freq)))
  
  # Test with < operator (1D table)
  result_lt <- table2(df$y < 10)
  expect_true("y<10" %in% names(dimnames(result_lt$freq)))
  
  # Test with >= operator (1D table)
  result_gte <- table2(df$x >= 10)
  expect_true("x>=10" %in% names(dimnames(result_gte$freq)))
  
  # Test with <= operator (1D table)
  result_lte <- table2(df$y <= 5)
  expect_true("y<=5" %in% names(dimnames(result_lte$freq)))
  
  # Test with == operator (1D table)
  result_eq <- table2(df$x == 15)
  expect_true("x==15" %in% names(dimnames(result_eq$freq)))
  
  # Test with != operator (1D table)
  result_neq <- table2(df$y != 3)
  expect_true("y!=3" %in% names(dimnames(result_neq$freq)))
})

#table2_037
test_that("table2 handles comparison operators in 2D tables", {
  # Create test data
  df <- data.frame(
    score = c(rep(5, 20), rep(15, 30), rep(8, 25), rep(18, 25)),
    grade = c(rep(6, 35), rep(9, 65))
  )
  
  # Test 2D table with comparison operators
  result <- table2(df$score > 10, df$grade > 7)
  
  # Check that both dimension names include the full expression
  dimn <- dimnames(result$freq)
  expect_true("score>10" %in% names(dimn))
  expect_true("grade>7" %in% names(dimn))
  
  # Verify the table has correct dimensions
  expect_equal(length(dim(result$freq)), 2)
  expect_equal(dim(result$freq), c(2, 2))
})

#table2_038
test_that("table2 truncates long comparison expressions", {
  # Create test data with a long variable name
  df <- data.frame(
    very_long_variable_name_that_exceeds_limit = c(rep(100, 50), rep(500, 50))
  )
  
  # Test with a long expression
  result <- table2(df$very_long_variable_name_that_exceeds_limit > 200)
  
  # Check that the name is truncated to 30 characters
  var_name <- names(dimnames(result$freq))[1]
  expect_lte(nchar(var_name), 30)
  expect_true(grepl("\\.\\.\\.$", var_name))  # Should end with "..."
})

#table2_039
test_that("table2 comparison operators work with proportions", {
  # Create test data
  df <- data.frame(
    x = c(rep(5, 30), rep(15, 70)),
    y = c(rep(3, 40), rep(12, 60))
  )
  
  # Test with proportions
  result <- table2(df$x > 10, df$y > 8, prop = "all")
  
  # Check that dimension names are preserved
  expect_true("x>10" %in% names(dimnames(result$freq)))
  expect_true("y>8" %in% names(dimnames(result$freq)))
  
  # Check that proportions were calculated
  expect_false(is.null(result$prop))
  expect_equal(sum(result$prop[1:2, 1:2]), 1, tolerance = 0.01)
})

#table2_040
test_that("table2 comparison operators work with chi-square test", {
  # Create test data
  set.seed(123)
  df <- data.frame(
    x = sample(c(5, 15), 100, replace = TRUE),
    y = sample(c(3, 12), 100, replace = TRUE)
  )
  
  # Test with chi-square
  result <- table2(df$x > 10, df$y > 8, chi = TRUE)
  
  # Check that dimension names are preserved
  expect_true("x>10" %in% names(dimnames(result$freq)))
  expect_true("y>8" %in% names(dimnames(result$freq)))
  
  # Check that chi-square test was performed
  expect_false(is.null(result$chisq))
  expect_true("statistic" %in% names(result$chisq))
  expect_true("p.value" %in% names(result$chisq))
})

#print.table2_001
test_that("print.table2 runs without errors", {
  df <- data.frame(
    group = c("A", "A", "B", "B", "A"),
    status = c("X", "Y", "X", "Y", "X")
  )
  
  tbl <- table2(df$group, df$status)
  
  # Should print without errors
  expect_error(print(tbl), NA)
  
  # Should return invisibly
  result <- print(tbl)
  expect_equal(result, tbl)
})

#print.table2_002
test_that("print.table2 handles proportion tables", {
  x <- c("A", "A", "B", "B", "A")
  y <- c("X", "Y", "X", "Y", "X")
  
  tbl <- table2(x, y, prop = 1)
  
  expect_error(print(tbl), NA)
})

#print.table2_003
test_that("print.table2 handles three-way tables", {
  df <- data.frame(
    x = c("A", "A", "B", "B"),
    y = c("X", "Y", "X", "Y"),
    z = c("high", "low", "high", "low")
  )
  
  tbl <- table2(df$x, df$y, df$z)
  
  expect_error(print(tbl), NA)
})

#print.table2_004
test_that("print.table2 handles tables without dimension names", {
  x <- c("A", "A", "B", "B")
  y <- c("X", "Y", "X", "Y")
  
  # Create table without using df$ syntax (no dimension names)
  tbl <- table2(x, y)
  
  # Should still print (may fall back to default print)
  expect_error(print(tbl), NA)
})

#' Make beeswarm plots for compared-stimulus designs
#'
#' Beeswarm plot for compared-stimulus designs from "Stimulus Sampling Reimagined"
#' (Simonsohn, Montealegre, & Evangelidis, 2024).
#'
#' @param data Data frame containing variables to be analyzed.
#' @param dv Name of the dependent variable.
#' @param stimulus Name of the variable containing the stimulus ID.
#' @param condition Name of the variable containing the condition indicator.
#' @param flip.conditions If `TRUE`, reverse the order of condition labels.
#' @param dv.is.percentage If `TRUE`, format values as percentages.
#' @param simtot Number of bootstraps for the confidence band under homogeneity.
#' @param confidence Confidence level for the band (default 95).
#' @param ylim Optional y-axis limits.
#' @param ylab1,ylab2 Labels on the y-axis (optional).
#' @param xlab1,xlab2 Labels on the x-axis (optional).
#' @param dot.spacing Horizontal distance between stimulus labels (`"auto"` by default).
#' @param col1,col2 Colors for the two conditions.
#' @param main Plot title.
#' @param watermark If `TRUE`, display package version watermark.
#' @param save.as File path for saving the figure (`.svg` or `.png`, optional).
#' @param svg.width Optional width when saving.
#' @param svg.height Optional height when saving.
#' @param ... Additional arguments passed to [graphics::plot.default()].
#'
#' @return Invisibly, a two-column matrix of beeswarm coordinates.
#' @importFrom beeswarm beeswarm
#' @importFrom grDevices adjustcolor dev.off png svg
#' @importFrom graphics axis legend mtext par plot segments text
#' @importFrom stats aggregate
#' @importFrom utils packageVersion
#' @export
stimulus.beeswarm <- function(
  data, dv, stimulus, condition,
  flip.conditions = FALSE,
  dv.is.percentage = FALSE,
  simtot = 500,
  confidence = 95,
  ylim = c(),
  ylab1 = '',
  ylab2 = '',
  xlab1 = '',
  xlab2 = NULL,
  dot.spacing = 'auto',
  col1 = 'blue4',
  col2 = 'red4',
  main = '',
  watermark = TRUE,
  save.as = '',
  svg.width = '',
  svg.height = '',
  ...
) {

#: 1 stimulus.beeswarm: validate -> means -> bootstrap -> plot
  args_passed <- as.list(match.call())[-1]
  args <- list(...)

  validate.beeswarm(
    data, dv, stimulus, condition, flip.conditions, dv.is.percentage,
    simtot, confidence, ylim, ylab1, ylab2, xlab1, xlab2, dot.spacing,
    col1, col2, main, watermark, save.as, svg.width, svg.height, args_passed
  )

  uc <- sort(as.character(unique(data[, condition])), decreasing = flip.conditions)
  dc1 <- data[, condition] == uc[1]
  dc2 <- data[, condition] == uc[2]

  ms1 <- stats::aggregate(data[dc1, dv], list(data[dc1, stimulus]), mean)
  ms2 <- stats::aggregate(data[dc2, dv], list(data[dc2, stimulus]), mean)
  ms1$condition <- uc[1]
  ms2$condition <- uc[2]
  names(ms1) <- names(ms2) <- c("stimulus", 'mean', 'condition')
  ms <- rbind(ms1, ms2)

  dataname <- clean_string(deparse(substitute(data)))
  md5k <- get.md5(list(dataname, dv, stimulus, condition, simtot))
  cache <- sp_stimulus_cache()

  if (does.cache.d.exist(md5k)) {
    maxmin_boot <- cache[[md5k]]
    if (sys.parent() == 0) {
      message2(
        "*Recycled results*:\n",
        "You had run this same analysis before with all the same variables and options.\n",
        "(data='", dataname, "' | dv='", dv, "' | stimulus='", stimulus, "' | condition='", condition, "' | simtot='", simtot, "')\n",
        "To save time, we are re-using saved results. To force new calculations\n",
        "change one of those parameters or clear your cache running: 'clear_stimulus_cache()'"
      )
    }
  } else {
    maxmin_boot <- get.maxmin.confidence(data, dv, stimulus, condition, simtot, confidence, ms1, ms2, dc1, dc2)
    cache[[md5k]] <- maxmin_boot
    .statuser_state$stimulus_cache <- cache
  }

  if (dot.spacing == 'auto') {
    stimulus.length <- mean(nchar(ms$stimulus))
    dot.spacing <- stimulus.length / 3 + 2
  }

  col1a <- grDevices::adjustcolor(col1, .75)
  col2a <- grDevices::adjustcolor(col2, .75)

  b <- beeswarm::beeswarm(ms$mean ~ ms$condition, spacing = dot.spacing, do.plot = FALSE, method = 'swarm')

  if (save.as != '') {
    filename <- save.as
    extension <- tools::file_ext(filename)
    w <- 9
    h <- 7
    if (svg.width != '') w <- svg.width
    if (svg.height != '') h <- svg.height
    if (extension == 'svg') grDevices::svg(filename, w, h)
    if (extension == 'png') grDevices::png(filename, w * 1000, h * 1000, res = 1000)
  }

  mar.before <- graphics::par("mar")
  mar.after <- mar.before
  custom_mar <- getOption("graphics.par")$mar
  if (!is.null(custom_mar)) mar.default <- custom_mar
  if (is.null(custom_mar)) mar.default <- c(5.1, 4.1, 4.1, 2.1)

  if (all(mar.before == mar.default)) {
    mar.after[3] <- ifelse(main == '', 1, 3)
    width.y.label <- nchar(max(pretty(ms$mean)))
    mar.after[2] <- max(width.y.label / 3.5, 5)
    if (ylab2 != '') mar.after[2] <- mar.after[2] + 1
    if (dv.is.percentage == TRUE) mar.after[2] <- mar.after[2] + 1
    graphics::par(mar = mar.after)
    on.exit(graphics::par(mar = mar.before), add = TRUE)
  }

  if (length(ylim) < 2) {
    ylim <- range(ms$mean)
    dy <- diff(ylim)
    ylim[2] <- ylim[2] + .25 * dy
  }

  graphics::plot(
    b$x, b$y, pch = NA,
    xlim = c(min(.75, b$x), max(2.25, b$x)),
    xlab = '', ylab = '', las = 1,
    yaxt = ifelse(dv.is.percentage, 'n', 's'),
    xaxt = 'n', ylim = ylim, ...
  )

  b1 <- b[b$x <= 1.5, , drop = FALSE]
  b2 <- b[b$x > 1.5, , drop = FALSE]
  graphics::text(b1$x, b1$y, ms[ms$mean == b1$y.orig, 'stimulus'], cex = .65, col = col1a)
  graphics::text(b2$x, b2$y, ms[ms$mean == b2$y.orig, 'stimulus'], cex = .65, col = col2a)

  if (xlab1 == '') xlab1 <- condition
  if (is.null(xlab2)) xlab2 <- uc
  graphics::mtext(side = 1, font = 2, xlab1, line = 3, cex = 1.65)
  graphics::mtext(side = 1, at = c(1, 2), line = 1.5, xlab2, cex = 1.25, font = 2)

  xt1 <- min(b1$x, 0.85)
  xt2 <- max(b1$x, 1.15)
  xt3 <- min(b2$x, 1.85)
  xt4 <- max(b2$x, 2.15)
  mean1 <- mean(data[data[, condition] == uc[1], dv], na.rm = TRUE)
  mean2 <- mean(data[data[, condition] == uc[2], dv], na.rm = TRUE)
  graphics::segments(x0 = xt1, x1 = xt2, y0 = mean1, y1 = mean1, col = col1, lwd = 3)
  graphics::segments(x0 = xt3, x1 = xt4, y0 = mean2, y1 = mean2, col = col2, lwd = 3)

  if (dv.is.percentage == FALSE) {
    rm1 <- round_smart(mean1)
    rm2 <- round_smart(mean2)
  } else {
    rm1 <- format_percent(mean1, 'auto')
    rm2 <- format_percent(mean2, 'auto')
  }
  graphics::text(xt2, mean1, paste0("M=", rm1), col = col1, cex = 1, pos = 4)
  graphics::text(xt3, mean2, paste0("M=", rm2), col = col2, cex = 1, pos = 2)

  b1L <- maxmin_boot$b1L
  b1H <- maxmin_boot$b1H
  b2L <- maxmin_boot$b2L
  b2H <- maxmin_boot$b2H
  graphics::polygon(x = c(xt1, xt1, xt2, xt2), y = c(b1L, b1H, b1H, b1L), col = grDevices::adjustcolor(col1, .1), border = NA)
  graphics::polygon(x = c(xt3, xt3, xt4, xt4), y = c(b2L, b2H, b2H, b2L), col = grDevices::adjustcolor(col2, .1), border = NA)

  if (ylab1 == '') ylab1 <- dv
  if (!"yaxt" %in% names(args)) {
    graphics::mtext(side = 2, line = mar.after[2] - 1.8, font = 2, cex = 1.65, ylab1)
    graphics::mtext(side = 2, line = mar.after[2] - 2.8, font = 3, cex = 1.25, ylab2, col = 'gray30')
  }
  if (dv.is.percentage == TRUE) {
    ys <- pretty(ms$mean)
    graphics::axis(side = 2, at = ys, paste0(ys * 100, "%"), las = 1)
  }

  if (main != '') graphics::mtext(side = 3, line = 1, font = 2, cex = 1.65, main)

  if (watermark == TRUE) {
    stim_vrs <- paste0("{statuser v", utils::packageVersion('statuser'), "}")
    graphics::mtext(side = 1, line = -1, cex = .7, stim_vrs, col = 'gray66', adj = 0, outer = TRUE)
  }

  kmax <- which.max(ms1$mean)
  stimulus.max <- ms1$stimulus[kmax]
  graphics::legend(
    'top',
    pch = c(21, NA, NA, NA),
    bty = 'n',
    lty = c(NA, 1, 1, NA),
    lwd = c(NA, 3, 20, NA),
    legend = c(
      paste0('Stimulus label, e.g., "', stimulus.max, '"'),
      'Mean across stimuli',
      paste0(confidence, "% confidence band for Max-to-Min stimulus range"),
      paste0("Based on ", simtot, " resamples under null of equal distributions")
    ),
    cex = 1,
    col = c(col1a, col1, grDevices::adjustcolor(col1, .1))
  )

  if (save.as != '') {
    message2("\nFigure was saved as '", save.as, "'")
    if (svg.width == "" & svg.height == "") {
      message2(paste0("NOTE: We used default width=", w, ", and height=", h, ", customize with svg.width & svg.height arguments."))
    }
    grDevices::dev.off()
  }

  if (save.as != '') {
    call.original <- match.call()
    call.original$save.as <- ''
    eval(call.original)
  }

  invisible(b[, 1:2])
}

#' Barplot of means
#'
#' Plots means, with confidence intervals, and (optionally) p-values for differences of means and interactions 
#'
#' @param formula A formula, e.g., \code{y ~ x1+x2}, where x1 & x2 are
#' grouping variables (e.g., condition indicators in a 2x2 experiment). 
#' The formula can include up to three grouping variables. The plot will 
#'  show contiguous bars for x1, and sets of bars for x2 and x3
#' @param data Optional data frame containing variables in the formula.
#' @param cluster Optional clustering variable when there are repeated 
#' observations per cluster 
#' e.g., \code{cluster="participant_ID"}. 
#' When provided, inference for reported tests is based on regressions with
#' clustered standard errors (via \code{lm2(..., clusters=...)}).
#' @param tests specifies which comparisons of means to report. Syntax involves 
#' putting column numbers in a character string, with a - to symbolize a 
#' comparison and a + symbolizing combination. For example tests="1-2" reports 
#' t-test comparing columns 1 and 2. tests="1-2,3-4" t-tests comparing columns 
#' 1 & 2 and another 3 & 4. To run an interaction use parentheses:
#' tests=\code{"(4-3)-(2-1)"} and can also be combined with simple tests. 
#' Main effects can be specified using `+`, for example (1+2)-(3+4) compares 
#' all observations in the first two columns with all observations in 
#' the next two columns.
#' @param quiet Logical. When \code{TRUE}, suppresses console messages from \code{plot_means()}.
#' @param order Controls the order of \code{x1} groups (bar order and colors).
#'   Use \code{-1} to reverse the default order (e.g., if plot shows 'male' first and 'female' second, order=-1 will flip that).
#' @param legend.title Character string. Title for the legend. If \code{NULL},
#'   no title is shown.
#' @param col Color(s) for \code{x1} bars. If \code{NULL}, colors are chosen
#'   automatically.
#' @param col.text Color for confidence intervals and other non-bar annotations.
#'   If \code{NULL}, defaults to a dark gray.
#' @param values.cex Numeric scalar controlling text size for mean value labels
#'   (and related annotations).
#' @param values.align Where within the bars to put the mean value labels: \code{"top"}, \code{"middle"},
#'   \code{"bottom"}, or \code{"none"}.
#' @param values.round Non-negative integer. Number of decimal places for mean
#'   value labels.
#' @param pvalue.cex Numeric scalar controlling p-value label size.
#' @param pvalue.col Color for p-value brackets/labels.
#' @param ci.level Confidence interval level for \code{ciL}/\code{ciH} in \code{$means}.
#'   Default is \code{95}. You can also pass a proportion (e.g., \code{0.95}).
#' @param buffer.top Either \code{"auto"} (default) or a numeric value. Extra
#'   vertical headroom (as a fraction of the data y-range) added above the
#'   maximum y value to make room for annotations. When \code{"auto"}, uses 0.35
#'   when an interaction p-value is shown (scenario 2) and 0.25 otherwise.
#' @param ... Additional arguments passed to \code{plot()} (e.g., \code{main},
#'   \code{ylim}, \code{ylab}).
#'
#' @details
#' When \code{tests="auto"}, the function reports a small default set of
#' differences-of-means tests (when applicable) and, in 2x2 designs, an
#' interaction test:
#' \describe{
#'   \item{Differences in means}{If \code{cluster} is \code{NULL}, these are Welch
#'     two-sample t-tests computed with \code{t.test(..., var.equal=FALSE)}. If
#'     \code{cluster} is provided, these comparisons are computed from a
#'     regression using \code{lm2()} with clustered standard errors.}
#'   \item{Interaction}{The interaction is tested using a linear regression fit
#'     with \code{lm2()}, even when \code{cluster} is \code{NULL}; when
#'     \code{cluster} is provided, the interaction test uses clustered standard
#'     errors.}
#' }
#' The regression-based tests use heteroskedasticity-robust inference (HC3) when
#' \code{cluster} is \code{NULL}. HC3 is a common small-sample adjustment to
#' White-type robust standard errors and is used to reduce sensitivity to
#' heteroskedasticity. When \code{cluster} is provided, \code{plot_means()}
#' instead uses clustered standard errors (robust to within-cluster correlation).
#'
#' In the returned \code{$means} table, \code{ciL} and \code{ciH} are the lower and
#' upper bounds of a \code{ci.level}\% confidence interval for the mean (when
#' available). The same confidence level is used for the confidence-interval
#' whiskers drawn in the figure.
#'
#' @return A minimal list returned invisibly with two elements:
#' \describe{
#'   \item{\code{means}}{A data frame of means (and, when available, confidence intervals)
#'     aligned to the plotting grid.}
#'   \item{\code{tests}}{A data frame of comparisons used for p-value
#'     annotation (or \code{NULL} if not applicable).}
#' }
#'
#' @examples
#' df <- data.frame(y = rnorm(100), group = rep(c("A", "B"), 50))
#' plot_means(y ~ group, data = df)
#'
#' df2 <- data.frame(
#'   y = rnorm(200),
#'   x1 = rep(c("A", "B"), 100),
#'   x2 = rep(c("X", "Y"), each = 100)
#' )
#' plot_means(y ~ x1 + x2, data = df2)
#'
#' df3 <- data.frame(
#'   y = rnorm(600),
#'   x1 = rep(c("control", "treatment"), times = 300),
#'   x2 = rep(rep(c("low", "high"), each = 150), times = 2),
#'   x3 = rep(c("online", "lab"), each = 300)
#' )
#' plot_means(y ~ x1 + x2 + x3, data = df3)
#'
#' @export
#' @importFrom grDevices recordPlot replayPlot
#' @importFrom stats na.pass
#'
#: 1 plot_means: validate -> descriptives -> params -> compute -> draw -> output/export
#: 2 plot_means_validate: NSE-safe arg evaluation + input validation/normalization
#: 3 plot_means_params: derive factor levels/order, colors, legend layout, buffer.top
#: 4 plot_means_compute: build full grid, compute CIs, and bar/layout vectors used for plotting
#: 5 plot_means_draw: render the plot in base graphics (bars, CIs, labels, legend, p-values)
#: 6 plot_means_compute_pvalues: compute and format p-values shown on the plot (scenario-specific)


#----------------------------------
# plot_means (exported) ----
plot_means <- function(formula,
                       data = NULL,
                       cluster = NULL,
                       tests = "auto",
                       quiet = FALSE,
                       order = NULL,
                       # Graphics / annotation options
                       legend.title = NULL,
                       col = NULL,
                       col.text = NULL,
                       values.cex = 1,
                       values.align = "top",
                       values.round = 1,
                       pvalue.cex = 0.9,
                       pvalue.col = "gray50",
                       ci.level = 95,
                       buffer.top = "auto",
                       ...) {
  # 2. Validate + normalize inputs (NSE-safe)
  #   (mc must be captured before anything is evaluated)
    mc <- match.call()
    calling_env <- parent.frame()
    
  # Track whether any console output was emitted by plot_means()
    printed_any <- FALSE
    
    msg2 <- function(...) {
      if (isTRUE(quiet)) return(invisible(NULL))
      printed_any <<- TRUE
      message2(...)
    }

  # 1. Validate and normalize inputs
  # Convert user-facing arguments into a single normalized list (`v`) used throughout.
  #   This is where we resolve NSE inputs (formula/data/cluster), validate types and
  #   lengths, and set defaults so downstream helpers can assume consistent fields.
    v <- plot_means_validate(
      mc = mc,
      data = data,
      order = order,
      legend.title = legend.title,
      col = col,
      col.text = col.text,
      cluster = cluster,
      values.cex = values.cex,
      values.align = values.align,
      values.round = values.round,
      tests = tests,
      pvalue.cex = pvalue.cex,
      pvalue.col = pvalue.col,
      ci.level = ci.level,
      buffer.top = buffer.top,
      calling_env = calling_env
    )

    formula <- v$formula_eval
    y_name <- v$y_name
    x_names <- v$x_names
    x1_name <- v$x1_name
    x2_name <- v$x2_name
    x3_name <- v$x3_name
    values.cex <- v$values.cex
    values.align <- v$values.align
    values.round <- v$values.round
    tests <- v$tests
    pvalue.cex <- v$pvalue.cex
    pvalue.col <- v$pvalue.col
    ci.level <- v$ci.level
    buffer.top <- v$buffer.top

  # 2. Compute descriptives (means) using statuser::desc_var()
    result <- eval(call("desc_var", v$formula_eval, data = v$data), envir = v$calling_env)

  # 2.1 Normalize desc_var output for single grouping variable
    result_plot <- result
    if (length(x_names) == 1 && "group" %in% names(result_plot) && !(x1_name %in% names(result_plot))) {
      result_plot[[x1_name]] <- as.character(result_plot$group)
    }

  # 2.2 CI settings (always computed)
    ci_level <- as.numeric(ci.level) / 100

  # 3. Levels/order/colors/buffer/legend layout (derived from data + args)
    params <- plot_means_params(v, result = result, result_plot = result_plot)
    result <- params$result
    x1_levels <- params$x1_levels
    x2_levels <- params$x2_levels
    x3_levels <- params$x3_levels
    k <- params$k
    col <- params$col_vec
    legend_horiz <- params$legend_horiz
    buffer_top_effective <- params$buffer_top_effective
    format_level_label <- params$format_level_label

  # 4. Results table with CI computation and bar layout
    comp <- plot_means_compute(v, params = params, result_plot = result_plot, ci_level = ci_level)
    merged <- comp$merged
    ci_map <- comp$ci_map
    x_lefts <- comp$x_lefts
    x_rights <- comp$x_rights
    x_centers_drawn <- comp$x_centers_drawn
    cell_keys_drawn <- comp$cell_keys_drawn
    n_total_drawn <- comp$n_total_drawn
    n_missing_drawn <- comp$n_missing_drawn
    heights <- comp$heights
    cols <- comp$cols
    block_centers <- comp$block_centers
    block_labels <- comp$block_labels
    x3_section_centers <- comp$x3_section_centers
    x3_section_labels <- comp$x3_section_labels
    x_pos <- comp$x_pos
    bar_width <- comp$bar_width
    bar_step <- comp$bar_step

  # 5. Prepare comparison table used for p-value drawing
    # Build one output table aligned with the plotting grid (keeps missing combos)
      means <- merged
      means$ciL <- NA_real_
      means$ciH <- NA_real_
      if (nrow(ci_map) > 0) {
        # Join CIs onto the grid via the same stable cell key used in CI model
        x2_col <- if (is.null(x2_name)) ".x2" else x2_name
        x3_col <- if (is.null(x3_name)) ".x3" else x3_name
        
        if (!(x2_col %in% names(means))) means[[x2_col]] <- "All"
        if (!(x3_col %in% names(means))) means[[x3_col]] <- "All"
        
        means$cell_key <- paste(
          as.character(means[[x1_name]]),
          as.character(means[[x2_col]]),
          as.character(means[[x3_col]]),
          sep = "|"
        )
        
        ci_idx <- match(means$cell_key, ci_map$cell_key)
        means$ciL <- ifelse(is.na(ci_idx), NA_real_, ci_map$lwr[ci_idx])
        means$ciH <- ifelse(is.na(ci_idx), NA_real_, ci_map$upr[ci_idx])
      }
      
      group_cols <- character(0)
      if (length(x_names) == 1) {
        group_cols <- x1_name
      } else {
        group_cols <- c(x1_name, x2_name, x3_name)
      }
      group_cols <- group_cols[!is.null(group_cols) & nzchar(group_cols)]
      group_cols <- intersect(group_cols, names(means))
      
      keep_cols <- c(group_cols, "mean", "sd", "n.total", "n.missing", "ciL", "ciH")
      keep_cols <- keep_cols[keep_cols %in% names(means)]
      means <- means[, keep_cols, drop = FALSE]
      rownames(means) <- NULL
    
    means_comparisons <- plot_means_compute_pvalues(v = v, params = params, mean_results = means, comp = comp)
    means_comparisons__out <- means_comparisons
    if (!is.null(means_comparisons__out) && is.data.frame(means_comparisons__out)) {
      drop_cols <- intersect(c("col1", "col2", "col3", "col4"), names(means_comparisons__out))
      if (length(drop_cols)) means_comparisons__out[drop_cols] <- NULL
    }
    
    if (!isTRUE(quiet) && !is.null(means_comparisons) && is.data.frame(means_comparisons) && nrow(means_comparisons) > 0) {
      fmt_p <- function(p) {
        if (!is.finite(p)) return("NA")
        if (p < 0.001) return("<0.001")
        format(round(p, 3), nsmall = 3, scientific = FALSE)
      }
      fmt_df <- function(df) {
        if (!is.finite(df)) return("NA")
        format(round(df, 1), nsmall = 1, scientific = FALSE)
      }
      fmt_df_int <- function(df) {
        if (!is.finite(df)) return("NA")
        format(round(df, 0), nsmall = 0, scientific = FALSE)
      }
      fmt_t <- function(t) {
        if (!is.finite(t)) return("NA")
        format(round(t, 2), nsmall = 2, scientific = FALSE)
      }
      
      method_fallback <- function(row) {
        # When not provided by compute_pvalues(), infer from cluster + test type.
        if (!is.null(v$cluster_vec)) return("regression (clustered SE)")
        if ("test_type" %in% names(row) && identical(as.character(row$test_type[1]), "interaction")) return("regression interaction")
        "Welch t-test"
      }
      
      lines <- character(0)
      any_welch <- FALSE
      any_interaction <- FALSE
      for (i in seq_len(nrow(means_comparisons))) {
        r <- means_comparisons[i, , drop = FALSE]
        
        label <- ""
        if (all(c("test_type", "col1", "col2") %in% names(r))) {
          tt <- as.character(r$test_type[1])
          if (identical(tt, "simple")) {
            label <- paste0(r$col1[1], " vs ", r$col2[1])
          } else if (identical(tt, "pooled") && all(c("cols_left", "cols_right") %in% names(r))) {
            left <- gsub(",", ",", as.character(r$cols_left[1]), fixed = TRUE)
            right <- gsub(",", ",", as.character(r$cols_right[1]), fixed = TRUE)
            label <- paste0(left, " vs ", right)
          } else if (identical(tt, "interaction") && all(c("col3", "col4") %in% names(r))) {
            label <- paste0("(", r$col1[1], "-", r$col2[1], ")-(", r$col3[1], "-", r$col4[1], ")")
          } else {
            label <- as.character(r$group1[1])
          }
        } else if (all(c("group1", "group2") %in% names(r)) && nzchar(as.character(r$group2[1]))) {
          label <- paste0(as.character(r$group1[1]), " vs ", as.character(r$group2[1]))
        } else if ("group1" %in% names(r)) {
          label <- as.character(r$group1[1])
        } else {
          label <- paste0("test ", i)
        }
        
      is_interaction_row <- (("test_type" %in% names(r) && identical(as.character(r$test_type[1]), "interaction")) ||
        ("group1" %in% names(r) && grepl("^interaction\\(", as.character(r$group1[1]))))
      
        method <- if ("method" %in% names(r) && nzchar(as.character(r$method[1]))) as.character(r$method[1]) else method_fallback(r)
        df_txt <- if ("df" %in% names(r)) {
          if (isTRUE(is_interaction_row)) fmt_df_int(as.numeric(r$df[1])) else fmt_df(as.numeric(r$df[1]))
        } else {
          "NA"
        }
        t_txt <- fmt_t(as.numeric(r$t.value[1]))
        p_txt <- fmt_p(as.numeric(r$p.value[1]))
        
        any_welch <- any_welch || identical(method, "Welch t-test")
        any_interaction <- any_interaction || isTRUE(is_interaction_row)
        
        if (identical(method, "Welch t-test")) {
          lines <- c(lines, paste0(i, ") ", label, ": t(", df_txt, ")=", t_txt, ", p=", p_txt))
        } else {
          lines <- c(lines, paste0(i, ") ", label, ": ", method, ": t(", df_txt, ")=", t_txt, ", p=", p_txt))
        }
      }
      
      footer <- character(0)
      if (identical(tests, "auto")) {
        footer <- c(footer, "You can specify which statistical tests to report with the `tests` argument.")
      }
      msg2(paste0("tests:\n", paste(lines, collapse = "\n"), if (length(footer)) paste0("\n", paste(footer, collapse = "\n")) else ""), col = "gray")
    }

  # 6. Draw plot
    plot_means_draw(
      v = v,
      params = params,
      comp = comp,
      y_name = y_name,
      x1_levels = x1_levels,
      x2_name = x2_name,
      x3_name = x3_name,
      x1_name = x1_name,
      col = col,
      legend.title = legend.title,
      col.text = col.text,
      values.cex = values.cex,
      values.align = values.align,
      values.round = values.round,
      tests = tests,
      means_comparisons = means_comparisons,
      pvalue.cex = pvalue.cex,
      pvalue.col = pvalue.col,
      buffer_top_effective = buffer_top_effective,
      ...
    )


  # 7. Prepare minimal output (returned invisibly)
    tests_out <- means_comparisons__out
    if (is.null(tests_out)) {
      tests_out <- "Use `tests` argument to request tests contrasting specific means"
    }
    out <- list(
      means = means,
      tests = tests_out
    )
  
    if (isTRUE(printed_any) && !isTRUE(quiet)) {
      msg2("\nSet `quiet=TRUE` to suppress this output.", col = "gray")
    }
    invisible(out)
}



#----------------------------------
# plot_means_validate: evaluate NSE inputs, validate args, and normalize to a single list `v` ----
plot_means_validate <- function(mc,
                                data,
                                order,
                                legend.title,
                                col,
                                col.text,
                                cluster,
                                values.cex,
                                values.align,
                                values.round,
                                tests,
                                pvalue.cex,
                                pvalue.col,
                                ci.level,
                                buffer.top,
                                calling_env) {
  # 1. Resolve and validate formula input (NSE-safe)
  #   \"Resolve\" here means: take what the user typed for `formula` (which may be a name in the
  #   calling environment) and evaluate it into an actual `formula` object we can inspect and use.
    formula_resolved <- evaluate_variable_arguments(
      arg_expr = mc$formula,
      arg_name = "formula",
      data = data,
      calling_env = calling_env,
      func_name = "plot_means",
      allow_null = FALSE
    )
    formula_eval <- formula_resolved$value
    
    validate_formula(formula_eval, data, func_name = "plot_means", calling_env = calling_env)
    if (!inherits(formula_eval, "formula")) {
      stop("plot_means(): First argument must be a formula like y ~ x1 + x2", call. = FALSE)
    }

  # If formula uses `$`, do not allow data= (use either one style or the other).
    .assert_no_dollar_with_data(formula_eval, data, func_name = "plot_means")

  # If formula uses `$` and data is NULL, normalize into a clean data+formula pair.
    if (is.null(data) && .formula_contains_dollar(formula_eval)) {
      normalized <- .normalize_dollar_formula_to_data_and_formula(
        formula = formula_eval,
        calling_env = calling_env,
        func_name = "plot_means"
      )
      formula_eval <- normalized$formula
      data <- normalized$data
    }
  
  # 2. Determine grouping variables (up to 3)
    lhs_expr <- formula_eval[[2L]]
    rhs_terms <- attr(stats::terms(formula_eval), "term.labels")
    if (length(rhs_terms) < 1) {
      stop("plot_means(): Formula must have at least one grouping variable: y ~ x1", call. = FALSE)
    }
    y_name <- .extract_formula_side_label(lhs_expr)
    x_names <- rhs_terms
    if (length(x_names) > 3) {
      stop("plot_means(): Currently supports up to 3 grouping variables: y ~ x1 + x2 + x3", call. = FALSE)
    }
    x1_name <- x_names[1]
    x2_name <- if (length(x_names) >= 2) x_names[2] else NULL
    x3_name <- if (length(x_names) >= 3) x_names[3] else NULL
  
  # 3. Validate display and testing options
  # 3.1 Validate label sizing argument
    if (!is.numeric(values.cex) || length(values.cex) != 1 || is.na(values.cex) || values.cex <= 0) {
      stop("plot_means(): 'values.cex' must be a single positive number", call. = FALSE)
    }
  
  # 3.2 Validate value label position
    if (!is.null(mc$values.pos) && !identical(mc$values.pos, quote(NULL))) {
      stop("plot_means(): `values.pos` was renamed to `values.align`", call. = FALSE)
    }
    values_align_effective <- match.arg(values.align, c("top", "middle", "bottom", "none"))
  
  # 3.3 Validate rounding argument for mean labels
    if (!is.numeric(values.round) || length(values.round) != 1 || is.na(values.round) || values.round < 0) {
      stop("plot_means(): 'values.round' must be a single non-negative number", call. = FALSE)
    }
    values.round <- as.integer(values.round)
  
  # 3.4 Validate tests argument
  #   Accept "auto", "none", or a custom comparison string like "3-1,4-2,(4-3)-(2-1)".
    if (is.character(tests) && length(tests) == 1 && !is.na(tests) && nzchar(tests)) {
      if (!(tests %in% c("auto", "none"))) {
        tests <- as.character(tests)
      } else {
        tests <- match.arg(tests, c("auto", "none"))
      }
    } else {
      stop("plot_means(): 'tests' must be a single string: \"auto\", \"none\", or custom comparisons like \"3-1, 4-2\"", call. = FALSE)
    }
  
  # 3.5 Validate p-value styling arguments
    if (!is.numeric(pvalue.cex) || length(pvalue.cex) != 1 || is.na(pvalue.cex) || pvalue.cex <= 0) {
      stop("plot_means(): 'pvalue.cex' must be a single positive number", call. = FALSE)
    }
    if (!is.character(pvalue.col) || length(pvalue.col) != 1 || is.na(pvalue.col) || !nzchar(pvalue.col)) {
      stop("plot_means(): 'pvalue.col' must be a single color name", call. = FALSE)
    }
  
  # 3.6 Validate ci.level argument (confidence interval level for ciL/ciH)
    if (!is.numeric(ci.level) || length(ci.level) != 1 || is.na(ci.level) || ci.level <= 0) {
      stop("plot_means(): 'ci.level' must be a single positive number (e.g., 95 or 0.95)", call. = FALSE)
    }
    if (ci.level < 1) ci.level <- ci.level * 100
    if (!is.finite(ci.level) || ci.level <= 0) {
      stop("plot_means(): 'ci.level' must be a single positive number (e.g., 95 or 0.95)", call. = FALSE)
    }
    if (ci.level >= 100) {
      stop("plot_means(): confidence can never be equal to or greater than 100%", call. = FALSE)
    }
  
  # 3.6 Validate buffer.top argument
    if (is.character(buffer.top) && length(buffer.top) == 1 && identical(buffer.top, "auto")) {
      buffer.top <- "auto"
    } else {
      if (!is.numeric(buffer.top) || length(buffer.top) != 1 || is.na(buffer.top) || buffer.top < 0) {
        stop("plot_means(): 'buffer.top' must be 'auto' or a single non-negative number (e.g., 0.3)", call. = FALSE)
      }
    }
  
  # 4. Resolve cluster argument (optional; for lm2(..., clusters=))
  #   Accept both `cluster=` (documented) and `clusters=` (alias) in the user call.
  #   If both are provided, error to avoid ambiguity.
    cluster_vec <- NULL
    has_cluster_expr <- !is.null(mc$cluster) && !identical(mc$cluster, quote(NULL))
    has_clusters_expr <- !is.null(mc$clusters) && !identical(mc$clusters, quote(NULL))
    if (isTRUE(has_cluster_expr) && isTRUE(has_clusters_expr)) {
      stop("plot_means(): use only one of `cluster=` or `clusters=` (not both)", call. = FALSE)
    }
    
    cluster_expr <- if (isTRUE(has_cluster_expr)) mc$cluster else if (isTRUE(has_clusters_expr)) mc$clusters else NULL
    if (!is.null(cluster_expr)) {
      cluster_resolved <- evaluate_variable_arguments(
        arg_expr = cluster_expr,
        arg_name = if (isTRUE(has_clusters_expr)) "clusters" else "cluster",
        data = data,
        calling_env = calling_env,
        func_name = "plot_means",
        allow_null = TRUE
      )
      cluster_vec <- cluster_resolved$value
      if (!is.null(cluster_vec)) {
        if (!is.atomic(cluster_vec) || length(cluster_vec) == 0) {
          stop("plot_means(): 'cluster' must be a vector (or a column name in 'data')", call. = FALSE)
        }
      }
    }
  
  list2(
    mc,
    formula_eval,
    y_name,
    x_names,
    x1_name,
    x2_name,
    x3_name,
    data,
    order,
    legend.title,
    col,
    col.text,
    cluster,
    cluster_vec,
    values.cex,
    values.align = values_align_effective,
    values.round,
    tests,
    pvalue.cex,
    pvalue.col,
    ci.level,
    buffer.top,
    calling_env
  )
}



#----------------------------------
# plot_means_params: derive plotting parameters (levels/order, colors, legend layout, buffer.top) ----
plot_means_params <- function(v, result, result_plot) {
  get_levels <- function(var_name, fallback_values) {
    if (!is.null(v$data) && is.data.frame(v$data) && var_name %in% names(v$data) && is.factor(v$data[[var_name]])) {
      return(levels(v$data[[var_name]]))
    }
    vals <- fallback_values
    if (is.null(vals) || !length(vals)) return(character(0))
    vals_chr <- as.character(vals)
    is_numeric_like <- grepl("^\\s*-?\\d+(\\.\\d+)?\\s*$", vals_chr)
    if (all(is_numeric_like)) {
      return(as.character(sort(unique(as.numeric(trimws(vals_chr))))))
    }
    sort(unique(vals_chr))
  }
  
  x1_levels <- get_levels(v$x1_name, if (v$x1_name %in% names(result_plot)) result_plot[[v$x1_name]] else NULL)
  x2_levels <- if (is.null(v$x2_name)) "All" else get_levels(v$x2_name, if (v$x2_name %in% names(result_plot)) result_plot[[v$x2_name]] else NULL)
  x3_levels <- if (is.null(v$x3_name)) "All" else get_levels(v$x3_name, if (v$x3_name %in% names(result_plot)) result_plot[[v$x3_name]] else NULL)
  
  # Apply ordering to x1 only (controls bar order/colors)
  #   `order` is interpreted as an instruction for the *within-block* series (x1):
  #   it changes the left-to-right bar order and, by extension, the mapping of
  #   `col_vec[i]` to the i-th x1 level. We do not reorder x2/x3 here because
  #   those define block layout on the x-axis (reordering them would change the
  #   overall panel/block arrangement rather than the series inside each block).
    if (!is.null(v$order)) {
      if (length(v$order) == 1 && is.numeric(v$order) && v$order == -1) {
        x1_levels <- rev(x1_levels)
      } else {
        missing_groups <- setdiff(x1_levels, v$order)
        extra_groups <- setdiff(v$order, x1_levels)
        
        if (length(missing_groups) > 0) {
          stop(
            sprintf("plot_means(): 'order' is missing x1 group(s): %s", paste(missing_groups, collapse = ", ")),
            call. = FALSE
          )
        }
        if (length(extra_groups) > 0) {
          warning(
            sprintf("plot_means(): 'order' contains x1 group(s) not in data: %s", paste(extra_groups, collapse = ", ")),
            call. = FALSE
          )
        }
        x1_levels <- v$order[v$order %in% x1_levels]
      }
    }
  
  # If there is a single grouping variable, reorder the returned table too
    if (length(v$x_names) == 1) {
      x1_in_result <- if (v$x1_name %in% names(result)) {
        as.character(result[[v$x1_name]])
      } else if ("group" %in% names(result)) {
        as.character(result$group)
      } else {
        NULL
      }
      
      if (!is.null(x1_in_result)) {
        row_idx <- match(x1_levels, x1_in_result)
        if (all(!is.na(row_idx))) {
          result <- result[row_idx, , drop = FALSE]
        }
      }
    }
  
  # Resolve colors for x1 levels
    k <- length(x1_levels)
    col_vec <- v$col
    if (is.null(col_vec)) {
      col_vec <- get.colors(k)
    } else {
      if (!is.character(col_vec)) stop("plot_means(): 'col' must be a character vector (color name(s))", call. = FALSE)
      if (length(col_vec) == 1) col_vec <- rep(col_vec, k)
      if (length(col_vec) != k) {
        stop(sprintf("plot_means(): 'col' must have length 1 or %d (number of x1 levels)", k), call. = FALSE)
      }
    }
  
  # Legend layout (used for auto buffer sizing too)
    max_legend_chars <- max(nchar(as.character(x1_levels)))
    if (!is.finite(max_legend_chars)) max_legend_chars <- 0
    legend_horiz <- isTRUE(max_legend_chars <= 10)
  
  # Resolve buffer.top when set to 'auto'
    is_default_interaction_scenario <- length(v$x_names) == 2 &&
      length(x1_levels) == 2 &&
      !is.null(v$x2_name) &&
      length(x2_levels) == 2 &&
      is.null(v$x3_name)
    
    # Treat custom diff-in-diff requests like the default interaction scenario for auto buffer sizing.
    # This ensures identical tests produce identical headroom regardless of whether they are requested
    # via tests="auto" or an explicit string like "(4-3)-(2-1),2-1,4-3".
    tests_str <- if (is.character(v$tests) && length(v$tests) == 1 && !is.na(v$tests)) gsub("\\s+", "", v$tests) else ""
    has_interaction_test <- nzchar(tests_str) && grepl("\\([0-9]+-[0-9]+\\)-\\([0-9]+-[0-9]+\\)", tests_str)
    
    show_interaction <- (identical(v$tests, "auto") && is_default_interaction_scenario) || has_interaction_test
    
    buffer_top_effective <- if (identical(v$buffer.top, "auto")) {
      base <- if (show_interaction) 0.3 else 0.2
      if (!legend_horiz) base <- base + 0.05
      base
    } else {
      v$buffer.top
    }
  
  # Label disambiguation overlap set and formatter
  #   When x2 and/or x3 are present, the same label text can appear in more than
  #   one role (e.g., an x1 level "A" and an x2 level "A"). If we print raw levels
  #   everywhere, axis labels become ambiguous. We precompute the set of labels
  #   that overlap across x1/x2/x3 and, only for those, format as "var=level"
  #   (e.g., "x2=A") to make the plot self-explanatory without being verbose.
    label_overlap_set <- character(0)
    if (!is.null(v$x2_name) || !is.null(v$x3_name)) {
      x1_lab <- as.character(x1_levels)
      x2_lab <- if (is.null(v$x2_name)) character(0) else as.character(x2_levels)
      x3_lab <- if (is.null(v$x3_name)) character(0) else as.character(x3_levels)
      
      present <- list(x1 = unique(x1_lab), x2 = unique(x2_lab), x3 = unique(x3_lab))
      all_labels <- unique(c(present$x1, present$x2, present$x3))
      label_overlap_set <- all_labels[sapply(all_labels, function(lbl) {
        sum(c(lbl %in% present$x1, lbl %in% present$x2, lbl %in% present$x3)) >= 2
      })]
    }
    
    format_level_label <- function(var_name, level) {
      level_chr <- as.character(level)
      if (!is.null(var_name) && nzchar(var_name) && level_chr %in% label_overlap_set) {
        return(paste0(var_name, "=", level_chr))
      }
      level_chr
    }
  
  list2(
    result = result,
    x1_levels,
    x2_levels,
    x3_levels,
    k,
    col_vec,
    legend_horiz,
    buffer_top_effective,
    format_level_label
  )
}



#----------------------------------
# Internal helpers used by multiple plot_means_* functions ----
plot_means_model_frame <- function(formula_eval, data, na.action, context = "model.frame") {
  mf <- tryCatch(
    model.frame(formula_eval, data = data, na.action = na.action),
    error = function(e) e
  )
  if (inherits(mf, "error")) {
    stop("plot_means(): failed to build model frame for ", context, ": ", conditionMessage(mf), call. = FALSE)
  }
  mf
}

plot_means_align_cluster <- function(v, mf) {
  if (is.null(v$cluster_vec)) return(NULL)
  if (length(v$cluster_vec) == nrow(mf)) return(v$cluster_vec)
  if (!is.null(v$data) && is.data.frame(v$data) && length(v$cluster_vec) == nrow(v$data)) {
    idx <- as.integer(rownames(mf))
    if (anyNA(idx)) idx <- seq_len(nrow(mf))
    return(v$cluster_vec[idx])
  }
  stop("plot_means(): 'cluster' length must match rows in 'data'", call. = FALSE)
}

plot_means_span <- function(y_min, y_max) {
  span <- (y_max - y_min)
  if (!is.finite(span) || span <= 0) span <- abs(y_max)
  if (!is.finite(span) || span <= 0) span <- 1
  span
}

plot_means_parse_tests <- function(tests_str) {
  if (!is.character(tests_str) || length(tests_str) != 1 || is.na(tests_str) || !nzchar(tests_str)) {
    stop("plot_means(): 'tests' must be a single non-empty string", call. = FALSE)
  }
  tests_str <- gsub("\\s+", "", tests_str)
  parts <- strsplit(tests_str, ",", fixed = TRUE)[[1]]
  parts <- parts[nzchar(parts)]
  if (!length(parts)) {
    stop("plot_means(): 'tests' is empty. Example: tests=\"3-1,4-2,(1+2)-(3+4),(4-3)-(2-1)\"", call. = FALSE)
  }
  
  parse_pair <- function(s) {
    m <- regexec("^([0-9]+)-([0-9]+)$", s)
    mm <- regmatches(s, m)[[1]]
    if (!length(mm)) return(NULL)
    a <- as.integer(mm[2])
    b <- as.integer(mm[3])
    if (!is.finite(a) || !is.finite(b) || a <= 0L || b <= 0L) return(NULL)
    list(a = a, b = b)
  }
  
  parse_sum <- function(s) {
    # one-or-more indices joined by '+', e.g. "1+3" or "2+1+4"
    if (!grepl("^[0-9]+(\\+[0-9]+)+$", s)) return(NULL)
    ss <- strsplit(s, "\\+", fixed = FALSE)[[1]]
    idx <- suppressWarnings(as.integer(ss))
    if (!length(idx) || any(!is.finite(idx)) || any(idx <= 0L)) return(NULL)
    idx
  }
  
  out <- vector("list", length(parts))
  for (i in seq_along(parts)) {
    p <- parts[i]
    # interaction: (a-b)-(c-d)
    m_int <- regexec("^\\(([0-9]+)-([0-9]+)\\)-\\(([0-9]+)-([0-9]+)\\)$", p)
    mm <- regmatches(p, m_int)[[1]]
    if (length(mm)) {
      a <- as.integer(mm[2]); b <- as.integer(mm[3]); c <- as.integer(mm[4]); d <- as.integer(mm[5])
      if (any(!is.finite(c(a, b, c, d))) || any(c(a, b, c, d) <= 0L)) {
        stop("plot_means(): invalid interaction test in 'tests': ", parts[i], call. = FALSE)
      }
      out[[i]] <- list(type = "interaction", a = a, b = b, c = c, d = d, expr = parts[i])
      next
    }
    # pooled: (a+b+...)-(c+d+...)
    m_pool <- regexec("^\\(([0-9\\+]+)\\)-\\(([0-9\\+]+)\\)$", p)
    mm <- regmatches(p, m_pool)[[1]]
    if (length(mm)) {
      left <- parse_sum(mm[2])
      right <- parse_sum(mm[3])
      if (is.null(left) || is.null(right)) {
        stop("plot_means(): invalid pooled test in 'tests': ", parts[i], call. = FALSE)
      }
      out[[i]] <- list(type = "pooled", left = left, right = right, expr = parts[i])
      next
    }
    pr <- parse_pair(p)
    if (!is.null(pr)) {
      out[[i]] <- list(type = "simple", a = pr$a, b = pr$b, expr = parts[i])
      next
    }
    stop("plot_means(): could not parse 'tests' entry: ", parts[i], ". Example: tests=\"3-1,4-2,(1+2)-(3+4),(4-3)-(2-1)\"", call. = FALSE)
  }
  out
}



#----------------------------------
# plot_means_compute: expand to a full x1/x2/x3 grid, compute CIs, and build bar/layout vectors ----
plot_means_compute <- function(v, params, result_plot, ci_level = 0.95) {
  x1_name <- v$x1_name
  x2_name <- v$x2_name
  x3_name <- v$x3_name
  x1_levels <- params$x1_levels
  x2_levels <- params$x2_levels
  x3_levels <- params$x3_levels
  k <- params$k
  col <- params$col_vec
  format_level_label <- params$format_level_label
  
  # Build complete grid of combinations, keeping empty slots
  #   Intent: preserve (x1,x2,x3) combinations with no observations so spacing and axes remain stable.
    result_key <- result_plot
    if (x1_name %in% names(result_key)) result_key[[x1_name]] <- as.character(result_key[[x1_name]])
    if (!is.null(x2_name) && x2_name %in% names(result_key)) result_key[[x2_name]] <- as.character(result_key[[x2_name]])
    if (!is.null(x3_name) && x3_name %in% names(result_key)) result_key[[x3_name]] <- as.character(result_key[[x3_name]])
    
    grid_df <- expand.grid(
      x1 = x1_levels,
      x2 = x2_levels,
      x3 = x3_levels,
      stringsAsFactors = FALSE
    )
    names(grid_df) <- c(x1_name, if (is.null(x2_name)) ".x2" else x2_name, if (is.null(x3_name)) ".x3" else x3_name)
    if (is.null(x2_name)) grid_df$.x2 <- "All"
    if (is.null(x3_name)) grid_df$.x3 <- "All"
    
    merge_by <- c(x1_name, if (is.null(x2_name)) ".x2" else x2_name, if (is.null(x3_name)) ".x3" else x3_name)
    if (is.null(x2_name)) result_key$.x2 <- "All"
    if (is.null(x3_name)) result_key$.x3 <- "All"
    
    merged <- merge(grid_df, result_key, by = merge_by, all.x = TRUE, sort = FALSE)
  
  # Confidence intervals for each plotted mean
  #   Default: per-cell t interval (one-sample t CI within each cell).
  #   Exceptions:
  #   - When clustering is used: regression-based CI via lm2(..., clusters=...)
  #   - 2x2 interaction scenario (tests="auto", x1/x2 both binary, no x3): regression-based CI
    ci_map <- data.frame(
      cell_key = character(0),
      lwr = numeric(0),
      upr = numeric(0),
      stringsAsFactors = FALSE
    )
    mf <- plot_means_model_frame(v$formula_eval, data = v$data, na.action = na.pass, context = "confidence intervals")
    if (nrow(mf) > 0) {
      df_m <- mf
      names(df_m)[1] <- ".__y"
      df_m[[x1_name]] <- as.character(df_m[[x1_name]])
      if (is.null(x2_name)) df_m$.x2 <- "All" else df_m[[x2_name]] <- as.character(df_m[[x2_name]])
      if (is.null(x3_name)) df_m$.x3 <- "All" else df_m[[x3_name]] <- as.character(df_m[[x3_name]])
      
      cluster_mf <- plot_means_align_cluster(v, df_m)
      if (!is.null(cluster_mf)) df_m$.__cluster <- cluster_mf
      
      x2_col <- if (is.null(x2_name)) ".x2" else x2_name
      x3_col <- if (is.null(x3_name)) ".x3" else x3_name
      
      complete_mask <- !is.na(df_m$.__y) &
        !is.na(df_m[[x1_name]]) &
        !is.na(df_m[[x2_col]]) &
        !is.na(df_m[[x3_col]])
      df_m <- df_m[complete_mask, , drop = FALSE]
      
      if (nrow(df_m) > 0) {
        df_m$cell_key <- paste(df_m[[x1_name]], df_m[[x2_col]], df_m[[x3_col]], sep = "|")
        df_m$cell_factor <- factor(df_m$cell_key)

      # Determine when to use regression-based CIs for the bars.
        is_2x2_interaction_scenario <- identical(v$tests, "auto") &&
          length(v$x_names) == 2 &&
          length(params$x1_levels) == 2 &&
          !is.null(v$x2_name) &&
          length(params$x2_levels) == 2 &&
          is.null(v$x3_name)
        
        use_regression_ci <- !is.null(v$cluster_vec) || isTRUE(is_2x2_interaction_scenario)
        
        if (!isTRUE(use_regression_ci)) {
        # Per-cell t interval for the mean (one-sample t CI within each cell)
          levels_cf <- levels(df_m$cell_factor)
          lwr <- rep(NA_real_, length(levels_cf))
          upr <- rep(NA_real_, length(levels_cf))
          alpha <- 1 - ci_level
          for (j in seq_along(levels_cf)) {
            key <- levels_cf[j]
            yy <- df_m$.__y[df_m$cell_key == key]
            yy <- yy[is.finite(yy)]
            n <- length(yy)
            if (n < 2) next
            m <- mean(yy)
            s <- stats::sd(yy)
            if (!is.finite(m) || !is.finite(s) || s < 0) next
            tcrit <- stats::qt(1 - alpha / 2, df = n - 1)
            half <- tcrit * s / sqrt(n)
            lwr[j] <- m - half
            upr[j] <- m + half
          }
          
          ci_map <- data.frame(
            cell_key = levels_cf,
            lwr = lwr,
            upr = upr,
            stringsAsFactors = FALSE
          )
        }
        
        if (isTRUE(use_regression_ci)) {
        # Regression-based CI from lm2 coef/vcov (robust or clustered SE)
          fit <- if (is.null(v$cluster_vec)) {
            lm2(.__y ~ 0 + cell_factor, data = df_m)
          } else {
            lm2(.__y ~ 0 + cell_factor, data = df_m, clusters = df_m$.__cluster)
          }
          
          beta <- stats::coef(fit)
          V <- stats::vcov(fit)
          if (!is.null(beta) && length(beta) > 0 && is.matrix(V) && nrow(V) == length(beta)) {
            levels_cf <- levels(df_m$cell_factor)
            newdata <- data.frame(cell_factor = factor(levels_cf, levels = levels_cf))
            X <- stats::model.matrix(~ 0 + cell_factor, data = newdata)
            
            b_idx <- match(colnames(X), names(beta))
            if (anyNA(b_idx)) {
              X <- X[, !is.na(b_idx), drop = FALSE]
              b_idx <- b_idx[!is.na(b_idx)]
            }
            beta_use <- beta[b_idx]
            V_use <- V[b_idx, b_idx, drop = FALSE]
            
            fit_hat <- as.numeric(X %*% beta_use)
            se_hat <- sqrt(pmax(0, diag(X %*% V_use %*% t(X))))
            
            df_fit <- fit$df
            if (length(df_fit) != 1) df_fit <- suppressWarnings(min(df_fit, na.rm = TRUE))
            if (!is.finite(df_fit) || df_fit <= 0) df_fit <- 1
            
            alpha <- 1 - ci_level
            tcrit <- stats::qt(1 - alpha / 2, df = df_fit)
            lwr <- fit_hat - tcrit * se_hat
            upr <- fit_hat + tcrit * se_hat
            
            ci_map <- data.frame(
              cell_key = levels_cf,
              lwr = lwr,
              upr = upr,
              stringsAsFactors = FALSE
            )
          }
        }
      }
    }
  
  # Bar positions and block labels
  #   Intent: compute x-positions once and return vectors used by the draw step.
  #   Performance: use a pre-keyed `merged` lookup + pre-allocation to avoid O(n^2) scans and repeated vector growth.
    gap_x2 <- 1
    gap_x3 <- 2
    bar_width <- 1
    bar_step <- 1
    
    x2_col <- if (is.null(x2_name)) ".x2" else x2_name
    x3_col <- if (is.null(x3_name)) ".x3" else x3_name
    
    # Pre-key merged once by "x1|x2|x3" for O(1) lookups inside the loop
    merged$cell_key <- paste(
      as.character(merged[[x1_name]]),
      as.character(merged[[x2_col]]),
      as.character(merged[[x3_col]]),
      sep = "|"
    )
    rownames(merged) <- merged$cell_key
    
    n_x2 <- length(x2_levels)
    n_x3 <- length(x3_levels)
    n_blocks <- n_x2 * n_x3
    n_cells  <- k * n_blocks
    
    block_centers <- numeric(n_blocks)
    block_labels  <- character(n_blocks)
    x3_section_centers <- numeric(n_x3)
    x3_section_labels  <- character(n_x3)
    
    # Pre-allocate per-bar vectors to max possible size; trim after
    x_lefts        <- numeric(n_cells)
    x_rights       <- numeric(n_cells)
    x_centers_drawn <- numeric(n_cells)
    cell_keys_drawn <- character(n_cells)
    n_total_drawn   <- numeric(n_cells)
    n_missing_drawn <- numeric(n_cells)
    heights         <- numeric(n_cells)
    cols            <- character(n_cells)
    drawn_idx       <- 0L
    
    x_pos    <- 1
    blk_idx  <- 0L
    
    for (x3_idx in seq_along(x3_levels)) {
      x3_val   <- x3_levels[x3_idx]
      x3_start <- x_pos
      
      for (x2_idx in seq_along(x2_levels)) {
        x2_val      <- x2_levels[x2_idx]
        blk_idx     <- blk_idx + 1L
        centers_block <- numeric(k)
        
        for (i in seq_len(k)) {
          x1_val  <- x1_levels[i]
          key     <- paste(as.character(x1_val), as.character(x2_val), as.character(x3_val), sep = "|")
          row_sel <- merged[key, , drop = FALSE]
          
          mean_val  <- if (!is.na(row_sel$mean[1])) row_sel$mean[1] else NA_real_
          n_total   <- if ("n.total"   %in% names(row_sel)) row_sel$n.total[1]   else NA_real_
          n_missing <- if ("n.missing" %in% names(row_sel)) row_sel$n.missing[1] else NA_real_
          
          x_center        <- x_pos + (i - 1L) * bar_step
          centers_block[i] <- x_center
          
          if (!is.na(mean_val)) {
            drawn_idx <- drawn_idx + 1L
            cell_keys_drawn[drawn_idx] <- key
            x_centers_drawn[drawn_idx] <- x_center
            n_total_drawn[drawn_idx]   <- n_total
            n_missing_drawn[drawn_idx] <- n_missing
            x_lefts[drawn_idx]         <- x_center - bar_width / 2
            x_rights[drawn_idx]        <- x_center + bar_width / 2
            heights[drawn_idx]         <- mean_val
            cols[drawn_idx]            <- col[i]
          }
        }
        
        block_centers[blk_idx] <- mean(centers_block)
        block_labels[blk_idx]  <- if (is.null(x2_name)) "" else format_level_label(x2_name, x2_val)
        
        x_pos <- x_pos + k * bar_step
        if (x2_idx < n_x2) x_pos <- x_pos + gap_x2
      }
      
      x3_end <- x_pos - bar_step
      x3_section_centers[x3_idx] <- (x3_start + x3_end) / 2
      x3_section_labels[x3_idx]  <- if (is.null(x3_name)) "" else format_level_label(x3_name, x3_val)
      if (x3_idx < n_x3) x_pos <- x_pos + gap_x3
    }
    
    # Trim pre-allocated vectors to actually drawn bars
    if (drawn_idx < n_cells) {
      keep <- seq_len(drawn_idx)
      cell_keys_drawn <- cell_keys_drawn[keep]
      x_centers_drawn <- x_centers_drawn[keep]
      n_total_drawn   <- n_total_drawn[keep]
      n_missing_drawn <- n_missing_drawn[keep]
      x_lefts         <- x_lefts[keep]
      x_rights        <- x_rights[keep]
      heights         <- heights[keep]
      cols            <- cols[keep]
    }
  
  list2(
    merged,
    ci_map,
    x_lefts,
    x_rights,
    x_centers_drawn,
    cell_keys_drawn,
    n_total_drawn,
    n_missing_drawn,
    heights,
    cols,
    block_centers,
    block_labels,
    x3_section_centers,
    x3_section_labels,
    x_pos,
    bar_width,
    bar_step
  )
}



#----------------------------------
# plot_means_draw: render bars, CIs, labels, legend, and p-value annotations in base graphics ----
plot_means_draw <- function(v,
                            params,
                            comp,
                            y_name,
                            x1_levels,
                            x2_name,
                            x3_name,
                            x1_name,
                            col,
                            legend.title,
                            col.text,
                            values.cex,
                            values.align,
                            values.round,
                            tests,
                            means_comparisons = NULL,
                            pvalue.cex,
                            pvalue.col,
                            buffer_top_effective,
                            ...) {
  heights <- comp$heights
  cols <- comp$cols
  x_lefts <- comp$x_lefts
  x_rights <- comp$x_rights
  x_centers_drawn <- comp$x_centers_drawn
  cell_keys_drawn <- comp$cell_keys_drawn
  n_total_drawn <- comp$n_total_drawn
  block_centers <- comp$block_centers
  block_labels <- comp$block_labels
  x3_section_centers <- comp$x3_section_centers
  x3_section_labels <- comp$x3_section_labels
  x_pos <- comp$x_pos
  bar_width <- comp$bar_width
  ci_map <- comp$ci_map
  legend_horiz <- params$legend_horiz
  
  dots <- list(...)
  # Establish plotting range.
  #   Bars are drawn from y=0 to `heights`, so we always include 0 in the y-limits.
  y_max_raw <- max(heights, na.rm = TRUE)
  if (!is.finite(y_max_raw)) y_max_raw <- 1
  y_max <- max(0, y_max_raw)
  y_min <- min(0, min(heights, na.rm = TRUE))
  if (!is.finite(y_min)) y_min <- 0
  
  if (nrow(ci_map) > 0) {
    lwr_f <- ci_map$lwr[is.finite(ci_map$lwr)]
    upr_f <- ci_map$upr[is.finite(ci_map$upr)]
    if (length(lwr_f)) y_min <- min(y_min, min(lwr_f))
    if (length(upr_f)) y_max <- max(y_max, max(upr_f), 0)
  }
  
  if (!"xlab" %in% names(dots)) dots$xlab <- ""
  
  # When x2 is present but x3 is not, show x2 name as x-axis label (centered)
    if (is.null(x3_name) && !is.null(x2_name) && nzchar(x2_name) && identical(dots$xlab, "")) {
      dots$xlab <- x2_name
    }
  if (!"ylab" %in% names(dots)) dots$ylab <- "Mean"
  if (!"main" %in% names(dots)) dots$main <- paste0("Means of ", y_name)
  
  y_span_data <- plot_means_span(y_min, y_max)

  # If everything is below 0, p-value brackets should be drawn "from below".
  #   Reserve extra bottom space so brackets/labels don't overlap bars (and so the
  #   interaction bracket, when present, isn't clipped at the bottom).
  all_below_zero <- (is.finite(y_max_raw) && y_max_raw <= 0)
  if (isTRUE(all_below_zero) && !identical(tests, "none") && !"ylim" %in% names(dots)) {
    is_interaction_scenario <- length(v$x_names) == 2 &&
      length(params$x1_levels) == 2 &&
      !is.null(v$x2_name) &&
      length(params$x2_levels) == 2 &&
      is.null(v$x3_name)
    extra_bottom <- if (isTRUE(is_interaction_scenario)) 0.28 else 0.18
    y_min <- y_min - extra_bottom * y_span_data
    y_span_data <- plot_means_span(y_min, y_max)
  }
  
  # Reserve a band below the lowest error whisker for n= labels.
  if (length(x_centers_drawn) && !"yaxt" %in% names(dots) && !"ylim" %in% names(dots)) {
    min_whisker <- y_min
    if (nrow(ci_map) > 0 && length(cell_keys_drawn) == length(x_centers_drawn)) {
      mi <- match(cell_keys_drawn, ci_map$cell_key)
      ok <- !is.na(mi)
      if (any(ok)) {
        lwr <- ci_map$lwr[mi[ok]]
        lwr <- lwr[is.finite(lwr)]
        if (length(lwr)) min_whisker <- min(min_whisker, min(lwr))
      }
    }
    # Minimal expansion: just enough room to place n= slightly below the lowest whisker.
    #   If all CIs are above 0, we still want n= below the bars (below 0), not inside them.
    min_anchor <- min(0, min_whisker)
    pad_n <- 0.04 * y_span_data
    y_n_target <- min_anchor - pad_n
    y_min_target <- y_n_target - pad_n
    y_min <- min(y_min, y_min_target)
    y_span_data <- plot_means_span(y_min, y_max)
  }

  # Extra headroom only when stacked p-value brackets require it.
  extra_top <- 0
  if (!identical(tests, "none") && !is.null(means_comparisons) && is.data.frame(means_comparisons) && nrow(means_comparisons) > 0) {
    assign_overlap_tier <- function(x0, x1) {
      n <- length(x0)
      if (!n) return(integer(0))
      left <- pmin(x0, x1)
      right <- pmax(x0, x1)
      ord <- order(left, right)
      tiers <- integer(n)
      tier_rights <- numeric(0)
      
      for (ii in ord) {
        placed <- FALSE
        if (length(tier_rights)) {
          for (t in seq_along(tier_rights)) {
            if (left[ii] > tier_rights[t]) {
              tiers[ii] <- t
              tier_rights[t] <- right[ii]
              placed <- TRUE
              break
            }
          }
        }
        if (!placed) {
          tiers[ii] <- length(tier_rights) + 1L
          tier_rights <- c(tier_rights, right[ii])
        }
      }
      tiers
    }
    
    assign_overlap_tier_in_order <- function(x0, x1) {
      # Greedy tier assignment in input order (stable), checking overlap against
      # all intervals already assigned to each tier.
      n <- length(x0)
      if (!n) return(integer(0))
      left <- pmin(x0, x1)
      right <- pmax(x0, x1)
      tiers <- integer(n)
      
      tier_lefts <- list()
      tier_rights <- list()
      
      overlaps_any <- function(l, r, L, R) {
        if (!length(L)) return(FALSE)
        # Treat touching endpoints as non-overlap so adjacent brackets can share a tier.
        any(!(r <= L | l >= R))
      }
      
      for (i in seq_len(n)) {
        l <- left[i]
        r <- right[i]
        if (!is.finite(l) || !is.finite(r)) next
        
        placed <- FALSE
        if (length(tier_lefts)) {
          for (t in seq_along(tier_lefts)) {
            if (!isTRUE(overlaps_any(l, r, tier_lefts[[t]], tier_rights[[t]]))) {
              tiers[i] <- t
              tier_lefts[[t]] <- c(tier_lefts[[t]], l)
              tier_rights[[t]] <- c(tier_rights[[t]], r)
              placed <- TRUE
              break
            }
          }
        }
        if (!placed) {
          tiers[i] <- length(tier_lefts) + 1L
          tier_lefts[[length(tier_lefts) + 1L]] <- l
          tier_rights[[length(tier_rights) + 1L]] <- r
        }
      }
      tiers
    }
    
    if (all(c("col1", "col2") %in% names(means_comparisons))) {
      simple_rows <- means_comparisons[is.na(means_comparisons$col3) & is.finite(means_comparisons$p.value), , drop = FALSE]
      
      x0 <- rep(NA_real_, nrow(simple_rows))
      x1 <- rep(NA_real_, nrow(simple_rows))
      for (r in seq_len(nrow(simple_rows))) {
        is_pooled <- ("test_type" %in% names(simple_rows)) && identical(as.character(simple_rows$test_type[r]), "pooled")
        if (isTRUE(is_pooled) && all(c("cols_left", "cols_right") %in% names(simple_rows))) {
          left <- suppressWarnings(as.integer(strsplit(as.character(simple_rows$cols_left[r]), ",", fixed = TRUE)[[1]]))
          right <- suppressWarnings(as.integer(strsplit(as.character(simple_rows$cols_right[r]), ",", fixed = TRUE)[[1]]))
          idx <- c(left, right)
          idx <- idx[is.finite(idx)]
          if (length(idx)) {
            xx <- x_centers_drawn[idx]
            x0[r] <- min(xx, na.rm = TRUE)
            x1[r] <- max(xx, na.rm = TRUE)
          }
        } else {
          i1 <- as.integer(simple_rows$col1[r])
          i2 <- as.integer(simple_rows$col2[r])
          if (is.finite(i1) && is.finite(i2)) {
            x0[r] <- x_centers_drawn[i1]
            x1[r] <- x_centers_drawn[i2]
          }
        }
      }
    } else {
      simple_rows <- means_comparisons[is.character(means_comparisons$group2) & nzchar(means_comparisons$group2) & is.finite(means_comparisons$p.value), , drop = FALSE]
      # Build a temporary label -> bar index map (idx_by_label is created later for drawing).
      group_label_tmp <- function(x1, x2 = NULL, x3 = NULL) {
        parts <- character(0)
        if (!is.null(x3) && nzchar(x3)) parts <- c(parts, x3)
        if (!is.null(x2) && nzchar(x2)) parts <- c(parts, x2)
        parts <- c(parts, x1)
        paste(parts, collapse = "_")
      }
      key_parts <- strsplit(cell_keys_drawn, "\\|")
      x1_vals <- vapply(key_parts, function(z) z[1], character(1))
      x2_vals <- vapply(key_parts, function(z) z[2], character(1))
      x3_vals <- vapply(key_parts, function(z) z[3], character(1))
      x2_use <- if (is.null(v$x2_name)) rep(list(NULL), length(x1_vals)) else as.list(x2_vals)
      x3_use <- if (is.null(v$x3_name)) rep(list(NULL), length(x1_vals)) else as.list(x3_vals)
      bar_labels <- mapply(group_label_tmp, x1 = x1_vals, x2 = x2_use, x3 = x3_use, USE.NAMES = FALSE)
      idx_by_label_tmp <- seq_along(bar_labels)
      names(idx_by_label_tmp) <- bar_labels
      
      i1 <- idx_by_label_tmp[as.character(simple_rows$group1)]
      i2 <- idx_by_label_tmp[as.character(simple_rows$group2)]
      x0 <- x_centers_drawn[as.integer(i1)]
      x1 <- x_centers_drawn[as.integer(i2)]
    }
    ok <- is.finite(x0) & is.finite(x1)
    if (any(ok)) {
      tiers <- assign_overlap_tier(x0[ok], x1[ok])
      step <- 0.05 * y_span_data
      extra_top <- (max(tiers, na.rm = TRUE) - 1L) * step
      if (!is.finite(extra_top) || extra_top < 0) extra_top <- 0
    }
  }
  
  ylim_top <- y_max + buffer_top_effective * y_span_data + extra_top
  if (!"ylim" %in% names(dots)) dots$ylim <- c(y_min, ylim_top)
  if (!"xlim" %in% names(dots)) dots$xlim <- c(0, x_pos)
  if (!"las" %in% names(dots)) dots$las <- 1
  if (!"font.lab" %in% names(dots)) dots$font.lab <- 2
  if (!"cex.lab" %in% names(dots)) dots$cex.lab <- 1.2
  if (!"cex.main" %in% names(dots)) dots$cex.main <- 1.38
  
  dots$type <- "n"
  dots$xaxt <- "n"
  
  user_provided_yaxt <- "yaxt" %in% names(dots)
  if (!user_provided_yaxt) dots$yaxt <- "n"
  
  plot_args <- c(list(x = 0, y = 0), dots)
  do.call(plot, plot_args)

  abline(h = 0, col = "gray80")
  
  if (!user_provided_yaxt && (is.null(dots$axes) || isTRUE(dots$axes))) {
    y_ticks <- pretty(c(y_min, y_max), n = 5)
    y_ticks <- y_ticks[y_ticks >= y_min & y_ticks <= y_max + 1e-9]
    axis(2, at = y_ticks, las = 1)
  }
  
  if (length(heights)) rect(x_lefts, 0, x_rights, heights, col = cols, border = cols)
  
  lum <- function(col_one) {
    rgb <- grDevices::col2rgb(col_one)
    as.numeric((0.299 * rgb[1, ] + 0.587 * rgb[2, ] + 0.114 * rgb[3, ]) / 255)
  }
  text_cols <- ifelse(sapply(cols, lum) < 0.5, "white", "black")
  
  if (nrow(ci_map) > 0 && length(x_centers_drawn) == length(cell_keys_drawn)) {
    eb_col <- "black"
    cap <- bar_width * 0.08
    match_idx <- match(cell_keys_drawn, ci_map$cell_key)
    ok <- !is.na(match_idx)
    if (any(ok)) {
      lwr <- ci_map$lwr[match_idx[ok]]
      upr <- ci_map$upr[match_idx[ok]]
      x_ok <- x_centers_drawn[ok]
      
      ok2 <- is.finite(lwr) & is.finite(upr)
      if (any(ok2)) {
        x_ok <- x_ok[ok2]
        lwr <- lwr[ok2]
        upr <- upr[ok2]
        
        segments(x_ok, lwr, x_ok, upr, col = eb_col, lwd = 1)
        segments(x_ok - cap, lwr, x_ok + cap, lwr, col = eb_col, lwd = 1)
        segments(x_ok - cap, upr, x_ok + cap, upr, col = eb_col, lwd = 1)
      }
    }
  }
  
  if (!identical(tests, "none") && !is.null(means_comparisons) && is.data.frame(means_comparisons) && nrow(means_comparisons) > 0) {
    pvalue_line <- function(x0, x1, y, p, from_below = FALSE) {
      n <- max(length(x0), length(x1), length(y), length(p))
      x0 <- rep_len(x0, n)
      x1 <- rep_len(x1, n)
      y <- rep_len(y, n)
      p <- rep_len(p, n)
      
      y_span <- diff(par("usr")[3:4])
      tick <- 0.015 * y_span
      
      segments(x0, y, x1, y, col = pvalue.col)
      if (isTRUE(from_below)) {
        segments(x0, y, x0, y + tick, col = pvalue.col)
        segments(x1, y, x1, y + tick, col = pvalue.col)
      } else {
        segments(x0, y - tick, x0, y, col = pvalue.col)
        segments(x1, y - tick, x1, y, col = pvalue.col)
      }
      
      x_mid <- (x0 + x1) / 2
      for (i in seq_len(n)) {
        p_txt <- format_p_expr(p[i], digits = 3)
        if (!is.null(p_txt)) {
          y_txt <- if (isTRUE(from_below)) y[i] - 0.03 * y_span else y[i] + 0.03 * y_span
          text2(x_mid[i], y_txt, p_txt, bg = "white", cex = pvalue.cex, col = pvalue.col, pad = 0, pad_v = 0)
        }
      }
      
      invisible(NULL)
    }

    pvalue_line_pooled <- function(xL0, xL1, xR0, xR1, y, p, from_below = FALSE) {
      # Draw a two-level bracket for pooled comparisons:
      # - a small bracket over the pooled-left bars, and one over pooled-right bars (y_sub)
      # - a main bracket connecting the two pooled groups (y)
      y_span <- diff(par("usr")[3:4])
      sub_drop <- 0.03 * y_span
      y_sub <- if (isTRUE(from_below)) y + sub_drop else y - sub_drop
      
      # Small grouping lines for each pooled group (flat: no end ticks)
      segments(xL0, y_sub, xL1, y_sub, col = pvalue.col)
      segments(xR0, y_sub, xR1, y_sub, col = pvalue.col)
      
      # Connect pooled-group midpoints up to the main bracket
      xLm <- (xL0 + xL1) / 2
      xRm <- (xR0 + xR1) / 2
      if (isTRUE(from_below)) {
        segments(xLm, y_sub, xLm, y, col = pvalue.col)
        segments(xRm, y_sub, xRm, y, col = pvalue.col)
      } else {
        segments(xLm, y, xLm, y_sub, col = pvalue.col)
        segments(xRm, y, xRm, y_sub, col = pvalue.col)
      }
      
      # Main bracket + p-value label
      pvalue_line(x0 = xLm, x1 = xRm, y = y, p = p, from_below = from_below)
      invisible(NULL)
    }
    
    format_p_expr <- function(p, digits = 3) {
      if (!is.finite(p)) return(NULL)
      min_threshold <- 10^(-digits)
      max_threshold <- 1 - 10^(-digits)
      min_str <- format(min_threshold, nsmall = digits, scientific = FALSE)
      max_str <- format(max_threshold, nsmall = digits, scientific = FALSE)
      
      if (p < min_threshold) return(as.expression(parse(text = paste0("italic(p) < ", min_str))))
      if (p > max_threshold) return(as.expression(parse(text = paste0("italic(p) > ", max_str))))
      
      p_clean <- round(p, digits)
      p_str <- format(p_clean, nsmall = digits, scientific = FALSE)
      as.expression(parse(text = paste0("italic(p) == ", p_str)))
    }
    
    # Map drawn bars to group labels used by plot_means_compute_pvalues().
    group_label <- function(x1, x2 = NULL, x3 = NULL) {
      parts <- character(0)
      if (!is.null(x3) && nzchar(x3)) parts <- c(parts, x3)
      if (!is.null(x2) && nzchar(x2)) parts <- c(parts, x2)
      parts <- c(parts, x1)
      paste(parts, collapse = "_")
    }
    
    if (!length(cell_keys_drawn)) return(invisible(NULL))
    key_parts <- strsplit(cell_keys_drawn, "\\|")
    x1_vals <- vapply(key_parts, function(z) z[1], character(1))
    x2_vals <- vapply(key_parts, function(z) z[2], character(1))
    x3_vals <- vapply(key_parts, function(z) z[3], character(1))
    x2_use <- if (is.null(v$x2_name)) rep(list(NULL), length(x1_vals)) else as.list(x2_vals)
    x3_use <- if (is.null(v$x3_name)) rep(list(NULL), length(x1_vals)) else as.list(x3_vals)
    bar_labels <- mapply(group_label, x1 = x1_vals, x2 = x2_use, x3 = x3_use, USE.NAMES = FALSE)
    idx_by_label <- seq_along(bar_labels)
    names(idx_by_label) <- bar_labels
    
    from_below <- (is.finite(max(heights, na.rm = TRUE)) && max(heights, na.rm = TRUE) <= 0)
    y_span <- diff(par("usr")[3:4])
    
    assign_overlap_tier <- function(x0, x1) {
      n <- length(x0)
      if (!n) return(integer(0))
      left <- pmin(x0, x1)
      right <- pmax(x0, x1)
      ord <- order(left, right)
      tiers <- integer(n)
      tier_rights <- numeric(0)
      
      for (ii in ord) {
        placed <- FALSE
        if (length(tier_rights)) {
          for (t in seq_along(tier_rights)) {
            if (left[ii] > tier_rights[t]) {
              tiers[ii] <- t
              tier_rights[t] <- right[ii]
              placed <- TRUE
              break
            }
          }
        }
        if (!placed) {
          tiers[ii] <- length(tier_rights) + 1L
          tier_rights <- c(tier_rights, right[ii])
        }
      }
      tiers
    }

    has_custom_cols <- all(c("col1", "col2") %in% names(means_comparisons))
    if (isTRUE(has_custom_cols)) {
      simple_rows <- means_comparisons[is.na(means_comparisons$col3) & is.finite(means_comparisons$p.value), , drop = FALSE]
      int_rows <- means_comparisons[is.finite(means_comparisons$col3) & is.finite(means_comparisons$col4) & is.finite(means_comparisons$p.value), , drop = FALSE]
      
      # 1) Compute a shared baseline y-position for all custom tests.
        involved_idx <- integer(0)
        if (nrow(simple_rows) > 0) {
          involved_idx <- unique(c(involved_idx, as.integer(simple_rows$col1), as.integer(simple_rows$col2)))
          if ("test_type" %in% names(simple_rows) && "cols_left" %in% names(simple_rows) && "cols_right" %in% names(simple_rows)) {
            pooled_idx <- which(as.character(simple_rows$test_type) == "pooled")
            if (length(pooled_idx)) {
              for (rr in pooled_idx) {
                left <- suppressWarnings(as.integer(strsplit(as.character(simple_rows$cols_left[rr]), ",", fixed = TRUE)[[1]]))
                right <- suppressWarnings(as.integer(strsplit(as.character(simple_rows$cols_right[rr]), ",", fixed = TRUE)[[1]]))
                involved_idx <- unique(c(involved_idx, left, right))
              }
            }
          }
        }
        if (nrow(int_rows) > 0) {
          involved_idx <- unique(c(involved_idx, as.integer(int_rows$col1), as.integer(int_rows$col2), as.integer(int_rows$col3), as.integer(int_rows$col4)))
        }
        involved_idx <- involved_idx[is.finite(involved_idx)]
        if (!length(involved_idx)) return(invisible(NULL))
        if (any(involved_idx < 1L) || any(involved_idx > length(x_centers_drawn))) {
          stop("plot_means(): invalid column index in custom tests (out of range)", call. = FALSE)
        }
        
        y_extreme <- if (isTRUE(from_below)) min(heights[involved_idx], na.rm = TRUE) else max(heights[involved_idx], na.rm = TRUE)
        if (nrow(ci_map) > 0) {
          k_all <- cell_keys_drawn[involved_idx]
          mi_all <- match(k_all, ci_map$cell_key)
          mi_all <- mi_all[!is.na(mi_all)]
          if (length(mi_all)) {
            if (isTRUE(from_below)) {
              y_extreme <- min(y_extreme, ci_map$lwr[mi_all], na.rm = TRUE)
            } else {
              y_extreme <- max(y_extreme, ci_map$upr[mi_all], na.rm = TRUE)
            }
          }
        }
        y_base <- if (isTRUE(from_below)) y_extreme - 0.02 * y_span else y_extreme + 0.02 * y_span
        
      # 2) Tier all custom tests together based on their x-spans (keep current x definitions).
        x0 <- rep(NA_real_, nrow(simple_rows))
        x1 <- rep(NA_real_, nrow(simple_rows))
        if (nrow(simple_rows) > 0) {
          for (r in seq_len(nrow(simple_rows))) {
            is_pooled <- ("test_type" %in% names(simple_rows)) && identical(as.character(simple_rows$test_type[r]), "pooled")
            if (isTRUE(is_pooled) && all(c("cols_left", "cols_right") %in% names(simple_rows))) {
              left <- suppressWarnings(as.integer(strsplit(as.character(simple_rows$cols_left[r]), ",", fixed = TRUE)[[1]]))
              right <- suppressWarnings(as.integer(strsplit(as.character(simple_rows$cols_right[r]), ",", fixed = TRUE)[[1]]))
              idx <- c(left, right)
              idx <- idx[is.finite(idx)]
              if (length(idx)) {
                xx <- x_centers_drawn[idx]
                x0[r] <- min(xx, na.rm = TRUE)
                x1[r] <- max(xx, na.rm = TRUE)
              }
            } else {
              i1 <- as.integer(simple_rows$col1[r])
              i2 <- as.integer(simple_rows$col2[r])
              if (is.finite(i1) && is.finite(i2)) {
                x0[r] <- x_centers_drawn[i1]
                x1[r] <- x_centers_drawn[i2]
              }
            }
          }
        }
        
        x0_int <- rep(NA_real_, nrow(int_rows))
        x1_int <- rep(NA_real_, nrow(int_rows))
        if (nrow(int_rows) > 0) {
          for (r in seq_len(nrow(int_rows))) {
            a <- as.integer(int_rows$col1[r]); b <- as.integer(int_rows$col2[r])
            c <- as.integer(int_rows$col3[r]); d <- as.integer(int_rows$col4[r])
            if (any(!is.finite(c(a, b, c, d)))) next
            x_ab <- mean(c(x_centers_drawn[a], x_centers_drawn[b]))
            x_cd <- mean(c(x_centers_drawn[c], x_centers_drawn[d]))
            x0_int[r] <- min(x_ab, x_cd, na.rm = TRUE)
            x1_int[r] <- max(x_ab, x_cd, na.rm = TRUE)
          }
        }
        
        x0_all <- c(x0, x0_int)
        x1_all <- c(x1, x1_int)
        tiers_all <- assign_overlap_tier_in_order(x0_all, x1_all)
        step <- 0.05 * y_span
        
      # 3) Draw simple/pooled tests at y determined solely by tier.
        if (nrow(simple_rows) > 0) {
          for (r in seq_len(nrow(simple_rows))) {
            off <- (tiers_all[r] - 1L) * step
            y <- if (isTRUE(from_below)) y_base - off else y_base + off
            if (!is.finite(x0[r]) || !is.finite(x1[r])) next
            
            is_pooled <- ("test_type" %in% names(simple_rows)) && identical(as.character(simple_rows$test_type[r]), "pooled")
            if (isTRUE(is_pooled) && all(c("cols_left", "cols_right") %in% names(simple_rows))) {
              left <- suppressWarnings(as.integer(strsplit(as.character(simple_rows$cols_left[r]), ",", fixed = TRUE)[[1]]))
              right <- suppressWarnings(as.integer(strsplit(as.character(simple_rows$cols_right[r]), ",", fixed = TRUE)[[1]]))
              left <- left[is.finite(left)]
              right <- right[is.finite(right)]
              if (!length(left) || !length(right)) next
              
              xL <- x_centers_drawn[left]
              xR <- x_centers_drawn[right]
              xL0 <- min(xL, na.rm = TRUE); xL1 <- max(xL, na.rm = TRUE)
              xR0 <- min(xR, na.rm = TRUE); xR1 <- max(xR, na.rm = TRUE)
              pvalue_line_pooled(xL0 = xL0, xL1 = xL1, xR0 = xR0, xR1 = xR1, y = y, p = simple_rows$p.value[r], from_below = from_below)
            } else {
              pvalue_line(x0 = x0[r], x1 = x1[r], y = y, p = simple_rows$p.value[r], from_below = from_below)
            }
          }
        }
      
      # 4) Draw interaction tests at y determined solely by tier (aligned with simple/pooled).
        if (nrow(int_rows) > 0) {
          n_simple <- nrow(simple_rows)
          for (r in seq_len(nrow(int_rows))) {
            a <- as.integer(int_rows$col1[r]); b <- as.integer(int_rows$col2[r])
            c <- as.integer(int_rows$col3[r]); d <- as.integer(int_rows$col4[r])
            if (any(!is.finite(c(a, b, c, d)))) next
            x_ab <- mean(c(x_centers_drawn[a], x_centers_drawn[b]))
            x_cd <- mean(c(x_centers_drawn[c], x_centers_drawn[d]))
            p_txt <- format_p_expr(int_rows$p.value[r], digits = 3)
            if (is.null(p_txt)) next
            
            tier_idx <- n_simple + r
            off <- (tiers_all[tier_idx] - 1L) * step
            y_anchor <- if (isTRUE(from_below)) y_base - off else y_base + off
            
            if (isTRUE(from_below)) {
              y_line <- y_anchor - 0.06 * y_span
              y_lab <- y_line - 0.025 * y_span
              y_line_from <- y_anchor - 0.045 * y_span
              segments(x_ab, y_line, x_ab, y_line_from, col = pvalue.col)
              segments(x_cd, y_line, x_cd, y_line_from, col = pvalue.col)
            } else {
              y_line <- y_anchor + 0.06 * y_span
              y_lab <- y_line + 0.025 * y_span
              y_line_from <- y_anchor + 0.045 * y_span
              segments(x_ab, y_line_from, x_ab, y_line, col = pvalue.col)
              segments(x_cd, y_line_from, x_cd, y_line, col = pvalue.col)
            }
            segments(x_ab, y_line, x_cd, y_line, col = pvalue.col)
            text2(mean(c(x_ab, x_cd)), y_lab, p_txt, bg = "white", cex = pvalue.cex, col = pvalue.col, pad = 0, pad_v = 0)
          }
        }
      
      invisible(NULL)
    }
    
    # Simple-effect comparisons: rows with a non-empty group2 and non-missing p.value
    simple_rows <- means_comparisons[is.character(means_comparisons$group2) & nzchar(means_comparisons$group2), , drop = FALSE]
    simple_rows <- simple_rows[is.finite(simple_rows$p.value), , drop = FALSE]
    if (nrow(simple_rows) > 0) {
      # Shared y-position: 2% beyond the most extreme CI among all involved bars
      involved <- unique(c(as.character(simple_rows$group1), as.character(simple_rows$group2)))
      idx_all <- idx_by_label[involved]
      idx_all <- idx_all[is.finite(idx_all)]
      y_simple <- NA_real_
      if (length(idx_all)) {
        y_extreme <- if (isTRUE(from_below)) min(heights[idx_all], na.rm = TRUE) else max(heights[idx_all], na.rm = TRUE)
        if (nrow(ci_map) > 0) {
          k_all <- cell_keys_drawn[idx_all]
          mi_all <- match(k_all, ci_map$cell_key)
          mi_all <- mi_all[!is.na(mi_all)]
          if (length(mi_all)) {
            if (isTRUE(from_below)) {
              y_extreme <- min(y_extreme, ci_map$lwr[mi_all], na.rm = TRUE)
            } else {
              y_extreme <- max(y_extreme, ci_map$upr[mi_all], na.rm = TRUE)
            }
          }
        }
        y_simple <- if (isTRUE(from_below)) y_extreme - 0.02 * y_span else y_extreme + 0.02 * y_span
      }
      
      if (is.finite(y_simple)) {
        x0 <- x_centers_drawn[as.integer(idx_by_label[as.character(simple_rows$group1)])]
        x1 <- x_centers_drawn[as.integer(idx_by_label[as.character(simple_rows$group2)])]
        tiers <- assign_overlap_tier(x0, x1)
        step <- 0.05 * y_span
        for (r in seq_len(nrow(simple_rows))) {
          i1 <- idx_by_label[as.character(simple_rows$group1[r])]
          i2 <- idx_by_label[as.character(simple_rows$group2[r])]
          if (!is.finite(i1) || !is.finite(i2)) next
          off <- (tiers[r] - 1L) * step
          y <- if (isTRUE(from_below)) y_simple - off else y_simple + off
          pvalue_line(x0 = x_centers_drawn[i1], x1 = x_centers_drawn[i2], y = y, p = simple_rows$p.value[r], from_below = from_below)
        }
      }
    }
    
    # Interaction comparison: a single row labeled interaction(x1:x2)
    int_rows <- means_comparisons[is.character(means_comparisons$group1) & grepl("^interaction\\(", means_comparisons$group1), , drop = FALSE]
    int_rows <- int_rows[is.finite(int_rows$p.value), , drop = FALSE]
    if (nrow(int_rows) > 0 && !is.null(v$x2_name) && length(params$x2_levels) == 2 && length(x1_levels) == 2) {
      p_int_txt <- format_p_expr(int_rows$p.value[1], digits = 3)
      if (!is.null(p_int_txt)) {
        x2_levels <- params$x2_levels
        x_mids <- rep(NA_real_, length(x2_levels))
        for (j in seq_along(x2_levels)) {
          g1 <- group_label(x1_levels[1], x2 = x2_levels[j])
          g2 <- group_label(x1_levels[2], x2 = x2_levels[j])
          i1 <- idx_by_label[g1]
          i2 <- idx_by_label[g2]
          if (!is.finite(i1) || !is.finite(i2)) next
          x_mids[j] <- mean(c(x_centers_drawn[i1], x_centers_drawn[i2]))
        }
        if (all(is.finite(x_mids))) {
          # Place interaction bracket offset from the simple-effect line, if present; otherwise anchor on CI extreme
          y_anchor <- if (exists("y_simple", inherits = FALSE) && is.finite(y_simple)) y_simple else {
            if (nrow(ci_map) > 0) {
              y_extreme <- if (isTRUE(from_below)) min(ci_map$lwr, na.rm = TRUE) else max(ci_map$upr, na.rm = TRUE)
              if (is.finite(y_extreme)) {
                if (isTRUE(from_below)) y_extreme - 0.02 * y_span else y_extreme + 0.02 * y_span
              } else {
                0
              }
            } else {
              0
            }
          }
          if (exists("tiers", inherits = FALSE) && length(tiers)) {
            max_off <- (max(tiers, na.rm = TRUE) - 1L) * (0.05 * y_span)
            y_anchor <- if (isTRUE(from_below)) y_anchor - max_off else y_anchor + max_off
          }
          if (isTRUE(from_below)) {
            y_line <- y_anchor - 0.06 * y_span
            y_lab <- y_line - 0.025 * y_span
            y_line_from <- y_anchor - 0.045 * y_span
            segments(x_mids[1], y_line, x_mids[1], y_line_from, col = pvalue.col)
            segments(x_mids[2], y_line, x_mids[2], y_line_from, col = pvalue.col)
          } else {
            y_line <- y_anchor + 0.06 * y_span
            y_lab <- y_line + 0.025 * y_span
            y_line_from <- y_anchor + 0.045 * y_span
            segments(x_mids[1], y_line_from, x_mids[1], y_line, col = pvalue.col)
            segments(x_mids[2], y_line_from, x_mids[2], y_line, col = pvalue.col)
          }
          segments(x_mids[1], y_line, x_mids[2], y_line, col = pvalue.col)
          text2(mean(x_mids), y_lab, p_int_txt, bg = "white", cex = pvalue.cex, col = pvalue.col, pad = 0, pad_v = 0)
        }
      }
    }
  }
  
  if (length(x_centers_drawn) > 0 && length(n_total_drawn) == length(x_centers_drawn)) {
    usr <- par("usr")
    y_span <- (usr[4] - usr[3])
    
    # Place sample sizes just below the lowest CI whisker (with a small pad), so we
    # don't have to create a large empty band under the plot.
    min_whisker <- 0
    if (nrow(ci_map) > 0 && length(cell_keys_drawn) == length(x_centers_drawn)) {
      mi <- match(cell_keys_drawn, ci_map$cell_key)
      ok <- !is.na(mi)
      if (any(ok)) {
        lwr <- ci_map$lwr[mi[ok]]
        lwr <- lwr[is.finite(lwr)]
        if (length(lwr)) min_whisker <- min(lwr, na.rm = TRUE)
      }
    }
    pad_n <- 0.04 * y_span
    min_anchor <- min(0, min_whisker)
    y_n <- min_anchor - pad_n
    y_n <- max(y_n, usr[3] + pad_n)
    
    n_vals <- ifelse(is.finite(n_total_drawn), n_total_drawn, NA)
    labels <- paste0("n=", n_vals)
    n_cex <- 0.9 * values.cex
    graphics::text(x_centers_drawn, y_n, labels, col = pvalue.col, cex = n_cex, adj = c(0.5, 0))
  }
  
  if (!identical(values.align, "none") && length(x_centers_drawn) > 0) {
    usr <- par("usr")
    pad_top <- 0.03 * (usr[4] - usr[3])
    pad_bot <- 0.06 * (usr[4] - usr[3])
    
    fmt <- formatC(heights, format = "f", digits = values.round)
    mean_labels <- if (identical(values.align, "bottom")) paste0("M=", fmt) else fmt
    
    neg <- is.finite(heights) & heights < 0
    y_mean <- if (identical(values.align, "top")) {
      # Place labels near the mean endpoint of each bar (not near the baseline at 0).
      # Positive bars end at `heights` above 0; negative bars end at `heights` below 0.
      ifelse(neg, heights + pad_top, heights - pad_top)
    } else if (identical(values.align, "middle")) {
      heights / 2
    } else if (identical(values.align, "bottom")) {
      # "bottom" still means the label is pulled toward the bar body, but anchored
      # near the mean endpoint rather than near 0 for negative bars.
      ifelse(neg, heights + pad_bot, heights - pad_bot)
    } else {
      rep(0, length(heights))
    }
    
    text2(
      x      = x_centers_drawn,
      y      = y_mean,
      labels = mean_labels,
      bg     = cols,
      cex    = values.cex,
      col    = text_cols,
      pad    = 0,
      pad_v  = 0
    )
  }
  
  usr <- par("usr")
  x_left <- usr[1] - 0.03 * (usr[2] - usr[1])
  old_xpd <- par("xpd")
  par(xpd = NA)
  on.exit(par(xpd = old_xpd), add = TRUE)
  
  if (is.null(x2_name) || !nzchar(x2_name)) {
    axis(1, at = block_centers, labels = block_labels, las = 1)
  } else {
    if (is.null(x3_name) || !nzchar(x3_name)) {
      axis(1, at = block_centers, labels = block_labels, las = 1)
    } else {
      axis(1, at = block_centers, labels = FALSE, las = 1)
      mtext(x2_name, side = 1, at = x_left, adj = 0, line = 1, font = 2, cex = 1.17)
      mtext(block_labels, side = 1, at = block_centers, line = 1, cex = 1.17)
      
      mtext(x3_name, side = 1, at = x_left, adj = 0, line = 2.5, font = 2, cex = 1.17)
      mtext(x3_section_labels, side = 1, at = x3_section_centers, line = 2.5, font = 2, cex = 1.17)
    }
  }
  
  if (params$k > 1) {
    w <- strwidth(x1_levels, cex = 1.3)
    w <- w[is.finite(w)]
    tw <- if (length(w)) max(w) else 0
    gap_w <- strwidth("      ", cex = 1.3)
    if (!is.finite(gap_w)) gap_w <- 0
    
    legend_args <- list(
      "top",
      legend = x1_levels,
      pch = 15,
      col = col,
      bty = "n",
      inset = 0.01,
      cex = 1.15,
      pt.cex = 2,
      x.intersp = 1.2,
      text.width = tw + gap_w,
      horiz = legend_horiz
    )
    if (!legend_horiz) {
      legend_args$text.width <- NULL
      legend_args$x.intersp <- 1.2
    }
    if (!is.null(legend.title)) {
      legend_args$title <- legend.title
      legend_args$title.font <- 2
    }
    do.call(legend, legend_args)
  }
  
  invisible(NULL)
}



#----------------------------------
# plot_means_compute_pvalues: run the default inferential tests and return p-values for annotation ----
#   This is only used when `tests = "auto"`. It returns a small table with the exact
#   comparisons needed by the plotting code (group labels, means, diffs, and p-values).
plot_means_compute_pvalues <- function(v, params, mean_results, comp = NULL) {
  if (identical(v$tests, "none")) return(NULL)

  # Custom tests: parse and compute from bar indices (left-to-right draw order)
  if (!identical(v$tests, "auto")) {
    if (is.null(comp) || !is.list(comp) || is.null(comp$cell_keys_drawn)) {
      stop("plot_means(): internal error: custom 'tests' requires computed bar layout", call. = FALSE)
    }
    specs <- plot_means_parse_tests(v$tests)
    cell_keys_drawn <- comp$cell_keys_drawn
    n_cols <- length(cell_keys_drawn)
    
    all_idx <- unlist(lapply(specs, function(s) {
      if (identical(s$type, "pooled")) {
        c(s$left, s$right)
      } else {
        unlist(s[c("a", "b", "c", "d")], use.names = FALSE)
      }
    }), use.names = FALSE)
    all_idx <- all_idx[is.finite(all_idx)]
    if (any(all_idx < 1L) || any(all_idx > n_cols)) {
      stop("plot_means(): invalid column index in 'tests'. There are ", n_cols, " plotted columns.", call. = FALSE)
    }
    
    mf_tests <- plot_means_model_frame(v$formula_eval, data = v$data, na.action = na.omit, context = "custom tests")
    if (nrow(mf_tests) == 0) return(NULL)
    names(mf_tests)[1] <- ".__y"
    
    x1_name <- v$x1_name
    x2_name <- v$x2_name
    x3_name <- v$x3_name
    x2_col <- if (is.null(x2_name)) ".x2" else x2_name
    x3_col <- if (is.null(x3_name)) ".x3" else x3_name
    
    mf_tests[[x1_name]] <- as.character(mf_tests[[x1_name]])
    if (is.null(x2_name)) mf_tests$.x2 <- "All" else mf_tests[[x2_name]] <- as.character(mf_tests[[x2_name]])
    if (is.null(x3_name)) mf_tests$.x3 <- "All" else mf_tests[[x3_name]] <- as.character(mf_tests[[x3_name]])
    
    complete_mask <- !is.na(mf_tests$.__y) &
      !is.na(mf_tests[[x1_name]]) &
      !is.na(mf_tests[[x2_col]]) &
      !is.na(mf_tests[[x3_col]])
    mf_tests <- mf_tests[complete_mask, , drop = FALSE]
    if (nrow(mf_tests) == 0) return(NULL)
    
    mf_tests$cell_key <- paste(mf_tests[[x1_name]], mf_tests[[x2_col]], mf_tests[[x3_col]], sep = "|")
    cluster_mf <- plot_means_align_cluster(v, mf_tests)
    if (!is.null(cluster_mf)) mf_tests$.__cluster <- cluster_mf
    
    calc_simple <- function(a, b) {
      kA <- cell_keys_drawn[a]
      kB <- cell_keys_drawn[b]
      df_sub <- mf_tests[mf_tests$cell_key %in% c(kA, kB), c(".__y", "cell_key", if (!is.null(cluster_mf)) ".__cluster" else NULL), drop = FALSE]
      if (nrow(df_sub) == 0) stop("plot_means(): no data for columns ", a, " and ", b, call. = FALSE)
      df_sub$.__g <- factor(ifelse(df_sub$cell_key == kA, "A", "B"), levels = c("A", "B"))
      if (length(unique(df_sub$.__g)) < 2) stop("plot_means(): not enough data to compare columns ", a, " and ", b, call. = FALSE)
      
      if (is.null(cluster_mf)) {
        tt <- stats::t.test(.__y ~ .__g, data = df_sub, var.equal = FALSE)
        meanA <- as.numeric(tt$estimate[[1]])
        meanB <- as.numeric(tt$estimate[[2]])
        list(
          mean1 = meanA,
          mean2 = meanB,
          diff = meanB - meanA,
          # Align sign with diff (B - A). t.test reports t for (A - B).
          t.value = -as.numeric(tt$statistic[[1]]),
          p.value = as.numeric(tt$p.value),
          df = as.numeric(tt$parameter[[1]]),
          method = "Welch t-test"
        )
      } else {
        fit <- lm2(.__y ~ .__g, data = df_sub, clusters = df_sub$.__cluster)
        tab <- attr(fit, "statuser_table")
        if (is.null(tab) || !is.data.frame(tab) || !"term" %in% names(tab)) stop("plot_means(): failed to extract lm2 table for custom test", call. = FALSE)
        idx <- which(grepl("^\\.__g", as.character(tab$term)))
        if (!length(idx)) stop("plot_means(): failed to find group term for custom test", call. = FALSE)
        tr <- tab[idx[1], , drop = FALSE]
        meanA <- mean(df_sub$.__y[df_sub$.__g == "A"], na.rm = TRUE)
        meanB <- mean(df_sub$.__y[df_sub$.__g == "B"], na.rm = TRUE)
        list(
          mean1 = as.numeric(meanA),
          mean2 = as.numeric(meanB),
          diff = as.numeric(tr$estimate),
          t.value = as.numeric(tr$t),
          p.value = as.numeric(tr$p.value),
          df = if ("df" %in% names(tr)) as.numeric(tr$df) else NA_real_,
          method = "regression (clustered SE)"
        )
      }
    }
    
    calc_interaction <- function(a, b, c, d) {
      keys <- unique(cell_keys_drawn[c(a, b, c, d)])
      df_sub <- mf_tests[mf_tests$cell_key %in% keys, c(".__y", "cell_key", if (!is.null(cluster_mf)) ".__cluster" else NULL), drop = FALSE]
      if (nrow(df_sub) == 0) stop("plot_means(): no data for interaction test", call. = FALSE)
      df_sub$.__g <- factor(df_sub$cell_key, levels = keys)
      
      fit <- if (is.null(cluster_mf)) {
        lm2(.__y ~ 0 + .__g, data = df_sub)
      } else {
        lm2(.__y ~ 0 + .__g, data = df_sub, clusters = df_sub$.__cluster)
      }
      beta <- stats::coef(fit)
      V <- stats::vcov(fit)
      if (is.null(beta) || is.null(V)) stop("plot_means(): failed to extract coef/vcov for interaction test", call. = FALSE)
      
      coef_names <- names(beta)
      name_for <- function(key) {
        target <- paste0(".__g", key)
        idx <- match(target, coef_names)
        if (is.na(idx)) stop("plot_means(): missing coefficient for bar in interaction test", call. = FALSE)
        coef_names[idx]
      }
      nA <- name_for(cell_keys_drawn[a])
      nB <- name_for(cell_keys_drawn[b])
      nC <- name_for(cell_keys_drawn[c])
      nD <- name_for(cell_keys_drawn[d])
      
      L <- rep(0, length(beta))
      names(L) <- coef_names
      L[nA] <-  1
      L[nB] <- -1
      L[nC] <- -1
      L[nD] <-  1
      
      est <- sum(L * beta)
      se <- sqrt(as.numeric(t(L) %*% V %*% L))
      if (!is.finite(se) || se <= 0) stop("plot_means(): could not compute SE for interaction test", call. = FALSE)
      t_val <- est / se
      df_fit <- fit$df
      if (length(df_fit) != 1) df_fit <- suppressWarnings(min(df_fit, na.rm = TRUE))
      if (!is.finite(df_fit)) df_fit <- suppressWarnings(min(fit$df, na.rm = TRUE))
      if (!is.finite(df_fit)) df_fit <- 1
      p_val <- 2 * stats::pt(-abs(t_val), df = df_fit)
      list(
        diff = as.numeric(est),
        t.value = as.numeric(t_val),
        p.value = as.numeric(p_val),
        df = as.numeric(df_fit),
        method = if (is.null(cluster_mf)) "regression interaction" else "regression interaction (clustered SE)"
      )
    }
    
    calc_pooled <- function(left, right) {
      kL <- unique(cell_keys_drawn[left])
      kR <- unique(cell_keys_drawn[right])
      keys <- unique(c(kL, kR))
      df_sub <- mf_tests[mf_tests$cell_key %in% keys, c(".__y", "cell_key", if (!is.null(cluster_mf)) ".__cluster" else NULL), drop = FALSE]
      if (nrow(df_sub) == 0) stop("plot_means(): no data for pooled test", call. = FALSE)
      df_sub$.__g <- factor(ifelse(df_sub$cell_key %in% kL, "A", "B"), levels = c("A", "B"))
      if (length(unique(df_sub$.__g)) < 2) stop("plot_means(): not enough data for pooled test", call. = FALSE)
      
      if (is.null(cluster_mf)) {
        tt <- stats::t.test(.__y ~ .__g, data = df_sub, var.equal = FALSE)
        meanA <- as.numeric(tt$estimate[[1]])
        meanB <- as.numeric(tt$estimate[[2]])
        list(
          mean1 = meanA,
          mean2 = meanB,
          diff = meanB - meanA,
          # Align sign with diff (B - A). t.test reports t for (A - B).
          t.value = -as.numeric(tt$statistic[[1]]),
          p.value = as.numeric(tt$p.value),
          df = as.numeric(tt$parameter[[1]]),
          method = "Welch t-test"
        )
      } else {
        fit <- lm2(.__y ~ .__g, data = df_sub, clusters = df_sub$.__cluster)
        tab <- attr(fit, "statuser_table")
        if (is.null(tab) || !is.data.frame(tab) || !"term" %in% names(tab)) stop("plot_means(): failed to extract lm2 table for pooled test", call. = FALSE)
        idx <- which(grepl("^\\.__g", as.character(tab$term)))
        if (!length(idx)) stop("plot_means(): failed to find group term for pooled test", call. = FALSE)
        tr <- tab[idx[1], , drop = FALSE]
        meanA <- mean(df_sub$.__y[df_sub$.__g == "A"], na.rm = TRUE)
        meanB <- mean(df_sub$.__y[df_sub$.__g == "B"], na.rm = TRUE)
        list(
          mean1 = as.numeric(meanA),
          mean2 = as.numeric(meanB),
          diff = as.numeric(tr$estimate),
          t.value = as.numeric(tr$t),
          p.value = as.numeric(tr$p.value),
          df = if ("df" %in% names(tr)) as.numeric(tr$df) else NA_real_,
          method = "regression (clustered SE)"
        )
      }
    }
    
    rows <- data.frame(
      group1 = character(0),
      group2 = character(0),
      mean1 = numeric(0),
      mean2 = numeric(0),
      diff = numeric(0),
      t.value = numeric(0),
      p.value = numeric(0),
      df = numeric(0),
      method = character(0),
      test_type = character(0),
      cols_left = character(0),
      cols_right = character(0),
      col1 = integer(0),
      col2 = integer(0),
      col3 = integer(0),
      col4 = integer(0),
      stringsAsFactors = FALSE
    )
    
    for (s in specs) {
      if (identical(s$type, "simple")) {
        res <- calc_simple(s$a, s$b)
        rows <- rbind(rows, data.frame(
          group1 = paste0("col", s$a),
          group2 = paste0("col", s$b),
          mean1 = res$mean1,
          mean2 = res$mean2,
          diff = res$diff,
          t.value = res$t.value,
          p.value = res$p.value,
          df = res$df,
          method = res$method,
          test_type = "simple",
          cols_left = "",
          cols_right = "",
          col1 = s$a,
          col2 = s$b,
          col3 = NA_integer_,
          col4 = NA_integer_,
          stringsAsFactors = FALSE
        ))
      } else if (identical(s$type, "pooled")) {
        res <- calc_pooled(s$left, s$right)
        involved <- c(s$left, s$right)
        rows <- rbind(rows, data.frame(
          group1 = paste0("(", paste(s$left, collapse = "+"), ")-(", paste(s$right, collapse = "+"), ")"),
          group2 = "",
          mean1 = res$mean1,
          mean2 = res$mean2,
          diff = res$diff,
          t.value = res$t.value,
          p.value = res$p.value,
          df = res$df,
          method = res$method,
          test_type = "pooled",
          cols_left = paste(s$left, collapse = ","),
          cols_right = paste(s$right, collapse = ","),
          col1 = min(involved),
          col2 = max(involved),
          col3 = NA_integer_,
          col4 = NA_integer_,
          stringsAsFactors = FALSE
        ))
      } else if (identical(s$type, "interaction")) {
        res <- calc_interaction(s$a, s$b, s$c, s$d)
        m_a <- mean(mf_tests$.__y[mf_tests$cell_key == cell_keys_drawn[s$a]], na.rm = TRUE)
        m_b <- mean(mf_tests$.__y[mf_tests$cell_key == cell_keys_drawn[s$b]], na.rm = TRUE)
        m_c <- mean(mf_tests$.__y[mf_tests$cell_key == cell_keys_drawn[s$c]], na.rm = TRUE)
        m_d <- mean(mf_tests$.__y[mf_tests$cell_key == cell_keys_drawn[s$d]], na.rm = TRUE)
        d_first <- as.numeric(m_a - m_b)
        d_second <- as.numeric(m_c - m_d)
        rows <- rbind(rows, data.frame(
          group1 = paste0("interaction(", s$a, "-", s$b, ")-(", s$c, "-", s$d, ")"),
          group2 = "",
          mean1 = d_first,
          mean2 = d_second,
          diff = res$diff,
          t.value = res$t.value,
          p.value = res$p.value,
          df = res$df,
          method = res$method,
          test_type = "interaction",
          cols_left = "",
          cols_right = "",
          col1 = s$a,
          col2 = s$b,
          col3 = s$c,
          col4 = s$d,
          stringsAsFactors = FALSE
        ))
      }
    }
    return(rows)
  }
  
  # Pull the relevant term row from the lm2() summary table.
  #   Non-obvious: lm2() stores the coefficient table in an attribute ("statuser_table"),
  #   and interaction term naming can vary, so we match by pattern.
  get_term_row <- function(fit, var_a, var_b = NULL, want_interaction = FALSE) {
    tab <- attr(fit, "statuser_table")
    if (is.null(tab) || !is.data.frame(tab) || !"term" %in% names(tab)) return(NULL)
    terms <- as.character(tab$term)
    if (want_interaction) {
      idx <- which(grepl(":", terms) & grepl(var_a, terms) & grepl(var_b, terms))
    } else {
      idx <- which(grepl(paste0("^", var_a), terms))
    }
    if (!length(idx)) return(NULL)
    tab[idx[1], , drop = FALSE]
  }
  
  # Build the model frame once, applying the same NA filtering policy as the tests.
  #   This keeps p-values consistent with the data actually available for inference.
  mf_tests <- plot_means_model_frame(v$formula_eval, data = v$data, na.action = na.omit, context = "plot_means_compute_pvalues()")
  if (nrow(mf_tests) == 0) return(NULL)
  
  names(mf_tests)[1] <- ".__y"
  mf_tests$.__x1 <- as.factor(mf_tests[[v$x1_name]])
  if (!is.null(v$x2_name)) mf_tests$.__x2 <- as.factor(mf_tests[[v$x2_name]])
  cluster_mf <- plot_means_align_cluster(v, mf_tests)
  if (!is.null(cluster_mf)) mf_tests$.__cluster <- cluster_mf
  
  # Fit helper that keeps the clustered and non-clustered paths in one place.
  #   Returns the fitted model (or NA) so callers can extract the term row they need.
  p_lm2_x1 <- function(df_sub) {
    if (nrow(df_sub) == 0) return(NA_real_)
    fit <- if (is.null(v$cluster_vec)) {
      lm2(.__y ~ .__x1, data = df_sub)
    } else {
      lm2(.__y ~ .__x1, data = df_sub, clusters = df_sub$.__cluster)
    }
    fit
  }

  ttest_x1 <- function(df_sub) {
    out <- list(t.value = NA_real_, p.value = NA_real_, df = NA_real_, method = "Welch t-test")
    if (nrow(df_sub) == 0) return(out)
    if (!(".__y" %in% names(df_sub)) || !(".__x1" %in% names(df_sub))) return(out)
    if (length(unique(df_sub$.__x1)) < 2) return(out)
    tt <- tryCatch(
      stats::t.test(.__y ~ .__x1, data = df_sub, var.equal = FALSE),
      error = function(e) NULL
    )
    if (is.null(tt)) return(out)
    out$t.value <- as.numeric(tt$statistic[[1]])
    out$p.value <- as.numeric(tt$p.value)
    out$df <- as.numeric(tt$parameter[[1]])
    out
  }
  
  x1_levels <- params$x1_levels
  x2_levels <- params$x2_levels
  
  x1_is_binary <- length(x1_levels) == 2
  x2_is_binary <- !is.null(v$x2_name) && length(x2_levels) == 2
  x3_is_null <- is.null(v$x3_name)

  group_label <- function(x1, x2 = NULL, x3 = NULL) {
    parts <- character(0)
    if (!is.null(x3) && nzchar(x3)) parts <- c(parts, x3)
    if (!is.null(x2) && nzchar(x2)) parts <- c(parts, x2)
    parts <- c(parts, x1)
    paste(parts, collapse = "_")
  }
  
  mean_lookup <- function(x1, x2 = NULL, x3 = NULL) {
    df <- mean_results
    keep <- rep(TRUE, nrow(df))
    if (!is.null(v$x1_name) && v$x1_name %in% names(df)) keep <- keep & as.character(df[[v$x1_name]]) == as.character(x1)
    if (!is.null(v$x2_name) && v$x2_name %in% names(df) && !is.null(x2)) keep <- keep & as.character(df[[v$x2_name]]) == as.character(x2)
    if (!is.null(v$x3_name) && v$x3_name %in% names(df) && !is.null(x3)) keep <- keep & as.character(df[[v$x3_name]]) == as.character(x3)
    val <- df$mean[keep]
    if (!length(val)) return(NA_real_)
    as.numeric(val[1])
  }
  
  # Output table: one row per displayed comparison
  #   group1/group2 are plot labels (used to map to specific bars), and mean/diff are included
  #   so the draw step can position p-value brackets relative to the bars.
  rows <- data.frame(
    group1 = character(0),
    group2 = character(0),
    mean1 = numeric(0),
    mean2 = numeric(0),
    diff = numeric(0),
    t.value = numeric(0),
    p.value = numeric(0),
    df = numeric(0),
    method = character(0),
    stringsAsFactors = FALSE
  )
  
  # Scenario 1: x1 only, binary
    if (length(v$x_names) == 1 && x1_is_binary) {
      df_sub <- mf_tests[, c(".__y", ".__x1"), drop = FALSE]
      if (is.null(v$cluster_vec)) {
        tt <- ttest_x1(df_sub)
        t_val <- tt$t.value
        p_val <- tt$p.value
        df_val <- tt$df
        method <- tt$method
      } else {
        fit <- p_lm2_x1(df_sub)
        tr <- get_term_row(fit, ".__x1")
        t_val <- if (is.null(tr)) NA_real_ else as.numeric(tr$t)
        p_val <- if (is.null(tr)) NA_real_ else as.numeric(tr$p.value)
        df_val <- if (is.null(tr) || !"df" %in% names(tr)) NA_real_ else as.numeric(tr$df)
        method <- "regression (clustered SE)"
      }
      g1 <- group_label(x1_levels[1])
      g2 <- group_label(x1_levels[2])
      m1 <- mean_lookup(x1_levels[1])
      m2 <- mean_lookup(x1_levels[2])
      rows <- rbind(rows, data.frame(
        group1 = g1,
        group2 = g2,
        mean1 = m1,
        mean2 = m2,
        diff = m2 - m1,
        t.value = t_val,
        p.value = p_val,
        df = df_val,
        method = method,
        stringsAsFactors = FALSE
      ))
      return(rows)
    }
  
  # Scenario 2: x1 and x2, both binary, no x3
    if (length(v$x_names) == 2 && x1_is_binary && x2_is_binary && x3_is_null) {
      for (x2v in x2_levels) {
        df_sub <- mf_tests[mf_tests$.__x2 == x2v, c(".__y", ".__x1"), drop = FALSE]
        if (is.null(v$cluster_vec)) {
          tt <- ttest_x1(df_sub)
          t_val <- tt$t.value
          p_val <- tt$p.value
          df_val <- tt$df
          method <- tt$method
        } else {
          fit <- p_lm2_x1(df_sub)
          tr <- get_term_row(fit, ".__x1")
          t_val <- if (is.null(tr)) NA_real_ else as.numeric(tr$t)
          p_val <- if (is.null(tr)) NA_real_ else as.numeric(tr$p.value)
          df_val <- if (is.null(tr) || !"df" %in% names(tr)) NA_real_ else as.numeric(tr$df)
          method <- "regression (clustered SE)"
        }
        
        g1 <- group_label(x1_levels[1], x2 = x2v)
        g2 <- group_label(x1_levels[2], x2 = x2v)
        m1 <- mean_lookup(x1_levels[1], x2 = x2v)
        m2 <- mean_lookup(x1_levels[2], x2 = x2v)
        rows <- rbind(rows, data.frame(
          group1 = g1,
          group2 = g2,
          mean1 = m1,
          mean2 = m2,
          diff = m2 - m1,
          t.value = t_val,
          p.value = p_val,
          df = df_val,
          method = method,
          stringsAsFactors = FALSE
        ))
      }
      
      # Interaction: always use lm2(), even when cluster is NULL
      fit_int <- if (is.null(v$cluster_vec)) {
        lm2(.__y ~ .__x1 * .__x2, data = mf_tests[, c(".__y", ".__x1", ".__x2"), drop = FALSE])
      } else {
        lm2(.__y ~ .__x1 * .__x2, data = mf_tests[, c(".__y", ".__x1", ".__x2", ".__cluster"), drop = FALSE], clusters = mf_tests$.__cluster)
      }
      tr_int <- get_term_row(fit_int, ".__x1", ".__x2", want_interaction = TRUE)
      if (!is.null(tr_int)) {
        d_first <- mean_lookup(x1_levels[2], x2 = x2_levels[1]) - mean_lookup(x1_levels[1], x2 = x2_levels[1])
        d_second <- mean_lookup(x1_levels[2], x2 = x2_levels[2]) - mean_lookup(x1_levels[1], x2 = x2_levels[2])
        rows <- rbind(rows, data.frame(
          group1 = paste0("interaction(", v$x1_name, ":", v$x2_name, ")"),
          group2 = "",
          mean1 = as.numeric(d_first),
          mean2 = as.numeric(d_second),
          diff = as.numeric(tr_int$estimate),
          t.value = as.numeric(tr_int$t),
          p.value = as.numeric(tr_int$p.value),
          df = if ("df" %in% names(tr_int)) as.numeric(tr_int$df) else NA_real_,
          method = if (is.null(v$cluster_vec)) "regression interaction" else "regression interaction (clustered SE)",
          stringsAsFactors = FALSE
        ))
      }
      
      return(rows)
    }
  
  # Scenario 3: x1 binary, x2 has >2 levels, no x3
    if (length(v$x_names) == 2 && x1_is_binary && !is.null(v$x2_name) && length(x2_levels) > 2 && x3_is_null) {
      for (x2v in x2_levels) {
        df_sub <- mf_tests[mf_tests$.__x2 == x2v, c(".__y", ".__x1"), drop = FALSE]
        if (is.null(v$cluster_vec)) {
          tt <- ttest_x1(df_sub)
          t_val <- tt$t.value
          p_val <- tt$p.value
          df_val <- tt$df
          method <- tt$method
        } else {
          fit <- p_lm2_x1(df_sub)
          tr <- get_term_row(fit, ".__x1")
          t_val <- if (is.null(tr)) NA_real_ else as.numeric(tr$t)
          p_val <- if (is.null(tr)) NA_real_ else as.numeric(tr$p.value)
          df_val <- if (is.null(tr) || !"df" %in% names(tr)) NA_real_ else as.numeric(tr$df)
          method <- "regression (clustered SE)"
        }
        
        g1 <- group_label(x1_levels[1], x2 = x2v)
        g2 <- group_label(x1_levels[2], x2 = x2v)
        m1 <- mean_lookup(x1_levels[1], x2 = x2v)
        m2 <- mean_lookup(x1_levels[2], x2 = x2v)
        rows <- rbind(rows, data.frame(
          group1 = g1,
          group2 = g2,
          mean1 = m1,
          mean2 = m2,
          diff = m2 - m1,
          t.value = t_val,
          p.value = p_val,
          df = df_val,
          method = method,
          stringsAsFactors = FALSE
        ))
      }
      
      return(rows)
    }
  
  NULL
}

#plot_means_001
test_that("plot_means runs with formula + data and returns desc_var object", {
  df <- data.frame(y = rnorm(100), group = rep(c("A", "B"), 50))

  result <- plot_means(y ~ group, data = df)

  expect_true(is.list(result))
  expect_true(is.data.frame(result$means))
})

#plot_means_002
test_that("plot_means works with multiple grouping variables (y ~ x1 + x2)", {
  df <- data.frame(
    y = rnorm(200),
    x1 = rep(c("A", "B"), 100),
    x2 = rep(c("X", "Y"), each = 100)
  )

  result <- plot_means(y ~ x1 + x2, data = df)

  expect_true(is.list(result))
  expect_true(is.data.frame(result$means))
})

#plot_means_003
test_that("plot_means order reorders groups for single grouping variable", {
  df <- data.frame(y = rnorm(100), group = rep(c("A", "B"), 50))

  result_default <- plot_means(y ~ group, data = df)
  expect_true(is.list(result_default))

  result_reversed <- plot_means(y ~ group, data = df, order = -1)
  expect_true(is.list(result_reversed))

  result_custom <- plot_means(y ~ group, data = df, order = c("B", "A"))
  expect_true(is.list(result_custom))
})

#plot_means_004
test_that("plot_means works with three grouping variables and missing combinations", {
  set.seed(1)
  df <- data.frame(
    y = rnorm(96),
    x1 = rep(c("A", "B"), 48),
    x2 = rep(rep(c("X", "Y"), each = 24), 2),
    x3 = rep(c("M", "N"), each = 48)
  )

  # Remove one x1 level from one (x2,x3) block to create a missing combination
  df <- df[!(df$x1 == "B" & df$x2 == "Y" & df$x3 == "N"), , drop = FALSE]

  result <- plot_means(y ~ x1 + x2 + x3, data = df)
  expect_true(is.list(result))
})

#plot_means_005
test_that("plot_means tests=auto works for scenario 1 (binary x1 only)", {
  df <- data.frame(y = rnorm(60), x1 = rep(c("A", "B"), each = 30))
  expect_error(plot_means(y ~ x1, data = df, tests = "auto"), NA)
})

#plot_means_006
test_that("plot_means tests=auto works for scenario 2 (binary x1 and x2)", {
  df <- data.frame(
    y = rnorm(120),
    x1 = rep(c("A", "B"), 60),
    x2 = rep(rep(c("X", "Y"), each = 30), 2)
  )
  expect_error(plot_means(y ~ x1 + x2, data = df, tests = "auto"), NA)
})

#plot_means_007
test_that("plot_means tests=auto works for scenario 3 (binary x1, x2 has >2 levels)", {
  df <- expand.grid(
    x1 = c("A", "B"),
    x2 = c("X", "Y", "Z"),
    rep = seq_len(30),
    stringsAsFactors = FALSE
  )
  df$y <- rnorm(nrow(df))
  
  expect_error(plot_means(y ~ x1 + x2, data = df, tests = "auto"), NA)
})

#plot_means_008
test_that("plot_means means + per-cell CIs match t.test() when cluster=NULL", {
  set.seed(123)
  df <- data.frame(
    y = rnorm(120),
    x1 = rep(c("A", "B", "C"), each = 40),
    stringsAsFactors = FALSE
  )
  ci_level <- 90
  
  pm <- plot_means(y ~ x1, data = df, tests = "none", ci.level = ci_level, quiet = TRUE)
  expect_true(is.data.frame(pm$means))
  
  # Means/sd/n should match base computations
  m <- tapply(df$y, df$x1, mean)
  s <- tapply(df$y, df$x1, sd)
  n <- tapply(df$y, df$x1, function(z) sum(is.finite(z)))
  
  idx <- match(pm$means$x1, names(m))
  expect_false(anyNA(idx))
  
  # desc_var()/plot_means may round display columns; allow small tolerance.
  expect_true(max(abs(unname(pm$means$mean) - as.numeric(m[idx]))) < 1e-3)
  expect_true(max(abs(unname(pm$means$sd) - as.numeric(s[idx]))) < 1e-3)
  expect_true(max(abs(unname(pm$means$n.total) - as.numeric(n[idx]))) < 1e-12)
  
  # CI should match one-sample t.test(y ~ 1) within each cell
  for (g in pm$means$x1) {
    yy <- df$y[df$x1 == g]
    tt <- stats::t.test(yy, conf.level = ci_level / 100)
    pm_row <- pm$means[pm$means$x1 == g, , drop = FALSE]
    expect_equal(pm_row$ciL, as.numeric(tt$conf.int[1]), tolerance = 1e-10)
    expect_equal(pm_row$ciH, as.numeric(tt$conf.int[2]), tolerance = 1e-10)
  }
})

#plot_means_010
test_that("plot_means tests=auto (scenario 1) matches Welch t.test()", {
  set.seed(1)
  df <- data.frame(y = rnorm(80), x1 = rep(c("A", "B"), each = 40))
  
  pm <- plot_means(y ~ x1, data = df, tests = "auto", ci.level = 95, quiet = TRUE)
  expect_true(is.data.frame(pm$means))
  expect_true(is.data.frame(pm$tests))
  expect_gt(nrow(pm$tests), 0)
  
  # First (and only) comparison should correspond to Welch two-sample t-test
  tt <- stats::t.test(y ~ x1, data = df, var.equal = FALSE)
  row <- pm$tests[1, , drop = FALSE]
  
  expect_true(is.finite(row$t.value))
  expect_true(is.finite(row$p.value))
  expect_equal(as.numeric(row$p.value), as.numeric(tt$p.value), tolerance = 1e-10)
  expect_equal(as.numeric(row$df), as.numeric(tt$parameter[[1]]), tolerance = 1e-10)
  # Our diff uses mean2 - mean1, and our t.value is aligned to that sign.
  expect_equal(as.numeric(row$diff), as.numeric(row$mean2 - row$mean1), tolerance = 1e-12)
})

#plot_means_011
test_that("plot_means 2x2 interaction p-value matches estimatr::lm_robust(HC3)", {
  skip_if_not_installed("estimatr")
  
  set.seed(2)
  df <- data.frame(
    y = rnorm(400),
    x1 = rep(c("A", "B"), 200),
    x2 = rep(rep(c("X", "Y"), each = 100), 2),
    stringsAsFactors = FALSE
  )
  
  pm <- plot_means(y ~ x1 + x2, data = df, tests = "auto", ci.level = 95, quiet = TRUE)
  expect_true(is.data.frame(pm$tests))
  
  int_row <- pm$tests[grepl("^interaction\\(", as.character(pm$tests$group1)), , drop = FALSE]
  expect_equal(nrow(int_row), 1)
  
  fit <- estimatr::lm_robust(y ~ x1 * x2, data = df, se_type = "HC3")
  sm <- summary(fit)
  coef_tab <- sm$coefficients
  
  int_term <- rownames(coef_tab)[grepl(":", rownames(coef_tab))]
  expect_equal(length(int_term), 1)
  
  expect_equal(as.numeric(int_row$p.value), as.numeric(coef_tab[int_term, "Pr(>|t|)"]), tolerance = 1e-10)
  expect_equal(as.numeric(int_row$t.value), as.numeric(coef_tab[int_term, "t value"]), tolerance = 1e-10)
  expect_equal(as.numeric(int_row$df), as.numeric(sm$df.residual), tolerance = 1e-10)
})

#plot_means_012
test_that("plot_means clustered CI uses regression-based intervals governed by ci.level", {
  skip_if_not_installed("estimatr")
  
  set.seed(3)
  n_cl <- 40
  n_per <- 20
  cl <- rep(seq_len(n_cl), each = n_per)
  df <- data.frame(
    y = rnorm(n_cl * n_per),
    x1 = rep(c("A", "B"), length.out = n_cl * n_per),
    cl = cl,
    stringsAsFactors = FALSE
  )
  
  ci_level <- 90
  pm <- plot_means(y ~ x1, data = df, cluster = df$cl, tests = "none", ci.level = ci_level, quiet = TRUE)
  
  # Reconstruct the same cell_key as plot_means_compute() uses (x1|All|All)
  df2 <- df
  df2$cell_key <- paste(as.character(df2$x1), "All", "All", sep = "|")
  df2$cell_factor <- factor(df2$cell_key)
  
  fit <- estimatr::lm_robust(y ~ 0 + cell_factor, data = df2, clusters = df2$cl, se_type = "CR2")
  beta <- stats::coef(fit)
  V <- stats::vcov(fit)
  df_fit <- fit$df
  if (length(df_fit) != 1) df_fit <- suppressWarnings(min(df_fit, na.rm = TRUE))
  if (!is.finite(df_fit) || df_fit <= 0) df_fit <- 1
  alpha <- 1 - ci_level / 100
  tcrit <- stats::qt(1 - alpha / 2, df = df_fit)
  
  lev <- levels(df2$cell_factor)
  newdata <- data.frame(cell_factor = factor(lev, levels = lev))
  X <- stats::model.matrix(~ 0 + cell_factor, data = newdata)
  
  b_idx <- match(colnames(X), names(beta))
  X <- X[, !is.na(b_idx), drop = FALSE]
  b_idx <- b_idx[!is.na(b_idx)]
  beta_use <- beta[b_idx]
  V_use <- V[b_idx, b_idx, drop = FALSE]
  
  fit_hat <- as.numeric(X %*% beta_use)
  se_hat <- sqrt(pmax(0, diag(X %*% V_use %*% t(X))))
  lwr <- fit_hat - tcrit * se_hat
  upr <- fit_hat + tcrit * se_hat
  
  ci_df <- data.frame(cell_key = lev, lwr = lwr, upr = upr, stringsAsFactors = FALSE)
  
  for (i in seq_len(nrow(pm$means))) {
    key <- paste(as.character(pm$means$x1[i]), "All", "All", sep = "|")
    j <- match(key, ci_df$cell_key)
    expect_false(is.na(j))
    expect_equal(pm$means$ciL[i], ci_df$lwr[j], tolerance = 1e-3)
    expect_equal(pm$means$ciH[i], ci_df$upr[j], tolerance = 1e-3)
  }
})

be_expect_equal_num <- function(a, b, tol = 1e-10) {
  
  if (is.null(a) && is.null(b)) {
    testthat::expect_true(TRUE)
    return(invisible(NULL))
  }
  
  if (is.null(a) || is.null(b)) {
    testthat::expect_identical(a, b)
    return(invisible(NULL))
  }
  
  if (isTRUE(all(is.na(a))) && isTRUE(all(is.na(b)))) {
    testthat::expect_true(TRUE)
    return(invisible(NULL))
  }
  
  if (is.numeric(a) || is.numeric(b)) {
    testthat::expect_equal(as.numeric(a), as.numeric(b), tolerance = tol)
    return(invisible(NULL))
  }
  
  testthat::expect_identical(a, b)
  invisible(NULL)
}

be_extract_linear_interaction_p <- function(model, xvar, zvar) {
  
  cf <- tryCatch(stats::coef(summary(model)), error = function(e) NULL)
  if (is.null(cf)) return(NA_real_)
  
  rn <- rownames(cf)
  if (is.null(rn)) return(NA_real_)
  
  candidates <- c(paste0(xvar, ":", zvar), paste0(zvar, ":", xvar))
  idx <- match(candidates, rn)
  idx <- idx[!is.na(idx)]
  if (length(idx) < 1) return(NA_real_)
  
  pcol <- intersect(colnames(cf), c("Pr(>|t|)", "Pr(>|z|)"))
  if (length(pcol) < 1) return(NA_real_)
  
  as.numeric(cf[idx[1], pcol[1]])
}

be_sig_from_ci <- function(conf_low, conf_high) {
  
  (conf_low > 0) | (conf_high < 0)
}

be_parse_p_from_apa_line <- function(x) {
  
  # Parse "p < .001" or "p = .023" into numeric (use 0.001 for p<.001)
  if (length(x) != 1 || is.na(x)) return(NA_real_)
  
  if (grepl("p\\s*<\\s*\\.001", x)) return(0.001)
  
  m <- regexec("p\\s*=\\s*([0-9]*\\.?[0-9]+)", x)
  r <- regmatches(x, m)
  if (length(r) < 1 || length(r[[1]]) < 2) return(NA_real_)
  
  as.numeric(r[[1]][2])
}

# Maintenance: add #function_NNN before each test_that().
# Run: Rscript tests/testthat/_annotate_test_ids.R
# (Working directory should be the package root.)

strip_test_ids <- function(lines) {
  n <- length(lines)
  if (n < 2L) {
    return(lines)
  }
  drop <- rep(FALSE, n)
  for (i in 2L:n) {
    if (grepl("^test_that\\(", lines[i]) &&
        grepl("^#[A-Za-z0-9._]+_[0-9]{3}\\s*$", lines[i - 1L])) {
      drop[i - 1L] <- TRUE
    }
  }
  lines[!drop]
}

test_title <- function(line) {
  m <- regexpr("^test_that\\(\"([^\"]+)\"", line, perl = TRUE)
  if (m < 1L) {
    return(NA_character_)
  }
  st <- attr(m, "capture.start")[1L]
  ln <- attr(m, "capture.length")[1L]
  substr(line, st, st + ln - 1L)
}

prefix_for_line <- function(basename, title) {
  if (is.na(title)) {
    return("unknown")
  }
  if (identical(basename, "test-backend_equivalence.R")) {
    if (grepl("^backend: t\\.test2", title)) {
      return("t.test2")
    }
    if (grepl("^backend: lm2", title)) {
      return("lm2")
    }
    if (grepl("^backend: interprobe", title)) {
      return("interprobe")
    }
    return("backend_equivalence")
  }
  if (identical(basename, "test-gam_functions.R")) {
    if (grepl("^scatter\\.gam", title)) {
      return("scatter.gam")
    }
    if (grepl("^plot_gam", title)) {
      return("plot_gam")
    }
    if (grepl("^twolines", title)) {
      return("twolines")
    }
    return("gam")
  }
  known <- c(
    "test-message2.R" = "message2",
    "test-list2.R" = "list2",
    "test-text2.R" = "text2",
    "test-format_pvalue.R" = "format_pvalue",
    "test-t.test2.R" = "t.test2",
    "test-lm2.R" = "lm2",
    "test-plot_cdf.R" = "plot_cdf",
    "test-plot_freq.R" = "plot_freq",
    "test-plot_density.R" = "plot_density",
    "test-plot_means.R" = "plot_means",
    "test-validate_plot.R" = "validate_plot",
    "test-table2.R" = "table2",
    "test-print_functions.R" = "print.table2",
    "test-convert_to_sql.R" = "convert_to_sql",
    "test-resize_images.R" = "resize_images",
    "test-desc_var.R" = "desc_var",
    "test-interprobe.R" = "interprobe"
  )
  key <- known[[basename]]
  if (is.null(key)) {
    stop("No prefix mapping for ", basename, call. = FALSE)
  }
  key
}

annotate_file <- function(path, bump) {
  basename <- basename(path)
  lines <- readLines(path, warn = FALSE)
  lines <- strip_test_ids(lines)
  out <- character()
  for (ln in lines) {
    if (grepl("^test_that\\(", ln)) {
      title <- test_title(ln)
      pfx <- prefix_for_line(basename, title)
      k <- bump(pfx)
      id_line <- sprintf("#%s_%03d", pfx, k)
      out <- c(out, id_line, ln)
    } else {
      out <- c(out, ln)
    }
  }
  writeLines(out, path)
  message("Wrote ", path)
}

script_dir <- function() {
  args <- commandArgs(trailingOnly = FALSE)
  z <- args[startsWith(args, "--file=")]
  if (length(z)) {
    return(dirname(normalizePath(sub("^--file=", "", z[1L]), winslash = "\\")))
  }
  getwd()
}

main <- function() {
  td <- file.path(script_dir())
  flist <- list.files(td, pattern = "^test-.*\\.R$", full.names = TRUE)
  if (!length(flist)) {
    stop("No test-*.R files in ", td, call. = FALSE)
  }
  flist <- sort(flist)
  # Cross-file backend checks last so function IDs (e.g. t.test2_001) stay in primary test files first.
  is_be <- basename(flist) == "test-backend_equivalence.R"
  flist <- c(flist[!is_be], flist[is_be])
  ctr <- new.env(parent = emptyenv())
  bump <- function(pfx) {
    cur <- ctr[[pfx]]
    if (is.null(cur)) {
      cur <- 0L
    }
    cur <- cur + 1L
    ctr[[pfx]] <- cur
    cur
  }
  for (f in flist) {
    annotate_file(f, bump)
  }
  invisible(NULL)
}

main()

# t.test2 print output for two-sample test is stable

    Code
      print(result)
    Output
      Two sample Welch t-test
      
            x      y   x-y  ci    ci.L  ci.H      t   df p.value N(x) N(y)      d
       101.03 103.16 -2.13 95% -11.105 6.845 -0.475 56.3   .6364   30   30 -0.123
      
      APA Style:
      t(56.3) = -0.48, p = .6364

# t.test2 print output for formula syntax is stable

    Code
      print(result)
    Output
      Two sample Welch t-test
      
       Group 1 Group 2   1-2  ci   ci.L   ci.H       t   df p.value N1 N2     d
         50.18   54.74 -4.56 95% -5.212 -3.908 -14.108 43.7  <.0001 25 25 -3.99
      
      Group 1: Control
      Group 2: Treatment
      
      APA Style:
      t(43.7) = -14.11, p < .0001

# t.test2 print output for paired test is stable

    Code
      print(result)
    Output
      Two sample paired t-test
      
       before   after before-after  ci   ci.L   ci.H      t df p.value  N
       101.92 106.105       -4.185 95% -5.742 -2.628 -5.627 19  <.0001 20
       r(before,after)
                  0.98
      
      APA Style:
      t(19.0) = -5.63, p < .0001

# t.test2 print output for one-sample test is stable

    Code
      print(result)
    Output
      One sample Student t-test
      
         mean  ci   ci.L    ci.H   t df p.value  N
       101.03 95% 94.002 108.058 0.3 29   .7665 30
      
      APA Style:
      t(29.0) = 0.30, p = .7665

# t.test2 print output with missing data is stable

    Code
      print(result)
    Output
      Two sample Welch t-test
      
             x      y    x-y  ci    ci.L  ci.H      t   df p.value N(x) N(y)     d
       101.026 103.16 -2.134 95% -11.737 7.469 -0.446 49.6   .6572   27   30 -0.12
      
      APA Style:
      t(49.6) = -0.45, p = .6572
      
      note: 'x' is missing 3 of 30 values, while 'y' is missing 0 of 30

# t.test2 print output with long group names is stable

    Code
      print(result)
    Output
      Two sample Welch t-test
      
       Group 1 Group 2    1-2  ci    ci.L  ci.H      t df p.value N1 N2      d
        72.295   76.92 -4.625 95% -12.412 3.162 -1.203 37   .2365 20 20 -0.381
      
      Group 1: Control Group B
      Group 2: Experimental Group A
      
      APA Style:
      t(37.0) = -1.20, p = .2365

# table2 print output for frequency table is stable

    Code
      print(result)
    Output
              response
      gender   Maybe No Yes
        Female     9 25  22
        Male      13 17  14

# table2 print output with row proportions is stable

    Code
      print(result)
    Output
              response
      gender   No Yes
        Female 29  27
        Male   26  18
      
      Row proportions:
              response
      gender   No    Yes   Total
        Female 0.518 0.482 1.000
        Male   0.591 0.409 1.000

# table2 print output with column proportions is stable

    Code
      print(result)
    Output
              response
      gender   No Yes
        Female 29  27
        Male   26  18
      
      Column proportions:
              response
      gender   No    Yes  
        Female 0.527 0.600
        Male   0.473 0.400
        Total  1.000 1.000

# table2 print output with chi-square test is stable

    Code
      print(result)
    Output
              response
      gender   No Yes
        Female 29  27
        Male   26  18
      
      
      	Pearson's Chi-squared test
      
      data:  result
      X-squared = 0.53129, df = 1, p-value = 0.4661
      

# table2 print output for three-way table is stable

    Code
      print(result)
    Output
      , , z = High
      
         y
      x    X  Y
        A  9  6
        B  5 11
      
      , , z = Low
      
         y
      x    X  Y
        A  3  9
        B  9  8
      

# Unit test index

This document lists **testthat** files under `tests/testthat/`, the **R source files** they primarily exercise, and short notes on what **groups of tests** cover. Test titles are quoted as in code (`test_that("...", { ... })`).

### Test IDs

Each `test_that()` block is preceded by a comment of the form `#<function>_<NNN>` (for example `#t.test2_001`). The name is the **R function** under test (`t.test2`, `plot_freq`, `scatter.gam`, `print.table2`, …). Numbers are **unique per function across the whole suite** (assigned in sorted file order, with `test-backend_equivalence.R` processed **last** so primary test files keep the lowest IDs). To refresh IDs after edits, run `Rscript tests/testthat/_annotate_test_ids.R` from the package root.

---

## `test-backend_equivalence.R` → cross-cutting (`R/t.test2.R`, `R/lm2.R`, `R/interprobe.R`, …)

Regression-style checks that **statuser** outputs stay aligned with reference implementations (base **stats**, **estimatr**, **marginaleffects**).

| Subset | What it checks |
|--------|----------------|
| **t.test2 vs `stats::t.test`** | Default and formula interfaces agree on core two-sample scenarios. |
| **lm2 vs `estimatr::lm_robust`** | Coefficients and inference match for a representative linear model. |
| **interprobe** | Johnson–Neyman / regions of significance and internal interaction **p**-values agree with **marginaleffects**-based references (binary **x** case). |

Helper code: `tests/testthat/helper-backend_equivalence.R`.

---

## `test-convert_to_sql.R` → `R/convert_to_sql.R`

| Subset | What it checks |
|--------|----------------|
| **File output** | Writes a SQL file; optional `INSERT` generation. |
| **Schema** | `create_table` behaviour. |
| **Types** | Date and numeric column handling. |
| **Safety** | Escaping single quotes in string data. |
| **Return value** | Invisible return. |

---

## `test-desc_var.R` → `R/desc_var.R`

| Subset | What it checks |
|--------|----------------|
| **Basics** | Correct summary stats; `digits`; expected column set. |
| **Inputs** | Data frames; columns named like arguments (`y`, `group`); formula and multi-way grouping; sorting of results. |
| **Missing / empty** | `NA` handling; empty groups; all-**NA** **y**; single observation per group. |
| **Diagnostics** | Missing factor combinations; perfectly overlapping groupers. |
| **Errors** | Non-numeric **y**; truly missing variables; evaluation order for `df$col` vs validation. |

---

## `test-format_pvalue.R` → `R/format_pvalue.R`

| Subset | What it checks |
|--------|----------------|
| **Formatting** | Ordinary **p**-values; vector input; `digits`; `include_p` / “includes_p” variants. |
| **Rules** | Threshold vs `digits`; boundary values; values outside \([0,1]\); negative inputs. |
| **Edge cases** | `NA`; empty vectors; single `NA`. |

---

## `test-gam_functions.R` → `R/scatter.gam.R`, `R/plot_gam.R`, `R/twolines.R`

| Subset | What it checks |
|--------|----------------|
| **scatter.gam** | Runs with data-frame and formula inputs; `data.dots`, `three.dots`, `k`; invalid data errors; `ylim` via `...` without duplicate-argument issues; sensible fitted values. |
| **plot_gam** | Errors for non-GAM objects; `predictor` and `quantile.others` validation; structure; reasonable predictions; multiple predictors. |
| **twolines** | `graph`, `quiet`, covariates; structure; U-shaped vs inverted-U detection. |

---

## `test-interprobe.R` → `R/interprobe.R` (and helpers in `R/interprobe_helpers.R`)

| Subset | What it checks |
|--------|----------------|
| **`lm` input** | Structure; fitted model returned when estimated internally; **APA** printing unless `quiet=TRUE`. |
| **Binary x** | **ti**-based GAM testing path (`gam_results_testing`). |
| **`lm2` input** | Works with **lm2** fits. |
| **`model` / `data` routing** | Fits **lm2** when `model = linear` (or sentinel) with data; vector interface with `model = linear`. |
| **Keyword vs object** | `linear` in calling env vs bare keyword; `lm` as keyword when appropriate. |
| **Errors** | Messages preserve intended model name. |

---

## `test-list2.R` → `R/list2.R`

| Subset | What it checks |
|--------|----------------|
| **Naming** | Names from symbols; explicit names; mixed named/unnamed. |
| **Edge cases** | Single argument; empty call. |
| **Values** | Preserved values; non-trivial expressions. |

---

## `test-lm2.R` → `R/lm2.R`

| Subset | What it checks |
|--------|----------------|
| **Defaults** | Statuser-flavoured output; **HC3** SEs by default. |
| **Coefficients** | Standardized betas; agreement with **`lm`** / **`lm_robust`**; classical vs robust SE differences. |
| **`...`** | Extra args passed through to **`lm_robust`**. |
| **Predict** | With/without `newdata`; with/without clustering; `se.fit`; alignment with **`lm_robust`**. |
| **Ecosystem** | **marginaleffects** (`predict`, `avg_slopes`, comparisons); **broom** (`tidy`, `glance`, `augment`); **modelsummary** (single and multi-model). |
| **Diagnostics in print** | Heteroskedasticity flags (! / !!!); collinearity on interactions (X); **SE.cluster** when clustered; cluster SE &gt; robust when clustering matters. |
| **`print.lm2`** | `notes` toggles; Call line; summary stats; interaction notes. |
| **Mean column** | Numeric vs factor level percentages. |
| **Data** | Missing values; single/many predictors; with/without `data` (env vs explicit). |
| **Snapshots** | Stable print output for representative cases. |

---

## `test-message2.R` → `R/message2.R`

| Subset | What it checks |
|--------|----------------|
| **Rendering** | Default message; colours; `font`; `stop`; multi-part messages. |
| **Return** | Invisible return. |

---

## `test-plot_cdf.R` → `R/plot_cdf.R`

| Subset | What it checks |
|--------|----------------|
| **Smoke / inputs** | Runs; `data` frame; formula; `df$col` on LHS/RHS without bare names in env; missing values. |
| **Options** | Custom args; `show.ks`; `show.quantiles`; group count variants. |
| **Return / errors** | Object structure; dataset name in errors. |
| **Edge cases** | Single group; all-**NA**; very small **n** per group. |
| **Order / factors** | `order`, `order = -1`, factor levels when `order` is `NULL`. |
| **Two-vector mode** | Basic and custom params; `order`; unequal lengths; legend margin; `xlim` padding. |

---

## `test-plot_density.R` → `R/plot_density.R`

| Subset | What it checks |
|--------|----------------|
| **Smoke / inputs** | Same themes as **plot_cdf**: runs, `data`, formula, `df$col`, missing. |
| **Options** | Custom parameters; `show.means`; passthrough to **`density()`**. |
| **Groups** | Different counts; single vs many groups. |
| **Return / errors** | Structure; dataset name in errors. |
| **Order / factors** | `order`, `-1`, factor levels. |
| **Two-vector mode** | Custom params; `order`; length mismatch; legend space. |

---

## `test-plot_freq.R` → `R/plot_freq.R`

| Subset | What it checks |
|--------|----------------|
| **Smoke / inputs** | Runs; `data`; grouping; formula / `df$col`; `freq` and general custom args; `value.labels`, `show.x.value`, `ticks.max`, `show.legend`. |
| **Return / errors** | Structure; dataset name in errors; missing values. |
| **Order** | Named order, validation, three groups, default sort, `order = -1` (including with factors). |
| **Two-vector mode** | Structure, params, `order`, missing, validation, name deduction, `data` columns, continuous **x**, `col`/`freq`. |
| **Layout** | `xlim` covers bars; pretty ticks for many levels; legend space. |

---

## `test-plot_means.R` → `R/plot_means.R`

| Subset | What it checks |
|--------|----------------|
| **Integration** | Formula + `data` returns **`desc_var`**-like object; multi-way formulas (`+`); `order`; three-way grouping with sparse cells. |
| **`tests = "auto"`** | Three scenarios (binary **x1** only; binary **x1** and **x2**; **x2** with &gt;2 levels). |
| **Numerical agreement** | Cell means/CIs vs **`t.test`** (no clusters); **Welch** **t** in scenario 1; 2×2 interaction **p** vs **`lm_robust`** **HC3**; clustered CIs follow `ci.level`. |

---

## `test-print_functions.R` → `R/print_table2.R` (methods for **table2** objects)

| Subset | What it checks |
|--------|----------------|
| **print.table2** | Frequency tables; proportion tables; three-way tables; tables without dimension names. |

---

## `test-resize_images.R` → `R/resize_images.R`

| Subset | What it checks |
|--------|----------------|
| **Robustness** | Missing folder; empty folder. |
| **Parameters** | `width` accepted. |

---

## `test-table2.R` → `R/table2.R`

| Subset | What it checks |
|--------|----------------|
| **Core** | One- and multi-way tables; `data` columns; `prop`, `digits`, `useNA`, `exclude`. |
| **Chi-square** | `chi` output; correction; match **`chisq.test`** / **`prop.test`** where applicable. |
| **Proportions** | Row/column/`ALL` vs **`prop.table`**; row/column sums. |
| **Single vector** | Structure; print; `prop`+`chi` prints chi once. |
| **Comparisons in cells** | Logical expressions in 1D/2D; truncation; with `prop` and `chi`. |
| **Equivalence** | Frequencies vs **`table()`**; **`freq`** snapshot stability for several print styles. |

---

## `test-text2.R` → `R/text2.R`

| Subset | What it checks |
|--------|----------------|
| **Layout** | Alignments (incl. vectorized); padding; `cex`; extra **`graphics::text`** args. |
| **Styling** | Per-label `col`; recycled `bg`; multiple labels. |
| **Edge cases** | Small/special cases covered in the last block. |

---

## `test-t.test2.R` → `R/t.test2.R` (and printing in `R/print_t.test2.R`)

| Subset | What it checks |
|--------|----------------|
| **API** | Export; class; column set. |
| **Inputs** | Formula with/without `data`; `df$col` formula; two vectors; paired; one-sample. |
| **Column labels** | Short names; long names; 0/1 as `var=value`; long names with 0/1. |
| **Missing data** | Reporting for two-sample and paired. |
| **Equivalence** | Matches **`stats::t.test`** (incl. paired, one-sample, formula, small **n**, extremes, factor/numeric grouping). |
| **Environment** | Variables resolved from env when `data` omitted. |
| **Print snapshots** | Stable printed output across designs (incl. missing and long names). |

---

## `test-stimulus.plot.R` → `R/stimulus.plot.R`, `R/stimulus.beeswarm.R`, `R/clear_stimulus_cache.R`, `R/stimulus_auxiliary.R`

| Subset | What it checks |
|--------|----------------|
| **Means plot** | Returns data frame with per-stimulus effect columns. |
| **Effects plot** | Returns list with observed effects, null band, resamples, heterogeneity p-value. |
| **Design validation** | Compared-stimulus data errors with message pointing to `stimulus.beeswarm()`. |
| **Beeswarm** | Runs on compared-stimulus subset; returns coordinate matrix. |
| **Cache** | `clear_stimulus_cache()` empties package resampling cache. |
| **Regression fixtures** | Salerno & Slepian effects/means outputs match saved RDS reference (tolerance 1e-4); model=all subset same pattern. |

Fixtures: `tests/testthat/data/stimulus_saved_results/*.rds`.

---

## `test-validate_plot.R` → `R/validate.R` (`validate_plot()`)

| Subset | What it checks |
|--------|----------------|
| **Happy paths** | Standard `(y, group)`; `data`; formula with/without `data`; optional `group`; `df$var`. |
| **Errors** | Invalid inputs; messages include variable name, function name, and length mismatch detail. |
| **Edge cases** | `y ~ 1`; explicit `data_name`. |

---

## `tests/testthat.R`

Loads **testthat** and the package; not a test script itself.

---

## Maintenance

After adding or renaming tests, regenerate this index (or extend the table for new `test-*.R` files). The canonical list of test files is:

`list.files("tests/testthat", pattern = "^test-.*\\.R$")`.

# lm2 print output format is stable

    Code
      print(result)
    Output
      Call: lm2(formula = mpg ~ wt + hp, data = mtcars)
      
                  estimate  SE.robust SE.classical t.value p.value std.estimate
      intercept    37.23**    2.230      1.599      16.70   <.0001        --   
      wt          -3.878**    0.769      0.633     -5.046   <.0001    -0.630   
      hp          -0.032**    .0094      .0090     -3.385    .0021    -0.361   
                    mean missing red.flag
      intercept      --    --        !   
      wt          3.217    0         --  
      hp          146.7    0         --  
      
      N = 32  | missing = 0  | df = 29  | R² = 0.827  | SE type: HC3 
      
      Note: to see explanations for lm2() output, run: `lm2_notes()`

# lm2 print output with hint is stable

    Code
      print(result)
    Output
      Call: lm2(formula = mpg ~ wt, data = mtcars)
      
                  estimate  SE.robust SE.classical t.value p.value std.estimate
      intercept    37.29**    2.427      1.878      15.36   <.0001        --   
      wt          -5.344**    0.738      0.559     -7.241   <.0001    -0.868   
                    mean missing red.flag
      intercept      --    --        !   
      wt          3.217    0         !   
      
      N = 32  | missing = 0  | df = 30  | R² = 0.753  | SE type: HC3 
      
      Note: to see explanations for lm2() output, run: `lm2_notes()`

# lm2 print output with interaction is stable

    Code
      print(result)
    Output
      Call: lm2(formula = mpg ~ wt * hp, data = mtcars)
      
                  estimate  SE.robust SE.classical t.value p.value std.estimate
      intercept    49.81**    5.113      3.605      9.741   <.0001        --   
      wt          -8.217**    1.655      1.270     -4.965   <.0001    -1.334   
      hp          -0.120**    0.032      0.025     -3.732    .0009    -1.366   
      wt:hp        0.028**    .0096      .0074      2.888    .0074     0.310   
                    mean missing r(x,z)   red.flag
      intercept      --    --          --     !   
      wt          3.217    0           --     !   
      hp          146.7    0           --     !   
      wt:hp       514.7    --      0.66**     ! X 
      
      N = 32  | missing = 0  | df = 28  | R² = 0.885  | SE type: HC3 
      
      Note: to see explanations for lm2() output, run: `lm2_notes()`

# lm2_notes output is stable

    Code
      lm2_notes()
    Output
      Notes:
        - ** p<.01 (based on SE.robust)
        - std.estimate is the standardized coefficient: beta = b * sd(x) / sd(y)
        - mean: for numeric variables, mean of x; for factors, % of observations
        - missing: number of observations excluded due to missing values
        - red.flag:
           !: robust & classical SE differ by more than 25%
           - Suggestion 1: to evaluate possible extreme skew or outliers:
             plot_density() or plot_freq() for mpg and for wt
           - Suggestion 2: to evaluate possible nonlinearity, do plot_gam(mpg ~ wt)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate.R
\name{validate_formula}
\alias{validate_formula}
\title{Validate Formula Variables}
\usage{
validate_formula(
  formula,
  data = NULL,
  func_name = "function",
  calling_env = parent.frame()
)
}
\arguments{
\item{formula}{A potential formula object to validate (can be any object).}

\item{data}{An optional data frame containing the variables.}

\item{func_name}{Character string. Name of the calling function (for error messages).}

\item{calling_env}{The environment in which to look for variables if data is not provided.
Defaults to parent.frame().}
}
\value{
Returns NULL invisibly. Stops with an error if validation fails.
}
\description{
Checks if the input is a formula and validates that all variables mentioned
in the formula exist either in the provided data frame or in the environment.
This is a lightweight validation function that should be called early in functions
that accept formula syntax.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var_labels.R
\name{var_labels}
\alias{var_labels}
\alias{var_labels<-}
\title{Get or set variable labels}
\usage{
var_labels(x)

var_labels(x) <- value
}
\arguments{
\item{x}{A vector or a data frame.}

\item{value}{Character label(s) to assign.}
}
\value{
- If \code{x} is a data frame: a named character vector with one entry per
  column (missing labels are returned as \code{NA_character_}).
- Otherwise: a single character string (or \code{NA_character_}).
}
\description{
Assign labels to describe variables in a data frame.
Can be called on a single variable or full data, to assign or to read existing
labels.
}
\details{
For a vector, this reads/writes a base-R \code{"label"} attribute.
For a data frame, this reads/writes the \code{"label"} attribute of each
column.
}
\examples{
df <- data.frame(x = 1:3, y = 4:6)

# Set labels for all columns
var_labels(df) <- c("this is x", "this is y")
var_labels(df)

# Set a label for a single column
var_labels(df$x) <- "this is x"
var_labels(df$x)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate.R
\name{validate_plot}
\alias{validate_plot}
\title{Validate Inputs for Plotting Functions}
\usage{
validate_plot(
  y,
  group = NULL,
  data = NULL,
  func_name = "plot",
  require_group = TRUE,
  data_name = NULL
)
}
\arguments{
\item{y}{A numeric vector, column name, or formula of the form y ~ group.}

\item{group}{A vector used to group the data, or a column name if data is provided.
Can be NULL for functions where group is optional.}

\item{data}{An optional data frame containing the variables.}

\item{func_name}{Character string. Name of the calling function (for error messages).}

\item{require_group}{Logical. If TRUE, group is required. If FALSE, group can be NULL.}

\item{data_name}{Character string. Name of the data argument (for error messages). 
If NULL, will attempt to infer from the call.}
}
\value{
A list containing:
\itemize{
  \item \code{y}: Validated y variable (numeric vector)
  \item \code{group}: Validated group variable (if provided, otherwise NULL)
  \item \code{y_name}: Clean variable name for y (for labels)
  \item \code{group_name}: Clean variable name for group (for labels)
  \item \code{y_name_raw}: Raw variable name for y (for error messages)
  \item \code{group_name_raw}: Raw variable name for group (for error messages)
  \item \code{data_name}: Name of data argument (for error messages)
}
}
\description{
Validates inputs for plotting functions that accept either formula syntax
(y ~ group) or standard syntax (y, group), with optional data frame.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list2.R
\name{list2}
\alias{list2}
\title{Enhanced alternative to list()}
\usage{
list2(...)
}
\arguments{
\item{...}{Objects to include in the list. Objects are automatically named 
based on their variable names unless explicit names are provided.}
}
\value{
A named list. Each element is named after the variable passed to 
  the function (or the explicit name if provided). The structure is identical 
  to a standard R list created with \code{\link[base]{list}}.
}
\description{
List with objects that are automatically named.
}
\details{
list2(x , y)      is equivalent to list(x = x , y = y)

list2(x , y2 = y) is equivalent to list(x = x , y2 = y)

Based on: \url{https://stackoverflow.com/questions/16951080/can-lists-be-created-that-name-themselves-based-on-input-object-names}
}
\examples{
x <- 1:5
y <- letters[1:3]
z <- matrix(1:4, nrow = 2)

# Create named list from objects
my_list <- list2(x, y, z)
names(my_list)  # "x" "y" "z"

# Works with explicit names too
my_list2 <- list2(a = x, b = y)
names(my_list2)  # "a" "b"

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twolines.R
\name{twolines}
\alias{twolines}
\title{Two-Lines Test of U-Shapes}
\usage{
twolines(
  f,
  graph = 1,
  link = "gaussian",
  data = NULL,
  pngfile = "",
  quiet = FALSE
)
}
\arguments{
\item{f}{A formula object specifying the model (e.g., y ~ x1 + x2 + x3).
The first predictor is the one tested for a u-shaped relationship.}

\item{graph}{Integer. If 1 (default), produces a plot. If 0, no plot is generated.}

\item{link}{Character string specifying the link function for the GAM model.
Default is "gaussian".}

\item{data}{An optional data frame containing the variables in the formula.
If not provided, variables are evaluated from the calling environment.}

\item{pngfile}{Optional character string. If provided, saves the plot to a PNG file
with the specified filename.}

\item{quiet}{Logical. If TRUE, suppresses the Robin Hood details messages.
Default is FALSE.}
}
\value{
A list containing:
\itemize{
  \item All elements from \code{reg2()}: \code{b1}, \code{b2}, \code{p1}, \code{p2},
    \code{z1}, \code{z2}, \code{u.sig}, \code{xc}, \code{glm1}, \code{glm2}, \code{rob1},
    \code{rob2}, \code{msg}, \code{yhat.smooth}
  \item \code{yobs}: Observed y values (adjusted for covariates if present)
  \item \code{y.hat}: Fitted values from GAM
  \item \code{y.ub}, \code{y.lb}: Upper and lower bounds for fitted values
  \item \code{y.most}: Most extreme fitted value
  \item \code{x.most}: x-value associated with most extreme fitted value
  \item \code{f}: Formula as character string
  \item \code{bx1}, \code{bx2}: Linear and quadratic coefficients from preliminary quadratic regression
  \item \code{minx}: Minimum x value
  \item \code{midflat}: Median of flat region
  \item \code{midz1}, \code{midz2}: Z-statistics at midpoint
}
}
\description{
Implements the two-lines test for U-shaped (or inverted U-shaped) relationships 
introduced by Simonsohn (2018).
}
\details{
Reference: 
Simonsohn, Uri (2018) "Two lines: A valid alternative to the invalid testing of 
U-shaped relationships with quadratic regressions." AMPPS, 538-555. 
\doi{10.1177/2515245918805755}


The test beings fitting a GAM model, predicting y with a smooth of x, and optionally with covariates. 
It identifies the interior most extreme value of fitted y, and adjusts from the matching x-value 
to set the breakpoint relying on the Robin Hood procedure introduced also by Simonsohn (2018).
It then estimates the (once) interrupted regression using that breakpoint,
and reports the slope and significance of the average slopes at either side of it. A U-shape 
is significant if the slopes are of opposite sign and are both individually significant.
}
\examples{
\donttest{
# Simple example with simulated data
set.seed(123)
x <- rnorm(100)
y <- -x^2 + rnorm(100)
data <- data.frame(x = x, y = y)
result <- twolines(y ~ x, data = data)

# With covariates
z <- rnorm(100)
y <- -x^2 + 0.5*z + rnorm(100)
data <- data.frame(x = x, y = y, z = z)
result <- twolines(y ~ x + z, data = data)

# Without data argument (variables evaluated from environment)
x <- rnorm(100)
y <- -x^2 + rnorm(100)
result <- twolines(y ~ x)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_freq.R
\name{plot_freq}
\alias{plot_freq}
\title{Plot frequencies for a variable (histogram without binning)}
\usage{
plot_freq(
  formula,
  y2 = NULL,
  data = NULL,
  freq = TRUE,
  order = NULL,
  col = "dodgerblue",
  lwd = 9,
  width = NULL,
  value.labels = "auto",
  ticks.max = 30,
  show.x.value = "auto",
  show.legend = TRUE,
  legend.title = NULL,
  col.text = NULL,
  ...
)
}
\arguments{
\item{formula}{Two possible uses (similar to \code{t.test()}):
\itemize{
  \item {Single Variable (possibly by subgroup)}: \code{plot_freq(y)} or \code{plot_freq(y~x)}
  \item {Contrast Two Variables}: \code{plot_freq(y1, y2)}
}}

\item{y2}{optional second variable when contrasting two variables \code{plot_freq(y1,y2)}}

\item{data}{An optional data frame containing the variables in the formula.}

\item{freq}{Logical. If TRUE (default), displays frequencies. If FALSE, displays percentages.}

\item{order}{Controls the order in which groups appear in the plot and legend. 
Use \code{-1} to reverse the default order. Alternatively, provide a vector specifying
the exact order (e.g., \code{c("B", "A", "C")}). If \code{NULL} (default), groups are 
ordered by their factor levels (if the grouping variable is a factor) or sorted 
alphabetically/numerically. Only applies when using grouped plots or comparing two variables.}

\item{col}{Color for the bars.}

\item{lwd}{Line width for the frequency bars. Default is 9.}

\item{width}{Numeric. Width of the frequency bars. If NULL (default), width is automatically calculated based on the spacing between values.}

\item{value.labels}{Controls value labeling. If numeric, shows labels for the \code{value.labels}
highest-frequency values (including ties). Use \code{-1} or \code{"all"} to show all labels and
\code{0} to show none. Use \code{"auto"} to label all values when there are 30 or fewer
unique values; otherwise label the single most frequent value. For backward compatibility, \code{TRUE}
is treated as \code{-1} and \code{FALSE} as \code{0}.}

\item{ticks.max}{Integer. Maximum number of unique x values to label on the x-axis. If there are more
than \code{ticks.max} unique values, \code{pretty()} ticks are used instead of labeling every value.}

\item{show.x.value}{Either \code{"auto"} (default), \code{TRUE}, or \code{FALSE}. If enabled, draws a
small \code{x=<value>} label just above each frequency value label. When \code{"auto"}, x labels are
shown only when plot_freq is not already labeling all x values on the x-axis.}

\item{show.legend}{Logical. If TRUE (default), displays a legend when \code{group} is specified. If FALSE, no legend is shown.}

\item{legend.title}{Character string. Title for the legend when \code{group} is specified. If NULL (default), no title is shown.}

\item{col.text}{Color for the value labels. If not specified, uses \code{col} for non-grouped plots or group colors for grouped plots.}

\item{...}{Pass on any argument accepted by \code{plot()} e.g., \code{xlab='x-axis'} , \code{main='Distribution of X'}}
}
\value{
Invisibly returns a data frame with values and their frequencies.
}
\description{
Creates a frequency plot showing the frequency of every observed value, optionally by group. 
Most frequent values are labeled by default.
}
\details{
This function creates a frequency plot where each observed value is shown
with its frequency. Unlike a histogram (no binning) and unlike a barplot
(which omits unobserved levels), unobserved values are shown at frequency 0.
}
\examples{
# Simple example
x <- c(1, 1, 2, 2, 2, 5, 5)
plot_freq(x)

# Pass on some common \code{plot()} arguments
plot_freq(x, col = "steelblue", xlab = "Value", ylab = "Frequency",ylim=c(0,7))

# Add to an existing plot
plot_freq(x, col = "dodgerblue")


# Compare two vectors
y1 <- c(1, 1, 2, 2, 2, 5, 5)
y2 <- c(1, 2, 2, 3, 3, 3)
plot_freq(y1, y2)

# Using a data frame with grouping
df <- data.frame(value = c(1, 1, 2, 2, 2, 5, 5), group = c("A", "A", "A", "B", "B", "A", "B"))
plot_freq(value ~ 1, data = df)  # single variable
plot_freq(value ~ group, data = df)  # with grouping

# Control group order in legend and plot
plot_freq(value ~ group, data = df, order = c("B", "A"))  # B first, then A
plot_freq(value ~ group, data = df, order = -1)  # Reverse default order

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate.R
\name{validate_lm2}
\alias{validate_lm2}
\title{Validate Inputs for lm2() Function}
\usage{
validate_lm2(
  formula,
  data = NULL,
  se_type = "HC3",
  se_type_missing = TRUE,
  dots = list(),
  calling_env = parent.frame()
)
}
\arguments{
\item{formula}{A formula specifying the model.}

\item{data}{An optional data frame containing the variables.}

\item{se_type}{The type of standard error to use.}

\item{se_type_missing}{Logical. Whether se_type was not explicitly provided by user.}

\item{dots}{Additional arguments passed to lm_robust (to check for clusters).}

\item{calling_env}{The environment in which to look for variables if data is not provided.}
}
\value{
A list containing:
\itemize{
  \item \code{data}: The data frame to use (either provided or constructed from vectors)
  \item \code{se_type}: The validated/adjusted se_type
  \item \code{has_clusters}: Logical indicating if clusters are being used
}
}
\description{
Validates se_type and clusters arguments for lm2().
Also handles creating a data frame from vectors if data is not provided.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interprobe.R
\name{interprobe}
\alias{interprobe}
\title{Probe interactions robustly to nonlinearities}
\usage{
interprobe(
  x = NULL,
  z = NULL,
  y = NULL,
  model = NULL,
  data = NULL,
  moderator.on.x.axis = TRUE,
  k = NULL,
  spotlights = NULL,
  spotlight.labels = NULL,
  histogram = TRUE,
  max.unique = 11,
  n.bin.continuous = 10,
  n.max = 50,
  xlab = "",
  ylab1 = "",
  ylab2 = "",
  cols = c("red4", "dodgerblue", "green4"),
  main1 = "GAM Simple Slopes",
  main2 = "GAM Johnson-Neyman",
  legend.round = c(1, 4),
  draw = "both",
  save.as = NULL,
  xlim = NULL,
  ylim1 = NULL,
  ylim2 = NULL,
  x.ticks = NULL,
  y1.ticks = NULL,
  y2.ticks = NULL,
  legend.simple.slopes = NULL,
  legend.jn = NULL,
  quiet = FALSE,
  probe.bins = 100
)
}
\arguments{
\item{x}{The focal predictor. Can be a name (bare or quoted) when `data` or `model`
is provided, or a numeric/factor vector when probing from vectors.}

\item{z}{The moderator. Same options as `x`.}

\item{y}{The dependent variable. Same options as `x`. Not required when `model` is supplied.}

\item{model}{By default `interprobe` estimates a GAM model predicting `y` with `x` and `z`.
You can instead probe a linear interaction by setting model=linear. You can also probe a 
model of your choice by running it separately, saving the output, and submitting it as the model 
argument to  interprobe. This is the way to include covariates for a probed interaction.}

\item{data}{Optional data frame containing `x`, `z`, and `y`.}

\item{moderator.on.x.axis}{Logical. If TRUE (default), moderator (`z`) is shown on the x-axis.}

\item{k}{Integer. Smoothness parameter passed to `mgcv::gam()` when estimating with the default
GAM engine.}

\item{spotlights}{Numeric vector of length 3. Values at which curves are computed.}

\item{spotlight.labels}{Character vector of length 3. Labels for the legend.}

\item{histogram}{Logical. If TRUE (default), show sample size distribution under the plot.}

\item{max.unique}{Integer. Threshold for treating a variable as continuous vs discrete.}

\item{n.bin.continuous}{Integer. Number of bins used in histogram when binning continuous values.}

\item{n.max}{Integer. Sample size at which line darkness/width saturates.}

\item{xlab}{Character. X-axis label.}

\item{ylab1}{Character. Y-axis label for simple slopes panel.}

\item{ylab2}{Character. Y-axis label for JN panel.}

\item{cols}{Character vector of length 3. Colors for the three curves.}

\item{main1}{Character. Title for simple slopes panel.}

\item{main2}{Character. Title for JN panel.}

\item{legend.round}{Integer vector length 2. Min/max decimals in legend.}

\item{draw}{Which plots to draw: `"both"` (default), `"simple slopes"` (or legacy `"simple.slopes"`), or `"jn"`.}

\item{save.as}{Optional file path to save plot (`.png` or `.svg`).}

\item{xlim}{Numeric vector length 2. X-axis limits.}

\item{ylim1}{Numeric vector length 2. Y-axis limits for simple slopes.}

\item{ylim2}{Numeric vector length 2. Y-axis limits for JN.}

\item{x.ticks}{Optional custom x-axis ticks.}

\item{y1.ticks}{Optional custom y-axis ticks for panel 1.}

\item{y2.ticks}{Optional custom y-axis ticks for panel 2.}

\item{legend.simple.slopes}{Optional legend title for simple slopes.}

\item{legend.jn}{Optional legend title for JN.}

\item{quiet}{Logical. If TRUE, reduces console output.}

\item{probe.bins}{Integer. Resolution for probing curves (larger = smoother/slower).}
}
\value{
Invisibly returns a list with:
\itemize{
  \item \code{simple.slopes}: data.frame of predicted values and confidence intervals
  \item \code{johnson.neyman}: data.frame of marginal effects and confidence intervals
  \item \code{frequencies}: data.frame with bin frequencies used for shading/histogram
  \item \code{gam_results}: the fitted GAM model when estimated inside \code{interprobe()}
  \item \code{gam_results_testing}: when \code{interprobe()} estimates a GAM internally and \code{x}
    has exactly 2 unique values, a separate GAM fit used for interaction testing (with a \code{ti()}
    term and numeric coding of \code{x})
  \item \code{lm2_results}: when \code{interprobe()} estimates a GAM internally, also returns the
    corresponding linear fit \code{lm2(y ~ x * z)} (or \code{NULL} if package \code{estimatr}
    is not installed)
}
}
\description{
Probes an interaction by estimating (or accepting) a model and computing:
- simple slopes ("spotlights") using predicted values
- Johnson-Neyman ("jn") curves using marginal effects
}
\details{
Designed for GAM models but works with any model supported by `marginaleffects`
(including `lm`, `glm`, `mgcv::gam`, and `lm2` / `estimatr::lm_robust`).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2.R
\name{table2}
\alias{table2}
\title{Enhanced alternative to table()}
\arguments{
\item{...}{same arguments as \code{\link[base]{table}}, plus the arguments shown below}

\item{prop}{report a table with:
\itemize{
  \item \code{prop="all"}: Proportions for full table (each cell / total)
  \item \code{prop="row"}: Proportions by row  ('rows' also accepted)
  \item \code{prop="col"}: Proportions by columns ('cols', 'column', 'columns' also accepted)
}}

\item{digits}{Number of decimal values to show for proportions}

\item{chi}{Logical. If \code{TRUE}, performs a chi-square test on frequency table,
reports results in APA format}

\item{correct}{Logical. If \code{TRUE}, applies Yates' continuity correction 
for 2x2 tables in the chi-square test. Default is \code{FALSE} (no correction).}
}
\value{
A list (object of class "table2") with the following components:
  \itemize{
    \item \code{freq}: frequency table 
    \item \code{prop}: proportions table 
    \item \code{chisq}: chi-square test  
  }
}
\description{
The function \code{\link[base]{table}} does not show variable 
 names when tabulating from a dataframe, requires running another
 function, \code{\link[base]{prop.table}},  to tabulate proportions 
 and yet another function, \code{\link[stats]{chisq.test}} to test difference of 
 proportions.  \code{table2} does what those three functions do, producing easier to 
 read output, and always shows variable names.
}
\examples{
# Create example data
df <- data.frame(
  group = c("A", "A", "B", "B", "A"),
  status = c("X", "Y", "X", "Y", "X")
)

# Enhanced table with variable names (2 variables)
table2(df$group, df$status)

# Enhanced table with variable names (3 variables)
df3 <- data.frame(
  x = c("A", "A", "B", "B"),
  y = c("X", "Y", "X", "Y"),
  z = c("high", "low", "high", "low")
)
table2(df3$x, df3$y, df3$z)

# Table with proportions
table2(df$group, df$status, prop = 'all')  # Overall proportions
table2(df$group, df$status, prop = 'row')  # Row proportions
table2(df$group, df$status, prop = 'col')  # Column proportions

# Table with chi-square test
table2(df$group, df$status, chi = TRUE,prop='all')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{evaluate_variable_arguments}
\alias{evaluate_variable_arguments}
\title{Evaluate Variable Arguments with NSE Support}
\usage{
evaluate_variable_arguments(
  arg_expr,
  arg_name = "arg",
  data = NULL,
  calling_env = parent.frame(),
  func_name = "function",
  allow_null = FALSE
)
}
\arguments{
\item{arg_expr}{Unevaluated expression (from match.call() or substitute())}

\item{arg_name}{Character string. Name of the argument (for error messages)}

\item{data}{Optional data frame to look up columns}

\item{calling_env}{Environment to evaluate symbols if data is NULL}

\item{func_name}{Character string. Name of calling function (for error messages)}

\item{allow_null}{Logical. If TRUE, NULL is a valid input. Default FALSE.}
}
\value{
A list with:
\itemize{
  \item \code{value}: The actual data (vector, formula, or NULL)
  \item \code{name}: Clean name for labels (e.g., "DV1")
  \item \code{name_raw}: Raw name for error messages (e.g., "df$DV1")
  \item \code{was_symbol}: Logical. TRUE if input was an unquoted name
}
}
\description{
Resolves bare symbols (unquoted names) or quoted strings to actual data,
either from a data frame or the calling environment. This enables both
\code{plot_density(DV1, data=df)} and \code{plot_density("DV1", data=df)}
to work identically.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_var.R
\name{desc_var}
\alias{desc_var}
\title{Describe a variable, optionally by groups}
\usage{
desc_var(y, group = NULL, data = NULL, digits = 3)
}
\arguments{
\item{y}{A numeric vector of values, a column name (character string or unquoted) if \code{data} is provided,
or a formula of the form \code{y ~ x} or \code{y ~ x1 + x2} (for multiple grouping variables).}

\item{group}{Optional grouping variable, if not provided computed for the full data.
Ignored if \code{y} is a formula.}

\item{data}{Optional data frame containing the variable(s).}

\item{digits}{Number of decimal places to round to. Default is 3.}
}
\value{
A data frame with one row per group (or one row if no group is specified) containing:
  \itemize{
    \item \code{group}: Group identifier
    \item \code{mean}: Mean
    \item \code{sd}: Standard deviation
    \item \code{se}: Standard error
    \item \code{median}: Median
    \item \code{min}: Minimum
    \item \code{max}: Maximum
    \item \code{mode}: Most frequent value
    \item \code{freq_mode}: Frequency of mode
    \item \code{mode2}: 2nd most frequent value
    \item \code{freq_mode2}: Frequency of 2nd mode
    \item \code{n.total}: Number of observations
    \item \code{n.missing}: Number of observations with missing (NA) values
    \item \code{n.unique}: Number of unique values
  }
  Columns may also carry a \code{"label"} attribute, which provides a short
  human-readable description of each column.
}
\description{
Returns a dataframe with one row per group.
}
\details{
The dependent variable (`y`) must be numeric. If you pass an existing
non-numeric variable (e.g., character, factor), `desc_var()` will stop with
a clear error indicating that the variable must be numeric.
}
\examples{
# With grouping
df <- data.frame(y = rnorm(100), group = rep(c("A", "B"), 50))
desc_var(y, group, data = df)

# Without grouping (full dataset)
desc_var(y, data = df)

# Direct vectors
y <- rnorm(100)
group <- rep(c("A", "B"), 50)
desc_var(y, group)

# With custom decimal places
desc_var(y, group, data = df, digits = 2)

# Using formula syntax: y ~ x
desc_var(y ~ group, data = df)

# Using formula syntax with multiple grouping variables: y ~ x1 + x2
df2 <- data.frame(y = rnorm(200), x1 = rep(c("A", "B"), 100), x2 = rep(c("X", "Y"), each = 100))
desc_var(y ~ x1 + x2, data = df2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate.R
\name{validate_table2}
\alias{validate_table2}
\title{Validate Inputs for table2() Function}
\usage{
validate_table2(..., data = NULL, func_name = "table2", data_name = NULL)
}
\arguments{
\item{...}{One or more variables to be tabulated.}

\item{data}{An optional data frame containing the variables.}

\item{func_name}{Character string. Name of the calling function (for error messages).
Default is "table2".}

\item{data_name}{Character string. Name of the data argument (for error messages).
If NULL, will attempt to infer from the call.}
}
\value{
A list containing:
\itemize{
  \item \code{dots}: List of evaluated variables (ready for base::table)
  \item \code{dot_expressions}: List of expressions (for variable name extraction)
  \item \code{data_name}: Name of data argument (for error messages)
}
}
\description{
Validates inputs for table2() function that accepts multiple variables via ...
with optional data frame.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lm2.R
\name{predict.lm2}
\alias{predict.lm2}
\title{Predict method for lm2 objects}
\usage{
\method{predict}{lm2}(object, newdata, ...)
}
\arguments{
\item{object}{An object of class \code{lm2}}

\item{newdata}{An optional data frame in which to look for variables with 
which to predict. If omitted, the original model data is used.}

\item{...}{Additional arguments passed to \code{\link[estimatr]{predict.lm_robust}},
including \code{se.fit} and \code{interval}.}
}
\value{
A vector of predicted values (or a list with \code{fit} and \code{se.fit} 
  if \code{se.fit = TRUE}, or a matrix with \code{fit}, \code{lwr}, \code{upr} 
  if \code{interval} is specified)
}
\description{
Predict method for lm2 objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_density.R
\name{plot_density}
\alias{plot_density}
\title{Plot density of a variable, optionally by another variable}
\usage{
plot_density(
  formula,
  y2 = NULL,
  data = NULL,
  order = NULL,
  show_means = TRUE,
  ...
)
}
\arguments{
\item{formula}{Two possible uses (similar to \code{t.test()}):
\itemize{
  \item {Single Variable (possibly by subgroup)}: \code{plot_density(y)} or \code{plot_density(y~x)}
  \item {Contrast Two Variables}: \code{plot_density(y1, y2)}
}}

\item{y2}{optional second variable when contrasting two variables \code{plot_density(y1,y2)}}

\item{data}{An optional data frame containing the variables in the formula.}

\item{order}{Controls the order in which groups appear in the plot and legend. 
Use \code{-1} to reverse the default order. Alternatively, provide a vector specifying
the exact order (e.g., \code{c("B", "A", "C")}). If \code{NULL} (default), groups are 
ordered by their factor levels (if the grouping variable is a factor) or sorted 
alphabetically/numerically. Only applies when using grouped plots.}

\item{show_means}{Logical. If TRUE (default), shows points at means.}

\item{...}{Additional arguments passed to plotting functions.}
}
\value{
Invisibly returns a list with the following element:
  \describe{
    \item{densities}{A named list of density objects (class \code{"density"}), 
      one for each group. Each density object contains \code{x} (evaluation points), 
      \code{y} (density estimates), \code{bw} (bandwidth), and other components 
      as returned by \code{\link[stats]{density}}. If no grouping variable is 
      provided, the list contains a single element named \code{"all"}.}
  }
  The function is primarily called for its side effect of creating a plot.
}
\description{
Plots the distribution of a variable by group, simply: \code{plot_density(y ~ x)}
}
\details{
Plot parameters like
\code{col}, \code{lwd}, \code{lty}, and \code{pch} can be specified as:
\itemize{
  \item A single value: applied to all groups
  \item A vector: applied to groups in order of unique \code{group} values
}
}
\examples{
# Basic usage with formula syntax (no grouping)
y <- rnorm(100)
plot_density(y)

# With grouping variable
group <- rep(c("A", "B", "C"), c(30, 40, 30))
plot_density(y ~ group)

# With custom colors (scalar - same for all)
plot_density(y ~ group, col = "blue")

# With custom colors (vector - different for each group)
plot_density(y ~ group, col = c("red", "green", "blue"))

# Multiple parameters
plot_density(y ~ group, col = c("red", "green", "blue"), lwd = c(1, 2, 3))

# With line type
plot_density(y ~ group, col = c("red", "green", "blue"), lty = c(1, 2, 3), lwd = 2)

# Using data frame
df <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
plot_density(value ~ group, data = df)
plot_density(value ~ group, data = df, col = c("red", "blue"))

# Compare two vectors
y1 <- rnorm(50)
y2 <- rnorm(50, mean = 1)
plot_density(y1, y2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_cdf.R
\name{plot_cdf}
\alias{plot_cdf}
\title{Plot Empirical Cumulative Distribution Functions by Group}
\usage{
plot_cdf(
  formula,
  y2 = NULL,
  data = NULL,
  order = NULL,
  show.ks = TRUE,
  show.quantiles = TRUE,
  ...
)
}
\arguments{
\item{formula}{Two possible uses (similar to \code{t.test()}):
\itemize{
  \item {Single Variable (possibly by subgroup)}: \code{plot_cdf(y)} or \code{plot_cdf(y~x)}
  \item {Contrast Two Variables}: \code{plot_cdf(y1, y2)}
}}

\item{y2}{optional second variable when contrasting two variables \code{plot_cdf(y1,y2)}}

\item{data}{An optional data frame containing the variables in the formula.
If \code{data} is not provided, variables are evaluated from the calling environment.}

\item{order}{Controls the order in which groups appear in the plot and legend. 
Use \code{-1} to reverse the default order. Alternatively, provide a vector specifying
the exact order (e.g., \code{c("B", "A", "C")}). If \code{NULL} (default), groups are 
ordered by their factor levels (if the grouping variable is a factor) or sorted 
alphabetically/numerically. Only applies when using grouped plots.}

\item{show.ks}{Logical. If TRUE (default), shows Kolmogorov-Smirnov test results
when there are exactly 2 groups. If FALSE, KS test results are not displayed.}

\item{show.quantiles}{Logical. If TRUE (default), shows horizontal lines and results
at 25th, 50th, and 75th percentiles when there are exactly 2 groups. If FALSE,
quantile lines and results are not displayed.}

\item{...}{Additional arguments passed to plotting functions. Can be single values
(applied to all groups) or vectors (applied element-wise to each group).
Common parameters include \code{col}, \code{lwd}, \code{lty}, \code{pch},
\code{type}, etc.}
}
\value{
Invisibly returns a list containing:
  \itemize{
    \item \code{ecdfs}: A list of ECDF function objects, one per group. Each can be
      called as a function to compute cumulative probabilities (e.g., \code{result$ecdfs[[1]](5)}
      returns P(X <= 5) for group 1).
    \item \code{ks_test}: (Only when exactly 2 groups) The Kolmogorov-Smirnov test result
      comparing the two distributions. Access p-value with \code{result$ks_test$p.value}.
    \item \code{quantile_regression_25}: (Only when exactly 2 groups) Quantile regression
      model for the 25th percentile.
    \item \code{quantile_regression_50}: (Only when exactly 2 groups) Quantile regression
      model for the 50th percentile (median).
    \item \code{quantile_regression_75}: (Only when exactly 2 groups) Quantile regression
      model for the 75th percentile.
    \item \code{warnings}: Any warnings captured during execution (if any).
  }
}
\description{
Plots empirical cumulative distribution functions (ECDFs) separately for
each unique value of a grouping variable, with support for vectorized
plotting parameters. If no grouping variable is provided, plots a single ECDF.
}
\examples{
# Basic usage with single variable (no grouping)
y <- rnorm(100)
plot_cdf(y)

# Basic usage with formula syntax and grouping
group <- rep(c("A", "B", "C"), c(30, 40, 30))
plot_cdf(y ~ group)

# With custom colors (scalar - same for all)
plot_cdf(y ~ group, col = "blue")

# With custom colors (vector - different for each group)
plot_cdf(y ~ group, col = c("red", "green", "blue"))

# Multiple parameters
plot_cdf(y ~ group, col = c("red", "green", "blue"), lwd = c(1, 2, 3))

# With line type and point character
plot_cdf(y ~ group, col = c("red", "green", "blue"), lty = c(1, 2, 3), lwd = 2)

# Using data frame
df <- data.frame(value = rnorm(100), group = rep(c("A", "B"), 50))
plot_cdf(value ~ group, data = df)
plot_cdf(value ~ group, data = df, col = c("red", "blue"))

# Compare two vectors
y1 <- rnorm(50)
y2 <- rnorm(50, mean = 1)
plot_cdf(y1, y2)

# Formula syntax without data (variables evaluated from environment)
widgetness <- rnorm(100)
gender <- rep(c("M", "F"), 50)
plot_cdf(widgetness ~ gender)

# Using the returned object
df <- data.frame(value = c(rnorm(50, 0), rnorm(50, 1)), group = rep(c("A", "B"), each = 50))
result <- plot_cdf(value ~ group, data = df)

# Use ECDF to find P(X <= 0.5) for group A
result$ecdfs[[1]](0.5)

# Access KS test p-value
result$ks_test$p.value

# Summarize median quantile regression
summary(result$quantile_regression_50)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_t.test2.R
\name{print.t.test2}
\alias{print.t.test2}
\title{Print method for t.test2 output}
\usage{
\method{print}{t.test2}(x, ...)
}
\arguments{
\item{x}{An object of class \code{t.test2}}

\item{...}{Additional arguments passed to print}
}
\value{
Invisibly returns the input object \code{x}. Called for its side effect 
  of printing a formatted t-test summary to the console, including means, 
  confidence intervals, test statistics, p-values, sample sizes, and 
  APA-formatted results.
}
\description{
Print method for t.test2 output
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_table2.R, R/table2.R
\name{print.table2}
\alias{print.table2}
\title{Print method for table2 output with centered column variable name}
\usage{
\method{print}{table2}(x, ...)

\method{print}{table2}(x, ...)
}
\arguments{
\item{x}{An object of class \code{table2}}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the input object \code{x}. Called for its side effect 
  of printing a formatted cross-tabulation table to the console. The output 
  includes frequencies, optional relative frequencies (row, column, or overall 
  proportions), and chi-squared test results when applicable.

Invisibly returns the original object
}
\description{
Print method for table2 output with centered column variable name

Print method for table2 objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lm2.R
\name{summary.lm2}
\alias{summary.lm2}
\title{Summary method for lm2 objects}
\usage{
\method{summary}{lm2}(object, ...)
}
\arguments{
\item{object}{An object of class \code{lm2}}

\item{...}{Additional arguments passed to \code{\link{print.lm2}}}
}
\value{
Invisibly returns the original object
}
\description{
Summary method for lm2 objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_gam.R
\name{plot_gam}
\alias{plot_gam}
\title{Plot GAM Model}
\usage{
plot_gam(
  model,
  predictor,
  quantile.others = 50,
  col = "blue4",
  bg = adjustcolor("dodgerblue", 0.2),
  plot2 = "auto",
  col2 = NULL,
  bg2 = "gray90",
  ...
)
}
\arguments{
\item{model}{A GAM model object fitted using \code{mgcv::gam()}.}

\item{predictor}{Character string specifying the name of the predictor variable
to plot on the x-axis.}

\item{quantile.others}{Number between 1 and 99 for quantile
at which all other predictors are held constant. Default is 50 (median).}

\item{col}{Color for the prediction line. Default is "blue4".}

\item{bg}{Background color for the confidence band. Default is
\code{adjustcolor('dodgerblue', .2)}.}

\item{plot2}{How to plot the distribution in the lower plot. Options: \code{'auto'} (default,
auto-select based on number of unique values), \code{'freq'} (always plot frequencies),
\code{'density'} (always plot the density) or \code{'none'} (neither). When \code{'auto'}, 
plots frequencies with predictor has less than 30 unique values, density otherwise.}

\item{col2}{Color for the lines/bars in the bottom distribution plot. Default is "dodgerblue"}

\item{bg2}{Background color for the bottom distribution plot. Default is "gray90".}

\item{...}{Additional arguments passed to \code{plot()} and \code{lines()}.}
}
\value{
Invisibly returns a list containing:
\itemize{
  \item \code{predictor_values}: The sequence of predictor values used
  \item \code{predicted}: The predicted values
  \item \code{se}: The standard errors
  \item \code{lower}: Lower confidence bound (predicted - 2*se)
  \item \code{upper}: Upper confidence bound (predicted + 2*se)
}
}
\description{
Plots fitted GAM values for focal predictor,
keeping any other predictors in the model at a specified quantile  (default: median)
}
\examples{
\donttest{
library(mgcv)
# Fit a GAM model
data(mtcars)
mtcars$cyl <- factor(mtcars$cyl)  # Convert to factor before fitting GAM
model <- gam(mpg ~ s(hp) + s(wt) + cyl, data = mtcars)

# Plot effect of hp (with other variables at median)
plot_gam(model, "hp")

# Plot effect of hp (with other variables at 25th percentile)
plot_gam(model, "hp", quantile.others = 25)

# Customize plot
plot_gam(model, "hp", main = "Effect of Horsepower", col = "blue", lwd = 2)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/text2.R
\name{text2}
\alias{text2}
\title{Enhanced alternative to text()}
\arguments{
\item{x, y}{coordinates for text placement}

\item{labels}{text to display}

\item{align}{alignment in relation to x coordinate ('left','center','right')}

\item{bg}{background color}

\item{cex}{character expansion factor}

\item{pad}{left/right padding in percentage (e.g., .03)}

\item{pad_v}{top/bottom padding in percentage (e.g., .25)}

\item{...}{Additional arguments passed to \code{\link[graphics]{text}}.}
}
\value{
No return value, called for side effects. Adds text with an optional 
  background rectangle to an existing plot.
}
\description{
Adds to text() optional background color and verbal alignment (align='center')
}
\examples{
# Create a simple plot
plot(1:10, 1:10, type = "n", main = "text2() - Alignment & Color")

# Alignment respect to x=5
text2(5, 8, "align='left' from 5", align = "left", bg = "yellow1")
text2(5, 7, "align='right' from 5", align = "right", bg = "blue", col = "white")
text2(5, 6, "align='center' from 5", align = "center", bg = "black", col = "white")
abline(v = 5, lty = 2)

# Multiple labels with different alignments
text2(c(2, 5, 8), c(5, 5, 5), 
      labels = c("Left", "Center", "Right"),
      align = c("left", "center", "right"),
      bg = c("pink", "lightblue", "lightgreen"))

# Text with custom font color (passed through ...)
text2(5, 3, "Red Text", col = "red", bg = "white")

# Padding examples
plot(1:10, 1:10, type = "n", main = "Padding Examples")

# Default padding (pad=0.03, pad_v=0.25)
text2(5, 8, "Default padding", bg = "lightblue")

# More horizontal padding
text2(5, 6, "Wide padding", pad = 0.2, bg = "lightgreen")

# More vertical padding
text2(5, 4, "Tall padding", pad_v = 0.8, bg = "lightyellow")

# Both padding increased
text2(5, 2, "Extra padding", pad = 0.15, pad_v = 0.6, bg = "pink")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scatter.gam.R
\name{scatter.gam}
\alias{scatter.gam}
\title{Scatter Plot with GAM Smooth Line}
\usage{
scatter.gam(
  x,
  y,
  data.dots = TRUE,
  three.dots = FALSE,
  data = NULL,
  k = NULL,
  plot.dist = NULL,
  dot.pch = 16,
  dot.col = adjustcolor("gray", 0.7),
  jitter = FALSE,
  ...
)
}
\arguments{
\item{x}{A numeric vector of x values, or a formula of the form \code{y ~ x}.}

\item{y}{A numeric vector of y values. Not used if \code{x} is a formula.}

\item{data.dots}{Logical. If TRUE, displays data on scatterplot}

\item{three.dots}{Logical. If TRUE, divides x into tertiles and puts markers on the average  x & y for each}

\item{data}{An optional data frame containing the variables \code{x} and \code{y}.}

\item{k}{Optional integer specifying the basis dimension for the smooth term
in the GAM model (passed to \code{s(x, k=k)}). If NULL (default), uses the
default basis dimension.}

\item{plot.dist}{Character string specifying how to plot the distribution of \code{x}
underneath the scatter plot. Options: \code{NULL} (default, auto-select based on
number of unique values), \code{"none"} (no distribution plot), \code{"plot_freq"}
(always use \code{plot_freq()}), or \code{"hist"} (always use \code{hist()}).
When \code{NULL}, uses \code{plot_freq()} if there are 25 or fewer unique values,
otherwise uses \code{hist()}.}

\item{dot.pch}{Plotting character for data points when \code{data.dots = TRUE}.
Default is 16 (filled circle).}

\item{dot.col}{Color for data points when \code{data.dots = TRUE}. Default is
\code{adjustcolor('gray', 0.7)} (semi-transparent gray).}

\item{jitter}{Logical. If TRUE, applies a small amount of jitter to data points
to reduce overplotting. Default is FALSE.}

\item{...}{Additional arguments passed to \code{plot()} and \code{gam()}.
Common plot arguments include:
\itemize{
  \item \code{main}: Custom title for the plot (e.g., \code{main = "My Title"})
  \item \code{col}: Color of the GAM smooth line (e.g., \code{col = "red"})
  \item \code{lwd}: Line width of the GAM smooth line (e.g., \code{lwd = 2})
  \item \code{xlim}, \code{ylim}: Axis limits (e.g., \code{xlim = c(0, 10)})
  \item \code{xlab}, \code{ylab}: Axis labels (e.g., \code{xlab = "Age"})
}}
}
\value{
Invisibly returns the fitted GAM model object.
}
\description{
Creates a scatter plot with a GAM (Generalized Additive Model) smooth line.
Supports both \code{scatter.gam(x, y)} and \code{scatter.gam(y ~ x)}.
}
\details{
This function fits a GAM model with a smooth term for x and plots the fitted
smooth line. The function uses the \code{mgcv} package's \code{gam()} function.

When \code{three.dots = TRUE}, the x variable is divided into three equal-sized
groups (tertiles), and the mean x and y values for each group are plotted as
points. This provides a simple summary of the relationship across the range of x.
}
\examples{
# Generate sample data for examples
x <- rnorm(100)
y <- 2*x + rnorm(100)

# Plot GAM smooth line only
scatter.gam(x, y)

# Equivalent call using formula syntax (y ~ x)
scatter.gam(y ~ x)

# Include scatter plot with underlying data points behind the GAM line
scatter.gam(x, y, data.dots = TRUE)

# Include summary points showing mean x and y for each tertile bin
scatter.gam(x, y, three.dots = TRUE)

# Customize the plot with a custom title, line color, and line width
scatter.gam(x, y, data.dots = TRUE, col = "red", lwd = 2, main = "GAM Fit")

# Control smoothness of the GAM line by specifying the basis dimension
scatter.gam(x, y, k = 10)

}
\seealso{
\code{\link[stats]{scatter.smooth}} for a simpler loess-based scatter plot smoother.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clear.R
\name{clear}
\alias{clear}
\title{Clear Plot, Global Environment, and Console}
\usage{
clear()
}
\value{
Invisibly returns NULL. Prints a colored confirmation message.
}
\description{
Clears plot, global environment, and console. On first use the user is
prompted to authorize clearing the environment to comply with CRAN rules.
}
\details{
This function performs three cleanup operations:
\itemize{
  \item \strong{Plot}: Closes all open graphics devices (except the null device)
  \item \strong{Global environment}: Removes all objects from the global environment
  \item \strong{Console}: Clears the console screen (only in interactive sessions)
}

\code{clear()} will not modify the global environment unless you have
previously typed "yes" when prompted. If you do not type "yes", you are asked
again next time; only "yes" is remembered for future sessions.

\strong{Warning}: This function deletes all objects in the global environment.
Save anything that you wish to keep before running.
}
\examples{
\donttest{
# Interactive use: clear workspace, console, and plots
# First run may prompt; once you type "yes", your preference is saved.
clear()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate.R
\name{validate_t.test2}
\alias{validate_t.test2}
\title{Validate Grouping Variable for t.test2()}
\usage{
validate_t.test2(
  group_var_name,
  data = NULL,
  calling_env = parent.frame(),
  data_name = NULL
)
}
\arguments{
\item{group_var_name}{Character string. Name of the grouping variable (for error messages).}

\item{data}{An optional data frame containing the variable.}

\item{calling_env}{The environment in which to look for the variable if data is not provided.}

\item{data_name}{Character string. Name of the data argument (for error messages).
If NULL, will attempt to infer from the call.}
}
\value{
The validated grouping variable (numeric or factor vector). Stops execution with an error message if validation fails.
}
\description{
Validates that a grouping variable exists and has exactly 2 levels for t-test.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lm2.R
\name{print.lm2}
\alias{print.lm2}
\title{Print method for lm2 objects}
\usage{
\method{print}{lm2}(x, notes, ...)
}
\arguments{
\item{x}{An object of class \code{lm2}}

\item{notes}{\code{deprecated}. Ignored. Use \code{\link{lm2_notes}} after printing.}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the original object
}
\description{
Print method for lm2 objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_means.R
\name{plot_means}
\alias{plot_means}
\title{Barplot of means}
\usage{
plot_means(
  formula,
  data = NULL,
  cluster = NULL,
  tests = "auto",
  quiet = FALSE,
  order = NULL,
  legend.title = NULL,
  col = NULL,
  col.text = NULL,
  values.cex = 1,
  values.align = "top",
  values.round = 1,
  pvalue.cex = 0.9,
  pvalue.col = "gray50",
  ci.level = 95,
  buffer.top = "auto",
  ...
)
}
\arguments{
\item{formula}{A formula, e.g., \code{y ~ x1+x2}, where x1 & x2 are
grouping variables (e.g., condition indicators in a 2x2 experiment). 
The formula can include up to three grouping variables. The plot will 
 show contiguous bars for x1, and sets of bars for x2 and x3}

\item{data}{Optional data frame containing variables in the formula.}

\item{cluster}{Optional clustering variable when there are repeated 
observations per cluster 
e.g., \code{cluster="participant_ID"}. 
When provided, inference for reported tests is based on regressions with
clustered standard errors (via \code{lm2(..., clusters=...)}).}

\item{tests}{specifies which comparisons of means to report. Syntax involves 
putting column numbers in a character string, with a - to symbolize a 
comparison and a + symbolizing combination. For example tests="1-2" reports 
t-test comparing columns 1 and 2. tests="1-2,3-4" t-tests comparing columns 
1 & 2 and another 3 & 4. To run an interaction use parentheses:
tests=\code{"(4-3)-(2-1)"} and can also be combined with simple tests. 
Main effects can be specified using `+`, for example (1+2)-(3+4) compares 
all observations in the first two columns with all observations in 
the next two columns.}

\item{quiet}{Logical. When \code{TRUE}, suppresses console messages from \code{plot_means()}.}

\item{order}{Controls the order of \code{x1} groups (bar order and colors).
Use \code{-1} to reverse the default order (e.g., if plot shows 'male' first and 'female' second, order=-1 will flip that).}

\item{legend.title}{Character string. Title for the legend. If \code{NULL},
no title is shown.}

\item{col}{Color(s) for \code{x1} bars. If \code{NULL}, colors are chosen
automatically.}

\item{col.text}{Color for confidence intervals and other non-bar annotations.
If \code{NULL}, defaults to a dark gray.}

\item{values.cex}{Numeric scalar controlling text size for mean value labels
(and related annotations).}

\item{values.align}{Where within the bars to put the mean value labels: \code{"top"}, \code{"middle"},
\code{"bottom"}, or \code{"none"}.}

\item{values.round}{Non-negative integer. Number of decimal places for mean
value labels.}

\item{pvalue.cex}{Numeric scalar controlling p-value label size.}

\item{pvalue.col}{Color for p-value brackets/labels.}

\item{ci.level}{Confidence interval level for \code{ciL}/\code{ciH} in \code{$means}.
Default is \code{95}. You can also pass a proportion (e.g., \code{0.95}).}

\item{buffer.top}{Either \code{"auto"} (default) or a numeric value. Extra
vertical headroom (as a fraction of the data y-range) added above the
maximum y value to make room for annotations. When \code{"auto"}, uses 0.35
when an interaction p-value is shown (scenario 2) and 0.25 otherwise.}

\item{...}{Additional arguments passed to \code{plot()} (e.g., \code{main},
\code{ylim}, \code{ylab}).}
}
\value{
A minimal list returned invisibly with two elements:
\describe{
  \item{\code{means}}{A data frame of means (and, when available, confidence intervals)
    aligned to the plotting grid.}
  \item{\code{tests}}{A data frame of comparisons used for p-value
    annotation (or \code{NULL} if not applicable).}
}
}
\description{
Plots means, with confidence intervals, and (optionally) p-values for differences of means and interactions
}
\details{
When \code{tests="auto"}, the function reports a small default set of
differences-of-means tests (when applicable) and, in 2x2 designs, an
interaction test:
\describe{
  \item{Differences in means}{If \code{cluster} is \code{NULL}, these are Welch
    two-sample t-tests computed with \code{t.test(..., var.equal=FALSE)}. If
    \code{cluster} is provided, these comparisons are computed from a
    regression using \code{lm2()} with clustered standard errors.}
  \item{Interaction}{The interaction is tested using a linear regression fit
    with \code{lm2()}, even when \code{cluster} is \code{NULL}; when
    \code{cluster} is provided, the interaction test uses clustered standard
    errors.}
}
The regression-based tests use heteroskedasticity-robust inference (HC3) when
\code{cluster} is \code{NULL}. HC3 is a common small-sample adjustment to
White-type robust standard errors and is used to reduce sensitivity to
heteroskedasticity. When \code{cluster} is provided, \code{plot_means()}
instead uses clustered standard errors (robust to within-cluster correlation).

In the returned \code{$means} table, \code{ciL} and \code{ciH} are the lower and
upper bounds of a \code{ci.level}\% confidence interval for the mean (when
available). The same confidence level is used for the confidence-interval
whiskers drawn in the figure.
}
\examples{
df <- data.frame(y = rnorm(100), group = rep(c("A", "B"), 50))
plot_means(y ~ group, data = df)

df2 <- data.frame(
  y = rnorm(200),
  x1 = rep(c("A", "B"), 100),
  x2 = rep(c("X", "Y"), each = 100)
)
plot_means(y ~ x1 + x2, data = df2)

df3 <- data.frame(
  y = rnorm(600),
  x1 = rep(c("control", "treatment"), times = 300),
  x2 = rep(rep(c("low", "high"), each = 150), times = 2),
  x3 = rep(c("online", "lab"), each = 300)
)
plot_means(y ~ x1 + x2 + x3, data = df3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stimulus.plot.R
\name{stimulus.plot}
\alias{stimulus.plot}
\title{Make stimulus plots as in "Stimulus Sampling Reimagined"}
\usage{
stimulus.plot(
  plot.type = "means",
  data,
  dv,
  condition,
  stimulus,
  participant = "",
  save.as = "",
  svg.width = "",
  svg.height = "",
  sort.by = "",
  flip.conditions = FALSE,
  model = c(),
  overall.estimate = c(),
  overall.ci = c(),
  overall.p = c(),
  overall.label = c(),
  ylab1 = "",
  ylab2 = "",
  xlab1 = "Stimuli",
  xlab2 = "",
  decimals = "auto",
  null.method = "shuffle",
  dv.is.percentage = FALSE,
  legend.title = "",
  simtot = 1000,
  watermark = TRUE,
  seed = 2024,
  ylim = c(),
  main = "",
  ...
)
}
\arguments{
\item{plot.type}{Either `"means"` or `"effects"`; determines what is plotted on the y-axis.}

\item{data}{Data frame containing variables to be analyzed.}

\item{dv}{Name of the dependent variable (e.g., `dv = 'y'`); quotes are not required.}

\item{condition}{Name of the variable containing the condition indicator.}

\item{stimulus}{Name of the variable containing the stimulus ID.}

\item{participant}{Name of the variable containing participant IDs; necessary for valid
inference when `plot.type = 'effects'` and each participant provided more than one observation.}

\item{save.as}{File path for saving the figure (`.svg` or `.png`, optional).}

\item{svg.width}{Optional width when saving to SVG/PNG.}

\item{svg.height}{Optional height when saving to SVG/PNG.}

\item{sort.by}{Variable to sort stimuli by. Defaults to sorting by observed effect size.}

\item{flip.conditions}{If `TRUE`, subtract the first condition from the second instead of the default.}

\item{model}{Method used to compute overall average: `'regression'`, `'intercepts'`,
`'slopes'`, and/or `'all'`.}

\item{overall.estimate}{Scalar or vector of overall average effect computed outside statuser.}

\item{overall.ci}{Confidence interval bounds for `overall.estimate`.}

\item{overall.p}{P-value for overall average effect computed outside statuser.}

\item{overall.label}{Label for overall averages on the x-axis.}

\item{ylab1, ylab2}{Labels on the y-axis (optional).}

\item{xlab1, xlab2}{Labels on the x-axis (optional).}

\item{decimals}{Number of decimals for value labels (`"auto"` by default).}

\item{null.method}{Null method for effects plots: `"shuffle"` or `"demean"`.}

\item{dv.is.percentage}{If `TRUE`, format the dependent variable as percentages.}

\item{legend.title}{Text with title above legend (optional).}

\item{simtot}{Number of resamples for heterogeneity under null (effects plots).}

\item{watermark}{If `TRUE`, display package version watermark.}

\item{seed}{Seed used when resampling for effects plots.}

\item{ylim}{Optional y-axis limits.}

\item{main}{Plot title.}

\item{...}{Additional arguments passed to [graphics::plot.default()].}
}
\value{
Invisibly, a data frame (means plot) or list (effects plot).
}
\description{
Build stimulus plots comparing results for individual stimuli in an experiment
(Simonsohn, Montealegre, & Evangelidis, 2024).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/message2.R
\name{message2}
\alias{message2}
\title{Enhanced alternative to message()}
\usage{
message2(..., col = "cyan", font = 1, stop = FALSE)
}
\arguments{
\item{...}{Message content to be printed. Multiple arguments are pasted together.}

\item{col}{text color. Default is "cyan".}

\item{font}{Integer. 1 for plain text (default), 2 for bold text.}

\item{stop}{Logical. If TRUE, stops execution (like \code{stop()}) but without printing "Error:".}
}
\value{
No return value, called for side effects. Prints a colored message 
  to the console. If \code{stop = TRUE}, execution is halted after printing 
  the message.
}
\description{
Add options to set color and to end execution of code (to be used as error message)
}
\details{
This function prints colored messages to the console. If ANSI color codes are supported
by the terminal, the message will be colored. Otherwise, it will be printed as plain text.
If \code{stop = TRUE}, execution will be halted after printing the message.
}
\examples{
message2("This is a plain cyan message", col = "cyan", font = 1)
message2("This is a bold cyan message", col = "cyan", font = 2)
message2("This is a bold red message", col = "red", font = 2)
\donttest{
cat("this will be shown")
try(message2("This stops execution", stop = TRUE), silent = TRUE)
cat("this will be shown after the try")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lm2.R
\name{lm2_notes}
\alias{lm2_notes}
\title{Print explanatory notes from the most recent \code{lm2} print}
\usage{
lm2_notes()
}
\value{
Invisibly returns the notes character string, or \code{NULL} if none stored.
}
\description{
After printing an \code{lm2} object, call \code{lm2_notes()} to display the
full \code{Notes:} block (significance legend, column definitions, red-flag
guidance, etc.).
}
\seealso{
\code{\link{lm2}}, \code{\link{print.lm2}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stimulus.beeswarm.R
\name{stimulus.beeswarm}
\alias{stimulus.beeswarm}
\title{Make beeswarm plots for compared-stimulus designs}
\usage{
stimulus.beeswarm(
  data,
  dv,
  stimulus,
  condition,
  flip.conditions = FALSE,
  dv.is.percentage = FALSE,
  simtot = 500,
  confidence = 95,
  ylim = c(),
  ylab1 = "",
  ylab2 = "",
  xlab1 = "",
  xlab2 = NULL,
  dot.spacing = "auto",
  col1 = "blue4",
  col2 = "red4",
  main = "",
  watermark = TRUE,
  save.as = "",
  svg.width = "",
  svg.height = "",
  ...
)
}
\arguments{
\item{data}{Data frame containing variables to be analyzed.}

\item{dv}{Name of the dependent variable.}

\item{stimulus}{Name of the variable containing the stimulus ID.}

\item{condition}{Name of the variable containing the condition indicator.}

\item{flip.conditions}{If `TRUE`, reverse the order of condition labels.}

\item{dv.is.percentage}{If `TRUE`, format values as percentages.}

\item{simtot}{Number of bootstraps for the confidence band under homogeneity.}

\item{confidence}{Confidence level for the band (default 95).}

\item{ylim}{Optional y-axis limits.}

\item{ylab1, ylab2}{Labels on the y-axis (optional).}

\item{xlab1, xlab2}{Labels on the x-axis (optional).}

\item{dot.spacing}{Horizontal distance between stimulus labels (`"auto"` by default).}

\item{col1, col2}{Colors for the two conditions.}

\item{main}{Plot title.}

\item{watermark}{If `TRUE`, display package version watermark.}

\item{save.as}{File path for saving the figure (`.svg` or `.png`, optional).}

\item{svg.width}{Optional width when saving.}

\item{svg.height}{Optional height when saving.}

\item{...}{Additional arguments passed to [graphics::plot.default()].}
}
\value{
Invisibly, a two-column matrix of beeswarm coordinates.
}
\description{
Beeswarm plot for compared-stimulus designs from "Stimulus Sampling Reimagined"
(Simonsohn, Montealegre, & Evangelidis, 2024).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_var.R
\name{print.desc_var}
\alias{print.desc_var}
\title{Print method for desc_var objects}
\usage{
\method{print}{desc_var}(x, ...)
}
\arguments{
\item{x}{An object of class \code{desc_var}}

\item{...}{Additional arguments passed to print.data.frame}
}
\value{
Invisibly returns the original object
}
\description{
Print method for desc_var objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format_pvalue.R
\name{format_pvalue}
\alias{format_pvalue}
\title{Format P-Values for Display}
\usage{
format_pvalue(p, digits = 4, include_p = FALSE)
}
\arguments{
\item{p}{A numeric vector of p-values to format.}

\item{digits}{Number of decimal places to round to. Default is 4.}

\item{include_p}{Logical. If TRUE, includes "p" prefix before the formatted
value (e.g., "p = .05"). Default is FALSE.}
}
\value{
A character vector of formatted p-values.
}
\description{
Formats p-values for clean display in figures and tables. e.g., p = .0231, p<.0001
}
\examples{
# Basic usage
format_pvalue(0.05)
format_pvalue(0.0001)

# More rounding
format_pvalue(0.0001,digits=2)

# Vector input
format_pvalue(c(0.05, 0.001, 0.00001, 0.99))

# With p prefix
format_pvalue(0.05, include_p = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/t.test2.R
\name{t.test2}
\alias{t.test2}
\title{Enhanced alternative to t.test()}
\arguments{
\item{x}{The first data argument passed to \code{\link[stats]{t.test}}. This can
be a numeric vector or a formula.}

\item{...}{Arguments passed to \code{\link[stats]{t.test}}}
}
\value{
A data frame with class \code{c("t.test2", "data.frame")} containing 
  a single row with the following columns:
  \describe{
    \item{mean columns}{One or two columns containing group means, named after 
      the input variables (e.g., \code{men}, \code{women}) or \code{Group 1}, 
      \code{Group 2} for long names.}
    \item{diff column}{For two-sample tests, the difference between means 
      (e.g., \code{men-women}).}
    \item{d}{For two-sample independent tests only, Cohen's d computed as the
      mean difference divided by the pooled standard deviation.}
    \item{ci}{The confidence level as a string (e.g., "95 percent").}
    \item{ci.L, ci.H}{Lower and upper bounds of the confidence interval.}
    \item{t}{The t-statistic.}
    \item{df}{Degrees of freedom.}
    \item{p.value}{The p-value.}
    \item{N columns}{Sample sizes, named \code{N(group1)}, \code{N(group2)} or 
      \code{N1}, \code{N2}. For paired tests, a single \code{N} column.}
    \item{correlation}{For paired tests only, the correlation between pairs.}
  }
  Attributes store additional information including missing value counts and 
  test type (one-sample, two-sample, paired, Welch vs. Student).
}
\description{
The basic t-test function in R, \code{\link[stats]{t.test}}, does not report 
 the observed difference of means, does not stipulate which mean
 is subtracted from which (i.e., whether it computed A-B or B-A), and
 presents the test results on the console in a verbose unorganized 
 paragraph of text. \code{t.test2} improves on all those counts, and 
 in addition, it reports the number of observations per group and if any observations 
 are missing it issues a warning. It returns a dataframe instead of a list.
 The function is also registered as \code{t.test2} for the \code{\link[base]{t}}
 generic to satisfy S3 registration checks while keeping direct calls to
 \code{t.test2(...)} unchanged.
}
\examples{
# Two-sample t-test
men <- rnorm(100, mean = 5, sd = 1)
women <- rnorm(100, mean = 4.8, sd = 1)
t.test2(men, women)

# Paired t-test
x <- rnorm(50, mean = 5, sd = 1)
y <- rnorm(50, mean = 5.2, sd = 1)
t.test2(x, y, paired = TRUE)

# One-sample t-test
data <- rnorm(100, mean = 5, sd = 1)
t.test2(data, mu = 0)

# Formula syntax
data <- data.frame(y = rnorm(100), group = rep(c("A", "B"), 50))
t.test2(y ~ group, data = data)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resize_images.R
\name{resize_images}
\alias{resize_images}
\title{Resize Images}
\usage{
resize_images(path, width)
}
\arguments{
\item{path}{Character string. Path to a folder containing image files, or path to a single image file.}

\item{width}{Numeric vector. Target width(s) in pixels for the output PNG files.
Can be a single value (recycled for all files) or a vector matching the number
of files found.}
}
\value{
Invisibly returns TRUE on success.
}
\description{
Saves images to PNG with a specified width. As input it accepts (SVG, PDF, EPS, JPG, JPEG, TIF, TIFF, BMP, PNG)
Saves to subdirectory '/resized' within input folder (or same directory as file if input is a single file)
}
\details{
This function:
\itemize{
  \item Searches for image files with extensions: svg, pdf, eps, jpg, jpeg, tif, tiff, bmp, png
  \item Creates a "resized" subfolder in the target directory if it doesn't exist
  \item Converts each file to PNG format at the specified width(s)
  \item Saves output files as: \code{originalname_width.png} in the resized subfolder
}

Supported input formats:
\itemize{
  \item Vector graphics: SVG, PDF, EPS (rasterized using rsvg/magick)
  \item Raster images: JPG, JPEG, TIF, TIFF, BMP, PNG
}
}
\note{
Dependencies required: \code{rsvg}, \code{magick}, and \code{tools} (base R).
SVG files are rasterized using \code{rsvg::rsvg()}, while PDF/EPS and other
formats are handled by \code{magick::image_read()}.
}
\examples{
\donttest{
# Create a temporary PNG file and resize it
tmp_png <- tempfile(fileext = ".png")
grDevices::png(tmp_png, width = 400, height = 300)
old_par <- graphics::par(no.readonly = TRUE)
graphics::par(mar = c(2, 2, 1, 1))
graphics::plot(1:2, 1:2, type = "n")
grDevices::dev.off()
graphics::par(old_par)
resize_images(tmp_png, width = 80)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lm2.R
\name{lm2}
\alias{lm2}
\title{Enhanced alternative to lm()}
\arguments{
\item{se_type}{The type of standard error to use. Default is \code{"HC3"}.
Without clusters: \code{"HC0"}, \code{"HC1"}, \code{"HC2"}, or \code{"HC3"}.
When \code{clusters} is specified, \code{se_type} is automatically set to \code{"CR2"}.}

\item{notes}{Logical. If TRUE (default), print explanatory notes below the table
when the result is printed.}

\item{clusters}{An optional variable indicating clusters for cluster-robust standard 
errors. When specified, \code{se_type} is automatically set to \code{"CR2"} 
(bias-reduced cluster-robust estimator). Passed to \code{\link[estimatr]{lm_robust}}.}

\item{fixed_effects}{An optional right-sided formula containing the fixed effects 
to be projected out (absorbed) before estimation. Useful for models with many 
fixed effect groups (e.g., \code{~ firm_id} or \code{~ firm_id + year}). 
Passed to \code{\link[estimatr]{lm_robust}}.}

\item{...}{Additional arguments passed to \code{\link[estimatr]{lm_robust}}.}
}
\value{
An object of class \code{c("lm2", "lm_robust", "lm")}. This inherits 
  from \code{\link[estimatr]{lm_robust}} and can be used with packages like 
  \code{marginaleffects}. The object contains all components of an \code{lm_robust} 
  object plus additional attributes:
  \describe{
    \item{statuser_table}{A data frame with columns: \code{term}, \code{estimate}, 
      \code{SE.robust}, \code{SE.classical}, \code{t}, \code{df}, \code{p.value}, 
      \code{B} (standardized coefficient), and optionally \code{SE.cluster} when 
      clustered standard errors are used.}
    \item{classical_fit}{The underlying \code{lm} object with classical standard errors.}
    \item{na_counts}{Integer vector of missing value counts per variable.}
    \item{n_missing}{Total number of observations excluded due to missing values.}
    \item{has_clusters}{Logical indicating whether clustered standard errors were used.}
  }
  When printed, displays a formatted regression table with robust and classical 
  standard errors, effect sizes, and diagnostic red flags.
}
\description{
Runs a linear regression with better defaults (robust SE), and richer & better 
formatted output than \code{lm}. For robust and clustered errors it relies on \code{\link[estimatr]{lm_robust}}. 
The output reports classical and robust errors, number of missing observations per
variable, an effect size column (standardized regression coefficient), and a red.flag column per variable 
flagging the need to conduct specific diagnostics. It relies by default on HC3 for standard errors;
\code{lm_robust} relies on HC2 (and Stata's 'reg y x, robust' on HC1), which can have
inflated false-positive rates in smaller samples (Long & Ervin, 2000).
}
\details{
Robust standard errors and clustered standard errors are computed using 
\code{\link[estimatr]{lm_robust}}; see the documentation of that function for details (using by default CR2 errors)
The output shows both standard errors and when clustering errors it reports all three.
The red.flag column is based on the difference between robust and classical standard errors.

The \code{red.flag} column provides diagnostic warnings:
\itemize{
  \item \code{!}, \code{!!}, \code{!!!}: Robust and classical standard errors differ by 
    more than 25\%, 50\%, or 100\%, respectively. Large differences may suggest model 
    misspecification or outliers (but they may also be benign). When encountering a red flag,
    authors should plot the distributions to look for outliers or skewed data, and use \code{\link{scatter.gam}}
    to look for possible nonlinearities in the relevant variables.
    King & Roberts (2015) propose a higher cutoff, at 100\%, and a bootstrapped significance test; 
    \code{statuser} does not follow either recommendation. The former seems too liberal, the 
    latter too time consuming to include in every regression, plus the focus here is on individual variables rather than joint tests.
  \item \code{X}: For interaction terms, the component variables are correlated (|r| > 0.3 or p < .05),
    which means the interaction term is likely to be biased. See Simonsohn (2024) "Interacting with curves"
    \doi{10.1177/25152459231207787}.
}
}
\examples{
# Basic usage with data argument
lm2(mpg ~ wt + hp, data = mtcars)

# Without data argument (variables from environment)
y <- mtcars$mpg
x1 <- mtcars$wt
x2 <- mtcars$hp
lm2(y ~ x1 + x2)

# RED FLAG EXAMPLES

# Example 1: red flag catches a nonlinearity
# True model is quadratic: y = x^2
set.seed(123)
x <- runif(200, -3, 3)
y <- x^2 + rnorm(200, sd = 2)

# lm2() shows red flag due to misspecification
lm2(y ~ x)

# Follow up with scatter.gam() to diagnose it
scatter.gam(x, y)

# Example 2: red flag catches an outlier in y
# True model is y = x, but one observation has a very large y value
set.seed(123)
x <- sort(rnorm(200))
y <- round(x + rnorm(200, sd = 2), 1)
y[200] <- 100  # Outlier

# lm2() flags x
lm2(y ~ x)

# Look at distribution of y to spot the outlier
plot_freq(y)

# Example 3: red flag catches an outlier in one predictor
# True model is y = x1 + x2, but x2 has an extreme value
set.seed(123)
x1 <- round(rnorm(200),.1)
x2 <- round(rnorm(200),.1)
y <- x1 + x2 + rnorm(200, sd = 0.5)
x2[200] <- 50  # Outlier in x2

# lm2() flags x2 (but not x1)
lm2(y ~ x1 + x2)

# Look at distribution of x2 to spot the outlier
plot_freq(x2)

# CLUSTERED STANDARD ERRORS
# When observations are grouped (e.g., students within schools),
# use clusters to account for within-group correlation
set.seed(123)
n_clusters <- 20
n_per_cluster <- 15
cluster_id <- rep(1:n_clusters, each = n_per_cluster)
cluster_effect <- rnorm(n_clusters, sd = 2)[cluster_id]
x <- rnorm(n_clusters * n_per_cluster)
y <- 1 + 0.5 * x + cluster_effect + rnorm(n_clusters * n_per_cluster)
mydata <- data.frame(y = y, x = x, cluster_id = cluster_id)

# Clustered SE (CR2) - note the SE.cluster column in output
lm2(y ~ x, data = mydata, clusters = cluster_id)

# FIXED EFFECTS
# Use fixed_effects to absorb group-level variation (e.g., firm or year effects)
# This is useful for panel data or when you have many fixed effect levels
set.seed(456)
n_firms <- 30
n_years <- 5
firm_id <- rep(1:n_firms, each = n_years)
year <- rep(2018:2022, times = n_firms)
firm_effect <- rnorm(n_firms, sd = 3)[firm_id]
x <- rnorm(n_firms * n_years)
y <- 2 + 0.8 * x + firm_effect + rnorm(n_firms * n_years)
panel <- data.frame(y = y, x = x, firm_id = factor(firm_id), year = factor(year))

# Absorb firm fixed effects (coefficient on x is estimated, firm dummies are not shown)
lm2(y ~ x, data = panel, fixed_effects = ~ firm_id)

# Two-way fixed effects (firm and year)
lm2(y ~ x, data = panel, fixed_effects = ~ firm_id + year)

}
\references{
King, G., & Roberts, M. E. (2015). How robust standard errors expose methodological 
problems they do not fix, and what to do about it. \emph{Political Analysis}, 23(2), 159-179.

Long, J. S., & Ervin, L. H. (2000). Using heteroscedasticity consistent standard errors 
in the linear regression model. \emph{The American Statistician}, 54(3), 217-224.

Simonsohn, U. (2024). Interacting with curves: How to validly test and probe 
interactions in the real (nonlinear) world. \emph{Advances in Methods and Practices in 
Psychological Science}, 7(1), 1-22.
}
\seealso{
\code{\link[estimatr]{lm_robust}}, \code{\link{scatter.gam}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statuser-package.R
\docType{package}
\name{statuser-package}
\alias{statuser}
\alias{statuser-package}
\title{Stat Tools for End Users}
\description{
Basic and custom statistical tools designed with end users in mind. 
Functions have optimized defaults, produce decluttered and informative output 
that is self-explanatory, and generate publication-ready results in 1 line of code.
}
\section{Basic Stats (improved)}{

\itemize{
  \item \code{\link{lm2}}: like lm(), with robust SE and much more informative output
  \item \code{\link{t.test2}}: like t.test(), decluttered, and more informative output 
  \item \code{\link{table2}}: like table(), showing variable names, and with proportions & chi2 built in
  \item \code{\link{desc_var}}: Descriptive statistics for variables (optional, by group(s))
}
}

\section{Custom tools from papers by Uri Simonsohn}{

\itemize{
  \item \code{\link{twolines}}: Two-lines test for U-shapes (Simonsohn 2018)
  \item \code{\link{interprobe}}: Probe and visualize nonlinear interactions (Simonsohn 2024; Montealegre & Simonsohn 2026)
  \item \code{\link{stimulus.plot}}: Stimulus plots for matched/treated-stimulus designs (Simonsohn, Montealegre, & Evangelidis 2024)
  \item \code{\link{stimulus.beeswarm}}: Beeswarm plots for compared-stimulus designs (Simonsohn, Montealegre, & Evangelidis 2024)
}
}

\section{Graphing}{

\itemize{
  \item \code{\link{scatter.gam}}: Makes scatter plot for x & y, with fitted GAM line y=f(x)
  \item \code{\link{plot_cdf}}: Plot empirical cumulative distribution functions (optional, by group)
  \item \code{\link{plot_density}}: Plot density functions (optional, by group)
  \item \code{\link{plot_freq}}: Plot frequency of observed values (optional, by group)
  \item \code{\link{plot_means}}: Barplot of means with confidence intervals and (optionally) tests. See also \code{\link{stimulus.plot}} for per-stimulus plots.
  \item \code{\link{plot_gam}}: Plot fitted GAM values for a focal predictor
  \item \code{\link{text2}}: like text() adding horizontal alignment and background color
}
}

\section{Formatting}{

\itemize{
  \item \code{\link{format_pvalue}}: Format p-values for display
  \item \code{\link{var_labels}}: Get/set variable labels (as base-R \code{"label"} attributes)
  \item \code{\link{message2}}: Print colored messages to console
  \item \code{\link{resize_images}}: Resize images (SVG, PDF, EPS, JPG, PNG, etc.) to PNG with specified width
}
}

\section{Miscellaneous}{

\itemize{
  \item \code{\link{clear}}: Clear environment, console, and all graphics devices
  \item \code{\link{list2}}: Like list(), but unnamed objects are automatically named
  \item \code{\link{convert_to_sql}}: Convert CSV files to SQL INSERT statements
}
}

\references{
Simonsohn, U. (2018). Two lines: A valid alternative to the invalid testing of 
U-shaped relationships with quadratic regressions. \emph{Advances in Methods and 
Practices in Psychological Science}, 1(4), 538-555. \doi{10.1177/2515245918805755}

Simonsohn, U. (2024). Interacting with curves: How to validly test and probe interactions in the real (nonlinear) world. \emph{Advances in Methods and Practices in Psychological Science}, 7(1), Article 25152459231207787. \doi{10.1177/25152459231207787}

Montealegre, D., & Simonsohn, U. (2026). \emph{Johnson Neyman 2.0} (working paper).

Simonsohn, U., Montealegre, A., & Evangelidis, I. (2024). Stimulus sampling reimagined.
\emph{Advances in Methods and Practices in Psychological Science}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/urisohn/statuser}
}
}
\author{
Uri Simonsohn \email{urisohn@gmail.com}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twolines.R
\name{reg2}
\alias{reg2}
\title{Two-Line Interrupted Regression}
\usage{
reg2(f, xc, graph = 1, family = "gaussian", data = NULL)
}
\arguments{
\item{f}{A formula object specifying the model (e.g., y ~ x1 + x2 + x3).
The first predictor is the one on which the u-shape is tested.}

\item{xc}{Numeric value specifying where to set the breakpoint.}

\item{graph}{Integer. If 1 (default), produces a plot. If 0, no plot is generated.}

\item{family}{Character string specifying the family for the GLM model.
Default is "gaussian" for OLS regression. Use "binomial" for probit models.}

\item{data}{A data frame containing the variables in the formula.}
}
\value{
A list containing:
\itemize{
  \item \code{b1}, \code{b2}: Slopes of the two regression lines
  \item \code{p1}, \code{p2}: P-values for the slopes
  \item \code{z1}, \code{z2}: Z-statistics for the slopes
  \item \code{u.sig}: Indicator (0/1) for whether u-shape is significant
  \item \code{xc}: The breakpoint value
  \item \code{glm1}, \code{glm2}: The fitted GLM models
  \item \code{rob1}, \code{rob2}: Robust coefficient test results
  \item \code{msg}: Messages about standard error computation
  \item \code{yhat.smooth}: Fitted smooth values (if graph=1)
}
}
\description{
Performs an interrupted regression analysis with heteroskedastic robust standard errors.
This function fits two regression lines with a breakpoint at a specified value of the
first predictor variable.
}
\details{
This function fits two interrupted regression lines with heteroskedastic robust
standard errors using the sandwich package. The first predictor variable is split
at the breakpoint \code{xc}, creating separate slopes before and after the breakpoint.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_to_sql.R
\name{convert_to_sql}
\alias{convert_to_sql}
\title{Convert CSV file to SQL INSERT statements}
\usage{
convert_to_sql(input, output, create_table = FALSE)
}
\arguments{
\item{input}{Character string. Path to the input CSV file.}

\item{output}{Character string. Path to the output SQL file where the
statements will be written.}

\item{create_table}{Logical. If \code{TRUE}, includes a CREATE TABLE
statement before the INSERT statements. Default is \code{FALSE}.}
}
\value{
Invisibly returns NULL. The function writes SQL statements to the
  specified output file.
}
\description{
Reads a CSV file and generates SQL statements to insert all rows. Optionally
can also generate a CREATE TABLE statement. The function automatically infers
column types (REAL for numeric, DATE for date strings matching YYYY-MM-DD
format, TEXT otherwise).
}
\details{
The function performs the following steps:
\enumerate{
  \item Reads the CSV file using \code{read.csv()} with
    \code{stringsAsFactors = FALSE}
  \item Infers SQL column types:
    \itemize{
      \item Numeric columns become \code{REAL}
      \item Date columns (matching YYYY-MM-DD format) become \code{DATE}
      \item All other columns become \code{TEXT}
    }
  \item If \code{create_table = TRUE}, generates a \code{CREATE TABLE}
    statement using the base filename (without extension) as the table name
  \item Generates \code{INSERT INTO} statements for each row
  \item Writes all SQL statements to the output file
}

Single quotes in text values are escaped by doubling them (SQL standard).
Numeric values are inserted without quotes, while text and date values are
wrapped in single quotes.
}
\examples{
# Convert a CSV file to SQL (INSERT statements only)
tmp_csv <- tempfile(fileext = ".csv")
tmp_sql <- tempfile(fileext = ".sql")
write.csv(
  data.frame(id = 1:2, value = c("a", "b"), date = c("2024-01-01", "2024-02-02")),
  tmp_csv,
  row.names = FALSE
)
convert_to_sql(tmp_csv, tmp_sql)

# Convert a CSV file to SQL with CREATE TABLE statement
convert_to_sql(tmp_csv, tmp_sql, create_table = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clear_stimulus_cache.R
\name{clear_stimulus_cache}
\alias{clear_stimulus_cache}
\title{Clear cache used by stimulus.plot() to reuse resampling results from identical previous calls.}
\usage{
clear_stimulus_cache()
}
\description{
When `stimulus.plot()` with `plot.type = 'effects'` is run, resamples are saved in
package state. If an identical call is run again (same data, dv, condition, number of
simulations, etc.), stored results are loaded instead of re-calculated. Force
recalculation by clearing the cache with this function.
}
